/**
*** (c) Copyright 2010-2018 Kalray SA.

***
*** This program is free software; you can redistribute it and/or
*** modify it under the terms of the GNU General Public License
*** as published by the Free Software Foundation; either version
*** 2 of the License, or (at your option) any later version.
***
*** This program is distributed in the hope that it will be useful,
*** but WITHOUT ANY WARRANTY; without even the implied warranty of
*** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*** General Public License for more details.
***
*** You should have received a copy of the GNU General Public License
*** along with this program; if not, write to the Free Software
*** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
***
**/

#include "sysdep.h"
#include <stddef.h>
#include <stdio.h>
#include "bfd.h"
#include "libbfd.h"
#include <elf/kv3.h>
#include <opcode/kv3.h>

#include <opcode/kv3_boolean.h>

int kvx_kv3_v1_regfiles[] = {
  0, 	/* KVX_REGFILE_FIRST_GPR */
  187, 	/* KVX_REGFILE_LAST_GPR */
  0, 	/* KVX_REGFILE_DEC_GPR */
  188, 	/* KVX_REGFILE_FIRST_PGR */
  251, 	/* KVX_REGFILE_LAST_PGR */
  64, 	/* KVX_REGFILE_DEC_PGR */
  252, 	/* KVX_REGFILE_FIRST_QGR */
  267, 	/* KVX_REGFILE_LAST_QGR */
  96, 	/* KVX_REGFILE_DEC_QGR */
  268, 	/* KVX_REGFILE_FIRST_SFR */
  1291, 	/* KVX_REGFILE_LAST_SFR */
  112, 	/* KVX_REGFILE_DEC_SFR */
  1292, 	/* KVX_REGFILE_FIRST_XBR */
  1547, 	/* KVX_REGFILE_LAST_XBR */
  624, 	/* KVX_REGFILE_DEC_XBR */
  1548, 	/* KVX_REGFILE_FIRST_XCR */
  2059, 	/* KVX_REGFILE_LAST_XCR */
  752, 	/* KVX_REGFILE_DEC_XCR */
  2060, 	/* KVX_REGFILE_FIRST_XMR */
  2075, 	/* KVX_REGFILE_LAST_XMR */
  1008, 	/* KVX_REGFILE_DEC_XMR */
  2076, 	/* KVX_REGFILE_FIRST_XVR */
  2267, 	/* KVX_REGFILE_LAST_XVR */
  1024, 	/* KVX_REGFILE_DEC_XVR */
  2268, 	/* KVX_REGFILE_FIRST_XWR */
  2331, 	/* KVX_REGFILE_LAST_XWR */
  1088, 	/* KVX_REGFILE_DEC_XWR */
  2332, 	/* KVX_REGFILE_REGISTERS*/
  1120, 	/* KVX_REGFILE_DEC_REGISTERS*/
};


kvx_Register kvx_kv3_v1_registers[] = {
  { 0, "$r0"}, /* 0 */
  { 0, "$r0r1.lo"}, /* 1 */
  { 0, "$r0r1r2r3.x"}, /* 2 */
  { 1, "$r1"}, /* 3 */
  { 1, "$r0r1.hi"}, /* 4 */
  { 1, "$r0r1r2r3.y"}, /* 5 */
  { 2, "$r2"}, /* 6 */
  { 2, "$r2r3.lo"}, /* 7 */
  { 2, "$r0r1r2r3.z"}, /* 8 */
  { 3, "$r3"}, /* 9 */
  { 3, "$r2r3.hi"}, /* 10 */
  { 3, "$r0r1r2r3.t"}, /* 11 */
  { 4, "$r4"}, /* 12 */
  { 4, "$r4r5.lo"}, /* 13 */
  { 4, "$r4r5r6r7.x"}, /* 14 */
  { 5, "$r5"}, /* 15 */
  { 5, "$r4r5.hi"}, /* 16 */
  { 5, "$r4r5r6r7.y"}, /* 17 */
  { 6, "$r6"}, /* 18 */
  { 6, "$r6r7.lo"}, /* 19 */
  { 6, "$r4r5r6r7.z"}, /* 20 */
  { 7, "$r7"}, /* 21 */
  { 7, "$r6r7.hi"}, /* 22 */
  { 7, "$r4r5r6r7.t"}, /* 23 */
  { 8, "$r8"}, /* 24 */
  { 8, "$r8r9.lo"}, /* 25 */
  { 8, "$r8r9r10r11.x"}, /* 26 */
  { 9, "$r9"}, /* 27 */
  { 9, "$r8r9.hi"}, /* 28 */
  { 9, "$r8r9r10r11.y"}, /* 29 */
  { 10, "$r10"}, /* 30 */
  { 10, "$r10r11.lo"}, /* 31 */
  { 10, "$r8r9r10r11.z"}, /* 32 */
  { 11, "$r11"}, /* 33 */
  { 11, "$r10r11.hi"}, /* 34 */
  { 11, "$r8r9r10r11.t"}, /* 35 */
  { 12, "$r12"}, /* 36 */
  { 12, "$sp"}, /* 37 */
  { 13, "$r13"}, /* 38 */
  { 13, "$tp"}, /* 39 */
  { 14, "$r14"}, /* 40 */
  { 14, "$fp"}, /* 41 */
  { 15, "$r15"}, /* 42 */
  { 15, "$rp"}, /* 43 */
  { 16, "$r16"}, /* 44 */
  { 16, "$r16r17.lo"}, /* 45 */
  { 16, "$r16r17r18r19.x"}, /* 46 */
  { 17, "$r17"}, /* 47 */
  { 17, "$r16r17.hi"}, /* 48 */
  { 17, "$r16r17r18r19.y"}, /* 49 */
  { 18, "$r18"}, /* 50 */
  { 18, "$r18r19.lo"}, /* 51 */
  { 18, "$r16r17r18r19.z"}, /* 52 */
  { 19, "$r19"}, /* 53 */
  { 19, "$r18r19.hi"}, /* 54 */
  { 19, "$r16r17r18r19.t"}, /* 55 */
  { 20, "$r20"}, /* 56 */
  { 20, "$r20r21.lo"}, /* 57 */
  { 20, "$r20r21r22r23.x"}, /* 58 */
  { 21, "$r21"}, /* 59 */
  { 21, "$r20r21.hi"}, /* 60 */
  { 21, "$r20r21r22r23.y"}, /* 61 */
  { 22, "$r22"}, /* 62 */
  { 22, "$r22r23.lo"}, /* 63 */
  { 22, "$r20r21r22r23.z"}, /* 64 */
  { 23, "$r23"}, /* 65 */
  { 23, "$r22r23.hi"}, /* 66 */
  { 23, "$r20r21r22r23.t"}, /* 67 */
  { 24, "$r24"}, /* 68 */
  { 24, "$r24r25.lo"}, /* 69 */
  { 24, "$r24r25r26r27.x"}, /* 70 */
  { 25, "$r25"}, /* 71 */
  { 25, "$r24r25.hi"}, /* 72 */
  { 25, "$r24r25r26r27.y"}, /* 73 */
  { 26, "$r26"}, /* 74 */
  { 26, "$r26r27.lo"}, /* 75 */
  { 26, "$r24r25r26r27.z"}, /* 76 */
  { 27, "$r27"}, /* 77 */
  { 27, "$r26r27.hi"}, /* 78 */
  { 27, "$r24r25r26r27.t"}, /* 79 */
  { 28, "$r28"}, /* 80 */
  { 28, "$r28r29.lo"}, /* 81 */
  { 28, "$r28r29r30r31.x"}, /* 82 */
  { 29, "$r29"}, /* 83 */
  { 29, "$r28r29.hi"}, /* 84 */
  { 29, "$r28r29r30r31.y"}, /* 85 */
  { 30, "$r30"}, /* 86 */
  { 30, "$r30r31.lo"}, /* 87 */
  { 30, "$r28r29r30r31.z"}, /* 88 */
  { 31, "$r31"}, /* 89 */
  { 31, "$r30r31.hi"}, /* 90 */
  { 31, "$r28r29r30r31.t"}, /* 91 */
  { 32, "$r32"}, /* 92 */
  { 32, "$r32r33.lo"}, /* 93 */
  { 32, "$r32r33r34r35.x"}, /* 94 */
  { 33, "$r33"}, /* 95 */
  { 33, "$r32r33.hi"}, /* 96 */
  { 33, "$r32r33r34r35.y"}, /* 97 */
  { 34, "$r34"}, /* 98 */
  { 34, "$r34r35.lo"}, /* 99 */
  { 34, "$r32r33r34r35.z"}, /* 100 */
  { 35, "$r35"}, /* 101 */
  { 35, "$r34r35.hi"}, /* 102 */
  { 35, "$r32r33r34r35.t"}, /* 103 */
  { 36, "$r36"}, /* 104 */
  { 36, "$r36r37.lo"}, /* 105 */
  { 36, "$r36r37r38r39.x"}, /* 106 */
  { 37, "$r37"}, /* 107 */
  { 37, "$r36r37.hi"}, /* 108 */
  { 37, "$r36r37r38r39.y"}, /* 109 */
  { 38, "$r38"}, /* 110 */
  { 38, "$r38r39.lo"}, /* 111 */
  { 38, "$r36r37r38r39.z"}, /* 112 */
  { 39, "$r39"}, /* 113 */
  { 39, "$r38r39.hi"}, /* 114 */
  { 39, "$r36r37r38r39.t"}, /* 115 */
  { 40, "$r40"}, /* 116 */
  { 40, "$r40r41.lo"}, /* 117 */
  { 40, "$r40r41r42r43.x"}, /* 118 */
  { 41, "$r41"}, /* 119 */
  { 41, "$r40r41.hi"}, /* 120 */
  { 41, "$r40r41r42r43.y"}, /* 121 */
  { 42, "$r42"}, /* 122 */
  { 42, "$r42r43.lo"}, /* 123 */
  { 42, "$r40r41r42r43.z"}, /* 124 */
  { 43, "$r43"}, /* 125 */
  { 43, "$r42r43.hi"}, /* 126 */
  { 43, "$r40r41r42r43.t"}, /* 127 */
  { 44, "$r44"}, /* 128 */
  { 44, "$r44r45.lo"}, /* 129 */
  { 44, "$r44r45r46r47.x"}, /* 130 */
  { 45, "$r45"}, /* 131 */
  { 45, "$r44r45.hi"}, /* 132 */
  { 45, "$r44r45r46r47.y"}, /* 133 */
  { 46, "$r46"}, /* 134 */
  { 46, "$r46r47.lo"}, /* 135 */
  { 46, "$r44r45r46r47.z"}, /* 136 */
  { 47, "$r47"}, /* 137 */
  { 47, "$r46r47.hi"}, /* 138 */
  { 47, "$r44r45r46r47.t"}, /* 139 */
  { 48, "$r48"}, /* 140 */
  { 48, "$r48r49.lo"}, /* 141 */
  { 48, "$r48r49r50r51.x"}, /* 142 */
  { 49, "$r49"}, /* 143 */
  { 49, "$r48r49.hi"}, /* 144 */
  { 49, "$r48r49r50r51.y"}, /* 145 */
  { 50, "$r50"}, /* 146 */
  { 50, "$r50r51.lo"}, /* 147 */
  { 50, "$r48r49r50r51.z"}, /* 148 */
  { 51, "$r51"}, /* 149 */
  { 51, "$r50r51.hi"}, /* 150 */
  { 51, "$r48r49r50r51.t"}, /* 151 */
  { 52, "$r52"}, /* 152 */
  { 52, "$r52r53.lo"}, /* 153 */
  { 52, "$r52r53r54r55.x"}, /* 154 */
  { 53, "$r53"}, /* 155 */
  { 53, "$r52r53.hi"}, /* 156 */
  { 53, "$r52r53r54r55.y"}, /* 157 */
  { 54, "$r54"}, /* 158 */
  { 54, "$r54r55.lo"}, /* 159 */
  { 54, "$r52r53r54r55.z"}, /* 160 */
  { 55, "$r55"}, /* 161 */
  { 55, "$r54r55.hi"}, /* 162 */
  { 55, "$r52r53r54r55.t"}, /* 163 */
  { 56, "$r56"}, /* 164 */
  { 56, "$r56r57.lo"}, /* 165 */
  { 56, "$r56r57r58r59.x"}, /* 166 */
  { 57, "$r57"}, /* 167 */
  { 57, "$r56r57.hi"}, /* 168 */
  { 57, "$r56r57r58r59.y"}, /* 169 */
  { 58, "$r58"}, /* 170 */
  { 58, "$r58r59.lo"}, /* 171 */
  { 58, "$r56r57r58r59.z"}, /* 172 */
  { 59, "$r59"}, /* 173 */
  { 59, "$r58r59.hi"}, /* 174 */
  { 59, "$r56r57r58r59.t"}, /* 175 */
  { 60, "$r60"}, /* 176 */
  { 60, "$r60r61.lo"}, /* 177 */
  { 60, "$r60r61r62r63.x"}, /* 178 */
  { 61, "$r61"}, /* 179 */
  { 61, "$r60r61.hi"}, /* 180 */
  { 61, "$r60r61r62r63.y"}, /* 181 */
  { 62, "$r62"}, /* 182 */
  { 62, "$r62r63.lo"}, /* 183 */
  { 62, "$r60r61r62r63.z"}, /* 184 */
  { 63, "$r63"}, /* 185 */
  { 63, "$r62r63.hi"}, /* 186 */
  { 63, "$r60r61r62r63.t"}, /* 187 */
  { 0, "$r0r1"}, /* 188 */
  { 0, "$r0r1r2r3.lo"}, /* 189 */
  { 1, "$r2r3"}, /* 190 */
  { 1, "$r0r1r2r3.hi"}, /* 191 */
  { 2, "$r4r5"}, /* 192 */
  { 2, "$r4r5r6r7.lo"}, /* 193 */
  { 3, "$r6r7"}, /* 194 */
  { 3, "$r4r5r6r7.hi"}, /* 195 */
  { 4, "$r8r9"}, /* 196 */
  { 4, "$r8r9r10r11.lo"}, /* 197 */
  { 5, "$r10r11"}, /* 198 */
  { 5, "$r8r9r10r11.hi"}, /* 199 */
  { 6, "$r12r13"}, /* 200 */
  { 6, "$r12r13r14r15.lo"}, /* 201 */
  { 7, "$r14r15"}, /* 202 */
  { 7, "$r12r13r14r15.hi"}, /* 203 */
  { 8, "$r16r17"}, /* 204 */
  { 8, "$r16r17r18r19.lo"}, /* 205 */
  { 9, "$r18r19"}, /* 206 */
  { 9, "$r16r17r18r19.hi"}, /* 207 */
  { 10, "$r20r21"}, /* 208 */
  { 10, "$r20r21r22r23.lo"}, /* 209 */
  { 11, "$r22r23"}, /* 210 */
  { 11, "$r20r21r22r23.hi"}, /* 211 */
  { 12, "$r24r25"}, /* 212 */
  { 12, "$r24r25r26r27.lo"}, /* 213 */
  { 13, "$r26r27"}, /* 214 */
  { 13, "$r24r25r26r27.hi"}, /* 215 */
  { 14, "$r28r29"}, /* 216 */
  { 14, "$r28r29r30r31.lo"}, /* 217 */
  { 15, "$r30r31"}, /* 218 */
  { 15, "$r28r29r30r31.hi"}, /* 219 */
  { 16, "$r32r33"}, /* 220 */
  { 16, "$r32r33r34r35.lo"}, /* 221 */
  { 17, "$r34r35"}, /* 222 */
  { 17, "$r32r33r34r35.hi"}, /* 223 */
  { 18, "$r36r37"}, /* 224 */
  { 18, "$r36r37r38r39.lo"}, /* 225 */
  { 19, "$r38r39"}, /* 226 */
  { 19, "$r36r37r38r39.hi"}, /* 227 */
  { 20, "$r40r41"}, /* 228 */
  { 20, "$r40r41r42r43.lo"}, /* 229 */
  { 21, "$r42r43"}, /* 230 */
  { 21, "$r40r41r42r43.hi"}, /* 231 */
  { 22, "$r44r45"}, /* 232 */
  { 22, "$r44r45r46r47.lo"}, /* 233 */
  { 23, "$r46r47"}, /* 234 */
  { 23, "$r44r45r46r47.hi"}, /* 235 */
  { 24, "$r48r49"}, /* 236 */
  { 24, "$r48r49r50r51.lo"}, /* 237 */
  { 25, "$r50r51"}, /* 238 */
  { 25, "$r48r49r50r51.hi"}, /* 239 */
  { 26, "$r52r53"}, /* 240 */
  { 26, "$r52r53r54r55.lo"}, /* 241 */
  { 27, "$r54r55"}, /* 242 */
  { 27, "$r52r53r54r55.hi"}, /* 243 */
  { 28, "$r56r57"}, /* 244 */
  { 28, "$r56r57r58r59.lo"}, /* 245 */
  { 29, "$r58r59"}, /* 246 */
  { 29, "$r56r57r58r59.hi"}, /* 247 */
  { 30, "$r60r61"}, /* 248 */
  { 30, "$r60r61r62r63.lo"}, /* 249 */
  { 31, "$r62r63"}, /* 250 */
  { 31, "$r60r61r62r63.hi"}, /* 251 */
  { 0, "$r0r1r2r3"}, /* 252 */
  { 1, "$r4r5r6r7"}, /* 253 */
  { 2, "$r8r9r10r11"}, /* 254 */
  { 3, "$r12r13r14r15"}, /* 255 */
  { 4, "$r16r17r18r19"}, /* 256 */
  { 5, "$r20r21r22r23"}, /* 257 */
  { 6, "$r24r25r26r27"}, /* 258 */
  { 7, "$r28r29r30r31"}, /* 259 */
  { 8, "$r32r33r34r35"}, /* 260 */
  { 9, "$r36r37r38r39"}, /* 261 */
  { 10, "$r40r41r42r43"}, /* 262 */
  { 11, "$r44r45r46r47"}, /* 263 */
  { 12, "$r48r49r50r51"}, /* 264 */
  { 13, "$r52r53r54r55"}, /* 265 */
  { 14, "$r56r57r58r59"}, /* 266 */
  { 15, "$r60r61r62r63"}, /* 267 */
  { 0, "$pc"}, /* 268 */
  { 0, "$s0"}, /* 269 */
  { 1, "$ps"}, /* 270 */
  { 1, "$s1"}, /* 271 */
  { 2, "$pcr"}, /* 272 */
  { 2, "$s2"}, /* 273 */
  { 3, "$ra"}, /* 274 */
  { 3, "$s3"}, /* 275 */
  { 4, "$cs"}, /* 276 */
  { 4, "$s4"}, /* 277 */
  { 5, "$csit"}, /* 278 */
  { 5, "$s5"}, /* 279 */
  { 6, "$aespc"}, /* 280 */
  { 6, "$s6"}, /* 281 */
  { 7, "$ls"}, /* 282 */
  { 7, "$s7"}, /* 283 */
  { 8, "$le"}, /* 284 */
  { 8, "$s8"}, /* 285 */
  { 9, "$lc"}, /* 286 */
  { 9, "$s9"}, /* 287 */
  { 10, "$ipe"}, /* 288 */
  { 10, "$s10"}, /* 289 */
  { 11, "$men"}, /* 290 */
  { 11, "$s11"}, /* 291 */
  { 12, "$pmc"}, /* 292 */
  { 12, "$s12"}, /* 293 */
  { 13, "$pm0"}, /* 294 */
  { 13, "$s13"}, /* 295 */
  { 14, "$pm1"}, /* 296 */
  { 14, "$s14"}, /* 297 */
  { 15, "$pm2"}, /* 298 */
  { 15, "$s15"}, /* 299 */
  { 16, "$pm3"}, /* 300 */
  { 16, "$s16"}, /* 301 */
  { 17, "$pmsa"}, /* 302 */
  { 17, "$s17"}, /* 303 */
  { 18, "$tcr"}, /* 304 */
  { 18, "$s18"}, /* 305 */
  { 19, "$t0v"}, /* 306 */
  { 19, "$s19"}, /* 307 */
  { 20, "$t1v"}, /* 308 */
  { 20, "$s20"}, /* 309 */
  { 21, "$t0r"}, /* 310 */
  { 21, "$s21"}, /* 311 */
  { 22, "$t1r"}, /* 312 */
  { 22, "$s22"}, /* 313 */
  { 23, "$wdv"}, /* 314 */
  { 23, "$s23"}, /* 315 */
  { 24, "$wdr"}, /* 316 */
  { 24, "$s24"}, /* 317 */
  { 25, "$ile"}, /* 318 */
  { 25, "$s25"}, /* 319 */
  { 26, "$ill"}, /* 320 */
  { 26, "$s26"}, /* 321 */
  { 27, "$ilr"}, /* 322 */
  { 27, "$s27"}, /* 323 */
  { 28, "$mmc"}, /* 324 */
  { 28, "$s28"}, /* 325 */
  { 29, "$tel"}, /* 326 */
  { 29, "$s29"}, /* 327 */
  { 30, "$teh"}, /* 328 */
  { 30, "$s30"}, /* 329 */
  { 31, "$res31"}, /* 330 */
  { 31, "$s31"}, /* 331 */
  { 32, "$syo"}, /* 332 */
  { 32, "$s32"}, /* 333 */
  { 33, "$hto"}, /* 334 */
  { 33, "$s33"}, /* 335 */
  { 34, "$ito"}, /* 336 */
  { 34, "$s34"}, /* 337 */
  { 35, "$do"}, /* 338 */
  { 35, "$s35"}, /* 339 */
  { 36, "$mo"}, /* 340 */
  { 36, "$s36"}, /* 341 */
  { 37, "$pso"}, /* 342 */
  { 37, "$s37"}, /* 343 */
  { 38, "$res38"}, /* 344 */
  { 38, "$s38"}, /* 345 */
  { 39, "$res39"}, /* 346 */
  { 39, "$s39"}, /* 347 */
  { 40, "$dc"}, /* 348 */
  { 40, "$s40"}, /* 349 */
  { 41, "$dba0"}, /* 350 */
  { 41, "$s41"}, /* 351 */
  { 42, "$dba1"}, /* 352 */
  { 42, "$s42"}, /* 353 */
  { 43, "$dwa0"}, /* 354 */
  { 43, "$s43"}, /* 355 */
  { 44, "$dwa1"}, /* 356 */
  { 44, "$s44"}, /* 357 */
  { 45, "$mes"}, /* 358 */
  { 45, "$s45"}, /* 359 */
  { 46, "$ws"}, /* 360 */
  { 46, "$s46"}, /* 361 */
  { 47, "$res47"}, /* 362 */
  { 47, "$s47"}, /* 363 */
  { 48, "$res48"}, /* 364 */
  { 48, "$s48"}, /* 365 */
  { 49, "$res49"}, /* 366 */
  { 49, "$s49"}, /* 367 */
  { 50, "$res50"}, /* 368 */
  { 50, "$s50"}, /* 369 */
  { 51, "$res51"}, /* 370 */
  { 51, "$s51"}, /* 371 */
  { 52, "$res52"}, /* 372 */
  { 52, "$s52"}, /* 373 */
  { 53, "$res53"}, /* 374 */
  { 53, "$s53"}, /* 375 */
  { 54, "$res54"}, /* 376 */
  { 54, "$s54"}, /* 377 */
  { 55, "$res55"}, /* 378 */
  { 55, "$s55"}, /* 379 */
  { 56, "$res56"}, /* 380 */
  { 56, "$s56"}, /* 381 */
  { 57, "$res57"}, /* 382 */
  { 57, "$s57"}, /* 383 */
  { 58, "$res58"}, /* 384 */
  { 58, "$s58"}, /* 385 */
  { 59, "$res59"}, /* 386 */
  { 59, "$s59"}, /* 387 */
  { 60, "$res60"}, /* 388 */
  { 60, "$s60"}, /* 389 */
  { 61, "$res61"}, /* 390 */
  { 61, "$s61"}, /* 391 */
  { 62, "$res62"}, /* 392 */
  { 62, "$s62"}, /* 393 */
  { 63, "$res63"}, /* 394 */
  { 63, "$s63"}, /* 395 */
  { 64, "$spc_pl0"}, /* 396 */
  { 64, "$s64"}, /* 397 */
  { 65, "$spc_pl1"}, /* 398 */
  { 65, "$s65"}, /* 399 */
  { 66, "$spc_pl2"}, /* 400 */
  { 66, "$s66"}, /* 401 */
  { 67, "$spc_pl3"}, /* 402 */
  { 67, "$s67"}, /* 403 */
  { 68, "$sps_pl0"}, /* 404 */
  { 68, "$s68"}, /* 405 */
  { 69, "$sps_pl1"}, /* 406 */
  { 69, "$s69"}, /* 407 */
  { 70, "$sps_pl2"}, /* 408 */
  { 70, "$s70"}, /* 409 */
  { 71, "$sps_pl3"}, /* 410 */
  { 71, "$s71"}, /* 411 */
  { 72, "$ea_pl0"}, /* 412 */
  { 72, "$s72"}, /* 413 */
  { 73, "$ea_pl1"}, /* 414 */
  { 73, "$s73"}, /* 415 */
  { 74, "$ea_pl2"}, /* 416 */
  { 74, "$s74"}, /* 417 */
  { 75, "$ea_pl3"}, /* 418 */
  { 75, "$s75"}, /* 419 */
  { 76, "$ev_pl0"}, /* 420 */
  { 76, "$s76"}, /* 421 */
  { 77, "$ev_pl1"}, /* 422 */
  { 77, "$s77"}, /* 423 */
  { 78, "$ev_pl2"}, /* 424 */
  { 78, "$s78"}, /* 425 */
  { 79, "$ev_pl3"}, /* 426 */
  { 79, "$s79"}, /* 427 */
  { 80, "$sr_pl0"}, /* 428 */
  { 80, "$s80"}, /* 429 */
  { 81, "$sr_pl1"}, /* 430 */
  { 81, "$s81"}, /* 431 */
  { 82, "$sr_pl2"}, /* 432 */
  { 82, "$s82"}, /* 433 */
  { 83, "$sr_pl3"}, /* 434 */
  { 83, "$s83"}, /* 435 */
  { 84, "$es_pl0"}, /* 436 */
  { 84, "$s84"}, /* 437 */
  { 85, "$es_pl1"}, /* 438 */
  { 85, "$s85"}, /* 439 */
  { 86, "$es_pl2"}, /* 440 */
  { 86, "$s86"}, /* 441 */
  { 87, "$es_pl3"}, /* 442 */
  { 87, "$s87"}, /* 443 */
  { 88, "$res88"}, /* 444 */
  { 88, "$s88"}, /* 445 */
  { 89, "$res89"}, /* 446 */
  { 89, "$s89"}, /* 447 */
  { 90, "$res90"}, /* 448 */
  { 90, "$s90"}, /* 449 */
  { 91, "$res91"}, /* 450 */
  { 91, "$s91"}, /* 451 */
  { 92, "$res92"}, /* 452 */
  { 92, "$s92"}, /* 453 */
  { 93, "$res93"}, /* 454 */
  { 93, "$s93"}, /* 455 */
  { 94, "$res94"}, /* 456 */
  { 94, "$s94"}, /* 457 */
  { 95, "$res95"}, /* 458 */
  { 95, "$s95"}, /* 459 */
  { 96, "$syow"}, /* 460 */
  { 96, "$s96"}, /* 461 */
  { 97, "$htow"}, /* 462 */
  { 97, "$s97"}, /* 463 */
  { 98, "$itow"}, /* 464 */
  { 98, "$s98"}, /* 465 */
  { 99, "$dow"}, /* 466 */
  { 99, "$s99"}, /* 467 */
  { 100, "$mow"}, /* 468 */
  { 100, "$s100"}, /* 469 */
  { 101, "$psow"}, /* 470 */
  { 101, "$s101"}, /* 471 */
  { 102, "$res102"}, /* 472 */
  { 102, "$s102"}, /* 473 */
  { 103, "$res103"}, /* 474 */
  { 103, "$s103"}, /* 475 */
  { 104, "$res104"}, /* 476 */
  { 104, "$s104"}, /* 477 */
  { 105, "$res105"}, /* 478 */
  { 105, "$s105"}, /* 479 */
  { 106, "$res106"}, /* 480 */
  { 106, "$s106"}, /* 481 */
  { 107, "$res107"}, /* 482 */
  { 107, "$s107"}, /* 483 */
  { 108, "$res108"}, /* 484 */
  { 108, "$s108"}, /* 485 */
  { 109, "$res109"}, /* 486 */
  { 109, "$s109"}, /* 487 */
  { 110, "$res110"}, /* 488 */
  { 110, "$s110"}, /* 489 */
  { 111, "$res111"}, /* 490 */
  { 111, "$s111"}, /* 491 */
  { 112, "$res112"}, /* 492 */
  { 112, "$s112"}, /* 493 */
  { 113, "$res113"}, /* 494 */
  { 113, "$s113"}, /* 495 */
  { 114, "$res114"}, /* 496 */
  { 114, "$s114"}, /* 497 */
  { 115, "$res115"}, /* 498 */
  { 115, "$s115"}, /* 499 */
  { 116, "$res116"}, /* 500 */
  { 116, "$s116"}, /* 501 */
  { 117, "$res117"}, /* 502 */
  { 117, "$s117"}, /* 503 */
  { 118, "$res118"}, /* 504 */
  { 118, "$s118"}, /* 505 */
  { 119, "$res119"}, /* 506 */
  { 119, "$s119"}, /* 507 */
  { 120, "$res120"}, /* 508 */
  { 120, "$s120"}, /* 509 */
  { 121, "$res121"}, /* 510 */
  { 121, "$s121"}, /* 511 */
  { 122, "$res122"}, /* 512 */
  { 122, "$s122"}, /* 513 */
  { 123, "$res123"}, /* 514 */
  { 123, "$s123"}, /* 515 */
  { 124, "$res124"}, /* 516 */
  { 124, "$s124"}, /* 517 */
  { 125, "$res125"}, /* 518 */
  { 125, "$s125"}, /* 519 */
  { 126, "$res126"}, /* 520 */
  { 126, "$s126"}, /* 521 */
  { 127, "$res127"}, /* 522 */
  { 127, "$s127"}, /* 523 */
  { 128, "$spc"}, /* 524 */
  { 128, "$s128"}, /* 525 */
  { 129, "$res129"}, /* 526 */
  { 129, "$s129"}, /* 527 */
  { 130, "$res130"}, /* 528 */
  { 130, "$s130"}, /* 529 */
  { 131, "$res131"}, /* 530 */
  { 131, "$s131"}, /* 531 */
  { 132, "$sps"}, /* 532 */
  { 132, "$s132"}, /* 533 */
  { 133, "$res133"}, /* 534 */
  { 133, "$s133"}, /* 535 */
  { 134, "$res134"}, /* 536 */
  { 134, "$s134"}, /* 537 */
  { 135, "$res135"}, /* 538 */
  { 135, "$s135"}, /* 539 */
  { 136, "$ea"}, /* 540 */
  { 136, "$s136"}, /* 541 */
  { 137, "$res137"}, /* 542 */
  { 137, "$s137"}, /* 543 */
  { 138, "$res138"}, /* 544 */
  { 138, "$s138"}, /* 545 */
  { 139, "$res139"}, /* 546 */
  { 139, "$s139"}, /* 547 */
  { 140, "$ev"}, /* 548 */
  { 140, "$s140"}, /* 549 */
  { 141, "$res141"}, /* 550 */
  { 141, "$s141"}, /* 551 */
  { 142, "$res142"}, /* 552 */
  { 142, "$s142"}, /* 553 */
  { 143, "$res143"}, /* 554 */
  { 143, "$s143"}, /* 555 */
  { 144, "$sr"}, /* 556 */
  { 144, "$s144"}, /* 557 */
  { 145, "$res145"}, /* 558 */
  { 145, "$s145"}, /* 559 */
  { 146, "$res146"}, /* 560 */
  { 146, "$s146"}, /* 561 */
  { 147, "$res147"}, /* 562 */
  { 147, "$s147"}, /* 563 */
  { 148, "$es"}, /* 564 */
  { 148, "$s148"}, /* 565 */
  { 149, "$res149"}, /* 566 */
  { 149, "$s149"}, /* 567 */
  { 150, "$res150"}, /* 568 */
  { 150, "$s150"}, /* 569 */
  { 151, "$res151"}, /* 570 */
  { 151, "$s151"}, /* 571 */
  { 152, "$res152"}, /* 572 */
  { 152, "$s152"}, /* 573 */
  { 153, "$res153"}, /* 574 */
  { 153, "$s153"}, /* 575 */
  { 154, "$res154"}, /* 576 */
  { 154, "$s154"}, /* 577 */
  { 155, "$res155"}, /* 578 */
  { 155, "$s155"}, /* 579 */
  { 156, "$res156"}, /* 580 */
  { 156, "$s156"}, /* 581 */
  { 157, "$res157"}, /* 582 */
  { 157, "$s157"}, /* 583 */
  { 158, "$res158"}, /* 584 */
  { 158, "$s158"}, /* 585 */
  { 159, "$res159"}, /* 586 */
  { 159, "$s159"}, /* 587 */
  { 160, "$res160"}, /* 588 */
  { 160, "$s160"}, /* 589 */
  { 161, "$res161"}, /* 590 */
  { 161, "$s161"}, /* 591 */
  { 162, "$res162"}, /* 592 */
  { 162, "$s162"}, /* 593 */
  { 163, "$res163"}, /* 594 */
  { 163, "$s163"}, /* 595 */
  { 164, "$res164"}, /* 596 */
  { 164, "$s164"}, /* 597 */
  { 165, "$res165"}, /* 598 */
  { 165, "$s165"}, /* 599 */
  { 166, "$res166"}, /* 600 */
  { 166, "$s166"}, /* 601 */
  { 167, "$res167"}, /* 602 */
  { 167, "$s167"}, /* 603 */
  { 168, "$res168"}, /* 604 */
  { 168, "$s168"}, /* 605 */
  { 169, "$res169"}, /* 606 */
  { 169, "$s169"}, /* 607 */
  { 170, "$res170"}, /* 608 */
  { 170, "$s170"}, /* 609 */
  { 171, "$res171"}, /* 610 */
  { 171, "$s171"}, /* 611 */
  { 172, "$res172"}, /* 612 */
  { 172, "$s172"}, /* 613 */
  { 173, "$res173"}, /* 614 */
  { 173, "$s173"}, /* 615 */
  { 174, "$res174"}, /* 616 */
  { 174, "$s174"}, /* 617 */
  { 175, "$res175"}, /* 618 */
  { 175, "$s175"}, /* 619 */
  { 176, "$res176"}, /* 620 */
  { 176, "$s176"}, /* 621 */
  { 177, "$res177"}, /* 622 */
  { 177, "$s177"}, /* 623 */
  { 178, "$res178"}, /* 624 */
  { 178, "$s178"}, /* 625 */
  { 179, "$res179"}, /* 626 */
  { 179, "$s179"}, /* 627 */
  { 180, "$res180"}, /* 628 */
  { 180, "$s180"}, /* 629 */
  { 181, "$res181"}, /* 630 */
  { 181, "$s181"}, /* 631 */
  { 182, "$res182"}, /* 632 */
  { 182, "$s182"}, /* 633 */
  { 183, "$res183"}, /* 634 */
  { 183, "$s183"}, /* 635 */
  { 184, "$res184"}, /* 636 */
  { 184, "$s184"}, /* 637 */
  { 185, "$res185"}, /* 638 */
  { 185, "$s185"}, /* 639 */
  { 186, "$res186"}, /* 640 */
  { 186, "$s186"}, /* 641 */
  { 187, "$res187"}, /* 642 */
  { 187, "$s187"}, /* 643 */
  { 188, "$res188"}, /* 644 */
  { 188, "$s188"}, /* 645 */
  { 189, "$res189"}, /* 646 */
  { 189, "$s189"}, /* 647 */
  { 190, "$res190"}, /* 648 */
  { 190, "$s190"}, /* 649 */
  { 191, "$res191"}, /* 650 */
  { 191, "$s191"}, /* 651 */
  { 192, "$res192"}, /* 652 */
  { 192, "$s192"}, /* 653 */
  { 193, "$res193"}, /* 654 */
  { 193, "$s193"}, /* 655 */
  { 194, "$res194"}, /* 656 */
  { 194, "$s194"}, /* 657 */
  { 195, "$res195"}, /* 658 */
  { 195, "$s195"}, /* 659 */
  { 196, "$res196"}, /* 660 */
  { 196, "$s196"}, /* 661 */
  { 197, "$res197"}, /* 662 */
  { 197, "$s197"}, /* 663 */
  { 198, "$res198"}, /* 664 */
  { 198, "$s198"}, /* 665 */
  { 199, "$res199"}, /* 666 */
  { 199, "$s199"}, /* 667 */
  { 200, "$res200"}, /* 668 */
  { 200, "$s200"}, /* 669 */
  { 201, "$res201"}, /* 670 */
  { 201, "$s201"}, /* 671 */
  { 202, "$res202"}, /* 672 */
  { 202, "$s202"}, /* 673 */
  { 203, "$res203"}, /* 674 */
  { 203, "$s203"}, /* 675 */
  { 204, "$res204"}, /* 676 */
  { 204, "$s204"}, /* 677 */
  { 205, "$res205"}, /* 678 */
  { 205, "$s205"}, /* 679 */
  { 206, "$res206"}, /* 680 */
  { 206, "$s206"}, /* 681 */
  { 207, "$res207"}, /* 682 */
  { 207, "$s207"}, /* 683 */
  { 208, "$res208"}, /* 684 */
  { 208, "$s208"}, /* 685 */
  { 209, "$res209"}, /* 686 */
  { 209, "$s209"}, /* 687 */
  { 210, "$res210"}, /* 688 */
  { 210, "$s210"}, /* 689 */
  { 211, "$res211"}, /* 690 */
  { 211, "$s211"}, /* 691 */
  { 212, "$res212"}, /* 692 */
  { 212, "$s212"}, /* 693 */
  { 213, "$res213"}, /* 694 */
  { 213, "$s213"}, /* 695 */
  { 214, "$res214"}, /* 696 */
  { 214, "$s214"}, /* 697 */
  { 215, "$res215"}, /* 698 */
  { 215, "$s215"}, /* 699 */
  { 216, "$res216"}, /* 700 */
  { 216, "$s216"}, /* 701 */
  { 217, "$res217"}, /* 702 */
  { 217, "$s217"}, /* 703 */
  { 218, "$res218"}, /* 704 */
  { 218, "$s218"}, /* 705 */
  { 219, "$res219"}, /* 706 */
  { 219, "$s219"}, /* 707 */
  { 220, "$res220"}, /* 708 */
  { 220, "$s220"}, /* 709 */
  { 221, "$res221"}, /* 710 */
  { 221, "$s221"}, /* 711 */
  { 222, "$res222"}, /* 712 */
  { 222, "$s222"}, /* 713 */
  { 223, "$res223"}, /* 714 */
  { 223, "$s223"}, /* 715 */
  { 224, "$res224"}, /* 716 */
  { 224, "$s224"}, /* 717 */
  { 225, "$res225"}, /* 718 */
  { 225, "$s225"}, /* 719 */
  { 226, "$res226"}, /* 720 */
  { 226, "$s226"}, /* 721 */
  { 227, "$res227"}, /* 722 */
  { 227, "$s227"}, /* 723 */
  { 228, "$res228"}, /* 724 */
  { 228, "$s228"}, /* 725 */
  { 229, "$res229"}, /* 726 */
  { 229, "$s229"}, /* 727 */
  { 230, "$res230"}, /* 728 */
  { 230, "$s230"}, /* 729 */
  { 231, "$res231"}, /* 730 */
  { 231, "$s231"}, /* 731 */
  { 232, "$res232"}, /* 732 */
  { 232, "$s232"}, /* 733 */
  { 233, "$res233"}, /* 734 */
  { 233, "$s233"}, /* 735 */
  { 234, "$res234"}, /* 736 */
  { 234, "$s234"}, /* 737 */
  { 235, "$res235"}, /* 738 */
  { 235, "$s235"}, /* 739 */
  { 236, "$res236"}, /* 740 */
  { 236, "$s236"}, /* 741 */
  { 237, "$res237"}, /* 742 */
  { 237, "$s237"}, /* 743 */
  { 238, "$res238"}, /* 744 */
  { 238, "$s238"}, /* 745 */
  { 239, "$res239"}, /* 746 */
  { 239, "$s239"}, /* 747 */
  { 240, "$res240"}, /* 748 */
  { 240, "$s240"}, /* 749 */
  { 241, "$res241"}, /* 750 */
  { 241, "$s241"}, /* 751 */
  { 242, "$res242"}, /* 752 */
  { 242, "$s242"}, /* 753 */
  { 243, "$res243"}, /* 754 */
  { 243, "$s243"}, /* 755 */
  { 244, "$res244"}, /* 756 */
  { 244, "$s244"}, /* 757 */
  { 245, "$res245"}, /* 758 */
  { 245, "$s245"}, /* 759 */
  { 246, "$res246"}, /* 760 */
  { 246, "$s246"}, /* 761 */
  { 247, "$res247"}, /* 762 */
  { 247, "$s247"}, /* 763 */
  { 248, "$res248"}, /* 764 */
  { 248, "$s248"}, /* 765 */
  { 249, "$res249"}, /* 766 */
  { 249, "$s249"}, /* 767 */
  { 250, "$res250"}, /* 768 */
  { 250, "$s250"}, /* 769 */
  { 251, "$res251"}, /* 770 */
  { 251, "$s251"}, /* 771 */
  { 252, "$res252"}, /* 772 */
  { 252, "$s252"}, /* 773 */
  { 253, "$res253"}, /* 774 */
  { 253, "$s253"}, /* 775 */
  { 254, "$res254"}, /* 776 */
  { 254, "$s254"}, /* 777 */
  { 255, "$res255"}, /* 778 */
  { 255, "$s255"}, /* 779 */
  { 256, "$vsfr0"}, /* 780 */
  { 256, "$s256"}, /* 781 */
  { 257, "$vsfr1"}, /* 782 */
  { 257, "$s257"}, /* 783 */
  { 258, "$vsfr2"}, /* 784 */
  { 258, "$s258"}, /* 785 */
  { 259, "$vsfr3"}, /* 786 */
  { 259, "$s259"}, /* 787 */
  { 260, "$vsfr4"}, /* 788 */
  { 260, "$s260"}, /* 789 */
  { 261, "$vsfr5"}, /* 790 */
  { 261, "$s261"}, /* 791 */
  { 262, "$vsfr6"}, /* 792 */
  { 262, "$s262"}, /* 793 */
  { 263, "$vsfr7"}, /* 794 */
  { 263, "$s263"}, /* 795 */
  { 264, "$vsfr8"}, /* 796 */
  { 264, "$s264"}, /* 797 */
  { 265, "$vsfr9"}, /* 798 */
  { 265, "$s265"}, /* 799 */
  { 266, "$vsfr10"}, /* 800 */
  { 266, "$s266"}, /* 801 */
  { 267, "$vsfr11"}, /* 802 */
  { 267, "$s267"}, /* 803 */
  { 268, "$vsfr12"}, /* 804 */
  { 268, "$s268"}, /* 805 */
  { 269, "$vsfr13"}, /* 806 */
  { 269, "$s269"}, /* 807 */
  { 270, "$vsfr14"}, /* 808 */
  { 270, "$s270"}, /* 809 */
  { 271, "$vsfr15"}, /* 810 */
  { 271, "$s271"}, /* 811 */
  { 272, "$vsfr16"}, /* 812 */
  { 272, "$s272"}, /* 813 */
  { 273, "$vsfr17"}, /* 814 */
  { 273, "$s273"}, /* 815 */
  { 274, "$vsfr18"}, /* 816 */
  { 274, "$s274"}, /* 817 */
  { 275, "$vsfr19"}, /* 818 */
  { 275, "$s275"}, /* 819 */
  { 276, "$vsfr20"}, /* 820 */
  { 276, "$s276"}, /* 821 */
  { 277, "$vsfr21"}, /* 822 */
  { 277, "$s277"}, /* 823 */
  { 278, "$vsfr22"}, /* 824 */
  { 278, "$s278"}, /* 825 */
  { 279, "$vsfr23"}, /* 826 */
  { 279, "$s279"}, /* 827 */
  { 280, "$vsfr24"}, /* 828 */
  { 280, "$s280"}, /* 829 */
  { 281, "$vsfr25"}, /* 830 */
  { 281, "$s281"}, /* 831 */
  { 282, "$vsfr26"}, /* 832 */
  { 282, "$s282"}, /* 833 */
  { 283, "$vsfr27"}, /* 834 */
  { 283, "$s283"}, /* 835 */
  { 284, "$vsfr28"}, /* 836 */
  { 284, "$s284"}, /* 837 */
  { 285, "$vsfr29"}, /* 838 */
  { 285, "$s285"}, /* 839 */
  { 286, "$vsfr30"}, /* 840 */
  { 286, "$s286"}, /* 841 */
  { 287, "$vsfr31"}, /* 842 */
  { 287, "$s287"}, /* 843 */
  { 288, "$vsfr32"}, /* 844 */
  { 288, "$s288"}, /* 845 */
  { 289, "$vsfr33"}, /* 846 */
  { 289, "$s289"}, /* 847 */
  { 290, "$vsfr34"}, /* 848 */
  { 290, "$s290"}, /* 849 */
  { 291, "$vsfr35"}, /* 850 */
  { 291, "$s291"}, /* 851 */
  { 292, "$vsfr36"}, /* 852 */
  { 292, "$s292"}, /* 853 */
  { 293, "$vsfr37"}, /* 854 */
  { 293, "$s293"}, /* 855 */
  { 294, "$vsfr38"}, /* 856 */
  { 294, "$s294"}, /* 857 */
  { 295, "$vsfr39"}, /* 858 */
  { 295, "$s295"}, /* 859 */
  { 296, "$vsfr40"}, /* 860 */
  { 296, "$s296"}, /* 861 */
  { 297, "$vsfr41"}, /* 862 */
  { 297, "$s297"}, /* 863 */
  { 298, "$vsfr42"}, /* 864 */
  { 298, "$s298"}, /* 865 */
  { 299, "$vsfr43"}, /* 866 */
  { 299, "$s299"}, /* 867 */
  { 300, "$vsfr44"}, /* 868 */
  { 300, "$s300"}, /* 869 */
  { 301, "$vsfr45"}, /* 870 */
  { 301, "$s301"}, /* 871 */
  { 302, "$vsfr46"}, /* 872 */
  { 302, "$s302"}, /* 873 */
  { 303, "$vsfr47"}, /* 874 */
  { 303, "$s303"}, /* 875 */
  { 304, "$vsfr48"}, /* 876 */
  { 304, "$s304"}, /* 877 */
  { 305, "$vsfr49"}, /* 878 */
  { 305, "$s305"}, /* 879 */
  { 306, "$vsfr50"}, /* 880 */
  { 306, "$s306"}, /* 881 */
  { 307, "$vsfr51"}, /* 882 */
  { 307, "$s307"}, /* 883 */
  { 308, "$vsfr52"}, /* 884 */
  { 308, "$s308"}, /* 885 */
  { 309, "$vsfr53"}, /* 886 */
  { 309, "$s309"}, /* 887 */
  { 310, "$vsfr54"}, /* 888 */
  { 310, "$s310"}, /* 889 */
  { 311, "$vsfr55"}, /* 890 */
  { 311, "$s311"}, /* 891 */
  { 312, "$vsfr56"}, /* 892 */
  { 312, "$s312"}, /* 893 */
  { 313, "$vsfr57"}, /* 894 */
  { 313, "$s313"}, /* 895 */
  { 314, "$vsfr58"}, /* 896 */
  { 314, "$s314"}, /* 897 */
  { 315, "$vsfr59"}, /* 898 */
  { 315, "$s315"}, /* 899 */
  { 316, "$vsfr60"}, /* 900 */
  { 316, "$s316"}, /* 901 */
  { 317, "$vsfr61"}, /* 902 */
  { 317, "$s317"}, /* 903 */
  { 318, "$vsfr62"}, /* 904 */
  { 318, "$s318"}, /* 905 */
  { 319, "$vsfr63"}, /* 906 */
  { 319, "$s319"}, /* 907 */
  { 320, "$vsfr64"}, /* 908 */
  { 320, "$s320"}, /* 909 */
  { 321, "$vsfr65"}, /* 910 */
  { 321, "$s321"}, /* 911 */
  { 322, "$vsfr66"}, /* 912 */
  { 322, "$s322"}, /* 913 */
  { 323, "$vsfr67"}, /* 914 */
  { 323, "$s323"}, /* 915 */
  { 324, "$vsfr68"}, /* 916 */
  { 324, "$s324"}, /* 917 */
  { 325, "$vsfr69"}, /* 918 */
  { 325, "$s325"}, /* 919 */
  { 326, "$vsfr70"}, /* 920 */
  { 326, "$s326"}, /* 921 */
  { 327, "$vsfr71"}, /* 922 */
  { 327, "$s327"}, /* 923 */
  { 328, "$vsfr72"}, /* 924 */
  { 328, "$s328"}, /* 925 */
  { 329, "$vsfr73"}, /* 926 */
  { 329, "$s329"}, /* 927 */
  { 330, "$vsfr74"}, /* 928 */
  { 330, "$s330"}, /* 929 */
  { 331, "$vsfr75"}, /* 930 */
  { 331, "$s331"}, /* 931 */
  { 332, "$vsfr76"}, /* 932 */
  { 332, "$s332"}, /* 933 */
  { 333, "$vsfr77"}, /* 934 */
  { 333, "$s333"}, /* 935 */
  { 334, "$vsfr78"}, /* 936 */
  { 334, "$s334"}, /* 937 */
  { 335, "$vsfr79"}, /* 938 */
  { 335, "$s335"}, /* 939 */
  { 336, "$vsfr80"}, /* 940 */
  { 336, "$s336"}, /* 941 */
  { 337, "$vsfr81"}, /* 942 */
  { 337, "$s337"}, /* 943 */
  { 338, "$vsfr82"}, /* 944 */
  { 338, "$s338"}, /* 945 */
  { 339, "$vsfr83"}, /* 946 */
  { 339, "$s339"}, /* 947 */
  { 340, "$vsfr84"}, /* 948 */
  { 340, "$s340"}, /* 949 */
  { 341, "$vsfr85"}, /* 950 */
  { 341, "$s341"}, /* 951 */
  { 342, "$vsfr86"}, /* 952 */
  { 342, "$s342"}, /* 953 */
  { 343, "$vsfr87"}, /* 954 */
  { 343, "$s343"}, /* 955 */
  { 344, "$vsfr88"}, /* 956 */
  { 344, "$s344"}, /* 957 */
  { 345, "$vsfr89"}, /* 958 */
  { 345, "$s345"}, /* 959 */
  { 346, "$vsfr90"}, /* 960 */
  { 346, "$s346"}, /* 961 */
  { 347, "$vsfr91"}, /* 962 */
  { 347, "$s347"}, /* 963 */
  { 348, "$vsfr92"}, /* 964 */
  { 348, "$s348"}, /* 965 */
  { 349, "$vsfr93"}, /* 966 */
  { 349, "$s349"}, /* 967 */
  { 350, "$vsfr94"}, /* 968 */
  { 350, "$s350"}, /* 969 */
  { 351, "$vsfr95"}, /* 970 */
  { 351, "$s351"}, /* 971 */
  { 352, "$vsfr96"}, /* 972 */
  { 352, "$s352"}, /* 973 */
  { 353, "$vsfr97"}, /* 974 */
  { 353, "$s353"}, /* 975 */
  { 354, "$vsfr98"}, /* 976 */
  { 354, "$s354"}, /* 977 */
  { 355, "$vsfr99"}, /* 978 */
  { 355, "$s355"}, /* 979 */
  { 356, "$vsfr100"}, /* 980 */
  { 356, "$s356"}, /* 981 */
  { 357, "$vsfr101"}, /* 982 */
  { 357, "$s357"}, /* 983 */
  { 358, "$vsfr102"}, /* 984 */
  { 358, "$s358"}, /* 985 */
  { 359, "$vsfr103"}, /* 986 */
  { 359, "$s359"}, /* 987 */
  { 360, "$vsfr104"}, /* 988 */
  { 360, "$s360"}, /* 989 */
  { 361, "$vsfr105"}, /* 990 */
  { 361, "$s361"}, /* 991 */
  { 362, "$vsfr106"}, /* 992 */
  { 362, "$s362"}, /* 993 */
  { 363, "$vsfr107"}, /* 994 */
  { 363, "$s363"}, /* 995 */
  { 364, "$vsfr108"}, /* 996 */
  { 364, "$s364"}, /* 997 */
  { 365, "$vsfr109"}, /* 998 */
  { 365, "$s365"}, /* 999 */
  { 366, "$vsfr110"}, /* 1000 */
  { 366, "$s366"}, /* 1001 */
  { 367, "$vsfr111"}, /* 1002 */
  { 367, "$s367"}, /* 1003 */
  { 368, "$vsfr112"}, /* 1004 */
  { 368, "$s368"}, /* 1005 */
  { 369, "$vsfr113"}, /* 1006 */
  { 369, "$s369"}, /* 1007 */
  { 370, "$vsfr114"}, /* 1008 */
  { 370, "$s370"}, /* 1009 */
  { 371, "$vsfr115"}, /* 1010 */
  { 371, "$s371"}, /* 1011 */
  { 372, "$vsfr116"}, /* 1012 */
  { 372, "$s372"}, /* 1013 */
  { 373, "$vsfr117"}, /* 1014 */
  { 373, "$s373"}, /* 1015 */
  { 374, "$vsfr118"}, /* 1016 */
  { 374, "$s374"}, /* 1017 */
  { 375, "$vsfr119"}, /* 1018 */
  { 375, "$s375"}, /* 1019 */
  { 376, "$vsfr120"}, /* 1020 */
  { 376, "$s376"}, /* 1021 */
  { 377, "$vsfr121"}, /* 1022 */
  { 377, "$s377"}, /* 1023 */
  { 378, "$vsfr122"}, /* 1024 */
  { 378, "$s378"}, /* 1025 */
  { 379, "$vsfr123"}, /* 1026 */
  { 379, "$s379"}, /* 1027 */
  { 380, "$vsfr124"}, /* 1028 */
  { 380, "$s380"}, /* 1029 */
  { 381, "$vsfr125"}, /* 1030 */
  { 381, "$s381"}, /* 1031 */
  { 382, "$vsfr126"}, /* 1032 */
  { 382, "$s382"}, /* 1033 */
  { 383, "$vsfr127"}, /* 1034 */
  { 383, "$s383"}, /* 1035 */
  { 384, "$vsfr128"}, /* 1036 */
  { 384, "$s384"}, /* 1037 */
  { 385, "$vsfr129"}, /* 1038 */
  { 385, "$s385"}, /* 1039 */
  { 386, "$vsfr130"}, /* 1040 */
  { 386, "$s386"}, /* 1041 */
  { 387, "$vsfr131"}, /* 1042 */
  { 387, "$s387"}, /* 1043 */
  { 388, "$vsfr132"}, /* 1044 */
  { 388, "$s388"}, /* 1045 */
  { 389, "$vsfr133"}, /* 1046 */
  { 389, "$s389"}, /* 1047 */
  { 390, "$vsfr134"}, /* 1048 */
  { 390, "$s390"}, /* 1049 */
  { 391, "$vsfr135"}, /* 1050 */
  { 391, "$s391"}, /* 1051 */
  { 392, "$vsfr136"}, /* 1052 */
  { 392, "$s392"}, /* 1053 */
  { 393, "$vsfr137"}, /* 1054 */
  { 393, "$s393"}, /* 1055 */
  { 394, "$vsfr138"}, /* 1056 */
  { 394, "$s394"}, /* 1057 */
  { 395, "$vsfr139"}, /* 1058 */
  { 395, "$s395"}, /* 1059 */
  { 396, "$vsfr140"}, /* 1060 */
  { 396, "$s396"}, /* 1061 */
  { 397, "$vsfr141"}, /* 1062 */
  { 397, "$s397"}, /* 1063 */
  { 398, "$vsfr142"}, /* 1064 */
  { 398, "$s398"}, /* 1065 */
  { 399, "$vsfr143"}, /* 1066 */
  { 399, "$s399"}, /* 1067 */
  { 400, "$vsfr144"}, /* 1068 */
  { 400, "$s400"}, /* 1069 */
  { 401, "$vsfr145"}, /* 1070 */
  { 401, "$s401"}, /* 1071 */
  { 402, "$vsfr146"}, /* 1072 */
  { 402, "$s402"}, /* 1073 */
  { 403, "$vsfr147"}, /* 1074 */
  { 403, "$s403"}, /* 1075 */
  { 404, "$vsfr148"}, /* 1076 */
  { 404, "$s404"}, /* 1077 */
  { 405, "$vsfr149"}, /* 1078 */
  { 405, "$s405"}, /* 1079 */
  { 406, "$vsfr150"}, /* 1080 */
  { 406, "$s406"}, /* 1081 */
  { 407, "$vsfr151"}, /* 1082 */
  { 407, "$s407"}, /* 1083 */
  { 408, "$vsfr152"}, /* 1084 */
  { 408, "$s408"}, /* 1085 */
  { 409, "$vsfr153"}, /* 1086 */
  { 409, "$s409"}, /* 1087 */
  { 410, "$vsfr154"}, /* 1088 */
  { 410, "$s410"}, /* 1089 */
  { 411, "$vsfr155"}, /* 1090 */
  { 411, "$s411"}, /* 1091 */
  { 412, "$vsfr156"}, /* 1092 */
  { 412, "$s412"}, /* 1093 */
  { 413, "$vsfr157"}, /* 1094 */
  { 413, "$s413"}, /* 1095 */
  { 414, "$vsfr158"}, /* 1096 */
  { 414, "$s414"}, /* 1097 */
  { 415, "$vsfr159"}, /* 1098 */
  { 415, "$s415"}, /* 1099 */
  { 416, "$vsfr160"}, /* 1100 */
  { 416, "$s416"}, /* 1101 */
  { 417, "$vsfr161"}, /* 1102 */
  { 417, "$s417"}, /* 1103 */
  { 418, "$vsfr162"}, /* 1104 */
  { 418, "$s418"}, /* 1105 */
  { 419, "$vsfr163"}, /* 1106 */
  { 419, "$s419"}, /* 1107 */
  { 420, "$vsfr164"}, /* 1108 */
  { 420, "$s420"}, /* 1109 */
  { 421, "$vsfr165"}, /* 1110 */
  { 421, "$s421"}, /* 1111 */
  { 422, "$vsfr166"}, /* 1112 */
  { 422, "$s422"}, /* 1113 */
  { 423, "$vsfr167"}, /* 1114 */
  { 423, "$s423"}, /* 1115 */
  { 424, "$vsfr168"}, /* 1116 */
  { 424, "$s424"}, /* 1117 */
  { 425, "$vsfr169"}, /* 1118 */
  { 425, "$s425"}, /* 1119 */
  { 426, "$vsfr170"}, /* 1120 */
  { 426, "$s426"}, /* 1121 */
  { 427, "$vsfr171"}, /* 1122 */
  { 427, "$s427"}, /* 1123 */
  { 428, "$vsfr172"}, /* 1124 */
  { 428, "$s428"}, /* 1125 */
  { 429, "$vsfr173"}, /* 1126 */
  { 429, "$s429"}, /* 1127 */
  { 430, "$vsfr174"}, /* 1128 */
  { 430, "$s430"}, /* 1129 */
  { 431, "$vsfr175"}, /* 1130 */
  { 431, "$s431"}, /* 1131 */
  { 432, "$vsfr176"}, /* 1132 */
  { 432, "$s432"}, /* 1133 */
  { 433, "$vsfr177"}, /* 1134 */
  { 433, "$s433"}, /* 1135 */
  { 434, "$vsfr178"}, /* 1136 */
  { 434, "$s434"}, /* 1137 */
  { 435, "$vsfr179"}, /* 1138 */
  { 435, "$s435"}, /* 1139 */
  { 436, "$vsfr180"}, /* 1140 */
  { 436, "$s436"}, /* 1141 */
  { 437, "$vsfr181"}, /* 1142 */
  { 437, "$s437"}, /* 1143 */
  { 438, "$vsfr182"}, /* 1144 */
  { 438, "$s438"}, /* 1145 */
  { 439, "$vsfr183"}, /* 1146 */
  { 439, "$s439"}, /* 1147 */
  { 440, "$vsfr184"}, /* 1148 */
  { 440, "$s440"}, /* 1149 */
  { 441, "$vsfr185"}, /* 1150 */
  { 441, "$s441"}, /* 1151 */
  { 442, "$vsfr186"}, /* 1152 */
  { 442, "$s442"}, /* 1153 */
  { 443, "$vsfr187"}, /* 1154 */
  { 443, "$s443"}, /* 1155 */
  { 444, "$vsfr188"}, /* 1156 */
  { 444, "$s444"}, /* 1157 */
  { 445, "$vsfr189"}, /* 1158 */
  { 445, "$s445"}, /* 1159 */
  { 446, "$vsfr190"}, /* 1160 */
  { 446, "$s446"}, /* 1161 */
  { 447, "$vsfr191"}, /* 1162 */
  { 447, "$s447"}, /* 1163 */
  { 448, "$vsfr192"}, /* 1164 */
  { 448, "$s448"}, /* 1165 */
  { 449, "$vsfr193"}, /* 1166 */
  { 449, "$s449"}, /* 1167 */
  { 450, "$vsfr194"}, /* 1168 */
  { 450, "$s450"}, /* 1169 */
  { 451, "$vsfr195"}, /* 1170 */
  { 451, "$s451"}, /* 1171 */
  { 452, "$vsfr196"}, /* 1172 */
  { 452, "$s452"}, /* 1173 */
  { 453, "$vsfr197"}, /* 1174 */
  { 453, "$s453"}, /* 1175 */
  { 454, "$vsfr198"}, /* 1176 */
  { 454, "$s454"}, /* 1177 */
  { 455, "$vsfr199"}, /* 1178 */
  { 455, "$s455"}, /* 1179 */
  { 456, "$vsfr200"}, /* 1180 */
  { 456, "$s456"}, /* 1181 */
  { 457, "$vsfr201"}, /* 1182 */
  { 457, "$s457"}, /* 1183 */
  { 458, "$vsfr202"}, /* 1184 */
  { 458, "$s458"}, /* 1185 */
  { 459, "$vsfr203"}, /* 1186 */
  { 459, "$s459"}, /* 1187 */
  { 460, "$vsfr204"}, /* 1188 */
  { 460, "$s460"}, /* 1189 */
  { 461, "$vsfr205"}, /* 1190 */
  { 461, "$s461"}, /* 1191 */
  { 462, "$vsfr206"}, /* 1192 */
  { 462, "$s462"}, /* 1193 */
  { 463, "$vsfr207"}, /* 1194 */
  { 463, "$s463"}, /* 1195 */
  { 464, "$vsfr208"}, /* 1196 */
  { 464, "$s464"}, /* 1197 */
  { 465, "$vsfr209"}, /* 1198 */
  { 465, "$s465"}, /* 1199 */
  { 466, "$vsfr210"}, /* 1200 */
  { 466, "$s466"}, /* 1201 */
  { 467, "$vsfr211"}, /* 1202 */
  { 467, "$s467"}, /* 1203 */
  { 468, "$vsfr212"}, /* 1204 */
  { 468, "$s468"}, /* 1205 */
  { 469, "$vsfr213"}, /* 1206 */
  { 469, "$s469"}, /* 1207 */
  { 470, "$vsfr214"}, /* 1208 */
  { 470, "$s470"}, /* 1209 */
  { 471, "$vsfr215"}, /* 1210 */
  { 471, "$s471"}, /* 1211 */
  { 472, "$vsfr216"}, /* 1212 */
  { 472, "$s472"}, /* 1213 */
  { 473, "$vsfr217"}, /* 1214 */
  { 473, "$s473"}, /* 1215 */
  { 474, "$vsfr218"}, /* 1216 */
  { 474, "$s474"}, /* 1217 */
  { 475, "$vsfr219"}, /* 1218 */
  { 475, "$s475"}, /* 1219 */
  { 476, "$vsfr220"}, /* 1220 */
  { 476, "$s476"}, /* 1221 */
  { 477, "$vsfr221"}, /* 1222 */
  { 477, "$s477"}, /* 1223 */
  { 478, "$vsfr222"}, /* 1224 */
  { 478, "$s478"}, /* 1225 */
  { 479, "$vsfr223"}, /* 1226 */
  { 479, "$s479"}, /* 1227 */
  { 480, "$vsfr224"}, /* 1228 */
  { 480, "$s480"}, /* 1229 */
  { 481, "$vsfr225"}, /* 1230 */
  { 481, "$s481"}, /* 1231 */
  { 482, "$vsfr226"}, /* 1232 */
  { 482, "$s482"}, /* 1233 */
  { 483, "$vsfr227"}, /* 1234 */
  { 483, "$s483"}, /* 1235 */
  { 484, "$vsfr228"}, /* 1236 */
  { 484, "$s484"}, /* 1237 */
  { 485, "$vsfr229"}, /* 1238 */
  { 485, "$s485"}, /* 1239 */
  { 486, "$vsfr230"}, /* 1240 */
  { 486, "$s486"}, /* 1241 */
  { 487, "$vsfr231"}, /* 1242 */
  { 487, "$s487"}, /* 1243 */
  { 488, "$vsfr232"}, /* 1244 */
  { 488, "$s488"}, /* 1245 */
  { 489, "$vsfr233"}, /* 1246 */
  { 489, "$s489"}, /* 1247 */
  { 490, "$vsfr234"}, /* 1248 */
  { 490, "$s490"}, /* 1249 */
  { 491, "$vsfr235"}, /* 1250 */
  { 491, "$s491"}, /* 1251 */
  { 492, "$vsfr236"}, /* 1252 */
  { 492, "$s492"}, /* 1253 */
  { 493, "$vsfr237"}, /* 1254 */
  { 493, "$s493"}, /* 1255 */
  { 494, "$vsfr238"}, /* 1256 */
  { 494, "$s494"}, /* 1257 */
  { 495, "$vsfr239"}, /* 1258 */
  { 495, "$s495"}, /* 1259 */
  { 496, "$vsfr240"}, /* 1260 */
  { 496, "$s496"}, /* 1261 */
  { 497, "$vsfr241"}, /* 1262 */
  { 497, "$s497"}, /* 1263 */
  { 498, "$vsfr242"}, /* 1264 */
  { 498, "$s498"}, /* 1265 */
  { 499, "$vsfr243"}, /* 1266 */
  { 499, "$s499"}, /* 1267 */
  { 500, "$vsfr244"}, /* 1268 */
  { 500, "$s500"}, /* 1269 */
  { 501, "$vsfr245"}, /* 1270 */
  { 501, "$s501"}, /* 1271 */
  { 502, "$vsfr246"}, /* 1272 */
  { 502, "$s502"}, /* 1273 */
  { 503, "$vsfr247"}, /* 1274 */
  { 503, "$s503"}, /* 1275 */
  { 504, "$vsfr248"}, /* 1276 */
  { 504, "$s504"}, /* 1277 */
  { 505, "$vsfr249"}, /* 1278 */
  { 505, "$s505"}, /* 1279 */
  { 506, "$vsfr250"}, /* 1280 */
  { 506, "$s506"}, /* 1281 */
  { 507, "$vsfr251"}, /* 1282 */
  { 507, "$s507"}, /* 1283 */
  { 508, "$vsfr252"}, /* 1284 */
  { 508, "$s508"}, /* 1285 */
  { 509, "$vsfr253"}, /* 1286 */
  { 509, "$s509"}, /* 1287 */
  { 510, "$vsfr254"}, /* 1288 */
  { 510, "$s510"}, /* 1289 */
  { 511, "$vsfr255"}, /* 1290 */
  { 511, "$s511"}, /* 1291 */
  { 0, "$a0_lo"}, /* 1292 */
  { 0, "$a0.lo"}, /* 1293 */
  { 1, "$a0_hi"}, /* 1294 */
  { 1, "$a0.hi"}, /* 1295 */
  { 2, "$a1_lo"}, /* 1296 */
  { 2, "$a1.lo"}, /* 1297 */
  { 3, "$a1_hi"}, /* 1298 */
  { 3, "$a1.hi"}, /* 1299 */
  { 4, "$a2_lo"}, /* 1300 */
  { 4, "$a2.lo"}, /* 1301 */
  { 5, "$a2_hi"}, /* 1302 */
  { 5, "$a2.hi"}, /* 1303 */
  { 6, "$a3_lo"}, /* 1304 */
  { 6, "$a3.lo"}, /* 1305 */
  { 7, "$a3_hi"}, /* 1306 */
  { 7, "$a3.hi"}, /* 1307 */
  { 8, "$a4_lo"}, /* 1308 */
  { 8, "$a4.lo"}, /* 1309 */
  { 9, "$a4_hi"}, /* 1310 */
  { 9, "$a4.hi"}, /* 1311 */
  { 10, "$a5_lo"}, /* 1312 */
  { 10, "$a5.lo"}, /* 1313 */
  { 11, "$a5_hi"}, /* 1314 */
  { 11, "$a5.hi"}, /* 1315 */
  { 12, "$a6_lo"}, /* 1316 */
  { 12, "$a6.lo"}, /* 1317 */
  { 13, "$a6_hi"}, /* 1318 */
  { 13, "$a6.hi"}, /* 1319 */
  { 14, "$a7_lo"}, /* 1320 */
  { 14, "$a7.lo"}, /* 1321 */
  { 15, "$a7_hi"}, /* 1322 */
  { 15, "$a7.hi"}, /* 1323 */
  { 16, "$a8_lo"}, /* 1324 */
  { 16, "$a8.lo"}, /* 1325 */
  { 17, "$a8_hi"}, /* 1326 */
  { 17, "$a8.hi"}, /* 1327 */
  { 18, "$a9_lo"}, /* 1328 */
  { 18, "$a9.lo"}, /* 1329 */
  { 19, "$a9_hi"}, /* 1330 */
  { 19, "$a9.hi"}, /* 1331 */
  { 20, "$a10_lo"}, /* 1332 */
  { 20, "$a10.lo"}, /* 1333 */
  { 21, "$a10_hi"}, /* 1334 */
  { 21, "$a10.hi"}, /* 1335 */
  { 22, "$a11_lo"}, /* 1336 */
  { 22, "$a11.lo"}, /* 1337 */
  { 23, "$a11_hi"}, /* 1338 */
  { 23, "$a11.hi"}, /* 1339 */
  { 24, "$a12_lo"}, /* 1340 */
  { 24, "$a12.lo"}, /* 1341 */
  { 25, "$a12_hi"}, /* 1342 */
  { 25, "$a12.hi"}, /* 1343 */
  { 26, "$a13_lo"}, /* 1344 */
  { 26, "$a13.lo"}, /* 1345 */
  { 27, "$a13_hi"}, /* 1346 */
  { 27, "$a13.hi"}, /* 1347 */
  { 28, "$a14_lo"}, /* 1348 */
  { 28, "$a14.lo"}, /* 1349 */
  { 29, "$a14_hi"}, /* 1350 */
  { 29, "$a14.hi"}, /* 1351 */
  { 30, "$a15_lo"}, /* 1352 */
  { 30, "$a15.lo"}, /* 1353 */
  { 31, "$a15_hi"}, /* 1354 */
  { 31, "$a15.hi"}, /* 1355 */
  { 32, "$a16_lo"}, /* 1356 */
  { 32, "$a16.lo"}, /* 1357 */
  { 33, "$a16_hi"}, /* 1358 */
  { 33, "$a16.hi"}, /* 1359 */
  { 34, "$a17_lo"}, /* 1360 */
  { 34, "$a17.lo"}, /* 1361 */
  { 35, "$a17_hi"}, /* 1362 */
  { 35, "$a17.hi"}, /* 1363 */
  { 36, "$a18_lo"}, /* 1364 */
  { 36, "$a18.lo"}, /* 1365 */
  { 37, "$a18_hi"}, /* 1366 */
  { 37, "$a18.hi"}, /* 1367 */
  { 38, "$a19_lo"}, /* 1368 */
  { 38, "$a19.lo"}, /* 1369 */
  { 39, "$a19_hi"}, /* 1370 */
  { 39, "$a19.hi"}, /* 1371 */
  { 40, "$a20_lo"}, /* 1372 */
  { 40, "$a20.lo"}, /* 1373 */
  { 41, "$a20_hi"}, /* 1374 */
  { 41, "$a20.hi"}, /* 1375 */
  { 42, "$a21_lo"}, /* 1376 */
  { 42, "$a21.lo"}, /* 1377 */
  { 43, "$a21_hi"}, /* 1378 */
  { 43, "$a21.hi"}, /* 1379 */
  { 44, "$a22_lo"}, /* 1380 */
  { 44, "$a22.lo"}, /* 1381 */
  { 45, "$a22_hi"}, /* 1382 */
  { 45, "$a22.hi"}, /* 1383 */
  { 46, "$a23_lo"}, /* 1384 */
  { 46, "$a23.lo"}, /* 1385 */
  { 47, "$a23_hi"}, /* 1386 */
  { 47, "$a23.hi"}, /* 1387 */
  { 48, "$a24_lo"}, /* 1388 */
  { 48, "$a24.lo"}, /* 1389 */
  { 49, "$a24_hi"}, /* 1390 */
  { 49, "$a24.hi"}, /* 1391 */
  { 50, "$a25_lo"}, /* 1392 */
  { 50, "$a25.lo"}, /* 1393 */
  { 51, "$a25_hi"}, /* 1394 */
  { 51, "$a25.hi"}, /* 1395 */
  { 52, "$a26_lo"}, /* 1396 */
  { 52, "$a26.lo"}, /* 1397 */
  { 53, "$a26_hi"}, /* 1398 */
  { 53, "$a26.hi"}, /* 1399 */
  { 54, "$a27_lo"}, /* 1400 */
  { 54, "$a27.lo"}, /* 1401 */
  { 55, "$a27_hi"}, /* 1402 */
  { 55, "$a27.hi"}, /* 1403 */
  { 56, "$a28_lo"}, /* 1404 */
  { 56, "$a28.lo"}, /* 1405 */
  { 57, "$a28_hi"}, /* 1406 */
  { 57, "$a28.hi"}, /* 1407 */
  { 58, "$a29_lo"}, /* 1408 */
  { 58, "$a29.lo"}, /* 1409 */
  { 59, "$a29_hi"}, /* 1410 */
  { 59, "$a29.hi"}, /* 1411 */
  { 60, "$a30_lo"}, /* 1412 */
  { 60, "$a30.lo"}, /* 1413 */
  { 61, "$a30_hi"}, /* 1414 */
  { 61, "$a30.hi"}, /* 1415 */
  { 62, "$a31_lo"}, /* 1416 */
  { 62, "$a31.lo"}, /* 1417 */
  { 63, "$a31_hi"}, /* 1418 */
  { 63, "$a31.hi"}, /* 1419 */
  { 64, "$a32_lo"}, /* 1420 */
  { 64, "$a32.lo"}, /* 1421 */
  { 65, "$a32_hi"}, /* 1422 */
  { 65, "$a32.hi"}, /* 1423 */
  { 66, "$a33_lo"}, /* 1424 */
  { 66, "$a33.lo"}, /* 1425 */
  { 67, "$a33_hi"}, /* 1426 */
  { 67, "$a33.hi"}, /* 1427 */
  { 68, "$a34_lo"}, /* 1428 */
  { 68, "$a34.lo"}, /* 1429 */
  { 69, "$a34_hi"}, /* 1430 */
  { 69, "$a34.hi"}, /* 1431 */
  { 70, "$a35_lo"}, /* 1432 */
  { 70, "$a35.lo"}, /* 1433 */
  { 71, "$a35_hi"}, /* 1434 */
  { 71, "$a35.hi"}, /* 1435 */
  { 72, "$a36_lo"}, /* 1436 */
  { 72, "$a36.lo"}, /* 1437 */
  { 73, "$a36_hi"}, /* 1438 */
  { 73, "$a36.hi"}, /* 1439 */
  { 74, "$a37_lo"}, /* 1440 */
  { 74, "$a37.lo"}, /* 1441 */
  { 75, "$a37_hi"}, /* 1442 */
  { 75, "$a37.hi"}, /* 1443 */
  { 76, "$a38_lo"}, /* 1444 */
  { 76, "$a38.lo"}, /* 1445 */
  { 77, "$a38_hi"}, /* 1446 */
  { 77, "$a38.hi"}, /* 1447 */
  { 78, "$a39_lo"}, /* 1448 */
  { 78, "$a39.lo"}, /* 1449 */
  { 79, "$a39_hi"}, /* 1450 */
  { 79, "$a39.hi"}, /* 1451 */
  { 80, "$a40_lo"}, /* 1452 */
  { 80, "$a40.lo"}, /* 1453 */
  { 81, "$a40_hi"}, /* 1454 */
  { 81, "$a40.hi"}, /* 1455 */
  { 82, "$a41_lo"}, /* 1456 */
  { 82, "$a41.lo"}, /* 1457 */
  { 83, "$a41_hi"}, /* 1458 */
  { 83, "$a41.hi"}, /* 1459 */
  { 84, "$a42_lo"}, /* 1460 */
  { 84, "$a42.lo"}, /* 1461 */
  { 85, "$a42_hi"}, /* 1462 */
  { 85, "$a42.hi"}, /* 1463 */
  { 86, "$a43_lo"}, /* 1464 */
  { 86, "$a43.lo"}, /* 1465 */
  { 87, "$a43_hi"}, /* 1466 */
  { 87, "$a43.hi"}, /* 1467 */
  { 88, "$a44_lo"}, /* 1468 */
  { 88, "$a44.lo"}, /* 1469 */
  { 89, "$a44_hi"}, /* 1470 */
  { 89, "$a44.hi"}, /* 1471 */
  { 90, "$a45_lo"}, /* 1472 */
  { 90, "$a45.lo"}, /* 1473 */
  { 91, "$a45_hi"}, /* 1474 */
  { 91, "$a45.hi"}, /* 1475 */
  { 92, "$a46_lo"}, /* 1476 */
  { 92, "$a46.lo"}, /* 1477 */
  { 93, "$a46_hi"}, /* 1478 */
  { 93, "$a46.hi"}, /* 1479 */
  { 94, "$a47_lo"}, /* 1480 */
  { 94, "$a47.lo"}, /* 1481 */
  { 95, "$a47_hi"}, /* 1482 */
  { 95, "$a47.hi"}, /* 1483 */
  { 96, "$a48_lo"}, /* 1484 */
  { 96, "$a48.lo"}, /* 1485 */
  { 97, "$a48_hi"}, /* 1486 */
  { 97, "$a48.hi"}, /* 1487 */
  { 98, "$a49_lo"}, /* 1488 */
  { 98, "$a49.lo"}, /* 1489 */
  { 99, "$a49_hi"}, /* 1490 */
  { 99, "$a49.hi"}, /* 1491 */
  { 100, "$a50_lo"}, /* 1492 */
  { 100, "$a50.lo"}, /* 1493 */
  { 101, "$a50_hi"}, /* 1494 */
  { 101, "$a50.hi"}, /* 1495 */
  { 102, "$a51_lo"}, /* 1496 */
  { 102, "$a51.lo"}, /* 1497 */
  { 103, "$a51_hi"}, /* 1498 */
  { 103, "$a51.hi"}, /* 1499 */
  { 104, "$a52_lo"}, /* 1500 */
  { 104, "$a52.lo"}, /* 1501 */
  { 105, "$a52_hi"}, /* 1502 */
  { 105, "$a52.hi"}, /* 1503 */
  { 106, "$a53_lo"}, /* 1504 */
  { 106, "$a53.lo"}, /* 1505 */
  { 107, "$a53_hi"}, /* 1506 */
  { 107, "$a53.hi"}, /* 1507 */
  { 108, "$a54_lo"}, /* 1508 */
  { 108, "$a54.lo"}, /* 1509 */
  { 109, "$a54_hi"}, /* 1510 */
  { 109, "$a54.hi"}, /* 1511 */
  { 110, "$a55_lo"}, /* 1512 */
  { 110, "$a55.lo"}, /* 1513 */
  { 111, "$a55_hi"}, /* 1514 */
  { 111, "$a55.hi"}, /* 1515 */
  { 112, "$a56_lo"}, /* 1516 */
  { 112, "$a56.lo"}, /* 1517 */
  { 113, "$a56_hi"}, /* 1518 */
  { 113, "$a56.hi"}, /* 1519 */
  { 114, "$a57_lo"}, /* 1520 */
  { 114, "$a57.lo"}, /* 1521 */
  { 115, "$a57_hi"}, /* 1522 */
  { 115, "$a57.hi"}, /* 1523 */
  { 116, "$a58_lo"}, /* 1524 */
  { 116, "$a58.lo"}, /* 1525 */
  { 117, "$a58_hi"}, /* 1526 */
  { 117, "$a58.hi"}, /* 1527 */
  { 118, "$a59_lo"}, /* 1528 */
  { 118, "$a59.lo"}, /* 1529 */
  { 119, "$a59_hi"}, /* 1530 */
  { 119, "$a59.hi"}, /* 1531 */
  { 120, "$a60_lo"}, /* 1532 */
  { 120, "$a60.lo"}, /* 1533 */
  { 121, "$a60_hi"}, /* 1534 */
  { 121, "$a60.hi"}, /* 1535 */
  { 122, "$a61_lo"}, /* 1536 */
  { 122, "$a61.lo"}, /* 1537 */
  { 123, "$a61_hi"}, /* 1538 */
  { 123, "$a61.hi"}, /* 1539 */
  { 124, "$a62_lo"}, /* 1540 */
  { 124, "$a62.lo"}, /* 1541 */
  { 125, "$a62_hi"}, /* 1542 */
  { 125, "$a62.hi"}, /* 1543 */
  { 126, "$a63_lo"}, /* 1544 */
  { 126, "$a63.lo"}, /* 1545 */
  { 127, "$a63_hi"}, /* 1546 */
  { 127, "$a63.hi"}, /* 1547 */
  { 0, "$a0_x"}, /* 1548 */
  { 0, "$a0.x"}, /* 1549 */
  { 1, "$a0_y"}, /* 1550 */
  { 1, "$a0.y"}, /* 1551 */
  { 2, "$a0_z"}, /* 1552 */
  { 2, "$a0.z"}, /* 1553 */
  { 3, "$a0_t"}, /* 1554 */
  { 3, "$a0.t"}, /* 1555 */
  { 4, "$a1_x"}, /* 1556 */
  { 4, "$a1.x"}, /* 1557 */
  { 5, "$a1_y"}, /* 1558 */
  { 5, "$a1.y"}, /* 1559 */
  { 6, "$a1_z"}, /* 1560 */
  { 6, "$a1.z"}, /* 1561 */
  { 7, "$a1_t"}, /* 1562 */
  { 7, "$a1.t"}, /* 1563 */
  { 8, "$a2_x"}, /* 1564 */
  { 8, "$a2.x"}, /* 1565 */
  { 9, "$a2_y"}, /* 1566 */
  { 9, "$a2.y"}, /* 1567 */
  { 10, "$a2_z"}, /* 1568 */
  { 10, "$a2.z"}, /* 1569 */
  { 11, "$a2_t"}, /* 1570 */
  { 11, "$a2.t"}, /* 1571 */
  { 12, "$a3_x"}, /* 1572 */
  { 12, "$a3.x"}, /* 1573 */
  { 13, "$a3_y"}, /* 1574 */
  { 13, "$a3.y"}, /* 1575 */
  { 14, "$a3_z"}, /* 1576 */
  { 14, "$a3.z"}, /* 1577 */
  { 15, "$a3_t"}, /* 1578 */
  { 15, "$a3.t"}, /* 1579 */
  { 16, "$a4_x"}, /* 1580 */
  { 16, "$a4.x"}, /* 1581 */
  { 17, "$a4_y"}, /* 1582 */
  { 17, "$a4.y"}, /* 1583 */
  { 18, "$a4_z"}, /* 1584 */
  { 18, "$a4.z"}, /* 1585 */
  { 19, "$a4_t"}, /* 1586 */
  { 19, "$a4.t"}, /* 1587 */
  { 20, "$a5_x"}, /* 1588 */
  { 20, "$a5.x"}, /* 1589 */
  { 21, "$a5_y"}, /* 1590 */
  { 21, "$a5.y"}, /* 1591 */
  { 22, "$a5_z"}, /* 1592 */
  { 22, "$a5.z"}, /* 1593 */
  { 23, "$a5_t"}, /* 1594 */
  { 23, "$a5.t"}, /* 1595 */
  { 24, "$a6_x"}, /* 1596 */
  { 24, "$a6.x"}, /* 1597 */
  { 25, "$a6_y"}, /* 1598 */
  { 25, "$a6.y"}, /* 1599 */
  { 26, "$a6_z"}, /* 1600 */
  { 26, "$a6.z"}, /* 1601 */
  { 27, "$a6_t"}, /* 1602 */
  { 27, "$a6.t"}, /* 1603 */
  { 28, "$a7_x"}, /* 1604 */
  { 28, "$a7.x"}, /* 1605 */
  { 29, "$a7_y"}, /* 1606 */
  { 29, "$a7.y"}, /* 1607 */
  { 30, "$a7_z"}, /* 1608 */
  { 30, "$a7.z"}, /* 1609 */
  { 31, "$a7_t"}, /* 1610 */
  { 31, "$a7.t"}, /* 1611 */
  { 32, "$a8_x"}, /* 1612 */
  { 32, "$a8.x"}, /* 1613 */
  { 33, "$a8_y"}, /* 1614 */
  { 33, "$a8.y"}, /* 1615 */
  { 34, "$a8_z"}, /* 1616 */
  { 34, "$a8.z"}, /* 1617 */
  { 35, "$a8_t"}, /* 1618 */
  { 35, "$a8.t"}, /* 1619 */
  { 36, "$a9_x"}, /* 1620 */
  { 36, "$a9.x"}, /* 1621 */
  { 37, "$a9_y"}, /* 1622 */
  { 37, "$a9.y"}, /* 1623 */
  { 38, "$a9_z"}, /* 1624 */
  { 38, "$a9.z"}, /* 1625 */
  { 39, "$a9_t"}, /* 1626 */
  { 39, "$a9.t"}, /* 1627 */
  { 40, "$a10_x"}, /* 1628 */
  { 40, "$a10.x"}, /* 1629 */
  { 41, "$a10_y"}, /* 1630 */
  { 41, "$a10.y"}, /* 1631 */
  { 42, "$a10_z"}, /* 1632 */
  { 42, "$a10.z"}, /* 1633 */
  { 43, "$a10_t"}, /* 1634 */
  { 43, "$a10.t"}, /* 1635 */
  { 44, "$a11_x"}, /* 1636 */
  { 44, "$a11.x"}, /* 1637 */
  { 45, "$a11_y"}, /* 1638 */
  { 45, "$a11.y"}, /* 1639 */
  { 46, "$a11_z"}, /* 1640 */
  { 46, "$a11.z"}, /* 1641 */
  { 47, "$a11_t"}, /* 1642 */
  { 47, "$a11.t"}, /* 1643 */
  { 48, "$a12_x"}, /* 1644 */
  { 48, "$a12.x"}, /* 1645 */
  { 49, "$a12_y"}, /* 1646 */
  { 49, "$a12.y"}, /* 1647 */
  { 50, "$a12_z"}, /* 1648 */
  { 50, "$a12.z"}, /* 1649 */
  { 51, "$a12_t"}, /* 1650 */
  { 51, "$a12.t"}, /* 1651 */
  { 52, "$a13_x"}, /* 1652 */
  { 52, "$a13.x"}, /* 1653 */
  { 53, "$a13_y"}, /* 1654 */
  { 53, "$a13.y"}, /* 1655 */
  { 54, "$a13_z"}, /* 1656 */
  { 54, "$a13.z"}, /* 1657 */
  { 55, "$a13_t"}, /* 1658 */
  { 55, "$a13.t"}, /* 1659 */
  { 56, "$a14_x"}, /* 1660 */
  { 56, "$a14.x"}, /* 1661 */
  { 57, "$a14_y"}, /* 1662 */
  { 57, "$a14.y"}, /* 1663 */
  { 58, "$a14_z"}, /* 1664 */
  { 58, "$a14.z"}, /* 1665 */
  { 59, "$a14_t"}, /* 1666 */
  { 59, "$a14.t"}, /* 1667 */
  { 60, "$a15_x"}, /* 1668 */
  { 60, "$a15.x"}, /* 1669 */
  { 61, "$a15_y"}, /* 1670 */
  { 61, "$a15.y"}, /* 1671 */
  { 62, "$a15_z"}, /* 1672 */
  { 62, "$a15.z"}, /* 1673 */
  { 63, "$a15_t"}, /* 1674 */
  { 63, "$a15.t"}, /* 1675 */
  { 64, "$a16_x"}, /* 1676 */
  { 64, "$a16.x"}, /* 1677 */
  { 65, "$a16_y"}, /* 1678 */
  { 65, "$a16.y"}, /* 1679 */
  { 66, "$a16_z"}, /* 1680 */
  { 66, "$a16.z"}, /* 1681 */
  { 67, "$a16_t"}, /* 1682 */
  { 67, "$a16.t"}, /* 1683 */
  { 68, "$a17_x"}, /* 1684 */
  { 68, "$a17.x"}, /* 1685 */
  { 69, "$a17_y"}, /* 1686 */
  { 69, "$a17.y"}, /* 1687 */
  { 70, "$a17_z"}, /* 1688 */
  { 70, "$a17.z"}, /* 1689 */
  { 71, "$a17_t"}, /* 1690 */
  { 71, "$a17.t"}, /* 1691 */
  { 72, "$a18_x"}, /* 1692 */
  { 72, "$a18.x"}, /* 1693 */
  { 73, "$a18_y"}, /* 1694 */
  { 73, "$a18.y"}, /* 1695 */
  { 74, "$a18_z"}, /* 1696 */
  { 74, "$a18.z"}, /* 1697 */
  { 75, "$a18_t"}, /* 1698 */
  { 75, "$a18.t"}, /* 1699 */
  { 76, "$a19_x"}, /* 1700 */
  { 76, "$a19.x"}, /* 1701 */
  { 77, "$a19_y"}, /* 1702 */
  { 77, "$a19.y"}, /* 1703 */
  { 78, "$a19_z"}, /* 1704 */
  { 78, "$a19.z"}, /* 1705 */
  { 79, "$a19_t"}, /* 1706 */
  { 79, "$a19.t"}, /* 1707 */
  { 80, "$a20_x"}, /* 1708 */
  { 80, "$a20.x"}, /* 1709 */
  { 81, "$a20_y"}, /* 1710 */
  { 81, "$a20.y"}, /* 1711 */
  { 82, "$a20_z"}, /* 1712 */
  { 82, "$a20.z"}, /* 1713 */
  { 83, "$a20_t"}, /* 1714 */
  { 83, "$a20.t"}, /* 1715 */
  { 84, "$a21_x"}, /* 1716 */
  { 84, "$a21.x"}, /* 1717 */
  { 85, "$a21_y"}, /* 1718 */
  { 85, "$a21.y"}, /* 1719 */
  { 86, "$a21_z"}, /* 1720 */
  { 86, "$a21.z"}, /* 1721 */
  { 87, "$a21_t"}, /* 1722 */
  { 87, "$a21.t"}, /* 1723 */
  { 88, "$a22_x"}, /* 1724 */
  { 88, "$a22.x"}, /* 1725 */
  { 89, "$a22_y"}, /* 1726 */
  { 89, "$a22.y"}, /* 1727 */
  { 90, "$a22_z"}, /* 1728 */
  { 90, "$a22.z"}, /* 1729 */
  { 91, "$a22_t"}, /* 1730 */
  { 91, "$a22.t"}, /* 1731 */
  { 92, "$a23_x"}, /* 1732 */
  { 92, "$a23.x"}, /* 1733 */
  { 93, "$a23_y"}, /* 1734 */
  { 93, "$a23.y"}, /* 1735 */
  { 94, "$a23_z"}, /* 1736 */
  { 94, "$a23.z"}, /* 1737 */
  { 95, "$a23_t"}, /* 1738 */
  { 95, "$a23.t"}, /* 1739 */
  { 96, "$a24_x"}, /* 1740 */
  { 96, "$a24.x"}, /* 1741 */
  { 97, "$a24_y"}, /* 1742 */
  { 97, "$a24.y"}, /* 1743 */
  { 98, "$a24_z"}, /* 1744 */
  { 98, "$a24.z"}, /* 1745 */
  { 99, "$a24_t"}, /* 1746 */
  { 99, "$a24.t"}, /* 1747 */
  { 100, "$a25_x"}, /* 1748 */
  { 100, "$a25.x"}, /* 1749 */
  { 101, "$a25_y"}, /* 1750 */
  { 101, "$a25.y"}, /* 1751 */
  { 102, "$a25_z"}, /* 1752 */
  { 102, "$a25.z"}, /* 1753 */
  { 103, "$a25_t"}, /* 1754 */
  { 103, "$a25.t"}, /* 1755 */
  { 104, "$a26_x"}, /* 1756 */
  { 104, "$a26.x"}, /* 1757 */
  { 105, "$a26_y"}, /* 1758 */
  { 105, "$a26.y"}, /* 1759 */
  { 106, "$a26_z"}, /* 1760 */
  { 106, "$a26.z"}, /* 1761 */
  { 107, "$a26_t"}, /* 1762 */
  { 107, "$a26.t"}, /* 1763 */
  { 108, "$a27_x"}, /* 1764 */
  { 108, "$a27.x"}, /* 1765 */
  { 109, "$a27_y"}, /* 1766 */
  { 109, "$a27.y"}, /* 1767 */
  { 110, "$a27_z"}, /* 1768 */
  { 110, "$a27.z"}, /* 1769 */
  { 111, "$a27_t"}, /* 1770 */
  { 111, "$a27.t"}, /* 1771 */
  { 112, "$a28_x"}, /* 1772 */
  { 112, "$a28.x"}, /* 1773 */
  { 113, "$a28_y"}, /* 1774 */
  { 113, "$a28.y"}, /* 1775 */
  { 114, "$a28_z"}, /* 1776 */
  { 114, "$a28.z"}, /* 1777 */
  { 115, "$a28_t"}, /* 1778 */
  { 115, "$a28.t"}, /* 1779 */
  { 116, "$a29_x"}, /* 1780 */
  { 116, "$a29.x"}, /* 1781 */
  { 117, "$a29_y"}, /* 1782 */
  { 117, "$a29.y"}, /* 1783 */
  { 118, "$a29_z"}, /* 1784 */
  { 118, "$a29.z"}, /* 1785 */
  { 119, "$a29_t"}, /* 1786 */
  { 119, "$a29.t"}, /* 1787 */
  { 120, "$a30_x"}, /* 1788 */
  { 120, "$a30.x"}, /* 1789 */
  { 121, "$a30_y"}, /* 1790 */
  { 121, "$a30.y"}, /* 1791 */
  { 122, "$a30_z"}, /* 1792 */
  { 122, "$a30.z"}, /* 1793 */
  { 123, "$a30_t"}, /* 1794 */
  { 123, "$a30.t"}, /* 1795 */
  { 124, "$a31_x"}, /* 1796 */
  { 124, "$a31.x"}, /* 1797 */
  { 125, "$a31_y"}, /* 1798 */
  { 125, "$a31.y"}, /* 1799 */
  { 126, "$a31_z"}, /* 1800 */
  { 126, "$a31.z"}, /* 1801 */
  { 127, "$a31_t"}, /* 1802 */
  { 127, "$a31.t"}, /* 1803 */
  { 128, "$a32_x"}, /* 1804 */
  { 128, "$a32.x"}, /* 1805 */
  { 129, "$a32_y"}, /* 1806 */
  { 129, "$a32.y"}, /* 1807 */
  { 130, "$a32_z"}, /* 1808 */
  { 130, "$a32.z"}, /* 1809 */
  { 131, "$a32_t"}, /* 1810 */
  { 131, "$a32.t"}, /* 1811 */
  { 132, "$a33_x"}, /* 1812 */
  { 132, "$a33.x"}, /* 1813 */
  { 133, "$a33_y"}, /* 1814 */
  { 133, "$a33.y"}, /* 1815 */
  { 134, "$a33_z"}, /* 1816 */
  { 134, "$a33.z"}, /* 1817 */
  { 135, "$a33_t"}, /* 1818 */
  { 135, "$a33.t"}, /* 1819 */
  { 136, "$a34_x"}, /* 1820 */
  { 136, "$a34.x"}, /* 1821 */
  { 137, "$a34_y"}, /* 1822 */
  { 137, "$a34.y"}, /* 1823 */
  { 138, "$a34_z"}, /* 1824 */
  { 138, "$a34.z"}, /* 1825 */
  { 139, "$a34_t"}, /* 1826 */
  { 139, "$a34.t"}, /* 1827 */
  { 140, "$a35_x"}, /* 1828 */
  { 140, "$a35.x"}, /* 1829 */
  { 141, "$a35_y"}, /* 1830 */
  { 141, "$a35.y"}, /* 1831 */
  { 142, "$a35_z"}, /* 1832 */
  { 142, "$a35.z"}, /* 1833 */
  { 143, "$a35_t"}, /* 1834 */
  { 143, "$a35.t"}, /* 1835 */
  { 144, "$a36_x"}, /* 1836 */
  { 144, "$a36.x"}, /* 1837 */
  { 145, "$a36_y"}, /* 1838 */
  { 145, "$a36.y"}, /* 1839 */
  { 146, "$a36_z"}, /* 1840 */
  { 146, "$a36.z"}, /* 1841 */
  { 147, "$a36_t"}, /* 1842 */
  { 147, "$a36.t"}, /* 1843 */
  { 148, "$a37_x"}, /* 1844 */
  { 148, "$a37.x"}, /* 1845 */
  { 149, "$a37_y"}, /* 1846 */
  { 149, "$a37.y"}, /* 1847 */
  { 150, "$a37_z"}, /* 1848 */
  { 150, "$a37.z"}, /* 1849 */
  { 151, "$a37_t"}, /* 1850 */
  { 151, "$a37.t"}, /* 1851 */
  { 152, "$a38_x"}, /* 1852 */
  { 152, "$a38.x"}, /* 1853 */
  { 153, "$a38_y"}, /* 1854 */
  { 153, "$a38.y"}, /* 1855 */
  { 154, "$a38_z"}, /* 1856 */
  { 154, "$a38.z"}, /* 1857 */
  { 155, "$a38_t"}, /* 1858 */
  { 155, "$a38.t"}, /* 1859 */
  { 156, "$a39_x"}, /* 1860 */
  { 156, "$a39.x"}, /* 1861 */
  { 157, "$a39_y"}, /* 1862 */
  { 157, "$a39.y"}, /* 1863 */
  { 158, "$a39_z"}, /* 1864 */
  { 158, "$a39.z"}, /* 1865 */
  { 159, "$a39_t"}, /* 1866 */
  { 159, "$a39.t"}, /* 1867 */
  { 160, "$a40_x"}, /* 1868 */
  { 160, "$a40.x"}, /* 1869 */
  { 161, "$a40_y"}, /* 1870 */
  { 161, "$a40.y"}, /* 1871 */
  { 162, "$a40_z"}, /* 1872 */
  { 162, "$a40.z"}, /* 1873 */
  { 163, "$a40_t"}, /* 1874 */
  { 163, "$a40.t"}, /* 1875 */
  { 164, "$a41_x"}, /* 1876 */
  { 164, "$a41.x"}, /* 1877 */
  { 165, "$a41_y"}, /* 1878 */
  { 165, "$a41.y"}, /* 1879 */
  { 166, "$a41_z"}, /* 1880 */
  { 166, "$a41.z"}, /* 1881 */
  { 167, "$a41_t"}, /* 1882 */
  { 167, "$a41.t"}, /* 1883 */
  { 168, "$a42_x"}, /* 1884 */
  { 168, "$a42.x"}, /* 1885 */
  { 169, "$a42_y"}, /* 1886 */
  { 169, "$a42.y"}, /* 1887 */
  { 170, "$a42_z"}, /* 1888 */
  { 170, "$a42.z"}, /* 1889 */
  { 171, "$a42_t"}, /* 1890 */
  { 171, "$a42.t"}, /* 1891 */
  { 172, "$a43_x"}, /* 1892 */
  { 172, "$a43.x"}, /* 1893 */
  { 173, "$a43_y"}, /* 1894 */
  { 173, "$a43.y"}, /* 1895 */
  { 174, "$a43_z"}, /* 1896 */
  { 174, "$a43.z"}, /* 1897 */
  { 175, "$a43_t"}, /* 1898 */
  { 175, "$a43.t"}, /* 1899 */
  { 176, "$a44_x"}, /* 1900 */
  { 176, "$a44.x"}, /* 1901 */
  { 177, "$a44_y"}, /* 1902 */
  { 177, "$a44.y"}, /* 1903 */
  { 178, "$a44_z"}, /* 1904 */
  { 178, "$a44.z"}, /* 1905 */
  { 179, "$a44_t"}, /* 1906 */
  { 179, "$a44.t"}, /* 1907 */
  { 180, "$a45_x"}, /* 1908 */
  { 180, "$a45.x"}, /* 1909 */
  { 181, "$a45_y"}, /* 1910 */
  { 181, "$a45.y"}, /* 1911 */
  { 182, "$a45_z"}, /* 1912 */
  { 182, "$a45.z"}, /* 1913 */
  { 183, "$a45_t"}, /* 1914 */
  { 183, "$a45.t"}, /* 1915 */
  { 184, "$a46_x"}, /* 1916 */
  { 184, "$a46.x"}, /* 1917 */
  { 185, "$a46_y"}, /* 1918 */
  { 185, "$a46.y"}, /* 1919 */
  { 186, "$a46_z"}, /* 1920 */
  { 186, "$a46.z"}, /* 1921 */
  { 187, "$a46_t"}, /* 1922 */
  { 187, "$a46.t"}, /* 1923 */
  { 188, "$a47_x"}, /* 1924 */
  { 188, "$a47.x"}, /* 1925 */
  { 189, "$a47_y"}, /* 1926 */
  { 189, "$a47.y"}, /* 1927 */
  { 190, "$a47_z"}, /* 1928 */
  { 190, "$a47.z"}, /* 1929 */
  { 191, "$a47_t"}, /* 1930 */
  { 191, "$a47.t"}, /* 1931 */
  { 192, "$a48_x"}, /* 1932 */
  { 192, "$a48.x"}, /* 1933 */
  { 193, "$a48_y"}, /* 1934 */
  { 193, "$a48.y"}, /* 1935 */
  { 194, "$a48_z"}, /* 1936 */
  { 194, "$a48.z"}, /* 1937 */
  { 195, "$a48_t"}, /* 1938 */
  { 195, "$a48.t"}, /* 1939 */
  { 196, "$a49_x"}, /* 1940 */
  { 196, "$a49.x"}, /* 1941 */
  { 197, "$a49_y"}, /* 1942 */
  { 197, "$a49.y"}, /* 1943 */
  { 198, "$a49_z"}, /* 1944 */
  { 198, "$a49.z"}, /* 1945 */
  { 199, "$a49_t"}, /* 1946 */
  { 199, "$a49.t"}, /* 1947 */
  { 200, "$a50_x"}, /* 1948 */
  { 200, "$a50.x"}, /* 1949 */
  { 201, "$a50_y"}, /* 1950 */
  { 201, "$a50.y"}, /* 1951 */
  { 202, "$a50_z"}, /* 1952 */
  { 202, "$a50.z"}, /* 1953 */
  { 203, "$a50_t"}, /* 1954 */
  { 203, "$a50.t"}, /* 1955 */
  { 204, "$a51_x"}, /* 1956 */
  { 204, "$a51.x"}, /* 1957 */
  { 205, "$a51_y"}, /* 1958 */
  { 205, "$a51.y"}, /* 1959 */
  { 206, "$a51_z"}, /* 1960 */
  { 206, "$a51.z"}, /* 1961 */
  { 207, "$a51_t"}, /* 1962 */
  { 207, "$a51.t"}, /* 1963 */
  { 208, "$a52_x"}, /* 1964 */
  { 208, "$a52.x"}, /* 1965 */
  { 209, "$a52_y"}, /* 1966 */
  { 209, "$a52.y"}, /* 1967 */
  { 210, "$a52_z"}, /* 1968 */
  { 210, "$a52.z"}, /* 1969 */
  { 211, "$a52_t"}, /* 1970 */
  { 211, "$a52.t"}, /* 1971 */
  { 212, "$a53_x"}, /* 1972 */
  { 212, "$a53.x"}, /* 1973 */
  { 213, "$a53_y"}, /* 1974 */
  { 213, "$a53.y"}, /* 1975 */
  { 214, "$a53_z"}, /* 1976 */
  { 214, "$a53.z"}, /* 1977 */
  { 215, "$a53_t"}, /* 1978 */
  { 215, "$a53.t"}, /* 1979 */
  { 216, "$a54_x"}, /* 1980 */
  { 216, "$a54.x"}, /* 1981 */
  { 217, "$a54_y"}, /* 1982 */
  { 217, "$a54.y"}, /* 1983 */
  { 218, "$a54_z"}, /* 1984 */
  { 218, "$a54.z"}, /* 1985 */
  { 219, "$a54_t"}, /* 1986 */
  { 219, "$a54.t"}, /* 1987 */
  { 220, "$a55_x"}, /* 1988 */
  { 220, "$a55.x"}, /* 1989 */
  { 221, "$a55_y"}, /* 1990 */
  { 221, "$a55.y"}, /* 1991 */
  { 222, "$a55_z"}, /* 1992 */
  { 222, "$a55.z"}, /* 1993 */
  { 223, "$a55_t"}, /* 1994 */
  { 223, "$a55.t"}, /* 1995 */
  { 224, "$a56_x"}, /* 1996 */
  { 224, "$a56.x"}, /* 1997 */
  { 225, "$a56_y"}, /* 1998 */
  { 225, "$a56.y"}, /* 1999 */
  { 226, "$a56_z"}, /* 2000 */
  { 226, "$a56.z"}, /* 2001 */
  { 227, "$a56_t"}, /* 2002 */
  { 227, "$a56.t"}, /* 2003 */
  { 228, "$a57_x"}, /* 2004 */
  { 228, "$a57.x"}, /* 2005 */
  { 229, "$a57_y"}, /* 2006 */
  { 229, "$a57.y"}, /* 2007 */
  { 230, "$a57_z"}, /* 2008 */
  { 230, "$a57.z"}, /* 2009 */
  { 231, "$a57_t"}, /* 2010 */
  { 231, "$a57.t"}, /* 2011 */
  { 232, "$a58_x"}, /* 2012 */
  { 232, "$a58.x"}, /* 2013 */
  { 233, "$a58_y"}, /* 2014 */
  { 233, "$a58.y"}, /* 2015 */
  { 234, "$a58_z"}, /* 2016 */
  { 234, "$a58.z"}, /* 2017 */
  { 235, "$a58_t"}, /* 2018 */
  { 235, "$a58.t"}, /* 2019 */
  { 236, "$a59_x"}, /* 2020 */
  { 236, "$a59.x"}, /* 2021 */
  { 237, "$a59_y"}, /* 2022 */
  { 237, "$a59.y"}, /* 2023 */
  { 238, "$a59_z"}, /* 2024 */
  { 238, "$a59.z"}, /* 2025 */
  { 239, "$a59_t"}, /* 2026 */
  { 239, "$a59.t"}, /* 2027 */
  { 240, "$a60_x"}, /* 2028 */
  { 240, "$a60.x"}, /* 2029 */
  { 241, "$a60_y"}, /* 2030 */
  { 241, "$a60.y"}, /* 2031 */
  { 242, "$a60_z"}, /* 2032 */
  { 242, "$a60.z"}, /* 2033 */
  { 243, "$a60_t"}, /* 2034 */
  { 243, "$a60.t"}, /* 2035 */
  { 244, "$a61_x"}, /* 2036 */
  { 244, "$a61.x"}, /* 2037 */
  { 245, "$a61_y"}, /* 2038 */
  { 245, "$a61.y"}, /* 2039 */
  { 246, "$a61_z"}, /* 2040 */
  { 246, "$a61.z"}, /* 2041 */
  { 247, "$a61_t"}, /* 2042 */
  { 247, "$a61.t"}, /* 2043 */
  { 248, "$a62_x"}, /* 2044 */
  { 248, "$a62.x"}, /* 2045 */
  { 249, "$a62_y"}, /* 2046 */
  { 249, "$a62.y"}, /* 2047 */
  { 250, "$a62_z"}, /* 2048 */
  { 250, "$a62.z"}, /* 2049 */
  { 251, "$a62_t"}, /* 2050 */
  { 251, "$a62.t"}, /* 2051 */
  { 252, "$a63_x"}, /* 2052 */
  { 252, "$a63.x"}, /* 2053 */
  { 253, "$a63_y"}, /* 2054 */
  { 253, "$a63.y"}, /* 2055 */
  { 254, "$a63_z"}, /* 2056 */
  { 254, "$a63.z"}, /* 2057 */
  { 255, "$a63_t"}, /* 2058 */
  { 255, "$a63.t"}, /* 2059 */
  { 0, "$a0a1a2a3"}, /* 2060 */
  { 1, "$a4a5a6a7"}, /* 2061 */
  { 2, "$a8a9a10a11"}, /* 2062 */
  { 3, "$a12a13a14a15"}, /* 2063 */
  { 4, "$a16a17a18a19"}, /* 2064 */
  { 5, "$a20a21a22a23"}, /* 2065 */
  { 6, "$a24a25a26a27"}, /* 2066 */
  { 7, "$a28a29a30a31"}, /* 2067 */
  { 8, "$a32a33a34a35"}, /* 2068 */
  { 9, "$a36a37a38a39"}, /* 2069 */
  { 10, "$a40a41a42a43"}, /* 2070 */
  { 11, "$a44a45a46a47"}, /* 2071 */
  { 12, "$a48a49a50a51"}, /* 2072 */
  { 13, "$a52a53a54a55"}, /* 2073 */
  { 14, "$a56a57a58a59"}, /* 2074 */
  { 15, "$a60a61a62a63"}, /* 2075 */
  { 0, "$a0"}, /* 2076 */
  { 0, "$a0a1.lo"}, /* 2077 */
  { 0, "$a0a1a2a3.x"}, /* 2078 */
  { 1, "$a1"}, /* 2079 */
  { 1, "$a0a1.hi"}, /* 2080 */
  { 1, "$a0a1a2a3.y"}, /* 2081 */
  { 2, "$a2"}, /* 2082 */
  { 2, "$a2a3.lo"}, /* 2083 */
  { 2, "$a0a1a2a3.z"}, /* 2084 */
  { 3, "$a3"}, /* 2085 */
  { 3, "$a2a3.hi"}, /* 2086 */
  { 3, "$a0a1a2a3.t"}, /* 2087 */
  { 4, "$a4"}, /* 2088 */
  { 4, "$a4a5.lo"}, /* 2089 */
  { 4, "$a4a5a6a7.x"}, /* 2090 */
  { 5, "$a5"}, /* 2091 */
  { 5, "$a4a5.hi"}, /* 2092 */
  { 5, "$a4a5a6a7.y"}, /* 2093 */
  { 6, "$a6"}, /* 2094 */
  { 6, "$a6a7.lo"}, /* 2095 */
  { 6, "$a4a5a6a7.z"}, /* 2096 */
  { 7, "$a7"}, /* 2097 */
  { 7, "$a6a7.hi"}, /* 2098 */
  { 7, "$a4a5a6a7.t"}, /* 2099 */
  { 8, "$a8"}, /* 2100 */
  { 8, "$a8a9.lo"}, /* 2101 */
  { 8, "$a8a9a10a11.x"}, /* 2102 */
  { 9, "$a9"}, /* 2103 */
  { 9, "$a8a9.hi"}, /* 2104 */
  { 9, "$a8a9a10a11.y"}, /* 2105 */
  { 10, "$a10"}, /* 2106 */
  { 10, "$a10a11.lo"}, /* 2107 */
  { 10, "$a8a9a10a11.z"}, /* 2108 */
  { 11, "$a11"}, /* 2109 */
  { 11, "$a10a11.hi"}, /* 2110 */
  { 11, "$a8a9a10a11.t"}, /* 2111 */
  { 12, "$a12"}, /* 2112 */
  { 12, "$a12a13.lo"}, /* 2113 */
  { 12, "$a12a13a14a15.x"}, /* 2114 */
  { 13, "$a13"}, /* 2115 */
  { 13, "$a12a13.hi"}, /* 2116 */
  { 13, "$a12a13a14a15.y"}, /* 2117 */
  { 14, "$a14"}, /* 2118 */
  { 14, "$a14a15.lo"}, /* 2119 */
  { 14, "$a12a13a14a15.z"}, /* 2120 */
  { 15, "$a15"}, /* 2121 */
  { 15, "$a14a15.hi"}, /* 2122 */
  { 15, "$a12a13a14a15.t"}, /* 2123 */
  { 16, "$a16"}, /* 2124 */
  { 16, "$a16a17.lo"}, /* 2125 */
  { 16, "$a16a17a18a19.x"}, /* 2126 */
  { 17, "$a17"}, /* 2127 */
  { 17, "$a16a17.hi"}, /* 2128 */
  { 17, "$a16a17a18a19.y"}, /* 2129 */
  { 18, "$a18"}, /* 2130 */
  { 18, "$a18a19.lo"}, /* 2131 */
  { 18, "$a16a17a18a19.z"}, /* 2132 */
  { 19, "$a19"}, /* 2133 */
  { 19, "$a18a19.hi"}, /* 2134 */
  { 19, "$a16a17a18a19.t"}, /* 2135 */
  { 20, "$a20"}, /* 2136 */
  { 20, "$a20a21.lo"}, /* 2137 */
  { 20, "$a20a21a22a23.x"}, /* 2138 */
  { 21, "$a21"}, /* 2139 */
  { 21, "$a20a21.hi"}, /* 2140 */
  { 21, "$a20a21a22a23.y"}, /* 2141 */
  { 22, "$a22"}, /* 2142 */
  { 22, "$a22a23.lo"}, /* 2143 */
  { 22, "$a20a21a22a23.z"}, /* 2144 */
  { 23, "$a23"}, /* 2145 */
  { 23, "$a22a23.hi"}, /* 2146 */
  { 23, "$a20a21a22a23.t"}, /* 2147 */
  { 24, "$a24"}, /* 2148 */
  { 24, "$a24a25.lo"}, /* 2149 */
  { 24, "$a24a25a26a27.x"}, /* 2150 */
  { 25, "$a25"}, /* 2151 */
  { 25, "$a24a25.hi"}, /* 2152 */
  { 25, "$a24a25a26a27.y"}, /* 2153 */
  { 26, "$a26"}, /* 2154 */
  { 26, "$a26a27.lo"}, /* 2155 */
  { 26, "$a24a25a26a27.z"}, /* 2156 */
  { 27, "$a27"}, /* 2157 */
  { 27, "$a26a27.hi"}, /* 2158 */
  { 27, "$a24a25a26a27.t"}, /* 2159 */
  { 28, "$a28"}, /* 2160 */
  { 28, "$a28a29.lo"}, /* 2161 */
  { 28, "$a28a29a30a31.x"}, /* 2162 */
  { 29, "$a29"}, /* 2163 */
  { 29, "$a28a29.hi"}, /* 2164 */
  { 29, "$a28a29a30a31.y"}, /* 2165 */
  { 30, "$a30"}, /* 2166 */
  { 30, "$a30a31.lo"}, /* 2167 */
  { 30, "$a28a29a30a31.z"}, /* 2168 */
  { 31, "$a31"}, /* 2169 */
  { 31, "$a30a31.hi"}, /* 2170 */
  { 31, "$a28a29a30a31.t"}, /* 2171 */
  { 32, "$a32"}, /* 2172 */
  { 32, "$a32a33.lo"}, /* 2173 */
  { 32, "$a32a33a34a35.x"}, /* 2174 */
  { 33, "$a33"}, /* 2175 */
  { 33, "$a32a33.hi"}, /* 2176 */
  { 33, "$a32a33a34a35.y"}, /* 2177 */
  { 34, "$a34"}, /* 2178 */
  { 34, "$a34a35.lo"}, /* 2179 */
  { 34, "$a32a33a34a35.z"}, /* 2180 */
  { 35, "$a35"}, /* 2181 */
  { 35, "$a34a35.hi"}, /* 2182 */
  { 35, "$a32a33a34a35.t"}, /* 2183 */
  { 36, "$a36"}, /* 2184 */
  { 36, "$a36a37.lo"}, /* 2185 */
  { 36, "$a36a37a38a39.x"}, /* 2186 */
  { 37, "$a37"}, /* 2187 */
  { 37, "$a36a37.hi"}, /* 2188 */
  { 37, "$a36a37a38a39.y"}, /* 2189 */
  { 38, "$a38"}, /* 2190 */
  { 38, "$a38a39.lo"}, /* 2191 */
  { 38, "$a36a37a38a39.z"}, /* 2192 */
  { 39, "$a39"}, /* 2193 */
  { 39, "$a38a39.hi"}, /* 2194 */
  { 39, "$a36a37a38a39.t"}, /* 2195 */
  { 40, "$a40"}, /* 2196 */
  { 40, "$a40a41.lo"}, /* 2197 */
  { 40, "$a40a41a42a43.x"}, /* 2198 */
  { 41, "$a41"}, /* 2199 */
  { 41, "$a40a41.hi"}, /* 2200 */
  { 41, "$a40a41a42a43.y"}, /* 2201 */
  { 42, "$a42"}, /* 2202 */
  { 42, "$a42a43.lo"}, /* 2203 */
  { 42, "$a40a41a42a43.z"}, /* 2204 */
  { 43, "$a43"}, /* 2205 */
  { 43, "$a42a43.hi"}, /* 2206 */
  { 43, "$a40a41a42a43.t"}, /* 2207 */
  { 44, "$a44"}, /* 2208 */
  { 44, "$a44a45.lo"}, /* 2209 */
  { 44, "$a44a45a46a47.x"}, /* 2210 */
  { 45, "$a45"}, /* 2211 */
  { 45, "$a44a45.hi"}, /* 2212 */
  { 45, "$a44a45a46a47.y"}, /* 2213 */
  { 46, "$a46"}, /* 2214 */
  { 46, "$a46a47.lo"}, /* 2215 */
  { 46, "$a44a45a46a47.z"}, /* 2216 */
  { 47, "$a47"}, /* 2217 */
  { 47, "$a46a47.hi"}, /* 2218 */
  { 47, "$a44a45a46a47.t"}, /* 2219 */
  { 48, "$a48"}, /* 2220 */
  { 48, "$a48a49.lo"}, /* 2221 */
  { 48, "$a48a49a50a51.x"}, /* 2222 */
  { 49, "$a49"}, /* 2223 */
  { 49, "$a48a49.hi"}, /* 2224 */
  { 49, "$a48a49a50a51.y"}, /* 2225 */
  { 50, "$a50"}, /* 2226 */
  { 50, "$a50a51.lo"}, /* 2227 */
  { 50, "$a48a49a50a51.z"}, /* 2228 */
  { 51, "$a51"}, /* 2229 */
  { 51, "$a50a51.hi"}, /* 2230 */
  { 51, "$a48a49a50a51.t"}, /* 2231 */
  { 52, "$a52"}, /* 2232 */
  { 52, "$a52a53.lo"}, /* 2233 */
  { 52, "$a52a53a54a55.x"}, /* 2234 */
  { 53, "$a53"}, /* 2235 */
  { 53, "$a52a53.hi"}, /* 2236 */
  { 53, "$a52a53a54a55.y"}, /* 2237 */
  { 54, "$a54"}, /* 2238 */
  { 54, "$a54a55.lo"}, /* 2239 */
  { 54, "$a52a53a54a55.z"}, /* 2240 */
  { 55, "$a55"}, /* 2241 */
  { 55, "$a54a55.hi"}, /* 2242 */
  { 55, "$a52a53a54a55.t"}, /* 2243 */
  { 56, "$a56"}, /* 2244 */
  { 56, "$a56a57.lo"}, /* 2245 */
  { 56, "$a56a57a58a59.x"}, /* 2246 */
  { 57, "$a57"}, /* 2247 */
  { 57, "$a56a57.hi"}, /* 2248 */
  { 57, "$a56a57a58a59.y"}, /* 2249 */
  { 58, "$a58"}, /* 2250 */
  { 58, "$a58a59.lo"}, /* 2251 */
  { 58, "$a56a57a58a59.z"}, /* 2252 */
  { 59, "$a59"}, /* 2253 */
  { 59, "$a58a59.hi"}, /* 2254 */
  { 59, "$a56a57a58a59.t"}, /* 2255 */
  { 60, "$a60"}, /* 2256 */
  { 60, "$a60a61.lo"}, /* 2257 */
  { 60, "$a60a61a62a63.x"}, /* 2258 */
  { 61, "$a61"}, /* 2259 */
  { 61, "$a60a61.hi"}, /* 2260 */
  { 61, "$a60a61a62a63.y"}, /* 2261 */
  { 62, "$a62"}, /* 2262 */
  { 62, "$a62a63.lo"}, /* 2263 */
  { 62, "$a60a61a62a63.z"}, /* 2264 */
  { 63, "$a63"}, /* 2265 */
  { 63, "$a62a63.hi"}, /* 2266 */
  { 63, "$a60a61a62a63.t"}, /* 2267 */
  { 0, "$a0a1"}, /* 2268 */
  { 0, "$a0a1a2a3.lo"}, /* 2269 */
  { 1, "$a2a3"}, /* 2270 */
  { 1, "$a0a1a2a3.hi"}, /* 2271 */
  { 2, "$a4a5"}, /* 2272 */
  { 2, "$a4a5a6a7.lo"}, /* 2273 */
  { 3, "$a6a7"}, /* 2274 */
  { 3, "$a4a5a6a7.hi"}, /* 2275 */
  { 4, "$a8a9"}, /* 2276 */
  { 4, "$a8a9a10a11.lo"}, /* 2277 */
  { 5, "$a10a11"}, /* 2278 */
  { 5, "$a8a9a10a11.hi"}, /* 2279 */
  { 6, "$a12a13"}, /* 2280 */
  { 6, "$a12a13a14a15.lo"}, /* 2281 */
  { 7, "$a14a15"}, /* 2282 */
  { 7, "$a12a13a14a15.hi"}, /* 2283 */
  { 8, "$a16a17"}, /* 2284 */
  { 8, "$a16a17a18a19.lo"}, /* 2285 */
  { 9, "$a18a19"}, /* 2286 */
  { 9, "$a16a17a18a19.hi"}, /* 2287 */
  { 10, "$a20a21"}, /* 2288 */
  { 10, "$a20a21a22a23.lo"}, /* 2289 */
  { 11, "$a22a23"}, /* 2290 */
  { 11, "$a20a21a22a23.hi"}, /* 2291 */
  { 12, "$a24a25"}, /* 2292 */
  { 12, "$a24a25a26a27.lo"}, /* 2293 */
  { 13, "$a26a27"}, /* 2294 */
  { 13, "$a24a25a26a27.hi"}, /* 2295 */
  { 14, "$a28a29"}, /* 2296 */
  { 14, "$a28a29a30a31.lo"}, /* 2297 */
  { 15, "$a30a31"}, /* 2298 */
  { 15, "$a28a29a30a31.hi"}, /* 2299 */
  { 16, "$a32a33"}, /* 2300 */
  { 16, "$a32a33a34a35.lo"}, /* 2301 */
  { 17, "$a34a35"}, /* 2302 */
  { 17, "$a32a33a34a35.hi"}, /* 2303 */
  { 18, "$a36a37"}, /* 2304 */
  { 18, "$a36a37a38a39.lo"}, /* 2305 */
  { 19, "$a38a39"}, /* 2306 */
  { 19, "$a36a37a38a39.hi"}, /* 2307 */
  { 20, "$a40a41"}, /* 2308 */
  { 20, "$a40a41a42a43.lo"}, /* 2309 */
  { 21, "$a42a43"}, /* 2310 */
  { 21, "$a40a41a42a43.hi"}, /* 2311 */
  { 22, "$a44a45"}, /* 2312 */
  { 22, "$a44a45a46a47.lo"}, /* 2313 */
  { 23, "$a46a47"}, /* 2314 */
  { 23, "$a44a45a46a47.hi"}, /* 2315 */
  { 24, "$a48a49"}, /* 2316 */
  { 24, "$a48a49a50a51.lo"}, /* 2317 */
  { 25, "$a50a51"}, /* 2318 */
  { 25, "$a48a49a50a51.hi"}, /* 2319 */
  { 26, "$a52a53"}, /* 2320 */
  { 26, "$a52a53a54a55.lo"}, /* 2321 */
  { 27, "$a54a55"}, /* 2322 */
  { 27, "$a52a53a54a55.hi"}, /* 2323 */
  { 28, "$a56a57"}, /* 2324 */
  { 28, "$a56a57a58a59.lo"}, /* 2325 */
  { 29, "$a58a59"}, /* 2326 */
  { 29, "$a56a57a58a59.hi"}, /* 2327 */
  { 30, "$a60a61"}, /* 2328 */
  { 30, "$a60a61a62a63.lo"}, /* 2329 */
  { 31, "$a62a63"}, /* 2330 */
  { 31, "$a60a61a62a63.hi"}, /* 2331 */
};


int kvx_kv3_v1_dec_registers[] = {
  0, /* 0 $r0 */
  3, /* 1 $r1 */
  6, /* 2 $r2 */
  9, /* 3 $r3 */
  12, /* 4 $r4 */
  15, /* 5 $r5 */
  18, /* 6 $r6 */
  21, /* 7 $r7 */
  24, /* 8 $r8 */
  27, /* 9 $r9 */
  30, /* 10 $r10 */
  33, /* 11 $r11 */
  36, /* 12 $r12 */
  38, /* 13 $r13 */
  40, /* 14 $r14 */
  42, /* 15 $r15 */
  44, /* 16 $r16 */
  47, /* 17 $r17 */
  50, /* 18 $r18 */
  53, /* 19 $r19 */
  56, /* 20 $r20 */
  59, /* 21 $r21 */
  62, /* 22 $r22 */
  65, /* 23 $r23 */
  68, /* 24 $r24 */
  71, /* 25 $r25 */
  74, /* 26 $r26 */
  77, /* 27 $r27 */
  80, /* 28 $r28 */
  83, /* 29 $r29 */
  86, /* 30 $r30 */
  89, /* 31 $r31 */
  92, /* 32 $r32 */
  95, /* 33 $r33 */
  98, /* 34 $r34 */
  101, /* 35 $r35 */
  104, /* 36 $r36 */
  107, /* 37 $r37 */
  110, /* 38 $r38 */
  113, /* 39 $r39 */
  116, /* 40 $r40 */
  119, /* 41 $r41 */
  122, /* 42 $r42 */
  125, /* 43 $r43 */
  128, /* 44 $r44 */
  131, /* 45 $r45 */
  134, /* 46 $r46 */
  137, /* 47 $r47 */
  140, /* 48 $r48 */
  143, /* 49 $r49 */
  146, /* 50 $r50 */
  149, /* 51 $r51 */
  152, /* 52 $r52 */
  155, /* 53 $r53 */
  158, /* 54 $r54 */
  161, /* 55 $r55 */
  164, /* 56 $r56 */
  167, /* 57 $r57 */
  170, /* 58 $r58 */
  173, /* 59 $r59 */
  176, /* 60 $r60 */
  179, /* 61 $r61 */
  182, /* 62 $r62 */
  185, /* 63 $r63 */
  188, /* 64 $r0r1 */
  190, /* 65 $r2r3 */
  192, /* 66 $r4r5 */
  194, /* 67 $r6r7 */
  196, /* 68 $r8r9 */
  198, /* 69 $r10r11 */
  200, /* 70 $r12r13 */
  202, /* 71 $r14r15 */
  204, /* 72 $r16r17 */
  206, /* 73 $r18r19 */
  208, /* 74 $r20r21 */
  210, /* 75 $r22r23 */
  212, /* 76 $r24r25 */
  214, /* 77 $r26r27 */
  216, /* 78 $r28r29 */
  218, /* 79 $r30r31 */
  220, /* 80 $r32r33 */
  222, /* 81 $r34r35 */
  224, /* 82 $r36r37 */
  226, /* 83 $r38r39 */
  228, /* 84 $r40r41 */
  230, /* 85 $r42r43 */
  232, /* 86 $r44r45 */
  234, /* 87 $r46r47 */
  236, /* 88 $r48r49 */
  238, /* 89 $r50r51 */
  240, /* 90 $r52r53 */
  242, /* 91 $r54r55 */
  244, /* 92 $r56r57 */
  246, /* 93 $r58r59 */
  248, /* 94 $r60r61 */
  250, /* 95 $r62r63 */
  252, /* 96 $r0r1r2r3 */
  253, /* 97 $r4r5r6r7 */
  254, /* 98 $r8r9r10r11 */
  255, /* 99 $r12r13r14r15 */
  256, /* 100 $r16r17r18r19 */
  257, /* 101 $r20r21r22r23 */
  258, /* 102 $r24r25r26r27 */
  259, /* 103 $r28r29r30r31 */
  260, /* 104 $r32r33r34r35 */
  261, /* 105 $r36r37r38r39 */
  262, /* 106 $r40r41r42r43 */
  263, /* 107 $r44r45r46r47 */
  264, /* 108 $r48r49r50r51 */
  265, /* 109 $r52r53r54r55 */
  266, /* 110 $r56r57r58r59 */
  267, /* 111 $r60r61r62r63 */
  268, /* 112 $pc */
  270, /* 113 $ps */
  272, /* 114 $pcr */
  274, /* 115 $ra */
  276, /* 116 $cs */
  278, /* 117 $csit */
  280, /* 118 $aespc */
  282, /* 119 $ls */
  284, /* 120 $le */
  286, /* 121 $lc */
  288, /* 122 $ipe */
  290, /* 123 $men */
  292, /* 124 $pmc */
  294, /* 125 $pm0 */
  296, /* 126 $pm1 */
  298, /* 127 $pm2 */
  300, /* 128 $pm3 */
  302, /* 129 $pmsa */
  304, /* 130 $tcr */
  306, /* 131 $t0v */
  308, /* 132 $t1v */
  310, /* 133 $t0r */
  312, /* 134 $t1r */
  314, /* 135 $wdv */
  316, /* 136 $wdr */
  318, /* 137 $ile */
  320, /* 138 $ill */
  322, /* 139 $ilr */
  324, /* 140 $mmc */
  326, /* 141 $tel */
  328, /* 142 $teh */
  330, /* 143 $res31 */
  332, /* 144 $syo */
  334, /* 145 $hto */
  336, /* 146 $ito */
  338, /* 147 $do */
  340, /* 148 $mo */
  342, /* 149 $pso */
  344, /* 150 $res38 */
  346, /* 151 $res39 */
  348, /* 152 $dc */
  350, /* 153 $dba0 */
  352, /* 154 $dba1 */
  354, /* 155 $dwa0 */
  356, /* 156 $dwa1 */
  358, /* 157 $mes */
  360, /* 158 $ws */
  362, /* 159 $res47 */
  364, /* 160 $res48 */
  366, /* 161 $res49 */
  368, /* 162 $res50 */
  370, /* 163 $res51 */
  372, /* 164 $res52 */
  374, /* 165 $res53 */
  376, /* 166 $res54 */
  378, /* 167 $res55 */
  380, /* 168 $res56 */
  382, /* 169 $res57 */
  384, /* 170 $res58 */
  386, /* 171 $res59 */
  388, /* 172 $res60 */
  390, /* 173 $res61 */
  392, /* 174 $res62 */
  394, /* 175 $res63 */
  396, /* 176 $spc_pl0 */
  398, /* 177 $spc_pl1 */
  400, /* 178 $spc_pl2 */
  402, /* 179 $spc_pl3 */
  404, /* 180 $sps_pl0 */
  406, /* 181 $sps_pl1 */
  408, /* 182 $sps_pl2 */
  410, /* 183 $sps_pl3 */
  412, /* 184 $ea_pl0 */
  414, /* 185 $ea_pl1 */
  416, /* 186 $ea_pl2 */
  418, /* 187 $ea_pl3 */
  420, /* 188 $ev_pl0 */
  422, /* 189 $ev_pl1 */
  424, /* 190 $ev_pl2 */
  426, /* 191 $ev_pl3 */
  428, /* 192 $sr_pl0 */
  430, /* 193 $sr_pl1 */
  432, /* 194 $sr_pl2 */
  434, /* 195 $sr_pl3 */
  436, /* 196 $es_pl0 */
  438, /* 197 $es_pl1 */
  440, /* 198 $es_pl2 */
  442, /* 199 $es_pl3 */
  444, /* 200 $res88 */
  446, /* 201 $res89 */
  448, /* 202 $res90 */
  450, /* 203 $res91 */
  452, /* 204 $res92 */
  454, /* 205 $res93 */
  456, /* 206 $res94 */
  458, /* 207 $res95 */
  460, /* 208 $syow */
  462, /* 209 $htow */
  464, /* 210 $itow */
  466, /* 211 $dow */
  468, /* 212 $mow */
  470, /* 213 $psow */
  472, /* 214 $res102 */
  474, /* 215 $res103 */
  476, /* 216 $res104 */
  478, /* 217 $res105 */
  480, /* 218 $res106 */
  482, /* 219 $res107 */
  484, /* 220 $res108 */
  486, /* 221 $res109 */
  488, /* 222 $res110 */
  490, /* 223 $res111 */
  492, /* 224 $res112 */
  494, /* 225 $res113 */
  496, /* 226 $res114 */
  498, /* 227 $res115 */
  500, /* 228 $res116 */
  502, /* 229 $res117 */
  504, /* 230 $res118 */
  506, /* 231 $res119 */
  508, /* 232 $res120 */
  510, /* 233 $res121 */
  512, /* 234 $res122 */
  514, /* 235 $res123 */
  516, /* 236 $res124 */
  518, /* 237 $res125 */
  520, /* 238 $res126 */
  522, /* 239 $res127 */
  524, /* 240 $spc */
  526, /* 241 $res129 */
  528, /* 242 $res130 */
  530, /* 243 $res131 */
  532, /* 244 $sps */
  534, /* 245 $res133 */
  536, /* 246 $res134 */
  538, /* 247 $res135 */
  540, /* 248 $ea */
  542, /* 249 $res137 */
  544, /* 250 $res138 */
  546, /* 251 $res139 */
  548, /* 252 $ev */
  550, /* 253 $res141 */
  552, /* 254 $res142 */
  554, /* 255 $res143 */
  556, /* 256 $sr */
  558, /* 257 $res145 */
  560, /* 258 $res146 */
  562, /* 259 $res147 */
  564, /* 260 $es */
  566, /* 261 $res149 */
  568, /* 262 $res150 */
  570, /* 263 $res151 */
  572, /* 264 $res152 */
  574, /* 265 $res153 */
  576, /* 266 $res154 */
  578, /* 267 $res155 */
  580, /* 268 $res156 */
  582, /* 269 $res157 */
  584, /* 270 $res158 */
  586, /* 271 $res159 */
  588, /* 272 $res160 */
  590, /* 273 $res161 */
  592, /* 274 $res162 */
  594, /* 275 $res163 */
  596, /* 276 $res164 */
  598, /* 277 $res165 */
  600, /* 278 $res166 */
  602, /* 279 $res167 */
  604, /* 280 $res168 */
  606, /* 281 $res169 */
  608, /* 282 $res170 */
  610, /* 283 $res171 */
  612, /* 284 $res172 */
  614, /* 285 $res173 */
  616, /* 286 $res174 */
  618, /* 287 $res175 */
  620, /* 288 $res176 */
  622, /* 289 $res177 */
  624, /* 290 $res178 */
  626, /* 291 $res179 */
  628, /* 292 $res180 */
  630, /* 293 $res181 */
  632, /* 294 $res182 */
  634, /* 295 $res183 */
  636, /* 296 $res184 */
  638, /* 297 $res185 */
  640, /* 298 $res186 */
  642, /* 299 $res187 */
  644, /* 300 $res188 */
  646, /* 301 $res189 */
  648, /* 302 $res190 */
  650, /* 303 $res191 */
  652, /* 304 $res192 */
  654, /* 305 $res193 */
  656, /* 306 $res194 */
  658, /* 307 $res195 */
  660, /* 308 $res196 */
  662, /* 309 $res197 */
  664, /* 310 $res198 */
  666, /* 311 $res199 */
  668, /* 312 $res200 */
  670, /* 313 $res201 */
  672, /* 314 $res202 */
  674, /* 315 $res203 */
  676, /* 316 $res204 */
  678, /* 317 $res205 */
  680, /* 318 $res206 */
  682, /* 319 $res207 */
  684, /* 320 $res208 */
  686, /* 321 $res209 */
  688, /* 322 $res210 */
  690, /* 323 $res211 */
  692, /* 324 $res212 */
  694, /* 325 $res213 */
  696, /* 326 $res214 */
  698, /* 327 $res215 */
  700, /* 328 $res216 */
  702, /* 329 $res217 */
  704, /* 330 $res218 */
  706, /* 331 $res219 */
  708, /* 332 $res220 */
  710, /* 333 $res221 */
  712, /* 334 $res222 */
  714, /* 335 $res223 */
  716, /* 336 $res224 */
  718, /* 337 $res225 */
  720, /* 338 $res226 */
  722, /* 339 $res227 */
  724, /* 340 $res228 */
  726, /* 341 $res229 */
  728, /* 342 $res230 */
  730, /* 343 $res231 */
  732, /* 344 $res232 */
  734, /* 345 $res233 */
  736, /* 346 $res234 */
  738, /* 347 $res235 */
  740, /* 348 $res236 */
  742, /* 349 $res237 */
  744, /* 350 $res238 */
  746, /* 351 $res239 */
  748, /* 352 $res240 */
  750, /* 353 $res241 */
  752, /* 354 $res242 */
  754, /* 355 $res243 */
  756, /* 356 $res244 */
  758, /* 357 $res245 */
  760, /* 358 $res246 */
  762, /* 359 $res247 */
  764, /* 360 $res248 */
  766, /* 361 $res249 */
  768, /* 362 $res250 */
  770, /* 363 $res251 */
  772, /* 364 $res252 */
  774, /* 365 $res253 */
  776, /* 366 $res254 */
  778, /* 367 $res255 */
  780, /* 368 $vsfr0 */
  782, /* 369 $vsfr1 */
  784, /* 370 $vsfr2 */
  786, /* 371 $vsfr3 */
  788, /* 372 $vsfr4 */
  790, /* 373 $vsfr5 */
  792, /* 374 $vsfr6 */
  794, /* 375 $vsfr7 */
  796, /* 376 $vsfr8 */
  798, /* 377 $vsfr9 */
  800, /* 378 $vsfr10 */
  802, /* 379 $vsfr11 */
  804, /* 380 $vsfr12 */
  806, /* 381 $vsfr13 */
  808, /* 382 $vsfr14 */
  810, /* 383 $vsfr15 */
  812, /* 384 $vsfr16 */
  814, /* 385 $vsfr17 */
  816, /* 386 $vsfr18 */
  818, /* 387 $vsfr19 */
  820, /* 388 $vsfr20 */
  822, /* 389 $vsfr21 */
  824, /* 390 $vsfr22 */
  826, /* 391 $vsfr23 */
  828, /* 392 $vsfr24 */
  830, /* 393 $vsfr25 */
  832, /* 394 $vsfr26 */
  834, /* 395 $vsfr27 */
  836, /* 396 $vsfr28 */
  838, /* 397 $vsfr29 */
  840, /* 398 $vsfr30 */
  842, /* 399 $vsfr31 */
  844, /* 400 $vsfr32 */
  846, /* 401 $vsfr33 */
  848, /* 402 $vsfr34 */
  850, /* 403 $vsfr35 */
  852, /* 404 $vsfr36 */
  854, /* 405 $vsfr37 */
  856, /* 406 $vsfr38 */
  858, /* 407 $vsfr39 */
  860, /* 408 $vsfr40 */
  862, /* 409 $vsfr41 */
  864, /* 410 $vsfr42 */
  866, /* 411 $vsfr43 */
  868, /* 412 $vsfr44 */
  870, /* 413 $vsfr45 */
  872, /* 414 $vsfr46 */
  874, /* 415 $vsfr47 */
  876, /* 416 $vsfr48 */
  878, /* 417 $vsfr49 */
  880, /* 418 $vsfr50 */
  882, /* 419 $vsfr51 */
  884, /* 420 $vsfr52 */
  886, /* 421 $vsfr53 */
  888, /* 422 $vsfr54 */
  890, /* 423 $vsfr55 */
  892, /* 424 $vsfr56 */
  894, /* 425 $vsfr57 */
  896, /* 426 $vsfr58 */
  898, /* 427 $vsfr59 */
  900, /* 428 $vsfr60 */
  902, /* 429 $vsfr61 */
  904, /* 430 $vsfr62 */
  906, /* 431 $vsfr63 */
  908, /* 432 $vsfr64 */
  910, /* 433 $vsfr65 */
  912, /* 434 $vsfr66 */
  914, /* 435 $vsfr67 */
  916, /* 436 $vsfr68 */
  918, /* 437 $vsfr69 */
  920, /* 438 $vsfr70 */
  922, /* 439 $vsfr71 */
  924, /* 440 $vsfr72 */
  926, /* 441 $vsfr73 */
  928, /* 442 $vsfr74 */
  930, /* 443 $vsfr75 */
  932, /* 444 $vsfr76 */
  934, /* 445 $vsfr77 */
  936, /* 446 $vsfr78 */
  938, /* 447 $vsfr79 */
  940, /* 448 $vsfr80 */
  942, /* 449 $vsfr81 */
  944, /* 450 $vsfr82 */
  946, /* 451 $vsfr83 */
  948, /* 452 $vsfr84 */
  950, /* 453 $vsfr85 */
  952, /* 454 $vsfr86 */
  954, /* 455 $vsfr87 */
  956, /* 456 $vsfr88 */
  958, /* 457 $vsfr89 */
  960, /* 458 $vsfr90 */
  962, /* 459 $vsfr91 */
  964, /* 460 $vsfr92 */
  966, /* 461 $vsfr93 */
  968, /* 462 $vsfr94 */
  970, /* 463 $vsfr95 */
  972, /* 464 $vsfr96 */
  974, /* 465 $vsfr97 */
  976, /* 466 $vsfr98 */
  978, /* 467 $vsfr99 */
  980, /* 468 $vsfr100 */
  982, /* 469 $vsfr101 */
  984, /* 470 $vsfr102 */
  986, /* 471 $vsfr103 */
  988, /* 472 $vsfr104 */
  990, /* 473 $vsfr105 */
  992, /* 474 $vsfr106 */
  994, /* 475 $vsfr107 */
  996, /* 476 $vsfr108 */
  998, /* 477 $vsfr109 */
  1000, /* 478 $vsfr110 */
  1002, /* 479 $vsfr111 */
  1004, /* 480 $vsfr112 */
  1006, /* 481 $vsfr113 */
  1008, /* 482 $vsfr114 */
  1010, /* 483 $vsfr115 */
  1012, /* 484 $vsfr116 */
  1014, /* 485 $vsfr117 */
  1016, /* 486 $vsfr118 */
  1018, /* 487 $vsfr119 */
  1020, /* 488 $vsfr120 */
  1022, /* 489 $vsfr121 */
  1024, /* 490 $vsfr122 */
  1026, /* 491 $vsfr123 */
  1028, /* 492 $vsfr124 */
  1030, /* 493 $vsfr125 */
  1032, /* 494 $vsfr126 */
  1034, /* 495 $vsfr127 */
  1036, /* 496 $vsfr128 */
  1038, /* 497 $vsfr129 */
  1040, /* 498 $vsfr130 */
  1042, /* 499 $vsfr131 */
  1044, /* 500 $vsfr132 */
  1046, /* 501 $vsfr133 */
  1048, /* 502 $vsfr134 */
  1050, /* 503 $vsfr135 */
  1052, /* 504 $vsfr136 */
  1054, /* 505 $vsfr137 */
  1056, /* 506 $vsfr138 */
  1058, /* 507 $vsfr139 */
  1060, /* 508 $vsfr140 */
  1062, /* 509 $vsfr141 */
  1064, /* 510 $vsfr142 */
  1066, /* 511 $vsfr143 */
  1068, /* 512 $vsfr144 */
  1070, /* 513 $vsfr145 */
  1072, /* 514 $vsfr146 */
  1074, /* 515 $vsfr147 */
  1076, /* 516 $vsfr148 */
  1078, /* 517 $vsfr149 */
  1080, /* 518 $vsfr150 */
  1082, /* 519 $vsfr151 */
  1084, /* 520 $vsfr152 */
  1086, /* 521 $vsfr153 */
  1088, /* 522 $vsfr154 */
  1090, /* 523 $vsfr155 */
  1092, /* 524 $vsfr156 */
  1094, /* 525 $vsfr157 */
  1096, /* 526 $vsfr158 */
  1098, /* 527 $vsfr159 */
  1100, /* 528 $vsfr160 */
  1102, /* 529 $vsfr161 */
  1104, /* 530 $vsfr162 */
  1106, /* 531 $vsfr163 */
  1108, /* 532 $vsfr164 */
  1110, /* 533 $vsfr165 */
  1112, /* 534 $vsfr166 */
  1114, /* 535 $vsfr167 */
  1116, /* 536 $vsfr168 */
  1118, /* 537 $vsfr169 */
  1120, /* 538 $vsfr170 */
  1122, /* 539 $vsfr171 */
  1124, /* 540 $vsfr172 */
  1126, /* 541 $vsfr173 */
  1128, /* 542 $vsfr174 */
  1130, /* 543 $vsfr175 */
  1132, /* 544 $vsfr176 */
  1134, /* 545 $vsfr177 */
  1136, /* 546 $vsfr178 */
  1138, /* 547 $vsfr179 */
  1140, /* 548 $vsfr180 */
  1142, /* 549 $vsfr181 */
  1144, /* 550 $vsfr182 */
  1146, /* 551 $vsfr183 */
  1148, /* 552 $vsfr184 */
  1150, /* 553 $vsfr185 */
  1152, /* 554 $vsfr186 */
  1154, /* 555 $vsfr187 */
  1156, /* 556 $vsfr188 */
  1158, /* 557 $vsfr189 */
  1160, /* 558 $vsfr190 */
  1162, /* 559 $vsfr191 */
  1164, /* 560 $vsfr192 */
  1166, /* 561 $vsfr193 */
  1168, /* 562 $vsfr194 */
  1170, /* 563 $vsfr195 */
  1172, /* 564 $vsfr196 */
  1174, /* 565 $vsfr197 */
  1176, /* 566 $vsfr198 */
  1178, /* 567 $vsfr199 */
  1180, /* 568 $vsfr200 */
  1182, /* 569 $vsfr201 */
  1184, /* 570 $vsfr202 */
  1186, /* 571 $vsfr203 */
  1188, /* 572 $vsfr204 */
  1190, /* 573 $vsfr205 */
  1192, /* 574 $vsfr206 */
  1194, /* 575 $vsfr207 */
  1196, /* 576 $vsfr208 */
  1198, /* 577 $vsfr209 */
  1200, /* 578 $vsfr210 */
  1202, /* 579 $vsfr211 */
  1204, /* 580 $vsfr212 */
  1206, /* 581 $vsfr213 */
  1208, /* 582 $vsfr214 */
  1210, /* 583 $vsfr215 */
  1212, /* 584 $vsfr216 */
  1214, /* 585 $vsfr217 */
  1216, /* 586 $vsfr218 */
  1218, /* 587 $vsfr219 */
  1220, /* 588 $vsfr220 */
  1222, /* 589 $vsfr221 */
  1224, /* 590 $vsfr222 */
  1226, /* 591 $vsfr223 */
  1228, /* 592 $vsfr224 */
  1230, /* 593 $vsfr225 */
  1232, /* 594 $vsfr226 */
  1234, /* 595 $vsfr227 */
  1236, /* 596 $vsfr228 */
  1238, /* 597 $vsfr229 */
  1240, /* 598 $vsfr230 */
  1242, /* 599 $vsfr231 */
  1244, /* 600 $vsfr232 */
  1246, /* 601 $vsfr233 */
  1248, /* 602 $vsfr234 */
  1250, /* 603 $vsfr235 */
  1252, /* 604 $vsfr236 */
  1254, /* 605 $vsfr237 */
  1256, /* 606 $vsfr238 */
  1258, /* 607 $vsfr239 */
  1260, /* 608 $vsfr240 */
  1262, /* 609 $vsfr241 */
  1264, /* 610 $vsfr242 */
  1266, /* 611 $vsfr243 */
  1268, /* 612 $vsfr244 */
  1270, /* 613 $vsfr245 */
  1272, /* 614 $vsfr246 */
  1274, /* 615 $vsfr247 */
  1276, /* 616 $vsfr248 */
  1278, /* 617 $vsfr249 */
  1280, /* 618 $vsfr250 */
  1282, /* 619 $vsfr251 */
  1284, /* 620 $vsfr252 */
  1286, /* 621 $vsfr253 */
  1288, /* 622 $vsfr254 */
  1290, /* 623 $vsfr255 */
  1292, /* 624 $a0_lo */
  1294, /* 625 $a0_hi */
  1296, /* 626 $a1_lo */
  1298, /* 627 $a1_hi */
  1300, /* 628 $a2_lo */
  1302, /* 629 $a2_hi */
  1304, /* 630 $a3_lo */
  1306, /* 631 $a3_hi */
  1308, /* 632 $a4_lo */
  1310, /* 633 $a4_hi */
  1312, /* 634 $a5_lo */
  1314, /* 635 $a5_hi */
  1316, /* 636 $a6_lo */
  1318, /* 637 $a6_hi */
  1320, /* 638 $a7_lo */
  1322, /* 639 $a7_hi */
  1324, /* 640 $a8_lo */
  1326, /* 641 $a8_hi */
  1328, /* 642 $a9_lo */
  1330, /* 643 $a9_hi */
  1332, /* 644 $a10_lo */
  1334, /* 645 $a10_hi */
  1336, /* 646 $a11_lo */
  1338, /* 647 $a11_hi */
  1340, /* 648 $a12_lo */
  1342, /* 649 $a12_hi */
  1344, /* 650 $a13_lo */
  1346, /* 651 $a13_hi */
  1348, /* 652 $a14_lo */
  1350, /* 653 $a14_hi */
  1352, /* 654 $a15_lo */
  1354, /* 655 $a15_hi */
  1356, /* 656 $a16_lo */
  1358, /* 657 $a16_hi */
  1360, /* 658 $a17_lo */
  1362, /* 659 $a17_hi */
  1364, /* 660 $a18_lo */
  1366, /* 661 $a18_hi */
  1368, /* 662 $a19_lo */
  1370, /* 663 $a19_hi */
  1372, /* 664 $a20_lo */
  1374, /* 665 $a20_hi */
  1376, /* 666 $a21_lo */
  1378, /* 667 $a21_hi */
  1380, /* 668 $a22_lo */
  1382, /* 669 $a22_hi */
  1384, /* 670 $a23_lo */
  1386, /* 671 $a23_hi */
  1388, /* 672 $a24_lo */
  1390, /* 673 $a24_hi */
  1392, /* 674 $a25_lo */
  1394, /* 675 $a25_hi */
  1396, /* 676 $a26_lo */
  1398, /* 677 $a26_hi */
  1400, /* 678 $a27_lo */
  1402, /* 679 $a27_hi */
  1404, /* 680 $a28_lo */
  1406, /* 681 $a28_hi */
  1408, /* 682 $a29_lo */
  1410, /* 683 $a29_hi */
  1412, /* 684 $a30_lo */
  1414, /* 685 $a30_hi */
  1416, /* 686 $a31_lo */
  1418, /* 687 $a31_hi */
  1420, /* 688 $a32_lo */
  1422, /* 689 $a32_hi */
  1424, /* 690 $a33_lo */
  1426, /* 691 $a33_hi */
  1428, /* 692 $a34_lo */
  1430, /* 693 $a34_hi */
  1432, /* 694 $a35_lo */
  1434, /* 695 $a35_hi */
  1436, /* 696 $a36_lo */
  1438, /* 697 $a36_hi */
  1440, /* 698 $a37_lo */
  1442, /* 699 $a37_hi */
  1444, /* 700 $a38_lo */
  1446, /* 701 $a38_hi */
  1448, /* 702 $a39_lo */
  1450, /* 703 $a39_hi */
  1452, /* 704 $a40_lo */
  1454, /* 705 $a40_hi */
  1456, /* 706 $a41_lo */
  1458, /* 707 $a41_hi */
  1460, /* 708 $a42_lo */
  1462, /* 709 $a42_hi */
  1464, /* 710 $a43_lo */
  1466, /* 711 $a43_hi */
  1468, /* 712 $a44_lo */
  1470, /* 713 $a44_hi */
  1472, /* 714 $a45_lo */
  1474, /* 715 $a45_hi */
  1476, /* 716 $a46_lo */
  1478, /* 717 $a46_hi */
  1480, /* 718 $a47_lo */
  1482, /* 719 $a47_hi */
  1484, /* 720 $a48_lo */
  1486, /* 721 $a48_hi */
  1488, /* 722 $a49_lo */
  1490, /* 723 $a49_hi */
  1492, /* 724 $a50_lo */
  1494, /* 725 $a50_hi */
  1496, /* 726 $a51_lo */
  1498, /* 727 $a51_hi */
  1500, /* 728 $a52_lo */
  1502, /* 729 $a52_hi */
  1504, /* 730 $a53_lo */
  1506, /* 731 $a53_hi */
  1508, /* 732 $a54_lo */
  1510, /* 733 $a54_hi */
  1512, /* 734 $a55_lo */
  1514, /* 735 $a55_hi */
  1516, /* 736 $a56_lo */
  1518, /* 737 $a56_hi */
  1520, /* 738 $a57_lo */
  1522, /* 739 $a57_hi */
  1524, /* 740 $a58_lo */
  1526, /* 741 $a58_hi */
  1528, /* 742 $a59_lo */
  1530, /* 743 $a59_hi */
  1532, /* 744 $a60_lo */
  1534, /* 745 $a60_hi */
  1536, /* 746 $a61_lo */
  1538, /* 747 $a61_hi */
  1540, /* 748 $a62_lo */
  1542, /* 749 $a62_hi */
  1544, /* 750 $a63_lo */
  1546, /* 751 $a63_hi */
  1548, /* 752 $a0_x */
  1550, /* 753 $a0_y */
  1552, /* 754 $a0_z */
  1554, /* 755 $a0_t */
  1556, /* 756 $a1_x */
  1558, /* 757 $a1_y */
  1560, /* 758 $a1_z */
  1562, /* 759 $a1_t */
  1564, /* 760 $a2_x */
  1566, /* 761 $a2_y */
  1568, /* 762 $a2_z */
  1570, /* 763 $a2_t */
  1572, /* 764 $a3_x */
  1574, /* 765 $a3_y */
  1576, /* 766 $a3_z */
  1578, /* 767 $a3_t */
  1580, /* 768 $a4_x */
  1582, /* 769 $a4_y */
  1584, /* 770 $a4_z */
  1586, /* 771 $a4_t */
  1588, /* 772 $a5_x */
  1590, /* 773 $a5_y */
  1592, /* 774 $a5_z */
  1594, /* 775 $a5_t */
  1596, /* 776 $a6_x */
  1598, /* 777 $a6_y */
  1600, /* 778 $a6_z */
  1602, /* 779 $a6_t */
  1604, /* 780 $a7_x */
  1606, /* 781 $a7_y */
  1608, /* 782 $a7_z */
  1610, /* 783 $a7_t */
  1612, /* 784 $a8_x */
  1614, /* 785 $a8_y */
  1616, /* 786 $a8_z */
  1618, /* 787 $a8_t */
  1620, /* 788 $a9_x */
  1622, /* 789 $a9_y */
  1624, /* 790 $a9_z */
  1626, /* 791 $a9_t */
  1628, /* 792 $a10_x */
  1630, /* 793 $a10_y */
  1632, /* 794 $a10_z */
  1634, /* 795 $a10_t */
  1636, /* 796 $a11_x */
  1638, /* 797 $a11_y */
  1640, /* 798 $a11_z */
  1642, /* 799 $a11_t */
  1644, /* 800 $a12_x */
  1646, /* 801 $a12_y */
  1648, /* 802 $a12_z */
  1650, /* 803 $a12_t */
  1652, /* 804 $a13_x */
  1654, /* 805 $a13_y */
  1656, /* 806 $a13_z */
  1658, /* 807 $a13_t */
  1660, /* 808 $a14_x */
  1662, /* 809 $a14_y */
  1664, /* 810 $a14_z */
  1666, /* 811 $a14_t */
  1668, /* 812 $a15_x */
  1670, /* 813 $a15_y */
  1672, /* 814 $a15_z */
  1674, /* 815 $a15_t */
  1676, /* 816 $a16_x */
  1678, /* 817 $a16_y */
  1680, /* 818 $a16_z */
  1682, /* 819 $a16_t */
  1684, /* 820 $a17_x */
  1686, /* 821 $a17_y */
  1688, /* 822 $a17_z */
  1690, /* 823 $a17_t */
  1692, /* 824 $a18_x */
  1694, /* 825 $a18_y */
  1696, /* 826 $a18_z */
  1698, /* 827 $a18_t */
  1700, /* 828 $a19_x */
  1702, /* 829 $a19_y */
  1704, /* 830 $a19_z */
  1706, /* 831 $a19_t */
  1708, /* 832 $a20_x */
  1710, /* 833 $a20_y */
  1712, /* 834 $a20_z */
  1714, /* 835 $a20_t */
  1716, /* 836 $a21_x */
  1718, /* 837 $a21_y */
  1720, /* 838 $a21_z */
  1722, /* 839 $a21_t */
  1724, /* 840 $a22_x */
  1726, /* 841 $a22_y */
  1728, /* 842 $a22_z */
  1730, /* 843 $a22_t */
  1732, /* 844 $a23_x */
  1734, /* 845 $a23_y */
  1736, /* 846 $a23_z */
  1738, /* 847 $a23_t */
  1740, /* 848 $a24_x */
  1742, /* 849 $a24_y */
  1744, /* 850 $a24_z */
  1746, /* 851 $a24_t */
  1748, /* 852 $a25_x */
  1750, /* 853 $a25_y */
  1752, /* 854 $a25_z */
  1754, /* 855 $a25_t */
  1756, /* 856 $a26_x */
  1758, /* 857 $a26_y */
  1760, /* 858 $a26_z */
  1762, /* 859 $a26_t */
  1764, /* 860 $a27_x */
  1766, /* 861 $a27_y */
  1768, /* 862 $a27_z */
  1770, /* 863 $a27_t */
  1772, /* 864 $a28_x */
  1774, /* 865 $a28_y */
  1776, /* 866 $a28_z */
  1778, /* 867 $a28_t */
  1780, /* 868 $a29_x */
  1782, /* 869 $a29_y */
  1784, /* 870 $a29_z */
  1786, /* 871 $a29_t */
  1788, /* 872 $a30_x */
  1790, /* 873 $a30_y */
  1792, /* 874 $a30_z */
  1794, /* 875 $a30_t */
  1796, /* 876 $a31_x */
  1798, /* 877 $a31_y */
  1800, /* 878 $a31_z */
  1802, /* 879 $a31_t */
  1804, /* 880 $a32_x */
  1806, /* 881 $a32_y */
  1808, /* 882 $a32_z */
  1810, /* 883 $a32_t */
  1812, /* 884 $a33_x */
  1814, /* 885 $a33_y */
  1816, /* 886 $a33_z */
  1818, /* 887 $a33_t */
  1820, /* 888 $a34_x */
  1822, /* 889 $a34_y */
  1824, /* 890 $a34_z */
  1826, /* 891 $a34_t */
  1828, /* 892 $a35_x */
  1830, /* 893 $a35_y */
  1832, /* 894 $a35_z */
  1834, /* 895 $a35_t */
  1836, /* 896 $a36_x */
  1838, /* 897 $a36_y */
  1840, /* 898 $a36_z */
  1842, /* 899 $a36_t */
  1844, /* 900 $a37_x */
  1846, /* 901 $a37_y */
  1848, /* 902 $a37_z */
  1850, /* 903 $a37_t */
  1852, /* 904 $a38_x */
  1854, /* 905 $a38_y */
  1856, /* 906 $a38_z */
  1858, /* 907 $a38_t */
  1860, /* 908 $a39_x */
  1862, /* 909 $a39_y */
  1864, /* 910 $a39_z */
  1866, /* 911 $a39_t */
  1868, /* 912 $a40_x */
  1870, /* 913 $a40_y */
  1872, /* 914 $a40_z */
  1874, /* 915 $a40_t */
  1876, /* 916 $a41_x */
  1878, /* 917 $a41_y */
  1880, /* 918 $a41_z */
  1882, /* 919 $a41_t */
  1884, /* 920 $a42_x */
  1886, /* 921 $a42_y */
  1888, /* 922 $a42_z */
  1890, /* 923 $a42_t */
  1892, /* 924 $a43_x */
  1894, /* 925 $a43_y */
  1896, /* 926 $a43_z */
  1898, /* 927 $a43_t */
  1900, /* 928 $a44_x */
  1902, /* 929 $a44_y */
  1904, /* 930 $a44_z */
  1906, /* 931 $a44_t */
  1908, /* 932 $a45_x */
  1910, /* 933 $a45_y */
  1912, /* 934 $a45_z */
  1914, /* 935 $a45_t */
  1916, /* 936 $a46_x */
  1918, /* 937 $a46_y */
  1920, /* 938 $a46_z */
  1922, /* 939 $a46_t */
  1924, /* 940 $a47_x */
  1926, /* 941 $a47_y */
  1928, /* 942 $a47_z */
  1930, /* 943 $a47_t */
  1932, /* 944 $a48_x */
  1934, /* 945 $a48_y */
  1936, /* 946 $a48_z */
  1938, /* 947 $a48_t */
  1940, /* 948 $a49_x */
  1942, /* 949 $a49_y */
  1944, /* 950 $a49_z */
  1946, /* 951 $a49_t */
  1948, /* 952 $a50_x */
  1950, /* 953 $a50_y */
  1952, /* 954 $a50_z */
  1954, /* 955 $a50_t */
  1956, /* 956 $a51_x */
  1958, /* 957 $a51_y */
  1960, /* 958 $a51_z */
  1962, /* 959 $a51_t */
  1964, /* 960 $a52_x */
  1966, /* 961 $a52_y */
  1968, /* 962 $a52_z */
  1970, /* 963 $a52_t */
  1972, /* 964 $a53_x */
  1974, /* 965 $a53_y */
  1976, /* 966 $a53_z */
  1978, /* 967 $a53_t */
  1980, /* 968 $a54_x */
  1982, /* 969 $a54_y */
  1984, /* 970 $a54_z */
  1986, /* 971 $a54_t */
  1988, /* 972 $a55_x */
  1990, /* 973 $a55_y */
  1992, /* 974 $a55_z */
  1994, /* 975 $a55_t */
  1996, /* 976 $a56_x */
  1998, /* 977 $a56_y */
  2000, /* 978 $a56_z */
  2002, /* 979 $a56_t */
  2004, /* 980 $a57_x */
  2006, /* 981 $a57_y */
  2008, /* 982 $a57_z */
  2010, /* 983 $a57_t */
  2012, /* 984 $a58_x */
  2014, /* 985 $a58_y */
  2016, /* 986 $a58_z */
  2018, /* 987 $a58_t */
  2020, /* 988 $a59_x */
  2022, /* 989 $a59_y */
  2024, /* 990 $a59_z */
  2026, /* 991 $a59_t */
  2028, /* 992 $a60_x */
  2030, /* 993 $a60_y */
  2032, /* 994 $a60_z */
  2034, /* 995 $a60_t */
  2036, /* 996 $a61_x */
  2038, /* 997 $a61_y */
  2040, /* 998 $a61_z */
  2042, /* 999 $a61_t */
  2044, /* 1000 $a62_x */
  2046, /* 1001 $a62_y */
  2048, /* 1002 $a62_z */
  2050, /* 1003 $a62_t */
  2052, /* 1004 $a63_x */
  2054, /* 1005 $a63_y */
  2056, /* 1006 $a63_z */
  2058, /* 1007 $a63_t */
  2060, /* 1008 $a0a1a2a3 */
  2061, /* 1009 $a4a5a6a7 */
  2062, /* 1010 $a8a9a10a11 */
  2063, /* 1011 $a12a13a14a15 */
  2064, /* 1012 $a16a17a18a19 */
  2065, /* 1013 $a20a21a22a23 */
  2066, /* 1014 $a24a25a26a27 */
  2067, /* 1015 $a28a29a30a31 */
  2068, /* 1016 $a32a33a34a35 */
  2069, /* 1017 $a36a37a38a39 */
  2070, /* 1018 $a40a41a42a43 */
  2071, /* 1019 $a44a45a46a47 */
  2072, /* 1020 $a48a49a50a51 */
  2073, /* 1021 $a52a53a54a55 */
  2074, /* 1022 $a56a57a58a59 */
  2075, /* 1023 $a60a61a62a63 */
  2076, /* 1024 $a0 */
  2079, /* 1025 $a1 */
  2082, /* 1026 $a2 */
  2085, /* 1027 $a3 */
  2088, /* 1028 $a4 */
  2091, /* 1029 $a5 */
  2094, /* 1030 $a6 */
  2097, /* 1031 $a7 */
  2100, /* 1032 $a8 */
  2103, /* 1033 $a9 */
  2106, /* 1034 $a10 */
  2109, /* 1035 $a11 */
  2112, /* 1036 $a12 */
  2115, /* 1037 $a13 */
  2118, /* 1038 $a14 */
  2121, /* 1039 $a15 */
  2124, /* 1040 $a16 */
  2127, /* 1041 $a17 */
  2130, /* 1042 $a18 */
  2133, /* 1043 $a19 */
  2136, /* 1044 $a20 */
  2139, /* 1045 $a21 */
  2142, /* 1046 $a22 */
  2145, /* 1047 $a23 */
  2148, /* 1048 $a24 */
  2151, /* 1049 $a25 */
  2154, /* 1050 $a26 */
  2157, /* 1051 $a27 */
  2160, /* 1052 $a28 */
  2163, /* 1053 $a29 */
  2166, /* 1054 $a30 */
  2169, /* 1055 $a31 */
  2172, /* 1056 $a32 */
  2175, /* 1057 $a33 */
  2178, /* 1058 $a34 */
  2181, /* 1059 $a35 */
  2184, /* 1060 $a36 */
  2187, /* 1061 $a37 */
  2190, /* 1062 $a38 */
  2193, /* 1063 $a39 */
  2196, /* 1064 $a40 */
  2199, /* 1065 $a41 */
  2202, /* 1066 $a42 */
  2205, /* 1067 $a43 */
  2208, /* 1068 $a44 */
  2211, /* 1069 $a45 */
  2214, /* 1070 $a46 */
  2217, /* 1071 $a47 */
  2220, /* 1072 $a48 */
  2223, /* 1073 $a49 */
  2226, /* 1074 $a50 */
  2229, /* 1075 $a51 */
  2232, /* 1076 $a52 */
  2235, /* 1077 $a53 */
  2238, /* 1078 $a54 */
  2241, /* 1079 $a55 */
  2244, /* 1080 $a56 */
  2247, /* 1081 $a57 */
  2250, /* 1082 $a58 */
  2253, /* 1083 $a59 */
  2256, /* 1084 $a60 */
  2259, /* 1085 $a61 */
  2262, /* 1086 $a62 */
  2265, /* 1087 $a63 */
  2268, /* 1088 $a0a1 */
  2270, /* 1089 $a2a3 */
  2272, /* 1090 $a4a5 */
  2274, /* 1091 $a6a7 */
  2276, /* 1092 $a8a9 */
  2278, /* 1093 $a10a11 */
  2280, /* 1094 $a12a13 */
  2282, /* 1095 $a14a15 */
  2284, /* 1096 $a16a17 */
  2286, /* 1097 $a18a19 */
  2288, /* 1098 $a20a21 */
  2290, /* 1099 $a22a23 */
  2292, /* 1100 $a24a25 */
  2294, /* 1101 $a26a27 */
  2296, /* 1102 $a28a29 */
  2298, /* 1103 $a30a31 */
  2300, /* 1104 $a32a33 */
  2302, /* 1105 $a34a35 */
  2304, /* 1106 $a36a37 */
  2306, /* 1107 $a38a39 */
  2308, /* 1108 $a40a41 */
  2310, /* 1109 $a42a43 */
  2312, /* 1110 $a44a45 */
  2314, /* 1111 $a46a47 */
  2316, /* 1112 $a48a49 */
  2318, /* 1113 $a50a51 */
  2320, /* 1114 $a52a53 */
  2322, /* 1115 $a54a55 */
  2324, /* 1116 $a56a57 */
  2326, /* 1117 $a58a59 */
  2328, /* 1118 $a60a61 */
  2330, /* 1119 $a62a63 */
};


int kvx_kv3_v2_regfiles[] = {
  0, 	/* KVX_REGFILE_FIRST_GPR */
  187, 	/* KVX_REGFILE_LAST_GPR */
  0, 	/* KVX_REGFILE_DEC_GPR */
  188, 	/* KVX_REGFILE_FIRST_PGR */
  251, 	/* KVX_REGFILE_LAST_PGR */
  64, 	/* KVX_REGFILE_DEC_PGR */
  252, 	/* KVX_REGFILE_FIRST_QGR */
  267, 	/* KVX_REGFILE_LAST_QGR */
  96, 	/* KVX_REGFILE_DEC_QGR */
  268, 	/* KVX_REGFILE_FIRST_SFR */
  1291, 	/* KVX_REGFILE_LAST_SFR */
  112, 	/* KVX_REGFILE_DEC_SFR */
  1292, 	/* KVX_REGFILE_FIRST_XBR */
  1547, 	/* KVX_REGFILE_LAST_XBR */
  624, 	/* KVX_REGFILE_DEC_XBR */
  1548, 	/* KVX_REGFILE_FIRST_XCR */
  2059, 	/* KVX_REGFILE_LAST_XCR */
  752, 	/* KVX_REGFILE_DEC_XCR */
  2060, 	/* KVX_REGFILE_FIRST_XMR */
  2075, 	/* KVX_REGFILE_LAST_XMR */
  1008, 	/* KVX_REGFILE_DEC_XMR */
  2076, 	/* KVX_REGFILE_FIRST_XVR */
  2267, 	/* KVX_REGFILE_LAST_XVR */
  1024, 	/* KVX_REGFILE_DEC_XVR */
  2268, 	/* KVX_REGFILE_FIRST_XWR */
  2331, 	/* KVX_REGFILE_LAST_XWR */
  1088, 	/* KVX_REGFILE_DEC_XWR */
  2332, 	/* KVX_REGFILE_REGISTERS*/
  1120, 	/* KVX_REGFILE_DEC_REGISTERS*/
};


kvx_Register kvx_kv3_v2_registers[] = {
  { 0, "$r0"}, /* 0 */
  { 0, "$r0r1.lo"}, /* 1 */
  { 0, "$r0r1r2r3.x"}, /* 2 */
  { 1, "$r1"}, /* 3 */
  { 1, "$r0r1.hi"}, /* 4 */
  { 1, "$r0r1r2r3.y"}, /* 5 */
  { 2, "$r2"}, /* 6 */
  { 2, "$r2r3.lo"}, /* 7 */
  { 2, "$r0r1r2r3.z"}, /* 8 */
  { 3, "$r3"}, /* 9 */
  { 3, "$r2r3.hi"}, /* 10 */
  { 3, "$r0r1r2r3.t"}, /* 11 */
  { 4, "$r4"}, /* 12 */
  { 4, "$r4r5.lo"}, /* 13 */
  { 4, "$r4r5r6r7.x"}, /* 14 */
  { 5, "$r5"}, /* 15 */
  { 5, "$r4r5.hi"}, /* 16 */
  { 5, "$r4r5r6r7.y"}, /* 17 */
  { 6, "$r6"}, /* 18 */
  { 6, "$r6r7.lo"}, /* 19 */
  { 6, "$r4r5r6r7.z"}, /* 20 */
  { 7, "$r7"}, /* 21 */
  { 7, "$r6r7.hi"}, /* 22 */
  { 7, "$r4r5r6r7.t"}, /* 23 */
  { 8, "$r8"}, /* 24 */
  { 8, "$r8r9.lo"}, /* 25 */
  { 8, "$r8r9r10r11.x"}, /* 26 */
  { 9, "$r9"}, /* 27 */
  { 9, "$r8r9.hi"}, /* 28 */
  { 9, "$r8r9r10r11.y"}, /* 29 */
  { 10, "$r10"}, /* 30 */
  { 10, "$r10r11.lo"}, /* 31 */
  { 10, "$r8r9r10r11.z"}, /* 32 */
  { 11, "$r11"}, /* 33 */
  { 11, "$r10r11.hi"}, /* 34 */
  { 11, "$r8r9r10r11.t"}, /* 35 */
  { 12, "$r12"}, /* 36 */
  { 12, "$sp"}, /* 37 */
  { 13, "$r13"}, /* 38 */
  { 13, "$tp"}, /* 39 */
  { 14, "$r14"}, /* 40 */
  { 14, "$fp"}, /* 41 */
  { 15, "$r15"}, /* 42 */
  { 15, "$rp"}, /* 43 */
  { 16, "$r16"}, /* 44 */
  { 16, "$r16r17.lo"}, /* 45 */
  { 16, "$r16r17r18r19.x"}, /* 46 */
  { 17, "$r17"}, /* 47 */
  { 17, "$r16r17.hi"}, /* 48 */
  { 17, "$r16r17r18r19.y"}, /* 49 */
  { 18, "$r18"}, /* 50 */
  { 18, "$r18r19.lo"}, /* 51 */
  { 18, "$r16r17r18r19.z"}, /* 52 */
  { 19, "$r19"}, /* 53 */
  { 19, "$r18r19.hi"}, /* 54 */
  { 19, "$r16r17r18r19.t"}, /* 55 */
  { 20, "$r20"}, /* 56 */
  { 20, "$r20r21.lo"}, /* 57 */
  { 20, "$r20r21r22r23.x"}, /* 58 */
  { 21, "$r21"}, /* 59 */
  { 21, "$r20r21.hi"}, /* 60 */
  { 21, "$r20r21r22r23.y"}, /* 61 */
  { 22, "$r22"}, /* 62 */
  { 22, "$r22r23.lo"}, /* 63 */
  { 22, "$r20r21r22r23.z"}, /* 64 */
  { 23, "$r23"}, /* 65 */
  { 23, "$r22r23.hi"}, /* 66 */
  { 23, "$r20r21r22r23.t"}, /* 67 */
  { 24, "$r24"}, /* 68 */
  { 24, "$r24r25.lo"}, /* 69 */
  { 24, "$r24r25r26r27.x"}, /* 70 */
  { 25, "$r25"}, /* 71 */
  { 25, "$r24r25.hi"}, /* 72 */
  { 25, "$r24r25r26r27.y"}, /* 73 */
  { 26, "$r26"}, /* 74 */
  { 26, "$r26r27.lo"}, /* 75 */
  { 26, "$r24r25r26r27.z"}, /* 76 */
  { 27, "$r27"}, /* 77 */
  { 27, "$r26r27.hi"}, /* 78 */
  { 27, "$r24r25r26r27.t"}, /* 79 */
  { 28, "$r28"}, /* 80 */
  { 28, "$r28r29.lo"}, /* 81 */
  { 28, "$r28r29r30r31.x"}, /* 82 */
  { 29, "$r29"}, /* 83 */
  { 29, "$r28r29.hi"}, /* 84 */
  { 29, "$r28r29r30r31.y"}, /* 85 */
  { 30, "$r30"}, /* 86 */
  { 30, "$r30r31.lo"}, /* 87 */
  { 30, "$r28r29r30r31.z"}, /* 88 */
  { 31, "$r31"}, /* 89 */
  { 31, "$r30r31.hi"}, /* 90 */
  { 31, "$r28r29r30r31.t"}, /* 91 */
  { 32, "$r32"}, /* 92 */
  { 32, "$r32r33.lo"}, /* 93 */
  { 32, "$r32r33r34r35.x"}, /* 94 */
  { 33, "$r33"}, /* 95 */
  { 33, "$r32r33.hi"}, /* 96 */
  { 33, "$r32r33r34r35.y"}, /* 97 */
  { 34, "$r34"}, /* 98 */
  { 34, "$r34r35.lo"}, /* 99 */
  { 34, "$r32r33r34r35.z"}, /* 100 */
  { 35, "$r35"}, /* 101 */
  { 35, "$r34r35.hi"}, /* 102 */
  { 35, "$r32r33r34r35.t"}, /* 103 */
  { 36, "$r36"}, /* 104 */
  { 36, "$r36r37.lo"}, /* 105 */
  { 36, "$r36r37r38r39.x"}, /* 106 */
  { 37, "$r37"}, /* 107 */
  { 37, "$r36r37.hi"}, /* 108 */
  { 37, "$r36r37r38r39.y"}, /* 109 */
  { 38, "$r38"}, /* 110 */
  { 38, "$r38r39.lo"}, /* 111 */
  { 38, "$r36r37r38r39.z"}, /* 112 */
  { 39, "$r39"}, /* 113 */
  { 39, "$r38r39.hi"}, /* 114 */
  { 39, "$r36r37r38r39.t"}, /* 115 */
  { 40, "$r40"}, /* 116 */
  { 40, "$r40r41.lo"}, /* 117 */
  { 40, "$r40r41r42r43.x"}, /* 118 */
  { 41, "$r41"}, /* 119 */
  { 41, "$r40r41.hi"}, /* 120 */
  { 41, "$r40r41r42r43.y"}, /* 121 */
  { 42, "$r42"}, /* 122 */
  { 42, "$r42r43.lo"}, /* 123 */
  { 42, "$r40r41r42r43.z"}, /* 124 */
  { 43, "$r43"}, /* 125 */
  { 43, "$r42r43.hi"}, /* 126 */
  { 43, "$r40r41r42r43.t"}, /* 127 */
  { 44, "$r44"}, /* 128 */
  { 44, "$r44r45.lo"}, /* 129 */
  { 44, "$r44r45r46r47.x"}, /* 130 */
  { 45, "$r45"}, /* 131 */
  { 45, "$r44r45.hi"}, /* 132 */
  { 45, "$r44r45r46r47.y"}, /* 133 */
  { 46, "$r46"}, /* 134 */
  { 46, "$r46r47.lo"}, /* 135 */
  { 46, "$r44r45r46r47.z"}, /* 136 */
  { 47, "$r47"}, /* 137 */
  { 47, "$r46r47.hi"}, /* 138 */
  { 47, "$r44r45r46r47.t"}, /* 139 */
  { 48, "$r48"}, /* 140 */
  { 48, "$r48r49.lo"}, /* 141 */
  { 48, "$r48r49r50r51.x"}, /* 142 */
  { 49, "$r49"}, /* 143 */
  { 49, "$r48r49.hi"}, /* 144 */
  { 49, "$r48r49r50r51.y"}, /* 145 */
  { 50, "$r50"}, /* 146 */
  { 50, "$r50r51.lo"}, /* 147 */
  { 50, "$r48r49r50r51.z"}, /* 148 */
  { 51, "$r51"}, /* 149 */
  { 51, "$r50r51.hi"}, /* 150 */
  { 51, "$r48r49r50r51.t"}, /* 151 */
  { 52, "$r52"}, /* 152 */
  { 52, "$r52r53.lo"}, /* 153 */
  { 52, "$r52r53r54r55.x"}, /* 154 */
  { 53, "$r53"}, /* 155 */
  { 53, "$r52r53.hi"}, /* 156 */
  { 53, "$r52r53r54r55.y"}, /* 157 */
  { 54, "$r54"}, /* 158 */
  { 54, "$r54r55.lo"}, /* 159 */
  { 54, "$r52r53r54r55.z"}, /* 160 */
  { 55, "$r55"}, /* 161 */
  { 55, "$r54r55.hi"}, /* 162 */
  { 55, "$r52r53r54r55.t"}, /* 163 */
  { 56, "$r56"}, /* 164 */
  { 56, "$r56r57.lo"}, /* 165 */
  { 56, "$r56r57r58r59.x"}, /* 166 */
  { 57, "$r57"}, /* 167 */
  { 57, "$r56r57.hi"}, /* 168 */
  { 57, "$r56r57r58r59.y"}, /* 169 */
  { 58, "$r58"}, /* 170 */
  { 58, "$r58r59.lo"}, /* 171 */
  { 58, "$r56r57r58r59.z"}, /* 172 */
  { 59, "$r59"}, /* 173 */
  { 59, "$r58r59.hi"}, /* 174 */
  { 59, "$r56r57r58r59.t"}, /* 175 */
  { 60, "$r60"}, /* 176 */
  { 60, "$r60r61.lo"}, /* 177 */
  { 60, "$r60r61r62r63.x"}, /* 178 */
  { 61, "$r61"}, /* 179 */
  { 61, "$r60r61.hi"}, /* 180 */
  { 61, "$r60r61r62r63.y"}, /* 181 */
  { 62, "$r62"}, /* 182 */
  { 62, "$r62r63.lo"}, /* 183 */
  { 62, "$r60r61r62r63.z"}, /* 184 */
  { 63, "$r63"}, /* 185 */
  { 63, "$r62r63.hi"}, /* 186 */
  { 63, "$r60r61r62r63.t"}, /* 187 */
  { 0, "$r0r1"}, /* 188 */
  { 0, "$r0r1r2r3.lo"}, /* 189 */
  { 1, "$r2r3"}, /* 190 */
  { 1, "$r0r1r2r3.hi"}, /* 191 */
  { 2, "$r4r5"}, /* 192 */
  { 2, "$r4r5r6r7.lo"}, /* 193 */
  { 3, "$r6r7"}, /* 194 */
  { 3, "$r4r5r6r7.hi"}, /* 195 */
  { 4, "$r8r9"}, /* 196 */
  { 4, "$r8r9r10r11.lo"}, /* 197 */
  { 5, "$r10r11"}, /* 198 */
  { 5, "$r8r9r10r11.hi"}, /* 199 */
  { 6, "$r12r13"}, /* 200 */
  { 6, "$r12r13r14r15.lo"}, /* 201 */
  { 7, "$r14r15"}, /* 202 */
  { 7, "$r12r13r14r15.hi"}, /* 203 */
  { 8, "$r16r17"}, /* 204 */
  { 8, "$r16r17r18r19.lo"}, /* 205 */
  { 9, "$r18r19"}, /* 206 */
  { 9, "$r16r17r18r19.hi"}, /* 207 */
  { 10, "$r20r21"}, /* 208 */
  { 10, "$r20r21r22r23.lo"}, /* 209 */
  { 11, "$r22r23"}, /* 210 */
  { 11, "$r20r21r22r23.hi"}, /* 211 */
  { 12, "$r24r25"}, /* 212 */
  { 12, "$r24r25r26r27.lo"}, /* 213 */
  { 13, "$r26r27"}, /* 214 */
  { 13, "$r24r25r26r27.hi"}, /* 215 */
  { 14, "$r28r29"}, /* 216 */
  { 14, "$r28r29r30r31.lo"}, /* 217 */
  { 15, "$r30r31"}, /* 218 */
  { 15, "$r28r29r30r31.hi"}, /* 219 */
  { 16, "$r32r33"}, /* 220 */
  { 16, "$r32r33r34r35.lo"}, /* 221 */
  { 17, "$r34r35"}, /* 222 */
  { 17, "$r32r33r34r35.hi"}, /* 223 */
  { 18, "$r36r37"}, /* 224 */
  { 18, "$r36r37r38r39.lo"}, /* 225 */
  { 19, "$r38r39"}, /* 226 */
  { 19, "$r36r37r38r39.hi"}, /* 227 */
  { 20, "$r40r41"}, /* 228 */
  { 20, "$r40r41r42r43.lo"}, /* 229 */
  { 21, "$r42r43"}, /* 230 */
  { 21, "$r40r41r42r43.hi"}, /* 231 */
  { 22, "$r44r45"}, /* 232 */
  { 22, "$r44r45r46r47.lo"}, /* 233 */
  { 23, "$r46r47"}, /* 234 */
  { 23, "$r44r45r46r47.hi"}, /* 235 */
  { 24, "$r48r49"}, /* 236 */
  { 24, "$r48r49r50r51.lo"}, /* 237 */
  { 25, "$r50r51"}, /* 238 */
  { 25, "$r48r49r50r51.hi"}, /* 239 */
  { 26, "$r52r53"}, /* 240 */
  { 26, "$r52r53r54r55.lo"}, /* 241 */
  { 27, "$r54r55"}, /* 242 */
  { 27, "$r52r53r54r55.hi"}, /* 243 */
  { 28, "$r56r57"}, /* 244 */
  { 28, "$r56r57r58r59.lo"}, /* 245 */
  { 29, "$r58r59"}, /* 246 */
  { 29, "$r56r57r58r59.hi"}, /* 247 */
  { 30, "$r60r61"}, /* 248 */
  { 30, "$r60r61r62r63.lo"}, /* 249 */
  { 31, "$r62r63"}, /* 250 */
  { 31, "$r60r61r62r63.hi"}, /* 251 */
  { 0, "$r0r1r2r3"}, /* 252 */
  { 1, "$r4r5r6r7"}, /* 253 */
  { 2, "$r8r9r10r11"}, /* 254 */
  { 3, "$r12r13r14r15"}, /* 255 */
  { 4, "$r16r17r18r19"}, /* 256 */
  { 5, "$r20r21r22r23"}, /* 257 */
  { 6, "$r24r25r26r27"}, /* 258 */
  { 7, "$r28r29r30r31"}, /* 259 */
  { 8, "$r32r33r34r35"}, /* 260 */
  { 9, "$r36r37r38r39"}, /* 261 */
  { 10, "$r40r41r42r43"}, /* 262 */
  { 11, "$r44r45r46r47"}, /* 263 */
  { 12, "$r48r49r50r51"}, /* 264 */
  { 13, "$r52r53r54r55"}, /* 265 */
  { 14, "$r56r57r58r59"}, /* 266 */
  { 15, "$r60r61r62r63"}, /* 267 */
  { 0, "$pc"}, /* 268 */
  { 0, "$s0"}, /* 269 */
  { 1, "$ps"}, /* 270 */
  { 1, "$s1"}, /* 271 */
  { 2, "$pcr"}, /* 272 */
  { 2, "$s2"}, /* 273 */
  { 3, "$ra"}, /* 274 */
  { 3, "$s3"}, /* 275 */
  { 4, "$cs"}, /* 276 */
  { 4, "$s4"}, /* 277 */
  { 5, "$csit"}, /* 278 */
  { 5, "$s5"}, /* 279 */
  { 6, "$aespc"}, /* 280 */
  { 6, "$s6"}, /* 281 */
  { 7, "$ls"}, /* 282 */
  { 7, "$s7"}, /* 283 */
  { 8, "$le"}, /* 284 */
  { 8, "$s8"}, /* 285 */
  { 9, "$lc"}, /* 286 */
  { 9, "$s9"}, /* 287 */
  { 10, "$ipe"}, /* 288 */
  { 10, "$s10"}, /* 289 */
  { 11, "$men"}, /* 290 */
  { 11, "$s11"}, /* 291 */
  { 12, "$pmc"}, /* 292 */
  { 12, "$s12"}, /* 293 */
  { 13, "$pm0"}, /* 294 */
  { 13, "$s13"}, /* 295 */
  { 14, "$pm1"}, /* 296 */
  { 14, "$s14"}, /* 297 */
  { 15, "$pm2"}, /* 298 */
  { 15, "$s15"}, /* 299 */
  { 16, "$pm3"}, /* 300 */
  { 16, "$s16"}, /* 301 */
  { 17, "$pmsa"}, /* 302 */
  { 17, "$s17"}, /* 303 */
  { 18, "$tcr"}, /* 304 */
  { 18, "$s18"}, /* 305 */
  { 19, "$t0v"}, /* 306 */
  { 19, "$s19"}, /* 307 */
  { 20, "$t1v"}, /* 308 */
  { 20, "$s20"}, /* 309 */
  { 21, "$t0r"}, /* 310 */
  { 21, "$s21"}, /* 311 */
  { 22, "$t1r"}, /* 312 */
  { 22, "$s22"}, /* 313 */
  { 23, "$wdv"}, /* 314 */
  { 23, "$s23"}, /* 315 */
  { 24, "$wdr"}, /* 316 */
  { 24, "$s24"}, /* 317 */
  { 25, "$ile"}, /* 318 */
  { 25, "$s25"}, /* 319 */
  { 26, "$ill"}, /* 320 */
  { 26, "$s26"}, /* 321 */
  { 27, "$ilr"}, /* 322 */
  { 27, "$s27"}, /* 323 */
  { 28, "$mmc"}, /* 324 */
  { 28, "$s28"}, /* 325 */
  { 29, "$tel"}, /* 326 */
  { 29, "$s29"}, /* 327 */
  { 30, "$teh"}, /* 328 */
  { 30, "$s30"}, /* 329 */
  { 31, "$ixc"}, /* 330 */
  { 31, "$s31"}, /* 331 */
  { 32, "$syo"}, /* 332 */
  { 32, "$s32"}, /* 333 */
  { 33, "$hto"}, /* 334 */
  { 33, "$s33"}, /* 335 */
  { 34, "$ito"}, /* 336 */
  { 34, "$s34"}, /* 337 */
  { 35, "$do"}, /* 338 */
  { 35, "$s35"}, /* 339 */
  { 36, "$mo"}, /* 340 */
  { 36, "$s36"}, /* 341 */
  { 37, "$pso"}, /* 342 */
  { 37, "$s37"}, /* 343 */
  { 38, "$tpcm0"}, /* 344 */
  { 38, "$s38"}, /* 345 */
  { 39, "$tpcm1"}, /* 346 */
  { 39, "$s39"}, /* 347 */
  { 40, "$res40"}, /* 348 */
  { 40, "$s40"}, /* 349 */
  { 41, "$dba0"}, /* 350 */
  { 41, "$s41"}, /* 351 */
  { 42, "$dba1"}, /* 352 */
  { 42, "$s42"}, /* 353 */
  { 43, "$dwa0"}, /* 354 */
  { 43, "$s43"}, /* 355 */
  { 44, "$dwa1"}, /* 356 */
  { 44, "$s44"}, /* 357 */
  { 45, "$mes"}, /* 358 */
  { 45, "$s45"}, /* 359 */
  { 46, "$ws"}, /* 360 */
  { 46, "$s46"}, /* 361 */
  { 47, "$dc0"}, /* 362 */
  { 47, "$s47"}, /* 363 */
  { 48, "$dc1"}, /* 364 */
  { 48, "$s48"}, /* 365 */
  { 49, "$dc2"}, /* 366 */
  { 49, "$s49"}, /* 367 */
  { 50, "$dc3"}, /* 368 */
  { 50, "$s50"}, /* 369 */
  { 51, "$dba2"}, /* 370 */
  { 51, "$s51"}, /* 371 */
  { 52, "$dba3"}, /* 372 */
  { 52, "$s52"}, /* 373 */
  { 53, "$dwa2"}, /* 374 */
  { 53, "$s53"}, /* 375 */
  { 54, "$dwa3"}, /* 376 */
  { 54, "$s54"}, /* 377 */
  { 55, "$tpcm2"}, /* 378 */
  { 55, "$s55"}, /* 379 */
  { 56, "$tpcmc"}, /* 380 */
  { 56, "$s56"}, /* 381 */
  { 57, "$res57"}, /* 382 */
  { 57, "$s57"}, /* 383 */
  { 58, "$res58"}, /* 384 */
  { 58, "$s58"}, /* 385 */
  { 59, "$res59"}, /* 386 */
  { 59, "$s59"}, /* 387 */
  { 60, "$res60"}, /* 388 */
  { 60, "$s60"}, /* 389 */
  { 61, "$res61"}, /* 390 */
  { 61, "$s61"}, /* 391 */
  { 62, "$res62"}, /* 392 */
  { 62, "$s62"}, /* 393 */
  { 63, "$res63"}, /* 394 */
  { 63, "$s63"}, /* 395 */
  { 64, "$spc_pl0"}, /* 396 */
  { 64, "$s64"}, /* 397 */
  { 65, "$spc_pl1"}, /* 398 */
  { 65, "$s65"}, /* 399 */
  { 66, "$spc_pl2"}, /* 400 */
  { 66, "$s66"}, /* 401 */
  { 67, "$spc_pl3"}, /* 402 */
  { 67, "$s67"}, /* 403 */
  { 68, "$sps_pl0"}, /* 404 */
  { 68, "$s68"}, /* 405 */
  { 69, "$sps_pl1"}, /* 406 */
  { 69, "$s69"}, /* 407 */
  { 70, "$sps_pl2"}, /* 408 */
  { 70, "$s70"}, /* 409 */
  { 71, "$sps_pl3"}, /* 410 */
  { 71, "$s71"}, /* 411 */
  { 72, "$ea_pl0"}, /* 412 */
  { 72, "$s72"}, /* 413 */
  { 73, "$ea_pl1"}, /* 414 */
  { 73, "$s73"}, /* 415 */
  { 74, "$ea_pl2"}, /* 416 */
  { 74, "$s74"}, /* 417 */
  { 75, "$ea_pl3"}, /* 418 */
  { 75, "$s75"}, /* 419 */
  { 76, "$ev_pl0"}, /* 420 */
  { 76, "$s76"}, /* 421 */
  { 77, "$ev_pl1"}, /* 422 */
  { 77, "$s77"}, /* 423 */
  { 78, "$ev_pl2"}, /* 424 */
  { 78, "$s78"}, /* 425 */
  { 79, "$ev_pl3"}, /* 426 */
  { 79, "$s79"}, /* 427 */
  { 80, "$sr_pl0"}, /* 428 */
  { 80, "$s80"}, /* 429 */
  { 81, "$sr_pl1"}, /* 430 */
  { 81, "$s81"}, /* 431 */
  { 82, "$sr_pl2"}, /* 432 */
  { 82, "$s82"}, /* 433 */
  { 83, "$sr_pl3"}, /* 434 */
  { 83, "$s83"}, /* 435 */
  { 84, "$es_pl0"}, /* 436 */
  { 84, "$s84"}, /* 437 */
  { 85, "$es_pl1"}, /* 438 */
  { 85, "$s85"}, /* 439 */
  { 86, "$es_pl2"}, /* 440 */
  { 86, "$s86"}, /* 441 */
  { 87, "$es_pl3"}, /* 442 */
  { 87, "$s87"}, /* 443 */
  { 88, "$sid_pl0"}, /* 444 */
  { 88, "$s88"}, /* 445 */
  { 89, "$sid_pl1"}, /* 446 */
  { 89, "$s89"}, /* 447 */
  { 90, "$sid_pl2"}, /* 448 */
  { 90, "$s90"}, /* 449 */
  { 91, "$sid_pl3"}, /* 450 */
  { 91, "$s91"}, /* 451 */
  { 92, "$sr1_pl0"}, /* 452 */
  { 92, "$s92"}, /* 453 */
  { 93, "$sr1_pl1"}, /* 454 */
  { 93, "$s93"}, /* 455 */
  { 94, "$sr1_pl2"}, /* 456 */
  { 94, "$s94"}, /* 457 */
  { 95, "$sr1_pl3"}, /* 458 */
  { 95, "$s95"}, /* 459 */
  { 96, "$syow"}, /* 460 */
  { 96, "$s96"}, /* 461 */
  { 97, "$htow"}, /* 462 */
  { 97, "$s97"}, /* 463 */
  { 98, "$itow"}, /* 464 */
  { 98, "$s98"}, /* 465 */
  { 99, "$dow"}, /* 466 */
  { 99, "$s99"}, /* 467 */
  { 100, "$mow"}, /* 468 */
  { 100, "$s100"}, /* 469 */
  { 101, "$psow"}, /* 470 */
  { 101, "$s101"}, /* 471 */
  { 102, "$res102"}, /* 472 */
  { 102, "$s102"}, /* 473 */
  { 103, "$res103"}, /* 474 */
  { 103, "$s103"}, /* 475 */
  { 104, "$res104"}, /* 476 */
  { 104, "$s104"}, /* 477 */
  { 105, "$res105"}, /* 478 */
  { 105, "$s105"}, /* 479 */
  { 106, "$res106"}, /* 480 */
  { 106, "$s106"}, /* 481 */
  { 107, "$res107"}, /* 482 */
  { 107, "$s107"}, /* 483 */
  { 108, "$res108"}, /* 484 */
  { 108, "$s108"}, /* 485 */
  { 109, "$res109"}, /* 486 */
  { 109, "$s109"}, /* 487 */
  { 110, "$res110"}, /* 488 */
  { 110, "$s110"}, /* 489 */
  { 111, "$res111"}, /* 490 */
  { 111, "$s111"}, /* 491 */
  { 112, "$res112"}, /* 492 */
  { 112, "$s112"}, /* 493 */
  { 113, "$res113"}, /* 494 */
  { 113, "$s113"}, /* 495 */
  { 114, "$res114"}, /* 496 */
  { 114, "$s114"}, /* 497 */
  { 115, "$res115"}, /* 498 */
  { 115, "$s115"}, /* 499 */
  { 116, "$res116"}, /* 500 */
  { 116, "$s116"}, /* 501 */
  { 117, "$res117"}, /* 502 */
  { 117, "$s117"}, /* 503 */
  { 118, "$res118"}, /* 504 */
  { 118, "$s118"}, /* 505 */
  { 119, "$res119"}, /* 506 */
  { 119, "$s119"}, /* 507 */
  { 120, "$res120"}, /* 508 */
  { 120, "$s120"}, /* 509 */
  { 121, "$res121"}, /* 510 */
  { 121, "$s121"}, /* 511 */
  { 122, "$res122"}, /* 512 */
  { 122, "$s122"}, /* 513 */
  { 123, "$res123"}, /* 514 */
  { 123, "$s123"}, /* 515 */
  { 124, "$res124"}, /* 516 */
  { 124, "$s124"}, /* 517 */
  { 125, "$res125"}, /* 518 */
  { 125, "$s125"}, /* 519 */
  { 126, "$res126"}, /* 520 */
  { 126, "$s126"}, /* 521 */
  { 127, "$res127"}, /* 522 */
  { 127, "$s127"}, /* 523 */
  { 128, "$spc"}, /* 524 */
  { 128, "$s128"}, /* 525 */
  { 129, "$res129"}, /* 526 */
  { 129, "$s129"}, /* 527 */
  { 130, "$res130"}, /* 528 */
  { 130, "$s130"}, /* 529 */
  { 131, "$res131"}, /* 530 */
  { 131, "$s131"}, /* 531 */
  { 132, "$sps"}, /* 532 */
  { 132, "$s132"}, /* 533 */
  { 133, "$res133"}, /* 534 */
  { 133, "$s133"}, /* 535 */
  { 134, "$res134"}, /* 536 */
  { 134, "$s134"}, /* 537 */
  { 135, "$res135"}, /* 538 */
  { 135, "$s135"}, /* 539 */
  { 136, "$ea"}, /* 540 */
  { 136, "$s136"}, /* 541 */
  { 137, "$res137"}, /* 542 */
  { 137, "$s137"}, /* 543 */
  { 138, "$res138"}, /* 544 */
  { 138, "$s138"}, /* 545 */
  { 139, "$res139"}, /* 546 */
  { 139, "$s139"}, /* 547 */
  { 140, "$ev"}, /* 548 */
  { 140, "$s140"}, /* 549 */
  { 141, "$res141"}, /* 550 */
  { 141, "$s141"}, /* 551 */
  { 142, "$res142"}, /* 552 */
  { 142, "$s142"}, /* 553 */
  { 143, "$res143"}, /* 554 */
  { 143, "$s143"}, /* 555 */
  { 144, "$sr"}, /* 556 */
  { 144, "$s144"}, /* 557 */
  { 145, "$res145"}, /* 558 */
  { 145, "$s145"}, /* 559 */
  { 146, "$res146"}, /* 560 */
  { 146, "$s146"}, /* 561 */
  { 147, "$res147"}, /* 562 */
  { 147, "$s147"}, /* 563 */
  { 148, "$es"}, /* 564 */
  { 148, "$s148"}, /* 565 */
  { 149, "$res149"}, /* 566 */
  { 149, "$s149"}, /* 567 */
  { 150, "$res150"}, /* 568 */
  { 150, "$s150"}, /* 569 */
  { 151, "$res151"}, /* 570 */
  { 151, "$s151"}, /* 571 */
  { 152, "$sid"}, /* 572 */
  { 152, "$s152"}, /* 573 */
  { 153, "$res153"}, /* 574 */
  { 153, "$s153"}, /* 575 */
  { 154, "$res154"}, /* 576 */
  { 154, "$s154"}, /* 577 */
  { 155, "$res155"}, /* 578 */
  { 155, "$s155"}, /* 579 */
  { 156, "$sr1"}, /* 580 */
  { 156, "$s156"}, /* 581 */
  { 157, "$res157"}, /* 582 */
  { 157, "$s157"}, /* 583 */
  { 158, "$res158"}, /* 584 */
  { 158, "$s158"}, /* 585 */
  { 159, "$res159"}, /* 586 */
  { 159, "$s159"}, /* 587 */
  { 160, "$res160"}, /* 588 */
  { 160, "$s160"}, /* 589 */
  { 161, "$res161"}, /* 590 */
  { 161, "$s161"}, /* 591 */
  { 162, "$res162"}, /* 592 */
  { 162, "$s162"}, /* 593 */
  { 163, "$res163"}, /* 594 */
  { 163, "$s163"}, /* 595 */
  { 164, "$res164"}, /* 596 */
  { 164, "$s164"}, /* 597 */
  { 165, "$res165"}, /* 598 */
  { 165, "$s165"}, /* 599 */
  { 166, "$res166"}, /* 600 */
  { 166, "$s166"}, /* 601 */
  { 167, "$res167"}, /* 602 */
  { 167, "$s167"}, /* 603 */
  { 168, "$res168"}, /* 604 */
  { 168, "$s168"}, /* 605 */
  { 169, "$res169"}, /* 606 */
  { 169, "$s169"}, /* 607 */
  { 170, "$res170"}, /* 608 */
  { 170, "$s170"}, /* 609 */
  { 171, "$res171"}, /* 610 */
  { 171, "$s171"}, /* 611 */
  { 172, "$res172"}, /* 612 */
  { 172, "$s172"}, /* 613 */
  { 173, "$res173"}, /* 614 */
  { 173, "$s173"}, /* 615 */
  { 174, "$res174"}, /* 616 */
  { 174, "$s174"}, /* 617 */
  { 175, "$res175"}, /* 618 */
  { 175, "$s175"}, /* 619 */
  { 176, "$res176"}, /* 620 */
  { 176, "$s176"}, /* 621 */
  { 177, "$res177"}, /* 622 */
  { 177, "$s177"}, /* 623 */
  { 178, "$res178"}, /* 624 */
  { 178, "$s178"}, /* 625 */
  { 179, "$res179"}, /* 626 */
  { 179, "$s179"}, /* 627 */
  { 180, "$res180"}, /* 628 */
  { 180, "$s180"}, /* 629 */
  { 181, "$res181"}, /* 630 */
  { 181, "$s181"}, /* 631 */
  { 182, "$res182"}, /* 632 */
  { 182, "$s182"}, /* 633 */
  { 183, "$res183"}, /* 634 */
  { 183, "$s183"}, /* 635 */
  { 184, "$res184"}, /* 636 */
  { 184, "$s184"}, /* 637 */
  { 185, "$res185"}, /* 638 */
  { 185, "$s185"}, /* 639 */
  { 186, "$res186"}, /* 640 */
  { 186, "$s186"}, /* 641 */
  { 187, "$res187"}, /* 642 */
  { 187, "$s187"}, /* 643 */
  { 188, "$res188"}, /* 644 */
  { 188, "$s188"}, /* 645 */
  { 189, "$res189"}, /* 646 */
  { 189, "$s189"}, /* 647 */
  { 190, "$res190"}, /* 648 */
  { 190, "$s190"}, /* 649 */
  { 191, "$res191"}, /* 650 */
  { 191, "$s191"}, /* 651 */
  { 192, "$res192"}, /* 652 */
  { 192, "$s192"}, /* 653 */
  { 193, "$res193"}, /* 654 */
  { 193, "$s193"}, /* 655 */
  { 194, "$res194"}, /* 656 */
  { 194, "$s194"}, /* 657 */
  { 195, "$res195"}, /* 658 */
  { 195, "$s195"}, /* 659 */
  { 196, "$res196"}, /* 660 */
  { 196, "$s196"}, /* 661 */
  { 197, "$res197"}, /* 662 */
  { 197, "$s197"}, /* 663 */
  { 198, "$res198"}, /* 664 */
  { 198, "$s198"}, /* 665 */
  { 199, "$res199"}, /* 666 */
  { 199, "$s199"}, /* 667 */
  { 200, "$res200"}, /* 668 */
  { 200, "$s200"}, /* 669 */
  { 201, "$res201"}, /* 670 */
  { 201, "$s201"}, /* 671 */
  { 202, "$res202"}, /* 672 */
  { 202, "$s202"}, /* 673 */
  { 203, "$res203"}, /* 674 */
  { 203, "$s203"}, /* 675 */
  { 204, "$res204"}, /* 676 */
  { 204, "$s204"}, /* 677 */
  { 205, "$res205"}, /* 678 */
  { 205, "$s205"}, /* 679 */
  { 206, "$res206"}, /* 680 */
  { 206, "$s206"}, /* 681 */
  { 207, "$res207"}, /* 682 */
  { 207, "$s207"}, /* 683 */
  { 208, "$res208"}, /* 684 */
  { 208, "$s208"}, /* 685 */
  { 209, "$res209"}, /* 686 */
  { 209, "$s209"}, /* 687 */
  { 210, "$res210"}, /* 688 */
  { 210, "$s210"}, /* 689 */
  { 211, "$res211"}, /* 690 */
  { 211, "$s211"}, /* 691 */
  { 212, "$res212"}, /* 692 */
  { 212, "$s212"}, /* 693 */
  { 213, "$res213"}, /* 694 */
  { 213, "$s213"}, /* 695 */
  { 214, "$res214"}, /* 696 */
  { 214, "$s214"}, /* 697 */
  { 215, "$res215"}, /* 698 */
  { 215, "$s215"}, /* 699 */
  { 216, "$res216"}, /* 700 */
  { 216, "$s216"}, /* 701 */
  { 217, "$res217"}, /* 702 */
  { 217, "$s217"}, /* 703 */
  { 218, "$res218"}, /* 704 */
  { 218, "$s218"}, /* 705 */
  { 219, "$res219"}, /* 706 */
  { 219, "$s219"}, /* 707 */
  { 220, "$res220"}, /* 708 */
  { 220, "$s220"}, /* 709 */
  { 221, "$res221"}, /* 710 */
  { 221, "$s221"}, /* 711 */
  { 222, "$res222"}, /* 712 */
  { 222, "$s222"}, /* 713 */
  { 223, "$res223"}, /* 714 */
  { 223, "$s223"}, /* 715 */
  { 224, "$res224"}, /* 716 */
  { 224, "$s224"}, /* 717 */
  { 225, "$res225"}, /* 718 */
  { 225, "$s225"}, /* 719 */
  { 226, "$res226"}, /* 720 */
  { 226, "$s226"}, /* 721 */
  { 227, "$res227"}, /* 722 */
  { 227, "$s227"}, /* 723 */
  { 228, "$res228"}, /* 724 */
  { 228, "$s228"}, /* 725 */
  { 229, "$res229"}, /* 726 */
  { 229, "$s229"}, /* 727 */
  { 230, "$res230"}, /* 728 */
  { 230, "$s230"}, /* 729 */
  { 231, "$res231"}, /* 730 */
  { 231, "$s231"}, /* 731 */
  { 232, "$res232"}, /* 732 */
  { 232, "$s232"}, /* 733 */
  { 233, "$res233"}, /* 734 */
  { 233, "$s233"}, /* 735 */
  { 234, "$res234"}, /* 736 */
  { 234, "$s234"}, /* 737 */
  { 235, "$res235"}, /* 738 */
  { 235, "$s235"}, /* 739 */
  { 236, "$res236"}, /* 740 */
  { 236, "$s236"}, /* 741 */
  { 237, "$res237"}, /* 742 */
  { 237, "$s237"}, /* 743 */
  { 238, "$res238"}, /* 744 */
  { 238, "$s238"}, /* 745 */
  { 239, "$res239"}, /* 746 */
  { 239, "$s239"}, /* 747 */
  { 240, "$res240"}, /* 748 */
  { 240, "$s240"}, /* 749 */
  { 241, "$res241"}, /* 750 */
  { 241, "$s241"}, /* 751 */
  { 242, "$res242"}, /* 752 */
  { 242, "$s242"}, /* 753 */
  { 243, "$res243"}, /* 754 */
  { 243, "$s243"}, /* 755 */
  { 244, "$res244"}, /* 756 */
  { 244, "$s244"}, /* 757 */
  { 245, "$res245"}, /* 758 */
  { 245, "$s245"}, /* 759 */
  { 246, "$res246"}, /* 760 */
  { 246, "$s246"}, /* 761 */
  { 247, "$res247"}, /* 762 */
  { 247, "$s247"}, /* 763 */
  { 248, "$res248"}, /* 764 */
  { 248, "$s248"}, /* 765 */
  { 249, "$res249"}, /* 766 */
  { 249, "$s249"}, /* 767 */
  { 250, "$res250"}, /* 768 */
  { 250, "$s250"}, /* 769 */
  { 251, "$res251"}, /* 770 */
  { 251, "$s251"}, /* 771 */
  { 252, "$res252"}, /* 772 */
  { 252, "$s252"}, /* 773 */
  { 253, "$res253"}, /* 774 */
  { 253, "$s253"}, /* 775 */
  { 254, "$res254"}, /* 776 */
  { 254, "$s254"}, /* 777 */
  { 255, "$res255"}, /* 778 */
  { 255, "$s255"}, /* 779 */
  { 256, "$vsfr0"}, /* 780 */
  { 256, "$s256"}, /* 781 */
  { 257, "$vsfr1"}, /* 782 */
  { 257, "$s257"}, /* 783 */
  { 258, "$vsfr2"}, /* 784 */
  { 258, "$s258"}, /* 785 */
  { 259, "$vsfr3"}, /* 786 */
  { 259, "$s259"}, /* 787 */
  { 260, "$vsfr4"}, /* 788 */
  { 260, "$s260"}, /* 789 */
  { 261, "$vsfr5"}, /* 790 */
  { 261, "$s261"}, /* 791 */
  { 262, "$vsfr6"}, /* 792 */
  { 262, "$s262"}, /* 793 */
  { 263, "$vsfr7"}, /* 794 */
  { 263, "$s263"}, /* 795 */
  { 264, "$vsfr8"}, /* 796 */
  { 264, "$s264"}, /* 797 */
  { 265, "$vsfr9"}, /* 798 */
  { 265, "$s265"}, /* 799 */
  { 266, "$vsfr10"}, /* 800 */
  { 266, "$s266"}, /* 801 */
  { 267, "$vsfr11"}, /* 802 */
  { 267, "$s267"}, /* 803 */
  { 268, "$vsfr12"}, /* 804 */
  { 268, "$s268"}, /* 805 */
  { 269, "$vsfr13"}, /* 806 */
  { 269, "$s269"}, /* 807 */
  { 270, "$vsfr14"}, /* 808 */
  { 270, "$s270"}, /* 809 */
  { 271, "$vsfr15"}, /* 810 */
  { 271, "$s271"}, /* 811 */
  { 272, "$vsfr16"}, /* 812 */
  { 272, "$s272"}, /* 813 */
  { 273, "$vsfr17"}, /* 814 */
  { 273, "$s273"}, /* 815 */
  { 274, "$vsfr18"}, /* 816 */
  { 274, "$s274"}, /* 817 */
  { 275, "$vsfr19"}, /* 818 */
  { 275, "$s275"}, /* 819 */
  { 276, "$vsfr20"}, /* 820 */
  { 276, "$s276"}, /* 821 */
  { 277, "$vsfr21"}, /* 822 */
  { 277, "$s277"}, /* 823 */
  { 278, "$vsfr22"}, /* 824 */
  { 278, "$s278"}, /* 825 */
  { 279, "$vsfr23"}, /* 826 */
  { 279, "$s279"}, /* 827 */
  { 280, "$vsfr24"}, /* 828 */
  { 280, "$s280"}, /* 829 */
  { 281, "$vsfr25"}, /* 830 */
  { 281, "$s281"}, /* 831 */
  { 282, "$vsfr26"}, /* 832 */
  { 282, "$s282"}, /* 833 */
  { 283, "$vsfr27"}, /* 834 */
  { 283, "$s283"}, /* 835 */
  { 284, "$vsfr28"}, /* 836 */
  { 284, "$s284"}, /* 837 */
  { 285, "$vsfr29"}, /* 838 */
  { 285, "$s285"}, /* 839 */
  { 286, "$vsfr30"}, /* 840 */
  { 286, "$s286"}, /* 841 */
  { 287, "$vsfr31"}, /* 842 */
  { 287, "$s287"}, /* 843 */
  { 288, "$vsfr32"}, /* 844 */
  { 288, "$s288"}, /* 845 */
  { 289, "$vsfr33"}, /* 846 */
  { 289, "$s289"}, /* 847 */
  { 290, "$vsfr34"}, /* 848 */
  { 290, "$s290"}, /* 849 */
  { 291, "$vsfr35"}, /* 850 */
  { 291, "$s291"}, /* 851 */
  { 292, "$vsfr36"}, /* 852 */
  { 292, "$s292"}, /* 853 */
  { 293, "$vsfr37"}, /* 854 */
  { 293, "$s293"}, /* 855 */
  { 294, "$vsfr38"}, /* 856 */
  { 294, "$s294"}, /* 857 */
  { 295, "$vsfr39"}, /* 858 */
  { 295, "$s295"}, /* 859 */
  { 296, "$vsfr40"}, /* 860 */
  { 296, "$s296"}, /* 861 */
  { 297, "$vsfr41"}, /* 862 */
  { 297, "$s297"}, /* 863 */
  { 298, "$vsfr42"}, /* 864 */
  { 298, "$s298"}, /* 865 */
  { 299, "$vsfr43"}, /* 866 */
  { 299, "$s299"}, /* 867 */
  { 300, "$vsfr44"}, /* 868 */
  { 300, "$s300"}, /* 869 */
  { 301, "$vsfr45"}, /* 870 */
  { 301, "$s301"}, /* 871 */
  { 302, "$vsfr46"}, /* 872 */
  { 302, "$s302"}, /* 873 */
  { 303, "$vsfr47"}, /* 874 */
  { 303, "$s303"}, /* 875 */
  { 304, "$vsfr48"}, /* 876 */
  { 304, "$s304"}, /* 877 */
  { 305, "$vsfr49"}, /* 878 */
  { 305, "$s305"}, /* 879 */
  { 306, "$vsfr50"}, /* 880 */
  { 306, "$s306"}, /* 881 */
  { 307, "$vsfr51"}, /* 882 */
  { 307, "$s307"}, /* 883 */
  { 308, "$vsfr52"}, /* 884 */
  { 308, "$s308"}, /* 885 */
  { 309, "$vsfr53"}, /* 886 */
  { 309, "$s309"}, /* 887 */
  { 310, "$vsfr54"}, /* 888 */
  { 310, "$s310"}, /* 889 */
  { 311, "$vsfr55"}, /* 890 */
  { 311, "$s311"}, /* 891 */
  { 312, "$vsfr56"}, /* 892 */
  { 312, "$s312"}, /* 893 */
  { 313, "$vsfr57"}, /* 894 */
  { 313, "$s313"}, /* 895 */
  { 314, "$vsfr58"}, /* 896 */
  { 314, "$s314"}, /* 897 */
  { 315, "$vsfr59"}, /* 898 */
  { 315, "$s315"}, /* 899 */
  { 316, "$vsfr60"}, /* 900 */
  { 316, "$s316"}, /* 901 */
  { 317, "$vsfr61"}, /* 902 */
  { 317, "$s317"}, /* 903 */
  { 318, "$vsfr62"}, /* 904 */
  { 318, "$s318"}, /* 905 */
  { 319, "$vsfr63"}, /* 906 */
  { 319, "$s319"}, /* 907 */
  { 320, "$vsfr64"}, /* 908 */
  { 320, "$s320"}, /* 909 */
  { 321, "$vsfr65"}, /* 910 */
  { 321, "$s321"}, /* 911 */
  { 322, "$vsfr66"}, /* 912 */
  { 322, "$s322"}, /* 913 */
  { 323, "$vsfr67"}, /* 914 */
  { 323, "$s323"}, /* 915 */
  { 324, "$vsfr68"}, /* 916 */
  { 324, "$s324"}, /* 917 */
  { 325, "$vsfr69"}, /* 918 */
  { 325, "$s325"}, /* 919 */
  { 326, "$vsfr70"}, /* 920 */
  { 326, "$s326"}, /* 921 */
  { 327, "$vsfr71"}, /* 922 */
  { 327, "$s327"}, /* 923 */
  { 328, "$vsfr72"}, /* 924 */
  { 328, "$s328"}, /* 925 */
  { 329, "$vsfr73"}, /* 926 */
  { 329, "$s329"}, /* 927 */
  { 330, "$vsfr74"}, /* 928 */
  { 330, "$s330"}, /* 929 */
  { 331, "$vsfr75"}, /* 930 */
  { 331, "$s331"}, /* 931 */
  { 332, "$vsfr76"}, /* 932 */
  { 332, "$s332"}, /* 933 */
  { 333, "$vsfr77"}, /* 934 */
  { 333, "$s333"}, /* 935 */
  { 334, "$vsfr78"}, /* 936 */
  { 334, "$s334"}, /* 937 */
  { 335, "$vsfr79"}, /* 938 */
  { 335, "$s335"}, /* 939 */
  { 336, "$vsfr80"}, /* 940 */
  { 336, "$s336"}, /* 941 */
  { 337, "$vsfr81"}, /* 942 */
  { 337, "$s337"}, /* 943 */
  { 338, "$vsfr82"}, /* 944 */
  { 338, "$s338"}, /* 945 */
  { 339, "$vsfr83"}, /* 946 */
  { 339, "$s339"}, /* 947 */
  { 340, "$vsfr84"}, /* 948 */
  { 340, "$s340"}, /* 949 */
  { 341, "$vsfr85"}, /* 950 */
  { 341, "$s341"}, /* 951 */
  { 342, "$vsfr86"}, /* 952 */
  { 342, "$s342"}, /* 953 */
  { 343, "$vsfr87"}, /* 954 */
  { 343, "$s343"}, /* 955 */
  { 344, "$vsfr88"}, /* 956 */
  { 344, "$s344"}, /* 957 */
  { 345, "$vsfr89"}, /* 958 */
  { 345, "$s345"}, /* 959 */
  { 346, "$vsfr90"}, /* 960 */
  { 346, "$s346"}, /* 961 */
  { 347, "$vsfr91"}, /* 962 */
  { 347, "$s347"}, /* 963 */
  { 348, "$vsfr92"}, /* 964 */
  { 348, "$s348"}, /* 965 */
  { 349, "$vsfr93"}, /* 966 */
  { 349, "$s349"}, /* 967 */
  { 350, "$vsfr94"}, /* 968 */
  { 350, "$s350"}, /* 969 */
  { 351, "$vsfr95"}, /* 970 */
  { 351, "$s351"}, /* 971 */
  { 352, "$vsfr96"}, /* 972 */
  { 352, "$s352"}, /* 973 */
  { 353, "$vsfr97"}, /* 974 */
  { 353, "$s353"}, /* 975 */
  { 354, "$vsfr98"}, /* 976 */
  { 354, "$s354"}, /* 977 */
  { 355, "$vsfr99"}, /* 978 */
  { 355, "$s355"}, /* 979 */
  { 356, "$vsfr100"}, /* 980 */
  { 356, "$s356"}, /* 981 */
  { 357, "$vsfr101"}, /* 982 */
  { 357, "$s357"}, /* 983 */
  { 358, "$vsfr102"}, /* 984 */
  { 358, "$s358"}, /* 985 */
  { 359, "$vsfr103"}, /* 986 */
  { 359, "$s359"}, /* 987 */
  { 360, "$vsfr104"}, /* 988 */
  { 360, "$s360"}, /* 989 */
  { 361, "$vsfr105"}, /* 990 */
  { 361, "$s361"}, /* 991 */
  { 362, "$vsfr106"}, /* 992 */
  { 362, "$s362"}, /* 993 */
  { 363, "$vsfr107"}, /* 994 */
  { 363, "$s363"}, /* 995 */
  { 364, "$vsfr108"}, /* 996 */
  { 364, "$s364"}, /* 997 */
  { 365, "$vsfr109"}, /* 998 */
  { 365, "$s365"}, /* 999 */
  { 366, "$vsfr110"}, /* 1000 */
  { 366, "$s366"}, /* 1001 */
  { 367, "$vsfr111"}, /* 1002 */
  { 367, "$s367"}, /* 1003 */
  { 368, "$vsfr112"}, /* 1004 */
  { 368, "$s368"}, /* 1005 */
  { 369, "$vsfr113"}, /* 1006 */
  { 369, "$s369"}, /* 1007 */
  { 370, "$vsfr114"}, /* 1008 */
  { 370, "$s370"}, /* 1009 */
  { 371, "$vsfr115"}, /* 1010 */
  { 371, "$s371"}, /* 1011 */
  { 372, "$vsfr116"}, /* 1012 */
  { 372, "$s372"}, /* 1013 */
  { 373, "$vsfr117"}, /* 1014 */
  { 373, "$s373"}, /* 1015 */
  { 374, "$vsfr118"}, /* 1016 */
  { 374, "$s374"}, /* 1017 */
  { 375, "$vsfr119"}, /* 1018 */
  { 375, "$s375"}, /* 1019 */
  { 376, "$vsfr120"}, /* 1020 */
  { 376, "$s376"}, /* 1021 */
  { 377, "$vsfr121"}, /* 1022 */
  { 377, "$s377"}, /* 1023 */
  { 378, "$vsfr122"}, /* 1024 */
  { 378, "$s378"}, /* 1025 */
  { 379, "$vsfr123"}, /* 1026 */
  { 379, "$s379"}, /* 1027 */
  { 380, "$vsfr124"}, /* 1028 */
  { 380, "$s380"}, /* 1029 */
  { 381, "$vsfr125"}, /* 1030 */
  { 381, "$s381"}, /* 1031 */
  { 382, "$vsfr126"}, /* 1032 */
  { 382, "$s382"}, /* 1033 */
  { 383, "$vsfr127"}, /* 1034 */
  { 383, "$s383"}, /* 1035 */
  { 384, "$vsfr128"}, /* 1036 */
  { 384, "$s384"}, /* 1037 */
  { 385, "$vsfr129"}, /* 1038 */
  { 385, "$s385"}, /* 1039 */
  { 386, "$vsfr130"}, /* 1040 */
  { 386, "$s386"}, /* 1041 */
  { 387, "$vsfr131"}, /* 1042 */
  { 387, "$s387"}, /* 1043 */
  { 388, "$vsfr132"}, /* 1044 */
  { 388, "$s388"}, /* 1045 */
  { 389, "$vsfr133"}, /* 1046 */
  { 389, "$s389"}, /* 1047 */
  { 390, "$vsfr134"}, /* 1048 */
  { 390, "$s390"}, /* 1049 */
  { 391, "$vsfr135"}, /* 1050 */
  { 391, "$s391"}, /* 1051 */
  { 392, "$vsfr136"}, /* 1052 */
  { 392, "$s392"}, /* 1053 */
  { 393, "$vsfr137"}, /* 1054 */
  { 393, "$s393"}, /* 1055 */
  { 394, "$vsfr138"}, /* 1056 */
  { 394, "$s394"}, /* 1057 */
  { 395, "$vsfr139"}, /* 1058 */
  { 395, "$s395"}, /* 1059 */
  { 396, "$vsfr140"}, /* 1060 */
  { 396, "$s396"}, /* 1061 */
  { 397, "$vsfr141"}, /* 1062 */
  { 397, "$s397"}, /* 1063 */
  { 398, "$vsfr142"}, /* 1064 */
  { 398, "$s398"}, /* 1065 */
  { 399, "$vsfr143"}, /* 1066 */
  { 399, "$s399"}, /* 1067 */
  { 400, "$vsfr144"}, /* 1068 */
  { 400, "$s400"}, /* 1069 */
  { 401, "$vsfr145"}, /* 1070 */
  { 401, "$s401"}, /* 1071 */
  { 402, "$vsfr146"}, /* 1072 */
  { 402, "$s402"}, /* 1073 */
  { 403, "$vsfr147"}, /* 1074 */
  { 403, "$s403"}, /* 1075 */
  { 404, "$vsfr148"}, /* 1076 */
  { 404, "$s404"}, /* 1077 */
  { 405, "$vsfr149"}, /* 1078 */
  { 405, "$s405"}, /* 1079 */
  { 406, "$vsfr150"}, /* 1080 */
  { 406, "$s406"}, /* 1081 */
  { 407, "$vsfr151"}, /* 1082 */
  { 407, "$s407"}, /* 1083 */
  { 408, "$vsfr152"}, /* 1084 */
  { 408, "$s408"}, /* 1085 */
  { 409, "$vsfr153"}, /* 1086 */
  { 409, "$s409"}, /* 1087 */
  { 410, "$vsfr154"}, /* 1088 */
  { 410, "$s410"}, /* 1089 */
  { 411, "$vsfr155"}, /* 1090 */
  { 411, "$s411"}, /* 1091 */
  { 412, "$vsfr156"}, /* 1092 */
  { 412, "$s412"}, /* 1093 */
  { 413, "$vsfr157"}, /* 1094 */
  { 413, "$s413"}, /* 1095 */
  { 414, "$vsfr158"}, /* 1096 */
  { 414, "$s414"}, /* 1097 */
  { 415, "$vsfr159"}, /* 1098 */
  { 415, "$s415"}, /* 1099 */
  { 416, "$vsfr160"}, /* 1100 */
  { 416, "$s416"}, /* 1101 */
  { 417, "$vsfr161"}, /* 1102 */
  { 417, "$s417"}, /* 1103 */
  { 418, "$vsfr162"}, /* 1104 */
  { 418, "$s418"}, /* 1105 */
  { 419, "$vsfr163"}, /* 1106 */
  { 419, "$s419"}, /* 1107 */
  { 420, "$vsfr164"}, /* 1108 */
  { 420, "$s420"}, /* 1109 */
  { 421, "$vsfr165"}, /* 1110 */
  { 421, "$s421"}, /* 1111 */
  { 422, "$vsfr166"}, /* 1112 */
  { 422, "$s422"}, /* 1113 */
  { 423, "$vsfr167"}, /* 1114 */
  { 423, "$s423"}, /* 1115 */
  { 424, "$vsfr168"}, /* 1116 */
  { 424, "$s424"}, /* 1117 */
  { 425, "$vsfr169"}, /* 1118 */
  { 425, "$s425"}, /* 1119 */
  { 426, "$vsfr170"}, /* 1120 */
  { 426, "$s426"}, /* 1121 */
  { 427, "$vsfr171"}, /* 1122 */
  { 427, "$s427"}, /* 1123 */
  { 428, "$vsfr172"}, /* 1124 */
  { 428, "$s428"}, /* 1125 */
  { 429, "$vsfr173"}, /* 1126 */
  { 429, "$s429"}, /* 1127 */
  { 430, "$vsfr174"}, /* 1128 */
  { 430, "$s430"}, /* 1129 */
  { 431, "$vsfr175"}, /* 1130 */
  { 431, "$s431"}, /* 1131 */
  { 432, "$vsfr176"}, /* 1132 */
  { 432, "$s432"}, /* 1133 */
  { 433, "$vsfr177"}, /* 1134 */
  { 433, "$s433"}, /* 1135 */
  { 434, "$vsfr178"}, /* 1136 */
  { 434, "$s434"}, /* 1137 */
  { 435, "$vsfr179"}, /* 1138 */
  { 435, "$s435"}, /* 1139 */
  { 436, "$vsfr180"}, /* 1140 */
  { 436, "$s436"}, /* 1141 */
  { 437, "$vsfr181"}, /* 1142 */
  { 437, "$s437"}, /* 1143 */
  { 438, "$vsfr182"}, /* 1144 */
  { 438, "$s438"}, /* 1145 */
  { 439, "$vsfr183"}, /* 1146 */
  { 439, "$s439"}, /* 1147 */
  { 440, "$vsfr184"}, /* 1148 */
  { 440, "$s440"}, /* 1149 */
  { 441, "$vsfr185"}, /* 1150 */
  { 441, "$s441"}, /* 1151 */
  { 442, "$vsfr186"}, /* 1152 */
  { 442, "$s442"}, /* 1153 */
  { 443, "$vsfr187"}, /* 1154 */
  { 443, "$s443"}, /* 1155 */
  { 444, "$vsfr188"}, /* 1156 */
  { 444, "$s444"}, /* 1157 */
  { 445, "$vsfr189"}, /* 1158 */
  { 445, "$s445"}, /* 1159 */
  { 446, "$vsfr190"}, /* 1160 */
  { 446, "$s446"}, /* 1161 */
  { 447, "$vsfr191"}, /* 1162 */
  { 447, "$s447"}, /* 1163 */
  { 448, "$vsfr192"}, /* 1164 */
  { 448, "$s448"}, /* 1165 */
  { 449, "$vsfr193"}, /* 1166 */
  { 449, "$s449"}, /* 1167 */
  { 450, "$vsfr194"}, /* 1168 */
  { 450, "$s450"}, /* 1169 */
  { 451, "$vsfr195"}, /* 1170 */
  { 451, "$s451"}, /* 1171 */
  { 452, "$vsfr196"}, /* 1172 */
  { 452, "$s452"}, /* 1173 */
  { 453, "$vsfr197"}, /* 1174 */
  { 453, "$s453"}, /* 1175 */
  { 454, "$vsfr198"}, /* 1176 */
  { 454, "$s454"}, /* 1177 */
  { 455, "$vsfr199"}, /* 1178 */
  { 455, "$s455"}, /* 1179 */
  { 456, "$vsfr200"}, /* 1180 */
  { 456, "$s456"}, /* 1181 */
  { 457, "$vsfr201"}, /* 1182 */
  { 457, "$s457"}, /* 1183 */
  { 458, "$vsfr202"}, /* 1184 */
  { 458, "$s458"}, /* 1185 */
  { 459, "$vsfr203"}, /* 1186 */
  { 459, "$s459"}, /* 1187 */
  { 460, "$vsfr204"}, /* 1188 */
  { 460, "$s460"}, /* 1189 */
  { 461, "$vsfr205"}, /* 1190 */
  { 461, "$s461"}, /* 1191 */
  { 462, "$vsfr206"}, /* 1192 */
  { 462, "$s462"}, /* 1193 */
  { 463, "$vsfr207"}, /* 1194 */
  { 463, "$s463"}, /* 1195 */
  { 464, "$vsfr208"}, /* 1196 */
  { 464, "$s464"}, /* 1197 */
  { 465, "$vsfr209"}, /* 1198 */
  { 465, "$s465"}, /* 1199 */
  { 466, "$vsfr210"}, /* 1200 */
  { 466, "$s466"}, /* 1201 */
  { 467, "$vsfr211"}, /* 1202 */
  { 467, "$s467"}, /* 1203 */
  { 468, "$vsfr212"}, /* 1204 */
  { 468, "$s468"}, /* 1205 */
  { 469, "$vsfr213"}, /* 1206 */
  { 469, "$s469"}, /* 1207 */
  { 470, "$vsfr214"}, /* 1208 */
  { 470, "$s470"}, /* 1209 */
  { 471, "$vsfr215"}, /* 1210 */
  { 471, "$s471"}, /* 1211 */
  { 472, "$vsfr216"}, /* 1212 */
  { 472, "$s472"}, /* 1213 */
  { 473, "$vsfr217"}, /* 1214 */
  { 473, "$s473"}, /* 1215 */
  { 474, "$vsfr218"}, /* 1216 */
  { 474, "$s474"}, /* 1217 */
  { 475, "$vsfr219"}, /* 1218 */
  { 475, "$s475"}, /* 1219 */
  { 476, "$vsfr220"}, /* 1220 */
  { 476, "$s476"}, /* 1221 */
  { 477, "$vsfr221"}, /* 1222 */
  { 477, "$s477"}, /* 1223 */
  { 478, "$vsfr222"}, /* 1224 */
  { 478, "$s478"}, /* 1225 */
  { 479, "$vsfr223"}, /* 1226 */
  { 479, "$s479"}, /* 1227 */
  { 480, "$vsfr224"}, /* 1228 */
  { 480, "$s480"}, /* 1229 */
  { 481, "$vsfr225"}, /* 1230 */
  { 481, "$s481"}, /* 1231 */
  { 482, "$vsfr226"}, /* 1232 */
  { 482, "$s482"}, /* 1233 */
  { 483, "$vsfr227"}, /* 1234 */
  { 483, "$s483"}, /* 1235 */
  { 484, "$vsfr228"}, /* 1236 */
  { 484, "$s484"}, /* 1237 */
  { 485, "$vsfr229"}, /* 1238 */
  { 485, "$s485"}, /* 1239 */
  { 486, "$vsfr230"}, /* 1240 */
  { 486, "$s486"}, /* 1241 */
  { 487, "$vsfr231"}, /* 1242 */
  { 487, "$s487"}, /* 1243 */
  { 488, "$vsfr232"}, /* 1244 */
  { 488, "$s488"}, /* 1245 */
  { 489, "$vsfr233"}, /* 1246 */
  { 489, "$s489"}, /* 1247 */
  { 490, "$vsfr234"}, /* 1248 */
  { 490, "$s490"}, /* 1249 */
  { 491, "$vsfr235"}, /* 1250 */
  { 491, "$s491"}, /* 1251 */
  { 492, "$vsfr236"}, /* 1252 */
  { 492, "$s492"}, /* 1253 */
  { 493, "$vsfr237"}, /* 1254 */
  { 493, "$s493"}, /* 1255 */
  { 494, "$vsfr238"}, /* 1256 */
  { 494, "$s494"}, /* 1257 */
  { 495, "$vsfr239"}, /* 1258 */
  { 495, "$s495"}, /* 1259 */
  { 496, "$vsfr240"}, /* 1260 */
  { 496, "$s496"}, /* 1261 */
  { 497, "$vsfr241"}, /* 1262 */
  { 497, "$s497"}, /* 1263 */
  { 498, "$vsfr242"}, /* 1264 */
  { 498, "$s498"}, /* 1265 */
  { 499, "$vsfr243"}, /* 1266 */
  { 499, "$s499"}, /* 1267 */
  { 500, "$vsfr244"}, /* 1268 */
  { 500, "$s500"}, /* 1269 */
  { 501, "$vsfr245"}, /* 1270 */
  { 501, "$s501"}, /* 1271 */
  { 502, "$vsfr246"}, /* 1272 */
  { 502, "$s502"}, /* 1273 */
  { 503, "$vsfr247"}, /* 1274 */
  { 503, "$s503"}, /* 1275 */
  { 504, "$vsfr248"}, /* 1276 */
  { 504, "$s504"}, /* 1277 */
  { 505, "$vsfr249"}, /* 1278 */
  { 505, "$s505"}, /* 1279 */
  { 506, "$vsfr250"}, /* 1280 */
  { 506, "$s506"}, /* 1281 */
  { 507, "$vsfr251"}, /* 1282 */
  { 507, "$s507"}, /* 1283 */
  { 508, "$vsfr252"}, /* 1284 */
  { 508, "$s508"}, /* 1285 */
  { 509, "$vsfr253"}, /* 1286 */
  { 509, "$s509"}, /* 1287 */
  { 510, "$vsfr254"}, /* 1288 */
  { 510, "$s510"}, /* 1289 */
  { 511, "$vsfr255"}, /* 1290 */
  { 511, "$s511"}, /* 1291 */
  { 0, "$a0_lo"}, /* 1292 */
  { 0, "$a0.lo"}, /* 1293 */
  { 1, "$a0_hi"}, /* 1294 */
  { 1, "$a0.hi"}, /* 1295 */
  { 2, "$a1_lo"}, /* 1296 */
  { 2, "$a1.lo"}, /* 1297 */
  { 3, "$a1_hi"}, /* 1298 */
  { 3, "$a1.hi"}, /* 1299 */
  { 4, "$a2_lo"}, /* 1300 */
  { 4, "$a2.lo"}, /* 1301 */
  { 5, "$a2_hi"}, /* 1302 */
  { 5, "$a2.hi"}, /* 1303 */
  { 6, "$a3_lo"}, /* 1304 */
  { 6, "$a3.lo"}, /* 1305 */
  { 7, "$a3_hi"}, /* 1306 */
  { 7, "$a3.hi"}, /* 1307 */
  { 8, "$a4_lo"}, /* 1308 */
  { 8, "$a4.lo"}, /* 1309 */
  { 9, "$a4_hi"}, /* 1310 */
  { 9, "$a4.hi"}, /* 1311 */
  { 10, "$a5_lo"}, /* 1312 */
  { 10, "$a5.lo"}, /* 1313 */
  { 11, "$a5_hi"}, /* 1314 */
  { 11, "$a5.hi"}, /* 1315 */
  { 12, "$a6_lo"}, /* 1316 */
  { 12, "$a6.lo"}, /* 1317 */
  { 13, "$a6_hi"}, /* 1318 */
  { 13, "$a6.hi"}, /* 1319 */
  { 14, "$a7_lo"}, /* 1320 */
  { 14, "$a7.lo"}, /* 1321 */
  { 15, "$a7_hi"}, /* 1322 */
  { 15, "$a7.hi"}, /* 1323 */
  { 16, "$a8_lo"}, /* 1324 */
  { 16, "$a8.lo"}, /* 1325 */
  { 17, "$a8_hi"}, /* 1326 */
  { 17, "$a8.hi"}, /* 1327 */
  { 18, "$a9_lo"}, /* 1328 */
  { 18, "$a9.lo"}, /* 1329 */
  { 19, "$a9_hi"}, /* 1330 */
  { 19, "$a9.hi"}, /* 1331 */
  { 20, "$a10_lo"}, /* 1332 */
  { 20, "$a10.lo"}, /* 1333 */
  { 21, "$a10_hi"}, /* 1334 */
  { 21, "$a10.hi"}, /* 1335 */
  { 22, "$a11_lo"}, /* 1336 */
  { 22, "$a11.lo"}, /* 1337 */
  { 23, "$a11_hi"}, /* 1338 */
  { 23, "$a11.hi"}, /* 1339 */
  { 24, "$a12_lo"}, /* 1340 */
  { 24, "$a12.lo"}, /* 1341 */
  { 25, "$a12_hi"}, /* 1342 */
  { 25, "$a12.hi"}, /* 1343 */
  { 26, "$a13_lo"}, /* 1344 */
  { 26, "$a13.lo"}, /* 1345 */
  { 27, "$a13_hi"}, /* 1346 */
  { 27, "$a13.hi"}, /* 1347 */
  { 28, "$a14_lo"}, /* 1348 */
  { 28, "$a14.lo"}, /* 1349 */
  { 29, "$a14_hi"}, /* 1350 */
  { 29, "$a14.hi"}, /* 1351 */
  { 30, "$a15_lo"}, /* 1352 */
  { 30, "$a15.lo"}, /* 1353 */
  { 31, "$a15_hi"}, /* 1354 */
  { 31, "$a15.hi"}, /* 1355 */
  { 32, "$a16_lo"}, /* 1356 */
  { 32, "$a16.lo"}, /* 1357 */
  { 33, "$a16_hi"}, /* 1358 */
  { 33, "$a16.hi"}, /* 1359 */
  { 34, "$a17_lo"}, /* 1360 */
  { 34, "$a17.lo"}, /* 1361 */
  { 35, "$a17_hi"}, /* 1362 */
  { 35, "$a17.hi"}, /* 1363 */
  { 36, "$a18_lo"}, /* 1364 */
  { 36, "$a18.lo"}, /* 1365 */
  { 37, "$a18_hi"}, /* 1366 */
  { 37, "$a18.hi"}, /* 1367 */
  { 38, "$a19_lo"}, /* 1368 */
  { 38, "$a19.lo"}, /* 1369 */
  { 39, "$a19_hi"}, /* 1370 */
  { 39, "$a19.hi"}, /* 1371 */
  { 40, "$a20_lo"}, /* 1372 */
  { 40, "$a20.lo"}, /* 1373 */
  { 41, "$a20_hi"}, /* 1374 */
  { 41, "$a20.hi"}, /* 1375 */
  { 42, "$a21_lo"}, /* 1376 */
  { 42, "$a21.lo"}, /* 1377 */
  { 43, "$a21_hi"}, /* 1378 */
  { 43, "$a21.hi"}, /* 1379 */
  { 44, "$a22_lo"}, /* 1380 */
  { 44, "$a22.lo"}, /* 1381 */
  { 45, "$a22_hi"}, /* 1382 */
  { 45, "$a22.hi"}, /* 1383 */
  { 46, "$a23_lo"}, /* 1384 */
  { 46, "$a23.lo"}, /* 1385 */
  { 47, "$a23_hi"}, /* 1386 */
  { 47, "$a23.hi"}, /* 1387 */
  { 48, "$a24_lo"}, /* 1388 */
  { 48, "$a24.lo"}, /* 1389 */
  { 49, "$a24_hi"}, /* 1390 */
  { 49, "$a24.hi"}, /* 1391 */
  { 50, "$a25_lo"}, /* 1392 */
  { 50, "$a25.lo"}, /* 1393 */
  { 51, "$a25_hi"}, /* 1394 */
  { 51, "$a25.hi"}, /* 1395 */
  { 52, "$a26_lo"}, /* 1396 */
  { 52, "$a26.lo"}, /* 1397 */
  { 53, "$a26_hi"}, /* 1398 */
  { 53, "$a26.hi"}, /* 1399 */
  { 54, "$a27_lo"}, /* 1400 */
  { 54, "$a27.lo"}, /* 1401 */
  { 55, "$a27_hi"}, /* 1402 */
  { 55, "$a27.hi"}, /* 1403 */
  { 56, "$a28_lo"}, /* 1404 */
  { 56, "$a28.lo"}, /* 1405 */
  { 57, "$a28_hi"}, /* 1406 */
  { 57, "$a28.hi"}, /* 1407 */
  { 58, "$a29_lo"}, /* 1408 */
  { 58, "$a29.lo"}, /* 1409 */
  { 59, "$a29_hi"}, /* 1410 */
  { 59, "$a29.hi"}, /* 1411 */
  { 60, "$a30_lo"}, /* 1412 */
  { 60, "$a30.lo"}, /* 1413 */
  { 61, "$a30_hi"}, /* 1414 */
  { 61, "$a30.hi"}, /* 1415 */
  { 62, "$a31_lo"}, /* 1416 */
  { 62, "$a31.lo"}, /* 1417 */
  { 63, "$a31_hi"}, /* 1418 */
  { 63, "$a31.hi"}, /* 1419 */
  { 64, "$a32_lo"}, /* 1420 */
  { 64, "$a32.lo"}, /* 1421 */
  { 65, "$a32_hi"}, /* 1422 */
  { 65, "$a32.hi"}, /* 1423 */
  { 66, "$a33_lo"}, /* 1424 */
  { 66, "$a33.lo"}, /* 1425 */
  { 67, "$a33_hi"}, /* 1426 */
  { 67, "$a33.hi"}, /* 1427 */
  { 68, "$a34_lo"}, /* 1428 */
  { 68, "$a34.lo"}, /* 1429 */
  { 69, "$a34_hi"}, /* 1430 */
  { 69, "$a34.hi"}, /* 1431 */
  { 70, "$a35_lo"}, /* 1432 */
  { 70, "$a35.lo"}, /* 1433 */
  { 71, "$a35_hi"}, /* 1434 */
  { 71, "$a35.hi"}, /* 1435 */
  { 72, "$a36_lo"}, /* 1436 */
  { 72, "$a36.lo"}, /* 1437 */
  { 73, "$a36_hi"}, /* 1438 */
  { 73, "$a36.hi"}, /* 1439 */
  { 74, "$a37_lo"}, /* 1440 */
  { 74, "$a37.lo"}, /* 1441 */
  { 75, "$a37_hi"}, /* 1442 */
  { 75, "$a37.hi"}, /* 1443 */
  { 76, "$a38_lo"}, /* 1444 */
  { 76, "$a38.lo"}, /* 1445 */
  { 77, "$a38_hi"}, /* 1446 */
  { 77, "$a38.hi"}, /* 1447 */
  { 78, "$a39_lo"}, /* 1448 */
  { 78, "$a39.lo"}, /* 1449 */
  { 79, "$a39_hi"}, /* 1450 */
  { 79, "$a39.hi"}, /* 1451 */
  { 80, "$a40_lo"}, /* 1452 */
  { 80, "$a40.lo"}, /* 1453 */
  { 81, "$a40_hi"}, /* 1454 */
  { 81, "$a40.hi"}, /* 1455 */
  { 82, "$a41_lo"}, /* 1456 */
  { 82, "$a41.lo"}, /* 1457 */
  { 83, "$a41_hi"}, /* 1458 */
  { 83, "$a41.hi"}, /* 1459 */
  { 84, "$a42_lo"}, /* 1460 */
  { 84, "$a42.lo"}, /* 1461 */
  { 85, "$a42_hi"}, /* 1462 */
  { 85, "$a42.hi"}, /* 1463 */
  { 86, "$a43_lo"}, /* 1464 */
  { 86, "$a43.lo"}, /* 1465 */
  { 87, "$a43_hi"}, /* 1466 */
  { 87, "$a43.hi"}, /* 1467 */
  { 88, "$a44_lo"}, /* 1468 */
  { 88, "$a44.lo"}, /* 1469 */
  { 89, "$a44_hi"}, /* 1470 */
  { 89, "$a44.hi"}, /* 1471 */
  { 90, "$a45_lo"}, /* 1472 */
  { 90, "$a45.lo"}, /* 1473 */
  { 91, "$a45_hi"}, /* 1474 */
  { 91, "$a45.hi"}, /* 1475 */
  { 92, "$a46_lo"}, /* 1476 */
  { 92, "$a46.lo"}, /* 1477 */
  { 93, "$a46_hi"}, /* 1478 */
  { 93, "$a46.hi"}, /* 1479 */
  { 94, "$a47_lo"}, /* 1480 */
  { 94, "$a47.lo"}, /* 1481 */
  { 95, "$a47_hi"}, /* 1482 */
  { 95, "$a47.hi"}, /* 1483 */
  { 96, "$a48_lo"}, /* 1484 */
  { 96, "$a48.lo"}, /* 1485 */
  { 97, "$a48_hi"}, /* 1486 */
  { 97, "$a48.hi"}, /* 1487 */
  { 98, "$a49_lo"}, /* 1488 */
  { 98, "$a49.lo"}, /* 1489 */
  { 99, "$a49_hi"}, /* 1490 */
  { 99, "$a49.hi"}, /* 1491 */
  { 100, "$a50_lo"}, /* 1492 */
  { 100, "$a50.lo"}, /* 1493 */
  { 101, "$a50_hi"}, /* 1494 */
  { 101, "$a50.hi"}, /* 1495 */
  { 102, "$a51_lo"}, /* 1496 */
  { 102, "$a51.lo"}, /* 1497 */
  { 103, "$a51_hi"}, /* 1498 */
  { 103, "$a51.hi"}, /* 1499 */
  { 104, "$a52_lo"}, /* 1500 */
  { 104, "$a52.lo"}, /* 1501 */
  { 105, "$a52_hi"}, /* 1502 */
  { 105, "$a52.hi"}, /* 1503 */
  { 106, "$a53_lo"}, /* 1504 */
  { 106, "$a53.lo"}, /* 1505 */
  { 107, "$a53_hi"}, /* 1506 */
  { 107, "$a53.hi"}, /* 1507 */
  { 108, "$a54_lo"}, /* 1508 */
  { 108, "$a54.lo"}, /* 1509 */
  { 109, "$a54_hi"}, /* 1510 */
  { 109, "$a54.hi"}, /* 1511 */
  { 110, "$a55_lo"}, /* 1512 */
  { 110, "$a55.lo"}, /* 1513 */
  { 111, "$a55_hi"}, /* 1514 */
  { 111, "$a55.hi"}, /* 1515 */
  { 112, "$a56_lo"}, /* 1516 */
  { 112, "$a56.lo"}, /* 1517 */
  { 113, "$a56_hi"}, /* 1518 */
  { 113, "$a56.hi"}, /* 1519 */
  { 114, "$a57_lo"}, /* 1520 */
  { 114, "$a57.lo"}, /* 1521 */
  { 115, "$a57_hi"}, /* 1522 */
  { 115, "$a57.hi"}, /* 1523 */
  { 116, "$a58_lo"}, /* 1524 */
  { 116, "$a58.lo"}, /* 1525 */
  { 117, "$a58_hi"}, /* 1526 */
  { 117, "$a58.hi"}, /* 1527 */
  { 118, "$a59_lo"}, /* 1528 */
  { 118, "$a59.lo"}, /* 1529 */
  { 119, "$a59_hi"}, /* 1530 */
  { 119, "$a59.hi"}, /* 1531 */
  { 120, "$a60_lo"}, /* 1532 */
  { 120, "$a60.lo"}, /* 1533 */
  { 121, "$a60_hi"}, /* 1534 */
  { 121, "$a60.hi"}, /* 1535 */
  { 122, "$a61_lo"}, /* 1536 */
  { 122, "$a61.lo"}, /* 1537 */
  { 123, "$a61_hi"}, /* 1538 */
  { 123, "$a61.hi"}, /* 1539 */
  { 124, "$a62_lo"}, /* 1540 */
  { 124, "$a62.lo"}, /* 1541 */
  { 125, "$a62_hi"}, /* 1542 */
  { 125, "$a62.hi"}, /* 1543 */
  { 126, "$a63_lo"}, /* 1544 */
  { 126, "$a63.lo"}, /* 1545 */
  { 127, "$a63_hi"}, /* 1546 */
  { 127, "$a63.hi"}, /* 1547 */
  { 0, "$a0_x"}, /* 1548 */
  { 0, "$a0.x"}, /* 1549 */
  { 1, "$a0_y"}, /* 1550 */
  { 1, "$a0.y"}, /* 1551 */
  { 2, "$a0_z"}, /* 1552 */
  { 2, "$a0.z"}, /* 1553 */
  { 3, "$a0_t"}, /* 1554 */
  { 3, "$a0.t"}, /* 1555 */
  { 4, "$a1_x"}, /* 1556 */
  { 4, "$a1.x"}, /* 1557 */
  { 5, "$a1_y"}, /* 1558 */
  { 5, "$a1.y"}, /* 1559 */
  { 6, "$a1_z"}, /* 1560 */
  { 6, "$a1.z"}, /* 1561 */
  { 7, "$a1_t"}, /* 1562 */
  { 7, "$a1.t"}, /* 1563 */
  { 8, "$a2_x"}, /* 1564 */
  { 8, "$a2.x"}, /* 1565 */
  { 9, "$a2_y"}, /* 1566 */
  { 9, "$a2.y"}, /* 1567 */
  { 10, "$a2_z"}, /* 1568 */
  { 10, "$a2.z"}, /* 1569 */
  { 11, "$a2_t"}, /* 1570 */
  { 11, "$a2.t"}, /* 1571 */
  { 12, "$a3_x"}, /* 1572 */
  { 12, "$a3.x"}, /* 1573 */
  { 13, "$a3_y"}, /* 1574 */
  { 13, "$a3.y"}, /* 1575 */
  { 14, "$a3_z"}, /* 1576 */
  { 14, "$a3.z"}, /* 1577 */
  { 15, "$a3_t"}, /* 1578 */
  { 15, "$a3.t"}, /* 1579 */
  { 16, "$a4_x"}, /* 1580 */
  { 16, "$a4.x"}, /* 1581 */
  { 17, "$a4_y"}, /* 1582 */
  { 17, "$a4.y"}, /* 1583 */
  { 18, "$a4_z"}, /* 1584 */
  { 18, "$a4.z"}, /* 1585 */
  { 19, "$a4_t"}, /* 1586 */
  { 19, "$a4.t"}, /* 1587 */
  { 20, "$a5_x"}, /* 1588 */
  { 20, "$a5.x"}, /* 1589 */
  { 21, "$a5_y"}, /* 1590 */
  { 21, "$a5.y"}, /* 1591 */
  { 22, "$a5_z"}, /* 1592 */
  { 22, "$a5.z"}, /* 1593 */
  { 23, "$a5_t"}, /* 1594 */
  { 23, "$a5.t"}, /* 1595 */
  { 24, "$a6_x"}, /* 1596 */
  { 24, "$a6.x"}, /* 1597 */
  { 25, "$a6_y"}, /* 1598 */
  { 25, "$a6.y"}, /* 1599 */
  { 26, "$a6_z"}, /* 1600 */
  { 26, "$a6.z"}, /* 1601 */
  { 27, "$a6_t"}, /* 1602 */
  { 27, "$a6.t"}, /* 1603 */
  { 28, "$a7_x"}, /* 1604 */
  { 28, "$a7.x"}, /* 1605 */
  { 29, "$a7_y"}, /* 1606 */
  { 29, "$a7.y"}, /* 1607 */
  { 30, "$a7_z"}, /* 1608 */
  { 30, "$a7.z"}, /* 1609 */
  { 31, "$a7_t"}, /* 1610 */
  { 31, "$a7.t"}, /* 1611 */
  { 32, "$a8_x"}, /* 1612 */
  { 32, "$a8.x"}, /* 1613 */
  { 33, "$a8_y"}, /* 1614 */
  { 33, "$a8.y"}, /* 1615 */
  { 34, "$a8_z"}, /* 1616 */
  { 34, "$a8.z"}, /* 1617 */
  { 35, "$a8_t"}, /* 1618 */
  { 35, "$a8.t"}, /* 1619 */
  { 36, "$a9_x"}, /* 1620 */
  { 36, "$a9.x"}, /* 1621 */
  { 37, "$a9_y"}, /* 1622 */
  { 37, "$a9.y"}, /* 1623 */
  { 38, "$a9_z"}, /* 1624 */
  { 38, "$a9.z"}, /* 1625 */
  { 39, "$a9_t"}, /* 1626 */
  { 39, "$a9.t"}, /* 1627 */
  { 40, "$a10_x"}, /* 1628 */
  { 40, "$a10.x"}, /* 1629 */
  { 41, "$a10_y"}, /* 1630 */
  { 41, "$a10.y"}, /* 1631 */
  { 42, "$a10_z"}, /* 1632 */
  { 42, "$a10.z"}, /* 1633 */
  { 43, "$a10_t"}, /* 1634 */
  { 43, "$a10.t"}, /* 1635 */
  { 44, "$a11_x"}, /* 1636 */
  { 44, "$a11.x"}, /* 1637 */
  { 45, "$a11_y"}, /* 1638 */
  { 45, "$a11.y"}, /* 1639 */
  { 46, "$a11_z"}, /* 1640 */
  { 46, "$a11.z"}, /* 1641 */
  { 47, "$a11_t"}, /* 1642 */
  { 47, "$a11.t"}, /* 1643 */
  { 48, "$a12_x"}, /* 1644 */
  { 48, "$a12.x"}, /* 1645 */
  { 49, "$a12_y"}, /* 1646 */
  { 49, "$a12.y"}, /* 1647 */
  { 50, "$a12_z"}, /* 1648 */
  { 50, "$a12.z"}, /* 1649 */
  { 51, "$a12_t"}, /* 1650 */
  { 51, "$a12.t"}, /* 1651 */
  { 52, "$a13_x"}, /* 1652 */
  { 52, "$a13.x"}, /* 1653 */
  { 53, "$a13_y"}, /* 1654 */
  { 53, "$a13.y"}, /* 1655 */
  { 54, "$a13_z"}, /* 1656 */
  { 54, "$a13.z"}, /* 1657 */
  { 55, "$a13_t"}, /* 1658 */
  { 55, "$a13.t"}, /* 1659 */
  { 56, "$a14_x"}, /* 1660 */
  { 56, "$a14.x"}, /* 1661 */
  { 57, "$a14_y"}, /* 1662 */
  { 57, "$a14.y"}, /* 1663 */
  { 58, "$a14_z"}, /* 1664 */
  { 58, "$a14.z"}, /* 1665 */
  { 59, "$a14_t"}, /* 1666 */
  { 59, "$a14.t"}, /* 1667 */
  { 60, "$a15_x"}, /* 1668 */
  { 60, "$a15.x"}, /* 1669 */
  { 61, "$a15_y"}, /* 1670 */
  { 61, "$a15.y"}, /* 1671 */
  { 62, "$a15_z"}, /* 1672 */
  { 62, "$a15.z"}, /* 1673 */
  { 63, "$a15_t"}, /* 1674 */
  { 63, "$a15.t"}, /* 1675 */
  { 64, "$a16_x"}, /* 1676 */
  { 64, "$a16.x"}, /* 1677 */
  { 65, "$a16_y"}, /* 1678 */
  { 65, "$a16.y"}, /* 1679 */
  { 66, "$a16_z"}, /* 1680 */
  { 66, "$a16.z"}, /* 1681 */
  { 67, "$a16_t"}, /* 1682 */
  { 67, "$a16.t"}, /* 1683 */
  { 68, "$a17_x"}, /* 1684 */
  { 68, "$a17.x"}, /* 1685 */
  { 69, "$a17_y"}, /* 1686 */
  { 69, "$a17.y"}, /* 1687 */
  { 70, "$a17_z"}, /* 1688 */
  { 70, "$a17.z"}, /* 1689 */
  { 71, "$a17_t"}, /* 1690 */
  { 71, "$a17.t"}, /* 1691 */
  { 72, "$a18_x"}, /* 1692 */
  { 72, "$a18.x"}, /* 1693 */
  { 73, "$a18_y"}, /* 1694 */
  { 73, "$a18.y"}, /* 1695 */
  { 74, "$a18_z"}, /* 1696 */
  { 74, "$a18.z"}, /* 1697 */
  { 75, "$a18_t"}, /* 1698 */
  { 75, "$a18.t"}, /* 1699 */
  { 76, "$a19_x"}, /* 1700 */
  { 76, "$a19.x"}, /* 1701 */
  { 77, "$a19_y"}, /* 1702 */
  { 77, "$a19.y"}, /* 1703 */
  { 78, "$a19_z"}, /* 1704 */
  { 78, "$a19.z"}, /* 1705 */
  { 79, "$a19_t"}, /* 1706 */
  { 79, "$a19.t"}, /* 1707 */
  { 80, "$a20_x"}, /* 1708 */
  { 80, "$a20.x"}, /* 1709 */
  { 81, "$a20_y"}, /* 1710 */
  { 81, "$a20.y"}, /* 1711 */
  { 82, "$a20_z"}, /* 1712 */
  { 82, "$a20.z"}, /* 1713 */
  { 83, "$a20_t"}, /* 1714 */
  { 83, "$a20.t"}, /* 1715 */
  { 84, "$a21_x"}, /* 1716 */
  { 84, "$a21.x"}, /* 1717 */
  { 85, "$a21_y"}, /* 1718 */
  { 85, "$a21.y"}, /* 1719 */
  { 86, "$a21_z"}, /* 1720 */
  { 86, "$a21.z"}, /* 1721 */
  { 87, "$a21_t"}, /* 1722 */
  { 87, "$a21.t"}, /* 1723 */
  { 88, "$a22_x"}, /* 1724 */
  { 88, "$a22.x"}, /* 1725 */
  { 89, "$a22_y"}, /* 1726 */
  { 89, "$a22.y"}, /* 1727 */
  { 90, "$a22_z"}, /* 1728 */
  { 90, "$a22.z"}, /* 1729 */
  { 91, "$a22_t"}, /* 1730 */
  { 91, "$a22.t"}, /* 1731 */
  { 92, "$a23_x"}, /* 1732 */
  { 92, "$a23.x"}, /* 1733 */
  { 93, "$a23_y"}, /* 1734 */
  { 93, "$a23.y"}, /* 1735 */
  { 94, "$a23_z"}, /* 1736 */
  { 94, "$a23.z"}, /* 1737 */
  { 95, "$a23_t"}, /* 1738 */
  { 95, "$a23.t"}, /* 1739 */
  { 96, "$a24_x"}, /* 1740 */
  { 96, "$a24.x"}, /* 1741 */
  { 97, "$a24_y"}, /* 1742 */
  { 97, "$a24.y"}, /* 1743 */
  { 98, "$a24_z"}, /* 1744 */
  { 98, "$a24.z"}, /* 1745 */
  { 99, "$a24_t"}, /* 1746 */
  { 99, "$a24.t"}, /* 1747 */
  { 100, "$a25_x"}, /* 1748 */
  { 100, "$a25.x"}, /* 1749 */
  { 101, "$a25_y"}, /* 1750 */
  { 101, "$a25.y"}, /* 1751 */
  { 102, "$a25_z"}, /* 1752 */
  { 102, "$a25.z"}, /* 1753 */
  { 103, "$a25_t"}, /* 1754 */
  { 103, "$a25.t"}, /* 1755 */
  { 104, "$a26_x"}, /* 1756 */
  { 104, "$a26.x"}, /* 1757 */
  { 105, "$a26_y"}, /* 1758 */
  { 105, "$a26.y"}, /* 1759 */
  { 106, "$a26_z"}, /* 1760 */
  { 106, "$a26.z"}, /* 1761 */
  { 107, "$a26_t"}, /* 1762 */
  { 107, "$a26.t"}, /* 1763 */
  { 108, "$a27_x"}, /* 1764 */
  { 108, "$a27.x"}, /* 1765 */
  { 109, "$a27_y"}, /* 1766 */
  { 109, "$a27.y"}, /* 1767 */
  { 110, "$a27_z"}, /* 1768 */
  { 110, "$a27.z"}, /* 1769 */
  { 111, "$a27_t"}, /* 1770 */
  { 111, "$a27.t"}, /* 1771 */
  { 112, "$a28_x"}, /* 1772 */
  { 112, "$a28.x"}, /* 1773 */
  { 113, "$a28_y"}, /* 1774 */
  { 113, "$a28.y"}, /* 1775 */
  { 114, "$a28_z"}, /* 1776 */
  { 114, "$a28.z"}, /* 1777 */
  { 115, "$a28_t"}, /* 1778 */
  { 115, "$a28.t"}, /* 1779 */
  { 116, "$a29_x"}, /* 1780 */
  { 116, "$a29.x"}, /* 1781 */
  { 117, "$a29_y"}, /* 1782 */
  { 117, "$a29.y"}, /* 1783 */
  { 118, "$a29_z"}, /* 1784 */
  { 118, "$a29.z"}, /* 1785 */
  { 119, "$a29_t"}, /* 1786 */
  { 119, "$a29.t"}, /* 1787 */
  { 120, "$a30_x"}, /* 1788 */
  { 120, "$a30.x"}, /* 1789 */
  { 121, "$a30_y"}, /* 1790 */
  { 121, "$a30.y"}, /* 1791 */
  { 122, "$a30_z"}, /* 1792 */
  { 122, "$a30.z"}, /* 1793 */
  { 123, "$a30_t"}, /* 1794 */
  { 123, "$a30.t"}, /* 1795 */
  { 124, "$a31_x"}, /* 1796 */
  { 124, "$a31.x"}, /* 1797 */
  { 125, "$a31_y"}, /* 1798 */
  { 125, "$a31.y"}, /* 1799 */
  { 126, "$a31_z"}, /* 1800 */
  { 126, "$a31.z"}, /* 1801 */
  { 127, "$a31_t"}, /* 1802 */
  { 127, "$a31.t"}, /* 1803 */
  { 128, "$a32_x"}, /* 1804 */
  { 128, "$a32.x"}, /* 1805 */
  { 129, "$a32_y"}, /* 1806 */
  { 129, "$a32.y"}, /* 1807 */
  { 130, "$a32_z"}, /* 1808 */
  { 130, "$a32.z"}, /* 1809 */
  { 131, "$a32_t"}, /* 1810 */
  { 131, "$a32.t"}, /* 1811 */
  { 132, "$a33_x"}, /* 1812 */
  { 132, "$a33.x"}, /* 1813 */
  { 133, "$a33_y"}, /* 1814 */
  { 133, "$a33.y"}, /* 1815 */
  { 134, "$a33_z"}, /* 1816 */
  { 134, "$a33.z"}, /* 1817 */
  { 135, "$a33_t"}, /* 1818 */
  { 135, "$a33.t"}, /* 1819 */
  { 136, "$a34_x"}, /* 1820 */
  { 136, "$a34.x"}, /* 1821 */
  { 137, "$a34_y"}, /* 1822 */
  { 137, "$a34.y"}, /* 1823 */
  { 138, "$a34_z"}, /* 1824 */
  { 138, "$a34.z"}, /* 1825 */
  { 139, "$a34_t"}, /* 1826 */
  { 139, "$a34.t"}, /* 1827 */
  { 140, "$a35_x"}, /* 1828 */
  { 140, "$a35.x"}, /* 1829 */
  { 141, "$a35_y"}, /* 1830 */
  { 141, "$a35.y"}, /* 1831 */
  { 142, "$a35_z"}, /* 1832 */
  { 142, "$a35.z"}, /* 1833 */
  { 143, "$a35_t"}, /* 1834 */
  { 143, "$a35.t"}, /* 1835 */
  { 144, "$a36_x"}, /* 1836 */
  { 144, "$a36.x"}, /* 1837 */
  { 145, "$a36_y"}, /* 1838 */
  { 145, "$a36.y"}, /* 1839 */
  { 146, "$a36_z"}, /* 1840 */
  { 146, "$a36.z"}, /* 1841 */
  { 147, "$a36_t"}, /* 1842 */
  { 147, "$a36.t"}, /* 1843 */
  { 148, "$a37_x"}, /* 1844 */
  { 148, "$a37.x"}, /* 1845 */
  { 149, "$a37_y"}, /* 1846 */
  { 149, "$a37.y"}, /* 1847 */
  { 150, "$a37_z"}, /* 1848 */
  { 150, "$a37.z"}, /* 1849 */
  { 151, "$a37_t"}, /* 1850 */
  { 151, "$a37.t"}, /* 1851 */
  { 152, "$a38_x"}, /* 1852 */
  { 152, "$a38.x"}, /* 1853 */
  { 153, "$a38_y"}, /* 1854 */
  { 153, "$a38.y"}, /* 1855 */
  { 154, "$a38_z"}, /* 1856 */
  { 154, "$a38.z"}, /* 1857 */
  { 155, "$a38_t"}, /* 1858 */
  { 155, "$a38.t"}, /* 1859 */
  { 156, "$a39_x"}, /* 1860 */
  { 156, "$a39.x"}, /* 1861 */
  { 157, "$a39_y"}, /* 1862 */
  { 157, "$a39.y"}, /* 1863 */
  { 158, "$a39_z"}, /* 1864 */
  { 158, "$a39.z"}, /* 1865 */
  { 159, "$a39_t"}, /* 1866 */
  { 159, "$a39.t"}, /* 1867 */
  { 160, "$a40_x"}, /* 1868 */
  { 160, "$a40.x"}, /* 1869 */
  { 161, "$a40_y"}, /* 1870 */
  { 161, "$a40.y"}, /* 1871 */
  { 162, "$a40_z"}, /* 1872 */
  { 162, "$a40.z"}, /* 1873 */
  { 163, "$a40_t"}, /* 1874 */
  { 163, "$a40.t"}, /* 1875 */
  { 164, "$a41_x"}, /* 1876 */
  { 164, "$a41.x"}, /* 1877 */
  { 165, "$a41_y"}, /* 1878 */
  { 165, "$a41.y"}, /* 1879 */
  { 166, "$a41_z"}, /* 1880 */
  { 166, "$a41.z"}, /* 1881 */
  { 167, "$a41_t"}, /* 1882 */
  { 167, "$a41.t"}, /* 1883 */
  { 168, "$a42_x"}, /* 1884 */
  { 168, "$a42.x"}, /* 1885 */
  { 169, "$a42_y"}, /* 1886 */
  { 169, "$a42.y"}, /* 1887 */
  { 170, "$a42_z"}, /* 1888 */
  { 170, "$a42.z"}, /* 1889 */
  { 171, "$a42_t"}, /* 1890 */
  { 171, "$a42.t"}, /* 1891 */
  { 172, "$a43_x"}, /* 1892 */
  { 172, "$a43.x"}, /* 1893 */
  { 173, "$a43_y"}, /* 1894 */
  { 173, "$a43.y"}, /* 1895 */
  { 174, "$a43_z"}, /* 1896 */
  { 174, "$a43.z"}, /* 1897 */
  { 175, "$a43_t"}, /* 1898 */
  { 175, "$a43.t"}, /* 1899 */
  { 176, "$a44_x"}, /* 1900 */
  { 176, "$a44.x"}, /* 1901 */
  { 177, "$a44_y"}, /* 1902 */
  { 177, "$a44.y"}, /* 1903 */
  { 178, "$a44_z"}, /* 1904 */
  { 178, "$a44.z"}, /* 1905 */
  { 179, "$a44_t"}, /* 1906 */
  { 179, "$a44.t"}, /* 1907 */
  { 180, "$a45_x"}, /* 1908 */
  { 180, "$a45.x"}, /* 1909 */
  { 181, "$a45_y"}, /* 1910 */
  { 181, "$a45.y"}, /* 1911 */
  { 182, "$a45_z"}, /* 1912 */
  { 182, "$a45.z"}, /* 1913 */
  { 183, "$a45_t"}, /* 1914 */
  { 183, "$a45.t"}, /* 1915 */
  { 184, "$a46_x"}, /* 1916 */
  { 184, "$a46.x"}, /* 1917 */
  { 185, "$a46_y"}, /* 1918 */
  { 185, "$a46.y"}, /* 1919 */
  { 186, "$a46_z"}, /* 1920 */
  { 186, "$a46.z"}, /* 1921 */
  { 187, "$a46_t"}, /* 1922 */
  { 187, "$a46.t"}, /* 1923 */
  { 188, "$a47_x"}, /* 1924 */
  { 188, "$a47.x"}, /* 1925 */
  { 189, "$a47_y"}, /* 1926 */
  { 189, "$a47.y"}, /* 1927 */
  { 190, "$a47_z"}, /* 1928 */
  { 190, "$a47.z"}, /* 1929 */
  { 191, "$a47_t"}, /* 1930 */
  { 191, "$a47.t"}, /* 1931 */
  { 192, "$a48_x"}, /* 1932 */
  { 192, "$a48.x"}, /* 1933 */
  { 193, "$a48_y"}, /* 1934 */
  { 193, "$a48.y"}, /* 1935 */
  { 194, "$a48_z"}, /* 1936 */
  { 194, "$a48.z"}, /* 1937 */
  { 195, "$a48_t"}, /* 1938 */
  { 195, "$a48.t"}, /* 1939 */
  { 196, "$a49_x"}, /* 1940 */
  { 196, "$a49.x"}, /* 1941 */
  { 197, "$a49_y"}, /* 1942 */
  { 197, "$a49.y"}, /* 1943 */
  { 198, "$a49_z"}, /* 1944 */
  { 198, "$a49.z"}, /* 1945 */
  { 199, "$a49_t"}, /* 1946 */
  { 199, "$a49.t"}, /* 1947 */
  { 200, "$a50_x"}, /* 1948 */
  { 200, "$a50.x"}, /* 1949 */
  { 201, "$a50_y"}, /* 1950 */
  { 201, "$a50.y"}, /* 1951 */
  { 202, "$a50_z"}, /* 1952 */
  { 202, "$a50.z"}, /* 1953 */
  { 203, "$a50_t"}, /* 1954 */
  { 203, "$a50.t"}, /* 1955 */
  { 204, "$a51_x"}, /* 1956 */
  { 204, "$a51.x"}, /* 1957 */
  { 205, "$a51_y"}, /* 1958 */
  { 205, "$a51.y"}, /* 1959 */
  { 206, "$a51_z"}, /* 1960 */
  { 206, "$a51.z"}, /* 1961 */
  { 207, "$a51_t"}, /* 1962 */
  { 207, "$a51.t"}, /* 1963 */
  { 208, "$a52_x"}, /* 1964 */
  { 208, "$a52.x"}, /* 1965 */
  { 209, "$a52_y"}, /* 1966 */
  { 209, "$a52.y"}, /* 1967 */
  { 210, "$a52_z"}, /* 1968 */
  { 210, "$a52.z"}, /* 1969 */
  { 211, "$a52_t"}, /* 1970 */
  { 211, "$a52.t"}, /* 1971 */
  { 212, "$a53_x"}, /* 1972 */
  { 212, "$a53.x"}, /* 1973 */
  { 213, "$a53_y"}, /* 1974 */
  { 213, "$a53.y"}, /* 1975 */
  { 214, "$a53_z"}, /* 1976 */
  { 214, "$a53.z"}, /* 1977 */
  { 215, "$a53_t"}, /* 1978 */
  { 215, "$a53.t"}, /* 1979 */
  { 216, "$a54_x"}, /* 1980 */
  { 216, "$a54.x"}, /* 1981 */
  { 217, "$a54_y"}, /* 1982 */
  { 217, "$a54.y"}, /* 1983 */
  { 218, "$a54_z"}, /* 1984 */
  { 218, "$a54.z"}, /* 1985 */
  { 219, "$a54_t"}, /* 1986 */
  { 219, "$a54.t"}, /* 1987 */
  { 220, "$a55_x"}, /* 1988 */
  { 220, "$a55.x"}, /* 1989 */
  { 221, "$a55_y"}, /* 1990 */
  { 221, "$a55.y"}, /* 1991 */
  { 222, "$a55_z"}, /* 1992 */
  { 222, "$a55.z"}, /* 1993 */
  { 223, "$a55_t"}, /* 1994 */
  { 223, "$a55.t"}, /* 1995 */
  { 224, "$a56_x"}, /* 1996 */
  { 224, "$a56.x"}, /* 1997 */
  { 225, "$a56_y"}, /* 1998 */
  { 225, "$a56.y"}, /* 1999 */
  { 226, "$a56_z"}, /* 2000 */
  { 226, "$a56.z"}, /* 2001 */
  { 227, "$a56_t"}, /* 2002 */
  { 227, "$a56.t"}, /* 2003 */
  { 228, "$a57_x"}, /* 2004 */
  { 228, "$a57.x"}, /* 2005 */
  { 229, "$a57_y"}, /* 2006 */
  { 229, "$a57.y"}, /* 2007 */
  { 230, "$a57_z"}, /* 2008 */
  { 230, "$a57.z"}, /* 2009 */
  { 231, "$a57_t"}, /* 2010 */
  { 231, "$a57.t"}, /* 2011 */
  { 232, "$a58_x"}, /* 2012 */
  { 232, "$a58.x"}, /* 2013 */
  { 233, "$a58_y"}, /* 2014 */
  { 233, "$a58.y"}, /* 2015 */
  { 234, "$a58_z"}, /* 2016 */
  { 234, "$a58.z"}, /* 2017 */
  { 235, "$a58_t"}, /* 2018 */
  { 235, "$a58.t"}, /* 2019 */
  { 236, "$a59_x"}, /* 2020 */
  { 236, "$a59.x"}, /* 2021 */
  { 237, "$a59_y"}, /* 2022 */
  { 237, "$a59.y"}, /* 2023 */
  { 238, "$a59_z"}, /* 2024 */
  { 238, "$a59.z"}, /* 2025 */
  { 239, "$a59_t"}, /* 2026 */
  { 239, "$a59.t"}, /* 2027 */
  { 240, "$a60_x"}, /* 2028 */
  { 240, "$a60.x"}, /* 2029 */
  { 241, "$a60_y"}, /* 2030 */
  { 241, "$a60.y"}, /* 2031 */
  { 242, "$a60_z"}, /* 2032 */
  { 242, "$a60.z"}, /* 2033 */
  { 243, "$a60_t"}, /* 2034 */
  { 243, "$a60.t"}, /* 2035 */
  { 244, "$a61_x"}, /* 2036 */
  { 244, "$a61.x"}, /* 2037 */
  { 245, "$a61_y"}, /* 2038 */
  { 245, "$a61.y"}, /* 2039 */
  { 246, "$a61_z"}, /* 2040 */
  { 246, "$a61.z"}, /* 2041 */
  { 247, "$a61_t"}, /* 2042 */
  { 247, "$a61.t"}, /* 2043 */
  { 248, "$a62_x"}, /* 2044 */
  { 248, "$a62.x"}, /* 2045 */
  { 249, "$a62_y"}, /* 2046 */
  { 249, "$a62.y"}, /* 2047 */
  { 250, "$a62_z"}, /* 2048 */
  { 250, "$a62.z"}, /* 2049 */
  { 251, "$a62_t"}, /* 2050 */
  { 251, "$a62.t"}, /* 2051 */
  { 252, "$a63_x"}, /* 2052 */
  { 252, "$a63.x"}, /* 2053 */
  { 253, "$a63_y"}, /* 2054 */
  { 253, "$a63.y"}, /* 2055 */
  { 254, "$a63_z"}, /* 2056 */
  { 254, "$a63.z"}, /* 2057 */
  { 255, "$a63_t"}, /* 2058 */
  { 255, "$a63.t"}, /* 2059 */
  { 0, "$a0a1a2a3"}, /* 2060 */
  { 1, "$a4a5a6a7"}, /* 2061 */
  { 2, "$a8a9a10a11"}, /* 2062 */
  { 3, "$a12a13a14a15"}, /* 2063 */
  { 4, "$a16a17a18a19"}, /* 2064 */
  { 5, "$a20a21a22a23"}, /* 2065 */
  { 6, "$a24a25a26a27"}, /* 2066 */
  { 7, "$a28a29a30a31"}, /* 2067 */
  { 8, "$a32a33a34a35"}, /* 2068 */
  { 9, "$a36a37a38a39"}, /* 2069 */
  { 10, "$a40a41a42a43"}, /* 2070 */
  { 11, "$a44a45a46a47"}, /* 2071 */
  { 12, "$a48a49a50a51"}, /* 2072 */
  { 13, "$a52a53a54a55"}, /* 2073 */
  { 14, "$a56a57a58a59"}, /* 2074 */
  { 15, "$a60a61a62a63"}, /* 2075 */
  { 0, "$a0"}, /* 2076 */
  { 0, "$a0a1.lo"}, /* 2077 */
  { 0, "$a0a1a2a3.x"}, /* 2078 */
  { 1, "$a1"}, /* 2079 */
  { 1, "$a0a1.hi"}, /* 2080 */
  { 1, "$a0a1a2a3.y"}, /* 2081 */
  { 2, "$a2"}, /* 2082 */
  { 2, "$a2a3.lo"}, /* 2083 */
  { 2, "$a0a1a2a3.z"}, /* 2084 */
  { 3, "$a3"}, /* 2085 */
  { 3, "$a2a3.hi"}, /* 2086 */
  { 3, "$a0a1a2a3.t"}, /* 2087 */
  { 4, "$a4"}, /* 2088 */
  { 4, "$a4a5.lo"}, /* 2089 */
  { 4, "$a4a5a6a7.x"}, /* 2090 */
  { 5, "$a5"}, /* 2091 */
  { 5, "$a4a5.hi"}, /* 2092 */
  { 5, "$a4a5a6a7.y"}, /* 2093 */
  { 6, "$a6"}, /* 2094 */
  { 6, "$a6a7.lo"}, /* 2095 */
  { 6, "$a4a5a6a7.z"}, /* 2096 */
  { 7, "$a7"}, /* 2097 */
  { 7, "$a6a7.hi"}, /* 2098 */
  { 7, "$a4a5a6a7.t"}, /* 2099 */
  { 8, "$a8"}, /* 2100 */
  { 8, "$a8a9.lo"}, /* 2101 */
  { 8, "$a8a9a10a11.x"}, /* 2102 */
  { 9, "$a9"}, /* 2103 */
  { 9, "$a8a9.hi"}, /* 2104 */
  { 9, "$a8a9a10a11.y"}, /* 2105 */
  { 10, "$a10"}, /* 2106 */
  { 10, "$a10a11.lo"}, /* 2107 */
  { 10, "$a8a9a10a11.z"}, /* 2108 */
  { 11, "$a11"}, /* 2109 */
  { 11, "$a10a11.hi"}, /* 2110 */
  { 11, "$a8a9a10a11.t"}, /* 2111 */
  { 12, "$a12"}, /* 2112 */
  { 12, "$a12a13.lo"}, /* 2113 */
  { 12, "$a12a13a14a15.x"}, /* 2114 */
  { 13, "$a13"}, /* 2115 */
  { 13, "$a12a13.hi"}, /* 2116 */
  { 13, "$a12a13a14a15.y"}, /* 2117 */
  { 14, "$a14"}, /* 2118 */
  { 14, "$a14a15.lo"}, /* 2119 */
  { 14, "$a12a13a14a15.z"}, /* 2120 */
  { 15, "$a15"}, /* 2121 */
  { 15, "$a14a15.hi"}, /* 2122 */
  { 15, "$a12a13a14a15.t"}, /* 2123 */
  { 16, "$a16"}, /* 2124 */
  { 16, "$a16a17.lo"}, /* 2125 */
  { 16, "$a16a17a18a19.x"}, /* 2126 */
  { 17, "$a17"}, /* 2127 */
  { 17, "$a16a17.hi"}, /* 2128 */
  { 17, "$a16a17a18a19.y"}, /* 2129 */
  { 18, "$a18"}, /* 2130 */
  { 18, "$a18a19.lo"}, /* 2131 */
  { 18, "$a16a17a18a19.z"}, /* 2132 */
  { 19, "$a19"}, /* 2133 */
  { 19, "$a18a19.hi"}, /* 2134 */
  { 19, "$a16a17a18a19.t"}, /* 2135 */
  { 20, "$a20"}, /* 2136 */
  { 20, "$a20a21.lo"}, /* 2137 */
  { 20, "$a20a21a22a23.x"}, /* 2138 */
  { 21, "$a21"}, /* 2139 */
  { 21, "$a20a21.hi"}, /* 2140 */
  { 21, "$a20a21a22a23.y"}, /* 2141 */
  { 22, "$a22"}, /* 2142 */
  { 22, "$a22a23.lo"}, /* 2143 */
  { 22, "$a20a21a22a23.z"}, /* 2144 */
  { 23, "$a23"}, /* 2145 */
  { 23, "$a22a23.hi"}, /* 2146 */
  { 23, "$a20a21a22a23.t"}, /* 2147 */
  { 24, "$a24"}, /* 2148 */
  { 24, "$a24a25.lo"}, /* 2149 */
  { 24, "$a24a25a26a27.x"}, /* 2150 */
  { 25, "$a25"}, /* 2151 */
  { 25, "$a24a25.hi"}, /* 2152 */
  { 25, "$a24a25a26a27.y"}, /* 2153 */
  { 26, "$a26"}, /* 2154 */
  { 26, "$a26a27.lo"}, /* 2155 */
  { 26, "$a24a25a26a27.z"}, /* 2156 */
  { 27, "$a27"}, /* 2157 */
  { 27, "$a26a27.hi"}, /* 2158 */
  { 27, "$a24a25a26a27.t"}, /* 2159 */
  { 28, "$a28"}, /* 2160 */
  { 28, "$a28a29.lo"}, /* 2161 */
  { 28, "$a28a29a30a31.x"}, /* 2162 */
  { 29, "$a29"}, /* 2163 */
  { 29, "$a28a29.hi"}, /* 2164 */
  { 29, "$a28a29a30a31.y"}, /* 2165 */
  { 30, "$a30"}, /* 2166 */
  { 30, "$a30a31.lo"}, /* 2167 */
  { 30, "$a28a29a30a31.z"}, /* 2168 */
  { 31, "$a31"}, /* 2169 */
  { 31, "$a30a31.hi"}, /* 2170 */
  { 31, "$a28a29a30a31.t"}, /* 2171 */
  { 32, "$a32"}, /* 2172 */
  { 32, "$a32a33.lo"}, /* 2173 */
  { 32, "$a32a33a34a35.x"}, /* 2174 */
  { 33, "$a33"}, /* 2175 */
  { 33, "$a32a33.hi"}, /* 2176 */
  { 33, "$a32a33a34a35.y"}, /* 2177 */
  { 34, "$a34"}, /* 2178 */
  { 34, "$a34a35.lo"}, /* 2179 */
  { 34, "$a32a33a34a35.z"}, /* 2180 */
  { 35, "$a35"}, /* 2181 */
  { 35, "$a34a35.hi"}, /* 2182 */
  { 35, "$a32a33a34a35.t"}, /* 2183 */
  { 36, "$a36"}, /* 2184 */
  { 36, "$a36a37.lo"}, /* 2185 */
  { 36, "$a36a37a38a39.x"}, /* 2186 */
  { 37, "$a37"}, /* 2187 */
  { 37, "$a36a37.hi"}, /* 2188 */
  { 37, "$a36a37a38a39.y"}, /* 2189 */
  { 38, "$a38"}, /* 2190 */
  { 38, "$a38a39.lo"}, /* 2191 */
  { 38, "$a36a37a38a39.z"}, /* 2192 */
  { 39, "$a39"}, /* 2193 */
  { 39, "$a38a39.hi"}, /* 2194 */
  { 39, "$a36a37a38a39.t"}, /* 2195 */
  { 40, "$a40"}, /* 2196 */
  { 40, "$a40a41.lo"}, /* 2197 */
  { 40, "$a40a41a42a43.x"}, /* 2198 */
  { 41, "$a41"}, /* 2199 */
  { 41, "$a40a41.hi"}, /* 2200 */
  { 41, "$a40a41a42a43.y"}, /* 2201 */
  { 42, "$a42"}, /* 2202 */
  { 42, "$a42a43.lo"}, /* 2203 */
  { 42, "$a40a41a42a43.z"}, /* 2204 */
  { 43, "$a43"}, /* 2205 */
  { 43, "$a42a43.hi"}, /* 2206 */
  { 43, "$a40a41a42a43.t"}, /* 2207 */
  { 44, "$a44"}, /* 2208 */
  { 44, "$a44a45.lo"}, /* 2209 */
  { 44, "$a44a45a46a47.x"}, /* 2210 */
  { 45, "$a45"}, /* 2211 */
  { 45, "$a44a45.hi"}, /* 2212 */
  { 45, "$a44a45a46a47.y"}, /* 2213 */
  { 46, "$a46"}, /* 2214 */
  { 46, "$a46a47.lo"}, /* 2215 */
  { 46, "$a44a45a46a47.z"}, /* 2216 */
  { 47, "$a47"}, /* 2217 */
  { 47, "$a46a47.hi"}, /* 2218 */
  { 47, "$a44a45a46a47.t"}, /* 2219 */
  { 48, "$a48"}, /* 2220 */
  { 48, "$a48a49.lo"}, /* 2221 */
  { 48, "$a48a49a50a51.x"}, /* 2222 */
  { 49, "$a49"}, /* 2223 */
  { 49, "$a48a49.hi"}, /* 2224 */
  { 49, "$a48a49a50a51.y"}, /* 2225 */
  { 50, "$a50"}, /* 2226 */
  { 50, "$a50a51.lo"}, /* 2227 */
  { 50, "$a48a49a50a51.z"}, /* 2228 */
  { 51, "$a51"}, /* 2229 */
  { 51, "$a50a51.hi"}, /* 2230 */
  { 51, "$a48a49a50a51.t"}, /* 2231 */
  { 52, "$a52"}, /* 2232 */
  { 52, "$a52a53.lo"}, /* 2233 */
  { 52, "$a52a53a54a55.x"}, /* 2234 */
  { 53, "$a53"}, /* 2235 */
  { 53, "$a52a53.hi"}, /* 2236 */
  { 53, "$a52a53a54a55.y"}, /* 2237 */
  { 54, "$a54"}, /* 2238 */
  { 54, "$a54a55.lo"}, /* 2239 */
  { 54, "$a52a53a54a55.z"}, /* 2240 */
  { 55, "$a55"}, /* 2241 */
  { 55, "$a54a55.hi"}, /* 2242 */
  { 55, "$a52a53a54a55.t"}, /* 2243 */
  { 56, "$a56"}, /* 2244 */
  { 56, "$a56a57.lo"}, /* 2245 */
  { 56, "$a56a57a58a59.x"}, /* 2246 */
  { 57, "$a57"}, /* 2247 */
  { 57, "$a56a57.hi"}, /* 2248 */
  { 57, "$a56a57a58a59.y"}, /* 2249 */
  { 58, "$a58"}, /* 2250 */
  { 58, "$a58a59.lo"}, /* 2251 */
  { 58, "$a56a57a58a59.z"}, /* 2252 */
  { 59, "$a59"}, /* 2253 */
  { 59, "$a58a59.hi"}, /* 2254 */
  { 59, "$a56a57a58a59.t"}, /* 2255 */
  { 60, "$a60"}, /* 2256 */
  { 60, "$a60a61.lo"}, /* 2257 */
  { 60, "$a60a61a62a63.x"}, /* 2258 */
  { 61, "$a61"}, /* 2259 */
  { 61, "$a60a61.hi"}, /* 2260 */
  { 61, "$a60a61a62a63.y"}, /* 2261 */
  { 62, "$a62"}, /* 2262 */
  { 62, "$a62a63.lo"}, /* 2263 */
  { 62, "$a60a61a62a63.z"}, /* 2264 */
  { 63, "$a63"}, /* 2265 */
  { 63, "$a62a63.hi"}, /* 2266 */
  { 63, "$a60a61a62a63.t"}, /* 2267 */
  { 0, "$a0a1"}, /* 2268 */
  { 0, "$a0a1a2a3.lo"}, /* 2269 */
  { 1, "$a2a3"}, /* 2270 */
  { 1, "$a0a1a2a3.hi"}, /* 2271 */
  { 2, "$a4a5"}, /* 2272 */
  { 2, "$a4a5a6a7.lo"}, /* 2273 */
  { 3, "$a6a7"}, /* 2274 */
  { 3, "$a4a5a6a7.hi"}, /* 2275 */
  { 4, "$a8a9"}, /* 2276 */
  { 4, "$a8a9a10a11.lo"}, /* 2277 */
  { 5, "$a10a11"}, /* 2278 */
  { 5, "$a8a9a10a11.hi"}, /* 2279 */
  { 6, "$a12a13"}, /* 2280 */
  { 6, "$a12a13a14a15.lo"}, /* 2281 */
  { 7, "$a14a15"}, /* 2282 */
  { 7, "$a12a13a14a15.hi"}, /* 2283 */
  { 8, "$a16a17"}, /* 2284 */
  { 8, "$a16a17a18a19.lo"}, /* 2285 */
  { 9, "$a18a19"}, /* 2286 */
  { 9, "$a16a17a18a19.hi"}, /* 2287 */
  { 10, "$a20a21"}, /* 2288 */
  { 10, "$a20a21a22a23.lo"}, /* 2289 */
  { 11, "$a22a23"}, /* 2290 */
  { 11, "$a20a21a22a23.hi"}, /* 2291 */
  { 12, "$a24a25"}, /* 2292 */
  { 12, "$a24a25a26a27.lo"}, /* 2293 */
  { 13, "$a26a27"}, /* 2294 */
  { 13, "$a24a25a26a27.hi"}, /* 2295 */
  { 14, "$a28a29"}, /* 2296 */
  { 14, "$a28a29a30a31.lo"}, /* 2297 */
  { 15, "$a30a31"}, /* 2298 */
  { 15, "$a28a29a30a31.hi"}, /* 2299 */
  { 16, "$a32a33"}, /* 2300 */
  { 16, "$a32a33a34a35.lo"}, /* 2301 */
  { 17, "$a34a35"}, /* 2302 */
  { 17, "$a32a33a34a35.hi"}, /* 2303 */
  { 18, "$a36a37"}, /* 2304 */
  { 18, "$a36a37a38a39.lo"}, /* 2305 */
  { 19, "$a38a39"}, /* 2306 */
  { 19, "$a36a37a38a39.hi"}, /* 2307 */
  { 20, "$a40a41"}, /* 2308 */
  { 20, "$a40a41a42a43.lo"}, /* 2309 */
  { 21, "$a42a43"}, /* 2310 */
  { 21, "$a40a41a42a43.hi"}, /* 2311 */
  { 22, "$a44a45"}, /* 2312 */
  { 22, "$a44a45a46a47.lo"}, /* 2313 */
  { 23, "$a46a47"}, /* 2314 */
  { 23, "$a44a45a46a47.hi"}, /* 2315 */
  { 24, "$a48a49"}, /* 2316 */
  { 24, "$a48a49a50a51.lo"}, /* 2317 */
  { 25, "$a50a51"}, /* 2318 */
  { 25, "$a48a49a50a51.hi"}, /* 2319 */
  { 26, "$a52a53"}, /* 2320 */
  { 26, "$a52a53a54a55.lo"}, /* 2321 */
  { 27, "$a54a55"}, /* 2322 */
  { 27, "$a52a53a54a55.hi"}, /* 2323 */
  { 28, "$a56a57"}, /* 2324 */
  { 28, "$a56a57a58a59.lo"}, /* 2325 */
  { 29, "$a58a59"}, /* 2326 */
  { 29, "$a56a57a58a59.hi"}, /* 2327 */
  { 30, "$a60a61"}, /* 2328 */
  { 30, "$a60a61a62a63.lo"}, /* 2329 */
  { 31, "$a62a63"}, /* 2330 */
  { 31, "$a60a61a62a63.hi"}, /* 2331 */
};


int kvx_kv3_v2_dec_registers[] = {
  0, /* 0 $r0 */
  3, /* 1 $r1 */
  6, /* 2 $r2 */
  9, /* 3 $r3 */
  12, /* 4 $r4 */
  15, /* 5 $r5 */
  18, /* 6 $r6 */
  21, /* 7 $r7 */
  24, /* 8 $r8 */
  27, /* 9 $r9 */
  30, /* 10 $r10 */
  33, /* 11 $r11 */
  36, /* 12 $r12 */
  38, /* 13 $r13 */
  40, /* 14 $r14 */
  42, /* 15 $r15 */
  44, /* 16 $r16 */
  47, /* 17 $r17 */
  50, /* 18 $r18 */
  53, /* 19 $r19 */
  56, /* 20 $r20 */
  59, /* 21 $r21 */
  62, /* 22 $r22 */
  65, /* 23 $r23 */
  68, /* 24 $r24 */
  71, /* 25 $r25 */
  74, /* 26 $r26 */
  77, /* 27 $r27 */
  80, /* 28 $r28 */
  83, /* 29 $r29 */
  86, /* 30 $r30 */
  89, /* 31 $r31 */
  92, /* 32 $r32 */
  95, /* 33 $r33 */
  98, /* 34 $r34 */
  101, /* 35 $r35 */
  104, /* 36 $r36 */
  107, /* 37 $r37 */
  110, /* 38 $r38 */
  113, /* 39 $r39 */
  116, /* 40 $r40 */
  119, /* 41 $r41 */
  122, /* 42 $r42 */
  125, /* 43 $r43 */
  128, /* 44 $r44 */
  131, /* 45 $r45 */
  134, /* 46 $r46 */
  137, /* 47 $r47 */
  140, /* 48 $r48 */
  143, /* 49 $r49 */
  146, /* 50 $r50 */
  149, /* 51 $r51 */
  152, /* 52 $r52 */
  155, /* 53 $r53 */
  158, /* 54 $r54 */
  161, /* 55 $r55 */
  164, /* 56 $r56 */
  167, /* 57 $r57 */
  170, /* 58 $r58 */
  173, /* 59 $r59 */
  176, /* 60 $r60 */
  179, /* 61 $r61 */
  182, /* 62 $r62 */
  185, /* 63 $r63 */
  188, /* 64 $r0r1 */
  190, /* 65 $r2r3 */
  192, /* 66 $r4r5 */
  194, /* 67 $r6r7 */
  196, /* 68 $r8r9 */
  198, /* 69 $r10r11 */
  200, /* 70 $r12r13 */
  202, /* 71 $r14r15 */
  204, /* 72 $r16r17 */
  206, /* 73 $r18r19 */
  208, /* 74 $r20r21 */
  210, /* 75 $r22r23 */
  212, /* 76 $r24r25 */
  214, /* 77 $r26r27 */
  216, /* 78 $r28r29 */
  218, /* 79 $r30r31 */
  220, /* 80 $r32r33 */
  222, /* 81 $r34r35 */
  224, /* 82 $r36r37 */
  226, /* 83 $r38r39 */
  228, /* 84 $r40r41 */
  230, /* 85 $r42r43 */
  232, /* 86 $r44r45 */
  234, /* 87 $r46r47 */
  236, /* 88 $r48r49 */
  238, /* 89 $r50r51 */
  240, /* 90 $r52r53 */
  242, /* 91 $r54r55 */
  244, /* 92 $r56r57 */
  246, /* 93 $r58r59 */
  248, /* 94 $r60r61 */
  250, /* 95 $r62r63 */
  252, /* 96 $r0r1r2r3 */
  253, /* 97 $r4r5r6r7 */
  254, /* 98 $r8r9r10r11 */
  255, /* 99 $r12r13r14r15 */
  256, /* 100 $r16r17r18r19 */
  257, /* 101 $r20r21r22r23 */
  258, /* 102 $r24r25r26r27 */
  259, /* 103 $r28r29r30r31 */
  260, /* 104 $r32r33r34r35 */
  261, /* 105 $r36r37r38r39 */
  262, /* 106 $r40r41r42r43 */
  263, /* 107 $r44r45r46r47 */
  264, /* 108 $r48r49r50r51 */
  265, /* 109 $r52r53r54r55 */
  266, /* 110 $r56r57r58r59 */
  267, /* 111 $r60r61r62r63 */
  268, /* 112 $pc */
  270, /* 113 $ps */
  272, /* 114 $pcr */
  274, /* 115 $ra */
  276, /* 116 $cs */
  278, /* 117 $csit */
  280, /* 118 $aespc */
  282, /* 119 $ls */
  284, /* 120 $le */
  286, /* 121 $lc */
  288, /* 122 $ipe */
  290, /* 123 $men */
  292, /* 124 $pmc */
  294, /* 125 $pm0 */
  296, /* 126 $pm1 */
  298, /* 127 $pm2 */
  300, /* 128 $pm3 */
  302, /* 129 $pmsa */
  304, /* 130 $tcr */
  306, /* 131 $t0v */
  308, /* 132 $t1v */
  310, /* 133 $t0r */
  312, /* 134 $t1r */
  314, /* 135 $wdv */
  316, /* 136 $wdr */
  318, /* 137 $ile */
  320, /* 138 $ill */
  322, /* 139 $ilr */
  324, /* 140 $mmc */
  326, /* 141 $tel */
  328, /* 142 $teh */
  330, /* 143 $ixc */
  332, /* 144 $syo */
  334, /* 145 $hto */
  336, /* 146 $ito */
  338, /* 147 $do */
  340, /* 148 $mo */
  342, /* 149 $pso */
  344, /* 150 $tpcm0 */
  346, /* 151 $tpcm1 */
  348, /* 152 $res40 */
  350, /* 153 $dba0 */
  352, /* 154 $dba1 */
  354, /* 155 $dwa0 */
  356, /* 156 $dwa1 */
  358, /* 157 $mes */
  360, /* 158 $ws */
  362, /* 159 $dc0 */
  364, /* 160 $dc1 */
  366, /* 161 $dc2 */
  368, /* 162 $dc3 */
  370, /* 163 $dba2 */
  372, /* 164 $dba3 */
  374, /* 165 $dwa2 */
  376, /* 166 $dwa3 */
  378, /* 167 $tpcm2 */
  380, /* 168 $tpcmc */
  382, /* 169 $res57 */
  384, /* 170 $res58 */
  386, /* 171 $res59 */
  388, /* 172 $res60 */
  390, /* 173 $res61 */
  392, /* 174 $res62 */
  394, /* 175 $res63 */
  396, /* 176 $spc_pl0 */
  398, /* 177 $spc_pl1 */
  400, /* 178 $spc_pl2 */
  402, /* 179 $spc_pl3 */
  404, /* 180 $sps_pl0 */
  406, /* 181 $sps_pl1 */
  408, /* 182 $sps_pl2 */
  410, /* 183 $sps_pl3 */
  412, /* 184 $ea_pl0 */
  414, /* 185 $ea_pl1 */
  416, /* 186 $ea_pl2 */
  418, /* 187 $ea_pl3 */
  420, /* 188 $ev_pl0 */
  422, /* 189 $ev_pl1 */
  424, /* 190 $ev_pl2 */
  426, /* 191 $ev_pl3 */
  428, /* 192 $sr_pl0 */
  430, /* 193 $sr_pl1 */
  432, /* 194 $sr_pl2 */
  434, /* 195 $sr_pl3 */
  436, /* 196 $es_pl0 */
  438, /* 197 $es_pl1 */
  440, /* 198 $es_pl2 */
  442, /* 199 $es_pl3 */
  444, /* 200 $sid_pl0 */
  446, /* 201 $sid_pl1 */
  448, /* 202 $sid_pl2 */
  450, /* 203 $sid_pl3 */
  452, /* 204 $sr1_pl0 */
  454, /* 205 $sr1_pl1 */
  456, /* 206 $sr1_pl2 */
  458, /* 207 $sr1_pl3 */
  460, /* 208 $syow */
  462, /* 209 $htow */
  464, /* 210 $itow */
  466, /* 211 $dow */
  468, /* 212 $mow */
  470, /* 213 $psow */
  472, /* 214 $res102 */
  474, /* 215 $res103 */
  476, /* 216 $res104 */
  478, /* 217 $res105 */
  480, /* 218 $res106 */
  482, /* 219 $res107 */
  484, /* 220 $res108 */
  486, /* 221 $res109 */
  488, /* 222 $res110 */
  490, /* 223 $res111 */
  492, /* 224 $res112 */
  494, /* 225 $res113 */
  496, /* 226 $res114 */
  498, /* 227 $res115 */
  500, /* 228 $res116 */
  502, /* 229 $res117 */
  504, /* 230 $res118 */
  506, /* 231 $res119 */
  508, /* 232 $res120 */
  510, /* 233 $res121 */
  512, /* 234 $res122 */
  514, /* 235 $res123 */
  516, /* 236 $res124 */
  518, /* 237 $res125 */
  520, /* 238 $res126 */
  522, /* 239 $res127 */
  524, /* 240 $spc */
  526, /* 241 $res129 */
  528, /* 242 $res130 */
  530, /* 243 $res131 */
  532, /* 244 $sps */
  534, /* 245 $res133 */
  536, /* 246 $res134 */
  538, /* 247 $res135 */
  540, /* 248 $ea */
  542, /* 249 $res137 */
  544, /* 250 $res138 */
  546, /* 251 $res139 */
  548, /* 252 $ev */
  550, /* 253 $res141 */
  552, /* 254 $res142 */
  554, /* 255 $res143 */
  556, /* 256 $sr */
  558, /* 257 $res145 */
  560, /* 258 $res146 */
  562, /* 259 $res147 */
  564, /* 260 $es */
  566, /* 261 $res149 */
  568, /* 262 $res150 */
  570, /* 263 $res151 */
  572, /* 264 $sid */
  574, /* 265 $res153 */
  576, /* 266 $res154 */
  578, /* 267 $res155 */
  580, /* 268 $sr1 */
  582, /* 269 $res157 */
  584, /* 270 $res158 */
  586, /* 271 $res159 */
  588, /* 272 $res160 */
  590, /* 273 $res161 */
  592, /* 274 $res162 */
  594, /* 275 $res163 */
  596, /* 276 $res164 */
  598, /* 277 $res165 */
  600, /* 278 $res166 */
  602, /* 279 $res167 */
  604, /* 280 $res168 */
  606, /* 281 $res169 */
  608, /* 282 $res170 */
  610, /* 283 $res171 */
  612, /* 284 $res172 */
  614, /* 285 $res173 */
  616, /* 286 $res174 */
  618, /* 287 $res175 */
  620, /* 288 $res176 */
  622, /* 289 $res177 */
  624, /* 290 $res178 */
  626, /* 291 $res179 */
  628, /* 292 $res180 */
  630, /* 293 $res181 */
  632, /* 294 $res182 */
  634, /* 295 $res183 */
  636, /* 296 $res184 */
  638, /* 297 $res185 */
  640, /* 298 $res186 */
  642, /* 299 $res187 */
  644, /* 300 $res188 */
  646, /* 301 $res189 */
  648, /* 302 $res190 */
  650, /* 303 $res191 */
  652, /* 304 $res192 */
  654, /* 305 $res193 */
  656, /* 306 $res194 */
  658, /* 307 $res195 */
  660, /* 308 $res196 */
  662, /* 309 $res197 */
  664, /* 310 $res198 */
  666, /* 311 $res199 */
  668, /* 312 $res200 */
  670, /* 313 $res201 */
  672, /* 314 $res202 */
  674, /* 315 $res203 */
  676, /* 316 $res204 */
  678, /* 317 $res205 */
  680, /* 318 $res206 */
  682, /* 319 $res207 */
  684, /* 320 $res208 */
  686, /* 321 $res209 */
  688, /* 322 $res210 */
  690, /* 323 $res211 */
  692, /* 324 $res212 */
  694, /* 325 $res213 */
  696, /* 326 $res214 */
  698, /* 327 $res215 */
  700, /* 328 $res216 */
  702, /* 329 $res217 */
  704, /* 330 $res218 */
  706, /* 331 $res219 */
  708, /* 332 $res220 */
  710, /* 333 $res221 */
  712, /* 334 $res222 */
  714, /* 335 $res223 */
  716, /* 336 $res224 */
  718, /* 337 $res225 */
  720, /* 338 $res226 */
  722, /* 339 $res227 */
  724, /* 340 $res228 */
  726, /* 341 $res229 */
  728, /* 342 $res230 */
  730, /* 343 $res231 */
  732, /* 344 $res232 */
  734, /* 345 $res233 */
  736, /* 346 $res234 */
  738, /* 347 $res235 */
  740, /* 348 $res236 */
  742, /* 349 $res237 */
  744, /* 350 $res238 */
  746, /* 351 $res239 */
  748, /* 352 $res240 */
  750, /* 353 $res241 */
  752, /* 354 $res242 */
  754, /* 355 $res243 */
  756, /* 356 $res244 */
  758, /* 357 $res245 */
  760, /* 358 $res246 */
  762, /* 359 $res247 */
  764, /* 360 $res248 */
  766, /* 361 $res249 */
  768, /* 362 $res250 */
  770, /* 363 $res251 */
  772, /* 364 $res252 */
  774, /* 365 $res253 */
  776, /* 366 $res254 */
  778, /* 367 $res255 */
  780, /* 368 $vsfr0 */
  782, /* 369 $vsfr1 */
  784, /* 370 $vsfr2 */
  786, /* 371 $vsfr3 */
  788, /* 372 $vsfr4 */
  790, /* 373 $vsfr5 */
  792, /* 374 $vsfr6 */
  794, /* 375 $vsfr7 */
  796, /* 376 $vsfr8 */
  798, /* 377 $vsfr9 */
  800, /* 378 $vsfr10 */
  802, /* 379 $vsfr11 */
  804, /* 380 $vsfr12 */
  806, /* 381 $vsfr13 */
  808, /* 382 $vsfr14 */
  810, /* 383 $vsfr15 */
  812, /* 384 $vsfr16 */
  814, /* 385 $vsfr17 */
  816, /* 386 $vsfr18 */
  818, /* 387 $vsfr19 */
  820, /* 388 $vsfr20 */
  822, /* 389 $vsfr21 */
  824, /* 390 $vsfr22 */
  826, /* 391 $vsfr23 */
  828, /* 392 $vsfr24 */
  830, /* 393 $vsfr25 */
  832, /* 394 $vsfr26 */
  834, /* 395 $vsfr27 */
  836, /* 396 $vsfr28 */
  838, /* 397 $vsfr29 */
  840, /* 398 $vsfr30 */
  842, /* 399 $vsfr31 */
  844, /* 400 $vsfr32 */
  846, /* 401 $vsfr33 */
  848, /* 402 $vsfr34 */
  850, /* 403 $vsfr35 */
  852, /* 404 $vsfr36 */
  854, /* 405 $vsfr37 */
  856, /* 406 $vsfr38 */
  858, /* 407 $vsfr39 */
  860, /* 408 $vsfr40 */
  862, /* 409 $vsfr41 */
  864, /* 410 $vsfr42 */
  866, /* 411 $vsfr43 */
  868, /* 412 $vsfr44 */
  870, /* 413 $vsfr45 */
  872, /* 414 $vsfr46 */
  874, /* 415 $vsfr47 */
  876, /* 416 $vsfr48 */
  878, /* 417 $vsfr49 */
  880, /* 418 $vsfr50 */
  882, /* 419 $vsfr51 */
  884, /* 420 $vsfr52 */
  886, /* 421 $vsfr53 */
  888, /* 422 $vsfr54 */
  890, /* 423 $vsfr55 */
  892, /* 424 $vsfr56 */
  894, /* 425 $vsfr57 */
  896, /* 426 $vsfr58 */
  898, /* 427 $vsfr59 */
  900, /* 428 $vsfr60 */
  902, /* 429 $vsfr61 */
  904, /* 430 $vsfr62 */
  906, /* 431 $vsfr63 */
  908, /* 432 $vsfr64 */
  910, /* 433 $vsfr65 */
  912, /* 434 $vsfr66 */
  914, /* 435 $vsfr67 */
  916, /* 436 $vsfr68 */
  918, /* 437 $vsfr69 */
  920, /* 438 $vsfr70 */
  922, /* 439 $vsfr71 */
  924, /* 440 $vsfr72 */
  926, /* 441 $vsfr73 */
  928, /* 442 $vsfr74 */
  930, /* 443 $vsfr75 */
  932, /* 444 $vsfr76 */
  934, /* 445 $vsfr77 */
  936, /* 446 $vsfr78 */
  938, /* 447 $vsfr79 */
  940, /* 448 $vsfr80 */
  942, /* 449 $vsfr81 */
  944, /* 450 $vsfr82 */
  946, /* 451 $vsfr83 */
  948, /* 452 $vsfr84 */
  950, /* 453 $vsfr85 */
  952, /* 454 $vsfr86 */
  954, /* 455 $vsfr87 */
  956, /* 456 $vsfr88 */
  958, /* 457 $vsfr89 */
  960, /* 458 $vsfr90 */
  962, /* 459 $vsfr91 */
  964, /* 460 $vsfr92 */
  966, /* 461 $vsfr93 */
  968, /* 462 $vsfr94 */
  970, /* 463 $vsfr95 */
  972, /* 464 $vsfr96 */
  974, /* 465 $vsfr97 */
  976, /* 466 $vsfr98 */
  978, /* 467 $vsfr99 */
  980, /* 468 $vsfr100 */
  982, /* 469 $vsfr101 */
  984, /* 470 $vsfr102 */
  986, /* 471 $vsfr103 */
  988, /* 472 $vsfr104 */
  990, /* 473 $vsfr105 */
  992, /* 474 $vsfr106 */
  994, /* 475 $vsfr107 */
  996, /* 476 $vsfr108 */
  998, /* 477 $vsfr109 */
  1000, /* 478 $vsfr110 */
  1002, /* 479 $vsfr111 */
  1004, /* 480 $vsfr112 */
  1006, /* 481 $vsfr113 */
  1008, /* 482 $vsfr114 */
  1010, /* 483 $vsfr115 */
  1012, /* 484 $vsfr116 */
  1014, /* 485 $vsfr117 */
  1016, /* 486 $vsfr118 */
  1018, /* 487 $vsfr119 */
  1020, /* 488 $vsfr120 */
  1022, /* 489 $vsfr121 */
  1024, /* 490 $vsfr122 */
  1026, /* 491 $vsfr123 */
  1028, /* 492 $vsfr124 */
  1030, /* 493 $vsfr125 */
  1032, /* 494 $vsfr126 */
  1034, /* 495 $vsfr127 */
  1036, /* 496 $vsfr128 */
  1038, /* 497 $vsfr129 */
  1040, /* 498 $vsfr130 */
  1042, /* 499 $vsfr131 */
  1044, /* 500 $vsfr132 */
  1046, /* 501 $vsfr133 */
  1048, /* 502 $vsfr134 */
  1050, /* 503 $vsfr135 */
  1052, /* 504 $vsfr136 */
  1054, /* 505 $vsfr137 */
  1056, /* 506 $vsfr138 */
  1058, /* 507 $vsfr139 */
  1060, /* 508 $vsfr140 */
  1062, /* 509 $vsfr141 */
  1064, /* 510 $vsfr142 */
  1066, /* 511 $vsfr143 */
  1068, /* 512 $vsfr144 */
  1070, /* 513 $vsfr145 */
  1072, /* 514 $vsfr146 */
  1074, /* 515 $vsfr147 */
  1076, /* 516 $vsfr148 */
  1078, /* 517 $vsfr149 */
  1080, /* 518 $vsfr150 */
  1082, /* 519 $vsfr151 */
  1084, /* 520 $vsfr152 */
  1086, /* 521 $vsfr153 */
  1088, /* 522 $vsfr154 */
  1090, /* 523 $vsfr155 */
  1092, /* 524 $vsfr156 */
  1094, /* 525 $vsfr157 */
  1096, /* 526 $vsfr158 */
  1098, /* 527 $vsfr159 */
  1100, /* 528 $vsfr160 */
  1102, /* 529 $vsfr161 */
  1104, /* 530 $vsfr162 */
  1106, /* 531 $vsfr163 */
  1108, /* 532 $vsfr164 */
  1110, /* 533 $vsfr165 */
  1112, /* 534 $vsfr166 */
  1114, /* 535 $vsfr167 */
  1116, /* 536 $vsfr168 */
  1118, /* 537 $vsfr169 */
  1120, /* 538 $vsfr170 */
  1122, /* 539 $vsfr171 */
  1124, /* 540 $vsfr172 */
  1126, /* 541 $vsfr173 */
  1128, /* 542 $vsfr174 */
  1130, /* 543 $vsfr175 */
  1132, /* 544 $vsfr176 */
  1134, /* 545 $vsfr177 */
  1136, /* 546 $vsfr178 */
  1138, /* 547 $vsfr179 */
  1140, /* 548 $vsfr180 */
  1142, /* 549 $vsfr181 */
  1144, /* 550 $vsfr182 */
  1146, /* 551 $vsfr183 */
  1148, /* 552 $vsfr184 */
  1150, /* 553 $vsfr185 */
  1152, /* 554 $vsfr186 */
  1154, /* 555 $vsfr187 */
  1156, /* 556 $vsfr188 */
  1158, /* 557 $vsfr189 */
  1160, /* 558 $vsfr190 */
  1162, /* 559 $vsfr191 */
  1164, /* 560 $vsfr192 */
  1166, /* 561 $vsfr193 */
  1168, /* 562 $vsfr194 */
  1170, /* 563 $vsfr195 */
  1172, /* 564 $vsfr196 */
  1174, /* 565 $vsfr197 */
  1176, /* 566 $vsfr198 */
  1178, /* 567 $vsfr199 */
  1180, /* 568 $vsfr200 */
  1182, /* 569 $vsfr201 */
  1184, /* 570 $vsfr202 */
  1186, /* 571 $vsfr203 */
  1188, /* 572 $vsfr204 */
  1190, /* 573 $vsfr205 */
  1192, /* 574 $vsfr206 */
  1194, /* 575 $vsfr207 */
  1196, /* 576 $vsfr208 */
  1198, /* 577 $vsfr209 */
  1200, /* 578 $vsfr210 */
  1202, /* 579 $vsfr211 */
  1204, /* 580 $vsfr212 */
  1206, /* 581 $vsfr213 */
  1208, /* 582 $vsfr214 */
  1210, /* 583 $vsfr215 */
  1212, /* 584 $vsfr216 */
  1214, /* 585 $vsfr217 */
  1216, /* 586 $vsfr218 */
  1218, /* 587 $vsfr219 */
  1220, /* 588 $vsfr220 */
  1222, /* 589 $vsfr221 */
  1224, /* 590 $vsfr222 */
  1226, /* 591 $vsfr223 */
  1228, /* 592 $vsfr224 */
  1230, /* 593 $vsfr225 */
  1232, /* 594 $vsfr226 */
  1234, /* 595 $vsfr227 */
  1236, /* 596 $vsfr228 */
  1238, /* 597 $vsfr229 */
  1240, /* 598 $vsfr230 */
  1242, /* 599 $vsfr231 */
  1244, /* 600 $vsfr232 */
  1246, /* 601 $vsfr233 */
  1248, /* 602 $vsfr234 */
  1250, /* 603 $vsfr235 */
  1252, /* 604 $vsfr236 */
  1254, /* 605 $vsfr237 */
  1256, /* 606 $vsfr238 */
  1258, /* 607 $vsfr239 */
  1260, /* 608 $vsfr240 */
  1262, /* 609 $vsfr241 */
  1264, /* 610 $vsfr242 */
  1266, /* 611 $vsfr243 */
  1268, /* 612 $vsfr244 */
  1270, /* 613 $vsfr245 */
  1272, /* 614 $vsfr246 */
  1274, /* 615 $vsfr247 */
  1276, /* 616 $vsfr248 */
  1278, /* 617 $vsfr249 */
  1280, /* 618 $vsfr250 */
  1282, /* 619 $vsfr251 */
  1284, /* 620 $vsfr252 */
  1286, /* 621 $vsfr253 */
  1288, /* 622 $vsfr254 */
  1290, /* 623 $vsfr255 */
  1292, /* 624 $a0_lo */
  1294, /* 625 $a0_hi */
  1296, /* 626 $a1_lo */
  1298, /* 627 $a1_hi */
  1300, /* 628 $a2_lo */
  1302, /* 629 $a2_hi */
  1304, /* 630 $a3_lo */
  1306, /* 631 $a3_hi */
  1308, /* 632 $a4_lo */
  1310, /* 633 $a4_hi */
  1312, /* 634 $a5_lo */
  1314, /* 635 $a5_hi */
  1316, /* 636 $a6_lo */
  1318, /* 637 $a6_hi */
  1320, /* 638 $a7_lo */
  1322, /* 639 $a7_hi */
  1324, /* 640 $a8_lo */
  1326, /* 641 $a8_hi */
  1328, /* 642 $a9_lo */
  1330, /* 643 $a9_hi */
  1332, /* 644 $a10_lo */
  1334, /* 645 $a10_hi */
  1336, /* 646 $a11_lo */
  1338, /* 647 $a11_hi */
  1340, /* 648 $a12_lo */
  1342, /* 649 $a12_hi */
  1344, /* 650 $a13_lo */
  1346, /* 651 $a13_hi */
  1348, /* 652 $a14_lo */
  1350, /* 653 $a14_hi */
  1352, /* 654 $a15_lo */
  1354, /* 655 $a15_hi */
  1356, /* 656 $a16_lo */
  1358, /* 657 $a16_hi */
  1360, /* 658 $a17_lo */
  1362, /* 659 $a17_hi */
  1364, /* 660 $a18_lo */
  1366, /* 661 $a18_hi */
  1368, /* 662 $a19_lo */
  1370, /* 663 $a19_hi */
  1372, /* 664 $a20_lo */
  1374, /* 665 $a20_hi */
  1376, /* 666 $a21_lo */
  1378, /* 667 $a21_hi */
  1380, /* 668 $a22_lo */
  1382, /* 669 $a22_hi */
  1384, /* 670 $a23_lo */
  1386, /* 671 $a23_hi */
  1388, /* 672 $a24_lo */
  1390, /* 673 $a24_hi */
  1392, /* 674 $a25_lo */
  1394, /* 675 $a25_hi */
  1396, /* 676 $a26_lo */
  1398, /* 677 $a26_hi */
  1400, /* 678 $a27_lo */
  1402, /* 679 $a27_hi */
  1404, /* 680 $a28_lo */
  1406, /* 681 $a28_hi */
  1408, /* 682 $a29_lo */
  1410, /* 683 $a29_hi */
  1412, /* 684 $a30_lo */
  1414, /* 685 $a30_hi */
  1416, /* 686 $a31_lo */
  1418, /* 687 $a31_hi */
  1420, /* 688 $a32_lo */
  1422, /* 689 $a32_hi */
  1424, /* 690 $a33_lo */
  1426, /* 691 $a33_hi */
  1428, /* 692 $a34_lo */
  1430, /* 693 $a34_hi */
  1432, /* 694 $a35_lo */
  1434, /* 695 $a35_hi */
  1436, /* 696 $a36_lo */
  1438, /* 697 $a36_hi */
  1440, /* 698 $a37_lo */
  1442, /* 699 $a37_hi */
  1444, /* 700 $a38_lo */
  1446, /* 701 $a38_hi */
  1448, /* 702 $a39_lo */
  1450, /* 703 $a39_hi */
  1452, /* 704 $a40_lo */
  1454, /* 705 $a40_hi */
  1456, /* 706 $a41_lo */
  1458, /* 707 $a41_hi */
  1460, /* 708 $a42_lo */
  1462, /* 709 $a42_hi */
  1464, /* 710 $a43_lo */
  1466, /* 711 $a43_hi */
  1468, /* 712 $a44_lo */
  1470, /* 713 $a44_hi */
  1472, /* 714 $a45_lo */
  1474, /* 715 $a45_hi */
  1476, /* 716 $a46_lo */
  1478, /* 717 $a46_hi */
  1480, /* 718 $a47_lo */
  1482, /* 719 $a47_hi */
  1484, /* 720 $a48_lo */
  1486, /* 721 $a48_hi */
  1488, /* 722 $a49_lo */
  1490, /* 723 $a49_hi */
  1492, /* 724 $a50_lo */
  1494, /* 725 $a50_hi */
  1496, /* 726 $a51_lo */
  1498, /* 727 $a51_hi */
  1500, /* 728 $a52_lo */
  1502, /* 729 $a52_hi */
  1504, /* 730 $a53_lo */
  1506, /* 731 $a53_hi */
  1508, /* 732 $a54_lo */
  1510, /* 733 $a54_hi */
  1512, /* 734 $a55_lo */
  1514, /* 735 $a55_hi */
  1516, /* 736 $a56_lo */
  1518, /* 737 $a56_hi */
  1520, /* 738 $a57_lo */
  1522, /* 739 $a57_hi */
  1524, /* 740 $a58_lo */
  1526, /* 741 $a58_hi */
  1528, /* 742 $a59_lo */
  1530, /* 743 $a59_hi */
  1532, /* 744 $a60_lo */
  1534, /* 745 $a60_hi */
  1536, /* 746 $a61_lo */
  1538, /* 747 $a61_hi */
  1540, /* 748 $a62_lo */
  1542, /* 749 $a62_hi */
  1544, /* 750 $a63_lo */
  1546, /* 751 $a63_hi */
  1548, /* 752 $a0_x */
  1550, /* 753 $a0_y */
  1552, /* 754 $a0_z */
  1554, /* 755 $a0_t */
  1556, /* 756 $a1_x */
  1558, /* 757 $a1_y */
  1560, /* 758 $a1_z */
  1562, /* 759 $a1_t */
  1564, /* 760 $a2_x */
  1566, /* 761 $a2_y */
  1568, /* 762 $a2_z */
  1570, /* 763 $a2_t */
  1572, /* 764 $a3_x */
  1574, /* 765 $a3_y */
  1576, /* 766 $a3_z */
  1578, /* 767 $a3_t */
  1580, /* 768 $a4_x */
  1582, /* 769 $a4_y */
  1584, /* 770 $a4_z */
  1586, /* 771 $a4_t */
  1588, /* 772 $a5_x */
  1590, /* 773 $a5_y */
  1592, /* 774 $a5_z */
  1594, /* 775 $a5_t */
  1596, /* 776 $a6_x */
  1598, /* 777 $a6_y */
  1600, /* 778 $a6_z */
  1602, /* 779 $a6_t */
  1604, /* 780 $a7_x */
  1606, /* 781 $a7_y */
  1608, /* 782 $a7_z */
  1610, /* 783 $a7_t */
  1612, /* 784 $a8_x */
  1614, /* 785 $a8_y */
  1616, /* 786 $a8_z */
  1618, /* 787 $a8_t */
  1620, /* 788 $a9_x */
  1622, /* 789 $a9_y */
  1624, /* 790 $a9_z */
  1626, /* 791 $a9_t */
  1628, /* 792 $a10_x */
  1630, /* 793 $a10_y */
  1632, /* 794 $a10_z */
  1634, /* 795 $a10_t */
  1636, /* 796 $a11_x */
  1638, /* 797 $a11_y */
  1640, /* 798 $a11_z */
  1642, /* 799 $a11_t */
  1644, /* 800 $a12_x */
  1646, /* 801 $a12_y */
  1648, /* 802 $a12_z */
  1650, /* 803 $a12_t */
  1652, /* 804 $a13_x */
  1654, /* 805 $a13_y */
  1656, /* 806 $a13_z */
  1658, /* 807 $a13_t */
  1660, /* 808 $a14_x */
  1662, /* 809 $a14_y */
  1664, /* 810 $a14_z */
  1666, /* 811 $a14_t */
  1668, /* 812 $a15_x */
  1670, /* 813 $a15_y */
  1672, /* 814 $a15_z */
  1674, /* 815 $a15_t */
  1676, /* 816 $a16_x */
  1678, /* 817 $a16_y */
  1680, /* 818 $a16_z */
  1682, /* 819 $a16_t */
  1684, /* 820 $a17_x */
  1686, /* 821 $a17_y */
  1688, /* 822 $a17_z */
  1690, /* 823 $a17_t */
  1692, /* 824 $a18_x */
  1694, /* 825 $a18_y */
  1696, /* 826 $a18_z */
  1698, /* 827 $a18_t */
  1700, /* 828 $a19_x */
  1702, /* 829 $a19_y */
  1704, /* 830 $a19_z */
  1706, /* 831 $a19_t */
  1708, /* 832 $a20_x */
  1710, /* 833 $a20_y */
  1712, /* 834 $a20_z */
  1714, /* 835 $a20_t */
  1716, /* 836 $a21_x */
  1718, /* 837 $a21_y */
  1720, /* 838 $a21_z */
  1722, /* 839 $a21_t */
  1724, /* 840 $a22_x */
  1726, /* 841 $a22_y */
  1728, /* 842 $a22_z */
  1730, /* 843 $a22_t */
  1732, /* 844 $a23_x */
  1734, /* 845 $a23_y */
  1736, /* 846 $a23_z */
  1738, /* 847 $a23_t */
  1740, /* 848 $a24_x */
  1742, /* 849 $a24_y */
  1744, /* 850 $a24_z */
  1746, /* 851 $a24_t */
  1748, /* 852 $a25_x */
  1750, /* 853 $a25_y */
  1752, /* 854 $a25_z */
  1754, /* 855 $a25_t */
  1756, /* 856 $a26_x */
  1758, /* 857 $a26_y */
  1760, /* 858 $a26_z */
  1762, /* 859 $a26_t */
  1764, /* 860 $a27_x */
  1766, /* 861 $a27_y */
  1768, /* 862 $a27_z */
  1770, /* 863 $a27_t */
  1772, /* 864 $a28_x */
  1774, /* 865 $a28_y */
  1776, /* 866 $a28_z */
  1778, /* 867 $a28_t */
  1780, /* 868 $a29_x */
  1782, /* 869 $a29_y */
  1784, /* 870 $a29_z */
  1786, /* 871 $a29_t */
  1788, /* 872 $a30_x */
  1790, /* 873 $a30_y */
  1792, /* 874 $a30_z */
  1794, /* 875 $a30_t */
  1796, /* 876 $a31_x */
  1798, /* 877 $a31_y */
  1800, /* 878 $a31_z */
  1802, /* 879 $a31_t */
  1804, /* 880 $a32_x */
  1806, /* 881 $a32_y */
  1808, /* 882 $a32_z */
  1810, /* 883 $a32_t */
  1812, /* 884 $a33_x */
  1814, /* 885 $a33_y */
  1816, /* 886 $a33_z */
  1818, /* 887 $a33_t */
  1820, /* 888 $a34_x */
  1822, /* 889 $a34_y */
  1824, /* 890 $a34_z */
  1826, /* 891 $a34_t */
  1828, /* 892 $a35_x */
  1830, /* 893 $a35_y */
  1832, /* 894 $a35_z */
  1834, /* 895 $a35_t */
  1836, /* 896 $a36_x */
  1838, /* 897 $a36_y */
  1840, /* 898 $a36_z */
  1842, /* 899 $a36_t */
  1844, /* 900 $a37_x */
  1846, /* 901 $a37_y */
  1848, /* 902 $a37_z */
  1850, /* 903 $a37_t */
  1852, /* 904 $a38_x */
  1854, /* 905 $a38_y */
  1856, /* 906 $a38_z */
  1858, /* 907 $a38_t */
  1860, /* 908 $a39_x */
  1862, /* 909 $a39_y */
  1864, /* 910 $a39_z */
  1866, /* 911 $a39_t */
  1868, /* 912 $a40_x */
  1870, /* 913 $a40_y */
  1872, /* 914 $a40_z */
  1874, /* 915 $a40_t */
  1876, /* 916 $a41_x */
  1878, /* 917 $a41_y */
  1880, /* 918 $a41_z */
  1882, /* 919 $a41_t */
  1884, /* 920 $a42_x */
  1886, /* 921 $a42_y */
  1888, /* 922 $a42_z */
  1890, /* 923 $a42_t */
  1892, /* 924 $a43_x */
  1894, /* 925 $a43_y */
  1896, /* 926 $a43_z */
  1898, /* 927 $a43_t */
  1900, /* 928 $a44_x */
  1902, /* 929 $a44_y */
  1904, /* 930 $a44_z */
  1906, /* 931 $a44_t */
  1908, /* 932 $a45_x */
  1910, /* 933 $a45_y */
  1912, /* 934 $a45_z */
  1914, /* 935 $a45_t */
  1916, /* 936 $a46_x */
  1918, /* 937 $a46_y */
  1920, /* 938 $a46_z */
  1922, /* 939 $a46_t */
  1924, /* 940 $a47_x */
  1926, /* 941 $a47_y */
  1928, /* 942 $a47_z */
  1930, /* 943 $a47_t */
  1932, /* 944 $a48_x */
  1934, /* 945 $a48_y */
  1936, /* 946 $a48_z */
  1938, /* 947 $a48_t */
  1940, /* 948 $a49_x */
  1942, /* 949 $a49_y */
  1944, /* 950 $a49_z */
  1946, /* 951 $a49_t */
  1948, /* 952 $a50_x */
  1950, /* 953 $a50_y */
  1952, /* 954 $a50_z */
  1954, /* 955 $a50_t */
  1956, /* 956 $a51_x */
  1958, /* 957 $a51_y */
  1960, /* 958 $a51_z */
  1962, /* 959 $a51_t */
  1964, /* 960 $a52_x */
  1966, /* 961 $a52_y */
  1968, /* 962 $a52_z */
  1970, /* 963 $a52_t */
  1972, /* 964 $a53_x */
  1974, /* 965 $a53_y */
  1976, /* 966 $a53_z */
  1978, /* 967 $a53_t */
  1980, /* 968 $a54_x */
  1982, /* 969 $a54_y */
  1984, /* 970 $a54_z */
  1986, /* 971 $a54_t */
  1988, /* 972 $a55_x */
  1990, /* 973 $a55_y */
  1992, /* 974 $a55_z */
  1994, /* 975 $a55_t */
  1996, /* 976 $a56_x */
  1998, /* 977 $a56_y */
  2000, /* 978 $a56_z */
  2002, /* 979 $a56_t */
  2004, /* 980 $a57_x */
  2006, /* 981 $a57_y */
  2008, /* 982 $a57_z */
  2010, /* 983 $a57_t */
  2012, /* 984 $a58_x */
  2014, /* 985 $a58_y */
  2016, /* 986 $a58_z */
  2018, /* 987 $a58_t */
  2020, /* 988 $a59_x */
  2022, /* 989 $a59_y */
  2024, /* 990 $a59_z */
  2026, /* 991 $a59_t */
  2028, /* 992 $a60_x */
  2030, /* 993 $a60_y */
  2032, /* 994 $a60_z */
  2034, /* 995 $a60_t */
  2036, /* 996 $a61_x */
  2038, /* 997 $a61_y */
  2040, /* 998 $a61_z */
  2042, /* 999 $a61_t */
  2044, /* 1000 $a62_x */
  2046, /* 1001 $a62_y */
  2048, /* 1002 $a62_z */
  2050, /* 1003 $a62_t */
  2052, /* 1004 $a63_x */
  2054, /* 1005 $a63_y */
  2056, /* 1006 $a63_z */
  2058, /* 1007 $a63_t */
  2060, /* 1008 $a0a1a2a3 */
  2061, /* 1009 $a4a5a6a7 */
  2062, /* 1010 $a8a9a10a11 */
  2063, /* 1011 $a12a13a14a15 */
  2064, /* 1012 $a16a17a18a19 */
  2065, /* 1013 $a20a21a22a23 */
  2066, /* 1014 $a24a25a26a27 */
  2067, /* 1015 $a28a29a30a31 */
  2068, /* 1016 $a32a33a34a35 */
  2069, /* 1017 $a36a37a38a39 */
  2070, /* 1018 $a40a41a42a43 */
  2071, /* 1019 $a44a45a46a47 */
  2072, /* 1020 $a48a49a50a51 */
  2073, /* 1021 $a52a53a54a55 */
  2074, /* 1022 $a56a57a58a59 */
  2075, /* 1023 $a60a61a62a63 */
  2076, /* 1024 $a0 */
  2079, /* 1025 $a1 */
  2082, /* 1026 $a2 */
  2085, /* 1027 $a3 */
  2088, /* 1028 $a4 */
  2091, /* 1029 $a5 */
  2094, /* 1030 $a6 */
  2097, /* 1031 $a7 */
  2100, /* 1032 $a8 */
  2103, /* 1033 $a9 */
  2106, /* 1034 $a10 */
  2109, /* 1035 $a11 */
  2112, /* 1036 $a12 */
  2115, /* 1037 $a13 */
  2118, /* 1038 $a14 */
  2121, /* 1039 $a15 */
  2124, /* 1040 $a16 */
  2127, /* 1041 $a17 */
  2130, /* 1042 $a18 */
  2133, /* 1043 $a19 */
  2136, /* 1044 $a20 */
  2139, /* 1045 $a21 */
  2142, /* 1046 $a22 */
  2145, /* 1047 $a23 */
  2148, /* 1048 $a24 */
  2151, /* 1049 $a25 */
  2154, /* 1050 $a26 */
  2157, /* 1051 $a27 */
  2160, /* 1052 $a28 */
  2163, /* 1053 $a29 */
  2166, /* 1054 $a30 */
  2169, /* 1055 $a31 */
  2172, /* 1056 $a32 */
  2175, /* 1057 $a33 */
  2178, /* 1058 $a34 */
  2181, /* 1059 $a35 */
  2184, /* 1060 $a36 */
  2187, /* 1061 $a37 */
  2190, /* 1062 $a38 */
  2193, /* 1063 $a39 */
  2196, /* 1064 $a40 */
  2199, /* 1065 $a41 */
  2202, /* 1066 $a42 */
  2205, /* 1067 $a43 */
  2208, /* 1068 $a44 */
  2211, /* 1069 $a45 */
  2214, /* 1070 $a46 */
  2217, /* 1071 $a47 */
  2220, /* 1072 $a48 */
  2223, /* 1073 $a49 */
  2226, /* 1074 $a50 */
  2229, /* 1075 $a51 */
  2232, /* 1076 $a52 */
  2235, /* 1077 $a53 */
  2238, /* 1078 $a54 */
  2241, /* 1079 $a55 */
  2244, /* 1080 $a56 */
  2247, /* 1081 $a57 */
  2250, /* 1082 $a58 */
  2253, /* 1083 $a59 */
  2256, /* 1084 $a60 */
  2259, /* 1085 $a61 */
  2262, /* 1086 $a62 */
  2265, /* 1087 $a63 */
  2268, /* 1088 $a0a1 */
  2270, /* 1089 $a2a3 */
  2272, /* 1090 $a4a5 */
  2274, /* 1091 $a6a7 */
  2276, /* 1092 $a8a9 */
  2278, /* 1093 $a10a11 */
  2280, /* 1094 $a12a13 */
  2282, /* 1095 $a14a15 */
  2284, /* 1096 $a16a17 */
  2286, /* 1097 $a18a19 */
  2288, /* 1098 $a20a21 */
  2290, /* 1099 $a22a23 */
  2292, /* 1100 $a24a25 */
  2294, /* 1101 $a26a27 */
  2296, /* 1102 $a28a29 */
  2298, /* 1103 $a30a31 */
  2300, /* 1104 $a32a33 */
  2302, /* 1105 $a34a35 */
  2304, /* 1106 $a36a37 */
  2306, /* 1107 $a38a39 */
  2308, /* 1108 $a40a41 */
  2310, /* 1109 $a42a43 */
  2312, /* 1110 $a44a45 */
  2314, /* 1111 $a46a47 */
  2316, /* 1112 $a48a49 */
  2318, /* 1113 $a50a51 */
  2320, /* 1114 $a52a53 */
  2322, /* 1115 $a54a55 */
  2324, /* 1116 $a56a57 */
  2326, /* 1117 $a58a59 */
  2328, /* 1118 $a60a61 */
  2330, /* 1119 $a62a63 */
};


kvx_reloc_t kv3_rel16_reloc __attribute__((unused)) = { 16, KVX_REL_ABS, 1, { R_KVX_16}};
kvx_reloc_t kv3_rel32_reloc __attribute__((unused)) = { 32, KVX_REL_ABS, 1, { R_KVX_32}};
kvx_reloc_t kv3_rel64_reloc __attribute__((unused)) = { 64, KVX_REL_ABS, 1, { R_KVX_64}};
kvx_reloc_t kv3_pcrel_signed16_reloc __attribute__((unused)) = { 16, KVX_REL_PC, 1, { R_KVX_S16_PCREL}};
kvx_reloc_t kv3_pcrel17_reloc __attribute__((unused)) = { 17, KVX_REL_PC, 1, { R_KVX_PCREL17}};
kvx_reloc_t kv3_pcrel27_reloc __attribute__((unused)) = { 27, KVX_REL_PC, 1, { R_KVX_PCREL27}};
kvx_reloc_t kv3_pcrel32_reloc __attribute__((unused)) = { 32, KVX_REL_PC, 1, { R_KVX_32_PCREL}};
kvx_reloc_t kv3_pcrel_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_PC, 2, { R_KVX_S37_PCREL_UP27, R_KVX_S37_PCREL_LO10}};
kvx_reloc_t kv3_pcrel_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_PC, 3, { R_KVX_S43_PCREL_EX6, R_KVX_S43_PCREL_UP27, R_KVX_S43_PCREL_LO10}};
kvx_reloc_t kv3_pcrel_signed64_reloc __attribute__((unused)) = { 64, KVX_REL_PC, 3, { R_KVX_S64_PCREL_EX27, R_KVX_S64_PCREL_UP27, R_KVX_S64_PCREL_LO10}};
kvx_reloc_t kv3_pcrel64_reloc __attribute__((unused)) = { 64, KVX_REL_PC, 1, { R_KVX_64_PCREL}};
kvx_reloc_t kv3_signed16_reloc __attribute__((unused)) = { 16, KVX_REL_ABS, 1, { R_KVX_S16}};
kvx_reloc_t kv3_signed32_reloc __attribute__((unused)) = { 32, KVX_REL_ABS, 2, { R_KVX_S32_UP27, R_KVX_S32_LO5}};
kvx_reloc_t kv3_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_ABS, 2, { R_KVX_S37_UP27, R_KVX_S37_LO10}};
kvx_reloc_t kv3_gotoff_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_GOT, 2, { R_KVX_S37_GOTOFF_UP27, R_KVX_S37_GOTOFF_LO10}};
kvx_reloc_t kv3_gotoff_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_GOT, 3, { R_KVX_S43_GOTOFF_EX6, R_KVX_S43_GOTOFF_UP27, R_KVX_S43_GOTOFF_LO10}};
kvx_reloc_t kv3_gotoff_32_reloc __attribute__((unused)) = { 32, KVX_REL_GOT, 1, { R_KVX_32_GOTOFF}};
kvx_reloc_t kv3_gotoff_64_reloc __attribute__((unused)) = { 64, KVX_REL_GOT, 1, { R_KVX_64_GOTOFF}};
kvx_reloc_t kv3_got_32_reloc __attribute__((unused)) = { 32, KVX_REL_GOT, 1, { R_KVX_32_GOT}};
kvx_reloc_t kv3_got_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_GOT, 2, { R_KVX_S37_GOT_UP27, R_KVX_S37_GOT_LO10}};
kvx_reloc_t kv3_got_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_GOT, 3, { R_KVX_S43_GOT_EX6, R_KVX_S43_GOT_UP27, R_KVX_S43_GOT_LO10}};
kvx_reloc_t kv3_got_64_reloc __attribute__((unused)) = { 64, KVX_REL_GOT, 1, { R_KVX_64_GOT}};
kvx_reloc_t kv3_glob_dat_reloc __attribute__((unused)) = { 32, KVX_REL_GOT, 1, { R_KVX_GLOB_DAT}};
kvx_reloc_t kv3_copy_reloc __attribute__((unused)) = { 32, KVX_REL_ABS, 1, { R_KVX_COPY}};
kvx_reloc_t kv3_jump_slot_reloc __attribute__((unused)) = { 32, KVX_REL_ABS, 1, { R_KVX_JMP_SLOT}};
kvx_reloc_t kv3_relative_reloc __attribute__((unused)) = { 32, KVX_REL_BASE, 1, { R_KVX_RELATIVE}};
kvx_reloc_t kv3_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_ABS, 3, { R_KVX_S43_EX6, R_KVX_S43_UP27, R_KVX_S43_LO10}};
kvx_reloc_t kv3_signed64_reloc __attribute__((unused)) = { 64, KVX_REL_ABS, 3, { R_KVX_S64_EX27, R_KVX_S64_UP27, R_KVX_S64_LO10}};
kvx_reloc_t kv3_gotaddr_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_PC, 2, { R_KVX_S37_GOTADDR_UP27, R_KVX_S37_GOTADDR_LO10}};
kvx_reloc_t kv3_gotaddr_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_PC, 3, { R_KVX_S43_GOTADDR_EX6, R_KVX_S43_GOTADDR_UP27, R_KVX_S43_GOTADDR_LO10}};
kvx_reloc_t kv3_gotaddr_signed64_reloc __attribute__((unused)) = { 64, KVX_REL_PC, 3, { R_KVX_S64_GOTADDR_EX27, R_KVX_S64_GOTADDR_UP27, R_KVX_S64_GOTADDR_LO10}};
kvx_reloc_t kv3_dtpmod64_reloc __attribute__((unused)) = { 64, KVX_REL_ABS, 1, { R_KVX_64_DTPMOD}};
kvx_reloc_t kv3_dtpoff64_reloc __attribute__((unused)) = { 64, KVX_REL_ABS, 1, { R_KVX_64_DTPOFF}};
kvx_reloc_t kv3_dtpoff_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_ABS, 2, { R_KVX_S37_TLS_DTPOFF_UP27, R_KVX_S37_TLS_DTPOFF_LO10}};
kvx_reloc_t kv3_dtpoff_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_ABS, 3, { R_KVX_S43_TLS_DTPOFF_EX6, R_KVX_S43_TLS_DTPOFF_UP27, R_KVX_S43_TLS_DTPOFF_LO10}};
kvx_reloc_t kv3_tlsgd_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_GOT, 2, { R_KVX_S37_TLS_GD_UP27, R_KVX_S37_TLS_GD_LO10}};
kvx_reloc_t kv3_tlsgd_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_GOT, 3, { R_KVX_S43_TLS_GD_EX6, R_KVX_S43_TLS_GD_UP27, R_KVX_S43_TLS_GD_LO10}};
kvx_reloc_t kv3_tlsld_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_GOT, 2, { R_KVX_S37_TLS_LD_UP27, R_KVX_S37_TLS_LD_LO10}};
kvx_reloc_t kv3_tlsld_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_GOT, 3, { R_KVX_S43_TLS_LD_EX6, R_KVX_S43_TLS_LD_UP27, R_KVX_S43_TLS_LD_LO10}};
kvx_reloc_t kv3_tpoff64_reloc __attribute__((unused)) = { 64, KVX_REL_TP, 1, { R_KVX_64_TPOFF}};
kvx_reloc_t kv3_tlsie_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_GOT, 2, { R_KVX_S37_TLS_IE_UP27, R_KVX_S37_TLS_IE_LO10}};
kvx_reloc_t kv3_tlsie_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_GOT, 3, { R_KVX_S43_TLS_IE_EX6, R_KVX_S43_TLS_IE_UP27, R_KVX_S43_TLS_IE_LO10}};
kvx_reloc_t kv3_tlsle_signed37_reloc __attribute__((unused)) = { 37, KVX_REL_TP, 2, { R_KVX_S37_TLS_LE_UP27, R_KVX_S37_TLS_LE_LO10}};
kvx_reloc_t kv3_tlsle_signed43_reloc __attribute__((unused)) = { 43, KVX_REL_TP, 3, { R_KVX_S43_TLS_LE_EX6, R_KVX_S43_TLS_LE_UP27, R_KVX_S43_TLS_LE_LO10}};
kvx_reloc_t kv3_rel8_reloc __attribute__((unused)) = { 8, KVX_REL_ABS, 1, { R_KVX_8}};

static kvx_reloc_t *kv3_boolcas_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_boolcas_opnd  __attribute__((unused)) = {"kv3_boolcas_opnd", Modifier_kv3_boolcas, 1, 0, 0, 0, 0, 0, 0, kv3_boolcas_opnd_relocs, 1, { { 1, 0, 25 }, } };
static kvx_reloc_t *kv3_branchcond_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_branchcond_opnd  __attribute__((unused)) = {"kv3_branchcond_opnd", Modifier_kv3_scalarcond, 4, 0, 0, 0, 0, 0, 0, kv3_branchcond_opnd_relocs, 1, { { 4, 0, 23 }, } };
static kvx_reloc_t *kv3_brknumber_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_brknumber_opnd  __attribute__((unused)) = {"kv3_brknumber_opnd", Immediate_kv3_brknumber, 2, 0, 0, 0, 0, 0, 0, kv3_brknumber_opnd_relocs, 1, { { 2, 0, 0 }, } };
static kvx_reloc_t *kv3_byteshift_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_byteshift_opnd  __attribute__((unused)) = {"kv3_byteshift_opnd", Immediate_kv3_unsigned6, 6, 0, 0, 0, 0, 0, 0, kv3_byteshift_opnd_relocs, 1, { { 6, 0, 0 }, } };
static kvx_reloc_t *kv3_cachelev_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_cachelev_opnd  __attribute__((unused)) = {"kv3_cachelev_opnd", Modifier_kv3_cachelev, 2, 0, 0, 0, 0, 0, 0, kv3_cachelev_opnd_relocs, 1, { { 2, 0, 24 }, } };
static kvx_reloc_t *kv3_channel_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_channel_opnd  __attribute__((unused)) = {"kv3_channel_opnd", Modifier_kv3_channel, 1, 0, 0, 0, 0, 0, 0, kv3_channel_opnd_relocs, 1, { { 1, 0, 24 }, } };
static kvx_reloc_t *kv3_column_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_column_opnd  __attribute__((unused)) = {"kv3_column_opnd", Modifier_kv3_column, 2, 0, 0, 0, 0, 0, 0, kv3_column_opnd_relocs, 1, { { 2, 0, 18 }, } };
static kvx_reloc_t *kv3_comparison_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_comparison_opnd  __attribute__((unused)) = {"kv3_comparison_opnd", Modifier_kv3_comparison, 4, 0, 0, 0, 0, 0, 0, kv3_comparison_opnd_relocs, 1, { { 4, 0, 24 }, } };
static kvx_reloc_t *kv3_conjugate_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_conjugate_opnd  __attribute__((unused)) = {"kv3_conjugate_opnd", Modifier_kv3_conjugate, 1, 0, 0, 0, 0, 0, 0, kv3_conjugate_opnd_relocs, 1, { { 1, 0, 24 }, } };
static kvx_reloc_t *kv3_doscale_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_doscale_opnd  __attribute__((unused)) = {"kv3_doscale_opnd", Modifier_kv3_doscale, 1, 0, 0, 0, 0, 0, 0, kv3_doscale_opnd_relocs, 1, { { 1, 0, 12 }, } };
static kvx_reloc_t *kv3_extend27_offset27_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_extend27_offset27_opnd  __attribute__((unused)) = {"kv3_extend27_offset27_opnd", Immediate_kv3_signed54, 54, 0, 0, kvxSIGNED, 0, 0, 0, kv3_extend27_offset27_opnd_relocs, 2, { { 27, 27, 64 }, { 27, 0, 32 }, } };
static kvx_reloc_t *kv3_extend27_upper27_lower10_opnd_relocs[] __attribute__((unused)) = {&kv3_pcrel_signed64_reloc, &kv3_signed64_reloc, &kv3_gotaddr_signed64_reloc, 0 };
static kvxbfield kv3_extend27_upper27_lower10_opnd  __attribute__((unused)) = {"kv3_extend27_upper27_lower10_opnd", Immediate_kv3_wrapped64, 64, 0, 0, kvxWRAPPED, 0, 0, 3, kv3_extend27_upper27_lower10_opnd_relocs, 3, { { 27, 37, 64 }, { 27, 10, 32 }, { 10, 0, 6 }, } };
static kvx_reloc_t *kv3_extend6_upper27_lower10_opnd_relocs[] __attribute__((unused)) = {&kv3_pcrel_signed43_reloc, &kv3_gotoff_signed43_reloc, &kv3_got_signed43_reloc, &kv3_signed43_reloc, &kv3_gotaddr_signed43_reloc, &kv3_dtpoff_signed43_reloc, &kv3_tlsgd_signed43_reloc, &kv3_tlsld_signed43_reloc, &kv3_tlsie_signed43_reloc, &kv3_tlsle_signed43_reloc, 0 };
static kvxbfield kv3_extend6_upper27_lower10_opnd  __attribute__((unused)) = {"kv3_extend6_upper27_lower10_opnd", Immediate_kv3_signed43, 43, 0, 0, kvxSIGNED, 0, 0, 10, kv3_extend6_upper27_lower10_opnd_relocs, 3, { { 6, 37, 0 }, { 27, 10, 32 }, { 10, 0, 6 }, } };
static kvx_reloc_t *kv3_floatcomp_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_floatcomp_opnd  __attribute__((unused)) = {"kv3_floatcomp_opnd", Modifier_kv3_floatcomp, 3, 0, 0, 0, 0, 0, 0, kv3_floatcomp_opnd_relocs, 1, { { 3, 0, 24 }, } };
static kvx_reloc_t *kv3_lsucond_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_lsucond_opnd  __attribute__((unused)) = {"kv3_lsucond_opnd", Modifier_kv3_scalarcond, 4, 0, 0, 0, 0, 0, 0, kv3_lsucond_opnd_relocs, 1, { { 4, 0, 12 }, } };
static kvx_reloc_t *kv3_multransm_multransl_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_multransm_multransl_opnd  __attribute__((unused)) = {"kv3_multransm_multransl_opnd", Modifier_kv3_transpose, 2, 0, 0, 0, 0, 0, 0, kv3_multransm_multransl_opnd_relocs, 2, { { 1, 1, 18 }, { 1, 0, 6 }, } };
static kvx_reloc_t *kv3_offset27_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_offset27_opnd  __attribute__((unused)) = {"kv3_offset27_opnd", Immediate_kv3_signed27, 27, 0, 0, kvxSIGNED, 0, 0, 0, kv3_offset27_opnd_relocs, 1, { { 27, 0, 32 }, } };
static kvx_reloc_t *kv3_pcrel17_opnd_relocs[] __attribute__((unused)) = {&kv3_pcrel17_reloc, 0 };
static kvxbfield kv3_pcrel17_opnd  __attribute__((unused)) = {"kv3_pcrel17_opnd", Immediate_kv3_pcrel17, 17, 2, 0, kvxSIGNED, 0, 0, 1, kv3_pcrel17_opnd_relocs, 1, { { 17, 0, 6 }, } };
static kvx_reloc_t *kv3_pcrel27_opnd_relocs[] __attribute__((unused)) = {&kv3_pcrel27_reloc, 0 };
static kvxbfield kv3_pcrel27_opnd  __attribute__((unused)) = {"kv3_pcrel27_opnd", Immediate_kv3_pcrel27, 27, 2, 0, kvxSIGNED, 0, 0, 1, kv3_pcrel27_opnd_relocs, 1, { { 27, 0, 0 }, } };
static kvx_reloc_t *kv3_rectify_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_rectify_opnd  __attribute__((unused)) = {"kv3_rectify_opnd", Modifier_kv3_rectify, 1, 0, 0, 0, 0, 0, 0, kv3_rectify_opnd_relocs, 1, { { 1, 0, 7 }, } };
static int kv3_registera_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registera_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera_opnd  __attribute__((unused)) = {"kv3_registera_opnd", RegClass_kv3_vectorReg, 6, 0, 0, 0, 64, kv3_registera_opnd_regs, 0, kv3_registera_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registera_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registera_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera_0_opnd  __attribute__((unused)) = {"kv3_registera_0_opnd", RegClass_kv3_vectorReg_0, 6, 2, 0, 0, 256, kv3_registera_0_opnd_regs, 0, kv3_registera_0_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registera_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registera_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera_1_opnd  __attribute__((unused)) = {"kv3_registera_1_opnd", RegClass_kv3_vectorReg_1, 6, 2, 1, 0, 256, kv3_registera_1_opnd_regs, 0, kv3_registera_1_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registera_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registera_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera_2_opnd  __attribute__((unused)) = {"kv3_registera_2_opnd", RegClass_kv3_vectorReg_2, 6, 2, 2, 0, 256, kv3_registera_2_opnd_regs, 0, kv3_registera_2_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registera_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registera_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera_3_opnd  __attribute__((unused)) = {"kv3_registera_3_opnd", RegClass_kv3_vectorReg_3, 6, 2, 3, 0, 256, kv3_registera_3_opnd_regs, 0, kv3_registera_3_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registera0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registera0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera0_opnd  __attribute__((unused)) = {"kv3_registera0_opnd", RegClass_kv3_blockReg0M4, 5, 2, 0, 0, 128, kv3_registera0_opnd_regs, 0, kv3_registera0_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registera1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registera1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera1_opnd  __attribute__((unused)) = {"kv3_registera1_opnd", RegClass_kv3_blockReg1M4, 5, 2, 1, 0, 128, kv3_registera1_opnd_regs, 0, kv3_registera1_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registera2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registera2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera2_opnd  __attribute__((unused)) = {"kv3_registera2_opnd", RegClass_kv3_blockReg2M4, 5, 2, 2, 0, 128, kv3_registera2_opnd_regs, 0, kv3_registera2_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registera3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registera3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registera3_opnd  __attribute__((unused)) = {"kv3_registera3_opnd", RegClass_kv3_blockReg3M4, 5, 2, 3, 0, 128, kv3_registera3_opnd_regs, 0, kv3_registera3_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerae_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerae_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerae_opnd  __attribute__((unused)) = {"kv3_registerae_opnd", RegClass_kv3_vectorRegE, 5, 1, 0, 0, 64, kv3_registerae_opnd_regs, 0, kv3_registerae_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerah_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerah_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerah_opnd  __attribute__((unused)) = {"kv3_registerah_opnd", RegClass_kv3_blockRegO, 6, 1, 1, 0, 128, kv3_registerah_opnd_regs, 0, kv3_registerah_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registeral_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registeral_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeral_opnd  __attribute__((unused)) = {"kv3_registeral_opnd", RegClass_kv3_blockRegE, 6, 1, 0, 0, 128, kv3_registeral_opnd_regs, 0, kv3_registeral_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registerao_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerao_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerao_opnd  __attribute__((unused)) = {"kv3_registerao_opnd", RegClass_kv3_vectorRegO, 5, 1, 1, 0, 64, kv3_registerao_opnd_regs, 0, kv3_registerao_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerap_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerap_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerap_opnd  __attribute__((unused)) = {"kv3_registerap_opnd", RegClass_kv3_wideReg, 5, 0, 0, 0, 32, kv3_registerap_opnd_regs, 0, kv3_registerap_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerap_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerap_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerap_0_opnd  __attribute__((unused)) = {"kv3_registerap_0_opnd", RegClass_kv3_wideReg_0, 5, 1, 0, 0, 64, kv3_registerap_0_opnd_regs, 0, kv3_registerap_0_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerap_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerap_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerap_1_opnd  __attribute__((unused)) = {"kv3_registerap_1_opnd", RegClass_kv3_wideReg_1, 5, 1, 1, 0, 64, kv3_registerap_1_opnd_regs, 0, kv3_registerap_1_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registeraq_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registeraq_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeraq_opnd  __attribute__((unused)) = {"kv3_registeraq_opnd", RegClass_kv3_matrixReg, 4, 0, 0, 0, 16, kv3_registeraq_opnd_regs, 0, kv3_registeraq_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registeraq_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registeraq_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeraq_0_opnd  __attribute__((unused)) = {"kv3_registeraq_0_opnd", RegClass_kv3_matrixReg_0, 4, 2, 0, 0, 64, kv3_registeraq_0_opnd_regs, 0, kv3_registeraq_0_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registeraq_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registeraq_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeraq_1_opnd  __attribute__((unused)) = {"kv3_registeraq_1_opnd", RegClass_kv3_matrixReg_1, 4, 2, 1, 0, 64, kv3_registeraq_1_opnd_regs, 0, kv3_registeraq_1_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registeraq_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registeraq_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeraq_2_opnd  __attribute__((unused)) = {"kv3_registeraq_2_opnd", RegClass_kv3_matrixReg_2, 4, 2, 2, 0, 64, kv3_registeraq_2_opnd_regs, 0, kv3_registeraq_2_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registeraq_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registeraq_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeraq_3_opnd  __attribute__((unused)) = {"kv3_registeraq_3_opnd", RegClass_kv3_matrixReg_3, 4, 2, 3, 0, 64, kv3_registeraq_3_opnd_regs, 0, kv3_registeraq_3_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registerat_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerat_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerat_opnd  __attribute__((unused)) = {"kv3_registerat_opnd", RegClass_kv3_coproReg3M4, 6, 2, 3, 0, 256, kv3_registerat_opnd_regs, 0, kv3_registerat_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registerax_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerax_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerax_opnd  __attribute__((unused)) = {"kv3_registerax_opnd", RegClass_kv3_coproReg0M4, 6, 2, 0, 0, 256, kv3_registerax_opnd_regs, 0, kv3_registerax_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registeray_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registeray_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeray_opnd  __attribute__((unused)) = {"kv3_registeray_opnd", RegClass_kv3_coproReg1M4, 6, 2, 1, 0, 256, kv3_registeray_opnd_regs, 0, kv3_registeray_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registeraz_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registeraz_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeraz_opnd  __attribute__((unused)) = {"kv3_registeraz_opnd", RegClass_kv3_coproReg2M4, 6, 2, 2, 0, 256, kv3_registeraz_opnd_regs, 0, kv3_registeraz_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registerb_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerb_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerb_opnd  __attribute__((unused)) = {"kv3_registerb_opnd", RegClass_kv3_vectorReg, 6, 0, 0, 0, 64, kv3_registerb_opnd_regs, 0, kv3_registerb_opnd_relocs, 1, { { 6, 0, 12 }, } };
static int kv3_registerb_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerb_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerb_0_opnd  __attribute__((unused)) = {"kv3_registerb_0_opnd", RegClass_kv3_vectorReg_0, 6, 2, 0, 0, 256, kv3_registerb_0_opnd_regs, 0, kv3_registerb_0_opnd_relocs, 1, { { 6, 0, 12 }, } };
static int kv3_registerb_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerb_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerb_1_opnd  __attribute__((unused)) = {"kv3_registerb_1_opnd", RegClass_kv3_vectorReg_1, 6, 2, 1, 0, 256, kv3_registerb_1_opnd_regs, 0, kv3_registerb_1_opnd_relocs, 1, { { 6, 0, 12 }, } };
static int kv3_registerb_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerb_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerb_2_opnd  __attribute__((unused)) = {"kv3_registerb_2_opnd", RegClass_kv3_vectorReg_2, 6, 2, 2, 0, 256, kv3_registerb_2_opnd_regs, 0, kv3_registerb_2_opnd_relocs, 1, { { 6, 0, 12 }, } };
static int kv3_registerb_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerb_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerb_3_opnd  __attribute__((unused)) = {"kv3_registerb_3_opnd", RegClass_kv3_vectorReg_3, 6, 2, 3, 0, 256, kv3_registerb_3_opnd_regs, 0, kv3_registerb_3_opnd_relocs, 1, { { 6, 0, 12 }, } };
static int kv3_registerbe_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerbe_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbe_opnd  __attribute__((unused)) = {"kv3_registerbe_opnd", RegClass_kv3_vectorRegE, 5, 1, 0, 0, 64, kv3_registerbe_opnd_regs, 0, kv3_registerbe_opnd_relocs, 1, { { 5, 0, 13 }, } };
static int kv3_registerbo_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerbo_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbo_opnd  __attribute__((unused)) = {"kv3_registerbo_opnd", RegClass_kv3_vectorRegO, 5, 1, 1, 0, 64, kv3_registerbo_opnd_regs, 0, kv3_registerbo_opnd_relocs, 1, { { 5, 0, 13 }, } };
static int kv3_registerbp_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerbp_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbp_opnd  __attribute__((unused)) = {"kv3_registerbp_opnd", RegClass_kv3_wideReg, 5, 0, 0, 0, 32, kv3_registerbp_opnd_regs, 0, kv3_registerbp_opnd_relocs, 1, { { 5, 0, 13 }, } };
static int kv3_registerbp_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerbp_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbp_0_opnd  __attribute__((unused)) = {"kv3_registerbp_0_opnd", RegClass_kv3_wideReg_0, 5, 1, 0, 0, 64, kv3_registerbp_0_opnd_regs, 0, kv3_registerbp_0_opnd_relocs, 1, { { 5, 0, 13 }, } };
static int kv3_registerbp_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerbp_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbp_1_opnd  __attribute__((unused)) = {"kv3_registerbp_1_opnd", RegClass_kv3_wideReg_1, 5, 1, 1, 0, 64, kv3_registerbp_1_opnd_regs, 0, kv3_registerbp_1_opnd_relocs, 1, { { 5, 0, 13 }, } };
static int kv3_registerbq_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerbq_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbq_opnd  __attribute__((unused)) = {"kv3_registerbq_opnd", RegClass_kv3_matrixReg, 4, 0, 0, 0, 16, kv3_registerbq_opnd_regs, 0, kv3_registerbq_opnd_relocs, 1, { { 4, 0, 14 }, } };
static int kv3_registerbq_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerbq_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbq_0_opnd  __attribute__((unused)) = {"kv3_registerbq_0_opnd", RegClass_kv3_matrixReg_0, 4, 2, 0, 0, 64, kv3_registerbq_0_opnd_regs, 0, kv3_registerbq_0_opnd_relocs, 1, { { 4, 0, 14 }, } };
static int kv3_registerbq_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerbq_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbq_1_opnd  __attribute__((unused)) = {"kv3_registerbq_1_opnd", RegClass_kv3_matrixReg_1, 4, 2, 1, 0, 64, kv3_registerbq_1_opnd_regs, 0, kv3_registerbq_1_opnd_relocs, 1, { { 4, 0, 14 }, } };
static int kv3_registerbq_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerbq_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbq_2_opnd  __attribute__((unused)) = {"kv3_registerbq_2_opnd", RegClass_kv3_matrixReg_2, 4, 2, 2, 0, 64, kv3_registerbq_2_opnd_regs, 0, kv3_registerbq_2_opnd_relocs, 1, { { 4, 0, 14 }, } };
static int kv3_registerbq_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerbq_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerbq_3_opnd  __attribute__((unused)) = {"kv3_registerbq_3_opnd", RegClass_kv3_matrixReg_3, 4, 2, 3, 0, 64, kv3_registerbq_3_opnd_regs, 0, kv3_registerbq_3_opnd_relocs, 1, { { 4, 0, 14 }, } };
static int kv3_registerc_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerc_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerc_opnd  __attribute__((unused)) = {"kv3_registerc_opnd", RegClass_kv3_vectorReg, 6, 0, 0, 0, 64, kv3_registerc_opnd_regs, 0, kv3_registerc_opnd_relocs, 1, { { 6, 0, 6 }, } };
static int kv3_registerc_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerc_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerc_0_opnd  __attribute__((unused)) = {"kv3_registerc_0_opnd", RegClass_kv3_vectorReg_0, 6, 2, 0, 0, 256, kv3_registerc_0_opnd_regs, 0, kv3_registerc_0_opnd_relocs, 1, { { 6, 0, 6 }, } };
static int kv3_registerc_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerc_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerc_1_opnd  __attribute__((unused)) = {"kv3_registerc_1_opnd", RegClass_kv3_vectorReg_1, 6, 2, 1, 0, 256, kv3_registerc_1_opnd_regs, 0, kv3_registerc_1_opnd_relocs, 1, { { 6, 0, 6 }, } };
static int kv3_registerc_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerc_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerc_2_opnd  __attribute__((unused)) = {"kv3_registerc_2_opnd", RegClass_kv3_vectorReg_2, 6, 2, 2, 0, 256, kv3_registerc_2_opnd_regs, 0, kv3_registerc_2_opnd_relocs, 1, { { 6, 0, 6 }, } };
static int kv3_registerc_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerc_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerc_3_opnd  __attribute__((unused)) = {"kv3_registerc_3_opnd", RegClass_kv3_vectorReg_3, 6, 2, 3, 0, 256, kv3_registerc_3_opnd_regs, 0, kv3_registerc_3_opnd_relocs, 1, { { 6, 0, 6 }, } };
static int kv3_registerce_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerce_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerce_opnd  __attribute__((unused)) = {"kv3_registerce_opnd", RegClass_kv3_vectorRegE, 5, 1, 0, 0, 64, kv3_registerce_opnd_regs, 0, kv3_registerce_opnd_relocs, 1, { { 5, 0, 7 }, } };
static int kv3_registerco_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerco_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerco_opnd  __attribute__((unused)) = {"kv3_registerco_opnd", RegClass_kv3_vectorRegO, 5, 1, 1, 0, 64, kv3_registerco_opnd_regs, 0, kv3_registerco_opnd_relocs, 1, { { 5, 0, 7 }, } };
static int kv3_registerd_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerd_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerd_opnd  __attribute__((unused)) = {"kv3_registerd_opnd", RegClass_kv3_vectorReg, 6, 0, 0, 0, 64, kv3_registerd_opnd_regs, 0, kv3_registerd_opnd_relocs, 1, { { 6, 0, 0 }, } };
static int kv3_registerd_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerd_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerd_0_opnd  __attribute__((unused)) = {"kv3_registerd_0_opnd", RegClass_kv3_vectorReg_0, 6, 2, 0, 0, 256, kv3_registerd_0_opnd_regs, 0, kv3_registerd_0_opnd_relocs, 1, { { 6, 0, 0 }, } };
static int kv3_registerd_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerd_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerd_1_opnd  __attribute__((unused)) = {"kv3_registerd_1_opnd", RegClass_kv3_vectorReg_1, 6, 2, 1, 0, 256, kv3_registerd_1_opnd_regs, 0, kv3_registerd_1_opnd_relocs, 1, { { 6, 0, 0 }, } };
static int kv3_registerd_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerd_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerd_2_opnd  __attribute__((unused)) = {"kv3_registerd_2_opnd", RegClass_kv3_vectorReg_2, 6, 2, 2, 0, 256, kv3_registerd_2_opnd_regs, 0, kv3_registerd_2_opnd_relocs, 1, { { 6, 0, 0 }, } };
static int kv3_registerd_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerd_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerd_3_opnd  __attribute__((unused)) = {"kv3_registerd_3_opnd", RegClass_kv3_vectorReg_3, 6, 2, 3, 0, 256, kv3_registerd_3_opnd_regs, 0, kv3_registerd_3_opnd_relocs, 1, { { 6, 0, 0 }, } };
static int kv3_registerdp_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerdp_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerdp_opnd  __attribute__((unused)) = {"kv3_registerdp_opnd", RegClass_kv3_wideReg, 5, 0, 0, 0, 32, kv3_registerdp_opnd_regs, 0, kv3_registerdp_opnd_relocs, 1, { { 5, 0, 1 }, } };
static int kv3_registerdp_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerdp_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerdp_0_opnd  __attribute__((unused)) = {"kv3_registerdp_0_opnd", RegClass_kv3_wideReg_0, 5, 1, 0, 0, 64, kv3_registerdp_0_opnd_regs, 0, kv3_registerdp_0_opnd_relocs, 1, { { 5, 0, 1 }, } };
static int kv3_registerdp_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerdp_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerdp_1_opnd  __attribute__((unused)) = {"kv3_registerdp_1_opnd", RegClass_kv3_wideReg_1, 5, 1, 1, 0, 64, kv3_registerdp_1_opnd_regs, 0, kv3_registerdp_1_opnd_relocs, 1, { { 5, 0, 1 }, } };
static int kv3_registere_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registere_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registere_opnd  __attribute__((unused)) = {"kv3_registere_opnd", RegClass_kv3_vectorReg, 6, 0, 0, 0, 64, kv3_registere_opnd_regs, 0, kv3_registere_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registere_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registere_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registere_0_opnd  __attribute__((unused)) = {"kv3_registere_0_opnd", RegClass_kv3_vectorReg_0, 6, 2, 0, 0, 256, kv3_registere_0_opnd_regs, 0, kv3_registere_0_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registere_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registere_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registere_1_opnd  __attribute__((unused)) = {"kv3_registere_1_opnd", RegClass_kv3_vectorReg_1, 6, 2, 1, 0, 256, kv3_registere_1_opnd_regs, 0, kv3_registere_1_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registere_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registere_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registere_2_opnd  __attribute__((unused)) = {"kv3_registere_2_opnd", RegClass_kv3_vectorReg_2, 6, 2, 2, 0, 256, kv3_registere_2_opnd_regs, 0, kv3_registere_2_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registere_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registere_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registere_3_opnd  __attribute__((unused)) = {"kv3_registere_3_opnd", RegClass_kv3_vectorReg_3, 6, 2, 3, 0, 256, kv3_registere_3_opnd_regs, 0, kv3_registere_3_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registerm_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerm_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerm_opnd  __attribute__((unused)) = {"kv3_registerm_opnd", RegClass_kv3_pairedReg, 5, 0, 0, 0, 32, kv3_registerm_opnd_regs, 0, kv3_registerm_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerm_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerm_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerm_0_opnd  __attribute__((unused)) = {"kv3_registerm_0_opnd", RegClass_kv3_pairedReg_0, 5, 1, 0, 0, 64, kv3_registerm_0_opnd_regs, 0, kv3_registerm_0_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerm_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerm_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerm_1_opnd  __attribute__((unused)) = {"kv3_registerm_1_opnd", RegClass_kv3_pairedReg_1, 5, 1, 1, 0, 64, kv3_registerm_1_opnd_regs, 0, kv3_registerm_1_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registern_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registern_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registern_opnd  __attribute__((unused)) = {"kv3_registern_opnd", RegClass_kv3_quadReg, 4, 0, 0, 0, 16, kv3_registern_opnd_regs, 0, kv3_registern_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registern_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registern_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registern_0_opnd  __attribute__((unused)) = {"kv3_registern_0_opnd", RegClass_kv3_quadReg_0, 4, 2, 0, 0, 64, kv3_registern_0_opnd_regs, 0, kv3_registern_0_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registern_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registern_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registern_1_opnd  __attribute__((unused)) = {"kv3_registern_1_opnd", RegClass_kv3_quadReg_1, 4, 2, 1, 0, 64, kv3_registern_1_opnd_regs, 0, kv3_registern_1_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registern_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registern_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registern_2_opnd  __attribute__((unused)) = {"kv3_registern_2_opnd", RegClass_kv3_quadReg_2, 4, 2, 2, 0, 64, kv3_registern_2_opnd_regs, 0, kv3_registern_2_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registern_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registern_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registern_3_opnd  __attribute__((unused)) = {"kv3_registern_3_opnd", RegClass_kv3_quadReg_3, 4, 2, 3, 0, 64, kv3_registern_3_opnd_regs, 0, kv3_registern_3_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registero_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registero_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registero_opnd  __attribute__((unused)) = {"kv3_registero_opnd", RegClass_kv3_pairedReg, 5, 0, 0, 0, 32, kv3_registero_opnd_regs, 0, kv3_registero_opnd_relocs, 1, { { 5, 0, 7 }, } };
static int kv3_registero_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registero_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registero_0_opnd  __attribute__((unused)) = {"kv3_registero_0_opnd", RegClass_kv3_pairedReg_0, 5, 1, 0, 0, 64, kv3_registero_0_opnd_regs, 0, kv3_registero_0_opnd_relocs, 1, { { 5, 0, 7 }, } };
static int kv3_registero_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registero_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registero_1_opnd  __attribute__((unused)) = {"kv3_registero_1_opnd", RegClass_kv3_pairedReg_1, 5, 1, 1, 0, 64, kv3_registero_1_opnd_regs, 0, kv3_registero_1_opnd_relocs, 1, { { 5, 0, 7 }, } };
static int kv3_registerp_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerp_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerp_opnd  __attribute__((unused)) = {"kv3_registerp_opnd", RegClass_kv3_pairedReg, 5, 0, 0, 0, 32, kv3_registerp_opnd_regs, 0, kv3_registerp_opnd_relocs, 1, { { 5, 0, 1 }, } };
static int kv3_registerp_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registerp_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerp_0_opnd  __attribute__((unused)) = {"kv3_registerp_0_opnd", RegClass_kv3_pairedReg_0, 5, 1, 0, 0, 64, kv3_registerp_0_opnd_regs, 0, kv3_registerp_0_opnd_relocs, 1, { { 5, 0, 1 }, } };
static int kv3_registerp_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registerp_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerp_1_opnd  __attribute__((unused)) = {"kv3_registerp_1_opnd", RegClass_kv3_pairedReg_1, 5, 1, 1, 0, 64, kv3_registerp_1_opnd_regs, 0, kv3_registerp_1_opnd_relocs, 1, { { 5, 0, 1 }, } };
static int kv3_registerq_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerq_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerq_opnd  __attribute__((unused)) = {"kv3_registerq_opnd", RegClass_kv3_quadReg, 4, 0, 0, 0, 16, kv3_registerq_opnd_regs, 0, kv3_registerq_opnd_relocs, 1, { { 4, 0, 8 }, } };
static int kv3_registerq_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerq_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerq_0_opnd  __attribute__((unused)) = {"kv3_registerq_0_opnd", RegClass_kv3_quadReg_0, 4, 2, 0, 0, 64, kv3_registerq_0_opnd_regs, 0, kv3_registerq_0_opnd_relocs, 1, { { 4, 0, 8 }, } };
static int kv3_registerq_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerq_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerq_1_opnd  __attribute__((unused)) = {"kv3_registerq_1_opnd", RegClass_kv3_quadReg_1, 4, 2, 1, 0, 64, kv3_registerq_1_opnd_regs, 0, kv3_registerq_1_opnd_relocs, 1, { { 4, 0, 8 }, } };
static int kv3_registerq_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerq_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerq_2_opnd  __attribute__((unused)) = {"kv3_registerq_2_opnd", RegClass_kv3_quadReg_2, 4, 2, 2, 0, 64, kv3_registerq_2_opnd_regs, 0, kv3_registerq_2_opnd_relocs, 1, { { 4, 0, 8 }, } };
static int kv3_registerq_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerq_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerq_3_opnd  __attribute__((unused)) = {"kv3_registerq_3_opnd", RegClass_kv3_quadReg_3, 4, 2, 3, 0, 64, kv3_registerq_3_opnd_regs, 0, kv3_registerq_3_opnd_relocs, 1, { { 4, 0, 8 }, } };
static int kv3_registerr_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerr_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerr_opnd  __attribute__((unused)) = {"kv3_registerr_opnd", RegClass_kv3_quadReg, 4, 0, 0, 0, 16, kv3_registerr_opnd_regs, 0, kv3_registerr_opnd_relocs, 1, { { 4, 0, 2 }, } };
static int kv3_registerr_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerr_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerr_0_opnd  __attribute__((unused)) = {"kv3_registerr_0_opnd", RegClass_kv3_quadReg_0, 4, 2, 0, 0, 64, kv3_registerr_0_opnd_regs, 0, kv3_registerr_0_opnd_relocs, 1, { { 4, 0, 2 }, } };
static int kv3_registerr_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerr_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerr_1_opnd  __attribute__((unused)) = {"kv3_registerr_1_opnd", RegClass_kv3_quadReg_1, 4, 2, 1, 0, 64, kv3_registerr_1_opnd_regs, 0, kv3_registerr_1_opnd_relocs, 1, { { 4, 0, 2 }, } };
static int kv3_registerr_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerr_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerr_2_opnd  __attribute__((unused)) = {"kv3_registerr_2_opnd", RegClass_kv3_quadReg_2, 4, 2, 2, 0, 64, kv3_registerr_2_opnd_regs, 0, kv3_registerr_2_opnd_relocs, 1, { { 4, 0, 2 }, } };
static int kv3_registerr_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerr_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerr_3_opnd  __attribute__((unused)) = {"kv3_registerr_3_opnd", RegClass_kv3_quadReg_3, 4, 2, 3, 0, 64, kv3_registerr_3_opnd_regs, 0, kv3_registerr_3_opnd_relocs, 1, { { 4, 0, 2 }, } };
static int kv3_registert_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registert_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registert_opnd  __attribute__((unused)) = {"kv3_registert_opnd", RegClass_kv3_singleReg, 6, 0, 0, 0, 64, kv3_registert_opnd_regs, 0, kv3_registert_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registeru_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registeru_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeru_opnd  __attribute__((unused)) = {"kv3_registeru_opnd", RegClass_kv3_pairedReg, 5, 0, 0, 0, 32, kv3_registeru_opnd_regs, 0, kv3_registeru_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registeru_0_opnd_regs[] = {
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 
      1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, };
static kvx_reloc_t *kv3_registeru_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeru_0_opnd  __attribute__((unused)) = {"kv3_registeru_0_opnd", RegClass_kv3_pairedReg_0, 5, 1, 0, 0, 64, kv3_registeru_0_opnd_regs, 0, kv3_registeru_0_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registeru_1_opnd_regs[] = {
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 
      0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, };
static kvx_reloc_t *kv3_registeru_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registeru_1_opnd  __attribute__((unused)) = {"kv3_registeru_1_opnd", RegClass_kv3_pairedReg_1, 5, 1, 1, 0, 64, kv3_registeru_1_opnd_regs, 0, kv3_registeru_1_opnd_relocs, 1, { { 5, 0, 19 }, } };
static int kv3_registerv_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerv_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerv_opnd  __attribute__((unused)) = {"kv3_registerv_opnd", RegClass_kv3_quadReg, 4, 0, 0, 0, 16, kv3_registerv_opnd_regs, 0, kv3_registerv_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registerv_0_opnd_regs[] = {
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, };
static kvx_reloc_t *kv3_registerv_0_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerv_0_opnd  __attribute__((unused)) = {"kv3_registerv_0_opnd", RegClass_kv3_quadReg_0, 4, 2, 0, 0, 64, kv3_registerv_0_opnd_regs, 0, kv3_registerv_0_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registerv_1_opnd_regs[] = {
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
      0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, };
static kvx_reloc_t *kv3_registerv_1_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerv_1_opnd  __attribute__((unused)) = {"kv3_registerv_1_opnd", RegClass_kv3_quadReg_1, 4, 2, 1, 0, 64, kv3_registerv_1_opnd_regs, 0, kv3_registerv_1_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registerv_2_opnd_regs[] = {
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 
      0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, };
static kvx_reloc_t *kv3_registerv_2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerv_2_opnd  __attribute__((unused)) = {"kv3_registerv_2_opnd", RegClass_kv3_quadReg_2, 4, 2, 2, 0, 64, kv3_registerv_2_opnd_regs, 0, kv3_registerv_2_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registerv_3_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 
      0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, };
static kvx_reloc_t *kv3_registerv_3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerv_3_opnd  __attribute__((unused)) = {"kv3_registerv_3_opnd", RegClass_kv3_quadReg_3, 4, 2, 3, 0, 64, kv3_registerv_3_opnd_regs, 0, kv3_registerv_3_opnd_relocs, 1, { { 4, 0, 20 }, } };
static int kv3_registerw_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerw_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerw_opnd  __attribute__((unused)) = {"kv3_registerw_opnd", RegClass_kv3_singleReg, 6, 0, 0, 0, 64, kv3_registerw_opnd_regs, 0, kv3_registerw_opnd_relocs, 1, { { 6, 0, 18 }, } };
static int kv3_registery_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registery_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registery_opnd  __attribute__((unused)) = {"kv3_registery_opnd", RegClass_kv3_singleReg, 6, 0, 0, 0, 64, kv3_registery_opnd_regs, 0, kv3_registery_opnd_relocs, 1, { { 6, 0, 6 }, } };
static int kv3_registerz_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_registerz_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_registerz_opnd  __attribute__((unused)) = {"kv3_registerz_opnd", RegClass_kv3_singleReg, 6, 0, 0, 0, 64, kv3_registerz_opnd_regs, 0, kv3_registerz_opnd_relocs, 1, { { 6, 0, 0 }, } };
static kvx_reloc_t *kv3_rounding_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_rounding_opnd  __attribute__((unused)) = {"kv3_rounding_opnd", Modifier_kv3_rounding, 3, 0, 0, 0, 0, 0, 0, kv3_rounding_opnd_relocs, 1, { { 3, 0, 12 }, } };
static kvx_reloc_t *kv3_rounding2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_rounding2_opnd  __attribute__((unused)) = {"kv3_rounding2_opnd", Modifier_kv3_rounding, 3, 0, 0, 0, 0, 0, 0, kv3_rounding2_opnd_relocs, 1, { { 3, 0, 8 }, } };
static kvx_reloc_t *kv3_roundint_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_roundint_opnd  __attribute__((unused)) = {"kv3_roundint_opnd", Modifier_kv3_roundint, 3, 0, 0, 0, 0, 0, 0, kv3_roundint_opnd_relocs, 1, { { 3, 0, 8 }, } };
static kvx_reloc_t *kv3_saturate_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_saturate_opnd  __attribute__((unused)) = {"kv3_saturate_opnd", Modifier_kv3_saturate, 1, 0, 0, 0, 0, 0, 0, kv3_saturate_opnd_relocs, 1, { { 1, 0, 7 }, } };
static kvx_reloc_t *kv3_scalarcond_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_scalarcond_opnd  __attribute__((unused)) = {"kv3_scalarcond_opnd", Modifier_kv3_scalarcond, 4, 0, 0, 0, 0, 0, 0, kv3_scalarcond_opnd_relocs, 1, { { 4, 0, 24 }, } };
static kvx_reloc_t *kv3_signed10_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_signed10_opnd  __attribute__((unused)) = {"kv3_signed10_opnd", Immediate_kv3_signed10, 10, 0, 0, kvxSIGNED, 0, 0, 0, kv3_signed10_opnd_relocs, 1, { { 10, 0, 6 }, } };
static kvx_reloc_t *kv3_signed16_opnd_relocs[] __attribute__((unused)) = {&kv3_pcrel_signed16_reloc, &kv3_signed16_reloc, 0 };
static kvxbfield kv3_signed16_opnd  __attribute__((unused)) = {"kv3_signed16_opnd", Immediate_kv3_signed16, 16, 0, 0, kvxSIGNED, 0, 0, 2, kv3_signed16_opnd_relocs, 1, { { 16, 0, 0 }, } };
static kvx_reloc_t *kv3_silent_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_silent_opnd  __attribute__((unused)) = {"kv3_silent_opnd", Modifier_kv3_silent, 1, 0, 0, 0, 0, 0, 0, kv3_silent_opnd_relocs, 1, { { 1, 0, 15 }, } };
static kvx_reloc_t *kv3_silent2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_silent2_opnd  __attribute__((unused)) = {"kv3_silent2_opnd", Modifier_kv3_silent, 1, 0, 0, 0, 0, 0, 0, kv3_silent2_opnd_relocs, 1, { { 1, 0, 11 }, } };
static kvx_reloc_t *kv3_simdcond_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_simdcond_opnd  __attribute__((unused)) = {"kv3_simdcond_opnd", Modifier_kv3_simplecond, 3, 0, 0, 0, 0, 0, 0, kv3_simdcond_opnd_relocs, 1, { { 3, 0, 24 }, } };
static kvx_reloc_t *kv3_speculate_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_speculate_opnd  __attribute__((unused)) = {"kv3_speculate_opnd", Modifier_kv3_speculate, 1, 0, 0, 0, 0, 0, 0, kv3_speculate_opnd_relocs, 1, { { 1, 0, 24 }, } };
static kvx_reloc_t *kv3_splat32_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_splat32_opnd  __attribute__((unused)) = {"kv3_splat32_opnd", Modifier_kv3_splat32, 1, 0, 0, 0, 0, 0, 0, kv3_splat32_opnd_relocs, 1, { { 1, 0, 11 }, } };
static kvx_reloc_t *kv3_startbit_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_startbit_opnd  __attribute__((unused)) = {"kv3_startbit_opnd", Immediate_kv3_unsigned6, 6, 0, 0, 0, 0, 0, 0, kv3_startbit_opnd_relocs, 1, { { 6, 0, 6 }, } };
static kvx_reloc_t *kv3_stopbit2_stopbit4_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_stopbit2_stopbit4_opnd  __attribute__((unused)) = {"kv3_stopbit2_stopbit4_opnd", Immediate_kv3_unsigned6, 6, 0, 0, 0, 0, 0, 0, kv3_stopbit2_stopbit4_opnd_relocs, 2, { { 2, 4, 24 }, { 4, 0, 12 }, } };
static kvx_reloc_t *kv3_sysnumber_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_sysnumber_opnd  __attribute__((unused)) = {"kv3_sysnumber_opnd", Immediate_kv3_sysnumber, 12, 0, 0, 0, 0, 0, 0, kv3_sysnumber_opnd_relocs, 1, { { 12, 0, 0 }, } };
static int kv3_systemalone_opnd_regs[] = {
      0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
static kvx_reloc_t *kv3_systemalone_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systemalone_opnd  __attribute__((unused)) = {"kv3_systemalone_opnd", RegClass_kv3_aloneReg, 4, 0, 0, 0, 512, kv3_systemalone_opnd_regs, 0, kv3_systemalone_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systemalone_opnd_regs[] = {
      0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
static kvx_reloc_t *v2_systemalone_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systemalone_opnd  __attribute__((unused)) = {"v2_systemalone_opnd", RegClass_v2_aloneReg, 4, 0, 0, 0, 512, v2_systemalone_opnd_regs, 0, v2_systemalone_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systemra_opnd_regs[] = {
      0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
static kvx_reloc_t *kv3_systemra_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systemra_opnd  __attribute__((unused)) = {"kv3_systemra_opnd", RegClass_kv3_onlyraReg, 1, 0, 0, 0, 512, kv3_systemra_opnd_regs, 0, kv3_systemra_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systems2_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_systems2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systems2_opnd  __attribute__((unused)) = {"kv3_systems2_opnd", RegClass_kv3_onlygetReg, 9, 0, 0, 0, 512, kv3_systems2_opnd_regs, 0, kv3_systems2_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systems2_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *v2_systems2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systems2_opnd  __attribute__((unused)) = {"v2_systems2_opnd", RegClass_v2_onlygetReg, 9, 0, 0, 0, 512, v2_systems2_opnd_regs, 0, v2_systems2_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systems3_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_systems3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systems3_opnd  __attribute__((unused)) = {"kv3_systems3_opnd", RegClass_kv3_systemReg, 9, 0, 0, 0, 512, kv3_systems3_opnd_regs, 0, kv3_systems3_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systems3_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *v2_systems3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systems3_opnd  __attribute__((unused)) = {"v2_systems3_opnd", RegClass_v2_systemReg, 9, 0, 0, 0, 512, v2_systems3_opnd_regs, 0, v2_systems3_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systems4_opnd_regs[] = {
      1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_systems4_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systems4_opnd  __attribute__((unused)) = {"kv3_systems4_opnd", RegClass_kv3_onlyswapReg, 9, 0, 0, 0, 512, kv3_systems4_opnd_regs, 0, kv3_systems4_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systems4_opnd_regs[] = {
      1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *v2_systems4_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systems4_opnd  __attribute__((unused)) = {"v2_systems4_opnd", RegClass_v2_onlyswapReg, 9, 0, 0, 0, 512, v2_systems4_opnd_regs, 0, v2_systems4_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systemt2_opnd_regs[] = {
      0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 
      0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
static kvx_reloc_t *kv3_systemt2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systemt2_opnd  __attribute__((unused)) = {"kv3_systemt2_opnd", RegClass_kv3_onlyfxReg, 5, 0, 0, 0, 512, kv3_systemt2_opnd_regs, 0, kv3_systemt2_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systemt2_opnd_regs[] = {
      0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 
      0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 
      1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
static kvx_reloc_t *v2_systemt2_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systemt2_opnd  __attribute__((unused)) = {"v2_systemt2_opnd", RegClass_v2_onlyfxReg, 5, 0, 0, 0, 512, v2_systemt2_opnd_regs, 0, v2_systemt2_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systemt3_opnd_regs[] = {
      0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_systemt3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systemt3_opnd  __attribute__((unused)) = {"kv3_systemt3_opnd", RegClass_kv3_onlysetReg, 9, 0, 0, 0, 512, kv3_systemt3_opnd_regs, 0, kv3_systemt3_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systemt3_opnd_regs[] = {
      0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *v2_systemt3_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systemt3_opnd  __attribute__((unused)) = {"v2_systemt3_opnd", RegClass_v2_onlysetReg, 9, 0, 0, 0, 512, v2_systemt3_opnd_regs, 0, v2_systemt3_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int kv3_systemt4_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *kv3_systemt4_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_systemt4_opnd  __attribute__((unused)) = {"kv3_systemt4_opnd", RegClass_kv3_systemReg, 9, 0, 0, 0, 512, kv3_systemt4_opnd_regs, 0, kv3_systemt4_opnd_relocs, 1, { { 9, 0, 6 }, } };
static int v2_systemt4_opnd_regs[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, };
static kvx_reloc_t *v2_systemt4_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield v2_systemt4_opnd  __attribute__((unused)) = {"v2_systemt4_opnd", RegClass_v2_systemReg, 9, 0, 0, 0, 512, v2_systemt4_opnd_regs, 0, v2_systemt4_opnd_relocs, 1, { { 9, 0, 6 }, } };
static kvx_reloc_t *kv3_unsigned6_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_unsigned6_opnd  __attribute__((unused)) = {"kv3_unsigned6_opnd", Immediate_kv3_unsigned6, 6, 0, 0, 0, 0, 0, 0, kv3_unsigned6_opnd_relocs, 1, { { 6, 0, 6 }, } };
static kvx_reloc_t *kv3_upper27_lower10_opnd_relocs[] __attribute__((unused)) = {&kv3_pcrel_signed37_reloc, &kv3_signed37_reloc, &kv3_gotoff_signed37_reloc, &kv3_got_signed37_reloc, &kv3_gotaddr_signed37_reloc, &kv3_dtpoff_signed37_reloc, &kv3_tlsgd_signed37_reloc, &kv3_tlsld_signed37_reloc, &kv3_tlsie_signed37_reloc, &kv3_tlsle_signed37_reloc, 0 };
static kvxbfield kv3_upper27_lower10_opnd  __attribute__((unused)) = {"kv3_upper27_lower10_opnd", Immediate_kv3_signed37, 37, 0, 0, kvxSIGNED, 0, 0, 10, kv3_upper27_lower10_opnd_relocs, 2, { { 27, 10, 32 }, { 10, 0, 6 }, } };
static kvx_reloc_t *kv3_upper27_lower5_opnd_relocs[] __attribute__((unused)) = {&kv3_signed32_reloc, 0 };
static kvxbfield kv3_upper27_lower5_opnd  __attribute__((unused)) = {"kv3_upper27_lower5_opnd", Immediate_kv3_wrapped32, 32, 0, 0, kvxWRAPPED, 0, 0, 1, kv3_upper27_lower5_opnd_relocs, 2, { { 27, 5, 32 }, { 5, 0, 6 }, } };
static kvx_reloc_t *kv3_variant_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_variant_opnd  __attribute__((unused)) = {"kv3_variant_opnd", Modifier_kv3_variant, 2, 0, 0, 0, 0, 0, 0, kv3_variant_opnd_relocs, 1, { { 2, 0, 24 }, } };
static kvx_reloc_t *kv3_xrounding_opnd_relocs[] __attribute__((unused)) = {0 };
static kvxbfield kv3_xrounding_opnd  __attribute__((unused)) = {"kv3_xrounding_opnd", Modifier_kv3_rounding, 3, 0, 0, 0, 0, 0, 0, kv3_xrounding_opnd_relocs, 1, { { 3, 0, 8 }, } };

const int kv3_resource_max = RESOURCE_MAX;

const char *kv3_resource_names[RESOURCE_MAX] = {
  "ISSUE",
  "TINY",
  "LITE",
  "FULL",
  "LSU",
  "MAU",
  "BCU",
  "TCA",
  "AUXR",
  "AUXW",
  "CRRP",
  "CRWL",
  "CRWH",
  "NOP"
};

const int kv3_v1_resources[RESOURCE_MAX] = { 8,4,2,1,1,1,1,1,1,1,1,1,1,4};
const int kv3_v2_resources[RESOURCE_MAX] = { 8,4,2,1,1,1,1,1,1,1,1,1,1,4};



#define RESERVATION_TABLE_LINES 1
const int kv3_reservation_table_lines = RESERVATION_TABLE_LINES;

static const int Reservation_kv3_ALL_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  8 /* cyc.0 Resource-kv3-ISSUE */,
  0 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_FULL_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  1 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_FULL_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  1 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_FULL_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  1 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_LITE_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_LITE_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_LITE_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_LITE_CRWH_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  1 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_LITE_CRWL_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  1 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  1 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_NOP_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  0 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  1 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_TINY_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_TINY_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_TINY_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_TINY_CRRP_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  1 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_ALU_TINY_CRWL_CRWH_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  1 /* cyc.0 Resource-kv3-CRWL */,
  1 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_BCU_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  0 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  1 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_BCU_CRRP_CRWL_CRWH_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  0 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  1 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  1 /* cyc.0 Resource-kv3-CRRP */,
  1 /* cyc.0 Resource-kv3-CRWL */,
  1 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_BCU_TINY_AUXW_CRRP_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  1 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  1 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_BCU_TINY_TINY_MAU_XNOP_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  2 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  1 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  4 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXR_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXR_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXR_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXR_AUXW_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXR_AUXW_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXR_AUXW_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXW_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXW_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_AUXW_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  1 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_CRRP_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  1 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_CRRP_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  1 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_LSU_CRRP_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  1 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  1 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_MAU_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_MAU_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_MAU_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_MAU_AUXR_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_MAU_AUXR_X_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  2 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_MAU_AUXR_Y_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  3 /* cyc.0 Resource-kv3-ISSUE */,
  1 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  1 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  0 /* cyc.0 Resource-kv3-TCA */,
  1 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

static const int Reservation_kv3_TCA_table[RESERVATION_TABLE_LINES * RESOURCE_MAX] = {
  1 /* cyc.0 Resource-kv3-ISSUE */,
  0 /* cyc.0 Resource-kv3-TINY */,
  0 /* cyc.0 Resource-kv3-LITE */,
  0 /* cyc.0 Resource-kv3-FULL */,
  0 /* cyc.0 Resource-kv3-LSU */,
  0 /* cyc.0 Resource-kv3-MAU */,
  0 /* cyc.0 Resource-kv3-BCU */,
  1 /* cyc.0 Resource-kv3-TCA */,
  0 /* cyc.0 Resource-kv3-AUXR */,
  0 /* cyc.0 Resource-kv3-AUXW */,
  0 /* cyc.0 Resource-kv3-CRRP */,
  0 /* cyc.0 Resource-kv3-CRWL */,
  0 /* cyc.0 Resource-kv3-CRWH */,
  0 /* cyc.0 Resource-kv3-NOP */
};

const int *kv3_reservation_table_table[] = {
  Reservation_kv3_ALL_table,
  Reservation_kv3_ALU_NOP_table,
  Reservation_kv3_ALU_TINY_table,
  Reservation_kv3_ALU_TINY_X_table,
  Reservation_kv3_ALU_TINY_Y_table,
  Reservation_kv3_ALU_LITE_table,
  Reservation_kv3_ALU_LITE_X_table,
  Reservation_kv3_ALU_LITE_Y_table,
  Reservation_kv3_ALU_LITE_CRWL_table,
  Reservation_kv3_ALU_LITE_CRWH_table,
  Reservation_kv3_ALU_FULL_table,
  Reservation_kv3_ALU_FULL_X_table,
  Reservation_kv3_ALU_FULL_Y_table,
  Reservation_kv3_BCU_table,
  Reservation_kv3_BCU_CRRP_CRWL_CRWH_table,
  Reservation_kv3_BCU_TINY_AUXW_CRRP_table,
  Reservation_kv3_BCU_TINY_TINY_MAU_XNOP_table,
  Reservation_kv3_TCA_table,
  Reservation_kv3_LSU_table,
  Reservation_kv3_LSU_X_table,
  Reservation_kv3_LSU_Y_table,
  Reservation_kv3_LSU_CRRP_table,
  Reservation_kv3_LSU_CRRP_X_table,
  Reservation_kv3_LSU_CRRP_Y_table,
  Reservation_kv3_LSU_AUXR_table,
  Reservation_kv3_LSU_AUXR_X_table,
  Reservation_kv3_LSU_AUXR_Y_table,
  Reservation_kv3_LSU_AUXW_table,
  Reservation_kv3_LSU_AUXW_X_table,
  Reservation_kv3_LSU_AUXW_Y_table,
  Reservation_kv3_LSU_AUXR_AUXW_table,
  Reservation_kv3_LSU_AUXR_AUXW_X_table,
  Reservation_kv3_LSU_AUXR_AUXW_Y_table,
  Reservation_kv3_MAU_table,
  Reservation_kv3_MAU_X_table,
  Reservation_kv3_MAU_Y_table,
  Reservation_kv3_MAU_AUXR_table,
  Reservation_kv3_MAU_AUXR_X_table,
  Reservation_kv3_MAU_AUXR_Y_table,
  Reservation_kv3_ALU_TINY_CRRP_table,
  Reservation_kv3_ALU_TINY_CRWL_CRWH_table
};

kv3opc_t kv3_v1_optab[] = {
	{"absd", {{ 0x64000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ABSD_registerW_registerZ_simple */
	{"abshq", {{ 0xf4013000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ABSHQ_registerW_registerZ_double */
	{"abswp", {{ 0xf4012000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ABSWP_registerW_registerZ_double */
	{"absw", {{ 0x74000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ABSW_registerW_registerZ_simple */
	{"copyd", {{ 0x6a000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-COPYD_registerW_registerZ_simple */
	{"copyv", {{ 0x01000f80, 0x7f001fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbe_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-COPYV_registerA_registerBe_simple */
	{"copyv", {{ 0x01001f80, 0x7f001fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbo_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-COPYV_registerA_registerBo_simple */
	{"copyw", {{ 0x7a000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-COPYW_registerW_registerZ_simple */
	{"movefo", {{ 0x00081f80, 0x7f0c1fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbe_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-MOVEFO_registerN_registerBe_simple */
	{"movefo", {{ 0x000c1f80, 0x7f0c1fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbo_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-MOVEFO_registerN_registerBo_simple */
	{"negd", {{ 0x65000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-NEGD_registerW_registerZ_simple */
	{"neghq", {{ 0xf5013000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-NEGHQ_registerW_registerZ_double */
	{"negwp", {{ 0xf5012000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-NEGWP_registerW_registerZ_double */
	{"negw", {{ 0x75000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-NEGW_registerW_registerZ_simple */
	{"notd", {{ 0x6c00ffc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-NOTD_registerW_registerZ_simple */
	{"notw", {{ 0x7c00ffc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-NOTW_registerW_registerZ_simple */
	{"satdh", {{ 0x7e022400, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-SATDH_registerW_registerZ_simple */
	{"satdw", {{ 0x7e022800, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-SATDW_registerW_registerZ_simple */
	{"sxbd", {{ 0x68037000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-SXBD_registerW_registerZ_simple */
	{"sxhd", {{ 0x6803f000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-SXHD_registerW_registerZ_simple */
	{"sxwd", {{ 0x6903f000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-SXWD_registerW_registerZ_simple */
	{"zxbd", {{ 0x78003fc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ZXBD_registerW_registerZ_simple */
	{"zxhd", {{ 0x6403f000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ZXHD_registerW_registerZ_simple */
	{"zxwd", {{ 0x7800ffc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v1-ZXWD_registerW_registerZ_simple */
	{"abdd", {{ 0x74010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDD_registerW_registerZ_registerY_simple */
	{"abdd", {{ 0x64000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDD_registerW_registerZ_s010_simple */
	{"abdd", {{ 0xe4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDD_registerW_registerZ_s037_double */
	{"abdd", {{ 0xe4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDD_registerW_registerZ_w064_triple */
	{"abdhq", {{ 0x74013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDHQ_registerW_registerZ_registerY_simple */
	{"abdhq", {{ 0xf4013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDHQ_splat32_registerW_registerZ_w032_double */
	{"abdhq.@", {{ 0xf4013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDHQ_splat32_registerW_registerZ_w032_double */
	{"abdwp", {{ 0x74012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDWP_registerW_registerZ_registerY_simple */
	{"abdwp", {{ 0xf4012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDWP_splat32_registerW_registerZ_w032_double */
	{"abdwp.@", {{ 0xf4012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDWP_splat32_registerW_registerZ_w032_double */
	{"abdw", {{ 0x74011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDW_registerW_registerZ_registerY_simple */
	{"abdw", {{ 0x74000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDW_registerW_registerZ_s010_simple */
	{"abdw", {{ 0xf4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ABDW_registerW_registerZ_s037_double */
	{"acswapd", {{ 0x3f03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPD_doscale_registerY_registerZ_registerU_simple */
	{"acswapd.xs", {{ 0x3f03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPD_doscale_registerY_registerZ_registerU_simple */
	{"acswapd.deqz", {{ 0x3f031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dgez", {{ 0x3f033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dgtz", {{ 0x3f035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dlez", {{ 0x3f034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dltz", {{ 0x3f032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dnez", {{ 0x3f030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.even", {{ 0x3f037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.odd", {{ 0x3f036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.weqz", {{ 0x3f039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wgez", {{ 0x3f03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wgtz", {{ 0x3f03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wlez", {{ 0x3f03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wltz", {{ 0x3f03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wnez", {{ 0x3f038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.deqz", {{ 0xbf031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dgez", {{ 0xbf033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dgtz", {{ 0xbf035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dlez", {{ 0xbf034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dltz", {{ 0xbf032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dnez", {{ 0xbf030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.even", {{ 0xbf037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.odd", {{ 0xbf036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.weqz", {{ 0xbf039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wgez", {{ 0xbf03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wgtz", {{ 0xbf03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wlez", {{ 0xbf03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wltz", {{ 0xbf03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wnez", {{ 0xbf038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.deqz", {{ 0xbf031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dgez", {{ 0xbf033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dgtz", {{ 0xbf035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dlez", {{ 0xbf034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dltz", {{ 0xbf032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dnez", {{ 0xbf030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.even", {{ 0xbf037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.odd", {{ 0xbf036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.weqz", {{ 0xbf039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wgez", {{ 0xbf03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wgtz", {{ 0xbf03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wlez", {{ 0xbf03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wltz", {{ 0xbf03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wnez", {{ 0xbf038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPD_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd", {{ 0x3f010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPD_s010_registerZ_registerU_simple */
	{"acswapd", {{ 0xbf010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPD_s037_registerZ_registerU_double */
	{"acswapd", {{ 0xbf010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPD_w064_registerZ_registerU_triple */
	{"acswapw", {{ 0x3e03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPW_doscale_registerY_registerZ_registerU_simple */
	{"acswapw.xs", {{ 0x3e03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPW_doscale_registerY_registerZ_registerU_simple */
	{"acswapw.deqz", {{ 0x3e031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dgez", {{ 0x3e033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dgtz", {{ 0x3e035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dlez", {{ 0x3e034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dltz", {{ 0x3e032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dnez", {{ 0x3e030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.even", {{ 0x3e037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.odd", {{ 0x3e036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.weqz", {{ 0x3e039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wgez", {{ 0x3e03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wgtz", {{ 0x3e03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wlez", {{ 0x3e03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wltz", {{ 0x3e03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wnez", {{ 0x3e038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.deqz", {{ 0xbe031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dgez", {{ 0xbe033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dgtz", {{ 0xbe035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dlez", {{ 0xbe034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dltz", {{ 0xbe032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dnez", {{ 0xbe030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.even", {{ 0xbe037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.odd", {{ 0xbe036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.weqz", {{ 0xbe039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wgez", {{ 0xbe03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wgtz", {{ 0xbe03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wlez", {{ 0xbe03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wltz", {{ 0xbe03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wnez", {{ 0xbe038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.deqz", {{ 0xbe031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dgez", {{ 0xbe033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dgtz", {{ 0xbe035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dlez", {{ 0xbe034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dltz", {{ 0xbe032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dnez", {{ 0xbe030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.even", {{ 0xbe037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.odd", {{ 0xbe036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.weqz", {{ 0xbe039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wgez", {{ 0xbe03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wgtz", {{ 0xbe03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wlez", {{ 0xbe03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wltz", {{ 0xbe03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wnez", {{ 0xbe038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ACSWAPW_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw", {{ 0x3e010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPW_s010_registerZ_registerU_simple */
	{"acswapw", {{ 0xbe010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPW_s037_registerZ_registerU_double */
	{"acswapw", {{ 0xbe010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ACSWAPW_w064_registerZ_registerU_triple */
	{"addcd.i", {{ 0x7e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD.I_registerW_registerZ_registerY_simple */
	{"addcid", {{ 0x7e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD.I_registerW_registerZ_registerY_simple */
	{"addd.ci", {{ 0x7e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD.I_registerW_registerZ_registerY_simple */
	{"addcd.i", {{ 0xfe01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD.I_registerW_registerZ_w032_double */
	{"addcid", {{ 0xfe01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD.I_registerW_registerZ_w032_double */
	{"addd.ci", {{ 0xfe01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD.I_registerW_registerZ_w032_double */
	{"addcd", {{ 0x7e01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD_registerW_registerZ_registerY_simple */
	{"addd.c", {{ 0x7e01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD_registerW_registerZ_registerY_simple */
	{"addcd", {{ 0xfe01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD_registerW_registerZ_w032_double */
	{"addd.c", {{ 0xfe01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCD_registerW_registerZ_w032_double */
	{"addchcp", {{ 0x7c013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCHCP_registerW_registerZ_registerY_simple */
	{"addchcp", {{ 0xfc013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCHCP_splat32_registerW_registerZ_w032_double */
	{"addchcp.@", {{ 0xfc013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCHCP_splat32_registerW_registerZ_w032_double */
	{"addcwc", {{ 0x7c012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCWC_registerW_registerZ_registerY_simple */
	{"addcwc", {{ 0xfc012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCWC_splat32_registerW_registerZ_w032_double */
	{"addcwc.@", {{ 0xfc012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDCWC_splat32_registerW_registerZ_w032_double */
	{"addd", {{ 0x71010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDD_registerW_registerZ_registerY_simple */
	{"addd", {{ 0x61000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDD_registerW_registerZ_s010_simple */
	{"addd", {{ 0xe1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDD_registerW_registerZ_s037_double */
	{"addd", {{ 0xe1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDD_registerW_registerZ_w064_triple */
	{"addhq", {{ 0x71013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDHQ_registerW_registerZ_registerY_simple */
	{"addhq", {{ 0xf1013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDHQ_splat32_registerW_registerZ_w032_double */
	{"addhq.@", {{ 0xf1013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDHQ_splat32_registerW_registerZ_w032_double */
	{"addsd", {{ 0x7e01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSD_registerW_registerZ_registerY_simple */
	{"addsd", {{ 0x6e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSD_registerW_registerZ_s010_simple */
	{"addsd", {{ 0xee010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSD_registerW_registerZ_s037_double */
	{"addsd", {{ 0xee010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSD_registerW_registerZ_w064_triple */
	{"addshq", {{ 0x7e01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSHQ_registerW_registerZ_registerY_simple */
	{"addshq", {{ 0xfe01f000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSHQ_splat32_registerW_registerZ_w032_double */
	{"addshq.@", {{ 0xfe01f800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSHQ_splat32_registerW_registerZ_w032_double */
	{"addswp", {{ 0x7e01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSWP_registerW_registerZ_registerY_simple */
	{"addswp", {{ 0xfe01e000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSWP_splat32_registerW_registerZ_w032_double */
	{"addswp.@", {{ 0xfe01e800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSWP_splat32_registerW_registerZ_w032_double */
	{"addsw", {{ 0x7e01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSW_registerW_registerZ_registerY_simple */
	{"addsw", {{ 0xfe01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDSW_registerW_registerZ_w032_double */
	{"adduwd", {{ 0x7a014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDUWD_registerW_registerZ_registerY_simple */
	{"adduwd", {{ 0xfa014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDUWD_splat32_registerW_registerZ_w032_double */
	{"adduwd.@", {{ 0xfa014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDUWD_splat32_registerW_registerZ_w032_double */
	{"addwd", {{ 0x78014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDWD_registerW_registerZ_registerY_simple */
	{"addwd", {{ 0xf8014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDWD_splat32_registerW_registerZ_w032_double */
	{"addwd.@", {{ 0xf8014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDWD_splat32_registerW_registerZ_w032_double */
	{"addwp", {{ 0x71012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDWP_registerW_registerZ_registerY_simple */
	{"addwp", {{ 0xf1012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDWP_splat32_registerW_registerZ_w032_double */
	{"addwp.@", {{ 0xf1012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDWP_splat32_registerW_registerZ_w032_double */
	{"addw", {{ 0x71011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDW_registerW_registerZ_registerY_simple */
	{"addw", {{ 0x71000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDW_registerW_registerZ_s010_simple */
	{"addw", {{ 0xf1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDW_registerW_registerZ_s037_double */
	{"addx16d", {{ 0x76014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16D_registerW_registerZ_registerY_simple */
	{"addx16d", {{ 0xf6014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16D_splat32_registerW_registerZ_w032_double */
	{"addx16d.@", {{ 0xf6014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16D_splat32_registerW_registerZ_w032_double */
	{"addx16hq", {{ 0x76017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16HQ_registerW_registerZ_registerY_simple */
	{"addx16hq", {{ 0xf6017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16HQ_splat32_registerW_registerZ_w032_double */
	{"addx16hq.@", {{ 0xf6017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16HQ_splat32_registerW_registerZ_w032_double */
	{"addx16uwd", {{ 0x7e018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16UWD_registerW_registerZ_registerY_simple */
	{"addx16uwd", {{ 0xfe018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16UWD_registerW_registerZ_w032_double */
	{"addx16wd", {{ 0x76018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16WD_registerW_registerZ_registerY_simple */
	{"addx16wd", {{ 0xf6018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16WD_registerW_registerZ_w032_double */
	{"addx16wp", {{ 0x76016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16WP_registerW_registerZ_registerY_simple */
	{"addx16wp", {{ 0xf6016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16WP_splat32_registerW_registerZ_w032_double */
	{"addx16wp.@", {{ 0xf6016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16WP_splat32_registerW_registerZ_w032_double */
	{"addx16w", {{ 0x76015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16W_registerW_registerZ_registerY_simple */
	{"addx16w", {{ 0xf6015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX16W_registerW_registerZ_w032_double */
	{"addx2d", {{ 0x70014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2D_registerW_registerZ_registerY_simple */
	{"addx2d", {{ 0xf0014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2D_splat32_registerW_registerZ_w032_double */
	{"addx2d.@", {{ 0xf0014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2D_splat32_registerW_registerZ_w032_double */
	{"addx2hq", {{ 0x70017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2HQ_registerW_registerZ_registerY_simple */
	{"addx2hq", {{ 0xf0017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2HQ_splat32_registerW_registerZ_w032_double */
	{"addx2hq.@", {{ 0xf0017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2HQ_splat32_registerW_registerZ_w032_double */
	{"addx2uwd", {{ 0x78018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2UWD_registerW_registerZ_registerY_simple */
	{"addx2uwd", {{ 0xf8018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2UWD_registerW_registerZ_w032_double */
	{"addx2wd", {{ 0x70018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2WD_registerW_registerZ_registerY_simple */
	{"addx2wd", {{ 0xf0018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2WD_registerW_registerZ_w032_double */
	{"addx2wp", {{ 0x70016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2WP_registerW_registerZ_registerY_simple */
	{"addx2wp", {{ 0xf0016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2WP_splat32_registerW_registerZ_w032_double */
	{"addx2wp.@", {{ 0xf0016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2WP_splat32_registerW_registerZ_w032_double */
	{"addx2w", {{ 0x70015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2W_registerW_registerZ_registerY_simple */
	{"addx2w", {{ 0xf0015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX2W_registerW_registerZ_w032_double */
	{"addx4d", {{ 0x72014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4D_registerW_registerZ_registerY_simple */
	{"addx4d", {{ 0xf2014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4D_splat32_registerW_registerZ_w032_double */
	{"addx4d.@", {{ 0xf2014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4D_splat32_registerW_registerZ_w032_double */
	{"addx4hq", {{ 0x72017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4HQ_registerW_registerZ_registerY_simple */
	{"addx4hq", {{ 0xf2017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4HQ_splat32_registerW_registerZ_w032_double */
	{"addx4hq.@", {{ 0xf2017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4HQ_splat32_registerW_registerZ_w032_double */
	{"addx4uwd", {{ 0x7a018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4UWD_registerW_registerZ_registerY_simple */
	{"addx4uwd", {{ 0xfa018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4UWD_registerW_registerZ_w032_double */
	{"addx4wd", {{ 0x72018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4WD_registerW_registerZ_registerY_simple */
	{"addx4wd", {{ 0xf2018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4WD_registerW_registerZ_w032_double */
	{"addx4wp", {{ 0x72016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4WP_registerW_registerZ_registerY_simple */
	{"addx4wp", {{ 0xf2016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4WP_splat32_registerW_registerZ_w032_double */
	{"addx4wp.@", {{ 0xf2016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4WP_splat32_registerW_registerZ_w032_double */
	{"addx4w", {{ 0x72015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4W_registerW_registerZ_registerY_simple */
	{"addx4w", {{ 0xf2015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX4W_registerW_registerZ_w032_double */
	{"addx8d", {{ 0x74014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8D_registerW_registerZ_registerY_simple */
	{"addx8d", {{ 0xf4014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8D_splat32_registerW_registerZ_w032_double */
	{"addx8d.@", {{ 0xf4014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8D_splat32_registerW_registerZ_w032_double */
	{"addx8hq", {{ 0x74017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8HQ_registerW_registerZ_registerY_simple */
	{"addx8hq", {{ 0xf4017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8HQ_splat32_registerW_registerZ_w032_double */
	{"addx8hq.@", {{ 0xf4017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8HQ_splat32_registerW_registerZ_w032_double */
	{"addx8uwd", {{ 0x7c018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8UWD_registerW_registerZ_registerY_simple */
	{"addx8uwd", {{ 0xfc018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8UWD_registerW_registerZ_w032_double */
	{"addx8wd", {{ 0x74018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8WD_registerW_registerZ_registerY_simple */
	{"addx8wd", {{ 0xf4018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8WD_registerW_registerZ_w032_double */
	{"addx8wp", {{ 0x74016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8WP_registerW_registerZ_registerY_simple */
	{"addx8wp", {{ 0xf4016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8WP_splat32_registerW_registerZ_w032_double */
	{"addx8wp.@", {{ 0xf4016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8WP_splat32_registerW_registerZ_w032_double */
	{"addx8w", {{ 0x74015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8W_registerW_registerZ_registerY_simple */
	{"addx8w", {{ 0xf4015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ADDX8W_registerW_registerZ_w032_double */
	{"aladdd", {{ 0x2f03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"afaddd", {{ 0x2f03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"aladdd.xs", {{ 0x2f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"afaddd.xs", {{ 0x2f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"aladdd.deqz", {{ 0x2f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.deqz", {{ 0x2f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dgez", {{ 0x2f033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dgez", {{ 0x2f033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dgtz", {{ 0x2f035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dgtz", {{ 0x2f035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dlez", {{ 0x2f034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dlez", {{ 0x2f034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dltz", {{ 0x2f032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dltz", {{ 0x2f032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dnez", {{ 0x2f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dnez", {{ 0x2f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.even", {{ 0x2f037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.even", {{ 0x2f037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.odd", {{ 0x2f036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.odd", {{ 0x2f036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.weqz", {{ 0x2f039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.weqz", {{ 0x2f039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wgez", {{ 0x2f03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wgez", {{ 0x2f03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wgtz", {{ 0x2f03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wgtz", {{ 0x2f03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wlez", {{ 0x2f03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wlez", {{ 0x2f03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wltz", {{ 0x2f03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wltz", {{ 0x2f03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wnez", {{ 0x2f038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wnez", {{ 0x2f038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd", {{ 0x2f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_s010_registerZ_registerT_simple */
	{"afaddd", {{ 0x2f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_s010_registerZ_registerT_simple */
	{"aladdd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_s037_registerZ_registerT_double */
	{"afaddd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_s037_registerZ_registerT_double */
	{"aladdd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_w064_registerZ_registerT_triple */
	{"afaddd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDD_w064_registerZ_registerT_triple */
	{"aladdw", {{ 0x2e03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"afaddw", {{ 0x2e03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"aladdw.xs", {{ 0x2e03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"afaddw.xs", {{ 0x2e03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"aladdw.deqz", {{ 0x2e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.deqz", {{ 0x2e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dgez", {{ 0x2e033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dgez", {{ 0x2e033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dgtz", {{ 0x2e035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dgtz", {{ 0x2e035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dlez", {{ 0x2e034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dlez", {{ 0x2e034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dltz", {{ 0x2e032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dltz", {{ 0x2e032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dnez", {{ 0x2e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dnez", {{ 0x2e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.even", {{ 0x2e037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.even", {{ 0x2e037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.odd", {{ 0x2e036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.odd", {{ 0x2e036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.weqz", {{ 0x2e039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.weqz", {{ 0x2e039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wgez", {{ 0x2e03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wgez", {{ 0x2e03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wgtz", {{ 0x2e03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wgtz", {{ 0x2e03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wlez", {{ 0x2e03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wlez", {{ 0x2e03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wltz", {{ 0x2e03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wltz", {{ 0x2e03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wnez", {{ 0x2e038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wnez", {{ 0x2e038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw", {{ 0x2e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_s010_registerZ_registerT_simple */
	{"afaddw", {{ 0x2e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_s010_registerZ_registerT_simple */
	{"aladdw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_s037_registerZ_registerT_double */
	{"afaddw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_s037_registerZ_registerT_double */
	{"aladdw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_w064_registerZ_registerT_triple */
	{"afaddw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-ALADDW_w064_registerZ_registerT_triple */
	{"alclrd", {{ 0x2b03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRD_doscale_registerW_registerY_registerZ_simple */
	{"alclrd.xs", {{ 0x2b03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRD_doscale_registerW_registerY_registerZ_simple */
	{"alclrd.deqz", {{ 0x2b031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dgez", {{ 0x2b033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dgtz", {{ 0x2b035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dlez", {{ 0x2b034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dltz", {{ 0x2b032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dnez", {{ 0x2b030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.even", {{ 0x2b037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.odd", {{ 0x2b036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.weqz", {{ 0x2b039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wgez", {{ 0x2b03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wgtz", {{ 0x2b03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wlez", {{ 0x2b03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wltz", {{ 0x2b03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wnez", {{ 0x2b038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.deqz", {{ 0xab031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dgez", {{ 0xab033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dgtz", {{ 0xab035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dlez", {{ 0xab034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dltz", {{ 0xab032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dnez", {{ 0xab030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.even", {{ 0xab037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.odd", {{ 0xab036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.weqz", {{ 0xab039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wgez", {{ 0xab03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wgtz", {{ 0xab03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wlez", {{ 0xab03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wltz", {{ 0xab03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wnez", {{ 0xab038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.deqz", {{ 0xab031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dgez", {{ 0xab033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dgtz", {{ 0xab035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dlez", {{ 0xab034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dltz", {{ 0xab032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dnez", {{ 0xab030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.even", {{ 0xab037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.odd", {{ 0xab036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.weqz", {{ 0xab039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wgez", {{ 0xab03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wgtz", {{ 0xab03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wlez", {{ 0xab03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wltz", {{ 0xab03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wnez", {{ 0xab038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd", {{ 0x2b010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRD_registerW_s010_registerZ_simple */
	{"alclrd", {{ 0xab010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRD_registerW_s037_registerZ_double */
	{"alclrd", {{ 0xab010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRD_registerW_w064_registerZ_triple */
	{"alclrw", {{ 0x2a03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRW_doscale_registerW_registerY_registerZ_simple */
	{"alclrw.xs", {{ 0x2a03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRW_doscale_registerW_registerY_registerZ_simple */
	{"alclrw.deqz", {{ 0x2a031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dgez", {{ 0x2a033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dgtz", {{ 0x2a035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dlez", {{ 0x2a034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dltz", {{ 0x2a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dnez", {{ 0x2a030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.even", {{ 0x2a037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.odd", {{ 0x2a036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.weqz", {{ 0x2a039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wgez", {{ 0x2a03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wgtz", {{ 0x2a03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wlez", {{ 0x2a03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wltz", {{ 0x2a03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wnez", {{ 0x2a038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.deqz", {{ 0xaa031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dgez", {{ 0xaa033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dgtz", {{ 0xaa035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dlez", {{ 0xaa034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dltz", {{ 0xaa032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dnez", {{ 0xaa030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.even", {{ 0xaa037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.odd", {{ 0xaa036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.weqz", {{ 0xaa039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wgez", {{ 0xaa03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wgtz", {{ 0xaa03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wlez", {{ 0xaa03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wltz", {{ 0xaa03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wnez", {{ 0xaa038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.deqz", {{ 0xaa031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dgez", {{ 0xaa033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dgtz", {{ 0xaa035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dlez", {{ 0xaa034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dltz", {{ 0xaa032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dnez", {{ 0xaa030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.even", {{ 0xaa037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.odd", {{ 0xaa036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.weqz", {{ 0xaa039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wgez", {{ 0xaa03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wgtz", {{ 0xaa03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wlez", {{ 0xaa03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wltz", {{ 0xaa03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wnez", {{ 0xaa038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw", {{ 0x2a010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRW_registerW_s010_registerZ_simple */
	{"alclrw", {{ 0xaa010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRW_registerW_s037_registerZ_double */
	{"alclrw", {{ 0xaa010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-ALCLRW_registerW_w064_registerZ_triple */
	{"aligno", {{ 0x00081040, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNO_registerN_registerBe_registerCo_registerZ_simple */
	{"aligno", {{ 0x00081000, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNO_registerN_registerBe_registerCo_u006_simple */
	{"aligno", {{ 0x000c1040, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNO_registerN_registerBo_registerCe_registerZ_simple */
	{"aligno", {{ 0x000c1000, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNO_registerN_registerBo_registerCe_u006_simple */
	{"alignv", {{ 0x01000040, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNV_registerA_registerBe_registerCo_registerZ_simple */
	{"alignv", {{ 0x01000000, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNV_registerA_registerBe_registerCo_u006_simple */
	{"alignv", {{ 0x01001040, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNV_registerA_registerBo_registerCe_registerZ_simple */
	{"alignv", {{ 0x01001000, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-ALIGNV_registerA_registerBo_registerCe_u006_simple */
	{"andd", {{ 0x78010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDD_registerW_registerZ_registerY_simple */
	{"andd", {{ 0x68000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDD_registerW_registerZ_s010_simple */
	{"andd", {{ 0xe8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDD_registerW_registerZ_s037_double */
	{"andd.@", {{ 0xf8010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDD_registerW_registerZ_w032_double */
	{"andd", {{ 0xe8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDD_registerW_registerZ_w064_triple */
	{"andnd", {{ 0x7e010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDND_registerW_registerZ_registerY_simple */
	{"andnd", {{ 0x6e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDND_registerW_registerZ_s010_simple */
	{"andnd", {{ 0xee000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDND_registerW_registerZ_s037_double */
	{"andnd.@", {{ 0xfe010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDND_registerW_registerZ_w032_double */
	{"andnd", {{ 0xee000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDND_registerW_registerZ_w064_triple */
	{"andnw", {{ 0x7e011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDNW_registerW_registerZ_registerY_simple */
	{"andnw", {{ 0x7e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDNW_registerW_registerZ_s010_simple */
	{"andnw", {{ 0xfe000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDNW_registerW_registerZ_s037_double */
	{"andw", {{ 0x78011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDW_registerW_registerZ_registerY_simple */
	{"andw", {{ 0x78000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDW_registerW_registerZ_s010_simple */
	{"andw", {{ 0xf8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ANDW_registerW_registerZ_s037_double */
	{"avghq", {{ 0x78017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGHQ_registerW_registerZ_registerY_simple */
	{"avghq", {{ 0xf8017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGHQ_splat32_registerW_registerZ_w032_double */
	{"avghq.@", {{ 0xf8017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGHQ_splat32_registerW_registerZ_w032_double */
	{"avgrhq", {{ 0x7a017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRHQ_registerW_registerZ_registerY_simple */
	{"avgrhq", {{ 0xfa017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRHQ_splat32_registerW_registerZ_w032_double */
	{"avgrhq.@", {{ 0xfa017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRHQ_splat32_registerW_registerZ_w032_double */
	{"avgruhq", {{ 0x7b017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUHQ_registerW_registerZ_registerY_simple */
	{"avgruhq", {{ 0xfb017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUHQ_splat32_registerW_registerZ_w032_double */
	{"avgruhq.@", {{ 0xfb017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUHQ_splat32_registerW_registerZ_w032_double */
	{"avgruwp", {{ 0x7b016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUWP_registerW_registerZ_registerY_simple */
	{"avgruwp", {{ 0xfb016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUWP_splat32_registerW_registerZ_w032_double */
	{"avgruwp.@", {{ 0xfb016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUWP_splat32_registerW_registerZ_w032_double */
	{"avgruw", {{ 0x7b015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUW_registerW_registerZ_registerY_simple */
	{"avgruw", {{ 0xfb015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRUW_registerW_registerZ_w032_double */
	{"avgrwp", {{ 0x7a016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRWP_registerW_registerZ_registerY_simple */
	{"avgrwp", {{ 0xfa016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRWP_splat32_registerW_registerZ_w032_double */
	{"avgrwp.@", {{ 0xfa016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRWP_splat32_registerW_registerZ_w032_double */
	{"avgrw", {{ 0x7a015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRW_registerW_registerZ_registerY_simple */
	{"avgrw", {{ 0xfa015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGRW_registerW_registerZ_w032_double */
	{"avguhq", {{ 0x79017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUHQ_registerW_registerZ_registerY_simple */
	{"avguhq", {{ 0xf9017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUHQ_splat32_registerW_registerZ_w032_double */
	{"avguhq.@", {{ 0xf9017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUHQ_splat32_registerW_registerZ_w032_double */
	{"avguwp", {{ 0x79016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUWP_registerW_registerZ_registerY_simple */
	{"avguwp", {{ 0xf9016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUWP_splat32_registerW_registerZ_w032_double */
	{"avguwp.@", {{ 0xf9016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUWP_splat32_registerW_registerZ_w032_double */
	{"avguw", {{ 0x79015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUW_registerW_registerZ_registerY_simple */
	{"avguw", {{ 0xf9015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGUW_registerW_registerZ_w032_double */
	{"avgwp", {{ 0x78016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGWP_registerW_registerZ_registerY_simple */
	{"avgwp", {{ 0xf8016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGWP_splat32_registerW_registerZ_w032_double */
	{"avgwp.@", {{ 0xf8016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGWP_splat32_registerW_registerZ_w032_double */
	{"avgw", {{ 0x78015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGW_registerW_registerZ_registerY_simple */
	{"avgw", {{ 0xf8015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-AVGW_registerW_registerZ_w032_double */
	{"await", {{ 0x0fa00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-AWAIT_simple */
	{"barrier", {{ 0x0fac0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-BARRIER_simple */
	{"call", {{ 0x18000000, 0x78000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_pcrel27_opnd, 0}, "", "%s"}, /* Opcode-v1-CALL_s027_simple */
	{"cbsd", {{ 0x72022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CBSD_registerW_registerZ_simple */
	{"cbswp", {{ 0x72024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CBSWP_registerW_registerZ_simple */
	{"cbsw", {{ 0x72023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CBSW_registerW_registerZ_simple */
	{"cb.deqz", {{ 0x08800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.dgez", {{ 0x09800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.dgtz", {{ 0x0a800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.dlez", {{ 0x0a000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.dltz", {{ 0x09000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.dnez", {{ 0x08000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.even", {{ 0x0b800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.odd", {{ 0x0b000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.weqz", {{ 0x0c800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.wgez", {{ 0x0d800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.wgtz", {{ 0x0e800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.wlez", {{ 0x0e000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.wltz", {{ 0x0d000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"cb.wnez", {{ 0x0c000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v1-CB_branchcond_registerZ_s017_simple */
	{"clrf", {{ 0x6c030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-CLRF_registerW_registerZ_u006_u006_simple */
	{"clsd", {{ 0x71022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CLSD_registerW_registerZ_simple */
	{"clswp", {{ 0x71024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CLSWP_registerW_registerZ_simple */
	{"clsw", {{ 0x71023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CLSW_registerW_registerZ_simple */
	{"clzd", {{ 0x70022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CLZD_registerW_registerZ_simple */
	{"clzwp", {{ 0x70024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CLZWP_registerW_registerZ_simple */
	{"clzw", {{ 0x70023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CLZW_registerW_registerZ_simple */
	{"cmoved.deqz", {{ 0x71020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dgez", {{ 0x73020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dgtz", {{ 0x75020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dlez", {{ 0x74020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dltz", {{ 0x72020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dnez", {{ 0x70020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.even", {{ 0x77020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.odd", {{ 0x76020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.weqz", {{ 0x79020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wgez", {{ 0x7b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wgtz", {{ 0x7d020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wlez", {{ 0x7c020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wltz", {{ 0x7a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wnez", {{ 0x78020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.deqz", {{ 0x61020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dgez", {{ 0x63020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dgtz", {{ 0x65020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dlez", {{ 0x64020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dltz", {{ 0x62020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dnez", {{ 0x60020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.even", {{ 0x67020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.odd", {{ 0x66020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.weqz", {{ 0x69020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wgez", {{ 0x6b020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wgtz", {{ 0x6d020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wlez", {{ 0x6c020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wltz", {{ 0x6a020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wnez", {{ 0x68020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.deqz", {{ 0xe1020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dgez", {{ 0xe3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dgtz", {{ 0xe5020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dlez", {{ 0xe4020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dltz", {{ 0xe2020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dnez", {{ 0xe0020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.even", {{ 0xe7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.odd", {{ 0xe6020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.weqz", {{ 0xe9020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wgez", {{ 0xeb020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wgtz", {{ 0xed020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wlez", {{ 0xec020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wltz", {{ 0xea020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wnez", {{ 0xe8020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.deqz", {{ 0xe1020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dgez", {{ 0xe3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dgtz", {{ 0xe5020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dlez", {{ 0xe4020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dltz", {{ 0xe2020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dnez", {{ 0xe0020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.even", {{ 0xe7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.odd", {{ 0xe6020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.weqz", {{ 0xe9020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wgez", {{ 0xeb020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wgtz", {{ 0xed020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wlez", {{ 0xec020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wltz", {{ 0xea020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wnez", {{ 0xe8020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmovehq.eqz", {{ 0x79021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.even", {{ 0x7f021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.gez", {{ 0x7b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.gtz", {{ 0x7d021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.lez", {{ 0x7c021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.ltz", {{ 0x7a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.nez", {{ 0x78021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.odd", {{ 0x7e021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.eqz", {{ 0x71021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.even", {{ 0x77021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.gez", {{ 0x73021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.gtz", {{ 0x75021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.lez", {{ 0x74021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.ltz", {{ 0x72021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.nez", {{ 0x70021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.odd", {{ 0x76021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v1-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmuldt", {{ 0x5b041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULDT_registerM_registerZ_registerY_simple */
	{"cmuldt", {{ 0x4b040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULDT_registerM_registerZ_s010_simple */
	{"cmuldt", {{ 0xcb040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULDT_registerM_registerZ_s037_double */
	{"cmuldt", {{ 0xcb040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULDT_registerM_registerZ_w064_triple */
	{"cmulghxdt", {{ 0x5f041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULGHXDT_registerM_registerZ_registerY_simple */
	{"cmulglxdt", {{ 0x5d041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULGLXDT_registerM_registerZ_registerY_simple */
	{"cmulgmxdt", {{ 0x5e041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULGMXDT_registerM_registerZ_registerY_simple */
	{"cmulxdt", {{ 0x5c041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CMULXDT_registerM_registerZ_registerY_simple */
	{"compd.all", {{ 0x7a01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.any", {{ 0x7c01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.eq", {{ 0x7101a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.ge", {{ 0x7301a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.geu", {{ 0x7701a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.gt", {{ 0x7501a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.gtu", {{ 0x7901a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.le", {{ 0x7401a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.leu", {{ 0x7801a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.lt", {{ 0x7201a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.ltu", {{ 0x7601a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.nall", {{ 0x7b01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.ne", {{ 0x7001a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.none", {{ 0x7d01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.all", {{ 0x6a010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.any", {{ 0x6c010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.eq", {{ 0x61010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.ge", {{ 0x63010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.geu", {{ 0x67010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.gt", {{ 0x65010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.gtu", {{ 0x69010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.le", {{ 0x64010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.leu", {{ 0x68010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.lt", {{ 0x62010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.ltu", {{ 0x66010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.nall", {{ 0x6b010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.ne", {{ 0x60010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.none", {{ 0x6d010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.all", {{ 0xea010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.any", {{ 0xec010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.eq", {{ 0xe1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.ge", {{ 0xe3010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.geu", {{ 0xe7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.gt", {{ 0xe5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.gtu", {{ 0xe9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.le", {{ 0xe4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.leu", {{ 0xe8010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.lt", {{ 0xe2010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.ltu", {{ 0xe6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.nall", {{ 0xeb010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.ne", {{ 0xe0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.none", {{ 0xed010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.all", {{ 0xea010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.any", {{ 0xec010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.eq", {{ 0xe1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.ge", {{ 0xe3010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.geu", {{ 0xe7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.gt", {{ 0xe5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.gtu", {{ 0xe9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.le", {{ 0xe4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.leu", {{ 0xe8010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.lt", {{ 0xe2010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.ltu", {{ 0xe6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.nall", {{ 0xeb010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.ne", {{ 0xe0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.none", {{ 0xed010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compnhq.all", {{ 0x7a01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.any", {{ 0x7c01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.eq", {{ 0x7101f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.ge", {{ 0x7301f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.geu", {{ 0x7701f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.gt", {{ 0x7501f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.gtu", {{ 0x7901f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.le", {{ 0x7401f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.leu", {{ 0x7801f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.lt", {{ 0x7201f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.ltu", {{ 0x7601f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.nall", {{ 0x7b01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.ne", {{ 0x7001f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.none", {{ 0x7d01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.all", {{ 0x7a01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.any", {{ 0x7c01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.eq", {{ 0x7101e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.ge", {{ 0x7301e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.geu", {{ 0x7701e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.gt", {{ 0x7501e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.gtu", {{ 0x7901e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.le", {{ 0x7401e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.leu", {{ 0x7801e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.lt", {{ 0x7201e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.ltu", {{ 0x7601e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.nall", {{ 0x7b01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.ne", {{ 0x7001e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.none", {{ 0x7d01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.all", {{ 0x7a01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.any", {{ 0x7c01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.eq", {{ 0x7101d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.ge", {{ 0x7301d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.geu", {{ 0x7701d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.gt", {{ 0x7501d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.gtu", {{ 0x7901d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.le", {{ 0x7401d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.leu", {{ 0x7801d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.lt", {{ 0x7201d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.ltu", {{ 0x7601d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.nall", {{ 0x7b01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.ne", {{ 0x7001d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.none", {{ 0x7d01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.all", {{ 0x7a01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.any", {{ 0x7c01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.eq", {{ 0x7101c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.ge", {{ 0x7301c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.geu", {{ 0x7701c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.gt", {{ 0x7501c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.gtu", {{ 0x7901c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.le", {{ 0x7401c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.leu", {{ 0x7801c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.lt", {{ 0x7201c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.ltu", {{ 0x7601c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.nall", {{ 0x7b01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.ne", {{ 0x7001c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.none", {{ 0x7d01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compw.all", {{ 0x7a01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.any", {{ 0x7c01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.eq", {{ 0x7101b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.ge", {{ 0x7301b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.geu", {{ 0x7701b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.gt", {{ 0x7501b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.gtu", {{ 0x7901b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.le", {{ 0x7401b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.leu", {{ 0x7801b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.lt", {{ 0x7201b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.ltu", {{ 0x7601b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.nall", {{ 0x7b01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.ne", {{ 0x7001b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.none", {{ 0x7d01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.all", {{ 0xfa01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.any", {{ 0xfc01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.eq", {{ 0xf101b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.ge", {{ 0xf301b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.geu", {{ 0xf701b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.gt", {{ 0xf501b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.gtu", {{ 0xf901b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.le", {{ 0xf401b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.leu", {{ 0xf801b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.lt", {{ 0xf201b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.ltu", {{ 0xf601b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.nall", {{ 0xfb01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.ne", {{ 0xf001b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.none", {{ 0xfd01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COMPW_comparison_registerW_registerZ_w032_double */
	{"convdhv0.rd.sat", {{ 0x05000200, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rd.satu", {{ 0x05000280, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rhu.sat", {{ 0x05000400, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rhu.satu", {{ 0x05000480, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rn.sat", {{ 0x05000000, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rn.satu", {{ 0x05000080, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.ru.sat", {{ 0x05000100, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.ru.satu", {{ 0x05000180, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rz.sat", {{ 0x05000300, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rz.satu", {{ 0x05000380, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv1.rd.sat", {{ 0x05001200, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rd.satu", {{ 0x05001280, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rhu.sat", {{ 0x05001400, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rhu.satu", {{ 0x05001480, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rn.sat", {{ 0x05001000, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rn.satu", {{ 0x05001080, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.ru.sat", {{ 0x05001100, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.ru.satu", {{ 0x05001180, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rz.sat", {{ 0x05001300, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rz.satu", {{ 0x05001380, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convwbv0.rd.sat", {{ 0x06000200, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rd.satu", {{ 0x06000280, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rhu.sat", {{ 0x06000400, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rhu.satu", {{ 0x06000480, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rn.sat", {{ 0x06000000, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rn.satu", {{ 0x06000080, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.ru.sat", {{ 0x06000100, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.ru.satu", {{ 0x06000180, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rz.sat", {{ 0x06000300, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv0.rz.satu", {{ 0x06000380, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerax_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV0_roundint_saturate_registerAx_registerBq_simple */
	{"convwbv1.rd.sat", {{ 0x06001200, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rd.satu", {{ 0x06001280, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rhu.sat", {{ 0x06001400, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rhu.satu", {{ 0x06001480, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rn.sat", {{ 0x06001000, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rn.satu", {{ 0x06001080, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.ru.sat", {{ 0x06001100, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.ru.satu", {{ 0x06001180, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rz.sat", {{ 0x06001300, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv1.rz.satu", {{ 0x06001380, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeray_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV1_roundint_saturate_registerAy_registerBq_simple */
	{"convwbv2.rd.sat", {{ 0x06002200, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rd.satu", {{ 0x06002280, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rhu.sat", {{ 0x06002400, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rhu.satu", {{ 0x06002480, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rn.sat", {{ 0x06002000, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rn.satu", {{ 0x06002080, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.ru.sat", {{ 0x06002100, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.ru.satu", {{ 0x06002180, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rz.sat", {{ 0x06002300, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv2.rz.satu", {{ 0x06002380, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraz_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV2_roundint_saturate_registerAz_registerBq_simple */
	{"convwbv3.rd.sat", {{ 0x06003200, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rd.satu", {{ 0x06003280, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rhu.sat", {{ 0x06003400, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rhu.satu", {{ 0x06003480, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rn.sat", {{ 0x06003000, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rn.satu", {{ 0x06003080, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.ru.sat", {{ 0x06003100, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.ru.satu", {{ 0x06003180, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rz.sat", {{ 0x06003300, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"convwbv3.rz.satu", {{ 0x06003380, 0x7f0037c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerat_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CONVWBV3_roundint_saturate_registerAt_registerBq_simple */
	{"copyo", {{ 0x3e050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registern_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-COPYO_registerN_registerR_simple */
	{"copyq", {{ 0x5f00f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-COPYQ_registerM_registerZ_registerY_simple */
	{"crcbellw", {{ 0x59002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCBELLW_registerW_registerZ_registerY_simple */
	{"crcbellw", {{ 0xd9002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCBELLW_registerW_registerZ_w032_double */
	{"crcbelmw", {{ 0x58002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCBELMW_registerW_registerZ_registerY_simple */
	{"crcbelmw", {{ 0xd8002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCBELMW_registerW_registerZ_w032_double */
	{"crclellw", {{ 0x5b002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCLELLW_registerW_registerZ_registerY_simple */
	{"crclellw", {{ 0xdb002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCLELLW_registerW_registerZ_w032_double */
	{"crclelmw", {{ 0x5a002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCLELMW_registerW_registerZ_registerY_simple */
	{"crclelmw", {{ 0xda002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-CRCLELMW_registerW_registerZ_w032_double */
	{"ctzd", {{ 0x73022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CTZD_registerW_registerZ_simple */
	{"ctzwp", {{ 0x73024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CTZWP_registerW_registerZ_simple */
	{"ctzw", {{ 0x73023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-CTZW_registerW_registerZ_simple */
	{"dinvall.deqz", {{ 0x3c1e1000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.dgez", {{ 0x3c1e3000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.dgtz", {{ 0x3c1e5000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.dlez", {{ 0x3c1e4000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.dltz", {{ 0x3c1e2000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.dnez", {{ 0x3c1e0000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.even", {{ 0x3c1e7000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.odd", {{ 0x3c1e6000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.weqz", {{ 0x3c1e9000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.wgez", {{ 0x3c1eb000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.wgtz", {{ 0x3c1ed000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.wlez", {{ 0x3c1ec000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.wltz", {{ 0x3c1ea000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.wnez", {{ 0x3c1e8000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_registerZ_simple */
	{"dinvall.deqz", {{ 0xbc1e1000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dgez", {{ 0xbc1e3000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dgtz", {{ 0xbc1e5000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dlez", {{ 0xbc1e4000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dltz", {{ 0xbc1e2000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dnez", {{ 0xbc1e0000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.even", {{ 0xbc1e7000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.odd", {{ 0xbc1e6000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.weqz", {{ 0xbc1e9000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wgez", {{ 0xbc1eb000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wgtz", {{ 0xbc1ed000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wlez", {{ 0xbc1ec000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wltz", {{ 0xbc1ea000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wnez", {{ 0xbc1e8000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s027_registerZ_double */
	{"dinvall.deqz", {{ 0xbc1e1000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dgez", {{ 0xbc1e3000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dgtz", {{ 0xbc1e5000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dlez", {{ 0xbc1e4000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dltz", {{ 0xbc1e2000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dnez", {{ 0xbc1e0000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.even", {{ 0xbc1e7000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.odd", {{ 0xbc1e6000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.weqz", {{ 0xbc1e9000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wgez", {{ 0xbc1eb000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wgtz", {{ 0xbc1ed000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wlez", {{ 0xbc1ec000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wltz", {{ 0xbc1ea000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wnez", {{ 0xbc1e8000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DINVALL_lsucond_registerY_s054_registerZ_triple */
	{"dinvall", {{ 0x3c1ee000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DINVALL_registerY_registerZ_simple */
	{"dinvall", {{ 0x3c1c0000, 0x7cff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DINVALL_s010_registerZ_simple */
	{"dinvall", {{ 0xbc1c0000, 0xfcff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DINVALL_s037_registerZ_double */
	{"dinvall", {{ 0xbc1c0000, 0xfcff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DINVALL_w064_registerZ_triple */
	{"dinval", {{ 0x3f8d0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {0}, "", ""}, /* Opcode-v1-DINVAL_simple */
	{"dot2suwdp", {{ 0x52002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2SUWDP_registerM_registerP_registerO_simple */
	{"dot2suwd", {{ 0x5e002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2SUWD_registerW_registerZ_registerY_simple */
	{"dot2suwd", {{ 0x4e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2SUWD_registerW_registerZ_s010_simple */
	{"dot2suwd", {{ 0xce000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2SUWD_registerW_registerZ_s037_double */
	{"dot2suwd", {{ 0xce000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2SUWD_registerW_registerZ_w064_triple */
	{"dot2uwdp", {{ 0x51002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2UWDP_registerM_registerP_registerO_simple */
	{"dot2uwd", {{ 0x5d002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2UWD_registerW_registerZ_registerY_simple */
	{"dot2uwd", {{ 0x4d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2UWD_registerW_registerZ_s010_simple */
	{"dot2uwd", {{ 0xcd000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2UWD_registerW_registerZ_s037_double */
	{"dot2uwd", {{ 0xcd000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2UWD_registerW_registerZ_w064_triple */
	{"dot2wdp", {{ 0x50002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2WDP_registerM_registerP_registerO_simple */
	{"dot2wd", {{ 0x5c002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2WD_registerW_registerZ_registerY_simple */
	{"dot2wd", {{ 0x4c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2WD_registerW_registerZ_s010_simple */
	{"dot2wd", {{ 0xcc000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2WD_registerW_registerZ_s037_double */
	{"dot2wd", {{ 0xcc000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2WD_registerW_registerZ_w064_triple */
	{"dot2wzp", {{ 0x53002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2WZP_registerM_registerP_registerO_simple */
	{"dot2w", {{ 0x5f002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2W_registerW_registerZ_registerY_simple */
	{"dot2w", {{ 0x4f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2W_registerW_registerZ_s010_simple */
	{"dot2w", {{ 0xcf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2W_registerW_registerZ_s037_double */
	{"dot2w", {{ 0xcf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-DOT2W_registerW_registerZ_w064_triple */
	{"dtouchl.deqz", {{ 0x3c0e1000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.dgez", {{ 0x3c0e3000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.dgtz", {{ 0x3c0e5000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.dlez", {{ 0x3c0e4000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.dltz", {{ 0x3c0e2000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.dnez", {{ 0x3c0e0000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.even", {{ 0x3c0e7000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.odd", {{ 0x3c0e6000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.weqz", {{ 0x3c0e9000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.wgez", {{ 0x3c0eb000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.wgtz", {{ 0x3c0ed000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.wlez", {{ 0x3c0ec000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.wltz", {{ 0x3c0ea000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.wnez", {{ 0x3c0e8000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_registerZ_simple */
	{"dtouchl.deqz", {{ 0xbc0e1000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dgez", {{ 0xbc0e3000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dgtz", {{ 0xbc0e5000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dlez", {{ 0xbc0e4000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dltz", {{ 0xbc0e2000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dnez", {{ 0xbc0e0000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.even", {{ 0xbc0e7000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.odd", {{ 0xbc0e6000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.weqz", {{ 0xbc0e9000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wgez", {{ 0xbc0eb000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wgtz", {{ 0xbc0ed000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wlez", {{ 0xbc0ec000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wltz", {{ 0xbc0ea000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wnez", {{ 0xbc0e8000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.deqz", {{ 0xbc0e1000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dgez", {{ 0xbc0e3000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dgtz", {{ 0xbc0e5000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dlez", {{ 0xbc0e4000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dltz", {{ 0xbc0e2000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dnez", {{ 0xbc0e0000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.even", {{ 0xbc0e7000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.odd", {{ 0xbc0e6000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.weqz", {{ 0xbc0e9000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wgez", {{ 0xbc0eb000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wgtz", {{ 0xbc0ed000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wlez", {{ 0xbc0ec000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wltz", {{ 0xbc0ea000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wnez", {{ 0xbc0e8000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DTOUCHL_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl", {{ 0x3c0ee000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DTOUCHL_registerY_registerZ_simple */
	{"dtouchl", {{ 0x3c0c0000, 0x7cff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DTOUCHL_s010_registerZ_simple */
	{"dtouchl", {{ 0xbc0c0000, 0xfcff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DTOUCHL_s037_registerZ_double */
	{"dtouchl", {{ 0xbc0c0000, 0xfcff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DTOUCHL_w064_registerZ_triple */
	{"dzerol.deqz", {{ 0x280f1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.dgez", {{ 0x280f3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.dgtz", {{ 0x280f5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.dlez", {{ 0x280f4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.dltz", {{ 0x280f2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.dnez", {{ 0x280f0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.even", {{ 0x280f7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.odd", {{ 0x280f6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.weqz", {{ 0x280f9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.wgez", {{ 0x280fb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.wgtz", {{ 0x280fd000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.wlez", {{ 0x280fc000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.wltz", {{ 0x280fa000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.wnez", {{ 0x280f8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_registerZ_simple */
	{"dzerol.deqz", {{ 0xa80f1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.dgez", {{ 0xa80f3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.dgtz", {{ 0xa80f5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.dlez", {{ 0xa80f4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.dltz", {{ 0xa80f2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.dnez", {{ 0xa80f0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.even", {{ 0xa80f7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.odd", {{ 0xa80f6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.weqz", {{ 0xa80f9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.wgez", {{ 0xa80fb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.wgtz", {{ 0xa80fd000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.wlez", {{ 0xa80fc000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.wltz", {{ 0xa80fa000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.wnez", {{ 0xa80f8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s027_registerZ_double */
	{"dzerol.deqz", {{ 0xa80f1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.dgez", {{ 0xa80f3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.dgtz", {{ 0xa80f5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.dlez", {{ 0xa80f4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.dltz", {{ 0xa80f2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.dnez", {{ 0xa80f0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.even", {{ 0xa80f7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.odd", {{ 0xa80f6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.weqz", {{ 0xa80f9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.wgez", {{ 0xa80fb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.wgtz", {{ 0xa80fd000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.wlez", {{ 0xa80fc000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.wltz", {{ 0xa80fa000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol.wnez", {{ 0xa80f8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-DZEROL_lsucond_registerY_s054_registerZ_triple */
	{"dzerol", {{ 0x280fe000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DZEROL_registerY_registerZ_simple */
	{"dzerol", {{ 0x280d0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DZEROL_s010_registerZ_simple */
	{"dzerol", {{ 0xa80d0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DZEROL_s037_registerZ_double */
	{"dzerol", {{ 0xa80d0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-DZEROL_w064_registerZ_triple */
	{"errop", {{ 0x00000000, 0x7fffffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-ERROP_simple */
	{"extfs", {{ 0x68030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-EXTFS_registerW_registerZ_u006_u006_simple */
	{"extfz", {{ 0x64030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-EXTFZ_registerW_registerZ_u006_u006_simple */
	{"fabsd", {{ 0x71032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FABSD_registerW_registerZ_simple */
	{"fabshq", {{ 0x77032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FABSHQ_registerW_registerZ_simple */
	{"fabswp", {{ 0x75032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FABSWP_registerW_registerZ_simple */
	{"fabsw", {{ 0x73032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FABSW_registerW_registerZ_simple */
	{"faddcdc.rd", {{ 0x5d032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rd", {{ 0x5d032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rd.s", {{ 0x5d03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rd.s", {{ 0x5d03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rna", {{ 0x5d034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rna", {{ 0x5d034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rna.s", {{ 0x5d03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rna.s", {{ 0x5d03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rnz", {{ 0x5d035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rnz", {{ 0x5d035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rnz.s", {{ 0x5d03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rnz.s", {{ 0x5d03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rn", {{ 0x5d030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rn", {{ 0x5d030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rn.s", {{ 0x5d038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rn.s", {{ 0x5d038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ro", {{ 0x5d036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ro", {{ 0x5d036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ro.s", {{ 0x5d03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ro.s", {{ 0x5d03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ru", {{ 0x5d031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ru", {{ 0x5d031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ru.s", {{ 0x5d039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ru.s", {{ 0x5d039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rz", {{ 0x5d033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rz", {{ 0x5d033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rz.s", {{ 0x5d03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rz.s", {{ 0x5d03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc", {{ 0x5d037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c", {{ 0x5d037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.s", {{ 0x5d03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.s", {{ 0x5d03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rd", {{ 0x59072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rd", {{ 0x59072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rd.s", {{ 0x5907a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rd.s", {{ 0x5907a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rna", {{ 0x59074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rna", {{ 0x59074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rna.s", {{ 0x5907c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rna.s", {{ 0x5907c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rnz", {{ 0x59075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rnz", {{ 0x59075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rnz.s", {{ 0x5907d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rnz.s", {{ 0x5907d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rn", {{ 0x59070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rn", {{ 0x59070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rn.s", {{ 0x59078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rn.s", {{ 0x59078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ro", {{ 0x59076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ro", {{ 0x59076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ro.s", {{ 0x5907e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ro.s", {{ 0x5907e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ru", {{ 0x59071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ru", {{ 0x59071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ru.s", {{ 0x59079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ru.s", {{ 0x59079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rz", {{ 0x59073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rz", {{ 0x59073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rz.s", {{ 0x5907b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rz.s", {{ 0x5907b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp", {{ 0x59077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c", {{ 0x59077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.s", {{ 0x5907f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.s", {{ 0x5907f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwc", {{ 0x43020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_registerW_registerZ_s010_simple */
	{"faddwp.c", {{ 0x43020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_registerW_registerZ_s010_simple */
	{"faddcwc", {{ 0xc3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_registerW_registerZ_s037_double */
	{"faddwp.c", {{ 0xc3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_registerW_registerZ_s037_double */
	{"faddcwc", {{ 0xc3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_registerW_registerZ_w064_triple */
	{"faddwp.c", {{ 0xc3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_registerW_registerZ_w064_triple */
	{"faddcwc.rd", {{ 0x53022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rd", {{ 0x53022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rd.s", {{ 0x5302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rd.s", {{ 0x5302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rna", {{ 0x53024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rna", {{ 0x53024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rna.s", {{ 0x5302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rna.s", {{ 0x5302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rnz", {{ 0x53025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rnz", {{ 0x53025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rnz.s", {{ 0x5302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rnz.s", {{ 0x5302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rn", {{ 0x53020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rn", {{ 0x53020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rn.s", {{ 0x53028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rn.s", {{ 0x53028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ro", {{ 0x53026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ro", {{ 0x53026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ro.s", {{ 0x5302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ro.s", {{ 0x5302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ru", {{ 0x53021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ru", {{ 0x53021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ru.s", {{ 0x53029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ru.s", {{ 0x53029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rz", {{ 0x53023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rz", {{ 0x53023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rz.s", {{ 0x5302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rz.s", {{ 0x5302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc", {{ 0x53027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c", {{ 0x53027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.s", {{ 0x5302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.s", {{ 0x5302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fadddp.rd", {{ 0x5c032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rd.s", {{ 0x5c03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rna", {{ 0x5c034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rna.s", {{ 0x5c03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rnz", {{ 0x5c035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rnz.s", {{ 0x5c03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rn", {{ 0x5c030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rn.s", {{ 0x5c038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ro", {{ 0x5c036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ro.s", {{ 0x5c03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ru", {{ 0x5c031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ru.s", {{ 0x5c039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rz", {{ 0x5c033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rz.s", {{ 0x5c03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp", {{ 0x5c037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.s", {{ 0x5c03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddd", {{ 0x40020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_registerW_registerZ_s010_simple */
	{"faddd", {{ 0xc0020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_registerW_registerZ_s037_double */
	{"faddd", {{ 0xc0020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_registerW_registerZ_w064_triple */
	{"faddd.rd", {{ 0x50022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rd.s", {{ 0x5002a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rna", {{ 0x50024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rna.s", {{ 0x5002c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rnz", {{ 0x50025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rnz.s", {{ 0x5002d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rn", {{ 0x50020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rn.s", {{ 0x50028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ro", {{ 0x50026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ro.s", {{ 0x5002e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ru", {{ 0x50021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ru.s", {{ 0x50029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rz", {{ 0x50023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rz.s", {{ 0x5002b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd", {{ 0x50027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.s", {{ 0x5002f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq", {{ 0x42020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_registerW_registerZ_s010_simple */
	{"faddhq", {{ 0xc2020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_registerW_registerZ_s037_double */
	{"faddhq", {{ 0xc2020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_registerW_registerZ_w064_triple */
	{"faddhq.rd", {{ 0x52022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rd.s", {{ 0x5202a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rna", {{ 0x52024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rna.s", {{ 0x5202c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rnz", {{ 0x52025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rnz.s", {{ 0x5202d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rn", {{ 0x52020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rn.s", {{ 0x52028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ro", {{ 0x52026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ro.s", {{ 0x5202e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ru", {{ 0x52021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ru.s", {{ 0x52029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rz", {{ 0x52023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rz.s", {{ 0x5202b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq", {{ 0x52027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.s", {{ 0x5202f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp", {{ 0x41020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_registerW_registerZ_s010_simple */
	{"faddwp", {{ 0xc1020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_registerW_registerZ_s037_double */
	{"faddwp", {{ 0xc1020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_registerW_registerZ_w064_triple */
	{"faddwp.rd", {{ 0x51022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rd.s", {{ 0x5102a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rna", {{ 0x51024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rna.s", {{ 0x5102c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rnz", {{ 0x51025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rnz.s", {{ 0x5102d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rn", {{ 0x51020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rn.s", {{ 0x51028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ro", {{ 0x51026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ro.s", {{ 0x5102e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ru", {{ 0x51021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ru.s", {{ 0x51029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rz", {{ 0x51023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rz.s", {{ 0x5102b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp", {{ 0x51027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.s", {{ 0x5102f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwq.rd", {{ 0x58072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rd.s", {{ 0x5807a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rna", {{ 0x58074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rna.s", {{ 0x5807c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rnz", {{ 0x58075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rnz.s", {{ 0x5807d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rn", {{ 0x58070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rn.s", {{ 0x58078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ro", {{ 0x58076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ro.s", {{ 0x5807e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ru", {{ 0x58071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ru.s", {{ 0x58079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rz", {{ 0x58073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rz.s", {{ 0x5807b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq", {{ 0x58077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.s", {{ 0x5807f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddw", {{ 0x4c020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_registerW_registerZ_s010_simple */
	{"faddw", {{ 0xcc020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_registerW_registerZ_s037_double */
	{"faddw", {{ 0xcc020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_registerW_registerZ_w064_triple */
	{"faddw.rd", {{ 0x5c022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rd.s", {{ 0x5c02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rna", {{ 0x5c024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rna.s", {{ 0x5c02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rnz", {{ 0x5c025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rnz.s", {{ 0x5c02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rn", {{ 0x5c020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rn.s", {{ 0x5c028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ro", {{ 0x5c026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ro.s", {{ 0x5c02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ru", {{ 0x5c021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ru.s", {{ 0x5c029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rz", {{ 0x5c023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rz.s", {{ 0x5c02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw", {{ 0x5c027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.s", {{ 0x5c02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fcdivd", {{ 0x71035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FCDIVD_silent2_registerW_registerP_simple */
	{"fcdivd.s", {{ 0x71035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FCDIVD_silent2_registerW_registerP_simple */
	{"fcdivwp", {{ 0x75035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FCDIVWP_silent2_registerW_registerP_simple */
	{"fcdivwp.s", {{ 0x75035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FCDIVWP_silent2_registerW_registerP_simple */
	{"fcdivw", {{ 0x73035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FCDIVW_silent2_registerW_registerP_simple */
	{"fcdivw.s", {{ 0x73035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FCDIVW_silent2_registerW_registerP_simple */
	{"fcompd.oeq", {{ 0x7a030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.oge", {{ 0x7e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.olt", {{ 0x7c030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.one", {{ 0x78030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.ueq", {{ 0x79030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.uge", {{ 0x7d030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.ult", {{ 0x7f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.une", {{ 0x7b030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.oeq", {{ 0xfa030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.oge", {{ 0xfe030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.olt", {{ 0xfc030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.one", {{ 0xf8030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.ueq", {{ 0xf9030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.uge", {{ 0xfd030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.ult", {{ 0xff030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.une", {{ 0xfb030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompnhq.oeq", {{ 0x7a031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.oge", {{ 0x7e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.olt", {{ 0x7c031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.one", {{ 0x78031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.ueq", {{ 0x79031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.uge", {{ 0x7d031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.ult", {{ 0x7f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.une", {{ 0x7b031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.oeq", {{ 0x72031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.oge", {{ 0x76031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.olt", {{ 0x74031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.one", {{ 0x70031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.ueq", {{ 0x71031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.uge", {{ 0x75031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.ult", {{ 0x77031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.une", {{ 0x73031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.oeq", {{ 0x72030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.oge", {{ 0x76030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.olt", {{ 0x74030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.one", {{ 0x70030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.ueq", {{ 0x71030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.uge", {{ 0x75030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.ult", {{ 0x77030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.une", {{ 0x73030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.oeq", {{ 0xf2030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.oge", {{ 0xf6030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.olt", {{ 0xf4030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.one", {{ 0xf0030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.ueq", {{ 0xf1030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.uge", {{ 0xf5030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.ult", {{ 0xf7030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.une", {{ 0xf3030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fdot2wdp.rd", {{ 0x5c072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rd.s", {{ 0x5c07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rna", {{ 0x5c074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rna.s", {{ 0x5c07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rnz", {{ 0x5c075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rnz.s", {{ 0x5c07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rn", {{ 0x5c070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rn.s", {{ 0x5c078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ro", {{ 0x5c076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ro.s", {{ 0x5c07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ru", {{ 0x5c071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ru.s", {{ 0x5c079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rz", {{ 0x5c073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rz.s", {{ 0x5c07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp", {{ 0x5c077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.s", {{ 0x5c07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wd", {{ 0x4d010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_registerW_registerZ_s010_simple */
	{"fdot2wd", {{ 0xcd010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_registerW_registerZ_s037_double */
	{"fdot2wd", {{ 0xcd010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_registerW_registerZ_w064_triple */
	{"fdot2wd.rd", {{ 0x5d012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rd.s", {{ 0x5d01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rna", {{ 0x5d014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rna.s", {{ 0x5d01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rnz", {{ 0x5d015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rnz.s", {{ 0x5d01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rn", {{ 0x5d010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rn.s", {{ 0x5d018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ro", {{ 0x5d016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ro.s", {{ 0x5d01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ru", {{ 0x5d011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ru.s", {{ 0x5d019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rz", {{ 0x5d013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rz.s", {{ 0x5d01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd", {{ 0x5d017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.s", {{ 0x5d01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wzp.rd", {{ 0x5d072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rd.s", {{ 0x5d07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rna", {{ 0x5d074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rna.s", {{ 0x5d07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rnz", {{ 0x5d075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rnz.s", {{ 0x5d07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rn", {{ 0x5d070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rn.s", {{ 0x5d078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ro", {{ 0x5d076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ro.s", {{ 0x5d07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ru", {{ 0x5d071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ru.s", {{ 0x5d079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rz", {{ 0x5d073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rz.s", {{ 0x5d07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp", {{ 0x5d077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.s", {{ 0x5d07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2w", {{ 0x4c010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_registerW_registerZ_s010_simple */
	{"fdot2w", {{ 0xcc010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_registerW_registerZ_s037_double */
	{"fdot2w", {{ 0xcc010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_registerW_registerZ_w064_triple */
	{"fdot2w.rd", {{ 0x5c012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rd.s", {{ 0x5c01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rna", {{ 0x5c014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rna.s", {{ 0x5c01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rnz", {{ 0x5c015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rnz.s", {{ 0x5c01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rn", {{ 0x5c010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rn.s", {{ 0x5c018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ro", {{ 0x5c016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ro.s", {{ 0x5c01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ru", {{ 0x5c011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ru.s", {{ 0x5c019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rz", {{ 0x5c013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rz.s", {{ 0x5c01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w", {{ 0x5c017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.s", {{ 0x5c01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fence", {{ 0x3fcd0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {0}, "", ""}, /* Opcode-v1-FENCE_simple */
	{"ffmad", {{ 0x40010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_registerW_registerZ_s010_simple */
	{"ffmad", {{ 0xc0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_registerW_registerZ_s037_double */
	{"ffmad", {{ 0xc0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_registerW_registerZ_w064_triple */
	{"ffmad.rd", {{ 0x50012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rd.s", {{ 0x5001a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rna", {{ 0x50014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rna.s", {{ 0x5001c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rnz", {{ 0x50015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rnz.s", {{ 0x5001d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rn", {{ 0x50010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rn.s", {{ 0x50018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ro", {{ 0x50016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ro.s", {{ 0x5001e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ru", {{ 0x50011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ru.s", {{ 0x50019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rz", {{ 0x50013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rz.s", {{ 0x5001b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad", {{ 0x50017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.s", {{ 0x5001f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq", {{ 0x43010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_registerW_registerZ_s010_simple */
	{"ffmahq", {{ 0xc3010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_registerW_registerZ_s037_double */
	{"ffmahq", {{ 0xc3010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_registerW_registerZ_w064_triple */
	{"ffmahq.rd", {{ 0x53012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rd.s", {{ 0x5301a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rna", {{ 0x53014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rna.s", {{ 0x5301c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rnz", {{ 0x53015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rnz.s", {{ 0x5301d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rn", {{ 0x53010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rn.s", {{ 0x53018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ro", {{ 0x53016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ro.s", {{ 0x5301e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ru", {{ 0x53011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ru.s", {{ 0x53019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rz", {{ 0x53013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rz.s", {{ 0x5301b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq", {{ 0x53017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.s", {{ 0x5301f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahwq", {{ 0x41030000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_registerM_registerZ_s010_simple */
	{"ffmahwq", {{ 0xc1030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_registerM_registerZ_s037_double */
	{"ffmahwq", {{ 0xc1030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_registerM_registerZ_w064_triple */
	{"ffmahwq.rd", {{ 0x51032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rd.s", {{ 0x5103a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rna", {{ 0x51034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rna.s", {{ 0x5103c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rnz", {{ 0x51035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rnz.s", {{ 0x5103d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rn", {{ 0x51030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rn.s", {{ 0x51038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ro", {{ 0x51036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ro.s", {{ 0x5103e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ru", {{ 0x51031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ru.s", {{ 0x51039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rz", {{ 0x51033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rz.s", {{ 0x5103b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq", {{ 0x51037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.s", {{ 0x5103f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahw", {{ 0x48020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_registerW_registerZ_s010_simple */
	{"ffmahw", {{ 0xc8020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_registerW_registerZ_s037_double */
	{"ffmahw", {{ 0xc8020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_registerW_registerZ_w064_triple */
	{"ffmahw.rd", {{ 0x58022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rd.s", {{ 0x5802a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rna", {{ 0x58024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rna.s", {{ 0x5802c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rnz", {{ 0x58025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rnz.s", {{ 0x5802d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rn", {{ 0x58020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rn.s", {{ 0x58028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ro", {{ 0x58026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ro.s", {{ 0x5802e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ru", {{ 0x58021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ru.s", {{ 0x58029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rz", {{ 0x58023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rz.s", {{ 0x5802b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw", {{ 0x58027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.s", {{ 0x5802f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawdp", {{ 0x40030000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_registerM_registerZ_s010_simple */
	{"ffmawdp", {{ 0xc0030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_registerM_registerZ_s037_double */
	{"ffmawdp", {{ 0xc0030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_registerM_registerZ_w064_triple */
	{"ffmawdp.rd", {{ 0x50032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rd.s", {{ 0x5003a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rna", {{ 0x50034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rna.s", {{ 0x5003c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rnz", {{ 0x50035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rnz.s", {{ 0x5003d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rn", {{ 0x50030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rn.s", {{ 0x50038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ro", {{ 0x50036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ro.s", {{ 0x5003e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ru", {{ 0x50031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ru.s", {{ 0x50039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rz", {{ 0x50033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rz.s", {{ 0x5003b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp", {{ 0x50037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.s", {{ 0x5003f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawd", {{ 0x41010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_registerW_registerZ_s010_simple */
	{"ffmawd", {{ 0xc1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_registerW_registerZ_s037_double */
	{"ffmawd", {{ 0xc1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_registerW_registerZ_w064_triple */
	{"ffmawd.rd", {{ 0x51012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rd.s", {{ 0x5101a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rna", {{ 0x51014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rna.s", {{ 0x5101c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rnz", {{ 0x51015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rnz.s", {{ 0x5101d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rn", {{ 0x51010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rn.s", {{ 0x51018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ro", {{ 0x51016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ro.s", {{ 0x5101e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ru", {{ 0x51011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ru.s", {{ 0x51019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rz", {{ 0x51013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rz.s", {{ 0x5101b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd", {{ 0x51017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.s", {{ 0x5101f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp", {{ 0x42010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_registerW_registerZ_s010_simple */
	{"ffmawp", {{ 0xc2010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_registerW_registerZ_s037_double */
	{"ffmawp", {{ 0xc2010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_registerW_registerZ_w064_triple */
	{"ffmawp.rd", {{ 0x52012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rd.s", {{ 0x5201a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rna", {{ 0x52014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rna.s", {{ 0x5201c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rnz", {{ 0x52015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rnz.s", {{ 0x5201d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rn", {{ 0x52010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rn.s", {{ 0x52018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ro", {{ 0x52016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ro.s", {{ 0x5201e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ru", {{ 0x52011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ru.s", {{ 0x52019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rz", {{ 0x52013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rz.s", {{ 0x5201b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp", {{ 0x52017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.s", {{ 0x5201f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw", {{ 0x49020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_registerW_registerZ_s010_simple */
	{"ffmaw", {{ 0xc9020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_registerW_registerZ_s037_double */
	{"ffmaw", {{ 0xc9020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_registerW_registerZ_w064_triple */
	{"ffmaw.rd", {{ 0x59022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rd.s", {{ 0x5902a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rna", {{ 0x59024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rna.s", {{ 0x5902c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rnz", {{ 0x59025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rnz.s", {{ 0x5902d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rn", {{ 0x59020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rn.s", {{ 0x59028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ro", {{ 0x59026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ro.s", {{ 0x5902e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ru", {{ 0x59021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ru.s", {{ 0x59029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rz", {{ 0x59023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rz.s", {{ 0x5902b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw", {{ 0x59027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.s", {{ 0x5902f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd", {{ 0x44010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_registerW_registerZ_s010_simple */
	{"ffmsd", {{ 0xc4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_registerW_registerZ_s037_double */
	{"ffmsd", {{ 0xc4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_registerW_registerZ_w064_triple */
	{"ffmsd.rd", {{ 0x54012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rd.s", {{ 0x5401a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rna", {{ 0x54014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rna.s", {{ 0x5401c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rnz", {{ 0x54015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rnz.s", {{ 0x5401d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rn", {{ 0x54010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rn.s", {{ 0x54018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ro", {{ 0x54016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ro.s", {{ 0x5401e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ru", {{ 0x54011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ru.s", {{ 0x54019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rz", {{ 0x54013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rz.s", {{ 0x5401b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd", {{ 0x54017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.s", {{ 0x5401f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq", {{ 0x47010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_registerW_registerZ_s010_simple */
	{"ffmshq", {{ 0xc7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_registerW_registerZ_s037_double */
	{"ffmshq", {{ 0xc7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_registerW_registerZ_w064_triple */
	{"ffmshq.rd", {{ 0x57012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rd.s", {{ 0x5701a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rna", {{ 0x57014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rna.s", {{ 0x5701c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rnz", {{ 0x57015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rnz.s", {{ 0x5701d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rn", {{ 0x57010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rn.s", {{ 0x57018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ro", {{ 0x57016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ro.s", {{ 0x5701e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ru", {{ 0x57011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ru.s", {{ 0x57019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rz", {{ 0x57013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rz.s", {{ 0x5701b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq", {{ 0x57017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.s", {{ 0x5701f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshwq", {{ 0x43030000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_registerM_registerZ_s010_simple */
	{"ffmshwq", {{ 0xc3030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_registerM_registerZ_s037_double */
	{"ffmshwq", {{ 0xc3030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_registerM_registerZ_w064_triple */
	{"ffmshwq.rd", {{ 0x53032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rd.s", {{ 0x5303a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rna", {{ 0x53034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rna.s", {{ 0x5303c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rnz", {{ 0x53035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rnz.s", {{ 0x5303d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rn", {{ 0x53030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rn.s", {{ 0x53038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ro", {{ 0x53036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ro.s", {{ 0x5303e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ru", {{ 0x53031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ru.s", {{ 0x53039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rz", {{ 0x53033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rz.s", {{ 0x5303b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq", {{ 0x53037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.s", {{ 0x5303f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshw", {{ 0x4a020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_registerW_registerZ_s010_simple */
	{"ffmshw", {{ 0xca020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_registerW_registerZ_s037_double */
	{"ffmshw", {{ 0xca020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_registerW_registerZ_w064_triple */
	{"ffmshw.rd", {{ 0x5a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rd.s", {{ 0x5a02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rna", {{ 0x5a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rna.s", {{ 0x5a02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rnz", {{ 0x5a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rnz.s", {{ 0x5a02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rn", {{ 0x5a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rn.s", {{ 0x5a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ro", {{ 0x5a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ro.s", {{ 0x5a02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ru", {{ 0x5a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ru.s", {{ 0x5a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rz", {{ 0x5a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rz.s", {{ 0x5a02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw", {{ 0x5a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.s", {{ 0x5a02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswdp", {{ 0x42030000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_registerM_registerZ_s010_simple */
	{"ffmswdp", {{ 0xc2030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_registerM_registerZ_s037_double */
	{"ffmswdp", {{ 0xc2030000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_registerM_registerZ_w064_triple */
	{"ffmswdp.rd", {{ 0x52032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rd.s", {{ 0x5203a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rna", {{ 0x52034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rna.s", {{ 0x5203c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rnz", {{ 0x52035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rnz.s", {{ 0x5203d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rn", {{ 0x52030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rn.s", {{ 0x52038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ro", {{ 0x52036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ro.s", {{ 0x5203e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ru", {{ 0x52031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ru.s", {{ 0x52039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rz", {{ 0x52033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rz.s", {{ 0x5203b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp", {{ 0x52037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.s", {{ 0x5203f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswd", {{ 0x45010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_registerW_registerZ_s010_simple */
	{"ffmswd", {{ 0xc5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_registerW_registerZ_s037_double */
	{"ffmswd", {{ 0xc5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_registerW_registerZ_w064_triple */
	{"ffmswd.rd", {{ 0x55012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rd.s", {{ 0x5501a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rna", {{ 0x55014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rna.s", {{ 0x5501c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rnz", {{ 0x55015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rnz.s", {{ 0x5501d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rn", {{ 0x55010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rn.s", {{ 0x55018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ro", {{ 0x55016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ro.s", {{ 0x5501e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ru", {{ 0x55011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ru.s", {{ 0x55019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rz", {{ 0x55013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rz.s", {{ 0x5501b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd", {{ 0x55017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.s", {{ 0x5501f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp", {{ 0x46010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_registerW_registerZ_s010_simple */
	{"ffmswp", {{ 0xc6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_registerW_registerZ_s037_double */
	{"ffmswp", {{ 0xc6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_registerW_registerZ_w064_triple */
	{"ffmswp.rd", {{ 0x56012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rd.s", {{ 0x5601a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rna", {{ 0x56014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rna.s", {{ 0x5601c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rnz", {{ 0x56015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rnz.s", {{ 0x5601d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rn", {{ 0x56010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rn.s", {{ 0x56018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ro", {{ 0x56016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ro.s", {{ 0x5601e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ru", {{ 0x56011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ru.s", {{ 0x56019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rz", {{ 0x56013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rz.s", {{ 0x5601b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp", {{ 0x56017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.s", {{ 0x5601f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw", {{ 0x4b020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_registerW_registerZ_s010_simple */
	{"ffmsw", {{ 0xcb020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_registerW_registerZ_s037_double */
	{"ffmsw", {{ 0xcb020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_registerW_registerZ_w064_triple */
	{"ffmsw.rd", {{ 0x5b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rd.s", {{ 0x5b02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rna", {{ 0x5b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rna.s", {{ 0x5b02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rnz", {{ 0x5b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rnz.s", {{ 0x5b02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rn", {{ 0x5b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rn.s", {{ 0x5b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ro", {{ 0x5b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ro.s", {{ 0x5b02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ru", {{ 0x5b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ru.s", {{ 0x5b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rz", {{ 0x5b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rz.s", {{ 0x5b02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw", {{ 0x5b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.s", {{ 0x5b02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fixedd.rd", {{ 0x46032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rd.s", {{ 0x4603a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rna", {{ 0x46034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rna.s", {{ 0x4603c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rnz", {{ 0x46035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rnz.s", {{ 0x4603d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rn", {{ 0x46030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rn.s", {{ 0x46038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ro", {{ 0x46036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ro.s", {{ 0x4603e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ru", {{ 0x46031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ru.s", {{ 0x46039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rz", {{ 0x46033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rz.s", {{ 0x4603b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd", {{ 0x46037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.s", {{ 0x4603f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rd", {{ 0x47032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rd.s", {{ 0x4703a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rna", {{ 0x47034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rna.s", {{ 0x4703c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rnz", {{ 0x47035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rnz.s", {{ 0x4703d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rn", {{ 0x47030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rn.s", {{ 0x47038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ro", {{ 0x47036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ro.s", {{ 0x4703e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ru", {{ 0x47031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ru.s", {{ 0x47039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rz", {{ 0x47033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rz.s", {{ 0x4703b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud", {{ 0x47037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.s", {{ 0x4703f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rd", {{ 0x4f032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rd.s", {{ 0x4f03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rna", {{ 0x4f034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rna.s", {{ 0x4f03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rnz", {{ 0x4f035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rnz.s", {{ 0x4f03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rn", {{ 0x4f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rn.s", {{ 0x4f038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ro", {{ 0x4f036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ro.s", {{ 0x4f03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ru", {{ 0x4f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ru.s", {{ 0x4f039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rz", {{ 0x4f033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rz.s", {{ 0x4f03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp", {{ 0x4f037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.s", {{ 0x4f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rd", {{ 0x4b032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rd.s", {{ 0x4b03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rna", {{ 0x4b034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rna.s", {{ 0x4b03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rnz", {{ 0x4b035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rnz.s", {{ 0x4b03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rn", {{ 0x4b030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rn.s", {{ 0x4b038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ro", {{ 0x4b036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ro.s", {{ 0x4b03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ru", {{ 0x4b031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ru.s", {{ 0x4b039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rz", {{ 0x4b033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rz.s", {{ 0x4b03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw", {{ 0x4b037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.s", {{ 0x4b03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rd", {{ 0x4e032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rd.s", {{ 0x4e03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rna", {{ 0x4e034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rna.s", {{ 0x4e03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rnz", {{ 0x4e035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rnz.s", {{ 0x4e03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rn", {{ 0x4e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rn.s", {{ 0x4e038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ro", {{ 0x4e036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ro.s", {{ 0x4e03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ru", {{ 0x4e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ru.s", {{ 0x4e039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rz", {{ 0x4e033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rz.s", {{ 0x4e03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp", {{ 0x4e037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.s", {{ 0x4e03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rd", {{ 0x4a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rd.s", {{ 0x4a03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rna", {{ 0x4a034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rna.s", {{ 0x4a03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rnz", {{ 0x4a035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rnz.s", {{ 0x4a03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rn", {{ 0x4a030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rn.s", {{ 0x4a038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ro", {{ 0x4a036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ro.s", {{ 0x4a03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ru", {{ 0x4a031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ru.s", {{ 0x4a039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rz", {{ 0x4a033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rz.s", {{ 0x4a03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw", {{ 0x4a037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.s", {{ 0x4a03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rd", {{ 0x44032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rd.s", {{ 0x4403a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rna", {{ 0x44034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rna.s", {{ 0x4403c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rnz", {{ 0x44035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rnz.s", {{ 0x4403d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rn", {{ 0x44030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rn.s", {{ 0x44038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ro", {{ 0x44036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ro.s", {{ 0x4403e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ru", {{ 0x44031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ru.s", {{ 0x44039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rz", {{ 0x44033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rz.s", {{ 0x4403b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd", {{ 0x44037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.s", {{ 0x4403f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rd", {{ 0x45032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rd.s", {{ 0x4503a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rna", {{ 0x45034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rna.s", {{ 0x4503c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rnz", {{ 0x45035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rnz.s", {{ 0x4503d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rn", {{ 0x45030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rn.s", {{ 0x45038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ro", {{ 0x45036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ro.s", {{ 0x4503e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ru", {{ 0x45031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ru.s", {{ 0x45039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rz", {{ 0x45033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rz.s", {{ 0x4503b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud", {{ 0x45037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.s", {{ 0x4503f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rd", {{ 0x4d032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rd.s", {{ 0x4d03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rna", {{ 0x4d034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rna.s", {{ 0x4d03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rnz", {{ 0x4d035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rnz.s", {{ 0x4d03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rn", {{ 0x4d030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rn.s", {{ 0x4d038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ro", {{ 0x4d036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ro.s", {{ 0x4d03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ru", {{ 0x4d031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ru.s", {{ 0x4d039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rz", {{ 0x4d033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rz.s", {{ 0x4d03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp", {{ 0x4d037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.s", {{ 0x4d03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rd", {{ 0x49032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rd.s", {{ 0x4903a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rna", {{ 0x49034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rna.s", {{ 0x4903c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rnz", {{ 0x49035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rnz.s", {{ 0x4903d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rn", {{ 0x49030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rn.s", {{ 0x49038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ro", {{ 0x49036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ro.s", {{ 0x4903e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ru", {{ 0x49031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ru.s", {{ 0x49039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rz", {{ 0x49033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rz.s", {{ 0x4903b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw", {{ 0x49037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.s", {{ 0x4903f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rd", {{ 0x4c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rd.s", {{ 0x4c03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rna", {{ 0x4c034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rna.s", {{ 0x4c03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rnz", {{ 0x4c035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rnz.s", {{ 0x4c03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rn", {{ 0x4c030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rn.s", {{ 0x4c038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ro", {{ 0x4c036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ro.s", {{ 0x4c03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ru", {{ 0x4c031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ru.s", {{ 0x4c039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rz", {{ 0x4c033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rz.s", {{ 0x4c03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp", {{ 0x4c037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.s", {{ 0x4c03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rd", {{ 0x48032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rd.s", {{ 0x4803a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rna", {{ 0x48034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rna.s", {{ 0x4803c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rnz", {{ 0x48035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rnz.s", {{ 0x4803d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rn", {{ 0x48030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rn.s", {{ 0x48038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ro", {{ 0x48036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ro.s", {{ 0x4803e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ru", {{ 0x48031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ru.s", {{ 0x48039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rz", {{ 0x48033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rz.s", {{ 0x4803b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw", {{ 0x48037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.s", {{ 0x4803f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"fmaxd", {{ 0x71038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMAXD_registerW_registerZ_registerY_simple */
	{"fmaxhq", {{ 0x77038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMAXHQ_registerW_registerZ_registerY_simple */
	{"fmaxwp", {{ 0x75038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMAXWP_registerW_registerZ_registerY_simple */
	{"fmaxw", {{ 0x73038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMAXW_registerW_registerZ_registerY_simple */
	{"fmind", {{ 0x70038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMIND_registerW_registerZ_registerY_simple */
	{"fminhq", {{ 0x76038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMINHQ_registerW_registerZ_registerY_simple */
	{"fminwp", {{ 0x74038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMINWP_registerW_registerZ_registerY_simple */
	{"fminw", {{ 0x72038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMINW_registerW_registerZ_registerY_simple */
	{"fmm212w.rd", {{ 0x54072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rd", {{ 0x54072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rd.s", {{ 0x5407a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rd.s", {{ 0x5407a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rna", {{ 0x54074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rna", {{ 0x54074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rna.s", {{ 0x5407c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rna.s", {{ 0x5407c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rnz", {{ 0x54075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rnz", {{ 0x54075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rnz.s", {{ 0x5407d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rnz.s", {{ 0x5407d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rn", {{ 0x54070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rn", {{ 0x54070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rn.s", {{ 0x54078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rn.s", {{ 0x54078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ro", {{ 0x54076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ro", {{ 0x54076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ro.s", {{ 0x5407e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ro.s", {{ 0x5407e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ru", {{ 0x54071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ru", {{ 0x54071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ru.s", {{ 0x54079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ru.s", {{ 0x54079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rz", {{ 0x54073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rz", {{ 0x54073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rz.s", {{ 0x5407b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rz.s", {{ 0x5407b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w", {{ 0x54077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq", {{ 0x54077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.s", {{ 0x5407f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.s", {{ 0x5407f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rd", {{ 0x54032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rd", {{ 0x54032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rd.s", {{ 0x5403a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rd.s", {{ 0x5403a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rna", {{ 0x54034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rna", {{ 0x54034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rna.s", {{ 0x5403c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rna.s", {{ 0x5403c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rnz", {{ 0x54035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rnz", {{ 0x54035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rnz.s", {{ 0x5403d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rnz.s", {{ 0x5403d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rn", {{ 0x54030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rn", {{ 0x54030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rn.s", {{ 0x54038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rn.s", {{ 0x54038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ro", {{ 0x54036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ro", {{ 0x54036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ro.s", {{ 0x5403e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ro.s", {{ 0x5403e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ru", {{ 0x54031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ru", {{ 0x54031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ru.s", {{ 0x54039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ru.s", {{ 0x54039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rz", {{ 0x54033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rz", {{ 0x54033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rz.s", {{ 0x5403b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rz.s", {{ 0x5403b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w", {{ 0x54037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq", {{ 0x54037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.s", {{ 0x5403f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.s", {{ 0x5403f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma242hw0", {{ 0x03000000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera0_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW0_registerA0_registerBp_registerC_registerD_simple */
	{"fmm4ahw0", {{ 0x03000000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera0_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW0_registerA0_registerBp_registerC_registerD_simple */
	{"fmma242hw1", {{ 0x03001000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera1_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW1_registerA1_registerBp_registerC_registerD_simple */
	{"fmm4ahw1", {{ 0x03001000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera1_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW1_registerA1_registerBp_registerC_registerD_simple */
	{"fmma242hw2", {{ 0x03040000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera2_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW2_registerA2_registerBp_registerC_registerD_simple */
	{"fmm4ahw2", {{ 0x03040000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera2_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW2_registerA2_registerBp_registerC_registerD_simple */
	{"fmma242hw3", {{ 0x03041000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera3_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW3_registerA3_registerBp_registerC_registerD_simple */
	{"fmm4ahw3", {{ 0x03041000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera3_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-FMMA242HW3_registerA3_registerBp_registerC_registerD_simple */
	{"fmms212w.rd", {{ 0x56032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rd", {{ 0x56032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rd.s", {{ 0x5603a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rd.s", {{ 0x5603a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rna", {{ 0x56034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rna", {{ 0x56034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rna.s", {{ 0x5603c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rna.s", {{ 0x5603c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rnz", {{ 0x56035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rnz", {{ 0x56035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rnz.s", {{ 0x5603d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rnz.s", {{ 0x5603d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rn", {{ 0x56030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rn", {{ 0x56030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rn.s", {{ 0x56038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rn.s", {{ 0x56038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ro", {{ 0x56036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ro", {{ 0x56036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ro.s", {{ 0x5603e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ro.s", {{ 0x5603e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ru", {{ 0x56031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ru", {{ 0x56031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ru.s", {{ 0x56039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ru.s", {{ 0x56039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rz", {{ 0x56033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rz", {{ 0x56033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rz.s", {{ 0x5603b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rz.s", {{ 0x5603b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w", {{ 0x56037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq", {{ 0x56037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.s", {{ 0x5603f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.s", {{ 0x5603f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwc", {{ 0x4f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_registerW_registerZ_s010_simple */
	{"fmulwc.c", {{ 0x4f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_registerW_registerZ_s010_simple */
	{"fmulcwc", {{ 0xcf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_registerW_registerZ_s037_double */
	{"fmulwc.c", {{ 0xcf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_registerW_registerZ_s037_double */
	{"fmulcwc", {{ 0xcf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_registerW_registerZ_w064_triple */
	{"fmulwc.c", {{ 0xcf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_registerW_registerZ_w064_triple */
	{"fmulcwc.rd", {{ 0x5f012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rd", {{ 0x5f012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rd.s", {{ 0x5f01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rd.s", {{ 0x5f01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rna", {{ 0x5f014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rna", {{ 0x5f014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rna.s", {{ 0x5f01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rna.s", {{ 0x5f01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rnz", {{ 0x5f015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rnz", {{ 0x5f015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rnz.s", {{ 0x5f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rnz.s", {{ 0x5f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rn", {{ 0x5f010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rn", {{ 0x5f010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rn.s", {{ 0x5f018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rn.s", {{ 0x5f018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ro", {{ 0x5f016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ro", {{ 0x5f016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ro.s", {{ 0x5f01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ro.s", {{ 0x5f01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ru", {{ 0x5f011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ru", {{ 0x5f011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ru.s", {{ 0x5f019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ru.s", {{ 0x5f019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rz", {{ 0x5f013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rz", {{ 0x5f013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rz.s", {{ 0x5f01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rz.s", {{ 0x5f01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc", {{ 0x5f017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c", {{ 0x5f017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.s", {{ 0x5f01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.s", {{ 0x5f01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwdc", {{ 0x43070000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_registerM_registerZ_s010_simple */
	{"fmulcwdc", {{ 0xc3070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_registerM_registerZ_s037_double */
	{"fmulcwdc", {{ 0xc3070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_registerM_registerZ_w064_triple */
	{"fmulcwdc.rd", {{ 0x53072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rd.s", {{ 0x5307a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rna", {{ 0x53074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rna.s", {{ 0x5307c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rnz", {{ 0x53075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rnz.s", {{ 0x5307d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rn", {{ 0x53070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rn.s", {{ 0x53078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.ro", {{ 0x53076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.ro.s", {{ 0x5307e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.ru", {{ 0x53071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.ru.s", {{ 0x53079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rz", {{ 0x53073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.rz.s", {{ 0x5307b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc", {{ 0x53077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulcwdc.s", {{ 0x5307f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULCWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmuld", {{ 0x48010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_registerW_registerZ_s010_simple */
	{"fmuld", {{ 0xc8010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_registerW_registerZ_s037_double */
	{"fmuld", {{ 0xc8010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_registerW_registerZ_w064_triple */
	{"fmuld.rd", {{ 0x58012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rd.s", {{ 0x5801a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rna", {{ 0x58014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rna.s", {{ 0x5801c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rnz", {{ 0x58015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rnz.s", {{ 0x5801d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rn", {{ 0x58010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rn.s", {{ 0x58018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ro", {{ 0x58016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ro.s", {{ 0x5801e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ru", {{ 0x58011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ru.s", {{ 0x58019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rz", {{ 0x58013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rz.s", {{ 0x5801b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld", {{ 0x58017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.s", {{ 0x5801f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq", {{ 0x4b010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_registerW_registerZ_s010_simple */
	{"fmulhq", {{ 0xcb010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_registerW_registerZ_s037_double */
	{"fmulhq", {{ 0xcb010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_registerW_registerZ_w064_triple */
	{"fmulhq.rd", {{ 0x5b012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rd.s", {{ 0x5b01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rna", {{ 0x5b014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rna.s", {{ 0x5b01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rnz", {{ 0x5b015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rnz.s", {{ 0x5b01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rn", {{ 0x5b010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rn.s", {{ 0x5b018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ro", {{ 0x5b016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ro.s", {{ 0x5b01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ru", {{ 0x5b011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ru.s", {{ 0x5b019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rz", {{ 0x5b013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rz.s", {{ 0x5b01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq", {{ 0x5b017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.s", {{ 0x5b01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhwq", {{ 0x41070000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_registerM_registerZ_s010_simple */
	{"fmulhwq", {{ 0xc1070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_registerM_registerZ_s037_double */
	{"fmulhwq", {{ 0xc1070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_registerM_registerZ_w064_triple */
	{"fmulhwq.rd", {{ 0x51072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rd.s", {{ 0x5107a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rna", {{ 0x51074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rna.s", {{ 0x5107c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rnz", {{ 0x51075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rnz.s", {{ 0x5107d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rn", {{ 0x51070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rn.s", {{ 0x51078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ro", {{ 0x51076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ro.s", {{ 0x5107e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ru", {{ 0x51071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ru.s", {{ 0x51079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rz", {{ 0x51073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rz.s", {{ 0x5107b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq", {{ 0x51077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.s", {{ 0x5107f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhw", {{ 0x4f020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_registerW_registerZ_s010_simple */
	{"fmulhw", {{ 0xcf020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_registerW_registerZ_s037_double */
	{"fmulhw", {{ 0xcf020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_registerW_registerZ_w064_triple */
	{"fmulhw.rd", {{ 0x5f022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rd.s", {{ 0x5f02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rna", {{ 0x5f024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rna.s", {{ 0x5f02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rnz", {{ 0x5f025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rnz.s", {{ 0x5f02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rn", {{ 0x5f020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rn.s", {{ 0x5f028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ro", {{ 0x5f026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ro.s", {{ 0x5f02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ru", {{ 0x5f021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ru.s", {{ 0x5f029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rz", {{ 0x5f023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rz.s", {{ 0x5f02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw", {{ 0x5f027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.s", {{ 0x5f02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc", {{ 0x4e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_registerW_registerZ_s010_simple */
	{"fmulwc", {{ 0xce010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_registerW_registerZ_s037_double */
	{"fmulwc", {{ 0xce010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_registerW_registerZ_w064_triple */
	{"fmulwc.rd", {{ 0x5e012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rd.s", {{ 0x5e01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rna", {{ 0x5e014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rna.s", {{ 0x5e01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rnz", {{ 0x5e015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rnz.s", {{ 0x5e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rn", {{ 0x5e010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rn.s", {{ 0x5e018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ro", {{ 0x5e016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ro.s", {{ 0x5e01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ru", {{ 0x5e011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ru.s", {{ 0x5e019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rz", {{ 0x5e013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rz.s", {{ 0x5e01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc", {{ 0x5e017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.s", {{ 0x5e01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwdc", {{ 0x42070000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_registerM_registerZ_s010_simple */
	{"fmulwdc", {{ 0xc2070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_registerM_registerZ_s037_double */
	{"fmulwdc", {{ 0xc2070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_registerM_registerZ_w064_triple */
	{"fmulwdc.rd", {{ 0x52072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rd.s", {{ 0x5207a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rna", {{ 0x52074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rna.s", {{ 0x5207c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rnz", {{ 0x52075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rnz.s", {{ 0x5207d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rn", {{ 0x52070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rn.s", {{ 0x52078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.ro", {{ 0x52076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.ro.s", {{ 0x5207e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.ru", {{ 0x52071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.ru.s", {{ 0x52079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rz", {{ 0x52073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.rz.s", {{ 0x5207b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc", {{ 0x52077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdc.s", {{ 0x5207f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDC_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp", {{ 0x40070000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_registerM_registerZ_s010_simple */
	{"fmulwdp", {{ 0xc0070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_registerM_registerZ_s037_double */
	{"fmulwdp", {{ 0xc0070000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_registerM_registerZ_w064_triple */
	{"fmulwdp.rd", {{ 0x50072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rd.s", {{ 0x5007a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rna", {{ 0x50074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rna.s", {{ 0x5007c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rnz", {{ 0x50075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rnz.s", {{ 0x5007d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rn", {{ 0x50070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rn.s", {{ 0x50078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ro", {{ 0x50076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ro.s", {{ 0x5007e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ru", {{ 0x50071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ru.s", {{ 0x50079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rz", {{ 0x50073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rz.s", {{ 0x5007b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp", {{ 0x50077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.s", {{ 0x5007f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwd", {{ 0x49010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_registerW_registerZ_s010_simple */
	{"fmulwd", {{ 0xc9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_registerW_registerZ_s037_double */
	{"fmulwd", {{ 0xc9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_registerW_registerZ_w064_triple */
	{"fmulwd.rd", {{ 0x59012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rd.s", {{ 0x5901a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rna", {{ 0x59014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rna.s", {{ 0x5901c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rnz", {{ 0x59015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rnz.s", {{ 0x5901d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rn", {{ 0x59010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rn.s", {{ 0x59018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ro", {{ 0x59016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ro.s", {{ 0x5901e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ru", {{ 0x59011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ru.s", {{ 0x59019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rz", {{ 0x59013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rz.s", {{ 0x5901b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd", {{ 0x59017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.s", {{ 0x5901f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp", {{ 0x4a010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_registerW_registerZ_s010_simple */
	{"fmulwp", {{ 0xca010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_registerW_registerZ_s037_double */
	{"fmulwp", {{ 0xca010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_registerW_registerZ_w064_triple */
	{"fmulwp.rd", {{ 0x5a012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rd.s", {{ 0x5a01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rna", {{ 0x5a014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rna.s", {{ 0x5a01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rnz", {{ 0x5a015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rnz.s", {{ 0x5a01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rn", {{ 0x5a010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rn.s", {{ 0x5a018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ro", {{ 0x5a016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ro.s", {{ 0x5a01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ru", {{ 0x5a011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ru.s", {{ 0x5a019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rz", {{ 0x5a013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rz.s", {{ 0x5a01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp", {{ 0x5a017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.s", {{ 0x5a01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwq.rd", {{ 0x5e072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rd.s", {{ 0x5e07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rna", {{ 0x5e074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rna.s", {{ 0x5e07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rnz", {{ 0x5e075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rnz.s", {{ 0x5e07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rn", {{ 0x5e070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rn.s", {{ 0x5e078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ro", {{ 0x5e076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ro.s", {{ 0x5e07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ru", {{ 0x5e071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ru.s", {{ 0x5e079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rz", {{ 0x5e073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rz.s", {{ 0x5e07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq", {{ 0x5e077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.s", {{ 0x5e07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulw", {{ 0x4e020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_registerW_registerZ_s010_simple */
	{"fmulw", {{ 0xce020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_registerW_registerZ_s037_double */
	{"fmulw", {{ 0xce020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_registerW_registerZ_w064_triple */
	{"fmulw.rd", {{ 0x5e022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rd.s", {{ 0x5e02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rna", {{ 0x5e024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rna.s", {{ 0x5e02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rnz", {{ 0x5e025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rnz.s", {{ 0x5e02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rn", {{ 0x5e020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rn.s", {{ 0x5e028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ro", {{ 0x5e026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ro.s", {{ 0x5e02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ru", {{ 0x5e021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ru.s", {{ 0x5e029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rz", {{ 0x5e023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rz.s", {{ 0x5e02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw", {{ 0x5e027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.s", {{ 0x5e02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fnarrowdwp.rd", {{ 0x7c036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rd.s", {{ 0x7c036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rna", {{ 0x7c036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rna.s", {{ 0x7c036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rnz", {{ 0x7c036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rnz.s", {{ 0x7c036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rn", {{ 0x7c036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rn.s", {{ 0x7c036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ro", {{ 0x7c036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ro.s", {{ 0x7c036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ru", {{ 0x7c036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ru.s", {{ 0x7c036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rz", {{ 0x7c036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rz.s", {{ 0x7c036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp", {{ 0x7c036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.s", {{ 0x7c036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdw.rd", {{ 0x78036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rd.s", {{ 0x78036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rna", {{ 0x78036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rna.s", {{ 0x78036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rnz", {{ 0x78036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rnz.s", {{ 0x78036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rn", {{ 0x78036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rn.s", {{ 0x78036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ro", {{ 0x78036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ro.s", {{ 0x78036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ru", {{ 0x78036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ru.s", {{ 0x78036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rz", {{ 0x78036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rz.s", {{ 0x78036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw", {{ 0x78036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.s", {{ 0x78036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwhq.rd", {{ 0x7e036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rd.s", {{ 0x7e036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rna", {{ 0x7e036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rna.s", {{ 0x7e036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rnz", {{ 0x7e036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rnz.s", {{ 0x7e036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rn", {{ 0x7e036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rn.s", {{ 0x7e036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ro", {{ 0x7e036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ro.s", {{ 0x7e036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ru", {{ 0x7e036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ru.s", {{ 0x7e036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rz", {{ 0x7e036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rz.s", {{ 0x7e036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq", {{ 0x7e036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.s", {{ 0x7e036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhv.rd", {{ 0x07000240, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rd.s", {{ 0x07000a40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rna", {{ 0x07000440, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rna.s", {{ 0x07000c40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rnz", {{ 0x07000540, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rnz.s", {{ 0x07000d40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rn", {{ 0x07000040, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rn.s", {{ 0x07000840, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ro", {{ 0x07000640, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ro.s", {{ 0x07000e40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ru", {{ 0x07000140, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ru.s", {{ 0x07000940, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rz", {{ 0x07000340, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rz.s", {{ 0x07000b40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv", {{ 0x07000740, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.s", {{ 0x07000f40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwh.rd", {{ 0x7a036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rd.s", {{ 0x7a036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rna", {{ 0x7a036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rna.s", {{ 0x7a036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rnz", {{ 0x7a036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rnz.s", {{ 0x7a036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rn", {{ 0x7a036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rn.s", {{ 0x7a036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ro", {{ 0x7a036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ro.s", {{ 0x7a036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ru", {{ 0x7a036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ru.s", {{ 0x7a036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rz", {{ 0x7a036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rz.s", {{ 0x7a036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh", {{ 0x7a036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.s", {{ 0x7a036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnegd", {{ 0x70032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNEGD_registerW_registerZ_simple */
	{"fneghq", {{ 0x76032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNEGHQ_registerW_registerZ_simple */
	{"fnegwp", {{ 0x74032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNEGWP_registerW_registerZ_simple */
	{"fnegw", {{ 0x72032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FNEGW_registerW_registerZ_simple */
	{"frecw.rd", {{ 0x72036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rd", {{ 0x72036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rd.s", {{ 0x72036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rd.s", {{ 0x72036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rna", {{ 0x72036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rna", {{ 0x72036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rna.s", {{ 0x72036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rna.s", {{ 0x72036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rnz", {{ 0x72036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rnz", {{ 0x72036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rnz.s", {{ 0x72036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rnz.s", {{ 0x72036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rn", {{ 0x72036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rn", {{ 0x72036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rn.s", {{ 0x72036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rn.s", {{ 0x72036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ro", {{ 0x72036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ro", {{ 0x72036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ro.s", {{ 0x72036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ro.s", {{ 0x72036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ru", {{ 0x72036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ru", {{ 0x72036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ru.s", {{ 0x72036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ru.s", {{ 0x72036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rz", {{ 0x72036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rz", {{ 0x72036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rz.s", {{ 0x72036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rz.s", {{ 0x72036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw", {{ 0x72036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw", {{ 0x72036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.s", {{ 0x72036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.s", {{ 0x72036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rd", {{ 0x73036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rd", {{ 0x73036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rd.s", {{ 0x73036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rd.s", {{ 0x73036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rna", {{ 0x73036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rna", {{ 0x73036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rna.s", {{ 0x73036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rna.s", {{ 0x73036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rnz", {{ 0x73036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rnz", {{ 0x73036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rnz.s", {{ 0x73036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rnz.s", {{ 0x73036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rn", {{ 0x73036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rn", {{ 0x73036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rn.s", {{ 0x73036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rn.s", {{ 0x73036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ro", {{ 0x73036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ro", {{ 0x73036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ro.s", {{ 0x73036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ro.s", {{ 0x73036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ru", {{ 0x73036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ru", {{ 0x73036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ru.s", {{ 0x73036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ru.s", {{ 0x73036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rz", {{ 0x73036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rz", {{ 0x73036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rz.s", {{ 0x73036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rz.s", {{ 0x73036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw", {{ 0x73036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw", {{ 0x73036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.s", {{ 0x73036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.s", {{ 0x73036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fsbfcdc.rd", {{ 0x5f032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rd", {{ 0x5f032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rd.s", {{ 0x5f03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rd.s", {{ 0x5f03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rna", {{ 0x5f034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rna", {{ 0x5f034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rna.s", {{ 0x5f03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rna.s", {{ 0x5f03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rnz", {{ 0x5f035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rnz", {{ 0x5f035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rnz.s", {{ 0x5f03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rnz.s", {{ 0x5f03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rn", {{ 0x5f030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rn", {{ 0x5f030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rn.s", {{ 0x5f038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rn.s", {{ 0x5f038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ro", {{ 0x5f036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ro", {{ 0x5f036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ro.s", {{ 0x5f03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ro.s", {{ 0x5f03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ru", {{ 0x5f031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ru", {{ 0x5f031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ru.s", {{ 0x5f039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ru.s", {{ 0x5f039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rz", {{ 0x5f033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rz", {{ 0x5f033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rz.s", {{ 0x5f03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rz.s", {{ 0x5f03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc", {{ 0x5f037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c", {{ 0x5f037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.s", {{ 0x5f03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.s", {{ 0x5f03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rd", {{ 0x5b072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rd", {{ 0x5b072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rd.s", {{ 0x5b07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rd.s", {{ 0x5b07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rna", {{ 0x5b074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rna", {{ 0x5b074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rna.s", {{ 0x5b07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rna.s", {{ 0x5b07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rnz", {{ 0x5b075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rnz", {{ 0x5b075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rnz.s", {{ 0x5b07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rnz.s", {{ 0x5b07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rn", {{ 0x5b070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rn", {{ 0x5b070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rn.s", {{ 0x5b078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rn.s", {{ 0x5b078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ro", {{ 0x5b076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ro", {{ 0x5b076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ro.s", {{ 0x5b07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ro.s", {{ 0x5b07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ru", {{ 0x5b071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ru", {{ 0x5b071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ru.s", {{ 0x5b079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ru.s", {{ 0x5b079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rz", {{ 0x5b073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rz", {{ 0x5b073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rz.s", {{ 0x5b07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rz.s", {{ 0x5b07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp", {{ 0x5b077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c", {{ 0x5b077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.s", {{ 0x5b07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.s", {{ 0x5b07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwc", {{ 0x47020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_registerW_registerZ_s010_simple */
	{"fsbfwp.c", {{ 0x47020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_registerW_registerZ_s010_simple */
	{"fsbfcwc", {{ 0xc7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_registerW_registerZ_s037_double */
	{"fsbfwp.c", {{ 0xc7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_registerW_registerZ_s037_double */
	{"fsbfcwc", {{ 0xc7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_registerW_registerZ_w064_triple */
	{"fsbfwp.c", {{ 0xc7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_registerW_registerZ_w064_triple */
	{"fsbfcwc.rd", {{ 0x57022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rd", {{ 0x57022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rd.s", {{ 0x5702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rd.s", {{ 0x5702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rna", {{ 0x57024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rna", {{ 0x57024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rna.s", {{ 0x5702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rna.s", {{ 0x5702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rnz", {{ 0x57025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rnz", {{ 0x57025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rnz.s", {{ 0x5702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rnz.s", {{ 0x5702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rn", {{ 0x57020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rn", {{ 0x57020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rn.s", {{ 0x57028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rn.s", {{ 0x57028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ro", {{ 0x57026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ro", {{ 0x57026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ro.s", {{ 0x5702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ro.s", {{ 0x5702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ru", {{ 0x57021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ru", {{ 0x57021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ru.s", {{ 0x57029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ru.s", {{ 0x57029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rz", {{ 0x57023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rz", {{ 0x57023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rz.s", {{ 0x5702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rz.s", {{ 0x5702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc", {{ 0x57027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c", {{ 0x57027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.s", {{ 0x5702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.s", {{ 0x5702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfdp.rd", {{ 0x5e032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rd.s", {{ 0x5e03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rna", {{ 0x5e034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rna.s", {{ 0x5e03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rnz", {{ 0x5e035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rnz.s", {{ 0x5e03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rn", {{ 0x5e030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rn.s", {{ 0x5e038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ro", {{ 0x5e036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ro.s", {{ 0x5e03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ru", {{ 0x5e031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ru.s", {{ 0x5e039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rz", {{ 0x5e033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rz.s", {{ 0x5e03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp", {{ 0x5e037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.s", {{ 0x5e03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfd", {{ 0x44020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_registerW_registerZ_s010_simple */
	{"fsbfd", {{ 0xc4020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_registerW_registerZ_s037_double */
	{"fsbfd", {{ 0xc4020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_registerW_registerZ_w064_triple */
	{"fsbfd.rd", {{ 0x54022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rd.s", {{ 0x5402a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rna", {{ 0x54024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rna.s", {{ 0x5402c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rnz", {{ 0x54025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rnz.s", {{ 0x5402d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rn", {{ 0x54020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rn.s", {{ 0x54028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ro", {{ 0x54026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ro.s", {{ 0x5402e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ru", {{ 0x54021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ru.s", {{ 0x54029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rz", {{ 0x54023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rz.s", {{ 0x5402b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd", {{ 0x54027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.s", {{ 0x5402f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq", {{ 0x46020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_registerW_registerZ_s010_simple */
	{"fsbfhq", {{ 0xc6020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_registerW_registerZ_s037_double */
	{"fsbfhq", {{ 0xc6020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_registerW_registerZ_w064_triple */
	{"fsbfhq.rd", {{ 0x56022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rd.s", {{ 0x5602a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rna", {{ 0x56024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rna.s", {{ 0x5602c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rnz", {{ 0x56025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rnz.s", {{ 0x5602d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rn", {{ 0x56020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rn.s", {{ 0x56028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ro", {{ 0x56026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ro.s", {{ 0x5602e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ru", {{ 0x56021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ru.s", {{ 0x56029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rz", {{ 0x56023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rz.s", {{ 0x5602b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq", {{ 0x56027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.s", {{ 0x5602f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp", {{ 0x45020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_registerW_registerZ_s010_simple */
	{"fsbfwp", {{ 0xc5020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_registerW_registerZ_s037_double */
	{"fsbfwp", {{ 0xc5020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_registerW_registerZ_w064_triple */
	{"fsbfwp.rd", {{ 0x55022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rd.s", {{ 0x5502a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rna", {{ 0x55024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rna.s", {{ 0x5502c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rnz", {{ 0x55025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rnz.s", {{ 0x5502d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rn", {{ 0x55020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rn.s", {{ 0x55028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ro", {{ 0x55026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ro.s", {{ 0x5502e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ru", {{ 0x55021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ru.s", {{ 0x55029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rz", {{ 0x55023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rz.s", {{ 0x5502b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp", {{ 0x55027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.s", {{ 0x5502f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwq.rd", {{ 0x5a072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rd.s", {{ 0x5a07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rna", {{ 0x5a074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rna.s", {{ 0x5a07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rnz", {{ 0x5a075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rnz.s", {{ 0x5a07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rn", {{ 0x5a070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rn.s", {{ 0x5a078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ro", {{ 0x5a076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ro.s", {{ 0x5a07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ru", {{ 0x5a071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ru.s", {{ 0x5a079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rz", {{ 0x5a073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rz.s", {{ 0x5a07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq", {{ 0x5a077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.s", {{ 0x5a07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfw", {{ 0x4d020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_registerW_registerZ_s010_simple */
	{"fsbfw", {{ 0xcd020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_registerW_registerZ_s037_double */
	{"fsbfw", {{ 0xcd020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_registerW_registerZ_w064_triple */
	{"fsbfw.rd", {{ 0x5d022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rd.s", {{ 0x5d02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rna", {{ 0x5d024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rna.s", {{ 0x5d02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rnz", {{ 0x5d025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rnz.s", {{ 0x5d02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rn", {{ 0x5d020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rn.s", {{ 0x5d028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ro", {{ 0x5d026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ro.s", {{ 0x5d02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ru", {{ 0x5d021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ru.s", {{ 0x5d029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rz", {{ 0x5d023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rz.s", {{ 0x5d02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw", {{ 0x5d027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.s", {{ 0x5d02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fscalewv.rd.s", {{ 0x07000a00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rd.s.relu", {{ 0x07000a80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rd", {{ 0x07000200, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rd.relu", {{ 0x07000280, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna.s", {{ 0x07000c00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna.s.relu", {{ 0x07000c80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna", {{ 0x07000400, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna.relu", {{ 0x07000480, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz.s", {{ 0x07000d00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz.s.relu", {{ 0x07000d80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz", {{ 0x07000500, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz.relu", {{ 0x07000580, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn.s", {{ 0x07000800, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn.s.relu", {{ 0x07000880, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn", {{ 0x07000000, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn.relu", {{ 0x07000080, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro.s", {{ 0x07000e00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro.s.relu", {{ 0x07000e80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro", {{ 0x07000600, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro.relu", {{ 0x07000680, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru.s", {{ 0x07000900, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru.s.relu", {{ 0x07000980, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru", {{ 0x07000100, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru.relu", {{ 0x07000180, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz.s", {{ 0x07000b00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz.s.relu", {{ 0x07000b80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz", {{ 0x07000300, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz.relu", {{ 0x07000380, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.s", {{ 0x07000f00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.s.relu", {{ 0x07000f80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv", {{ 0x07000700, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.relu", {{ 0x07000780, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fsdivd", {{ 0x70035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSDIVD_silent2_registerW_registerP_simple */
	{"fsdivd.s", {{ 0x70035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSDIVD_silent2_registerW_registerP_simple */
	{"fsdivwp", {{ 0x74035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSDIVWP_silent2_registerW_registerP_simple */
	{"fsdivwp.s", {{ 0x74035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSDIVWP_silent2_registerW_registerP_simple */
	{"fsdivw", {{ 0x72035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSDIVW_silent2_registerW_registerP_simple */
	{"fsdivw.s", {{ 0x72035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSDIVW_silent2_registerW_registerP_simple */
	{"fsrecd", {{ 0x70034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECD_silent2_registerW_registerZ_simple */
	{"fsinvd", {{ 0x70034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECD_silent2_registerW_registerZ_simple */
	{"fsrecd.s", {{ 0x70034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECD_silent2_registerW_registerZ_simple */
	{"fsinvd.s", {{ 0x70034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECD_silent2_registerW_registerZ_simple */
	{"fsrecwp", {{ 0x74034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsinvwp", {{ 0x74034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsrecwp.s", {{ 0x74034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsinvwp.s", {{ 0x74034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsrecw", {{ 0x72034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECW_silent2_registerW_registerZ_simple */
	{"fsinvw", {{ 0x72034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECW_silent2_registerW_registerZ_simple */
	{"fsrecw.s", {{ 0x72034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECW_silent2_registerW_registerZ_simple */
	{"fsinvw.s", {{ 0x72034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRECW_silent2_registerW_registerZ_simple */
	{"fsrsrd", {{ 0x78032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRD_registerW_registerZ_simple */
	{"fsisrd", {{ 0x78032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRD_registerW_registerZ_simple */
	{"fsrsrd.s", {{ 0x78032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRD_registerW_registerZ_simple */
	{"fsrsrwp", {{ 0x7c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRWP_registerW_registerZ_simple */
	{"fsisrwp", {{ 0x7c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRWP_registerW_registerZ_simple */
	{"fsrsrwp.s", {{ 0x7c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRWP_registerW_registerZ_simple */
	{"fsrsrw", {{ 0x7a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRW_registerW_registerZ_simple */
	{"fsisrw", {{ 0x7a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRW_registerW_registerZ_simple */
	{"fsrsrw.s", {{ 0x7a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FSRSRW_registerW_registerZ_simple */
	{"fwidenlhwp", {{ 0x7c033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENLHWP_silent2_registerW_registerZ_simple */
	{"fwidenlhwp.s", {{ 0x7c033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENLHWP_silent2_registerW_registerZ_simple */
	{"fwidenlhw", {{ 0x7a033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENLHW_silent2_registerW_registerZ_simple */
	{"fwidenlhw.s", {{ 0x7a033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENLHW_silent2_registerW_registerZ_simple */
	{"fwidenlwd", {{ 0x78033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENLWD_silent2_registerW_registerZ_simple */
	{"fwidenlwd.s", {{ 0x78033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENLWD_silent2_registerW_registerZ_simple */
	{"fwidenmhwp", {{ 0x7d033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENMHWP_silent2_registerW_registerZ_simple */
	{"fwidenmhwp.s", {{ 0x7d033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENMHWP_silent2_registerW_registerZ_simple */
	{"fwidenmhw", {{ 0x7b033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENMHW_silent2_registerW_registerZ_simple */
	{"fwidenmhw.s", {{ 0x7b033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENMHW_silent2_registerW_registerZ_simple */
	{"fwidenmwd", {{ 0x79033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENMWD_silent2_registerW_registerZ_simple */
	{"fwidenmwd.s", {{ 0x79033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-FWIDENMWD_silent2_registerW_registerZ_simple */
	{"get", {{ 0x0fc40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &kv3_systems2_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-GET_registerZ_systemS2_simple */
	{"get", {{ 0x0fc40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &kv3_systems3_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-GET_registerZ_systemS3_simple */
	{"goto", {{ 0x10000000, 0x78000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_pcrel27_opnd, 0}, "", "%s"}, /* Opcode-v1-GOTO_s027_simple */
	{"icall", {{ 0x0fdc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v1-ICALL_registerZ_simple */
	{"iget", {{ 0x0fcc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v1-IGET_registerZ_simple */
	{"igoto", {{ 0x0fd80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v1-IGOTO_registerZ_simple */
	{"iinvals.deqz", {{ 0x3c5e1000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.dgez", {{ 0x3c5e3000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.dgtz", {{ 0x3c5e5000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.dlez", {{ 0x3c5e4000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.dltz", {{ 0x3c5e2000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.dnez", {{ 0x3c5e0000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.even", {{ 0x3c5e7000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.odd", {{ 0x3c5e6000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.weqz", {{ 0x3c5e9000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.wgez", {{ 0x3c5eb000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.wgtz", {{ 0x3c5ed000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.wlez", {{ 0x3c5ec000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.wltz", {{ 0x3c5ea000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.wnez", {{ 0x3c5e8000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_registerZ_simple */
	{"iinvals.deqz", {{ 0xbc5e1000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dgez", {{ 0xbc5e3000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dgtz", {{ 0xbc5e5000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dlez", {{ 0xbc5e4000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dltz", {{ 0xbc5e2000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dnez", {{ 0xbc5e0000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.even", {{ 0xbc5e7000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.odd", {{ 0xbc5e6000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.weqz", {{ 0xbc5e9000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wgez", {{ 0xbc5eb000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wgtz", {{ 0xbc5ed000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wlez", {{ 0xbc5ec000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wltz", {{ 0xbc5ea000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wnez", {{ 0xbc5e8000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s027_registerZ_double */
	{"iinvals.deqz", {{ 0xbc5e1000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dgez", {{ 0xbc5e3000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dgtz", {{ 0xbc5e5000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dlez", {{ 0xbc5e4000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dltz", {{ 0xbc5e2000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dnez", {{ 0xbc5e0000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.even", {{ 0xbc5e7000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.odd", {{ 0xbc5e6000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.weqz", {{ 0xbc5e9000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wgez", {{ 0xbc5eb000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wgtz", {{ 0xbc5ed000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wlez", {{ 0xbc5ec000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wltz", {{ 0xbc5ea000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wnez", {{ 0xbc5e8000, 0xfcfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v1-IINVALS_lsucond_registerY_s054_registerZ_triple */
	{"iinvals", {{ 0x3c5ee000, 0x7cfff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-IINVALS_registerY_registerZ_simple */
	{"iinvals", {{ 0x3c5c0000, 0x7cff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-IINVALS_s010_registerZ_simple */
	{"iinvals", {{ 0xbc5c0000, 0xfcff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-IINVALS_s037_registerZ_double */
	{"iinvals", {{ 0xbc5c0000, 0xfcff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v1-IINVALS_w064_registerZ_triple */
	{"iinval", {{ 0x3f9d0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {0}, "", ""}, /* Opcode-v1-IINVAL_simple */
	{"insf", {{ 0x60030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-INSF_registerW_registerZ_u006_u006_simple */
	{"landd", {{ 0x7c014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDD_registerW_registerZ_registerY_simple */
	{"landd", {{ 0xfc014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDD_splat32_registerW_registerZ_w032_double */
	{"landd.@", {{ 0xfc014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDD_splat32_registerW_registerZ_w032_double */
	{"landhq", {{ 0x7c017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDHQ_registerW_registerZ_registerY_simple */
	{"landhq", {{ 0xfc017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDHQ_splat32_registerW_registerZ_w032_double */
	{"landhq.@", {{ 0xfc017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDHQ_splat32_registerW_registerZ_w032_double */
	{"landwp", {{ 0x7c016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDWP_registerW_registerZ_registerY_simple */
	{"landwp", {{ 0xfc016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDWP_splat32_registerW_registerZ_w032_double */
	{"landwp.@", {{ 0xfc016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDWP_splat32_registerW_registerZ_w032_double */
	{"landw", {{ 0x7c015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDW_registerW_registerZ_registerY_simple */
	{"landw", {{ 0xfc015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LANDW_registerW_registerZ_w032_double */
	{"lbs.s", {{ 0x2502e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.s.xs", {{ 0x2502f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.us", {{ 0x2702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.us.xs", {{ 0x2702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.u", {{ 0x2602e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.u.xs", {{ 0x2602f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs", {{ 0x2402e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.xs", {{ 0x2402f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.s.deqz", {{ 0x25021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dgez", {{ 0x25023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dgtz", {{ 0x25025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dlez", {{ 0x25024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dltz", {{ 0x25022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dnez", {{ 0x25020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.even", {{ 0x25027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.odd", {{ 0x25026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.weqz", {{ 0x25029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wgez", {{ 0x2502b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wgtz", {{ 0x2502d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wlez", {{ 0x2502c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wltz", {{ 0x2502a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wnez", {{ 0x25028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.deqz", {{ 0x27021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dgez", {{ 0x27023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dgtz", {{ 0x27025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dlez", {{ 0x27024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dltz", {{ 0x27022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dnez", {{ 0x27020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.even", {{ 0x27027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.odd", {{ 0x27026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.weqz", {{ 0x27029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wgez", {{ 0x2702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wgtz", {{ 0x2702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wlez", {{ 0x2702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wltz", {{ 0x2702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wnez", {{ 0x27028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.deqz", {{ 0x26021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dgez", {{ 0x26023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dgtz", {{ 0x26025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dlez", {{ 0x26024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dltz", {{ 0x26022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dnez", {{ 0x26020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.even", {{ 0x26027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.odd", {{ 0x26026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.weqz", {{ 0x26029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wgez", {{ 0x2602b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wgtz", {{ 0x2602d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wlez", {{ 0x2602c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wltz", {{ 0x2602a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wnez", {{ 0x26028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.deqz", {{ 0x24021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dgez", {{ 0x24023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dgtz", {{ 0x24025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dlez", {{ 0x24024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dltz", {{ 0x24022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dnez", {{ 0x24020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.even", {{ 0x24027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.odd", {{ 0x24026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.weqz", {{ 0x24029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wgez", {{ 0x2402b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wgtz", {{ 0x2402d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wlez", {{ 0x2402c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wltz", {{ 0x2402a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wnez", {{ 0x24028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.deqz", {{ 0xa5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dgez", {{ 0xa5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dgtz", {{ 0xa5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dlez", {{ 0xa5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dltz", {{ 0xa5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dnez", {{ 0xa5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.even", {{ 0xa5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.odd", {{ 0xa5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.weqz", {{ 0xa5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wgez", {{ 0xa502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wgtz", {{ 0xa502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wlez", {{ 0xa502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wltz", {{ 0xa502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wnez", {{ 0xa5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.deqz", {{ 0xa7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dgez", {{ 0xa7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dgtz", {{ 0xa7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dlez", {{ 0xa7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dltz", {{ 0xa7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dnez", {{ 0xa7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.even", {{ 0xa7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.odd", {{ 0xa7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.weqz", {{ 0xa7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wgez", {{ 0xa702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wgtz", {{ 0xa702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wlez", {{ 0xa702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wltz", {{ 0xa702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wnez", {{ 0xa7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.deqz", {{ 0xa6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dgez", {{ 0xa6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dgtz", {{ 0xa6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dlez", {{ 0xa6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dltz", {{ 0xa6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dnez", {{ 0xa6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.even", {{ 0xa6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.odd", {{ 0xa6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.weqz", {{ 0xa6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wgez", {{ 0xa602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wgtz", {{ 0xa602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wlez", {{ 0xa602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wltz", {{ 0xa602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wnez", {{ 0xa6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.deqz", {{ 0xa4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dgez", {{ 0xa4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dgtz", {{ 0xa4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dlez", {{ 0xa4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dltz", {{ 0xa4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dnez", {{ 0xa4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.even", {{ 0xa4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.odd", {{ 0xa4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.weqz", {{ 0xa4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wgez", {{ 0xa402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wgtz", {{ 0xa402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wlez", {{ 0xa402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wltz", {{ 0xa402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wnez", {{ 0xa4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.deqz", {{ 0xa5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dgez", {{ 0xa5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dgtz", {{ 0xa5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dlez", {{ 0xa5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dltz", {{ 0xa5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dnez", {{ 0xa5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.even", {{ 0xa5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.odd", {{ 0xa5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.weqz", {{ 0xa5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wgez", {{ 0xa502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wgtz", {{ 0xa502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wlez", {{ 0xa502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wltz", {{ 0xa502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wnez", {{ 0xa5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.deqz", {{ 0xa7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dgez", {{ 0xa7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dgtz", {{ 0xa7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dlez", {{ 0xa7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dltz", {{ 0xa7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dnez", {{ 0xa7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.even", {{ 0xa7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.odd", {{ 0xa7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.weqz", {{ 0xa7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wgez", {{ 0xa702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wgtz", {{ 0xa702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wlez", {{ 0xa702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wltz", {{ 0xa702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wnez", {{ 0xa7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.deqz", {{ 0xa6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dgez", {{ 0xa6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dgtz", {{ 0xa6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dlez", {{ 0xa6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dltz", {{ 0xa6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dnez", {{ 0xa6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.even", {{ 0xa6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.odd", {{ 0xa6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.weqz", {{ 0xa6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wgez", {{ 0xa602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wgtz", {{ 0xa602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wlez", {{ 0xa602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wltz", {{ 0xa602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wnez", {{ 0xa6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.deqz", {{ 0xa4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dgez", {{ 0xa4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dgtz", {{ 0xa4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dlez", {{ 0xa4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dltz", {{ 0xa4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dnez", {{ 0xa4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.even", {{ 0xa4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.odd", {{ 0xa4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.weqz", {{ 0xa4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wgez", {{ 0xa402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wgtz", {{ 0xa402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wlez", {{ 0xa402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wltz", {{ 0xa402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wnez", {{ 0xa4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs", {{ 0x24000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs.s", {{ 0x25000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs.u", {{ 0x26000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs.us", {{ 0x27000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs", {{ 0xa4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s037_registerZ_double */
	{"lbs.s", {{ 0xa5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s037_registerZ_double */
	{"lbs.u", {{ 0xa6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s037_registerZ_double */
	{"lbs.us", {{ 0xa7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_s037_registerZ_double */
	{"lbs", {{ 0xa4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_w064_registerZ_triple */
	{"lbs.s", {{ 0xa5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_w064_registerZ_triple */
	{"lbs.u", {{ 0xa6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_w064_registerZ_triple */
	{"lbs.us", {{ 0xa7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBS_variant_registerW_w064_registerZ_triple */
	{"lbz.s", {{ 0x2102e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.s.xs", {{ 0x2102f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.us", {{ 0x2302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.us.xs", {{ 0x2302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.u", {{ 0x2202e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.u.xs", {{ 0x2202f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz", {{ 0x2002e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.xs", {{ 0x2002f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.s.deqz", {{ 0x21021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dgez", {{ 0x21023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dgtz", {{ 0x21025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dlez", {{ 0x21024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dltz", {{ 0x21022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dnez", {{ 0x21020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.even", {{ 0x21027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.odd", {{ 0x21026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.weqz", {{ 0x21029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wgez", {{ 0x2102b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wgtz", {{ 0x2102d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wlez", {{ 0x2102c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wltz", {{ 0x2102a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wnez", {{ 0x21028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.deqz", {{ 0x23021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dgez", {{ 0x23023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dgtz", {{ 0x23025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dlez", {{ 0x23024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dltz", {{ 0x23022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dnez", {{ 0x23020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.even", {{ 0x23027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.odd", {{ 0x23026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.weqz", {{ 0x23029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wgez", {{ 0x2302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wgtz", {{ 0x2302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wlez", {{ 0x2302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wltz", {{ 0x2302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wnez", {{ 0x23028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.deqz", {{ 0x22021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dgez", {{ 0x22023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dgtz", {{ 0x22025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dlez", {{ 0x22024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dltz", {{ 0x22022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dnez", {{ 0x22020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.even", {{ 0x22027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.odd", {{ 0x22026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.weqz", {{ 0x22029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wgez", {{ 0x2202b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wgtz", {{ 0x2202d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wlez", {{ 0x2202c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wltz", {{ 0x2202a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wnez", {{ 0x22028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.deqz", {{ 0x20021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dgez", {{ 0x20023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dgtz", {{ 0x20025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dlez", {{ 0x20024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dltz", {{ 0x20022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dnez", {{ 0x20020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.even", {{ 0x20027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.odd", {{ 0x20026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.weqz", {{ 0x20029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wgez", {{ 0x2002b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wgtz", {{ 0x2002d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wlez", {{ 0x2002c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wltz", {{ 0x2002a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wnez", {{ 0x20028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.deqz", {{ 0xa1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dgez", {{ 0xa1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dgtz", {{ 0xa1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dlez", {{ 0xa1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dltz", {{ 0xa1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dnez", {{ 0xa1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.even", {{ 0xa1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.odd", {{ 0xa1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.weqz", {{ 0xa1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wgez", {{ 0xa102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wgtz", {{ 0xa102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wlez", {{ 0xa102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wltz", {{ 0xa102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wnez", {{ 0xa1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.deqz", {{ 0xa3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dgez", {{ 0xa3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dgtz", {{ 0xa3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dlez", {{ 0xa3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dltz", {{ 0xa3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dnez", {{ 0xa3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.even", {{ 0xa3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.odd", {{ 0xa3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.weqz", {{ 0xa3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wgez", {{ 0xa302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wgtz", {{ 0xa302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wlez", {{ 0xa302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wltz", {{ 0xa302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wnez", {{ 0xa3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.deqz", {{ 0xa2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dgez", {{ 0xa2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dgtz", {{ 0xa2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dlez", {{ 0xa2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dltz", {{ 0xa2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dnez", {{ 0xa2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.even", {{ 0xa2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.odd", {{ 0xa2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.weqz", {{ 0xa2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wgez", {{ 0xa202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wgtz", {{ 0xa202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wlez", {{ 0xa202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wltz", {{ 0xa202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wnez", {{ 0xa2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.deqz", {{ 0xa0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dgez", {{ 0xa0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dgtz", {{ 0xa0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dlez", {{ 0xa0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dltz", {{ 0xa0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dnez", {{ 0xa0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.even", {{ 0xa0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.odd", {{ 0xa0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.weqz", {{ 0xa0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wgez", {{ 0xa002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wgtz", {{ 0xa002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wlez", {{ 0xa002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wltz", {{ 0xa002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wnez", {{ 0xa0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.deqz", {{ 0xa1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dgez", {{ 0xa1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dgtz", {{ 0xa1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dlez", {{ 0xa1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dltz", {{ 0xa1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dnez", {{ 0xa1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.even", {{ 0xa1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.odd", {{ 0xa1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.weqz", {{ 0xa1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wgez", {{ 0xa102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wgtz", {{ 0xa102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wlez", {{ 0xa102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wltz", {{ 0xa102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wnez", {{ 0xa1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.deqz", {{ 0xa3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dgez", {{ 0xa3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dgtz", {{ 0xa3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dlez", {{ 0xa3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dltz", {{ 0xa3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dnez", {{ 0xa3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.even", {{ 0xa3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.odd", {{ 0xa3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.weqz", {{ 0xa3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wgez", {{ 0xa302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wgtz", {{ 0xa302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wlez", {{ 0xa302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wltz", {{ 0xa302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wnez", {{ 0xa3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.deqz", {{ 0xa2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dgez", {{ 0xa2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dgtz", {{ 0xa2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dlez", {{ 0xa2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dltz", {{ 0xa2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dnez", {{ 0xa2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.even", {{ 0xa2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.odd", {{ 0xa2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.weqz", {{ 0xa2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wgez", {{ 0xa202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wgtz", {{ 0xa202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wlez", {{ 0xa202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wltz", {{ 0xa202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wnez", {{ 0xa2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.deqz", {{ 0xa0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dgez", {{ 0xa0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dgtz", {{ 0xa0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dlez", {{ 0xa0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dltz", {{ 0xa0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dnez", {{ 0xa0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.even", {{ 0xa0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.odd", {{ 0xa0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.weqz", {{ 0xa0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wgez", {{ 0xa002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wgtz", {{ 0xa002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wlez", {{ 0xa002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wltz", {{ 0xa002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wnez", {{ 0xa0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz", {{ 0x20000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz.s", {{ 0x21000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz.u", {{ 0x22000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz.us", {{ 0x23000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz", {{ 0xa0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz.s", {{ 0xa1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz.u", {{ 0xa2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz.us", {{ 0xa3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz", {{ 0xa0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_w064_registerZ_triple */
	{"lbz.s", {{ 0xa1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_w064_registerZ_triple */
	{"lbz.u", {{ 0xa2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_w064_registerZ_triple */
	{"lbz.us", {{ 0xa3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LBZ_variant_registerW_w064_registerZ_triple */
	{"ld.s", {{ 0x3902e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.s.xs", {{ 0x3902f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.us", {{ 0x3b02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.us.xs", {{ 0x3b02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.u", {{ 0x3a02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.u.xs", {{ 0x3a02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld", {{ 0x3802e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.xs", {{ 0x3802f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.s.deqz", {{ 0x39021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dgez", {{ 0x39023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dgtz", {{ 0x39025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dlez", {{ 0x39024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dltz", {{ 0x39022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dnez", {{ 0x39020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.even", {{ 0x39027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.odd", {{ 0x39026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.weqz", {{ 0x39029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wgez", {{ 0x3902b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wgtz", {{ 0x3902d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wlez", {{ 0x3902c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wltz", {{ 0x3902a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wnez", {{ 0x39028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.deqz", {{ 0x3b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dgez", {{ 0x3b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dgtz", {{ 0x3b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dlez", {{ 0x3b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dltz", {{ 0x3b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dnez", {{ 0x3b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.even", {{ 0x3b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.odd", {{ 0x3b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.weqz", {{ 0x3b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wgez", {{ 0x3b02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wgtz", {{ 0x3b02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wlez", {{ 0x3b02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wltz", {{ 0x3b02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wnez", {{ 0x3b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.deqz", {{ 0x3a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dgez", {{ 0x3a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dgtz", {{ 0x3a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dlez", {{ 0x3a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dltz", {{ 0x3a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dnez", {{ 0x3a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.even", {{ 0x3a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.odd", {{ 0x3a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.weqz", {{ 0x3a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wgez", {{ 0x3a02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wgtz", {{ 0x3a02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wlez", {{ 0x3a02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wltz", {{ 0x3a02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wnez", {{ 0x3a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.deqz", {{ 0x38021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dgez", {{ 0x38023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dgtz", {{ 0x38025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dlez", {{ 0x38024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dltz", {{ 0x38022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dnez", {{ 0x38020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.even", {{ 0x38027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.odd", {{ 0x38026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.weqz", {{ 0x38029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wgez", {{ 0x3802b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wgtz", {{ 0x3802d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wlez", {{ 0x3802c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wltz", {{ 0x3802a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wnez", {{ 0x38028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.deqz", {{ 0xb9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dgez", {{ 0xb9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dgtz", {{ 0xb9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dlez", {{ 0xb9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dltz", {{ 0xb9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dnez", {{ 0xb9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.even", {{ 0xb9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.odd", {{ 0xb9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.weqz", {{ 0xb9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wgez", {{ 0xb902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wgtz", {{ 0xb902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wlez", {{ 0xb902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wltz", {{ 0xb902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wnez", {{ 0xb9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.deqz", {{ 0xbb021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dgez", {{ 0xbb023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dgtz", {{ 0xbb025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dlez", {{ 0xbb024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dltz", {{ 0xbb022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dnez", {{ 0xbb020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.even", {{ 0xbb027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.odd", {{ 0xbb026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.weqz", {{ 0xbb029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wgez", {{ 0xbb02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wgtz", {{ 0xbb02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wlez", {{ 0xbb02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wltz", {{ 0xbb02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wnez", {{ 0xbb028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.deqz", {{ 0xba021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dgez", {{ 0xba023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dgtz", {{ 0xba025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dlez", {{ 0xba024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dltz", {{ 0xba022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dnez", {{ 0xba020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.even", {{ 0xba027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.odd", {{ 0xba026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.weqz", {{ 0xba029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wgez", {{ 0xba02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wgtz", {{ 0xba02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wlez", {{ 0xba02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wltz", {{ 0xba02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wnez", {{ 0xba028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.deqz", {{ 0xb8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dgez", {{ 0xb8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dgtz", {{ 0xb8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dlez", {{ 0xb8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dltz", {{ 0xb8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dnez", {{ 0xb8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.even", {{ 0xb8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.odd", {{ 0xb8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.weqz", {{ 0xb8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wgez", {{ 0xb802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wgtz", {{ 0xb802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wlez", {{ 0xb802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wltz", {{ 0xb802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wnez", {{ 0xb8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.deqz", {{ 0xb9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dgez", {{ 0xb9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dgtz", {{ 0xb9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dlez", {{ 0xb9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dltz", {{ 0xb9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dnez", {{ 0xb9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.even", {{ 0xb9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.odd", {{ 0xb9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.weqz", {{ 0xb9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wgez", {{ 0xb902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wgtz", {{ 0xb902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wlez", {{ 0xb902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wltz", {{ 0xb902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wnez", {{ 0xb9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.deqz", {{ 0xbb021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dgez", {{ 0xbb023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dgtz", {{ 0xbb025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dlez", {{ 0xbb024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dltz", {{ 0xbb022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dnez", {{ 0xbb020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.even", {{ 0xbb027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.odd", {{ 0xbb026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.weqz", {{ 0xbb029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wgez", {{ 0xbb02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wgtz", {{ 0xbb02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wlez", {{ 0xbb02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wltz", {{ 0xbb02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wnez", {{ 0xbb028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.deqz", {{ 0xba021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dgez", {{ 0xba023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dgtz", {{ 0xba025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dlez", {{ 0xba024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dltz", {{ 0xba022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dnez", {{ 0xba020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.even", {{ 0xba027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.odd", {{ 0xba026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.weqz", {{ 0xba029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wgez", {{ 0xba02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wgtz", {{ 0xba02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wlez", {{ 0xba02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wltz", {{ 0xba02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wnez", {{ 0xba028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.deqz", {{ 0xb8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dgez", {{ 0xb8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dgtz", {{ 0xb8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dlez", {{ 0xb8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dltz", {{ 0xb8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dnez", {{ 0xb8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.even", {{ 0xb8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.odd", {{ 0xb8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.weqz", {{ 0xb8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wgez", {{ 0xb802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wgtz", {{ 0xb802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wlez", {{ 0xb802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wltz", {{ 0xb802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wnez", {{ 0xb8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld", {{ 0x38000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s010_registerZ_simple */
	{"ld.s", {{ 0x39000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s010_registerZ_simple */
	{"ld.u", {{ 0x3a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s010_registerZ_simple */
	{"ld.us", {{ 0x3b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s010_registerZ_simple */
	{"ld", {{ 0xb8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s037_registerZ_double */
	{"ld.s", {{ 0xb9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s037_registerZ_double */
	{"ld.u", {{ 0xba000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s037_registerZ_double */
	{"ld.us", {{ 0xbb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_s037_registerZ_double */
	{"ld", {{ 0xb8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_w064_registerZ_triple */
	{"ld.s", {{ 0xb9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_w064_registerZ_triple */
	{"ld.u", {{ 0xba000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_w064_registerZ_triple */
	{"ld.us", {{ 0xbb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LD_variant_registerW_w064_registerZ_triple */
	{"lhs.s", {{ 0x2d02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.s.xs", {{ 0x2d02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.us", {{ 0x2f02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.us.xs", {{ 0x2f02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.u", {{ 0x2e02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.u.xs", {{ 0x2e02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs", {{ 0x2c02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.xs", {{ 0x2c02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.s.deqz", {{ 0x2d021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dgez", {{ 0x2d023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dgtz", {{ 0x2d025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dlez", {{ 0x2d024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dltz", {{ 0x2d022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dnez", {{ 0x2d020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.even", {{ 0x2d027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.odd", {{ 0x2d026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.weqz", {{ 0x2d029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wgez", {{ 0x2d02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wgtz", {{ 0x2d02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wlez", {{ 0x2d02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wltz", {{ 0x2d02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wnez", {{ 0x2d028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.deqz", {{ 0x2f021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dgez", {{ 0x2f023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dgtz", {{ 0x2f025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dlez", {{ 0x2f024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dltz", {{ 0x2f022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dnez", {{ 0x2f020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.even", {{ 0x2f027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.odd", {{ 0x2f026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.weqz", {{ 0x2f029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wgez", {{ 0x2f02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wgtz", {{ 0x2f02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wlez", {{ 0x2f02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wltz", {{ 0x2f02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wnez", {{ 0x2f028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.deqz", {{ 0x2e021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dgez", {{ 0x2e023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dgtz", {{ 0x2e025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dlez", {{ 0x2e024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dltz", {{ 0x2e022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dnez", {{ 0x2e020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.even", {{ 0x2e027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.odd", {{ 0x2e026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.weqz", {{ 0x2e029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wgez", {{ 0x2e02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wgtz", {{ 0x2e02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wlez", {{ 0x2e02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wltz", {{ 0x2e02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wnez", {{ 0x2e028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.deqz", {{ 0x2c021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dgez", {{ 0x2c023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dgtz", {{ 0x2c025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dlez", {{ 0x2c024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dltz", {{ 0x2c022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dnez", {{ 0x2c020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.even", {{ 0x2c027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.odd", {{ 0x2c026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.weqz", {{ 0x2c029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wgez", {{ 0x2c02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wgtz", {{ 0x2c02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wlez", {{ 0x2c02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wltz", {{ 0x2c02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wnez", {{ 0x2c028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.deqz", {{ 0xad021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dgez", {{ 0xad023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dgtz", {{ 0xad025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dlez", {{ 0xad024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dltz", {{ 0xad022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dnez", {{ 0xad020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.even", {{ 0xad027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.odd", {{ 0xad026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.weqz", {{ 0xad029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wgez", {{ 0xad02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wgtz", {{ 0xad02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wlez", {{ 0xad02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wltz", {{ 0xad02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wnez", {{ 0xad028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.deqz", {{ 0xaf021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dgez", {{ 0xaf023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dgtz", {{ 0xaf025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dlez", {{ 0xaf024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dltz", {{ 0xaf022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dnez", {{ 0xaf020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.even", {{ 0xaf027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.odd", {{ 0xaf026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.weqz", {{ 0xaf029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wgez", {{ 0xaf02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wgtz", {{ 0xaf02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wlez", {{ 0xaf02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wltz", {{ 0xaf02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wnez", {{ 0xaf028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.deqz", {{ 0xae021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dgez", {{ 0xae023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dgtz", {{ 0xae025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dlez", {{ 0xae024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dltz", {{ 0xae022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dnez", {{ 0xae020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.even", {{ 0xae027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.odd", {{ 0xae026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.weqz", {{ 0xae029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wgez", {{ 0xae02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wgtz", {{ 0xae02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wlez", {{ 0xae02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wltz", {{ 0xae02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wnez", {{ 0xae028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.deqz", {{ 0xac021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dgez", {{ 0xac023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dgtz", {{ 0xac025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dlez", {{ 0xac024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dltz", {{ 0xac022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dnez", {{ 0xac020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.even", {{ 0xac027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.odd", {{ 0xac026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.weqz", {{ 0xac029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wgez", {{ 0xac02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wgtz", {{ 0xac02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wlez", {{ 0xac02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wltz", {{ 0xac02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wnez", {{ 0xac028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.deqz", {{ 0xad021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dgez", {{ 0xad023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dgtz", {{ 0xad025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dlez", {{ 0xad024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dltz", {{ 0xad022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dnez", {{ 0xad020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.even", {{ 0xad027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.odd", {{ 0xad026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.weqz", {{ 0xad029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wgez", {{ 0xad02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wgtz", {{ 0xad02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wlez", {{ 0xad02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wltz", {{ 0xad02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wnez", {{ 0xad028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.deqz", {{ 0xaf021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dgez", {{ 0xaf023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dgtz", {{ 0xaf025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dlez", {{ 0xaf024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dltz", {{ 0xaf022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dnez", {{ 0xaf020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.even", {{ 0xaf027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.odd", {{ 0xaf026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.weqz", {{ 0xaf029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wgez", {{ 0xaf02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wgtz", {{ 0xaf02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wlez", {{ 0xaf02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wltz", {{ 0xaf02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wnez", {{ 0xaf028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.deqz", {{ 0xae021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dgez", {{ 0xae023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dgtz", {{ 0xae025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dlez", {{ 0xae024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dltz", {{ 0xae022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dnez", {{ 0xae020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.even", {{ 0xae027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.odd", {{ 0xae026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.weqz", {{ 0xae029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wgez", {{ 0xae02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wgtz", {{ 0xae02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wlez", {{ 0xae02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wltz", {{ 0xae02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wnez", {{ 0xae028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.deqz", {{ 0xac021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dgez", {{ 0xac023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dgtz", {{ 0xac025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dlez", {{ 0xac024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dltz", {{ 0xac022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dnez", {{ 0xac020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.even", {{ 0xac027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.odd", {{ 0xac026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.weqz", {{ 0xac029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wgez", {{ 0xac02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wgtz", {{ 0xac02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wlez", {{ 0xac02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wltz", {{ 0xac02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wnez", {{ 0xac028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs", {{ 0x2c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs.s", {{ 0x2d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs.u", {{ 0x2e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs.us", {{ 0x2f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs", {{ 0xac000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s037_registerZ_double */
	{"lhs.s", {{ 0xad000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s037_registerZ_double */
	{"lhs.u", {{ 0xae000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s037_registerZ_double */
	{"lhs.us", {{ 0xaf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_s037_registerZ_double */
	{"lhs", {{ 0xac000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_w064_registerZ_triple */
	{"lhs.s", {{ 0xad000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_w064_registerZ_triple */
	{"lhs.u", {{ 0xae000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_w064_registerZ_triple */
	{"lhs.us", {{ 0xaf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHS_variant_registerW_w064_registerZ_triple */
	{"lhz.s", {{ 0x2902e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.s.xs", {{ 0x2902f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.us", {{ 0x2b02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.us.xs", {{ 0x2b02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.u", {{ 0x2a02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.u.xs", {{ 0x2a02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz", {{ 0x2802e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.xs", {{ 0x2802f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.s.deqz", {{ 0x29021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dgez", {{ 0x29023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dgtz", {{ 0x29025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dlez", {{ 0x29024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dltz", {{ 0x29022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dnez", {{ 0x29020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.even", {{ 0x29027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.odd", {{ 0x29026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.weqz", {{ 0x29029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wgez", {{ 0x2902b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wgtz", {{ 0x2902d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wlez", {{ 0x2902c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wltz", {{ 0x2902a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wnez", {{ 0x29028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.deqz", {{ 0x2b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dgez", {{ 0x2b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dgtz", {{ 0x2b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dlez", {{ 0x2b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dltz", {{ 0x2b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dnez", {{ 0x2b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.even", {{ 0x2b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.odd", {{ 0x2b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.weqz", {{ 0x2b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wgez", {{ 0x2b02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wgtz", {{ 0x2b02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wlez", {{ 0x2b02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wltz", {{ 0x2b02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wnez", {{ 0x2b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.deqz", {{ 0x2a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dgez", {{ 0x2a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dgtz", {{ 0x2a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dlez", {{ 0x2a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dltz", {{ 0x2a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dnez", {{ 0x2a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.even", {{ 0x2a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.odd", {{ 0x2a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.weqz", {{ 0x2a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wgez", {{ 0x2a02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wgtz", {{ 0x2a02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wlez", {{ 0x2a02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wltz", {{ 0x2a02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wnez", {{ 0x2a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.deqz", {{ 0x28021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dgez", {{ 0x28023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dgtz", {{ 0x28025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dlez", {{ 0x28024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dltz", {{ 0x28022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dnez", {{ 0x28020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.even", {{ 0x28027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.odd", {{ 0x28026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.weqz", {{ 0x28029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wgez", {{ 0x2802b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wgtz", {{ 0x2802d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wlez", {{ 0x2802c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wltz", {{ 0x2802a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wnez", {{ 0x28028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.deqz", {{ 0xa9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dgez", {{ 0xa9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dgtz", {{ 0xa9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dlez", {{ 0xa9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dltz", {{ 0xa9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dnez", {{ 0xa9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.even", {{ 0xa9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.odd", {{ 0xa9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.weqz", {{ 0xa9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wgez", {{ 0xa902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wgtz", {{ 0xa902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wlez", {{ 0xa902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wltz", {{ 0xa902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wnez", {{ 0xa9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.deqz", {{ 0xab021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dgez", {{ 0xab023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dgtz", {{ 0xab025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dlez", {{ 0xab024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dltz", {{ 0xab022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dnez", {{ 0xab020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.even", {{ 0xab027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.odd", {{ 0xab026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.weqz", {{ 0xab029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wgez", {{ 0xab02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wgtz", {{ 0xab02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wlez", {{ 0xab02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wltz", {{ 0xab02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wnez", {{ 0xab028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.deqz", {{ 0xaa021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dgez", {{ 0xaa023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dgtz", {{ 0xaa025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dlez", {{ 0xaa024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dltz", {{ 0xaa022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dnez", {{ 0xaa020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.even", {{ 0xaa027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.odd", {{ 0xaa026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.weqz", {{ 0xaa029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wgez", {{ 0xaa02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wgtz", {{ 0xaa02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wlez", {{ 0xaa02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wltz", {{ 0xaa02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wnez", {{ 0xaa028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.deqz", {{ 0xa8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dgez", {{ 0xa8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dgtz", {{ 0xa8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dlez", {{ 0xa8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dltz", {{ 0xa8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dnez", {{ 0xa8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.even", {{ 0xa8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.odd", {{ 0xa8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.weqz", {{ 0xa8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wgez", {{ 0xa802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wgtz", {{ 0xa802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wlez", {{ 0xa802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wltz", {{ 0xa802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wnez", {{ 0xa8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.deqz", {{ 0xa9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dgez", {{ 0xa9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dgtz", {{ 0xa9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dlez", {{ 0xa9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dltz", {{ 0xa9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dnez", {{ 0xa9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.even", {{ 0xa9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.odd", {{ 0xa9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.weqz", {{ 0xa9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wgez", {{ 0xa902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wgtz", {{ 0xa902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wlez", {{ 0xa902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wltz", {{ 0xa902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wnez", {{ 0xa9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.deqz", {{ 0xab021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dgez", {{ 0xab023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dgtz", {{ 0xab025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dlez", {{ 0xab024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dltz", {{ 0xab022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dnez", {{ 0xab020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.even", {{ 0xab027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.odd", {{ 0xab026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.weqz", {{ 0xab029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wgez", {{ 0xab02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wgtz", {{ 0xab02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wlez", {{ 0xab02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wltz", {{ 0xab02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wnez", {{ 0xab028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.deqz", {{ 0xaa021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dgez", {{ 0xaa023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dgtz", {{ 0xaa025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dlez", {{ 0xaa024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dltz", {{ 0xaa022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dnez", {{ 0xaa020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.even", {{ 0xaa027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.odd", {{ 0xaa026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.weqz", {{ 0xaa029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wgez", {{ 0xaa02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wgtz", {{ 0xaa02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wlez", {{ 0xaa02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wltz", {{ 0xaa02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wnez", {{ 0xaa028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.deqz", {{ 0xa8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dgez", {{ 0xa8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dgtz", {{ 0xa8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dlez", {{ 0xa8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dltz", {{ 0xa8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dnez", {{ 0xa8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.even", {{ 0xa8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.odd", {{ 0xa8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.weqz", {{ 0xa8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wgez", {{ 0xa802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wgtz", {{ 0xa802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wlez", {{ 0xa802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wltz", {{ 0xa802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wnez", {{ 0xa8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz", {{ 0x28000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz.s", {{ 0x29000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz.u", {{ 0x2a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz.us", {{ 0x2b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz", {{ 0xa8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz.s", {{ 0xa9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz.u", {{ 0xaa000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz.us", {{ 0xab000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz", {{ 0xa8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_w064_registerZ_triple */
	{"lhz.s", {{ 0xa9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_w064_registerZ_triple */
	{"lhz.u", {{ 0xaa000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_w064_registerZ_triple */
	{"lhz.us", {{ 0xab000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LHZ_variant_registerW_w064_registerZ_triple */
	{"lnandd", {{ 0x7d014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDD_registerW_registerZ_registerY_simple */
	{"lnandd", {{ 0xfd014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDD_splat32_registerW_registerZ_w032_double */
	{"lnandd.@", {{ 0xfd014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDD_splat32_registerW_registerZ_w032_double */
	{"lnandhq", {{ 0x7d017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDHQ_registerW_registerZ_registerY_simple */
	{"lnandhq", {{ 0xfd017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDHQ_splat32_registerW_registerZ_w032_double */
	{"lnandhq.@", {{ 0xfd017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDHQ_splat32_registerW_registerZ_w032_double */
	{"lnandwp", {{ 0x7d016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDWP_registerW_registerZ_registerY_simple */
	{"lnandwp", {{ 0xfd016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDWP_splat32_registerW_registerZ_w032_double */
	{"lnandwp.@", {{ 0xfd016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDWP_splat32_registerW_registerZ_w032_double */
	{"lnandw", {{ 0x7d015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDW_registerW_registerZ_registerY_simple */
	{"lnandw", {{ 0xfd015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNANDW_registerW_registerZ_w032_double */
	{"lnord", {{ 0x7f014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORD_registerW_registerZ_registerY_simple */
	{"lnord", {{ 0xff014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORD_splat32_registerW_registerZ_w032_double */
	{"lnord.@", {{ 0xff014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORD_splat32_registerW_registerZ_w032_double */
	{"lnorhq", {{ 0x7f017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORHQ_registerW_registerZ_registerY_simple */
	{"lnorhq", {{ 0xff017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORHQ_splat32_registerW_registerZ_w032_double */
	{"lnorhq.@", {{ 0xff017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORHQ_splat32_registerW_registerZ_w032_double */
	{"lnorwp", {{ 0x7f016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORWP_registerW_registerZ_registerY_simple */
	{"lnorwp", {{ 0xff016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORWP_splat32_registerW_registerZ_w032_double */
	{"lnorwp.@", {{ 0xff016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORWP_splat32_registerW_registerZ_w032_double */
	{"lnorw", {{ 0x7f015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORW_registerW_registerZ_registerY_simple */
	{"lnorw", {{ 0xff015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LNORW_registerW_registerZ_w032_double */
	{"loopdo", {{ 0x0f000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-LOOPDO_registerZ_s017_simple */
	{"lord", {{ 0x7e014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORD_registerW_registerZ_registerY_simple */
	{"lord", {{ 0xfe014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORD_splat32_registerW_registerZ_w032_double */
	{"lord.@", {{ 0xfe014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORD_splat32_registerW_registerZ_w032_double */
	{"lorhq", {{ 0x7e017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORHQ_registerW_registerZ_registerY_simple */
	{"lorhq", {{ 0xfe017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORHQ_splat32_registerW_registerZ_w032_double */
	{"lorhq.@", {{ 0xfe017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORHQ_splat32_registerW_registerZ_w032_double */
	{"lorwp", {{ 0x7e016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORWP_registerW_registerZ_registerY_simple */
	{"lorwp", {{ 0xfe016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORWP_splat32_registerW_registerZ_w032_double */
	{"lorwp.@", {{ 0xfe016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORWP_splat32_registerW_registerZ_w032_double */
	{"lorw", {{ 0x7e015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORW_registerW_registerZ_registerY_simple */
	{"lorw", {{ 0xfe015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-LORW_registerW_registerZ_w032_double */
	{"lo.s", {{ 0x3d06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.s.xs", {{ 0x3d06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.us", {{ 0x3f06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.us.xs", {{ 0x3f06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.u", {{ 0x3e06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.u.xs", {{ 0x3e06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo", {{ 0x3c06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.xs", {{ 0x3c06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.s.deqz", {{ 0x3d061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dgez", {{ 0x3d063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dgtz", {{ 0x3d065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dlez", {{ 0x3d064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dltz", {{ 0x3d062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dnez", {{ 0x3d060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.even", {{ 0x3d067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.odd", {{ 0x3d066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.weqz", {{ 0x3d069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wgez", {{ 0x3d06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wgtz", {{ 0x3d06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wlez", {{ 0x3d06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wltz", {{ 0x3d06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wnez", {{ 0x3d068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.deqz", {{ 0x3f061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dgez", {{ 0x3f063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dgtz", {{ 0x3f065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dlez", {{ 0x3f064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dltz", {{ 0x3f062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dnez", {{ 0x3f060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.even", {{ 0x3f067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.odd", {{ 0x3f066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.weqz", {{ 0x3f069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wgez", {{ 0x3f06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wgtz", {{ 0x3f06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wlez", {{ 0x3f06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wltz", {{ 0x3f06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wnez", {{ 0x3f068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.deqz", {{ 0x3e061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dgez", {{ 0x3e063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dgtz", {{ 0x3e065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dlez", {{ 0x3e064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dltz", {{ 0x3e062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dnez", {{ 0x3e060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.even", {{ 0x3e067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.odd", {{ 0x3e066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.weqz", {{ 0x3e069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wgez", {{ 0x3e06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wgtz", {{ 0x3e06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wlez", {{ 0x3e06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wltz", {{ 0x3e06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wnez", {{ 0x3e068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.deqz", {{ 0x3c061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dgez", {{ 0x3c063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dgtz", {{ 0x3c065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dlez", {{ 0x3c064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dltz", {{ 0x3c062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dnez", {{ 0x3c060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.even", {{ 0x3c067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.odd", {{ 0x3c066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.weqz", {{ 0x3c069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wgez", {{ 0x3c06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wgtz", {{ 0x3c06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wlez", {{ 0x3c06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wltz", {{ 0x3c06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wnez", {{ 0x3c068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.deqz", {{ 0xbd061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dgez", {{ 0xbd063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dgtz", {{ 0xbd065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dlez", {{ 0xbd064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dltz", {{ 0xbd062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dnez", {{ 0xbd060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.even", {{ 0xbd067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.odd", {{ 0xbd066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.weqz", {{ 0xbd069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wgez", {{ 0xbd06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wgtz", {{ 0xbd06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wlez", {{ 0xbd06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wltz", {{ 0xbd06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wnez", {{ 0xbd068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.deqz", {{ 0xbf061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dgez", {{ 0xbf063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dgtz", {{ 0xbf065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dlez", {{ 0xbf064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dltz", {{ 0xbf062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dnez", {{ 0xbf060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.even", {{ 0xbf067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.odd", {{ 0xbf066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.weqz", {{ 0xbf069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wgez", {{ 0xbf06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wgtz", {{ 0xbf06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wlez", {{ 0xbf06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wltz", {{ 0xbf06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wnez", {{ 0xbf068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.deqz", {{ 0xbe061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dgez", {{ 0xbe063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dgtz", {{ 0xbe065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dlez", {{ 0xbe064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dltz", {{ 0xbe062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dnez", {{ 0xbe060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.even", {{ 0xbe067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.odd", {{ 0xbe066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.weqz", {{ 0xbe069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wgez", {{ 0xbe06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wgtz", {{ 0xbe06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wlez", {{ 0xbe06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wltz", {{ 0xbe06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wnez", {{ 0xbe068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.deqz", {{ 0xbc061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dgez", {{ 0xbc063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dgtz", {{ 0xbc065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dlez", {{ 0xbc064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dltz", {{ 0xbc062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dnez", {{ 0xbc060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.even", {{ 0xbc067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.odd", {{ 0xbc066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.weqz", {{ 0xbc069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wgez", {{ 0xbc06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wgtz", {{ 0xbc06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wlez", {{ 0xbc06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wltz", {{ 0xbc06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wnez", {{ 0xbc068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.deqz", {{ 0xbd061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dgez", {{ 0xbd063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dgtz", {{ 0xbd065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dlez", {{ 0xbd064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dltz", {{ 0xbd062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dnez", {{ 0xbd060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.even", {{ 0xbd067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.odd", {{ 0xbd066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.weqz", {{ 0xbd069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wgez", {{ 0xbd06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wgtz", {{ 0xbd06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wlez", {{ 0xbd06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wltz", {{ 0xbd06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wnez", {{ 0xbd068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.deqz", {{ 0xbf061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dgez", {{ 0xbf063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dgtz", {{ 0xbf065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dlez", {{ 0xbf064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dltz", {{ 0xbf062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dnez", {{ 0xbf060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.even", {{ 0xbf067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.odd", {{ 0xbf066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.weqz", {{ 0xbf069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wgez", {{ 0xbf06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wgtz", {{ 0xbf06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wlez", {{ 0xbf06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wltz", {{ 0xbf06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wnez", {{ 0xbf068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.deqz", {{ 0xbe061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dgez", {{ 0xbe063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dgtz", {{ 0xbe065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dlez", {{ 0xbe064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dltz", {{ 0xbe062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dnez", {{ 0xbe060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.even", {{ 0xbe067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.odd", {{ 0xbe066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.weqz", {{ 0xbe069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wgez", {{ 0xbe06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wgtz", {{ 0xbe06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wlez", {{ 0xbe06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wltz", {{ 0xbe06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wnez", {{ 0xbe068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.deqz", {{ 0xbc061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dgez", {{ 0xbc063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dgtz", {{ 0xbc065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dlez", {{ 0xbc064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dltz", {{ 0xbc062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dnez", {{ 0xbc060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.even", {{ 0xbc067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.odd", {{ 0xbc066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.weqz", {{ 0xbc069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wgez", {{ 0xbc06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wgtz", {{ 0xbc06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wlez", {{ 0xbc06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wltz", {{ 0xbc06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wnez", {{ 0xbc068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo", {{ 0x3c040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s010_registerZ_simple */
	{"lo.s", {{ 0x3d040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s010_registerZ_simple */
	{"lo.u", {{ 0x3e040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s010_registerZ_simple */
	{"lo.us", {{ 0x3f040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s010_registerZ_simple */
	{"lo", {{ 0xbc040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s037_registerZ_double */
	{"lo.s", {{ 0xbd040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s037_registerZ_double */
	{"lo.u", {{ 0xbe040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s037_registerZ_double */
	{"lo.us", {{ 0xbf040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_s037_registerZ_double */
	{"lo", {{ 0xbc040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_w064_registerZ_triple */
	{"lo.s", {{ 0xbd040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_w064_registerZ_triple */
	{"lo.u", {{ 0xbe040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_w064_registerZ_triple */
	{"lo.us", {{ 0xbf040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LO_variant_registerN_w064_registerZ_triple */
	{"lq.s", {{ 0x3d02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.s.xs", {{ 0x3d02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.us", {{ 0x3f02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.us.xs", {{ 0x3f02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.u", {{ 0x3e02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.u.xs", {{ 0x3e02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq", {{ 0x3c02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.xs", {{ 0x3c02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.s.deqz", {{ 0x3d021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dgez", {{ 0x3d023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dgtz", {{ 0x3d025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dlez", {{ 0x3d024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dltz", {{ 0x3d022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dnez", {{ 0x3d020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.even", {{ 0x3d027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.odd", {{ 0x3d026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.weqz", {{ 0x3d029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wgez", {{ 0x3d02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wgtz", {{ 0x3d02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wlez", {{ 0x3d02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wltz", {{ 0x3d02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wnez", {{ 0x3d028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.deqz", {{ 0x3f021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dgez", {{ 0x3f023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dgtz", {{ 0x3f025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dlez", {{ 0x3f024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dltz", {{ 0x3f022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dnez", {{ 0x3f020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.even", {{ 0x3f027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.odd", {{ 0x3f026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.weqz", {{ 0x3f029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wgez", {{ 0x3f02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wgtz", {{ 0x3f02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wlez", {{ 0x3f02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wltz", {{ 0x3f02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wnez", {{ 0x3f028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.deqz", {{ 0x3e021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dgez", {{ 0x3e023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dgtz", {{ 0x3e025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dlez", {{ 0x3e024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dltz", {{ 0x3e022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dnez", {{ 0x3e020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.even", {{ 0x3e027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.odd", {{ 0x3e026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.weqz", {{ 0x3e029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wgez", {{ 0x3e02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wgtz", {{ 0x3e02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wlez", {{ 0x3e02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wltz", {{ 0x3e02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wnez", {{ 0x3e028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.deqz", {{ 0x3c021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dgez", {{ 0x3c023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dgtz", {{ 0x3c025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dlez", {{ 0x3c024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dltz", {{ 0x3c022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dnez", {{ 0x3c020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.even", {{ 0x3c027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.odd", {{ 0x3c026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.weqz", {{ 0x3c029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wgez", {{ 0x3c02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wgtz", {{ 0x3c02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wlez", {{ 0x3c02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wltz", {{ 0x3c02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wnez", {{ 0x3c028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.deqz", {{ 0xbd021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dgez", {{ 0xbd023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dgtz", {{ 0xbd025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dlez", {{ 0xbd024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dltz", {{ 0xbd022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dnez", {{ 0xbd020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.even", {{ 0xbd027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.odd", {{ 0xbd026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.weqz", {{ 0xbd029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wgez", {{ 0xbd02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wgtz", {{ 0xbd02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wlez", {{ 0xbd02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wltz", {{ 0xbd02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wnez", {{ 0xbd028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.deqz", {{ 0xbf021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dgez", {{ 0xbf023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dgtz", {{ 0xbf025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dlez", {{ 0xbf024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dltz", {{ 0xbf022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dnez", {{ 0xbf020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.even", {{ 0xbf027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.odd", {{ 0xbf026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.weqz", {{ 0xbf029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wgez", {{ 0xbf02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wgtz", {{ 0xbf02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wlez", {{ 0xbf02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wltz", {{ 0xbf02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wnez", {{ 0xbf028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.deqz", {{ 0xbe021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dgez", {{ 0xbe023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dgtz", {{ 0xbe025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dlez", {{ 0xbe024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dltz", {{ 0xbe022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dnez", {{ 0xbe020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.even", {{ 0xbe027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.odd", {{ 0xbe026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.weqz", {{ 0xbe029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wgez", {{ 0xbe02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wgtz", {{ 0xbe02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wlez", {{ 0xbe02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wltz", {{ 0xbe02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wnez", {{ 0xbe028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.deqz", {{ 0xbc021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dgez", {{ 0xbc023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dgtz", {{ 0xbc025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dlez", {{ 0xbc024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dltz", {{ 0xbc022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dnez", {{ 0xbc020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.even", {{ 0xbc027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.odd", {{ 0xbc026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.weqz", {{ 0xbc029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wgez", {{ 0xbc02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wgtz", {{ 0xbc02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wlez", {{ 0xbc02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wltz", {{ 0xbc02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wnez", {{ 0xbc028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.deqz", {{ 0xbd021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dgez", {{ 0xbd023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dgtz", {{ 0xbd025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dlez", {{ 0xbd024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dltz", {{ 0xbd022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dnez", {{ 0xbd020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.even", {{ 0xbd027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.odd", {{ 0xbd026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.weqz", {{ 0xbd029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wgez", {{ 0xbd02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wgtz", {{ 0xbd02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wlez", {{ 0xbd02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wltz", {{ 0xbd02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wnez", {{ 0xbd028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.deqz", {{ 0xbf021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dgez", {{ 0xbf023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dgtz", {{ 0xbf025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dlez", {{ 0xbf024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dltz", {{ 0xbf022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dnez", {{ 0xbf020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.even", {{ 0xbf027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.odd", {{ 0xbf026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.weqz", {{ 0xbf029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wgez", {{ 0xbf02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wgtz", {{ 0xbf02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wlez", {{ 0xbf02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wltz", {{ 0xbf02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wnez", {{ 0xbf028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.deqz", {{ 0xbe021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dgez", {{ 0xbe023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dgtz", {{ 0xbe025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dlez", {{ 0xbe024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dltz", {{ 0xbe022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dnez", {{ 0xbe020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.even", {{ 0xbe027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.odd", {{ 0xbe026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.weqz", {{ 0xbe029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wgez", {{ 0xbe02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wgtz", {{ 0xbe02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wlez", {{ 0xbe02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wltz", {{ 0xbe02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wnez", {{ 0xbe028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.deqz", {{ 0xbc021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dgez", {{ 0xbc023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dgtz", {{ 0xbc025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dlez", {{ 0xbc024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dltz", {{ 0xbc022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dnez", {{ 0xbc020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.even", {{ 0xbc027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.odd", {{ 0xbc026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.weqz", {{ 0xbc029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wgez", {{ 0xbc02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wgtz", {{ 0xbc02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wlez", {{ 0xbc02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wltz", {{ 0xbc02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wnez", {{ 0xbc028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq", {{ 0x3c000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s010_registerZ_simple */
	{"lq.s", {{ 0x3d000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s010_registerZ_simple */
	{"lq.u", {{ 0x3e000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s010_registerZ_simple */
	{"lq.us", {{ 0x3f000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s010_registerZ_simple */
	{"lq", {{ 0xbc000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s037_registerZ_double */
	{"lq.s", {{ 0xbd000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s037_registerZ_double */
	{"lq.u", {{ 0xbe000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s037_registerZ_double */
	{"lq.us", {{ 0xbf000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_s037_registerZ_double */
	{"lq", {{ 0xbc000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_w064_registerZ_triple */
	{"lq.s", {{ 0xbd000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_w064_registerZ_triple */
	{"lq.u", {{ 0xbe000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_w064_registerZ_triple */
	{"lq.us", {{ 0xbf000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LQ_variant_registerM_w064_registerZ_triple */
	{"lv.c0.s", {{ 0x2303e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0.s.xs", {{ 0x2303f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0", {{ 0x2203e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0.xs", {{ 0x2203f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1.s", {{ 0x2307e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1.s.xs", {{ 0x2307f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1", {{ 0x2207e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1.xs", {{ 0x2207f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2.s", {{ 0x230be000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2.s.xs", {{ 0x230bf000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2", {{ 0x220be000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2.xs", {{ 0x220bf000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3.s", {{ 0x230fe000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3.s.xs", {{ 0x230ff000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3", {{ 0x220fe000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3.xs", {{ 0x220ff000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0.s.deqz", {{ 0x23031000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dgez", {{ 0x23033000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dgtz", {{ 0x23035000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dlez", {{ 0x23034000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dltz", {{ 0x23032000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dnez", {{ 0x23030000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.even", {{ 0x23037000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.odd", {{ 0x23036000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.weqz", {{ 0x23039000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wgez", {{ 0x2303b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wgtz", {{ 0x2303d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wlez", {{ 0x2303c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wltz", {{ 0x2303a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wnez", {{ 0x23038000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.deqz", {{ 0x22031000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dgez", {{ 0x22033000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dgtz", {{ 0x22035000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dlez", {{ 0x22034000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dltz", {{ 0x22032000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dnez", {{ 0x22030000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.even", {{ 0x22037000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.odd", {{ 0x22036000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.weqz", {{ 0x22039000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wgez", {{ 0x2203b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wgtz", {{ 0x2203d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wlez", {{ 0x2203c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wltz", {{ 0x2203a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wnez", {{ 0x22038000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.deqz", {{ 0x23071000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dgez", {{ 0x23073000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dgtz", {{ 0x23075000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dlez", {{ 0x23074000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dltz", {{ 0x23072000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dnez", {{ 0x23070000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.even", {{ 0x23077000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.odd", {{ 0x23076000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.weqz", {{ 0x23079000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wgez", {{ 0x2307b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wgtz", {{ 0x2307d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wlez", {{ 0x2307c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wltz", {{ 0x2307a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wnez", {{ 0x23078000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.deqz", {{ 0x22071000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dgez", {{ 0x22073000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dgtz", {{ 0x22075000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dlez", {{ 0x22074000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dltz", {{ 0x22072000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dnez", {{ 0x22070000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.even", {{ 0x22077000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.odd", {{ 0x22076000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.weqz", {{ 0x22079000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wgez", {{ 0x2207b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wgtz", {{ 0x2207d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wlez", {{ 0x2207c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wltz", {{ 0x2207a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wnez", {{ 0x22078000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.deqz", {{ 0x230b1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dgez", {{ 0x230b3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dgtz", {{ 0x230b5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dlez", {{ 0x230b4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dltz", {{ 0x230b2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dnez", {{ 0x230b0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.even", {{ 0x230b7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.odd", {{ 0x230b6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.weqz", {{ 0x230b9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wgez", {{ 0x230bb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wgtz", {{ 0x230bd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wlez", {{ 0x230bc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wltz", {{ 0x230ba000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wnez", {{ 0x230b8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.deqz", {{ 0x220b1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dgez", {{ 0x220b3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dgtz", {{ 0x220b5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dlez", {{ 0x220b4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dltz", {{ 0x220b2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dnez", {{ 0x220b0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.even", {{ 0x220b7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.odd", {{ 0x220b6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.weqz", {{ 0x220b9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wgez", {{ 0x220bb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wgtz", {{ 0x220bd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wlez", {{ 0x220bc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wltz", {{ 0x220ba000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wnez", {{ 0x220b8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.deqz", {{ 0x230f1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dgez", {{ 0x230f3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dgtz", {{ 0x230f5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dlez", {{ 0x230f4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dltz", {{ 0x230f2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dnez", {{ 0x230f0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.even", {{ 0x230f7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.odd", {{ 0x230f6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.weqz", {{ 0x230f9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wgez", {{ 0x230fb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wgtz", {{ 0x230fd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wlez", {{ 0x230fc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wltz", {{ 0x230fa000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wnez", {{ 0x230f8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.deqz", {{ 0x220f1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dgez", {{ 0x220f3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dgtz", {{ 0x220f5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dlez", {{ 0x220f4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dltz", {{ 0x220f2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dnez", {{ 0x220f0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.even", {{ 0x220f7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.odd", {{ 0x220f6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.weqz", {{ 0x220f9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wgez", {{ 0x220fb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wgtz", {{ 0x220fd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wlez", {{ 0x220fc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wltz", {{ 0x220fa000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wnez", {{ 0x220f8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.deqz", {{ 0xa3031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dgez", {{ 0xa3033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dgtz", {{ 0xa3035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dlez", {{ 0xa3034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dltz", {{ 0xa3032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dnez", {{ 0xa3030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.even", {{ 0xa3037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.odd", {{ 0xa3036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.weqz", {{ 0xa3039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wgez", {{ 0xa303b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wgtz", {{ 0xa303d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wlez", {{ 0xa303c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wltz", {{ 0xa303a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wnez", {{ 0xa3038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.deqz", {{ 0xa2031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dgez", {{ 0xa2033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dgtz", {{ 0xa2035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dlez", {{ 0xa2034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dltz", {{ 0xa2032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dnez", {{ 0xa2030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.even", {{ 0xa2037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.odd", {{ 0xa2036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.weqz", {{ 0xa2039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wgez", {{ 0xa203b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wgtz", {{ 0xa203d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wlez", {{ 0xa203c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wltz", {{ 0xa203a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wnez", {{ 0xa2038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.deqz", {{ 0xa3071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dgez", {{ 0xa3073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dgtz", {{ 0xa3075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dlez", {{ 0xa3074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dltz", {{ 0xa3072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dnez", {{ 0xa3070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.even", {{ 0xa3077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.odd", {{ 0xa3076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.weqz", {{ 0xa3079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wgez", {{ 0xa307b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wgtz", {{ 0xa307d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wlez", {{ 0xa307c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wltz", {{ 0xa307a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wnez", {{ 0xa3078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.deqz", {{ 0xa2071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dgez", {{ 0xa2073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dgtz", {{ 0xa2075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dlez", {{ 0xa2074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dltz", {{ 0xa2072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dnez", {{ 0xa2070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.even", {{ 0xa2077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.odd", {{ 0xa2076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.weqz", {{ 0xa2079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wgez", {{ 0xa207b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wgtz", {{ 0xa207d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wlez", {{ 0xa207c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wltz", {{ 0xa207a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wnez", {{ 0xa2078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.deqz", {{ 0xa30b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dgez", {{ 0xa30b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dgtz", {{ 0xa30b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dlez", {{ 0xa30b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dltz", {{ 0xa30b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dnez", {{ 0xa30b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.even", {{ 0xa30b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.odd", {{ 0xa30b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.weqz", {{ 0xa30b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wgez", {{ 0xa30bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wgtz", {{ 0xa30bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wlez", {{ 0xa30bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wltz", {{ 0xa30ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wnez", {{ 0xa30b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.deqz", {{ 0xa20b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dgez", {{ 0xa20b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dgtz", {{ 0xa20b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dlez", {{ 0xa20b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dltz", {{ 0xa20b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dnez", {{ 0xa20b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.even", {{ 0xa20b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.odd", {{ 0xa20b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.weqz", {{ 0xa20b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wgez", {{ 0xa20bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wgtz", {{ 0xa20bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wlez", {{ 0xa20bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wltz", {{ 0xa20ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wnez", {{ 0xa20b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.deqz", {{ 0xa30f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dgez", {{ 0xa30f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dgtz", {{ 0xa30f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dlez", {{ 0xa30f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dltz", {{ 0xa30f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dnez", {{ 0xa30f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.even", {{ 0xa30f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.odd", {{ 0xa30f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.weqz", {{ 0xa30f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wgez", {{ 0xa30fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wgtz", {{ 0xa30fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wlez", {{ 0xa30fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wltz", {{ 0xa30fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wnez", {{ 0xa30f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.deqz", {{ 0xa20f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dgez", {{ 0xa20f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dgtz", {{ 0xa20f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dlez", {{ 0xa20f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dltz", {{ 0xa20f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dnez", {{ 0xa20f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.even", {{ 0xa20f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.odd", {{ 0xa20f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.weqz", {{ 0xa20f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wgez", {{ 0xa20fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wgtz", {{ 0xa20fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wlez", {{ 0xa20fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wltz", {{ 0xa20fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wnez", {{ 0xa20f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.deqz", {{ 0xa3031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dgez", {{ 0xa3033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dgtz", {{ 0xa3035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dlez", {{ 0xa3034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dltz", {{ 0xa3032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dnez", {{ 0xa3030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.even", {{ 0xa3037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.odd", {{ 0xa3036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.weqz", {{ 0xa3039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wgez", {{ 0xa303b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wgtz", {{ 0xa303d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wlez", {{ 0xa303c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wltz", {{ 0xa303a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wnez", {{ 0xa3038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.deqz", {{ 0xa2031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dgez", {{ 0xa2033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dgtz", {{ 0xa2035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dlez", {{ 0xa2034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dltz", {{ 0xa2032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dnez", {{ 0xa2030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.even", {{ 0xa2037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.odd", {{ 0xa2036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.weqz", {{ 0xa2039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wgez", {{ 0xa203b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wgtz", {{ 0xa203d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wlez", {{ 0xa203c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wltz", {{ 0xa203a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wnez", {{ 0xa2038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.deqz", {{ 0xa3071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dgez", {{ 0xa3073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dgtz", {{ 0xa3075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dlez", {{ 0xa3074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dltz", {{ 0xa3072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dnez", {{ 0xa3070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.even", {{ 0xa3077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.odd", {{ 0xa3076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.weqz", {{ 0xa3079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wgez", {{ 0xa307b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wgtz", {{ 0xa307d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wlez", {{ 0xa307c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wltz", {{ 0xa307a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wnez", {{ 0xa3078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.deqz", {{ 0xa2071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dgez", {{ 0xa2073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dgtz", {{ 0xa2075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dlez", {{ 0xa2074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dltz", {{ 0xa2072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dnez", {{ 0xa2070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.even", {{ 0xa2077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.odd", {{ 0xa2076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.weqz", {{ 0xa2079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wgez", {{ 0xa207b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wgtz", {{ 0xa207d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wlez", {{ 0xa207c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wltz", {{ 0xa207a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wnez", {{ 0xa2078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.deqz", {{ 0xa30b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dgez", {{ 0xa30b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dgtz", {{ 0xa30b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dlez", {{ 0xa30b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dltz", {{ 0xa30b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dnez", {{ 0xa30b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.even", {{ 0xa30b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.odd", {{ 0xa30b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.weqz", {{ 0xa30b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wgez", {{ 0xa30bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wgtz", {{ 0xa30bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wlez", {{ 0xa30bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wltz", {{ 0xa30ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wnez", {{ 0xa30b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.deqz", {{ 0xa20b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dgez", {{ 0xa20b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dgtz", {{ 0xa20b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dlez", {{ 0xa20b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dltz", {{ 0xa20b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dnez", {{ 0xa20b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.even", {{ 0xa20b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.odd", {{ 0xa20b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.weqz", {{ 0xa20b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wgez", {{ 0xa20bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wgtz", {{ 0xa20bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wlez", {{ 0xa20bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wltz", {{ 0xa20ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wnez", {{ 0xa20b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.deqz", {{ 0xa30f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dgez", {{ 0xa30f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dgtz", {{ 0xa30f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dlez", {{ 0xa30f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dltz", {{ 0xa30f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dnez", {{ 0xa30f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.even", {{ 0xa30f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.odd", {{ 0xa30f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.weqz", {{ 0xa30f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wgez", {{ 0xa30fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wgtz", {{ 0xa30fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wlez", {{ 0xa30fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wltz", {{ 0xa30fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wnez", {{ 0xa30f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.deqz", {{ 0xa20f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dgez", {{ 0xa20f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dgtz", {{ 0xa20f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dlez", {{ 0xa20f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dltz", {{ 0xa20f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dnez", {{ 0xa20f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.even", {{ 0xa20f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.odd", {{ 0xa20f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.weqz", {{ 0xa20f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wgez", {{ 0xa20fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wgtz", {{ 0xa20fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wlez", {{ 0xa20fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wltz", {{ 0xa20fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wnez", {{ 0xa20f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0", {{ 0x22010000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c0.s", {{ 0x23010000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c1", {{ 0x22050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c1.s", {{ 0x23050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c2", {{ 0x22090000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c2.s", {{ 0x23090000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c3", {{ 0x220d0000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c3.s", {{ 0x230d0000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c0", {{ 0xa2010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c0.s", {{ 0xa3010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c1", {{ 0xa2050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c1.s", {{ 0xa3050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c2", {{ 0xa2090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c2.s", {{ 0xa3090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c3", {{ 0xa20d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c3.s", {{ 0xa30d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c0", {{ 0xa2010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c0.s", {{ 0xa3010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c1", {{ 0xa2050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c1.s", {{ 0xa3050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c2", {{ 0xa2090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c2.s", {{ 0xa3090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c3", {{ 0xa20d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c3.s", {{ 0xa30d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.s", {{ 0x2103e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv.s.xs", {{ 0x2103f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv", {{ 0x2003e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv.xs", {{ 0x2003f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv.s.deqz", {{ 0x21031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dgez", {{ 0x21033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dgtz", {{ 0x21035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dlez", {{ 0x21034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dltz", {{ 0x21032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dnez", {{ 0x21030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.even", {{ 0x21037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.odd", {{ 0x21036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.weqz", {{ 0x21039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wgez", {{ 0x2103b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wgtz", {{ 0x2103d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wlez", {{ 0x2103c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wltz", {{ 0x2103a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wnez", {{ 0x21038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.deqz", {{ 0x20031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dgez", {{ 0x20033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dgtz", {{ 0x20035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dlez", {{ 0x20034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dltz", {{ 0x20032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dnez", {{ 0x20030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.even", {{ 0x20037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.odd", {{ 0x20036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.weqz", {{ 0x20039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wgez", {{ 0x2003b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wgtz", {{ 0x2003d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wlez", {{ 0x2003c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wltz", {{ 0x2003a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wnez", {{ 0x20038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.deqz", {{ 0xa1031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dgez", {{ 0xa1033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dgtz", {{ 0xa1035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dlez", {{ 0xa1034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dltz", {{ 0xa1032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dnez", {{ 0xa1030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.even", {{ 0xa1037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.odd", {{ 0xa1036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.weqz", {{ 0xa1039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wgez", {{ 0xa103b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wgtz", {{ 0xa103d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wlez", {{ 0xa103c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wltz", {{ 0xa103a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wnez", {{ 0xa1038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.deqz", {{ 0xa0031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dgez", {{ 0xa0033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dgtz", {{ 0xa0035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dlez", {{ 0xa0034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dltz", {{ 0xa0032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dnez", {{ 0xa0030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.even", {{ 0xa0037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.odd", {{ 0xa0036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.weqz", {{ 0xa0039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wgez", {{ 0xa003b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wgtz", {{ 0xa003d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wlez", {{ 0xa003c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wltz", {{ 0xa003a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wnez", {{ 0xa0038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.deqz", {{ 0xa1031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dgez", {{ 0xa1033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dgtz", {{ 0xa1035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dlez", {{ 0xa1034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dltz", {{ 0xa1032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dnez", {{ 0xa1030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.even", {{ 0xa1037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.odd", {{ 0xa1036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.weqz", {{ 0xa1039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wgez", {{ 0xa103b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wgtz", {{ 0xa103d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wlez", {{ 0xa103c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wltz", {{ 0xa103a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wnez", {{ 0xa1038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.deqz", {{ 0xa0031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dgez", {{ 0xa0033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dgtz", {{ 0xa0035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dlez", {{ 0xa0034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dltz", {{ 0xa0032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dnez", {{ 0xa0030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.even", {{ 0xa0037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.odd", {{ 0xa0036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.weqz", {{ 0xa0039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wgez", {{ 0xa003b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wgtz", {{ 0xa003d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wlez", {{ 0xa003c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wltz", {{ 0xa003a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wnez", {{ 0xa0038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv", {{ 0x20010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_registerA_s010_registerZ_simple */
	{"lv.s", {{ 0x21010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_registerA_s010_registerZ_simple */
	{"lv", {{ 0xa0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registera_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_registerA_s037_registerZ_double */
	{"lv.s", {{ 0xa1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registera_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_registerA_s037_registerZ_double */
	{"lv", {{ 0xa0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registera_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_registerA_w064_registerZ_triple */
	{"lv.s", {{ 0xa1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registera_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LV_speculate_registerA_w064_registerZ_triple */
	{"lws.s", {{ 0x3502e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.s.xs", {{ 0x3502f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.us", {{ 0x3702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.us.xs", {{ 0x3702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.u", {{ 0x3602e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.u.xs", {{ 0x3602f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws", {{ 0x3402e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.xs", {{ 0x3402f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.s.deqz", {{ 0x35021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dgez", {{ 0x35023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dgtz", {{ 0x35025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dlez", {{ 0x35024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dltz", {{ 0x35022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dnez", {{ 0x35020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.even", {{ 0x35027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.odd", {{ 0x35026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.weqz", {{ 0x35029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wgez", {{ 0x3502b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wgtz", {{ 0x3502d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wlez", {{ 0x3502c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wltz", {{ 0x3502a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wnez", {{ 0x35028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.deqz", {{ 0x37021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dgez", {{ 0x37023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dgtz", {{ 0x37025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dlez", {{ 0x37024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dltz", {{ 0x37022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dnez", {{ 0x37020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.even", {{ 0x37027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.odd", {{ 0x37026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.weqz", {{ 0x37029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wgez", {{ 0x3702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wgtz", {{ 0x3702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wlez", {{ 0x3702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wltz", {{ 0x3702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wnez", {{ 0x37028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.deqz", {{ 0x36021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dgez", {{ 0x36023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dgtz", {{ 0x36025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dlez", {{ 0x36024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dltz", {{ 0x36022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dnez", {{ 0x36020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.even", {{ 0x36027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.odd", {{ 0x36026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.weqz", {{ 0x36029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wgez", {{ 0x3602b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wgtz", {{ 0x3602d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wlez", {{ 0x3602c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wltz", {{ 0x3602a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wnez", {{ 0x36028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.deqz", {{ 0x34021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dgez", {{ 0x34023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dgtz", {{ 0x34025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dlez", {{ 0x34024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dltz", {{ 0x34022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dnez", {{ 0x34020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.even", {{ 0x34027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.odd", {{ 0x34026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.weqz", {{ 0x34029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wgez", {{ 0x3402b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wgtz", {{ 0x3402d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wlez", {{ 0x3402c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wltz", {{ 0x3402a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wnez", {{ 0x34028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.deqz", {{ 0xb5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dgez", {{ 0xb5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dgtz", {{ 0xb5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dlez", {{ 0xb5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dltz", {{ 0xb5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dnez", {{ 0xb5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.even", {{ 0xb5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.odd", {{ 0xb5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.weqz", {{ 0xb5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wgez", {{ 0xb502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wgtz", {{ 0xb502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wlez", {{ 0xb502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wltz", {{ 0xb502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wnez", {{ 0xb5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.deqz", {{ 0xb7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dgez", {{ 0xb7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dgtz", {{ 0xb7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dlez", {{ 0xb7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dltz", {{ 0xb7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dnez", {{ 0xb7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.even", {{ 0xb7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.odd", {{ 0xb7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.weqz", {{ 0xb7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wgez", {{ 0xb702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wgtz", {{ 0xb702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wlez", {{ 0xb702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wltz", {{ 0xb702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wnez", {{ 0xb7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.deqz", {{ 0xb6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dgez", {{ 0xb6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dgtz", {{ 0xb6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dlez", {{ 0xb6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dltz", {{ 0xb6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dnez", {{ 0xb6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.even", {{ 0xb6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.odd", {{ 0xb6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.weqz", {{ 0xb6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wgez", {{ 0xb602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wgtz", {{ 0xb602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wlez", {{ 0xb602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wltz", {{ 0xb602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wnez", {{ 0xb6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.deqz", {{ 0xb4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dgez", {{ 0xb4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dgtz", {{ 0xb4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dlez", {{ 0xb4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dltz", {{ 0xb4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dnez", {{ 0xb4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.even", {{ 0xb4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.odd", {{ 0xb4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.weqz", {{ 0xb4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wgez", {{ 0xb402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wgtz", {{ 0xb402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wlez", {{ 0xb402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wltz", {{ 0xb402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wnez", {{ 0xb4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.deqz", {{ 0xb5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dgez", {{ 0xb5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dgtz", {{ 0xb5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dlez", {{ 0xb5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dltz", {{ 0xb5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dnez", {{ 0xb5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.even", {{ 0xb5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.odd", {{ 0xb5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.weqz", {{ 0xb5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wgez", {{ 0xb502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wgtz", {{ 0xb502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wlez", {{ 0xb502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wltz", {{ 0xb502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wnez", {{ 0xb5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.deqz", {{ 0xb7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dgez", {{ 0xb7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dgtz", {{ 0xb7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dlez", {{ 0xb7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dltz", {{ 0xb7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dnez", {{ 0xb7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.even", {{ 0xb7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.odd", {{ 0xb7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.weqz", {{ 0xb7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wgez", {{ 0xb702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wgtz", {{ 0xb702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wlez", {{ 0xb702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wltz", {{ 0xb702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wnez", {{ 0xb7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.deqz", {{ 0xb6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dgez", {{ 0xb6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dgtz", {{ 0xb6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dlez", {{ 0xb6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dltz", {{ 0xb6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dnez", {{ 0xb6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.even", {{ 0xb6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.odd", {{ 0xb6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.weqz", {{ 0xb6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wgez", {{ 0xb602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wgtz", {{ 0xb602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wlez", {{ 0xb602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wltz", {{ 0xb602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wnez", {{ 0xb6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.deqz", {{ 0xb4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dgez", {{ 0xb4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dgtz", {{ 0xb4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dlez", {{ 0xb4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dltz", {{ 0xb4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dnez", {{ 0xb4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.even", {{ 0xb4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.odd", {{ 0xb4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.weqz", {{ 0xb4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wgez", {{ 0xb402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wgtz", {{ 0xb402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wlez", {{ 0xb402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wltz", {{ 0xb402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wnez", {{ 0xb4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws", {{ 0x34000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s010_registerZ_simple */
	{"lws.s", {{ 0x35000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s010_registerZ_simple */
	{"lws.u", {{ 0x36000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s010_registerZ_simple */
	{"lws.us", {{ 0x37000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s010_registerZ_simple */
	{"lws", {{ 0xb4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s037_registerZ_double */
	{"lws.s", {{ 0xb5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s037_registerZ_double */
	{"lws.u", {{ 0xb6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s037_registerZ_double */
	{"lws.us", {{ 0xb7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_s037_registerZ_double */
	{"lws", {{ 0xb4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_w064_registerZ_triple */
	{"lws.s", {{ 0xb5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_w064_registerZ_triple */
	{"lws.u", {{ 0xb6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_w064_registerZ_triple */
	{"lws.us", {{ 0xb7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWS_variant_registerW_w064_registerZ_triple */
	{"lwz.s", {{ 0x3102e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.s.xs", {{ 0x3102f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.us", {{ 0x3302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.us.xs", {{ 0x3302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.u", {{ 0x3202e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.u.xs", {{ 0x3202f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz", {{ 0x3002e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.xs", {{ 0x3002f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.s.deqz", {{ 0x31021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dgez", {{ 0x31023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dgtz", {{ 0x31025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dlez", {{ 0x31024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dltz", {{ 0x31022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dnez", {{ 0x31020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.even", {{ 0x31027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.odd", {{ 0x31026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.weqz", {{ 0x31029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wgez", {{ 0x3102b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wgtz", {{ 0x3102d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wlez", {{ 0x3102c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wltz", {{ 0x3102a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wnez", {{ 0x31028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.deqz", {{ 0x33021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dgez", {{ 0x33023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dgtz", {{ 0x33025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dlez", {{ 0x33024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dltz", {{ 0x33022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dnez", {{ 0x33020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.even", {{ 0x33027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.odd", {{ 0x33026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.weqz", {{ 0x33029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wgez", {{ 0x3302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wgtz", {{ 0x3302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wlez", {{ 0x3302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wltz", {{ 0x3302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wnez", {{ 0x33028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.deqz", {{ 0x32021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dgez", {{ 0x32023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dgtz", {{ 0x32025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dlez", {{ 0x32024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dltz", {{ 0x32022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dnez", {{ 0x32020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.even", {{ 0x32027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.odd", {{ 0x32026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.weqz", {{ 0x32029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wgez", {{ 0x3202b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wgtz", {{ 0x3202d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wlez", {{ 0x3202c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wltz", {{ 0x3202a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wnez", {{ 0x32028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.deqz", {{ 0x30021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dgez", {{ 0x30023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dgtz", {{ 0x30025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dlez", {{ 0x30024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dltz", {{ 0x30022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dnez", {{ 0x30020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.even", {{ 0x30027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.odd", {{ 0x30026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.weqz", {{ 0x30029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wgez", {{ 0x3002b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wgtz", {{ 0x3002d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wlez", {{ 0x3002c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wltz", {{ 0x3002a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wnez", {{ 0x30028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.deqz", {{ 0xb1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dgez", {{ 0xb1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dgtz", {{ 0xb1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dlez", {{ 0xb1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dltz", {{ 0xb1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dnez", {{ 0xb1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.even", {{ 0xb1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.odd", {{ 0xb1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.weqz", {{ 0xb1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wgez", {{ 0xb102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wgtz", {{ 0xb102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wlez", {{ 0xb102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wltz", {{ 0xb102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wnez", {{ 0xb1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.deqz", {{ 0xb3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dgez", {{ 0xb3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dgtz", {{ 0xb3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dlez", {{ 0xb3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dltz", {{ 0xb3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dnez", {{ 0xb3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.even", {{ 0xb3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.odd", {{ 0xb3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.weqz", {{ 0xb3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wgez", {{ 0xb302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wgtz", {{ 0xb302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wlez", {{ 0xb302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wltz", {{ 0xb302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wnez", {{ 0xb3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.deqz", {{ 0xb2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dgez", {{ 0xb2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dgtz", {{ 0xb2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dlez", {{ 0xb2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dltz", {{ 0xb2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dnez", {{ 0xb2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.even", {{ 0xb2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.odd", {{ 0xb2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.weqz", {{ 0xb2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wgez", {{ 0xb202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wgtz", {{ 0xb202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wlez", {{ 0xb202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wltz", {{ 0xb202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wnez", {{ 0xb2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.deqz", {{ 0xb0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dgez", {{ 0xb0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dgtz", {{ 0xb0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dlez", {{ 0xb0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dltz", {{ 0xb0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dnez", {{ 0xb0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.even", {{ 0xb0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.odd", {{ 0xb0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.weqz", {{ 0xb0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wgez", {{ 0xb002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wgtz", {{ 0xb002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wlez", {{ 0xb002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wltz", {{ 0xb002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wnez", {{ 0xb0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.deqz", {{ 0xb1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dgez", {{ 0xb1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dgtz", {{ 0xb1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dlez", {{ 0xb1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dltz", {{ 0xb1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dnez", {{ 0xb1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.even", {{ 0xb1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.odd", {{ 0xb1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.weqz", {{ 0xb1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wgez", {{ 0xb102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wgtz", {{ 0xb102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wlez", {{ 0xb102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wltz", {{ 0xb102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wnez", {{ 0xb1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.deqz", {{ 0xb3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dgez", {{ 0xb3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dgtz", {{ 0xb3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dlez", {{ 0xb3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dltz", {{ 0xb3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dnez", {{ 0xb3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.even", {{ 0xb3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.odd", {{ 0xb3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.weqz", {{ 0xb3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wgez", {{ 0xb302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wgtz", {{ 0xb302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wlez", {{ 0xb302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wltz", {{ 0xb302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wnez", {{ 0xb3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.deqz", {{ 0xb2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dgez", {{ 0xb2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dgtz", {{ 0xb2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dlez", {{ 0xb2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dltz", {{ 0xb2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dnez", {{ 0xb2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.even", {{ 0xb2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.odd", {{ 0xb2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.weqz", {{ 0xb2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wgez", {{ 0xb202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wgtz", {{ 0xb202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wlez", {{ 0xb202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wltz", {{ 0xb202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wnez", {{ 0xb2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.deqz", {{ 0xb0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dgez", {{ 0xb0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dgtz", {{ 0xb0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dlez", {{ 0xb0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dltz", {{ 0xb0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dnez", {{ 0xb0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.even", {{ 0xb0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.odd", {{ 0xb0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.weqz", {{ 0xb0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wgez", {{ 0xb002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wgtz", {{ 0xb002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wlez", {{ 0xb002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wltz", {{ 0xb002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wnez", {{ 0xb0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v1-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz", {{ 0x30000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz.s", {{ 0x31000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz.u", {{ 0x32000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz.us", {{ 0x33000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz", {{ 0xb0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz.s", {{ 0xb1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz.u", {{ 0xb2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz.us", {{ 0xb3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz", {{ 0xb0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_w064_registerZ_triple */
	{"lwz.s", {{ 0xb1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_w064_registerZ_triple */
	{"lwz.u", {{ 0xb2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_w064_registerZ_triple */
	{"lwz.us", {{ 0xb3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v1-LWZ_variant_registerW_w064_registerZ_triple */
	{"madddt", {{ 0x58001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDDT_registerM_registerZ_registerY_simple */
	{"madddt", {{ 0x48000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDDT_registerM_registerZ_s010_simple */
	{"madddt", {{ 0xc8000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDDT_registerM_registerZ_s037_double */
	{"madddt", {{ 0xc8000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDDT_registerM_registerZ_w064_triple */
	{"maddd", {{ 0x50000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDD_registerW_registerZ_registerY_simple */
	{"maddd", {{ 0x40000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDD_registerW_registerZ_s010_simple */
	{"maddd", {{ 0xc0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDD_registerW_registerZ_s037_double */
	{"maddd", {{ 0xc0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDD_registerW_registerZ_w064_triple */
	{"maddhq", {{ 0x52000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDHQ_registerW_registerZ_registerY_simple */
	{"maddhq", {{ 0x42000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDHQ_registerW_registerZ_s010_simple */
	{"maddhq", {{ 0xc2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDHQ_registerW_registerZ_s037_double */
	{"maddhq", {{ 0xc2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDHQ_registerW_registerZ_w064_triple */
	{"maddhwq", {{ 0x50004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDHWQ_registerM_registerZ_registerY_simple */
	{"maddsudt", {{ 0x5a001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUDT_registerM_registerZ_registerY_simple */
	{"maddsudt", {{ 0x4a000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUDT_registerM_registerZ_s010_simple */
	{"maddsudt", {{ 0xca000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUDT_registerM_registerZ_s037_double */
	{"maddsudt", {{ 0xca000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUDT_registerM_registerZ_w064_triple */
	{"maddsuhwq", {{ 0x52004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUHWQ_registerM_registerZ_registerY_simple */
	{"maddsuwdp", {{ 0x5a000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUWDP_registerM_registerZ_registerY_simple */
	{"maddsuwd", {{ 0x52003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUWD_registerW_registerZ_registerY_simple */
	{"maddsuwd", {{ 0xd2003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDSUWD_registerW_registerZ_w032_double */
	{"maddudt", {{ 0x59001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUDT_registerM_registerZ_registerY_simple */
	{"maddudt", {{ 0x49000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUDT_registerM_registerZ_s010_simple */
	{"maddudt", {{ 0xc9000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUDT_registerM_registerZ_s037_double */
	{"maddudt", {{ 0xc9000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUDT_registerM_registerZ_w064_triple */
	{"madduhwq", {{ 0x51004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUHWQ_registerM_registerZ_registerY_simple */
	{"madduwdp", {{ 0x59000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUWDP_registerM_registerZ_registerY_simple */
	{"madduwd", {{ 0x51003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUWD_registerW_registerZ_registerY_simple */
	{"madduwd", {{ 0xd1003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUWD_registerW_registerZ_w032_double */
	{"madduzdt", {{ 0x5b001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUZDT_registerM_registerZ_registerY_simple */
	{"madduzdt", {{ 0x4b000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUZDT_registerM_registerZ_s010_simple */
	{"madduzdt", {{ 0xcb000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUZDT_registerM_registerZ_s037_double */
	{"madduzdt", {{ 0xcb000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDUZDT_registerM_registerZ_w064_triple */
	{"maddwdp", {{ 0x58000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWDP_registerM_registerZ_registerY_simple */
	{"maddwd", {{ 0x50003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWD_registerW_registerZ_registerY_simple */
	{"maddwd", {{ 0xd0003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWD_registerW_registerZ_w032_double */
	{"maddwp", {{ 0x51000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWP_registerW_registerZ_registerY_simple */
	{"maddwp", {{ 0x41000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWP_registerW_registerZ_s010_simple */
	{"maddwp", {{ 0xc1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWP_registerW_registerZ_s037_double */
	{"maddwp", {{ 0xc1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDWP_registerW_registerZ_w064_triple */
	{"maddw", {{ 0x53003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDW_registerW_registerZ_registerY_simple */
	{"maddw", {{ 0xd3003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MADDW_registerW_registerZ_w032_double */
	{"make", {{ 0x60000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_signed16_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-MAKE_registerW_s016_simple */
	{"make", {{ 0xe0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_extend6_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-MAKE_registerW_s043_double */
	{"make", {{ 0xe0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-MAKE_registerW_w064_triple */
	{"maxd", {{ 0x73010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXD_registerW_registerZ_registerY_simple */
	{"maxd", {{ 0x63000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXD_registerW_registerZ_s010_simple */
	{"maxd", {{ 0xe3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXD_registerW_registerZ_s037_double */
	{"maxd", {{ 0xe3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXD_registerW_registerZ_w064_triple */
	{"maxhq", {{ 0x73013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXHQ_registerW_registerZ_registerY_simple */
	{"maxhq", {{ 0xf3013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXHQ_splat32_registerW_registerZ_w032_double */
	{"maxhq.@", {{ 0xf3013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXHQ_splat32_registerW_registerZ_w032_double */
	{"maxud", {{ 0x77010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUD_registerW_registerZ_registerY_simple */
	{"maxud", {{ 0x67000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUD_registerW_registerZ_s010_simple */
	{"maxud", {{ 0xe7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUD_registerW_registerZ_s037_double */
	{"maxud", {{ 0xe7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUD_registerW_registerZ_w064_triple */
	{"maxuhq", {{ 0x77013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUHQ_registerW_registerZ_registerY_simple */
	{"maxuhq", {{ 0xf7013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUHQ_splat32_registerW_registerZ_w032_double */
	{"maxuhq.@", {{ 0xf7013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUHQ_splat32_registerW_registerZ_w032_double */
	{"maxuwp", {{ 0x77012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUWP_registerW_registerZ_registerY_simple */
	{"maxuwp", {{ 0xf7012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUWP_splat32_registerW_registerZ_w032_double */
	{"maxuwp.@", {{ 0xf7012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUWP_splat32_registerW_registerZ_w032_double */
	{"maxuw", {{ 0x77011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUW_registerW_registerZ_registerY_simple */
	{"maxuw", {{ 0x77000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUW_registerW_registerZ_s010_simple */
	{"maxuw", {{ 0xf7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXUW_registerW_registerZ_s037_double */
	{"maxwp", {{ 0x73012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXWP_registerW_registerZ_registerY_simple */
	{"maxwp", {{ 0xf3012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXWP_splat32_registerW_registerZ_w032_double */
	{"maxwp.@", {{ 0xf3012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXWP_splat32_registerW_registerZ_w032_double */
	{"maxw", {{ 0x73011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXW_registerW_registerZ_registerY_simple */
	{"maxw", {{ 0x73000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXW_registerW_registerZ_s010_simple */
	{"maxw", {{ 0xf3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MAXW_registerW_registerZ_s037_double */
	{"mind", {{ 0x72010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MIND_registerW_registerZ_registerY_simple */
	{"mind", {{ 0x62000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MIND_registerW_registerZ_s010_simple */
	{"mind", {{ 0xe2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MIND_registerW_registerZ_s037_double */
	{"mind", {{ 0xe2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MIND_registerW_registerZ_w064_triple */
	{"minhq", {{ 0x72013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINHQ_registerW_registerZ_registerY_simple */
	{"minhq", {{ 0xf2013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINHQ_splat32_registerW_registerZ_w032_double */
	{"minhq.@", {{ 0xf2013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINHQ_splat32_registerW_registerZ_w032_double */
	{"minud", {{ 0x76010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUD_registerW_registerZ_registerY_simple */
	{"minud", {{ 0x66000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUD_registerW_registerZ_s010_simple */
	{"minud", {{ 0xe6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUD_registerW_registerZ_s037_double */
	{"minud", {{ 0xe6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUD_registerW_registerZ_w064_triple */
	{"minuhq", {{ 0x76013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUHQ_registerW_registerZ_registerY_simple */
	{"minuhq", {{ 0xf6013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUHQ_splat32_registerW_registerZ_w032_double */
	{"minuhq.@", {{ 0xf6013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUHQ_splat32_registerW_registerZ_w032_double */
	{"minuwp", {{ 0x76012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUWP_registerW_registerZ_registerY_simple */
	{"minuwp", {{ 0xf6012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUWP_splat32_registerW_registerZ_w032_double */
	{"minuwp.@", {{ 0xf6012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUWP_splat32_registerW_registerZ_w032_double */
	{"minuw", {{ 0x76011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUW_registerW_registerZ_registerY_simple */
	{"minuw", {{ 0x76000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUW_registerW_registerZ_s010_simple */
	{"minuw", {{ 0xf6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINUW_registerW_registerZ_s037_double */
	{"minwp", {{ 0x72012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINWP_registerW_registerZ_registerY_simple */
	{"minwp", {{ 0xf2012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINWP_splat32_registerW_registerZ_w032_double */
	{"minwp.@", {{ 0xf2012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINWP_splat32_registerW_registerZ_w032_double */
	{"minw", {{ 0x72011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINW_registerW_registerZ_registerY_simple */
	{"minw", {{ 0x72000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINW_registerW_registerZ_s010_simple */
	{"minw", {{ 0xf2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MINW_registerW_registerZ_s037_double */
	{"mm212w", {{ 0x53001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MM212W_registerM_registerZ_registerY_simple */
	{"mm2wq", {{ 0x53001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MM212W_registerM_registerZ_registerY_simple */
	{"mma212w", {{ 0x5b000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MMA212W_registerM_registerZ_registerY_simple */
	{"mm2awq", {{ 0x5b000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MMA212W_registerM_registerZ_registerY_simple */
	{"mma444hbd0", {{ 0x04000000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444HBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mm4ahbd0", {{ 0x04000000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444HBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mma444hbd1", {{ 0x04040000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444HBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mm4ahbd1", {{ 0x04040000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444HBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mma444hd", {{ 0x04080000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444HD_registerAq_registerBq_registerC_registerD_simple */
	{"mm4ahd", {{ 0x04080000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444HD_registerAq_registerBq_registerC_registerD_simple */
	{"mma444suhbd0", {{ 0x04002000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444SUHBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mm4asuhbd0", {{ 0x04002000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444SUHBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mma444suhbd1", {{ 0x04042000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444SUHBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mm4asuhbd1", {{ 0x04042000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444SUHBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mma444suhd", {{ 0x04082000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444SUHD_registerAq_registerBq_registerC_registerD_simple */
	{"mm4asuhd", {{ 0x04082000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444SUHD_registerAq_registerBq_registerC_registerD_simple */
	{"mma444uhbd0", {{ 0x04001000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444UHBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mm4auhbd0", {{ 0x04001000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444UHBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mma444uhbd1", {{ 0x04041000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444UHBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mm4auhbd1", {{ 0x04041000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444UHBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mma444uhd", {{ 0x04081000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444UHD_registerAq_registerBq_registerC_registerD_simple */
	{"mm4auhd", {{ 0x04081000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444UHD_registerAq_registerBq_registerC_registerD_simple */
	{"mma444ushbd0", {{ 0x04003000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444USHBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mm4aushbd0", {{ 0x04003000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444USHBD0_registerAq_registerBq_registerC_registerD_simple */
	{"mma444ushbd1", {{ 0x04043000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444USHBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mm4aushbd1", {{ 0x04043000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444USHBD1_registerAq_registerBq_registerC_registerD_simple */
	{"mma444ushd", {{ 0x04083000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444USHD_registerAq_registerBq_registerC_registerD_simple */
	{"mm4aushd", {{ 0x04083000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA444USHD_registerAq_registerBq_registerC_registerD_simple */
	{"mma484bw", {{ 0x02000000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484BW_registerAp_registerBp_registerC_registerD_simple */
	{"mm4abw", {{ 0x02000000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484BW_registerAp_registerBp_registerC_registerD_simple */
	{"mma484subw", {{ 0x02040000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484SUBW_registerAp_registerBp_registerC_registerD_simple */
	{"mm4asubw", {{ 0x02040000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484SUBW_registerAp_registerBp_registerC_registerD_simple */
	{"mma484ubw", {{ 0x02001000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484UBW_registerAp_registerBp_registerC_registerD_simple */
	{"mm4aubw", {{ 0x02001000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484UBW_registerAp_registerBp_registerC_registerD_simple */
	{"mma484usbw", {{ 0x02041000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484USBW_registerAp_registerBp_registerC_registerD_simple */
	{"mm4ausbw", {{ 0x02041000, 0x7f041000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerc_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v1-MMA484USBW_registerAp_registerBp_registerC_registerD_simple */
	{"mms212w", {{ 0x5f000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MMS212W_registerM_registerZ_registerY_simple */
	{"mm2swq", {{ 0x5f000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MMS212W_registerM_registerZ_registerY_simple */
	{"movetq", {{ 0x7f02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_CRWH, {&kv3_registerah_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MOVETQ_registerAh_registerZ_registerY_simple */
	{"movetq", {{ 0x7f02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_CRWL, {&kv3_registeral_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MOVETQ_registerAl_registerZ_registerY_simple */
	{"msbfdt", {{ 0x5c001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFDT_registerM_registerZ_registerY_simple */
	{"msbfd", {{ 0x54000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFD_registerW_registerZ_registerY_simple */
	{"msbfhq", {{ 0x56000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFHQ_registerW_registerZ_registerY_simple */
	{"msbfhwq", {{ 0x54004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFHWQ_registerM_registerZ_registerY_simple */
	{"msbfsudt", {{ 0x5e001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFSUDT_registerM_registerZ_registerY_simple */
	{"msbfsuhwq", {{ 0x56004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFSUHWQ_registerM_registerZ_registerY_simple */
	{"msbfsuwdp", {{ 0x5e000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFSUWDP_registerM_registerZ_registerY_simple */
	{"msbfsuwd", {{ 0x56003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFSUWD_registerW_registerZ_registerY_simple */
	{"msbfsuwd", {{ 0xd6003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFSUWD_registerW_registerZ_w032_double */
	{"msbfudt", {{ 0x5d001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFUDT_registerM_registerZ_registerY_simple */
	{"msbfuhwq", {{ 0x55004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFUHWQ_registerM_registerZ_registerY_simple */
	{"msbfuwdp", {{ 0x5d000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFUWDP_registerM_registerZ_registerY_simple */
	{"msbfuwd", {{ 0x55003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFUWD_registerW_registerZ_registerY_simple */
	{"msbfuwd", {{ 0xd5003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFUWD_registerW_registerZ_w032_double */
	{"msbfuzdt", {{ 0x5f001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFUZDT_registerM_registerZ_registerY_simple */
	{"msbfwdp", {{ 0x5c000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFWDP_registerM_registerZ_registerY_simple */
	{"msbfwd", {{ 0x54003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFWD_registerW_registerZ_registerY_simple */
	{"msbfwd", {{ 0xd4003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFWD_registerW_registerZ_w032_double */
	{"msbfwp", {{ 0x55000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFWP_registerW_registerZ_registerY_simple */
	{"msbfw", {{ 0x57003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFW_registerW_registerZ_registerY_simple */
	{"msbfw", {{ 0xd7003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MSBFW_registerW_registerZ_w032_double */
	{"mt44d", {{ 0x040c0000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-MT44D_registerAq_registerBq_simple */
	{"mt4x4d", {{ 0x040c0000, 0x7f0c3000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-MT44D_registerAq_registerBq_simple */
	{"mulcwc", {{ 0x55002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULCWC_registerW_registerZ_registerY_simple */
	{"mulcwdc", {{ 0x57002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULCWDC_registerM_registerZ_registerY_simple */
	{"muldt", {{ 0x58041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULDT_registerM_registerZ_registerY_simple */
	{"muldt", {{ 0x48040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULDT_registerM_registerZ_s010_simple */
	{"muldt", {{ 0xc8040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULDT_registerM_registerZ_s037_double */
	{"muldt", {{ 0xc8040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULDT_registerM_registerZ_w064_triple */
	{"muld", {{ 0x54001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULD_registerW_registerZ_registerY_simple */
	{"muld", {{ 0x44000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULD_registerW_registerZ_s010_simple */
	{"muld", {{ 0xc4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULD_registerW_registerZ_s037_double */
	{"muld", {{ 0xc4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULD_registerW_registerZ_w064_triple */
	{"mulhq", {{ 0x56001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULHQ_registerW_registerZ_registerY_simple */
	{"mulhq", {{ 0x46000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULHQ_registerW_registerZ_s010_simple */
	{"mulhq", {{ 0xc6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULHQ_registerW_registerZ_s037_double */
	{"mulhq", {{ 0xc6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULHQ_registerW_registerZ_w064_triple */
	{"mulhwq", {{ 0x58004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULHWQ_registerM_registerZ_registerY_simple */
	{"mulsudt", {{ 0x5a041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUDT_registerM_registerZ_registerY_simple */
	{"mulsudt", {{ 0x4a040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUDT_registerM_registerZ_s010_simple */
	{"mulsudt", {{ 0xca040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUDT_registerM_registerZ_s037_double */
	{"mulsudt", {{ 0xca040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUDT_registerM_registerZ_w064_triple */
	{"mulsuhwq", {{ 0x5a004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUHWQ_registerM_registerZ_registerY_simple */
	{"mulsuwdp", {{ 0x52001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUWDP_registerM_registerZ_registerY_simple */
	{"mulsuwd", {{ 0x5a003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUWD_registerW_registerZ_registerY_simple */
	{"mulsuwd", {{ 0xda003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULSUWD_registerW_registerZ_w032_double */
	{"muludt", {{ 0x59041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUDT_registerM_registerZ_registerY_simple */
	{"muludt", {{ 0x49040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUDT_registerM_registerZ_s010_simple */
	{"muludt", {{ 0xc9040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUDT_registerM_registerZ_s037_double */
	{"muludt", {{ 0xc9040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUDT_registerM_registerZ_w064_triple */
	{"muluhwq", {{ 0x59004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUHWQ_registerM_registerZ_registerY_simple */
	{"muluwdp", {{ 0x51001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUWDP_registerM_registerZ_registerY_simple */
	{"muluwd", {{ 0x59003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUWD_registerW_registerZ_registerY_simple */
	{"muluwd", {{ 0xd9003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULUWD_registerW_registerZ_w032_double */
	{"mulwc", {{ 0x57001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWC_registerW_registerZ_registerY_simple */
	{"mulwc", {{ 0x47000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWC_registerW_registerZ_s010_simple */
	{"mulwc", {{ 0xc7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWC_registerW_registerZ_s037_double */
	{"mulwc", {{ 0xc7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWC_registerW_registerZ_w064_triple */
	{"mulwdc", {{ 0x56002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWDC_registerM_registerZ_registerY_simple */
	{"mulwdp", {{ 0x50001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWDP_registerM_registerZ_registerY_simple */
	{"mulwd", {{ 0x58003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWD_registerW_registerZ_registerY_simple */
	{"mulwd", {{ 0xd8003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWD_registerW_registerZ_w032_double */
	{"mulwp", {{ 0x55001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWP_registerW_registerZ_registerY_simple */
	{"mulwp", {{ 0x45000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWP_registerW_registerZ_s010_simple */
	{"mulwp", {{ 0xc5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWP_registerW_registerZ_s037_double */
	{"mulwp", {{ 0xc5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWP_registerW_registerZ_w064_triple */
	{"mulwq", {{ 0x54002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULWQ_registerM_registerP_registerO_simple */
	{"mulw", {{ 0x5b003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULW_registerW_registerZ_registerY_simple */
	{"mulw", {{ 0xdb003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-MULW_registerW_registerZ_w032_double */
	{"nandd", {{ 0x79010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDD_registerW_registerZ_registerY_simple */
	{"nandd", {{ 0x69000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDD_registerW_registerZ_s010_simple */
	{"nandd", {{ 0xe9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDD_registerW_registerZ_s037_double */
	{"nandd.@", {{ 0xf9010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDD_registerW_registerZ_w032_double */
	{"nandd", {{ 0xe9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDD_registerW_registerZ_w064_triple */
	{"nandw", {{ 0x79011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDW_registerW_registerZ_registerY_simple */
	{"nandw", {{ 0x79000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDW_registerW_registerZ_s010_simple */
	{"nandw", {{ 0xf9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NANDW_registerW_registerZ_s037_double */
	{"nop", {{ 0x7f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_NOP, ((int)0 << 8) | (int)Reservation_kv3_ALU_NOP, {0}, "", ""}, /* Opcode-v1-NOP_simple */
	{"nord", {{ 0x7b010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORD_registerW_registerZ_registerY_simple */
	{"nord", {{ 0x6b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORD_registerW_registerZ_s010_simple */
	{"nord", {{ 0xeb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORD_registerW_registerZ_s037_double */
	{"nord.@", {{ 0xfb010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORD_registerW_registerZ_w032_double */
	{"nord", {{ 0xeb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORD_registerW_registerZ_w064_triple */
	{"norw", {{ 0x7b011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORW_registerW_registerZ_registerY_simple */
	{"norw", {{ 0x7b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORW_registerW_registerZ_s010_simple */
	{"norw", {{ 0xfb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NORW_registerW_registerZ_s037_double */
	{"nxord", {{ 0x7d010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORD_registerW_registerZ_registerY_simple */
	{"nxord", {{ 0x6d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORD_registerW_registerZ_s010_simple */
	{"nxord", {{ 0xed000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORD_registerW_registerZ_s037_double */
	{"nxord.@", {{ 0xfd010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORD_registerW_registerZ_w032_double */
	{"nxord", {{ 0xed000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORD_registerW_registerZ_w064_triple */
	{"nxorw", {{ 0x7d011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORW_registerW_registerZ_registerY_simple */
	{"nxorw", {{ 0x7d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORW_registerW_registerZ_s010_simple */
	{"nxorw", {{ 0xfd000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-NXORW_registerW_registerZ_s037_double */
	{"ord", {{ 0x7a010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORD_registerW_registerZ_registerY_simple */
	{"ord", {{ 0x6a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORD_registerW_registerZ_s010_simple */
	{"ord", {{ 0xea000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORD_registerW_registerZ_s037_double */
	{"ord.@", {{ 0xfa010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORD_registerW_registerZ_w032_double */
	{"ord", {{ 0xea000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORD_registerW_registerZ_w064_triple */
	{"ornd", {{ 0x7f010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORND_registerW_registerZ_registerY_simple */
	{"ornd", {{ 0x6f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORND_registerW_registerZ_s010_simple */
	{"ornd", {{ 0xef000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORND_registerW_registerZ_s037_double */
	{"ornd.@", {{ 0xff010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORND_registerW_registerZ_w032_double */
	{"ornd", {{ 0xef000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORND_registerW_registerZ_w064_triple */
	{"ornw", {{ 0x7f011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORNW_registerW_registerZ_registerY_simple */
	{"ornw", {{ 0x7f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORNW_registerW_registerZ_s010_simple */
	{"ornw", {{ 0xff000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORNW_registerW_registerZ_s037_double */
	{"orw", {{ 0x7a011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORW_registerW_registerZ_registerY_simple */
	{"orw", {{ 0x7a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORW_registerW_registerZ_s010_simple */
	{"orw", {{ 0xfa000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ORW_registerW_registerZ_s037_double */
	{"pcrel", {{ 0x70000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_signed16_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-PCREL_registerW_s016_simple */
	{"pcrel", {{ 0xf0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_extend6_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-PCREL_registerW_s043_double */
	{"pcrel", {{ 0xf0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_FULL_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-PCREL_registerW_w064_triple */
	{"ret", {{ 0x0fd00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {0}, "", ""}, /* Opcode-v1-RET_simple */
	{"rfe", {{ 0x0fd40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-RFE_simple */
	{"rolwps", {{ 0x7e028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ROLWPS_registerW_registerZ_registerY_simple */
	{"rolwps", {{ 0x7e024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ROLWPS_registerW_registerZ_u006_simple */
	{"rolw", {{ 0x7e027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ROLW_registerW_registerZ_registerY_simple */
	{"rolw", {{ 0x7e023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-ROLW_registerW_registerZ_u006_simple */
	{"rorwps", {{ 0x7f028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-RORWPS_registerW_registerZ_registerY_simple */
	{"rorwps", {{ 0x7f024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-RORWPS_registerW_registerZ_u006_simple */
	{"rorw", {{ 0x7f027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-RORW_registerW_registerZ_registerY_simple */
	{"rorw", {{ 0x7f023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-RORW_registerW_registerZ_u006_simple */
	{"rswap", {{ 0x0fc80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_registerz_opnd, &kv3_systemalone_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-RSWAP_registerZ_systemAlone_simple */
	{"rswap", {{ 0x0fc80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &kv3_systems3_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-RSWAP_registerZ_systemS3_simple */
	{"rswap", {{ 0x0fc80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &kv3_systems4_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-RSWAP_registerZ_systemS4_simple */
	{"satd", {{ 0x7e026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SATD_registerW_registerZ_registerY_simple */
	{"satd", {{ 0x7e022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SATD_registerW_registerZ_u006_simple */
	{"sbfcd.i", {{ 0x7f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD.I_registerW_registerZ_registerY_simple */
	{"sbfcid", {{ 0x7f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD.I_registerW_registerZ_registerY_simple */
	{"sbfd.ci", {{ 0x7f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD.I_registerW_registerZ_registerY_simple */
	{"sbfcd.i", {{ 0xff01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD.I_registerW_registerZ_w032_double */
	{"sbfcid", {{ 0xff01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD.I_registerW_registerZ_w032_double */
	{"sbfd.ci", {{ 0xff01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD.I_registerW_registerZ_w032_double */
	{"sbfcd", {{ 0x7f01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD_registerW_registerZ_registerY_simple */
	{"sbfd.c", {{ 0x7f01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD_registerW_registerZ_registerY_simple */
	{"sbfcd", {{ 0xff01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD_registerW_registerZ_w032_double */
	{"sbfd.c", {{ 0xff01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCD_registerW_registerZ_w032_double */
	{"sbfchcp", {{ 0x7d013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCHCP_registerW_registerZ_registerY_simple */
	{"sbfchcp", {{ 0xfd013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCHCP_splat32_registerW_registerZ_w032_double */
	{"sbfchcp.@", {{ 0xfd013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCHCP_splat32_registerW_registerZ_w032_double */
	{"sbfcwc", {{ 0x7d012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCWC_registerW_registerZ_registerY_simple */
	{"sbfcwc", {{ 0xfd012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCWC_splat32_registerW_registerZ_w032_double */
	{"sbfcwc.@", {{ 0xfd012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFCWC_splat32_registerW_registerZ_w032_double */
	{"sbfd", {{ 0x75010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFD_registerW_registerZ_registerY_simple */
	{"sbfd", {{ 0x65000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFD_registerW_registerZ_s010_simple */
	{"sbfd", {{ 0xe5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFD_registerW_registerZ_s037_double */
	{"sbfd", {{ 0xe5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFD_registerW_registerZ_w064_triple */
	{"sbfhq", {{ 0x75013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFHQ_registerW_registerZ_registerY_simple */
	{"sbfhq", {{ 0xf5013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFHQ_splat32_registerW_registerZ_w032_double */
	{"sbfhq.@", {{ 0xf5013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFHQ_splat32_registerW_registerZ_w032_double */
	{"sbfsd", {{ 0x7f01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSD_registerW_registerZ_registerY_simple */
	{"sbfsd", {{ 0x6f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSD_registerW_registerZ_s010_simple */
	{"sbfsd", {{ 0xef010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSD_registerW_registerZ_s037_double */
	{"sbfsd", {{ 0xef010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSD_registerW_registerZ_w064_triple */
	{"sbfshq", {{ 0x7f01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSHQ_registerW_registerZ_registerY_simple */
	{"sbfshq", {{ 0xff01f000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSHQ_splat32_registerW_registerZ_w032_double */
	{"sbfshq.@", {{ 0xff01f800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSHQ_splat32_registerW_registerZ_w032_double */
	{"sbfswp", {{ 0x7f01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSWP_registerW_registerZ_registerY_simple */
	{"sbfswp", {{ 0xff01e000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSWP_splat32_registerW_registerZ_w032_double */
	{"sbfswp.@", {{ 0xff01e800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSWP_splat32_registerW_registerZ_w032_double */
	{"sbfsw", {{ 0x7f01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSW_registerW_registerZ_registerY_simple */
	{"sbfsw", {{ 0xff01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFSW_registerW_registerZ_w032_double */
	{"sbfuwd", {{ 0x7b014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFUWD_registerW_registerZ_registerY_simple */
	{"sbfuwd", {{ 0xfb014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFUWD_splat32_registerW_registerZ_w032_double */
	{"sbfuwd.@", {{ 0xfb014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFUWD_splat32_registerW_registerZ_w032_double */
	{"sbfwd", {{ 0x79014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFWD_registerW_registerZ_registerY_simple */
	{"sbfwd", {{ 0xf9014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFWD_splat32_registerW_registerZ_w032_double */
	{"sbfwd.@", {{ 0xf9014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFWD_splat32_registerW_registerZ_w032_double */
	{"sbfwp", {{ 0x75012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFWP_registerW_registerZ_registerY_simple */
	{"sbfwp", {{ 0xf5012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFWP_splat32_registerW_registerZ_w032_double */
	{"sbfwp.@", {{ 0xf5012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFWP_splat32_registerW_registerZ_w032_double */
	{"sbfw", {{ 0x75011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFW_registerW_registerZ_registerY_simple */
	{"sbfw", {{ 0x75000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFW_registerW_registerZ_s010_simple */
	{"sbfw", {{ 0xf5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFW_registerW_registerZ_s037_double */
	{"sbfx16d", {{ 0x77014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16D_registerW_registerZ_registerY_simple */
	{"sbfx16d", {{ 0xf7014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16D_splat32_registerW_registerZ_w032_double */
	{"sbfx16d.@", {{ 0xf7014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16D_splat32_registerW_registerZ_w032_double */
	{"sbfx16hq", {{ 0x77017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16HQ_registerW_registerZ_registerY_simple */
	{"sbfx16hq", {{ 0xf7017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx16hq.@", {{ 0xf7017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx16uwd", {{ 0x7f018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16UWD_registerW_registerZ_registerY_simple */
	{"sbfx16uwd", {{ 0xff018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16UWD_registerW_registerZ_w032_double */
	{"sbfx16wd", {{ 0x77018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16WD_registerW_registerZ_registerY_simple */
	{"sbfx16wd", {{ 0xf7018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16WD_registerW_registerZ_w032_double */
	{"sbfx16wp", {{ 0x77016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16WP_registerW_registerZ_registerY_simple */
	{"sbfx16wp", {{ 0xf7016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16WP_splat32_registerW_registerZ_w032_double */
	{"sbfx16wp.@", {{ 0xf7016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16WP_splat32_registerW_registerZ_w032_double */
	{"sbfx16w", {{ 0x77015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16W_registerW_registerZ_registerY_simple */
	{"sbfx16w", {{ 0xf7015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX16W_registerW_registerZ_w032_double */
	{"sbfx2d", {{ 0x71014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2D_registerW_registerZ_registerY_simple */
	{"sbfx2d", {{ 0xf1014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2D_splat32_registerW_registerZ_w032_double */
	{"sbfx2d.@", {{ 0xf1014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2D_splat32_registerW_registerZ_w032_double */
	{"sbfx2hq", {{ 0x71017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2HQ_registerW_registerZ_registerY_simple */
	{"sbfx2hq", {{ 0xf1017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx2hq.@", {{ 0xf1017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx2uwd", {{ 0x79018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2UWD_registerW_registerZ_registerY_simple */
	{"sbfx2uwd", {{ 0xf9018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2UWD_registerW_registerZ_w032_double */
	{"sbfx2wd", {{ 0x71018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2WD_registerW_registerZ_registerY_simple */
	{"sbfx2wd", {{ 0xf1018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2WD_registerW_registerZ_w032_double */
	{"sbfx2wp", {{ 0x71016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2WP_registerW_registerZ_registerY_simple */
	{"sbfx2wp", {{ 0xf1016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2WP_splat32_registerW_registerZ_w032_double */
	{"sbfx2wp.@", {{ 0xf1016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2WP_splat32_registerW_registerZ_w032_double */
	{"sbfx2w", {{ 0x71015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2W_registerW_registerZ_registerY_simple */
	{"sbfx2w", {{ 0xf1015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX2W_registerW_registerZ_w032_double */
	{"sbfx4d", {{ 0x73014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4D_registerW_registerZ_registerY_simple */
	{"sbfx4d", {{ 0xf3014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4D_splat32_registerW_registerZ_w032_double */
	{"sbfx4d.@", {{ 0xf3014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4D_splat32_registerW_registerZ_w032_double */
	{"sbfx4hq", {{ 0x73017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4HQ_registerW_registerZ_registerY_simple */
	{"sbfx4hq", {{ 0xf3017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx4hq.@", {{ 0xf3017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx4uwd", {{ 0x7b018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4UWD_registerW_registerZ_registerY_simple */
	{"sbfx4uwd", {{ 0xfb018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4UWD_registerW_registerZ_w032_double */
	{"sbfx4wd", {{ 0x73018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4WD_registerW_registerZ_registerY_simple */
	{"sbfx4wd", {{ 0xf3018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4WD_registerW_registerZ_w032_double */
	{"sbfx4wp", {{ 0x73016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4WP_registerW_registerZ_registerY_simple */
	{"sbfx4wp", {{ 0xf3016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4WP_splat32_registerW_registerZ_w032_double */
	{"sbfx4wp.@", {{ 0xf3016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4WP_splat32_registerW_registerZ_w032_double */
	{"sbfx4w", {{ 0x73015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4W_registerW_registerZ_registerY_simple */
	{"sbfx4w", {{ 0xf3015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX4W_registerW_registerZ_w032_double */
	{"sbfx8d", {{ 0x75014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8D_registerW_registerZ_registerY_simple */
	{"sbfx8d", {{ 0xf5014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8D_splat32_registerW_registerZ_w032_double */
	{"sbfx8d.@", {{ 0xf5014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8D_splat32_registerW_registerZ_w032_double */
	{"sbfx8hq", {{ 0x75017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8HQ_registerW_registerZ_registerY_simple */
	{"sbfx8hq", {{ 0xf5017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx8hq.@", {{ 0xf5017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx8uwd", {{ 0x7d018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8UWD_registerW_registerZ_registerY_simple */
	{"sbfx8uwd", {{ 0xfd018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8UWD_registerW_registerZ_w032_double */
	{"sbfx8wd", {{ 0x75018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8WD_registerW_registerZ_registerY_simple */
	{"sbfx8wd", {{ 0xf5018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8WD_registerW_registerZ_w032_double */
	{"sbfx8wp", {{ 0x75016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8WP_registerW_registerZ_registerY_simple */
	{"sbfx8wp", {{ 0xf5016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8WP_splat32_registerW_registerZ_w032_double */
	{"sbfx8wp.@", {{ 0xf5016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8WP_splat32_registerW_registerZ_w032_double */
	{"sbfx8w", {{ 0x75015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8W_registerW_registerZ_registerY_simple */
	{"sbfx8w", {{ 0xf5015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBFX8W_registerW_registerZ_w032_double */
	{"sbmm8", {{ 0x7e020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMM8_registerW_registerZ_registerY_simple */
	{"sbmm8", {{ 0x6e020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMM8_registerW_registerZ_s010_simple */
	{"sbmm8", {{ 0xee020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMM8_registerW_registerZ_s037_double */
	{"sbmm8.@", {{ 0xfe020800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMM8_registerW_registerZ_w032_double */
	{"sbmm8", {{ 0xee020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMM8_registerW_registerZ_w064_triple */
	{"sbmmt8", {{ 0x7f020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMMT8_registerW_registerZ_registerY_simple */
	{"sbmmt8", {{ 0x6f020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMMT8_registerW_registerZ_s010_simple */
	{"sbmmt8", {{ 0xef020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMMT8_registerW_registerZ_s037_double */
	{"sbmmt8.@", {{ 0xff020800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMMT8_registerW_registerZ_w032_double */
	{"sbmmt8", {{ 0xef020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SBMMT8_registerW_registerZ_w064_triple */
	{"sb", {{ 0x2403e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SB_doscale_registerY_registerZ_registerT_simple */
	{"sb.xs", {{ 0x2403f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SB_doscale_registerY_registerZ_registerT_simple */
	{"sb.deqz", {{ 0x24031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dgez", {{ 0x24033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dgtz", {{ 0x24035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dlez", {{ 0x24034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dltz", {{ 0x24032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dnez", {{ 0x24030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.even", {{ 0x24037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.odd", {{ 0x24036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.weqz", {{ 0x24039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wgez", {{ 0x2403b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wgtz", {{ 0x2403d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wlez", {{ 0x2403c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wltz", {{ 0x2403a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wnez", {{ 0x24038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.deqz", {{ 0xa4031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dgez", {{ 0xa4033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dgtz", {{ 0xa4035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dlez", {{ 0xa4034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dltz", {{ 0xa4032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dnez", {{ 0xa4030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.even", {{ 0xa4037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.odd", {{ 0xa4036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.weqz", {{ 0xa4039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wgez", {{ 0xa403b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wgtz", {{ 0xa403d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wlez", {{ 0xa403c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wltz", {{ 0xa403a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wnez", {{ 0xa4038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.deqz", {{ 0xa4031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dgez", {{ 0xa4033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dgtz", {{ 0xa4035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dlez", {{ 0xa4034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dltz", {{ 0xa4032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dnez", {{ 0xa4030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.even", {{ 0xa4037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.odd", {{ 0xa4036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.weqz", {{ 0xa4039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wgez", {{ 0xa403b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wgtz", {{ 0xa403d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wlez", {{ 0xa403c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wltz", {{ 0xa403a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wnez", {{ 0xa4038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb", {{ 0x24010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SB_s010_registerZ_registerT_simple */
	{"sb", {{ 0xa4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SB_s037_registerZ_registerT_double */
	{"sb", {{ 0xa4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SB_w064_registerZ_registerT_triple */
	{"scall", {{ 0x0fe40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v1-SCALL_registerZ_simple */
	{"scall", {{ 0x0fe00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_sysnumber_opnd, 0}, "", "%s"}, /* Opcode-v1-SCALL_u012_simple */
	{"sd", {{ 0x2703e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SD_doscale_registerY_registerZ_registerT_simple */
	{"sd.xs", {{ 0x2703f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SD_doscale_registerY_registerZ_registerT_simple */
	{"sd.deqz", {{ 0x27031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dgez", {{ 0x27033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dgtz", {{ 0x27035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dlez", {{ 0x27034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dltz", {{ 0x27032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dnez", {{ 0x27030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.even", {{ 0x27037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.odd", {{ 0x27036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.weqz", {{ 0x27039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wgez", {{ 0x2703b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wgtz", {{ 0x2703d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wlez", {{ 0x2703c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wltz", {{ 0x2703a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wnez", {{ 0x27038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.deqz", {{ 0xa7031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dgez", {{ 0xa7033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dgtz", {{ 0xa7035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dlez", {{ 0xa7034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dltz", {{ 0xa7032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dnez", {{ 0xa7030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.even", {{ 0xa7037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.odd", {{ 0xa7036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.weqz", {{ 0xa7039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wgez", {{ 0xa703b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wgtz", {{ 0xa703d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wlez", {{ 0xa703c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wltz", {{ 0xa703a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wnez", {{ 0xa7038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.deqz", {{ 0xa7031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dgez", {{ 0xa7033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dgtz", {{ 0xa7035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dlez", {{ 0xa7034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dltz", {{ 0xa7032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dnez", {{ 0xa7030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.even", {{ 0xa7037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.odd", {{ 0xa7036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.weqz", {{ 0xa7039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wgez", {{ 0xa703b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wgtz", {{ 0xa703d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wlez", {{ 0xa703c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wltz", {{ 0xa703a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wnez", {{ 0xa7038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd", {{ 0x27010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SD_s010_registerZ_registerT_simple */
	{"sd", {{ 0xa7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SD_s037_registerZ_registerT_double */
	{"sd", {{ 0xa7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SD_w064_registerZ_registerT_triple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_systemalone_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SET_systemAlone_registerZ_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemra_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SET_systemRA_registerZ_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemt3_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SET_systemT3_registerZ_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemt4_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SET_systemT4_registerZ_simple */
	{"sh", {{ 0x2503e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SH_doscale_registerY_registerZ_registerT_simple */
	{"sh.xs", {{ 0x2503f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SH_doscale_registerY_registerZ_registerT_simple */
	{"sh.deqz", {{ 0x25031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dgez", {{ 0x25033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dgtz", {{ 0x25035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dlez", {{ 0x25034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dltz", {{ 0x25032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dnez", {{ 0x25030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.even", {{ 0x25037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.odd", {{ 0x25036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.weqz", {{ 0x25039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wgez", {{ 0x2503b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wgtz", {{ 0x2503d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wlez", {{ 0x2503c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wltz", {{ 0x2503a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wnez", {{ 0x25038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.deqz", {{ 0xa5031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dgez", {{ 0xa5033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dgtz", {{ 0xa5035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dlez", {{ 0xa5034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dltz", {{ 0xa5032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dnez", {{ 0xa5030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.even", {{ 0xa5037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.odd", {{ 0xa5036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.weqz", {{ 0xa5039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wgez", {{ 0xa503b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wgtz", {{ 0xa503d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wlez", {{ 0xa503c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wltz", {{ 0xa503a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wnez", {{ 0xa5038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.deqz", {{ 0xa5031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dgez", {{ 0xa5033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dgtz", {{ 0xa5035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dlez", {{ 0xa5034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dltz", {{ 0xa5032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dnez", {{ 0xa5030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.even", {{ 0xa5037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.odd", {{ 0xa5036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.weqz", {{ 0xa5039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wgez", {{ 0xa503b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wgtz", {{ 0xa503d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wlez", {{ 0xa503c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wltz", {{ 0xa503a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wnez", {{ 0xa5038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh", {{ 0x25010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SH_s010_registerZ_registerT_simple */
	{"sh", {{ 0xa5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SH_s037_registerZ_registerT_double */
	{"sh", {{ 0xa5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SH_w064_registerZ_registerT_triple */
	{"sleep", {{ 0x0fa40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-SLEEP_simple */
	{"slld", {{ 0x79026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLD_registerW_registerZ_registerY_simple */
	{"slld", {{ 0x79022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLD_registerW_registerZ_u006_simple */
	{"sllhqs", {{ 0x79029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLHQS_registerW_registerZ_registerY_simple */
	{"sllhqs", {{ 0x79025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLHQS_registerW_registerZ_u006_simple */
	{"sllwps", {{ 0x79028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLWPS_registerW_registerZ_registerY_simple */
	{"sllwps", {{ 0x79024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLWPS_registerW_registerZ_u006_simple */
	{"sllw", {{ 0x79027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLW_registerW_registerZ_registerY_simple */
	{"sllw", {{ 0x79023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLLW_registerW_registerZ_u006_simple */
	{"slsd", {{ 0x7c026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSD_registerW_registerZ_registerY_simple */
	{"slsd", {{ 0x7c022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSD_registerW_registerZ_u006_simple */
	{"slshqs", {{ 0x7c029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSHQS_registerW_registerZ_registerY_simple */
	{"slshqs", {{ 0x7c025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSHQS_registerW_registerZ_u006_simple */
	{"slswps", {{ 0x7c028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSWPS_registerW_registerZ_registerY_simple */
	{"slswps", {{ 0x7c024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSWPS_registerW_registerZ_u006_simple */
	{"slsw", {{ 0x7c027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSW_registerW_registerZ_registerY_simple */
	{"slsw", {{ 0x7c023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SLSW_registerW_registerZ_u006_simple */
	{"so", {{ 0x2807e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SO_doscale_registerY_registerZ_registerV_simple */
	{"so.xs", {{ 0x2807f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SO_doscale_registerY_registerZ_registerV_simple */
	{"so.deqz", {{ 0x28071000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dgez", {{ 0x28073000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dgtz", {{ 0x28075000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dlez", {{ 0x28074000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dltz", {{ 0x28072000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dnez", {{ 0x28070000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.even", {{ 0x28077000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.odd", {{ 0x28076000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.weqz", {{ 0x28079000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wgez", {{ 0x2807b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wgtz", {{ 0x2807d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wlez", {{ 0x2807c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wltz", {{ 0x2807a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wnez", {{ 0x28078000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.deqz", {{ 0xa8071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dgez", {{ 0xa8073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dgtz", {{ 0xa8075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dlez", {{ 0xa8074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dltz", {{ 0xa8072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dnez", {{ 0xa8070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.even", {{ 0xa8077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.odd", {{ 0xa8076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.weqz", {{ 0xa8079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wgez", {{ 0xa807b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wgtz", {{ 0xa807d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wlez", {{ 0xa807c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wltz", {{ 0xa807a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wnez", {{ 0xa8078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.deqz", {{ 0xa8071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dgez", {{ 0xa8073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dgtz", {{ 0xa8075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dlez", {{ 0xa8074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dltz", {{ 0xa8072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dnez", {{ 0xa8070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.even", {{ 0xa8077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.odd", {{ 0xa8076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.weqz", {{ 0xa8079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wgez", {{ 0xa807b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wgtz", {{ 0xa807d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wlez", {{ 0xa807c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wltz", {{ 0xa807a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wnez", {{ 0xa8078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so", {{ 0x28050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SO_s010_registerZ_registerV_simple */
	{"so", {{ 0xa8050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SO_s037_registerZ_registerV_double */
	{"so", {{ 0xa8050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SO_w064_registerZ_registerV_triple */
	{"sq", {{ 0x2803e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SQ_doscale_registerY_registerZ_registerU_simple */
	{"sq.xs", {{ 0x2803f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SQ_doscale_registerY_registerZ_registerU_simple */
	{"sq.deqz", {{ 0x28031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dgez", {{ 0x28033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dgtz", {{ 0x28035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dlez", {{ 0x28034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dltz", {{ 0x28032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dnez", {{ 0x28030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.even", {{ 0x28037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.odd", {{ 0x28036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.weqz", {{ 0x28039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wgez", {{ 0x2803b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wgtz", {{ 0x2803d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wlez", {{ 0x2803c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wltz", {{ 0x2803a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wnez", {{ 0x28038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.deqz", {{ 0xa8031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dgez", {{ 0xa8033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dgtz", {{ 0xa8035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dlez", {{ 0xa8034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dltz", {{ 0xa8032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dnez", {{ 0xa8030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.even", {{ 0xa8037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.odd", {{ 0xa8036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.weqz", {{ 0xa8039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wgez", {{ 0xa803b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wgtz", {{ 0xa803d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wlez", {{ 0xa803c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wltz", {{ 0xa803a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wnez", {{ 0xa8038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.deqz", {{ 0xa8031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dgez", {{ 0xa8033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dgtz", {{ 0xa8035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dlez", {{ 0xa8034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dltz", {{ 0xa8032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dnez", {{ 0xa8030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.even", {{ 0xa8037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.odd", {{ 0xa8036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.weqz", {{ 0xa8039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wgez", {{ 0xa803b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wgtz", {{ 0xa803d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wlez", {{ 0xa803c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wltz", {{ 0xa803a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wnez", {{ 0xa8038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq", {{ 0x28010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SQ_s010_registerZ_registerU_simple */
	{"sq", {{ 0xa8010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SQ_s037_registerZ_registerU_double */
	{"sq", {{ 0xa8010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SQ_w064_registerZ_registerU_triple */
	{"srad", {{ 0x7a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAD_registerW_registerZ_registerY_simple */
	{"srad", {{ 0x7a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAD_registerW_registerZ_u006_simple */
	{"srahqs", {{ 0x7a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAHQS_registerW_registerZ_registerY_simple */
	{"srahqs", {{ 0x7a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAHQS_registerW_registerZ_u006_simple */
	{"srawps", {{ 0x7a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAWPS_registerW_registerZ_registerY_simple */
	{"srawps", {{ 0x7a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAWPS_registerW_registerZ_u006_simple */
	{"sraw", {{ 0x7a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAW_registerW_registerZ_registerY_simple */
	{"sraw", {{ 0x7a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRAW_registerW_registerZ_u006_simple */
	{"srld", {{ 0x7b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLD_registerW_registerZ_registerY_simple */
	{"srld", {{ 0x7b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLD_registerW_registerZ_u006_simple */
	{"srlhqs", {{ 0x7b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLHQS_registerW_registerZ_registerY_simple */
	{"srlhqs", {{ 0x7b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLHQS_registerW_registerZ_u006_simple */
	{"srlwps", {{ 0x7b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLWPS_registerW_registerZ_registerY_simple */
	{"srlwps", {{ 0x7b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLWPS_registerW_registerZ_u006_simple */
	{"srlw", {{ 0x7b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLW_registerW_registerZ_registerY_simple */
	{"srlw", {{ 0x7b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRLW_registerW_registerZ_u006_simple */
	{"srsd", {{ 0x78026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSD_registerW_registerZ_registerY_simple */
	{"srsd", {{ 0x78022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSD_registerW_registerZ_u006_simple */
	{"srshqs", {{ 0x78029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSHQS_registerW_registerZ_registerY_simple */
	{"srshqs", {{ 0x78025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSHQS_registerW_registerZ_u006_simple */
	{"srswps", {{ 0x78028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSWPS_registerW_registerZ_registerY_simple */
	{"srswps", {{ 0x78024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSWPS_registerW_registerZ_u006_simple */
	{"srsw", {{ 0x78027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSW_registerW_registerZ_registerY_simple */
	{"srsw", {{ 0x78023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-SRSW_registerW_registerZ_u006_simple */
	{"stop", {{ 0x0fa80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-STOP_simple */
	{"stsud", {{ 0x70010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-STSUD_registerW_registerZ_registerY_simple */
	{"stsuw", {{ 0x70011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-STSUW_registerW_registerZ_registerY_simple */
	{"sv", {{ 0x2903e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SV_doscale_registerY_registerZ_registerE_simple */
	{"sv.xs", {{ 0x2903f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SV_doscale_registerY_registerZ_registerE_simple */
	{"sv.deqz", {{ 0x29031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dgez", {{ 0x29033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dgtz", {{ 0x29035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dlez", {{ 0x29034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dltz", {{ 0x29032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dnez", {{ 0x29030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.even", {{ 0x29037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.odd", {{ 0x29036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.weqz", {{ 0x29039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wgez", {{ 0x2903b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wgtz", {{ 0x2903d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wlez", {{ 0x2903c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wltz", {{ 0x2903a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wnez", {{ 0x29038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.deqz", {{ 0xa9031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dgez", {{ 0xa9033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dgtz", {{ 0xa9035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dlez", {{ 0xa9034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dltz", {{ 0xa9032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dnez", {{ 0xa9030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.even", {{ 0xa9037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.odd", {{ 0xa9036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.weqz", {{ 0xa9039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wgez", {{ 0xa903b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wgtz", {{ 0xa903d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wlez", {{ 0xa903c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wltz", {{ 0xa903a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wnez", {{ 0xa9038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.deqz", {{ 0xa9031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dgez", {{ 0xa9033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dgtz", {{ 0xa9035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dlez", {{ 0xa9034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dltz", {{ 0xa9032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dnez", {{ 0xa9030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.even", {{ 0xa9037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.odd", {{ 0xa9036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.weqz", {{ 0xa9039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wgez", {{ 0xa903b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wgtz", {{ 0xa903d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wlez", {{ 0xa903c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wltz", {{ 0xa903a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wnez", {{ 0xa9038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv", {{ 0x29010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SV_s010_registerZ_registerE_simple */
	{"sv", {{ 0xa9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SV_s037_registerZ_registerE_double */
	{"sv", {{ 0xa9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SV_w064_registerZ_registerE_triple */
	{"sw", {{ 0x2603e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SW_doscale_registerY_registerZ_registerT_simple */
	{"sw.xs", {{ 0x2603f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SW_doscale_registerY_registerZ_registerT_simple */
	{"sw.deqz", {{ 0x26031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dgez", {{ 0x26033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dgtz", {{ 0x26035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dlez", {{ 0x26034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dltz", {{ 0x26032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dnez", {{ 0x26030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.even", {{ 0x26037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.odd", {{ 0x26036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.weqz", {{ 0x26039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wgez", {{ 0x2603b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wgtz", {{ 0x2603d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wlez", {{ 0x2603c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wltz", {{ 0x2603a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wnez", {{ 0x26038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.deqz", {{ 0xa6031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dgez", {{ 0xa6033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dgtz", {{ 0xa6035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dlez", {{ 0xa6034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dltz", {{ 0xa6032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dnez", {{ 0xa6030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.even", {{ 0xa6037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.odd", {{ 0xa6036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.weqz", {{ 0xa6039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wgez", {{ 0xa603b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wgtz", {{ 0xa603d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wlez", {{ 0xa603c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wltz", {{ 0xa603a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wnez", {{ 0xa6038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.deqz", {{ 0xa6031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dgez", {{ 0xa6033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dgtz", {{ 0xa6035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dlez", {{ 0xa6034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dltz", {{ 0xa6032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dnez", {{ 0xa6030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.even", {{ 0xa6037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.odd", {{ 0xa6036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.weqz", {{ 0xa6039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wgez", {{ 0xa603b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wgtz", {{ 0xa603d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wlez", {{ 0xa603c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wltz", {{ 0xa603a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wnez", {{ 0xa6038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v1-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw", {{ 0x26010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SW_s010_registerZ_registerT_simple */
	{"sw", {{ 0xa6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SW_s037_registerZ_registerT_double */
	{"sw", {{ 0xa6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v1-SW_w064_registerZ_registerT_triple */
	{"sxlbhq", {{ 0x76025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SXLBHQ_registerW_registerZ_simple */
	{"sxlhwp", {{ 0x76024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SXLHWP_registerW_registerZ_simple */
	{"sxmbhq", {{ 0x77025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SXMBHQ_registerW_registerZ_simple */
	{"sxmhwp", {{ 0x77024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v1-SXMHWP_registerW_registerZ_simple */
	{"syncgroup", {{ 0x0fb40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v1-SYNCGROUP_registerZ_simple */
	{"tlbdinval", {{ 0x0f8c0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-TLBDINVAL_simple */
	{"tlbiinval", {{ 0x0f900000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-TLBIINVAL_simple */
	{"tlbprobe", {{ 0x0f840000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-TLBPROBE_simple */
	{"tlbread", {{ 0x0f800000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-TLBREAD_simple */
	{"tlbwrite", {{ 0x0f880000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v1-TLBWRITE_simple */
	{"waitit", {{ 0x0fb00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v1-WAITIT_registerZ_simple */
	{"wfxl", {{ 0x0fb80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_systemalone_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-WFXL_systemAlone_registerZ_simple */
	{"wfxl", {{ 0x0fb80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemt2_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-WFXL_systemT2_registerZ_simple */
	{"wfxl", {{ 0x0fb80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemt4_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-WFXL_systemT4_registerZ_simple */
	{"wfxm", {{ 0x0fbc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_systemalone_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-WFXM_systemAlone_registerZ_simple */
	{"wfxm", {{ 0x0fbc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemt2_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-WFXM_systemT2_registerZ_simple */
	{"wfxm", {{ 0x0fbc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemt4_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v1-WFXM_systemT4_registerZ_simple */
	{"xord", {{ 0x7c010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORD_registerW_registerZ_registerY_simple */
	{"xord", {{ 0x6c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORD_registerW_registerZ_s010_simple */
	{"xord", {{ 0xec000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORD_registerW_registerZ_s037_double */
	{"xord.@", {{ 0xfc010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORD_registerW_registerZ_w032_double */
	{"xord", {{ 0xec000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORD_registerW_registerZ_w064_triple */
	{"xorw", {{ 0x7c011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORW_registerW_registerZ_registerY_simple */
	{"xorw", {{ 0x7c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORW_registerW_registerZ_s010_simple */
	{"xorw", {{ 0xfc000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v1-XORW_registerW_registerZ_s037_double */
/* Number of instructions : 5401 */
	{"", { }, 0, 0, 0, 0, { }, "", ""}};

const Kvx_Core_Info kvx_kv3_v1_core_info = { kv3_v1_optab, { "kv3-1", "\0" }, kv3_v1_resources, true, { ELF_KVX_CORE_KV3_1, -1 }  };

kv3opc_t kv3_v2_optab[] = {
	{"absd", {{ 0x64000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ABSD_registerW_registerZ_simple */
	{"abshq", {{ 0xf4013000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ABSHQ_registerW_registerZ_double */
	{"abswp", {{ 0xf4012000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ABSWP_registerW_registerZ_double */
	{"absw", {{ 0x74000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ABSW_registerW_registerZ_simple */
	{"copyd", {{ 0x6a000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-COPYD_registerW_registerZ_simple */
	{"copyv", {{ 0x01000f80, 0x7f001fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbe_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-COPYV_registerA_registerBe_simple */
	{"copyv", {{ 0x01001f80, 0x7f001fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbo_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-COPYV_registerA_registerBo_simple */
	{"copyw", {{ 0x7a000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-COPYW_registerW_registerZ_simple */
	{"movefo", {{ 0x00081f80, 0x7f0c1fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbe_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-MOVEFO_registerN_registerBe_simple */
	{"movefo", {{ 0x000c1f80, 0x7f0c1fff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbo_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-MOVEFO_registerN_registerBo_simple */
	{"negd", {{ 0x65000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-NEGD_registerW_registerZ_simple */
	{"neghq", {{ 0xf5013000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-NEGHQ_registerW_registerZ_double */
	{"negwp", {{ 0xf5012000, 0xff03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x67ffffff, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-NEGWP_registerW_registerZ_double */
	{"negw", {{ 0x75000000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-NEGW_registerW_registerZ_simple */
	{"notd", {{ 0x6c00ffc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-NOTD_registerW_registerZ_simple */
	{"notw", {{ 0x7c00ffc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-NOTW_registerW_registerZ_simple */
	{"satdh", {{ 0x7e022400, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SATDH_registerW_registerZ_simple */
	{"satdw", {{ 0x7e022800, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SATDW_registerW_registerZ_simple */
	{"satudh", {{ 0x7f022400, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SATUDH_registerW_registerZ_simple */
	{"satudw", {{ 0x7f022800, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SATUDW_registerW_registerZ_simple */
	{"sxbd", {{ 0x68037000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SXBD_registerW_registerZ_simple */
	{"sxhd", {{ 0x6803f000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SXHD_registerW_registerZ_simple */
	{"sxwd", {{ 0x6903f000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-SXWD_registerW_registerZ_simple */
	{"zxbd", {{ 0x78003fc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ZXBD_registerW_registerZ_simple */
	{"zxhd", {{ 0x6403f000, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ZXHD_registerW_registerZ_simple */
	{"zxwd", {{ 0x7800ffc0, 0x7f03ffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* A_Opcode-v2-ZXWD_registerW_registerZ_simple */
	{"abdd", {{ 0x74010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDD_registerW_registerZ_registerY_simple */
	{"abdd", {{ 0x64000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDD_registerW_registerZ_s010_simple */
	{"abdd", {{ 0xe4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDD_registerW_registerZ_s037_double */
	{"abdd", {{ 0xe4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDD_registerW_registerZ_w064_triple */
	{"abdhq", {{ 0x74013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDHQ_registerW_registerZ_registerY_simple */
	{"abdhq", {{ 0xf4013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDHQ_splat32_registerW_registerZ_w032_double */
	{"abdhq.@", {{ 0xf4013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDHQ_splat32_registerW_registerZ_w032_double */
	{"abdwp", {{ 0x74012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDWP_registerW_registerZ_registerY_simple */
	{"abdwp", {{ 0xf4012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDWP_splat32_registerW_registerZ_w032_double */
	{"abdwp.@", {{ 0xf4012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDWP_splat32_registerW_registerZ_w032_double */
	{"abdw", {{ 0x74011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDW_registerW_registerZ_registerY_simple */
	{"abdw", {{ 0x74000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDW_registerW_registerZ_s010_simple */
	{"abdw", {{ 0xf4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ABDW_registerW_registerZ_s037_double */
	{"acswapd.v", {{ 0x3d03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapd.v.xs", {{ 0x3d03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapd", {{ 0x3f03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapd.xs", {{ 0x3f03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapd.v.deqz", {{ 0x3d031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.dgez", {{ 0x3d033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.dgtz", {{ 0x3d035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.dlez", {{ 0x3d034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.dltz", {{ 0x3d032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.dnez", {{ 0x3d030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.even", {{ 0x3d037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.odd", {{ 0x3d036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.weqz", {{ 0x3d039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.wgez", {{ 0x3d03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.wgtz", {{ 0x3d03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.wlez", {{ 0x3d03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.wltz", {{ 0x3d03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.wnez", {{ 0x3d038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.deqz", {{ 0x3f031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dgez", {{ 0x3f033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dgtz", {{ 0x3f035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dlez", {{ 0x3f034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dltz", {{ 0x3f032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.dnez", {{ 0x3f030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.even", {{ 0x3f037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.odd", {{ 0x3f036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.weqz", {{ 0x3f039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wgez", {{ 0x3f03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wgtz", {{ 0x3f03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wlez", {{ 0x3f03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wltz", {{ 0x3f03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.wnez", {{ 0x3f038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapd.v.deqz", {{ 0xbd031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.dgez", {{ 0xbd033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.dgtz", {{ 0xbd035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.dlez", {{ 0xbd034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.dltz", {{ 0xbd032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.dnez", {{ 0xbd030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.even", {{ 0xbd037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.odd", {{ 0xbd036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.weqz", {{ 0xbd039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.wgez", {{ 0xbd03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.wgtz", {{ 0xbd03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.wlez", {{ 0xbd03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.wltz", {{ 0xbd03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.wnez", {{ 0xbd038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.deqz", {{ 0xbf031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dgez", {{ 0xbf033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dgtz", {{ 0xbf035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dlez", {{ 0xbf034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dltz", {{ 0xbf032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.dnez", {{ 0xbf030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.even", {{ 0xbf037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.odd", {{ 0xbf036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.weqz", {{ 0xbf039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wgez", {{ 0xbf03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wgtz", {{ 0xbf03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wlez", {{ 0xbf03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wltz", {{ 0xbf03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.wnez", {{ 0xbf038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapd.v.deqz", {{ 0xbd031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.dgez", {{ 0xbd033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.dgtz", {{ 0xbd035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.dlez", {{ 0xbd034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.dltz", {{ 0xbd032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.dnez", {{ 0xbd030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.even", {{ 0xbd037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.odd", {{ 0xbd036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.weqz", {{ 0xbd039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.wgez", {{ 0xbd03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.wgtz", {{ 0xbd03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.wlez", {{ 0xbd03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.wltz", {{ 0xbd03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.v.wnez", {{ 0xbd038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.deqz", {{ 0xbf031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dgez", {{ 0xbf033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dgtz", {{ 0xbf035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dlez", {{ 0xbf034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dltz", {{ 0xbf032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.dnez", {{ 0xbf030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.even", {{ 0xbf037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.odd", {{ 0xbf036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.weqz", {{ 0xbf039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wgez", {{ 0xbf03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wgtz", {{ 0xbf03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wlez", {{ 0xbf03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wltz", {{ 0xbf03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd.wnez", {{ 0xbf038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapd", {{ 0x3f010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_s010_registerZ_registerU_simple */
	{"acswapd.v", {{ 0x3d010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_s010_registerZ_registerU_simple */
	{"acswapd", {{ 0xbf010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_s037_registerZ_registerU_double */
	{"acswapd.v", {{ 0xbd010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPD_boolcas_s037_registerZ_registerU_double */
	{"acswapw.v", {{ 0x3c03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapw.v.xs", {{ 0x3c03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapw", {{ 0x3e03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapw.xs", {{ 0x3e03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_doscale_registerY_registerZ_registerU_simple */
	{"acswapw.v.deqz", {{ 0x3c031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.dgez", {{ 0x3c033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.dgtz", {{ 0x3c035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.dlez", {{ 0x3c034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.dltz", {{ 0x3c032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.dnez", {{ 0x3c030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.even", {{ 0x3c037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.odd", {{ 0x3c036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.weqz", {{ 0x3c039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.wgez", {{ 0x3c03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.wgtz", {{ 0x3c03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.wlez", {{ 0x3c03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.wltz", {{ 0x3c03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.wnez", {{ 0x3c038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.deqz", {{ 0x3e031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dgez", {{ 0x3e033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dgtz", {{ 0x3e035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dlez", {{ 0x3e034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dltz", {{ 0x3e032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.dnez", {{ 0x3e030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.even", {{ 0x3e037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.odd", {{ 0x3e036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.weqz", {{ 0x3e039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wgez", {{ 0x3e03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wgtz", {{ 0x3e03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wlez", {{ 0x3e03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wltz", {{ 0x3e03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.wnez", {{ 0x3e038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_registerZ_registerU_simple */
	{"acswapw.v.deqz", {{ 0xbc031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.dgez", {{ 0xbc033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.dgtz", {{ 0xbc035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.dlez", {{ 0xbc034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.dltz", {{ 0xbc032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.dnez", {{ 0xbc030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.even", {{ 0xbc037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.odd", {{ 0xbc036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.weqz", {{ 0xbc039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.wgez", {{ 0xbc03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.wgtz", {{ 0xbc03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.wlez", {{ 0xbc03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.wltz", {{ 0xbc03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.wnez", {{ 0xbc038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.deqz", {{ 0xbe031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dgez", {{ 0xbe033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dgtz", {{ 0xbe035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dlez", {{ 0xbe034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dltz", {{ 0xbe032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.dnez", {{ 0xbe030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.even", {{ 0xbe037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.odd", {{ 0xbe036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.weqz", {{ 0xbe039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wgez", {{ 0xbe03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wgtz", {{ 0xbe03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wlez", {{ 0xbe03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wltz", {{ 0xbe03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.wnez", {{ 0xbe038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s027_registerZ_registerU_double */
	{"acswapw.v.deqz", {{ 0xbc031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.dgez", {{ 0xbc033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.dgtz", {{ 0xbc035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.dlez", {{ 0xbc034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.dltz", {{ 0xbc032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.dnez", {{ 0xbc030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.even", {{ 0xbc037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.odd", {{ 0xbc036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.weqz", {{ 0xbc039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.wgez", {{ 0xbc03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.wgtz", {{ 0xbc03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.wlez", {{ 0xbc03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.wltz", {{ 0xbc03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.v.wnez", {{ 0xbc038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.deqz", {{ 0xbe031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dgez", {{ 0xbe033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dgtz", {{ 0xbe035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dlez", {{ 0xbe034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dltz", {{ 0xbe032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.dnez", {{ 0xbe030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.even", {{ 0xbe037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.odd", {{ 0xbe036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.weqz", {{ 0xbe039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wgez", {{ 0xbe03b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wgtz", {{ 0xbe03d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wlez", {{ 0xbe03c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wltz", {{ 0xbe03a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw.wnez", {{ 0xbe038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_lsucond_registerY_s054_registerZ_registerU_triple */
	{"acswapw", {{ 0x3e010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_s010_registerZ_registerU_simple */
	{"acswapw.v", {{ 0x3c010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_s010_registerZ_registerU_simple */
	{"acswapw", {{ 0xbe010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_s037_registerZ_registerU_double */
	{"acswapw.v", {{ 0xbc010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ACSWAPW_boolcas_s037_registerZ_registerU_double */
	{"addcd.i", {{ 0x7e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD.I_registerW_registerZ_registerY_simple */
	{"addcid", {{ 0x7e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD.I_registerW_registerZ_registerY_simple */
	{"addd.ci", {{ 0x7e01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD.I_registerW_registerZ_registerY_simple */
	{"addcd.i", {{ 0xfe01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD.I_registerW_registerZ_w032_double */
	{"addcid", {{ 0xfe01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD.I_registerW_registerZ_w032_double */
	{"addd.ci", {{ 0xfe01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD.I_registerW_registerZ_w032_double */
	{"addcd", {{ 0x7e01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD_registerW_registerZ_registerY_simple */
	{"addd.c", {{ 0x7e01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD_registerW_registerZ_registerY_simple */
	{"addcd", {{ 0xfe01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD_registerW_registerZ_w032_double */
	{"addd.c", {{ 0xfe01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCD_registerW_registerZ_w032_double */
	{"addchcp", {{ 0x7c013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCHCP_registerW_registerZ_registerY_simple */
	{"addchcp", {{ 0xfc013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCHCP_splat32_registerW_registerZ_w032_double */
	{"addchcp.@", {{ 0xfc013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCHCP_splat32_registerW_registerZ_w032_double */
	{"addcwc", {{ 0x7c012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCWC_registerW_registerZ_registerY_simple */
	{"addcwc", {{ 0xfc012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCWC_splat32_registerW_registerZ_w032_double */
	{"addcwc.@", {{ 0xfc012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDCWC_splat32_registerW_registerZ_w032_double */
	{"addd", {{ 0x71010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDD_registerW_registerZ_registerY_simple */
	{"addd", {{ 0x61000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDD_registerW_registerZ_s010_simple */
	{"addd", {{ 0xe1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDD_registerW_registerZ_s037_double */
	{"addd", {{ 0xe1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDD_registerW_registerZ_w064_triple */
	{"addhq", {{ 0x71013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDHQ_registerW_registerZ_registerY_simple */
	{"addhq", {{ 0xf1013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDHQ_splat32_registerW_registerZ_w032_double */
	{"addhq.@", {{ 0xf1013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDHQ_splat32_registerW_registerZ_w032_double */
	{"addsd", {{ 0x7e01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSD_registerW_registerZ_registerY_simple */
	{"addsd", {{ 0x6e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSD_registerW_registerZ_s010_simple */
	{"addsd", {{ 0xee010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSD_registerW_registerZ_s037_double */
	{"addsd", {{ 0xee010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSD_registerW_registerZ_w064_triple */
	{"addshq", {{ 0x7e01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSHQ_registerW_registerZ_registerY_simple */
	{"addshq", {{ 0xfe01f000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSHQ_splat32_registerW_registerZ_w032_double */
	{"addshq.@", {{ 0xfe01f800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSHQ_splat32_registerW_registerZ_w032_double */
	{"addswp", {{ 0x7e01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSWP_registerW_registerZ_registerY_simple */
	{"addswp", {{ 0xfe01e000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSWP_splat32_registerW_registerZ_w032_double */
	{"addswp.@", {{ 0xfe01e800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSWP_splat32_registerW_registerZ_w032_double */
	{"addsw", {{ 0x7e01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSW_registerW_registerZ_registerY_simple */
	{"addsw", {{ 0xfe01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDSW_registerW_registerZ_w032_double */
	{"adduwd", {{ 0x7a014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDUWD_registerW_registerZ_registerY_simple */
	{"adduwd", {{ 0xfa014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDUWD_splat32_registerW_registerZ_w032_double */
	{"adduwd.@", {{ 0xfa014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDUWD_splat32_registerW_registerZ_w032_double */
	{"addwd", {{ 0x78014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDWD_registerW_registerZ_registerY_simple */
	{"addwd", {{ 0xf8014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDWD_splat32_registerW_registerZ_w032_double */
	{"addwd.@", {{ 0xf8014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDWD_splat32_registerW_registerZ_w032_double */
	{"addwp", {{ 0x71012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDWP_registerW_registerZ_registerY_simple */
	{"addwp", {{ 0xf1012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDWP_splat32_registerW_registerZ_w032_double */
	{"addwp.@", {{ 0xf1012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDWP_splat32_registerW_registerZ_w032_double */
	{"addw", {{ 0x71011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDW_registerW_registerZ_registerY_simple */
	{"addw", {{ 0x71000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDW_registerW_registerZ_s010_simple */
	{"addw", {{ 0xf1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDW_registerW_registerZ_s037_double */
	{"addx16d", {{ 0x76014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16D_registerW_registerZ_registerY_simple */
	{"addx16d", {{ 0xf6014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16D_splat32_registerW_registerZ_w032_double */
	{"addx16d.@", {{ 0xf6014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16D_splat32_registerW_registerZ_w032_double */
	{"addx16hq", {{ 0x76017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16HQ_registerW_registerZ_registerY_simple */
	{"addx16hq", {{ 0xf6017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16HQ_splat32_registerW_registerZ_w032_double */
	{"addx16hq.@", {{ 0xf6017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16HQ_splat32_registerW_registerZ_w032_double */
	{"addx16uwd", {{ 0x7e018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16UWD_registerW_registerZ_registerY_simple */
	{"addx16uwd", {{ 0xfe018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16UWD_registerW_registerZ_w032_double */
	{"addx16wd", {{ 0x76018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16WD_registerW_registerZ_registerY_simple */
	{"addx16wd", {{ 0xf6018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16WD_registerW_registerZ_w032_double */
	{"addx16wp", {{ 0x76016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16WP_registerW_registerZ_registerY_simple */
	{"addx16wp", {{ 0xf6016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16WP_splat32_registerW_registerZ_w032_double */
	{"addx16wp.@", {{ 0xf6016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16WP_splat32_registerW_registerZ_w032_double */
	{"addx16w", {{ 0x76015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16W_registerW_registerZ_registerY_simple */
	{"addx16w", {{ 0xf6015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX16W_registerW_registerZ_w032_double */
	{"addx2d", {{ 0x70014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2D_registerW_registerZ_registerY_simple */
	{"addx2d", {{ 0xf0014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2D_splat32_registerW_registerZ_w032_double */
	{"addx2d.@", {{ 0xf0014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2D_splat32_registerW_registerZ_w032_double */
	{"addx2hq", {{ 0x70017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2HQ_registerW_registerZ_registerY_simple */
	{"addx2hq", {{ 0xf0017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2HQ_splat32_registerW_registerZ_w032_double */
	{"addx2hq.@", {{ 0xf0017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2HQ_splat32_registerW_registerZ_w032_double */
	{"addx2uwd", {{ 0x78018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2UWD_registerW_registerZ_registerY_simple */
	{"addx2uwd", {{ 0xf8018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2UWD_registerW_registerZ_w032_double */
	{"addx2wd", {{ 0x70018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2WD_registerW_registerZ_registerY_simple */
	{"addx2wd", {{ 0xf0018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2WD_registerW_registerZ_w032_double */
	{"addx2wp", {{ 0x70016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2WP_registerW_registerZ_registerY_simple */
	{"addx2wp", {{ 0xf0016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2WP_splat32_registerW_registerZ_w032_double */
	{"addx2wp.@", {{ 0xf0016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2WP_splat32_registerW_registerZ_w032_double */
	{"addx2w", {{ 0x70015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2W_registerW_registerZ_registerY_simple */
	{"addx2w", {{ 0xf0015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX2W_registerW_registerZ_w032_double */
	{"addx4d", {{ 0x72014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4D_registerW_registerZ_registerY_simple */
	{"addx4d", {{ 0xf2014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4D_splat32_registerW_registerZ_w032_double */
	{"addx4d.@", {{ 0xf2014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4D_splat32_registerW_registerZ_w032_double */
	{"addx4hq", {{ 0x72017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4HQ_registerW_registerZ_registerY_simple */
	{"addx4hq", {{ 0xf2017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4HQ_splat32_registerW_registerZ_w032_double */
	{"addx4hq.@", {{ 0xf2017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4HQ_splat32_registerW_registerZ_w032_double */
	{"addx4uwd", {{ 0x7a018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4UWD_registerW_registerZ_registerY_simple */
	{"addx4uwd", {{ 0xfa018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4UWD_registerW_registerZ_w032_double */
	{"addx4wd", {{ 0x72018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4WD_registerW_registerZ_registerY_simple */
	{"addx4wd", {{ 0xf2018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4WD_registerW_registerZ_w032_double */
	{"addx4wp", {{ 0x72016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4WP_registerW_registerZ_registerY_simple */
	{"addx4wp", {{ 0xf2016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4WP_splat32_registerW_registerZ_w032_double */
	{"addx4wp.@", {{ 0xf2016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4WP_splat32_registerW_registerZ_w032_double */
	{"addx4w", {{ 0x72015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4W_registerW_registerZ_registerY_simple */
	{"addx4w", {{ 0xf2015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX4W_registerW_registerZ_w032_double */
	{"addx8d", {{ 0x74014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8D_registerW_registerZ_registerY_simple */
	{"addx8d", {{ 0xf4014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8D_splat32_registerW_registerZ_w032_double */
	{"addx8d.@", {{ 0xf4014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8D_splat32_registerW_registerZ_w032_double */
	{"addx8hq", {{ 0x74017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8HQ_registerW_registerZ_registerY_simple */
	{"addx8hq", {{ 0xf4017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8HQ_splat32_registerW_registerZ_w032_double */
	{"addx8hq.@", {{ 0xf4017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8HQ_splat32_registerW_registerZ_w032_double */
	{"addx8uwd", {{ 0x7c018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8UWD_registerW_registerZ_registerY_simple */
	{"addx8uwd", {{ 0xfc018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8UWD_registerW_registerZ_w032_double */
	{"addx8wd", {{ 0x74018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8WD_registerW_registerZ_registerY_simple */
	{"addx8wd", {{ 0xf4018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8WD_registerW_registerZ_w032_double */
	{"addx8wp", {{ 0x74016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8WP_registerW_registerZ_registerY_simple */
	{"addx8wp", {{ 0xf4016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8WP_splat32_registerW_registerZ_w032_double */
	{"addx8wp.@", {{ 0xf4016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8WP_splat32_registerW_registerZ_w032_double */
	{"addx8w", {{ 0x74015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8W_registerW_registerZ_registerY_simple */
	{"addx8w", {{ 0xf4015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ADDX8W_registerW_registerZ_w032_double */
	{"aladdd", {{ 0x2f03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"afaddd", {{ 0x2f03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"aladdd.xs", {{ 0x2f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"afaddd.xs", {{ 0x2f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_doscale_registerY_registerZ_registerT_simple */
	{"aladdd.deqz", {{ 0x2f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.deqz", {{ 0x2f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dgez", {{ 0x2f033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dgez", {{ 0x2f033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dgtz", {{ 0x2f035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dgtz", {{ 0x2f035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dlez", {{ 0x2f034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dlez", {{ 0x2f034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dltz", {{ 0x2f032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dltz", {{ 0x2f032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.dnez", {{ 0x2f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.dnez", {{ 0x2f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.even", {{ 0x2f037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.even", {{ 0x2f037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.odd", {{ 0x2f036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.odd", {{ 0x2f036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.weqz", {{ 0x2f039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.weqz", {{ 0x2f039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wgez", {{ 0x2f03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wgez", {{ 0x2f03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wgtz", {{ 0x2f03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wgtz", {{ 0x2f03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wlez", {{ 0x2f03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wlez", {{ 0x2f03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wltz", {{ 0x2f03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wltz", {{ 0x2f03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.wnez", {{ 0x2f038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"afaddd.wnez", {{ 0x2f038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_registerZ_registerT_simple */
	{"aladdd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.deqz", {{ 0xaf031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dgez", {{ 0xaf033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dgtz", {{ 0xaf035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dlez", {{ 0xaf034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dltz", {{ 0xaf032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.dnez", {{ 0xaf030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.even", {{ 0xaf037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.odd", {{ 0xaf036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.weqz", {{ 0xaf039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wgez", {{ 0xaf03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wgtz", {{ 0xaf03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wlez", {{ 0xaf03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wltz", {{ 0xaf03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddd.wnez", {{ 0xaf038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdd", {{ 0x2f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_s010_registerZ_registerT_simple */
	{"afaddd", {{ 0x2f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_s010_registerZ_registerT_simple */
	{"aladdd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_s037_registerZ_registerT_double */
	{"afaddd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_s037_registerZ_registerT_double */
	{"aladdd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_w064_registerZ_registerT_triple */
	{"afaddd", {{ 0xaf010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDD_w064_registerZ_registerT_triple */
	{"aladdw", {{ 0x2e03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"afaddw", {{ 0x2e03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"aladdw.xs", {{ 0x2e03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"afaddw.xs", {{ 0x2e03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_doscale_registerY_registerZ_registerT_simple */
	{"aladdw.deqz", {{ 0x2e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.deqz", {{ 0x2e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dgez", {{ 0x2e033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dgez", {{ 0x2e033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dgtz", {{ 0x2e035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dgtz", {{ 0x2e035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dlez", {{ 0x2e034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dlez", {{ 0x2e034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dltz", {{ 0x2e032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dltz", {{ 0x2e032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.dnez", {{ 0x2e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.dnez", {{ 0x2e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.even", {{ 0x2e037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.even", {{ 0x2e037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.odd", {{ 0x2e036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.odd", {{ 0x2e036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.weqz", {{ 0x2e039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.weqz", {{ 0x2e039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wgez", {{ 0x2e03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wgez", {{ 0x2e03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wgtz", {{ 0x2e03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wgtz", {{ 0x2e03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wlez", {{ 0x2e03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wlez", {{ 0x2e03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wltz", {{ 0x2e03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wltz", {{ 0x2e03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.wnez", {{ 0x2e038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"afaddw.wnez", {{ 0x2e038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_registerZ_registerT_simple */
	{"aladdw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"afaddw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s027_registerZ_registerT_double */
	{"aladdw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.deqz", {{ 0xae031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dgez", {{ 0xae033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dgtz", {{ 0xae035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dlez", {{ 0xae034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dltz", {{ 0xae032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.dnez", {{ 0xae030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.even", {{ 0xae037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.odd", {{ 0xae036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.weqz", {{ 0xae039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wgez", {{ 0xae03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wgtz", {{ 0xae03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wlez", {{ 0xae03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wltz", {{ 0xae03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"afaddw.wnez", {{ 0xae038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-ALADDW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"aladdw", {{ 0x2e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_s010_registerZ_registerT_simple */
	{"afaddw", {{ 0x2e010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_s010_registerZ_registerT_simple */
	{"aladdw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_s037_registerZ_registerT_double */
	{"afaddw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_s037_registerZ_registerT_double */
	{"aladdw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_w064_registerZ_registerT_triple */
	{"afaddw", {{ 0xae010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-ALADDW_w064_registerZ_registerT_triple */
	{"alclrd", {{ 0x2b03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRD_doscale_registerW_registerY_registerZ_simple */
	{"alclrd.xs", {{ 0x2b03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRD_doscale_registerW_registerY_registerZ_simple */
	{"alclrd.deqz", {{ 0x2b031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dgez", {{ 0x2b033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dgtz", {{ 0x2b035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dlez", {{ 0x2b034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dltz", {{ 0x2b032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.dnez", {{ 0x2b030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.even", {{ 0x2b037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.odd", {{ 0x2b036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.weqz", {{ 0x2b039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wgez", {{ 0x2b03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wgtz", {{ 0x2b03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wlez", {{ 0x2b03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wltz", {{ 0x2b03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.wnez", {{ 0x2b038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_registerZ_simple */
	{"alclrd.deqz", {{ 0xab031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dgez", {{ 0xab033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dgtz", {{ 0xab035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dlez", {{ 0xab034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dltz", {{ 0xab032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.dnez", {{ 0xab030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.even", {{ 0xab037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.odd", {{ 0xab036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.weqz", {{ 0xab039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wgez", {{ 0xab03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wgtz", {{ 0xab03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wlez", {{ 0xab03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wltz", {{ 0xab03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.wnez", {{ 0xab038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrd.deqz", {{ 0xab031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dgez", {{ 0xab033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dgtz", {{ 0xab035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dlez", {{ 0xab034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dltz", {{ 0xab032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.dnez", {{ 0xab030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.even", {{ 0xab037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.odd", {{ 0xab036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.weqz", {{ 0xab039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wgez", {{ 0xab03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wgtz", {{ 0xab03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wlez", {{ 0xab03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wltz", {{ 0xab03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd.wnez", {{ 0xab038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRD_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrd", {{ 0x2b010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRD_registerW_s010_registerZ_simple */
	{"alclrd", {{ 0xab010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRD_registerW_s037_registerZ_double */
	{"alclrd", {{ 0xab010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRD_registerW_w064_registerZ_triple */
	{"alclrw", {{ 0x2a03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRW_doscale_registerW_registerY_registerZ_simple */
	{"alclrw.xs", {{ 0x2a03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRW_doscale_registerW_registerY_registerZ_simple */
	{"alclrw.deqz", {{ 0x2a031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dgez", {{ 0x2a033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dgtz", {{ 0x2a035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dlez", {{ 0x2a034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dltz", {{ 0x2a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.dnez", {{ 0x2a030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.even", {{ 0x2a037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.odd", {{ 0x2a036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.weqz", {{ 0x2a039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wgez", {{ 0x2a03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wgtz", {{ 0x2a03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wlez", {{ 0x2a03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wltz", {{ 0x2a03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.wnez", {{ 0x2a038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_registerZ_simple */
	{"alclrw.deqz", {{ 0xaa031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dgez", {{ 0xaa033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dgtz", {{ 0xaa035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dlez", {{ 0xaa034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dltz", {{ 0xaa032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.dnez", {{ 0xaa030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.even", {{ 0xaa037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.odd", {{ 0xaa036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.weqz", {{ 0xaa039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wgez", {{ 0xaa03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wgtz", {{ 0xaa03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wlez", {{ 0xaa03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wltz", {{ 0xaa03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.wnez", {{ 0xaa038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s027_registerZ_double */
	{"alclrw.deqz", {{ 0xaa031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dgez", {{ 0xaa033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dgtz", {{ 0xaa035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dlez", {{ 0xaa034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dltz", {{ 0xaa032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.dnez", {{ 0xaa030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.even", {{ 0xaa037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.odd", {{ 0xaa036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.weqz", {{ 0xaa039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wgez", {{ 0xaa03b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wgtz", {{ 0xaa03d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wlez", {{ 0xaa03c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wltz", {{ 0xaa03a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw.wnez", {{ 0xaa038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-ALCLRW_lsucond_registerY_registerW_s054_registerZ_triple */
	{"alclrw", {{ 0x2a010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRW_registerW_s010_registerZ_simple */
	{"alclrw", {{ 0xaa010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRW_registerW_s037_registerZ_double */
	{"alclrw", {{ 0xaa010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-ALCLRW_registerW_w064_registerZ_triple */
	{"aligno", {{ 0x00081040, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNO_registerN_registerBe_registerCo_registerZ_simple */
	{"aligno", {{ 0x00081000, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNO_registerN_registerBe_registerCo_u006_simple */
	{"aligno", {{ 0x000c1040, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNO_registerN_registerBo_registerCe_registerZ_simple */
	{"aligno", {{ 0x000c1000, 0x7f0c1040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_AUXW_CRRP, {&kv3_registern_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNO_registerN_registerBo_registerCe_u006_simple */
	{"alignv", {{ 0x01000040, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNV_registerA_registerBe_registerCo_registerZ_simple */
	{"alignv", {{ 0x01000000, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbe_opnd, &kv3_registerco_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNV_registerA_registerBe_registerCo_u006_simple */
	{"alignv", {{ 0x01001040, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNV_registerA_registerBo_registerCe_registerZ_simple */
	{"alignv", {{ 0x01001000, 0x7f001040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_CRRP_CRWL_CRWH, {&kv3_registera_opnd, &kv3_registerbo_opnd, &kv3_registerce_opnd, &kv3_byteshift_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-ALIGNV_registerA_registerBo_registerCe_u006_simple */
	{"andd", {{ 0x78010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDD_registerW_registerZ_registerY_simple */
	{"andd", {{ 0x68000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDD_registerW_registerZ_s010_simple */
	{"andd", {{ 0xe8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDD_registerW_registerZ_s037_double */
	{"andd.@", {{ 0xf8010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDD_registerW_registerZ_w032_double */
	{"andd", {{ 0xe8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDD_registerW_registerZ_w064_triple */
	{"andnd", {{ 0x7e010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDND_registerW_registerZ_registerY_simple */
	{"andnd", {{ 0x6e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDND_registerW_registerZ_s010_simple */
	{"andnd", {{ 0xee000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDND_registerW_registerZ_s037_double */
	{"andnd.@", {{ 0xfe010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDND_registerW_registerZ_w032_double */
	{"andnd", {{ 0xee000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDND_registerW_registerZ_w064_triple */
	{"andnw", {{ 0x7e011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDNW_registerW_registerZ_registerY_simple */
	{"andnw", {{ 0x7e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDNW_registerW_registerZ_s010_simple */
	{"andnw", {{ 0xfe000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDNW_registerW_registerZ_s037_double */
	{"andw", {{ 0x78011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDW_registerW_registerZ_registerY_simple */
	{"andw", {{ 0x78000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDW_registerW_registerZ_s010_simple */
	{"andw", {{ 0xf8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ANDW_registerW_registerZ_s037_double */
	{"avghq", {{ 0x78017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGHQ_registerW_registerZ_registerY_simple */
	{"avghq", {{ 0xf8017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGHQ_splat32_registerW_registerZ_w032_double */
	{"avghq.@", {{ 0xf8017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGHQ_splat32_registerW_registerZ_w032_double */
	{"avgrhq", {{ 0x7a017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRHQ_registerW_registerZ_registerY_simple */
	{"avgrhq", {{ 0xfa017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRHQ_splat32_registerW_registerZ_w032_double */
	{"avgrhq.@", {{ 0xfa017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRHQ_splat32_registerW_registerZ_w032_double */
	{"avgruhq", {{ 0x7b017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUHQ_registerW_registerZ_registerY_simple */
	{"avgruhq", {{ 0xfb017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUHQ_splat32_registerW_registerZ_w032_double */
	{"avgruhq.@", {{ 0xfb017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUHQ_splat32_registerW_registerZ_w032_double */
	{"avgruwp", {{ 0x7b016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUWP_registerW_registerZ_registerY_simple */
	{"avgruwp", {{ 0xfb016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUWP_splat32_registerW_registerZ_w032_double */
	{"avgruwp.@", {{ 0xfb016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUWP_splat32_registerW_registerZ_w032_double */
	{"avgruw", {{ 0x7b015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUW_registerW_registerZ_registerY_simple */
	{"avgruw", {{ 0xfb015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRUW_registerW_registerZ_w032_double */
	{"avgrwp", {{ 0x7a016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRWP_registerW_registerZ_registerY_simple */
	{"avgrwp", {{ 0xfa016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRWP_splat32_registerW_registerZ_w032_double */
	{"avgrwp.@", {{ 0xfa016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRWP_splat32_registerW_registerZ_w032_double */
	{"avgrw", {{ 0x7a015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRW_registerW_registerZ_registerY_simple */
	{"avgrw", {{ 0xfa015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGRW_registerW_registerZ_w032_double */
	{"avguhq", {{ 0x79017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUHQ_registerW_registerZ_registerY_simple */
	{"avguhq", {{ 0xf9017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUHQ_splat32_registerW_registerZ_w032_double */
	{"avguhq.@", {{ 0xf9017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUHQ_splat32_registerW_registerZ_w032_double */
	{"avguwp", {{ 0x79016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUWP_registerW_registerZ_registerY_simple */
	{"avguwp", {{ 0xf9016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUWP_splat32_registerW_registerZ_w032_double */
	{"avguwp.@", {{ 0xf9016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUWP_splat32_registerW_registerZ_w032_double */
	{"avguw", {{ 0x79015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUW_registerW_registerZ_registerY_simple */
	{"avguw", {{ 0xf9015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGUW_registerW_registerZ_w032_double */
	{"avgwp", {{ 0x78016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGWP_registerW_registerZ_registerY_simple */
	{"avgwp", {{ 0xf8016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGWP_splat32_registerW_registerZ_w032_double */
	{"avgwp.@", {{ 0xf8016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGWP_splat32_registerW_registerZ_w032_double */
	{"avgw", {{ 0x78015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGW_registerW_registerZ_registerY_simple */
	{"avgw", {{ 0xf8015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-AVGW_registerW_registerZ_w032_double */
	{"await", {{ 0x0fa00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-AWAIT_simple */
	{"barrier", {{ 0x0fac0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-BARRIER_simple */
	{"break", {{ 0x00000040, 0x7fffffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_brknumber_opnd, 0}, "", "%s"}, /* Opcode-v2-BREAK_u002_simple */
	{"call", {{ 0x18000000, 0x78000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_pcrel27_opnd, 0}, "", "%s"}, /* Opcode-v2-CALL_s027_simple */
	{"cbsd", {{ 0x72022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CBSD_registerW_registerZ_simple */
	{"cbswp", {{ 0x72024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CBSWP_registerW_registerZ_simple */
	{"cbsw", {{ 0x72023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CBSW_registerW_registerZ_simple */
	{"cb.deqz", {{ 0x08800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.dgez", {{ 0x09800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.dgtz", {{ 0x0a800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.dlez", {{ 0x0a000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.dltz", {{ 0x09000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.dnez", {{ 0x08000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.even", {{ 0x0b800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.odd", {{ 0x0b000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.weqz", {{ 0x0c800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.wgez", {{ 0x0d800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.wgtz", {{ 0x0e800000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.wlez", {{ 0x0e000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.wltz", {{ 0x0d000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"cb.wnez", {{ 0x0c000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s? %s"}, /* Opcode-v2-CB_branchcond_registerZ_s017_simple */
	{"clrf", {{ 0x6c030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-CLRF_registerW_registerZ_u006_u006_simple */
	{"clsd", {{ 0x71022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CLSD_registerW_registerZ_simple */
	{"clswp", {{ 0x71024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CLSWP_registerW_registerZ_simple */
	{"clsw", {{ 0x71023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CLSW_registerW_registerZ_simple */
	{"clzd", {{ 0x70022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CLZD_registerW_registerZ_simple */
	{"clzwp", {{ 0x70024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CLZWP_registerW_registerZ_simple */
	{"clzw", {{ 0x70023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CLZW_registerW_registerZ_simple */
	{"cmoved.deqz", {{ 0x71020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dgez", {{ 0x73020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dgtz", {{ 0x75020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dlez", {{ 0x74020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dltz", {{ 0x72020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.dnez", {{ 0x70020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.even", {{ 0x77020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.odd", {{ 0x76020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.weqz", {{ 0x79020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wgez", {{ 0x7b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wgtz", {{ 0x7d020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wlez", {{ 0x7c020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wltz", {{ 0x7a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.wnez", {{ 0x78020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_registerY_simple */
	{"cmoved.deqz", {{ 0x61020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dgez", {{ 0x63020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dgtz", {{ 0x65020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dlez", {{ 0x64020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dltz", {{ 0x62020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.dnez", {{ 0x60020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.even", {{ 0x67020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.odd", {{ 0x66020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.weqz", {{ 0x69020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wgez", {{ 0x6b020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wgtz", {{ 0x6d020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wlez", {{ 0x6c020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wltz", {{ 0x6a020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.wnez", {{ 0x68020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_signed10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s010_simple */
	{"cmoved.deqz", {{ 0xe1020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dgez", {{ 0xe3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dgtz", {{ 0xe5020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dlez", {{ 0xe4020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dltz", {{ 0xe2020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.dnez", {{ 0xe0020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.even", {{ 0xe7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.odd", {{ 0xe6020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.weqz", {{ 0xe9020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wgez", {{ 0xeb020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wgtz", {{ 0xed020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wlez", {{ 0xec020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wltz", {{ 0xea020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.wnez", {{ 0xe8020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_s037_double */
	{"cmoved.deqz", {{ 0xe1020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dgez", {{ 0xe3020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dgtz", {{ 0xe5020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dlez", {{ 0xe4020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dltz", {{ 0xe2020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.dnez", {{ 0xe0020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.even", {{ 0xe7020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.odd", {{ 0xe6020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.weqz", {{ 0xe9020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wgez", {{ 0xeb020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wgtz", {{ 0xed020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wlez", {{ 0xec020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wltz", {{ 0xea020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmoved.wnez", {{ 0xe8020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVED_scalarcond_registerZ_registerW_w064_triple */
	{"cmovehq.eqz", {{ 0x79021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.even", {{ 0x7f021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.gez", {{ 0x7b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.gtz", {{ 0x7d021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.lez", {{ 0x7c021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.ltz", {{ 0x7a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.nez", {{ 0x78021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovehq.odd", {{ 0x7e021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEHQ_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.eqz", {{ 0x71021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.even", {{ 0x77021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.gez", {{ 0x73021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.gtz", {{ 0x75021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.lez", {{ 0x74021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.ltz", {{ 0x72021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.nez", {{ 0x70021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmovewp.odd", {{ 0x76021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerz_opnd, &kv3_registerw_opnd, &kv3_registery_opnd, 0}, "", "%s? %s = %s"}, /* Opcode-v2-CMOVEWP_simdcond_registerZ_registerW_registerY_simple */
	{"cmuldt", {{ 0x5b041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULDT_registerM_registerZ_registerY_simple */
	{"cmuldt", {{ 0x4b040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULDT_registerM_registerZ_s010_simple */
	{"cmuldt", {{ 0xcb040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULDT_registerM_registerZ_s037_double */
	{"cmuldt", {{ 0xcb040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULDT_registerM_registerZ_w064_triple */
	{"cmulghxdt", {{ 0x5f041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULGHXDT_registerM_registerZ_registerY_simple */
	{"cmulglxdt", {{ 0x5d041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULGLXDT_registerM_registerZ_registerY_simple */
	{"cmulgmxdt", {{ 0x5e041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULGMXDT_registerM_registerZ_registerY_simple */
	{"cmulxdt", {{ 0x5c041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CMULXDT_registerM_registerZ_registerY_simple */
	{"compd.all", {{ 0x7a01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.any", {{ 0x7c01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.eq", {{ 0x7101a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.ge", {{ 0x7301a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.geu", {{ 0x7701a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.gt", {{ 0x7501a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.gtu", {{ 0x7901a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.le", {{ 0x7401a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.leu", {{ 0x7801a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.lt", {{ 0x7201a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.ltu", {{ 0x7601a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.nall", {{ 0x7b01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.ne", {{ 0x7001a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.none", {{ 0x7d01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_registerY_simple */
	{"compd.all", {{ 0x6a010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.any", {{ 0x6c010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.eq", {{ 0x61010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.ge", {{ 0x63010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.geu", {{ 0x67010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.gt", {{ 0x65010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.gtu", {{ 0x69010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.le", {{ 0x64010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.leu", {{ 0x68010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.lt", {{ 0x62010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.ltu", {{ 0x66010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.nall", {{ 0x6b010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.ne", {{ 0x60010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.none", {{ 0x6d010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s010_simple */
	{"compd.all", {{ 0xea010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.any", {{ 0xec010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.eq", {{ 0xe1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.ge", {{ 0xe3010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.geu", {{ 0xe7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.gt", {{ 0xe5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.gtu", {{ 0xe9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.le", {{ 0xe4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.leu", {{ 0xe8010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.lt", {{ 0xe2010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.ltu", {{ 0xe6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.nall", {{ 0xeb010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.ne", {{ 0xe0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.none", {{ 0xed010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_s037_double */
	{"compd.all", {{ 0xea010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.any", {{ 0xec010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.eq", {{ 0xe1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.ge", {{ 0xe3010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.geu", {{ 0xe7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.gt", {{ 0xe5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.gtu", {{ 0xe9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.le", {{ 0xe4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.leu", {{ 0xe8010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.lt", {{ 0xe2010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.ltu", {{ 0xe6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.nall", {{ 0xeb010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.ne", {{ 0xe0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compd.none", {{ 0xed010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPD_comparison_registerW_registerZ_w064_triple */
	{"compnhq.all", {{ 0x7a01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.any", {{ 0x7c01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.eq", {{ 0x7101f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.ge", {{ 0x7301f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.geu", {{ 0x7701f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.gt", {{ 0x7501f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.gtu", {{ 0x7901f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.le", {{ 0x7401f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.leu", {{ 0x7801f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.lt", {{ 0x7201f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.ltu", {{ 0x7601f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.nall", {{ 0x7b01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.ne", {{ 0x7001f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnhq.none", {{ 0x7d01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNHQ_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.all", {{ 0x7a01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.any", {{ 0x7c01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.eq", {{ 0x7101e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.ge", {{ 0x7301e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.geu", {{ 0x7701e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.gt", {{ 0x7501e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.gtu", {{ 0x7901e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.le", {{ 0x7401e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.leu", {{ 0x7801e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.lt", {{ 0x7201e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.ltu", {{ 0x7601e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.nall", {{ 0x7b01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.ne", {{ 0x7001e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compnwp.none", {{ 0x7d01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPNWP_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.all", {{ 0x7a01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.any", {{ 0x7c01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.eq", {{ 0x7101d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.ge", {{ 0x7301d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.geu", {{ 0x7701d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.gt", {{ 0x7501d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.gtu", {{ 0x7901d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.le", {{ 0x7401d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.leu", {{ 0x7801d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.lt", {{ 0x7201d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.ltu", {{ 0x7601d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.nall", {{ 0x7b01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.ne", {{ 0x7001d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compuwd.none", {{ 0x7d01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPUWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.all", {{ 0x7a01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.any", {{ 0x7c01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.eq", {{ 0x7101c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.ge", {{ 0x7301c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.geu", {{ 0x7701c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.gt", {{ 0x7501c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.gtu", {{ 0x7901c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.le", {{ 0x7401c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.leu", {{ 0x7801c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.lt", {{ 0x7201c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.ltu", {{ 0x7601c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.nall", {{ 0x7b01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.ne", {{ 0x7001c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compwd.none", {{ 0x7d01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPWD_comparison_registerW_registerZ_registerY_simple */
	{"compw.all", {{ 0x7a01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.any", {{ 0x7c01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.eq", {{ 0x7101b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.ge", {{ 0x7301b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.geu", {{ 0x7701b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.gt", {{ 0x7501b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.gtu", {{ 0x7901b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.le", {{ 0x7401b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.leu", {{ 0x7801b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.lt", {{ 0x7201b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.ltu", {{ 0x7601b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.nall", {{ 0x7b01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.ne", {{ 0x7001b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.none", {{ 0x7d01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_registerY_simple */
	{"compw.all", {{ 0xfa01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.any", {{ 0xfc01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.eq", {{ 0xf101b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.ge", {{ 0xf301b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.geu", {{ 0xf701b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.gt", {{ 0xf501b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.gtu", {{ 0xf901b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.le", {{ 0xf401b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.leu", {{ 0xf801b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.lt", {{ 0xf201b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.ltu", {{ 0xf601b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.nall", {{ 0xfb01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.ne", {{ 0xf001b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"compw.none", {{ 0xfd01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COMPW_comparison_registerW_registerZ_w032_double */
	{"convdhv0.rd.sat", {{ 0x07001240, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rd.satu", {{ 0x070012c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rhu.sat", {{ 0x07001440, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rhu.satu", {{ 0x070014c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rn.sat", {{ 0x07001040, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rn.satu", {{ 0x070010c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.ru.sat", {{ 0x07001140, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.ru.satu", {{ 0x070011c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rz.sat", {{ 0x07001340, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv0.rz.satu", {{ 0x070013c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeral_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV0_roundint_saturate_registerAl_registerBq_simple */
	{"convdhv1.rd.sat", {{ 0x07003240, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rd.satu", {{ 0x070032c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rhu.sat", {{ 0x07003440, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rhu.satu", {{ 0x070034c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rn.sat", {{ 0x07003040, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rn.satu", {{ 0x070030c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.ru.sat", {{ 0x07003140, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.ru.satu", {{ 0x070031c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rz.sat", {{ 0x07003340, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"convdhv1.rz.satu", {{ 0x070033c0, 0x7f003fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerah_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CONVDHV1_roundint_saturate_registerAh_registerBq_simple */
	{"copyo", {{ 0x3e050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_AUXW, {&kv3_registern_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-COPYO_registerN_registerR_simple */
	{"copyq", {{ 0x5f00f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-COPYQ_registerM_registerZ_registerY_simple */
	{"crcbellw", {{ 0x59002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCBELLW_registerW_registerZ_registerY_simple */
	{"crcbellw", {{ 0xd9002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCBELLW_registerW_registerZ_w032_double */
	{"crcbelmw", {{ 0x58002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCBELMW_registerW_registerZ_registerY_simple */
	{"crcbelmw", {{ 0xd8002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCBELMW_registerW_registerZ_w032_double */
	{"crclellw", {{ 0x5b002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCLELLW_registerW_registerZ_registerY_simple */
	{"crclellw", {{ 0xdb002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCLELLW_registerW_registerZ_w032_double */
	{"crclelmw", {{ 0x5a002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCLELMW_registerW_registerZ_registerY_simple */
	{"crclelmw", {{ 0xda002000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-CRCLELMW_registerW_registerZ_w032_double */
	{"ctzd", {{ 0x73022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CTZD_registerW_registerZ_simple */
	{"ctzwp", {{ 0x73024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CTZWP_registerW_registerZ_simple */
	{"ctzw", {{ 0x73023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-CTZW_registerW_registerZ_simple */
	{"dinvall.l1l2.deqz", {{ 0x3e1e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.dgez", {{ 0x3e1e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.dgtz", {{ 0x3e1e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.dlez", {{ 0x3e1e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.dltz", {{ 0x3e1e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.dnez", {{ 0x3e1e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.even", {{ 0x3e1e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.odd", {{ 0x3e1e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.weqz", {{ 0x3e1e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.wgez", {{ 0x3e1eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.wgtz", {{ 0x3e1ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.wlez", {{ 0x3e1ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.wltz", {{ 0x3e1ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.wnez", {{ 0x3e1e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.deqz", {{ 0x3d1e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.dgez", {{ 0x3d1e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.dgtz", {{ 0x3d1e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.dlez", {{ 0x3d1e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.dltz", {{ 0x3d1e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.dnez", {{ 0x3d1e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.even", {{ 0x3d1e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.odd", {{ 0x3d1e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.weqz", {{ 0x3d1e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.wgez", {{ 0x3d1eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.wgtz", {{ 0x3d1ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.wlez", {{ 0x3d1ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.wltz", {{ 0x3d1ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l2.wnez", {{ 0x3d1e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.deqz", {{ 0x3c1e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.dgez", {{ 0x3c1e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.dgtz", {{ 0x3c1e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.dlez", {{ 0x3c1e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.dltz", {{ 0x3c1e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.dnez", {{ 0x3c1e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.even", {{ 0x3c1e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.odd", {{ 0x3c1e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.weqz", {{ 0x3c1e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.wgez", {{ 0x3c1eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.wgtz", {{ 0x3c1ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.wlez", {{ 0x3c1ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.wltz", {{ 0x3c1ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.wnez", {{ 0x3c1e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_registerZ_simple */
	{"dinvall.l1l2.deqz", {{ 0xbe1e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.dgez", {{ 0xbe1e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.dgtz", {{ 0xbe1e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.dlez", {{ 0xbe1e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.dltz", {{ 0xbe1e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.dnez", {{ 0xbe1e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.even", {{ 0xbe1e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.odd", {{ 0xbe1e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.weqz", {{ 0xbe1e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.wgez", {{ 0xbe1eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.wgtz", {{ 0xbe1ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.wlez", {{ 0xbe1ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.wltz", {{ 0xbe1ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.wnez", {{ 0xbe1e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.deqz", {{ 0xbd1e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.dgez", {{ 0xbd1e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.dgtz", {{ 0xbd1e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.dlez", {{ 0xbd1e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.dltz", {{ 0xbd1e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.dnez", {{ 0xbd1e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.even", {{ 0xbd1e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.odd", {{ 0xbd1e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.weqz", {{ 0xbd1e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.wgez", {{ 0xbd1eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.wgtz", {{ 0xbd1ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.wlez", {{ 0xbd1ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.wltz", {{ 0xbd1ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l2.wnez", {{ 0xbd1e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.deqz", {{ 0xbc1e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dgez", {{ 0xbc1e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dgtz", {{ 0xbc1e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dlez", {{ 0xbc1e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dltz", {{ 0xbc1e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.dnez", {{ 0xbc1e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.even", {{ 0xbc1e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.odd", {{ 0xbc1e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.weqz", {{ 0xbc1e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wgez", {{ 0xbc1eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wgtz", {{ 0xbc1ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wlez", {{ 0xbc1ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wltz", {{ 0xbc1ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.wnez", {{ 0xbc1e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dinvall.l1l2.deqz", {{ 0xbe1e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.dgez", {{ 0xbe1e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.dgtz", {{ 0xbe1e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.dlez", {{ 0xbe1e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.dltz", {{ 0xbe1e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.dnez", {{ 0xbe1e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.even", {{ 0xbe1e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.odd", {{ 0xbe1e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.weqz", {{ 0xbe1e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.wgez", {{ 0xbe1eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.wgtz", {{ 0xbe1ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.wlez", {{ 0xbe1ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.wltz", {{ 0xbe1ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l1l2.wnez", {{ 0xbe1e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.deqz", {{ 0xbd1e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.dgez", {{ 0xbd1e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.dgtz", {{ 0xbd1e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.dlez", {{ 0xbd1e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.dltz", {{ 0xbd1e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.dnez", {{ 0xbd1e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.even", {{ 0xbd1e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.odd", {{ 0xbd1e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.weqz", {{ 0xbd1e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.wgez", {{ 0xbd1eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.wgtz", {{ 0xbd1ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.wlez", {{ 0xbd1ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.wltz", {{ 0xbd1ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.l2.wnez", {{ 0xbd1e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.deqz", {{ 0xbc1e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dgez", {{ 0xbc1e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dgtz", {{ 0xbc1e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dlez", {{ 0xbc1e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dltz", {{ 0xbc1e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.dnez", {{ 0xbc1e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.even", {{ 0xbc1e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.odd", {{ 0xbc1e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.weqz", {{ 0xbc1e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wgez", {{ 0xbc1eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wgtz", {{ 0xbc1ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wlez", {{ 0xbc1ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wltz", {{ 0xbc1ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall.wnez", {{ 0xbc1e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DINVALL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dinvall", {{ 0x3c1ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_registerY_registerZ_simple */
	{"dinvall.l1l2", {{ 0x3e1ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_registerY_registerZ_simple */
	{"dinvall.l2", {{ 0x3d1ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_registerY_registerZ_simple */
	{"dinvall", {{ 0x3c1c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_s010_registerZ_simple */
	{"dinvall.l1l2", {{ 0x3e1c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_s010_registerZ_simple */
	{"dinvall.l2", {{ 0x3d1c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_s010_registerZ_simple */
	{"dinvall", {{ 0xbc1c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_s037_registerZ_double */
	{"dinvall.l1l2", {{ 0xbe1c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_s037_registerZ_double */
	{"dinvall.l2", {{ 0xbd1c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_s037_registerZ_double */
	{"dinvall", {{ 0xbc1c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_w064_registerZ_triple */
	{"dinvall.l1l2", {{ 0xbe1c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_w064_registerZ_triple */
	{"dinvall.l2", {{ 0xbd1c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DINVALL_cachelev_w064_registerZ_triple */
	{"dinval", {{ 0x3f8d0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {0}, "", ""}, /* Opcode-v2-DINVAL_simple */
	{"dot2suwdp", {{ 0x52002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2SUWDP_registerM_registerP_registerO_simple */
	{"dot2suwd", {{ 0x5e002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2SUWD_registerW_registerZ_registerY_simple */
	{"dot2suwd", {{ 0x4e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2SUWD_registerW_registerZ_s010_simple */
	{"dot2suwd", {{ 0xce000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2SUWD_registerW_registerZ_s037_double */
	{"dot2suwd", {{ 0xce000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2SUWD_registerW_registerZ_w064_triple */
	{"dot2uwdp", {{ 0x51002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2UWDP_registerM_registerP_registerO_simple */
	{"dot2uwd", {{ 0x5d002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2UWD_registerW_registerZ_registerY_simple */
	{"dot2uwd", {{ 0x4d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2UWD_registerW_registerZ_s010_simple */
	{"dot2uwd", {{ 0xcd000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2UWD_registerW_registerZ_s037_double */
	{"dot2uwd", {{ 0xcd000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2UWD_registerW_registerZ_w064_triple */
	{"dot2wdp", {{ 0x50002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2WDP_registerM_registerP_registerO_simple */
	{"dot2wd", {{ 0x5c002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2WD_registerW_registerZ_registerY_simple */
	{"dot2wd", {{ 0x4c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2WD_registerW_registerZ_s010_simple */
	{"dot2wd", {{ 0xcc000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2WD_registerW_registerZ_s037_double */
	{"dot2wd", {{ 0xcc000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2WD_registerW_registerZ_w064_triple */
	{"dot2wzp", {{ 0x53002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2WZP_registerM_registerP_registerO_simple */
	{"dot2w", {{ 0x5f002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2W_registerW_registerZ_registerY_simple */
	{"dot2w", {{ 0x4f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2W_registerW_registerZ_s010_simple */
	{"dot2w", {{ 0xcf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2W_registerW_registerZ_s037_double */
	{"dot2w", {{ 0xcf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-DOT2W_registerW_registerZ_w064_triple */
	{"dtouchl.l1l2.deqz", {{ 0x3e0e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.dgez", {{ 0x3e0e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.dgtz", {{ 0x3e0e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.dlez", {{ 0x3e0e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.dltz", {{ 0x3e0e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.dnez", {{ 0x3e0e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.even", {{ 0x3e0e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.odd", {{ 0x3e0e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.weqz", {{ 0x3e0e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.wgez", {{ 0x3e0eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.wgtz", {{ 0x3e0ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.wlez", {{ 0x3e0ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.wltz", {{ 0x3e0ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.wnez", {{ 0x3e0e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.deqz", {{ 0x3d0e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.dgez", {{ 0x3d0e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.dgtz", {{ 0x3d0e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.dlez", {{ 0x3d0e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.dltz", {{ 0x3d0e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.dnez", {{ 0x3d0e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.even", {{ 0x3d0e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.odd", {{ 0x3d0e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.weqz", {{ 0x3d0e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.wgez", {{ 0x3d0eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.wgtz", {{ 0x3d0ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.wlez", {{ 0x3d0ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.wltz", {{ 0x3d0ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l2.wnez", {{ 0x3d0e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.deqz", {{ 0x3c0e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.dgez", {{ 0x3c0e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.dgtz", {{ 0x3c0e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.dlez", {{ 0x3c0e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.dltz", {{ 0x3c0e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.dnez", {{ 0x3c0e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.even", {{ 0x3c0e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.odd", {{ 0x3c0e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.weqz", {{ 0x3c0e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.wgez", {{ 0x3c0eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.wgtz", {{ 0x3c0ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.wlez", {{ 0x3c0ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.wltz", {{ 0x3c0ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.wnez", {{ 0x3c0e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_registerZ_simple */
	{"dtouchl.l1l2.deqz", {{ 0xbe0e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.dgez", {{ 0xbe0e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.dgtz", {{ 0xbe0e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.dlez", {{ 0xbe0e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.dltz", {{ 0xbe0e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.dnez", {{ 0xbe0e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.even", {{ 0xbe0e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.odd", {{ 0xbe0e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.weqz", {{ 0xbe0e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.wgez", {{ 0xbe0eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.wgtz", {{ 0xbe0ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.wlez", {{ 0xbe0ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.wltz", {{ 0xbe0ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.wnez", {{ 0xbe0e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.deqz", {{ 0xbd0e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.dgez", {{ 0xbd0e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.dgtz", {{ 0xbd0e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.dlez", {{ 0xbd0e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.dltz", {{ 0xbd0e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.dnez", {{ 0xbd0e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.even", {{ 0xbd0e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.odd", {{ 0xbd0e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.weqz", {{ 0xbd0e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.wgez", {{ 0xbd0eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.wgtz", {{ 0xbd0ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.wlez", {{ 0xbd0ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.wltz", {{ 0xbd0ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l2.wnez", {{ 0xbd0e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.deqz", {{ 0xbc0e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dgez", {{ 0xbc0e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dgtz", {{ 0xbc0e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dlez", {{ 0xbc0e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dltz", {{ 0xbc0e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.dnez", {{ 0xbc0e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.even", {{ 0xbc0e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.odd", {{ 0xbc0e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.weqz", {{ 0xbc0e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wgez", {{ 0xbc0eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wgtz", {{ 0xbc0ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wlez", {{ 0xbc0ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wltz", {{ 0xbc0ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.wnez", {{ 0xbc0e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s027_registerZ_double */
	{"dtouchl.l1l2.deqz", {{ 0xbe0e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.dgez", {{ 0xbe0e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.dgtz", {{ 0xbe0e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.dlez", {{ 0xbe0e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.dltz", {{ 0xbe0e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.dnez", {{ 0xbe0e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.even", {{ 0xbe0e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.odd", {{ 0xbe0e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.weqz", {{ 0xbe0e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.wgez", {{ 0xbe0eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.wgtz", {{ 0xbe0ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.wlez", {{ 0xbe0ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.wltz", {{ 0xbe0ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l1l2.wnez", {{ 0xbe0e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.deqz", {{ 0xbd0e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.dgez", {{ 0xbd0e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.dgtz", {{ 0xbd0e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.dlez", {{ 0xbd0e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.dltz", {{ 0xbd0e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.dnez", {{ 0xbd0e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.even", {{ 0xbd0e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.odd", {{ 0xbd0e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.weqz", {{ 0xbd0e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.wgez", {{ 0xbd0eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.wgtz", {{ 0xbd0ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.wlez", {{ 0xbd0ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.wltz", {{ 0xbd0ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.l2.wnez", {{ 0xbd0e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.deqz", {{ 0xbc0e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dgez", {{ 0xbc0e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dgtz", {{ 0xbc0e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dlez", {{ 0xbc0e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dltz", {{ 0xbc0e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.dnez", {{ 0xbc0e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.even", {{ 0xbc0e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.odd", {{ 0xbc0e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.weqz", {{ 0xbc0e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wgez", {{ 0xbc0eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wgtz", {{ 0xbc0ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wlez", {{ 0xbc0ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wltz", {{ 0xbc0ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl.wnez", {{ 0xbc0e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"dtouchl", {{ 0x3c0ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_registerY_registerZ_simple */
	{"dtouchl.l1l2", {{ 0x3e0ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_registerY_registerZ_simple */
	{"dtouchl.l2", {{ 0x3d0ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_registerY_registerZ_simple */
	{"dtouchl", {{ 0x3c0c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_s010_registerZ_simple */
	{"dtouchl.l1l2", {{ 0x3e0c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_s010_registerZ_simple */
	{"dtouchl.l2", {{ 0x3d0c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_s010_registerZ_simple */
	{"dtouchl", {{ 0xbc0c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_s037_registerZ_double */
	{"dtouchl.l1l2", {{ 0xbe0c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_s037_registerZ_double */
	{"dtouchl.l2", {{ 0xbd0c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_s037_registerZ_double */
	{"dtouchl", {{ 0xbc0c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_w064_registerZ_triple */
	{"dtouchl.l1l2", {{ 0xbe0c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_w064_registerZ_triple */
	{"dtouchl.l2", {{ 0xbd0c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-DTOUCHL_cachelev_w064_registerZ_triple */
	{"errop", {{ 0x00000000, 0x7fffffc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-ERROP_simple */
	{"extfs", {{ 0x68030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-EXTFS_registerW_registerZ_u006_u006_simple */
	{"extfz", {{ 0x64030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-EXTFZ_registerW_registerZ_u006_u006_simple */
	{"fabsd", {{ 0x71032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FABSD_registerW_registerZ_simple */
	{"fabshq", {{ 0x77032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FABSHQ_registerW_registerZ_simple */
	{"fabswp", {{ 0x75032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FABSWP_registerW_registerZ_simple */
	{"fabsw", {{ 0x73032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FABSW_registerW_registerZ_simple */
	{"faddcdc.rd", {{ 0x5d032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rd", {{ 0x5d032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rd.s", {{ 0x5d03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rd.s", {{ 0x5d03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rna", {{ 0x5d034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rna", {{ 0x5d034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rna.s", {{ 0x5d03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rna.s", {{ 0x5d03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rnz", {{ 0x5d035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rnz", {{ 0x5d035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rnz.s", {{ 0x5d03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rnz.s", {{ 0x5d03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rn", {{ 0x5d030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rn", {{ 0x5d030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rn.s", {{ 0x5d038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rn.s", {{ 0x5d038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ro", {{ 0x5d036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ro", {{ 0x5d036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ro.s", {{ 0x5d03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ro.s", {{ 0x5d03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ru", {{ 0x5d031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ru", {{ 0x5d031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.ru.s", {{ 0x5d039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.ru.s", {{ 0x5d039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rz", {{ 0x5d033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rz", {{ 0x5d033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.rz.s", {{ 0x5d03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.rz.s", {{ 0x5d03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc", {{ 0x5d037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c", {{ 0x5d037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcdc.s", {{ 0x5d03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.c.s", {{ 0x5d03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rd", {{ 0x59072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rd", {{ 0x59072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rd.s", {{ 0x5907a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rd.s", {{ 0x5907a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rna", {{ 0x59074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rna", {{ 0x59074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rna.s", {{ 0x5907c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rna.s", {{ 0x5907c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rnz", {{ 0x59075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rnz", {{ 0x59075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rnz.s", {{ 0x5907d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rnz.s", {{ 0x5907d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rn", {{ 0x59070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rn", {{ 0x59070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rn.s", {{ 0x59078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rn.s", {{ 0x59078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ro", {{ 0x59076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ro", {{ 0x59076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ro.s", {{ 0x5907e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ro.s", {{ 0x5907e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ru", {{ 0x59071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ru", {{ 0x59071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.ru.s", {{ 0x59079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.ru.s", {{ 0x59079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rz", {{ 0x59073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rz", {{ 0x59073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.rz.s", {{ 0x5907b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.rz.s", {{ 0x5907b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp", {{ 0x59077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c", {{ 0x59077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwcp.s", {{ 0x5907f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.c.s", {{ 0x5907f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddcwc.rd", {{ 0x53022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rd", {{ 0x53022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rd.s", {{ 0x5302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rd.s", {{ 0x5302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rna", {{ 0x53024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rna", {{ 0x53024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rna.s", {{ 0x5302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rna.s", {{ 0x5302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rnz", {{ 0x53025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rnz", {{ 0x53025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rnz.s", {{ 0x5302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rnz.s", {{ 0x5302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rn", {{ 0x53020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rn", {{ 0x53020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rn.s", {{ 0x53028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rn.s", {{ 0x53028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ro", {{ 0x53026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ro", {{ 0x53026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ro.s", {{ 0x5302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ro.s", {{ 0x5302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ru", {{ 0x53021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ru", {{ 0x53021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.ru.s", {{ 0x53029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.ru.s", {{ 0x53029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rz", {{ 0x53023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rz", {{ 0x53023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.rz.s", {{ 0x5302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.rz.s", {{ 0x5302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc", {{ 0x53027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c", {{ 0x53027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddcwc.s", {{ 0x5302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.c.s", {{ 0x5302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fadddp.rd", {{ 0x5c032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rd.s", {{ 0x5c03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rna", {{ 0x5c034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rna.s", {{ 0x5c03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rnz", {{ 0x5c035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rnz.s", {{ 0x5c03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rn", {{ 0x5c030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rn.s", {{ 0x5c038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ro", {{ 0x5c036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ro.s", {{ 0x5c03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ru", {{ 0x5c031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.ru.s", {{ 0x5c039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rz", {{ 0x5c033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.rz.s", {{ 0x5c03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp", {{ 0x5c037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fadddp.s", {{ 0x5c03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDDP_rounding_silent_registerM_registerP_registerO_simple */
	{"faddd.rd", {{ 0x50022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rd.s", {{ 0x5002a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rna", {{ 0x50024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rna.s", {{ 0x5002c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rnz", {{ 0x50025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rnz.s", {{ 0x5002d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rn", {{ 0x50020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rn.s", {{ 0x50028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ro", {{ 0x50026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ro.s", {{ 0x5002e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ru", {{ 0x50021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.ru.s", {{ 0x50029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rz", {{ 0x50023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.rz.s", {{ 0x5002b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd", {{ 0x50027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddd.s", {{ 0x5002f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDD_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddho.rd", {{ 0x56072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rd.s", {{ 0x5607a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rna", {{ 0x56074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rna.s", {{ 0x5607c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rnz", {{ 0x56075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rnz.s", {{ 0x5607d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rn", {{ 0x56070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rn.s", {{ 0x56078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.ro", {{ 0x56076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.ro.s", {{ 0x5607e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.ru", {{ 0x56071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.ru.s", {{ 0x56079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rz", {{ 0x56073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.rz.s", {{ 0x5607b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho", {{ 0x56077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddho.s", {{ 0x5607f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHO_rounding_silent_registerM_registerP_registerO_simple */
	{"faddhq.rd", {{ 0x52022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rd.s", {{ 0x5202a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rna", {{ 0x52024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rna.s", {{ 0x5202c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rnz", {{ 0x52025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rnz.s", {{ 0x5202d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rn", {{ 0x52020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rn.s", {{ 0x52028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ro", {{ 0x52026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ro.s", {{ 0x5202e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ru", {{ 0x52021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.ru.s", {{ 0x52029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rz", {{ 0x52023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.rz.s", {{ 0x5202b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq", {{ 0x52027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddhq.s", {{ 0x5202f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rd", {{ 0x51022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rd.s", {{ 0x5102a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rna", {{ 0x51024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rna.s", {{ 0x5102c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rnz", {{ 0x51025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rnz.s", {{ 0x5102d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rn", {{ 0x51020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rn.s", {{ 0x51028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ro", {{ 0x51026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ro.s", {{ 0x5102e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ru", {{ 0x51021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.ru.s", {{ 0x51029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rz", {{ 0x51023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.rz.s", {{ 0x5102b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp", {{ 0x51027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwp.s", {{ 0x5102f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddwq.rd", {{ 0x58072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rd.s", {{ 0x5807a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rna", {{ 0x58074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rna.s", {{ 0x5807c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rnz", {{ 0x58075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rnz.s", {{ 0x5807d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rn", {{ 0x58070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rn.s", {{ 0x58078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ro", {{ 0x58076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ro.s", {{ 0x5807e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ru", {{ 0x58071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.ru.s", {{ 0x58079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rz", {{ 0x58073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.rz.s", {{ 0x5807b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq", {{ 0x58077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddwq.s", {{ 0x5807f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"faddw.rd", {{ 0x5c022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rd.s", {{ 0x5c02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rna", {{ 0x5c024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rna.s", {{ 0x5c02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rnz", {{ 0x5c025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rnz.s", {{ 0x5c02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rn", {{ 0x5c020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rn.s", {{ 0x5c028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ro", {{ 0x5c026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ro.s", {{ 0x5c02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ru", {{ 0x5c021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.ru.s", {{ 0x5c029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rz", {{ 0x5c023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.rz.s", {{ 0x5c02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw", {{ 0x5c027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"faddw.s", {{ 0x5c02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FADDW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fcdivd", {{ 0x71035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FCDIVD_silent2_registerW_registerP_simple */
	{"fcdivd.s", {{ 0x71035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FCDIVD_silent2_registerW_registerP_simple */
	{"fcdivwp", {{ 0x75035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FCDIVWP_silent2_registerW_registerP_simple */
	{"fcdivwp.s", {{ 0x75035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FCDIVWP_silent2_registerW_registerP_simple */
	{"fcdivw", {{ 0x73035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FCDIVW_silent2_registerW_registerP_simple */
	{"fcdivw.s", {{ 0x73035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FCDIVW_silent2_registerW_registerP_simple */
	{"fcompd.oeq", {{ 0x7a030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.oge", {{ 0x7e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.olt", {{ 0x7c030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.one", {{ 0x78030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.ueq", {{ 0x79030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.uge", {{ 0x7d030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.ult", {{ 0x7f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.une", {{ 0x7b030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompd.oeq", {{ 0xfa030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.oge", {{ 0xfe030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.olt", {{ 0xfc030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.one", {{ 0xf8030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.ueq", {{ 0xf9030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.uge", {{ 0xfd030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.ult", {{ 0xff030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompd.une", {{ 0xfb030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPD_floatcomp_registerW_registerZ_w032_double */
	{"fcompnhq.oeq", {{ 0x7a031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.oge", {{ 0x7e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.olt", {{ 0x7c031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.one", {{ 0x78031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.ueq", {{ 0x79031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.uge", {{ 0x7d031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.ult", {{ 0x7f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnhq.une", {{ 0x7b031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNHQ_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.oeq", {{ 0x72031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.oge", {{ 0x76031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.olt", {{ 0x74031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.one", {{ 0x70031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.ueq", {{ 0x71031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.uge", {{ 0x75031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.ult", {{ 0x77031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompnwp.une", {{ 0x73031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPNWP_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.oeq", {{ 0x72030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.oge", {{ 0x76030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.olt", {{ 0x74030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.one", {{ 0x70030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.ueq", {{ 0x71030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.uge", {{ 0x75030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.ult", {{ 0x77030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.une", {{ 0x73030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_registerY_simple */
	{"fcompw.oeq", {{ 0xf2030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.oge", {{ 0xf6030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.olt", {{ 0xf4030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.one", {{ 0xf0030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.ueq", {{ 0xf1030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.uge", {{ 0xf5030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.ult", {{ 0xf7030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fcompw.une", {{ 0xf3030000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FCOMPW_floatcomp_registerW_registerZ_w032_double */
	{"fdot2wdp.rd", {{ 0x5c072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rd.s", {{ 0x5c07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rna", {{ 0x5c074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rna.s", {{ 0x5c07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rnz", {{ 0x5c075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rnz.s", {{ 0x5c07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rn", {{ 0x5c070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rn.s", {{ 0x5c078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ro", {{ 0x5c076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ro.s", {{ 0x5c07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ru", {{ 0x5c071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.ru.s", {{ 0x5c079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rz", {{ 0x5c073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.rz.s", {{ 0x5c07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp", {{ 0x5c077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wdp.s", {{ 0x5c07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wd.rd", {{ 0x5d012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rd.s", {{ 0x5d01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rna", {{ 0x5d014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rna.s", {{ 0x5d01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rnz", {{ 0x5d015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rnz.s", {{ 0x5d01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rn", {{ 0x5d010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rn.s", {{ 0x5d018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ro", {{ 0x5d016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ro.s", {{ 0x5d01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ru", {{ 0x5d011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.ru.s", {{ 0x5d019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rz", {{ 0x5d013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.rz.s", {{ 0x5d01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd", {{ 0x5d017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wd.s", {{ 0x5d01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2wzp.rd", {{ 0x5d072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rd.s", {{ 0x5d07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rna", {{ 0x5d074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rna.s", {{ 0x5d07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rnz", {{ 0x5d075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rnz.s", {{ 0x5d07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rn", {{ 0x5d070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rn.s", {{ 0x5d078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ro", {{ 0x5d076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ro.s", {{ 0x5d07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ru", {{ 0x5d071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.ru.s", {{ 0x5d079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rz", {{ 0x5d073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.rz.s", {{ 0x5d07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp", {{ 0x5d077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2wzp.s", {{ 0x5d07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2WZP_rounding_silent_registerM_registerP_registerO_simple */
	{"fdot2w.rd", {{ 0x5c012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rd.s", {{ 0x5c01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rna", {{ 0x5c014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rna.s", {{ 0x5c01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rnz", {{ 0x5c015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rnz.s", {{ 0x5c01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rn", {{ 0x5c010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rn.s", {{ 0x5c018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ro", {{ 0x5c016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ro.s", {{ 0x5c01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ru", {{ 0x5c011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.ru.s", {{ 0x5c019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rz", {{ 0x5c013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.rz.s", {{ 0x5c01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w", {{ 0x5c017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fdot2w.s", {{ 0x5c01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FDOT2W_rounding_silent_registerW_registerZ_registerY_simple */
	{"fence", {{ 0x3fcd0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {0}, "", ""}, /* Opcode-v2-FENCE_simple */
	{"ffdmaswp.rd", {{ 0x47022000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rd.s", {{ 0x4702a000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rna", {{ 0x47024000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rna.s", {{ 0x4702c000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rnz", {{ 0x47025000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rnz.s", {{ 0x4702d000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rn", {{ 0x47020000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rn.s", {{ 0x47028000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.ro", {{ 0x47026000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.ro.s", {{ 0x4702e000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.ru", {{ 0x47021000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.ru.s", {{ 0x47029000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rz", {{ 0x47023000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.rz.s", {{ 0x4702b000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp", {{ 0x47027000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswp.s", {{ 0x4702f000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmaswq.rd", {{ 0x47022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rd.s", {{ 0x4702a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rna", {{ 0x47024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rna.s", {{ 0x4702c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rnz", {{ 0x47025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rnz.s", {{ 0x4702d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rn", {{ 0x47020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rn.s", {{ 0x47028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.ro", {{ 0x47026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.ro.s", {{ 0x4702e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.ru", {{ 0x47021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.ru.s", {{ 0x47029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rz", {{ 0x47023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.rz.s", {{ 0x4702b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq", {{ 0x47027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaswq.s", {{ 0x4702f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmasw.rd", {{ 0x47012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rd.s", {{ 0x4701a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rna", {{ 0x47014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rna.s", {{ 0x4701c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rnz", {{ 0x47015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rnz.s", {{ 0x4701d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rn", {{ 0x47010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rn.s", {{ 0x47018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.ro", {{ 0x47016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.ro.s", {{ 0x4701e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.ru", {{ 0x47011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.ru.s", {{ 0x47019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rz", {{ 0x47013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.rz.s", {{ 0x4701b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw", {{ 0x47017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmasw.s", {{ 0x4701f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMASW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmawp.rd", {{ 0x42022000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rd.s", {{ 0x4202a000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rna", {{ 0x42024000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rna.s", {{ 0x4202c000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rnz", {{ 0x42025000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rnz.s", {{ 0x4202d000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rn", {{ 0x42020000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rn.s", {{ 0x42028000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.ro", {{ 0x42026000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.ro.s", {{ 0x4202e000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.ru", {{ 0x42021000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.ru.s", {{ 0x42029000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rz", {{ 0x42023000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.rz.s", {{ 0x4202b000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp", {{ 0x42027000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawp.s", {{ 0x4202f000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmawq.rd", {{ 0x42022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rd.s", {{ 0x4202a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rna", {{ 0x42024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rna.s", {{ 0x4202c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rnz", {{ 0x42025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rnz.s", {{ 0x4202d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rn", {{ 0x42020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rn.s", {{ 0x42028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.ro", {{ 0x42026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.ro.s", {{ 0x4202e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.ru", {{ 0x42021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.ru.s", {{ 0x42029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rz", {{ 0x42023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.rz.s", {{ 0x4202b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq", {{ 0x42027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmawq.s", {{ 0x4202f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmaw.rd", {{ 0x42012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rd.s", {{ 0x4201a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rna", {{ 0x42014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rna.s", {{ 0x4201c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rnz", {{ 0x42015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rnz.s", {{ 0x4201d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rn", {{ 0x42010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rn.s", {{ 0x42018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.ro", {{ 0x42016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.ro.s", {{ 0x4201e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.ru", {{ 0x42011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.ru.s", {{ 0x42019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rz", {{ 0x42013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.rz.s", {{ 0x4201b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw", {{ 0x42017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmaw.s", {{ 0x4201f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdawp.rd", {{ 0x44022000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rd.s", {{ 0x4402a000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rna", {{ 0x44024000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rna.s", {{ 0x4402c000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rnz", {{ 0x44025000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rnz.s", {{ 0x4402d000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rn", {{ 0x44020000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rn.s", {{ 0x44028000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.ro", {{ 0x44026000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.ro.s", {{ 0x4402e000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.ru", {{ 0x44021000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.ru.s", {{ 0x44029000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rz", {{ 0x44023000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.rz.s", {{ 0x4402b000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp", {{ 0x44027000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawp.s", {{ 0x4402f000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdawq.rd", {{ 0x44022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rd.s", {{ 0x4402a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rna", {{ 0x44024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rna.s", {{ 0x4402c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rnz", {{ 0x44025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rnz.s", {{ 0x4402d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rn", {{ 0x44020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rn.s", {{ 0x44028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.ro", {{ 0x44026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.ro.s", {{ 0x4402e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.ru", {{ 0x44021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.ru.s", {{ 0x44029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rz", {{ 0x44023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.rz.s", {{ 0x4402b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq", {{ 0x44027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdawq.s", {{ 0x4402f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdaw.rd", {{ 0x44012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rd.s", {{ 0x4401a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rna", {{ 0x44014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rna.s", {{ 0x4401c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rnz", {{ 0x44015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rnz.s", {{ 0x4401d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rn", {{ 0x44010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rn.s", {{ 0x44018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.ro", {{ 0x44016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.ro.s", {{ 0x4401e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.ru", {{ 0x44011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.ru.s", {{ 0x44019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rz", {{ 0x44013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.rz.s", {{ 0x4401b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw", {{ 0x44017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdaw.s", {{ 0x4401f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdswp.rd", {{ 0x46022000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rd.s", {{ 0x4602a000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rna", {{ 0x46024000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rna.s", {{ 0x4602c000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rnz", {{ 0x46025000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rnz.s", {{ 0x4602d000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rn", {{ 0x46020000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rn.s", {{ 0x46028000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.ro", {{ 0x46026000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.ro.s", {{ 0x4602e000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.ru", {{ 0x46021000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.ru.s", {{ 0x46029000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rz", {{ 0x46023000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.rz.s", {{ 0x4602b000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp", {{ 0x46027000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswp.s", {{ 0x4602f000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmdswq.rd", {{ 0x46022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rd.s", {{ 0x4602a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rna", {{ 0x46024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rna.s", {{ 0x4602c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rnz", {{ 0x46025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rnz.s", {{ 0x4602d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rn", {{ 0x46020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rn.s", {{ 0x46028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.ro", {{ 0x46026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.ro.s", {{ 0x4602e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.ru", {{ 0x46021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.ru.s", {{ 0x46029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rz", {{ 0x46023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.rz.s", {{ 0x4602b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq", {{ 0x46027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdswq.s", {{ 0x4602f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmdsw.rd", {{ 0x46012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rd.s", {{ 0x4601a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rna", {{ 0x46014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rna.s", {{ 0x4601c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rnz", {{ 0x46015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rnz.s", {{ 0x4601d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rn", {{ 0x46010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rn.s", {{ 0x46018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.ro", {{ 0x46016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.ro.s", {{ 0x4601e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.ru", {{ 0x46011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.ru.s", {{ 0x46019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rz", {{ 0x46013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.rz.s", {{ 0x4601b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw", {{ 0x46017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmdsw.s", {{ 0x4601f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMDSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsawp.rd", {{ 0x45022000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rd.s", {{ 0x4502a000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rna", {{ 0x45024000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rna.s", {{ 0x4502c000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rnz", {{ 0x45025000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rnz.s", {{ 0x4502d000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rn", {{ 0x45020000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rn.s", {{ 0x45028000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.ro", {{ 0x45026000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.ro.s", {{ 0x4502e000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.ru", {{ 0x45021000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.ru.s", {{ 0x45029000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rz", {{ 0x45023000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.rz.s", {{ 0x4502b000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp", {{ 0x45027000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawp.s", {{ 0x4502f000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmsawq.rd", {{ 0x45022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rd.s", {{ 0x4502a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rna", {{ 0x45024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rna.s", {{ 0x4502c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rnz", {{ 0x45025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rnz.s", {{ 0x4502d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rn", {{ 0x45020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rn.s", {{ 0x45028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.ro", {{ 0x45026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.ro.s", {{ 0x4502e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.ru", {{ 0x45021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.ru.s", {{ 0x45029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rz", {{ 0x45023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.rz.s", {{ 0x4502b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq", {{ 0x45027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsawq.s", {{ 0x4502f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsaw.rd", {{ 0x45012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rd.s", {{ 0x4501a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rna", {{ 0x45014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rna.s", {{ 0x4501c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rnz", {{ 0x45015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rnz.s", {{ 0x4501d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rn", {{ 0x45010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rn.s", {{ 0x45018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.ro", {{ 0x45016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.ro.s", {{ 0x4501e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.ru", {{ 0x45011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.ru.s", {{ 0x45019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rz", {{ 0x45013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.rz.s", {{ 0x4501b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw", {{ 0x45017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsaw.s", {{ 0x4501f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmswp.rd", {{ 0x43022000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rd.s", {{ 0x4302a000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rna", {{ 0x43024000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rna.s", {{ 0x4302c000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rnz", {{ 0x43025000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rnz.s", {{ 0x4302d000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rn", {{ 0x43020000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rn.s", {{ 0x43028000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.ro", {{ 0x43026000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.ro.s", {{ 0x4302e000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.ru", {{ 0x43021000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.ru.s", {{ 0x43029000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rz", {{ 0x43023000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.rz.s", {{ 0x4302b000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp", {{ 0x43027000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswp.s", {{ 0x4302f000, 0x7f03f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWP_rounding_silent_registerW_registerP_registerO_simple */
	{"ffdmswq.rd", {{ 0x43022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rd.s", {{ 0x4302a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rna", {{ 0x43024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rna.s", {{ 0x4302c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rnz", {{ 0x43025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rnz.s", {{ 0x4302d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rn", {{ 0x43020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rn.s", {{ 0x43028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.ro", {{ 0x43026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.ro.s", {{ 0x4302e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.ru", {{ 0x43021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.ru.s", {{ 0x43029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rz", {{ 0x43023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.rz.s", {{ 0x4302b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq", {{ 0x43027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmswq.s", {{ 0x4302f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerq_opnd, &kv3_registerr_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSWQ_rounding_silent_registerM_registerQ_registerR_simple */
	{"ffdmsw.rd", {{ 0x43012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rd.s", {{ 0x4301a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rna", {{ 0x43014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rna.s", {{ 0x4301c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rnz", {{ 0x43015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rnz.s", {{ 0x4301d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rn", {{ 0x43010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rn.s", {{ 0x43018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.ro", {{ 0x43016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.ro.s", {{ 0x4301e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.ru", {{ 0x43011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.ru.s", {{ 0x43019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rz", {{ 0x43013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.rz.s", {{ 0x4301b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw", {{ 0x43017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffdmsw.s", {{ 0x4301f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFDMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rd", {{ 0x50012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rd.s", {{ 0x5001a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rna", {{ 0x50014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rna.s", {{ 0x5001c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rnz", {{ 0x50015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rnz.s", {{ 0x5001d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rn", {{ 0x50010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rn.s", {{ 0x50018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ro", {{ 0x50016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ro.s", {{ 0x5001e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ru", {{ 0x50011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.ru.s", {{ 0x50019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rz", {{ 0x50013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.rz.s", {{ 0x5001b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad", {{ 0x50017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmad.s", {{ 0x5001f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaho.rd", {{ 0x5a032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rd.s", {{ 0x5a03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rna", {{ 0x5a034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rna.s", {{ 0x5a03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rnz", {{ 0x5a035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rnz.s", {{ 0x5a03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rn", {{ 0x5a030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rn.s", {{ 0x5a038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.ro", {{ 0x5a036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.ro.s", {{ 0x5a03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.ru", {{ 0x5a031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.ru.s", {{ 0x5a039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rz", {{ 0x5a033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.rz.s", {{ 0x5a03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho", {{ 0x5a037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaho.s", {{ 0x5a03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmahq.rd", {{ 0x53012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rd.s", {{ 0x5301a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rna", {{ 0x53014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rna.s", {{ 0x5301c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rnz", {{ 0x53015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rnz.s", {{ 0x5301d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rn", {{ 0x53010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rn.s", {{ 0x53018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ro", {{ 0x53016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ro.s", {{ 0x5301e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ru", {{ 0x53011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.ru.s", {{ 0x53019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rz", {{ 0x53013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.rz.s", {{ 0x5301b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq", {{ 0x53017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahq.s", {{ 0x5301f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahwq.rd", {{ 0x51032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rd.s", {{ 0x5103a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rna", {{ 0x51034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rna.s", {{ 0x5103c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rnz", {{ 0x51035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rnz.s", {{ 0x5103d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rn", {{ 0x51030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rn.s", {{ 0x51038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ro", {{ 0x51036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ro.s", {{ 0x5103e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ru", {{ 0x51031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.ru.s", {{ 0x51039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rz", {{ 0x51033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.rz.s", {{ 0x5103b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq", {{ 0x51037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahwq.s", {{ 0x5103f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmahw.rd", {{ 0x58022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rd.s", {{ 0x5802a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rna", {{ 0x58024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rna.s", {{ 0x5802c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rnz", {{ 0x58025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rnz.s", {{ 0x5802d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rn", {{ 0x58020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rn.s", {{ 0x58028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ro", {{ 0x58026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ro.s", {{ 0x5802e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ru", {{ 0x58021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.ru.s", {{ 0x58029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rz", {{ 0x58023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.rz.s", {{ 0x5802b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw", {{ 0x58027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmahw.s", {{ 0x5802f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawcp.c.rd", {{ 0x49022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rd.s", {{ 0x4902a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rna", {{ 0x49024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rna.s", {{ 0x4902c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rnz", {{ 0x49025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rnz.s", {{ 0x4902d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rn", {{ 0x49020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rn.s", {{ 0x49028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.ro", {{ 0x49026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.ro.s", {{ 0x4902e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.ru", {{ 0x49021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.ru.s", {{ 0x49029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rz", {{ 0x49023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.rz.s", {{ 0x4902b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c", {{ 0x49027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.c.s", {{ 0x4902f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rd", {{ 0x48022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rd.s", {{ 0x4802a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rna", {{ 0x48024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rna.s", {{ 0x4802c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rnz", {{ 0x48025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rnz.s", {{ 0x4802d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rn", {{ 0x48020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rn.s", {{ 0x48028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.ro", {{ 0x48026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.ro.s", {{ 0x4802e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.ru", {{ 0x48021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.ru.s", {{ 0x48029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rz", {{ 0x48023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.rz.s", {{ 0x4802b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp", {{ 0x48027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawcp.s", {{ 0x4802f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawc.c.rd", {{ 0x49012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rd.s", {{ 0x4901a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rna", {{ 0x49014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rna.s", {{ 0x4901c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rnz", {{ 0x49015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rnz.s", {{ 0x4901d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rn", {{ 0x49010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rn.s", {{ 0x49018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.ro", {{ 0x49016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.ro.s", {{ 0x4901e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.ru", {{ 0x49011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.ru.s", {{ 0x49019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rz", {{ 0x49013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.rz.s", {{ 0x4901b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c", {{ 0x49017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.c.s", {{ 0x4901f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rd", {{ 0x48012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rd.s", {{ 0x4801a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rna", {{ 0x48014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rna.s", {{ 0x4801c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rnz", {{ 0x48015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rnz.s", {{ 0x4801d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rn", {{ 0x48010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rn.s", {{ 0x48018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.ro", {{ 0x48016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.ro.s", {{ 0x4801e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.ru", {{ 0x48011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.ru.s", {{ 0x48019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rz", {{ 0x48013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.rz.s", {{ 0x4801b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc", {{ 0x48017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawc.s", {{ 0x4801f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawdp.rd", {{ 0x50032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rd.s", {{ 0x5003a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rna", {{ 0x50034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rna.s", {{ 0x5003c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rnz", {{ 0x50035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rnz.s", {{ 0x5003d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rn", {{ 0x50030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rn.s", {{ 0x50038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ro", {{ 0x50036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ro.s", {{ 0x5003e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ru", {{ 0x50031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.ru.s", {{ 0x50039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rz", {{ 0x50033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.rz.s", {{ 0x5003b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp", {{ 0x50037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawdp.s", {{ 0x5003f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmawd.rd", {{ 0x51012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rd.s", {{ 0x5101a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rna", {{ 0x51014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rna.s", {{ 0x5101c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rnz", {{ 0x51015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rnz.s", {{ 0x5101d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rn", {{ 0x51010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rn.s", {{ 0x51018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ro", {{ 0x51016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ro.s", {{ 0x5101e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ru", {{ 0x51011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.ru.s", {{ 0x51019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rz", {{ 0x51013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.rz.s", {{ 0x5101b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd", {{ 0x51017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawd.s", {{ 0x5101f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rd", {{ 0x52012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rd.s", {{ 0x5201a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rna", {{ 0x52014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rna.s", {{ 0x5201c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rnz", {{ 0x52015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rnz.s", {{ 0x5201d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rn", {{ 0x52010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rn.s", {{ 0x52018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ro", {{ 0x52016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ro.s", {{ 0x5201e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ru", {{ 0x52011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.ru.s", {{ 0x52019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rz", {{ 0x52013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.rz.s", {{ 0x5201b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp", {{ 0x52017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawp.s", {{ 0x5201f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmawq.rd", {{ 0x58032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rd.s", {{ 0x5803a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rna", {{ 0x58034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rna.s", {{ 0x5803c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rnz", {{ 0x58035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rnz.s", {{ 0x5803d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rn", {{ 0x58030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rn.s", {{ 0x58038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.ro", {{ 0x58036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.ro.s", {{ 0x5803e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.ru", {{ 0x58031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.ru.s", {{ 0x58039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rz", {{ 0x58033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.rz.s", {{ 0x5803b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq", {{ 0x58037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmawq.s", {{ 0x5803f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmaw.rd", {{ 0x59022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rd.s", {{ 0x5902a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rna", {{ 0x59024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rna.s", {{ 0x5902c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rnz", {{ 0x59025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rnz.s", {{ 0x5902d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rn", {{ 0x59020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rn.s", {{ 0x59028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ro", {{ 0x59026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ro.s", {{ 0x5902e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ru", {{ 0x59021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.ru.s", {{ 0x59029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rz", {{ 0x59023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.rz.s", {{ 0x5902b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw", {{ 0x59027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmaw.s", {{ 0x5902f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMAW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rd", {{ 0x54012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rd.s", {{ 0x5401a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rna", {{ 0x54014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rna.s", {{ 0x5401c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rnz", {{ 0x54015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rnz.s", {{ 0x5401d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rn", {{ 0x54010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rn.s", {{ 0x54018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ro", {{ 0x54016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ro.s", {{ 0x5401e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ru", {{ 0x54011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.ru.s", {{ 0x54019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rz", {{ 0x54013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.rz.s", {{ 0x5401b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd", {{ 0x54017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsd.s", {{ 0x5401f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsho.rd", {{ 0x5b032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rd.s", {{ 0x5b03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rna", {{ 0x5b034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rna.s", {{ 0x5b03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rnz", {{ 0x5b035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rnz.s", {{ 0x5b03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rn", {{ 0x5b030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rn.s", {{ 0x5b038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.ro", {{ 0x5b036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.ro.s", {{ 0x5b03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.ru", {{ 0x5b031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.ru.s", {{ 0x5b039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rz", {{ 0x5b033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.rz.s", {{ 0x5b03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho", {{ 0x5b037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsho.s", {{ 0x5b03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHO_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmshq.rd", {{ 0x57012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rd.s", {{ 0x5701a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rna", {{ 0x57014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rna.s", {{ 0x5701c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rnz", {{ 0x57015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rnz.s", {{ 0x5701d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rn", {{ 0x57010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rn.s", {{ 0x57018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ro", {{ 0x57016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ro.s", {{ 0x5701e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ru", {{ 0x57011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.ru.s", {{ 0x57019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rz", {{ 0x57013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.rz.s", {{ 0x5701b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq", {{ 0x57017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshq.s", {{ 0x5701f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshwq.rd", {{ 0x53032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rd.s", {{ 0x5303a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rna", {{ 0x53034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rna.s", {{ 0x5303c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rnz", {{ 0x53035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rnz.s", {{ 0x5303d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rn", {{ 0x53030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rn.s", {{ 0x53038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ro", {{ 0x53036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ro.s", {{ 0x5303e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ru", {{ 0x53031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.ru.s", {{ 0x53039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rz", {{ 0x53033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.rz.s", {{ 0x5303b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq", {{ 0x53037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshwq.s", {{ 0x5303f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmshw.rd", {{ 0x5a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rd.s", {{ 0x5a02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rna", {{ 0x5a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rna.s", {{ 0x5a02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rnz", {{ 0x5a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rnz.s", {{ 0x5a02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rn", {{ 0x5a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rn.s", {{ 0x5a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ro", {{ 0x5a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ro.s", {{ 0x5a02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ru", {{ 0x5a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.ru.s", {{ 0x5a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rz", {{ 0x5a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.rz.s", {{ 0x5a02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw", {{ 0x5a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmshw.s", {{ 0x5a02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswcp.c.rd", {{ 0x4b022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rd.s", {{ 0x4b02a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rna", {{ 0x4b024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rna.s", {{ 0x4b02c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rnz", {{ 0x4b025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rnz.s", {{ 0x4b02d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rn", {{ 0x4b020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rn.s", {{ 0x4b028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.ro", {{ 0x4b026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.ro.s", {{ 0x4b02e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.ru", {{ 0x4b021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.ru.s", {{ 0x4b029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rz", {{ 0x4b023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.rz.s", {{ 0x4b02b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c", {{ 0x4b027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.c.s", {{ 0x4b02f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rd", {{ 0x4a022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rd.s", {{ 0x4a02a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rna", {{ 0x4a024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rna.s", {{ 0x4a02c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rnz", {{ 0x4a025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rnz.s", {{ 0x4a02d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rn", {{ 0x4a020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rn.s", {{ 0x4a028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.ro", {{ 0x4a026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.ro.s", {{ 0x4a02e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.ru", {{ 0x4a021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.ru.s", {{ 0x4a029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rz", {{ 0x4a023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.rz.s", {{ 0x4a02b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp", {{ 0x4a027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswcp.s", {{ 0x4a02f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswc.c.rd", {{ 0x4b012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rd.s", {{ 0x4b01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rna", {{ 0x4b014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rna.s", {{ 0x4b01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rnz", {{ 0x4b015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rnz.s", {{ 0x4b01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rn", {{ 0x4b010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rn.s", {{ 0x4b018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.ro", {{ 0x4b016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.ro.s", {{ 0x4b01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.ru", {{ 0x4b011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.ru.s", {{ 0x4b019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rz", {{ 0x4b013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.rz.s", {{ 0x4b01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c", {{ 0x4b017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.c.s", {{ 0x4b01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rd", {{ 0x4a012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rd.s", {{ 0x4a01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rna", {{ 0x4a014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rna.s", {{ 0x4a01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rnz", {{ 0x4a015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rnz.s", {{ 0x4a01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rn", {{ 0x4a010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rn.s", {{ 0x4a018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.ro", {{ 0x4a016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.ro.s", {{ 0x4a01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.ru", {{ 0x4a011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.ru.s", {{ 0x4a019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rz", {{ 0x4a013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.rz.s", {{ 0x4a01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc", {{ 0x4a017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswc.s", {{ 0x4a01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswdp.rd", {{ 0x52032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rd.s", {{ 0x5203a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rna", {{ 0x52034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rna.s", {{ 0x5203c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rnz", {{ 0x52035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rnz.s", {{ 0x5203d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rn", {{ 0x52030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rn.s", {{ 0x52038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ro", {{ 0x52036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ro.s", {{ 0x5203e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ru", {{ 0x52031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.ru.s", {{ 0x52039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rz", {{ 0x52033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.rz.s", {{ 0x5203b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp", {{ 0x52037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswdp.s", {{ 0x5203f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"ffmswd.rd", {{ 0x55012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rd.s", {{ 0x5501a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rna", {{ 0x55014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rna.s", {{ 0x5501c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rnz", {{ 0x55015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rnz.s", {{ 0x5501d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rn", {{ 0x55010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rn.s", {{ 0x55018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ro", {{ 0x55016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ro.s", {{ 0x5501e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ru", {{ 0x55011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.ru.s", {{ 0x55019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rz", {{ 0x55013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.rz.s", {{ 0x5501b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd", {{ 0x55017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswd.s", {{ 0x5501f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rd", {{ 0x56012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rd.s", {{ 0x5601a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rna", {{ 0x56014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rna.s", {{ 0x5601c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rnz", {{ 0x56015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rnz.s", {{ 0x5601d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rn", {{ 0x56010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rn.s", {{ 0x56018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ro", {{ 0x56016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ro.s", {{ 0x5601e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ru", {{ 0x56011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.ru.s", {{ 0x56019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rz", {{ 0x56013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.rz.s", {{ 0x5601b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp", {{ 0x56017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswp.s", {{ 0x5601f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmswq.rd", {{ 0x59032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rd.s", {{ 0x5903a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rna", {{ 0x59034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rna.s", {{ 0x5903c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rnz", {{ 0x59035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rnz.s", {{ 0x5903d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rn", {{ 0x59030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rn.s", {{ 0x59038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.ro", {{ 0x59036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.ro.s", {{ 0x5903e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.ru", {{ 0x59031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.ru.s", {{ 0x59039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rz", {{ 0x59033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.rz.s", {{ 0x5903b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq", {{ 0x59037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmswq.s", {{ 0x5903f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"ffmsw.rd", {{ 0x5b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rd.s", {{ 0x5b02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rna", {{ 0x5b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rna.s", {{ 0x5b02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rnz", {{ 0x5b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rnz.s", {{ 0x5b02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rn", {{ 0x5b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rn.s", {{ 0x5b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ro", {{ 0x5b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ro.s", {{ 0x5b02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ru", {{ 0x5b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.ru.s", {{ 0x5b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rz", {{ 0x5b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.rz.s", {{ 0x5b02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw", {{ 0x5b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"ffmsw.s", {{ 0x5b02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FFMSW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fixedd.rd", {{ 0x46032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rd.s", {{ 0x4603a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rna", {{ 0x46034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rna.s", {{ 0x4603c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rnz", {{ 0x46035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rnz.s", {{ 0x4603d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rn", {{ 0x46030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rn.s", {{ 0x46038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ro", {{ 0x46036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ro.s", {{ 0x4603e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ru", {{ 0x46031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.ru.s", {{ 0x46039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rz", {{ 0x46033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.rz.s", {{ 0x4603b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd", {{ 0x46037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedd.s", {{ 0x4603f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rd", {{ 0x47032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rd.s", {{ 0x4703a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rna", {{ 0x47034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rna.s", {{ 0x4703c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rnz", {{ 0x47035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rnz.s", {{ 0x4703d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rn", {{ 0x47030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rn.s", {{ 0x47038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ro", {{ 0x47036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ro.s", {{ 0x4703e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ru", {{ 0x47031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.ru.s", {{ 0x47039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rz", {{ 0x47033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.rz.s", {{ 0x4703b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud", {{ 0x47037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedud.s", {{ 0x4703f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUD_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rd", {{ 0x4f032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rd.s", {{ 0x4f03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rna", {{ 0x4f034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rna.s", {{ 0x4f03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rnz", {{ 0x4f035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rnz.s", {{ 0x4f03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rn", {{ 0x4f030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rn.s", {{ 0x4f038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ro", {{ 0x4f036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ro.s", {{ 0x4f03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ru", {{ 0x4f031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.ru.s", {{ 0x4f039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rz", {{ 0x4f033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.rz.s", {{ 0x4f03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp", {{ 0x4f037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduwp.s", {{ 0x4f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rd", {{ 0x4b032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rd.s", {{ 0x4b03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rna", {{ 0x4b034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rna.s", {{ 0x4b03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rnz", {{ 0x4b035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rnz.s", {{ 0x4b03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rn", {{ 0x4b030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rn.s", {{ 0x4b038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ro", {{ 0x4b036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ro.s", {{ 0x4b03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ru", {{ 0x4b031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.ru.s", {{ 0x4b039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rz", {{ 0x4b033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.rz.s", {{ 0x4b03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw", {{ 0x4b037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixeduw.s", {{ 0x4b03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDUW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rd", {{ 0x4e032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rd.s", {{ 0x4e03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rna", {{ 0x4e034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rna.s", {{ 0x4e03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rnz", {{ 0x4e035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rnz.s", {{ 0x4e03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rn", {{ 0x4e030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rn.s", {{ 0x4e038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ro", {{ 0x4e036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ro.s", {{ 0x4e03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ru", {{ 0x4e031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.ru.s", {{ 0x4e039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rz", {{ 0x4e033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.rz.s", {{ 0x4e03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp", {{ 0x4e037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedwp.s", {{ 0x4e03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDWP_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rd", {{ 0x4a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rd.s", {{ 0x4a03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rna", {{ 0x4a034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rna.s", {{ 0x4a03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rnz", {{ 0x4a035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rnz.s", {{ 0x4a03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rn", {{ 0x4a030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rn.s", {{ 0x4a038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ro", {{ 0x4a036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ro.s", {{ 0x4a03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ru", {{ 0x4a031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.ru.s", {{ 0x4a039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rz", {{ 0x4a033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.rz.s", {{ 0x4a03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw", {{ 0x4a037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"fixedw.s", {{ 0x4a03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FIXEDW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rd", {{ 0x44032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rd.s", {{ 0x4403a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rna", {{ 0x44034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rna.s", {{ 0x4403c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rnz", {{ 0x44035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rnz.s", {{ 0x4403d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rn", {{ 0x44030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rn.s", {{ 0x44038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ro", {{ 0x44036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ro.s", {{ 0x4403e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ru", {{ 0x44031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.ru.s", {{ 0x44039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rz", {{ 0x44033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.rz.s", {{ 0x4403b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd", {{ 0x44037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatd.s", {{ 0x4403f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rd", {{ 0x45032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rd.s", {{ 0x4503a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rna", {{ 0x45034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rna.s", {{ 0x4503c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rnz", {{ 0x45035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rnz.s", {{ 0x4503d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rn", {{ 0x45030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rn.s", {{ 0x45038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ro", {{ 0x45036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ro.s", {{ 0x4503e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ru", {{ 0x45031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.ru.s", {{ 0x45039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rz", {{ 0x45033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.rz.s", {{ 0x4503b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud", {{ 0x45037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatud.s", {{ 0x4503f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUD_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rd", {{ 0x4d032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rd.s", {{ 0x4d03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rna", {{ 0x4d034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rna.s", {{ 0x4d03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rnz", {{ 0x4d035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rnz.s", {{ 0x4d03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rn", {{ 0x4d030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rn.s", {{ 0x4d038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ro", {{ 0x4d036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ro.s", {{ 0x4d03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ru", {{ 0x4d031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.ru.s", {{ 0x4d039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rz", {{ 0x4d033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.rz.s", {{ 0x4d03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp", {{ 0x4d037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuwp.s", {{ 0x4d03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rd", {{ 0x49032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rd.s", {{ 0x4903a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rna", {{ 0x49034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rna.s", {{ 0x4903c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rnz", {{ 0x49035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rnz.s", {{ 0x4903d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rn", {{ 0x49030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rn.s", {{ 0x49038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ro", {{ 0x49036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ro.s", {{ 0x4903e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ru", {{ 0x49031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.ru.s", {{ 0x49039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rz", {{ 0x49033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.rz.s", {{ 0x4903b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw", {{ 0x49037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatuw.s", {{ 0x4903f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATUW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rd", {{ 0x4c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rd.s", {{ 0x4c03a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rna", {{ 0x4c034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rna.s", {{ 0x4c03c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rnz", {{ 0x4c035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rnz.s", {{ 0x4c03d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rn", {{ 0x4c030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rn.s", {{ 0x4c038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ro", {{ 0x4c036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ro.s", {{ 0x4c03e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ru", {{ 0x4c031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.ru.s", {{ 0x4c039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rz", {{ 0x4c033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.rz.s", {{ 0x4c03b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp", {{ 0x4c037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatwp.s", {{ 0x4c03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATWP_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rd", {{ 0x48032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rd.s", {{ 0x4803a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rna", {{ 0x48034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rna.s", {{ 0x4803c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rnz", {{ 0x48035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rnz.s", {{ 0x4803d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rn", {{ 0x48030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rn.s", {{ 0x48038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ro", {{ 0x48036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ro.s", {{ 0x4803e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ru", {{ 0x48031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.ru.s", {{ 0x48039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rz", {{ 0x48033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.rz.s", {{ 0x4803b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw", {{ 0x48037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"floatw.s", {{ 0x4803f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FLOATW_rounding_silent_registerW_registerZ_u006_simple */
	{"fmaxd", {{ 0x71038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMAXD_registerW_registerZ_registerY_simple */
	{"fmaxhq", {{ 0x77038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMAXHQ_registerW_registerZ_registerY_simple */
	{"fmaxwp", {{ 0x75038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMAXWP_registerW_registerZ_registerY_simple */
	{"fmaxw", {{ 0x73038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMAXW_registerW_registerZ_registerY_simple */
	{"fmind", {{ 0x70038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMIND_registerW_registerZ_registerY_simple */
	{"fminhq", {{ 0x76038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMINHQ_registerW_registerZ_registerY_simple */
	{"fminwp", {{ 0x74038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMINWP_registerW_registerZ_registerY_simple */
	{"fminw", {{ 0x72038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMINW_registerW_registerZ_registerY_simple */
	{"fmm212w.rd", {{ 0x4c012000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rd", {{ 0x4c012000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rd.s", {{ 0x4c01a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rd.s", {{ 0x4c01a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rna", {{ 0x4c014000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rna", {{ 0x4c014000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rna.s", {{ 0x4c01c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rna.s", {{ 0x4c01c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rnz", {{ 0x4c015000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rnz", {{ 0x4c015000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rnz.s", {{ 0x4c01d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rnz.s", {{ 0x4c01d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rn", {{ 0x4c010000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rn", {{ 0x4c010000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rn.s", {{ 0x4c018000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rn.s", {{ 0x4c018000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ro", {{ 0x4c016000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ro", {{ 0x4c016000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ro.s", {{ 0x4c01e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ro.s", {{ 0x4c01e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ru", {{ 0x4c011000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ru", {{ 0x4c011000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.ru.s", {{ 0x4c019000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.ru.s", {{ 0x4c019000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rz", {{ 0x4c013000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rz", {{ 0x4c013000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.rz.s", {{ 0x4c01b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.rz.s", {{ 0x4c01b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w", {{ 0x4c017000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq", {{ 0x4c017000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm212w.s", {{ 0x4c01f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2wq.s", {{ 0x4c01f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm222w.nt.rd", {{ 0x4c062000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rd.s", {{ 0x4c06a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rna", {{ 0x4c064000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rna.s", {{ 0x4c06c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rnz", {{ 0x4c065000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rnz.s", {{ 0x4c06d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rn", {{ 0x4c060000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rn.s", {{ 0x4c068000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.ro", {{ 0x4c066000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.ro.s", {{ 0x4c06e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.ru", {{ 0x4c061000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.ru.s", {{ 0x4c069000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rz", {{ 0x4c063000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.rz.s", {{ 0x4c06b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt", {{ 0x4c067000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.nt.s", {{ 0x4c06f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rd", {{ 0x4c022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rd.s", {{ 0x4c02a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rna", {{ 0x4c024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rna.s", {{ 0x4c02c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rnz", {{ 0x4c025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rnz.s", {{ 0x4c02d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rn", {{ 0x4c020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rn.s", {{ 0x4c028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.ro", {{ 0x4c026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.ro.s", {{ 0x4c02e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.ru", {{ 0x4c021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.ru.s", {{ 0x4c029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rz", {{ 0x4c023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.rz.s", {{ 0x4c02b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn", {{ 0x4c027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tn.s", {{ 0x4c02f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rd", {{ 0x4c062040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rd.s", {{ 0x4c06a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rna", {{ 0x4c064040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rna.s", {{ 0x4c06c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rnz", {{ 0x4c065040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rnz.s", {{ 0x4c06d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rn", {{ 0x4c060040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rn.s", {{ 0x4c068040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.ro", {{ 0x4c066040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.ro.s", {{ 0x4c06e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.ru", {{ 0x4c061040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.ru.s", {{ 0x4c069040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rz", {{ 0x4c063040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.rz.s", {{ 0x4c06b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt", {{ 0x4c067040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.tt.s", {{ 0x4c06f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rd", {{ 0x4c022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rd.s", {{ 0x4c02a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rna", {{ 0x4c024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rna.s", {{ 0x4c02c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rnz", {{ 0x4c025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rnz.s", {{ 0x4c02d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rn", {{ 0x4c020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rn.s", {{ 0x4c028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.ro", {{ 0x4c026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.ro.s", {{ 0x4c02e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.ru", {{ 0x4c021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.ru.s", {{ 0x4c029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rz", {{ 0x4c023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.rz.s", {{ 0x4c02b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w", {{ 0x4c027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmm222w.s", {{ 0x4c02f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMM222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma212w.rd", {{ 0x4e012000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rd", {{ 0x4e012000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rd.s", {{ 0x4e01a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rd.s", {{ 0x4e01a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rna", {{ 0x4e014000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rna", {{ 0x4e014000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rna.s", {{ 0x4e01c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rna.s", {{ 0x4e01c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rnz", {{ 0x4e015000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rnz", {{ 0x4e015000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rnz.s", {{ 0x4e01d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rnz.s", {{ 0x4e01d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rn", {{ 0x4e010000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rn", {{ 0x4e010000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rn.s", {{ 0x4e018000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rn.s", {{ 0x4e018000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ro", {{ 0x4e016000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ro", {{ 0x4e016000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ro.s", {{ 0x4e01e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ro.s", {{ 0x4e01e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ru", {{ 0x4e011000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ru", {{ 0x4e011000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.ru.s", {{ 0x4e019000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.ru.s", {{ 0x4e019000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rz", {{ 0x4e013000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rz", {{ 0x4e013000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.rz.s", {{ 0x4e01b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.rz.s", {{ 0x4e01b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w", {{ 0x4e017000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq", {{ 0x4e017000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma212w.s", {{ 0x4e01f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2awq.s", {{ 0x4e01f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmma222w.nt.rd", {{ 0x4e062000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rd.s", {{ 0x4e06a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rna", {{ 0x4e064000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rna.s", {{ 0x4e06c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rnz", {{ 0x4e065000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rnz.s", {{ 0x4e06d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rn", {{ 0x4e060000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rn.s", {{ 0x4e068000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.ro", {{ 0x4e066000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.ro.s", {{ 0x4e06e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.ru", {{ 0x4e061000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.ru.s", {{ 0x4e069000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rz", {{ 0x4e063000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.rz.s", {{ 0x4e06b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt", {{ 0x4e067000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.nt.s", {{ 0x4e06f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rd", {{ 0x4e022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rd.s", {{ 0x4e02a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rna", {{ 0x4e024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rna.s", {{ 0x4e02c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rnz", {{ 0x4e025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rnz.s", {{ 0x4e02d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rn", {{ 0x4e020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rn.s", {{ 0x4e028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.ro", {{ 0x4e026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.ro.s", {{ 0x4e02e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.ru", {{ 0x4e021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.ru.s", {{ 0x4e029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rz", {{ 0x4e023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.rz.s", {{ 0x4e02b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn", {{ 0x4e027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tn.s", {{ 0x4e02f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rd", {{ 0x4e062040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rd.s", {{ 0x4e06a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rna", {{ 0x4e064040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rna.s", {{ 0x4e06c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rnz", {{ 0x4e065040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rnz.s", {{ 0x4e06d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rn", {{ 0x4e060040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rn.s", {{ 0x4e068040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.ro", {{ 0x4e066040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.ro.s", {{ 0x4e06e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.ru", {{ 0x4e061040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.ru.s", {{ 0x4e069040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rz", {{ 0x4e063040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.rz.s", {{ 0x4e06b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt", {{ 0x4e067040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.tt.s", {{ 0x4e06f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rd", {{ 0x4e022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rd.s", {{ 0x4e02a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rna", {{ 0x4e024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rna.s", {{ 0x4e02c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rnz", {{ 0x4e025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rnz.s", {{ 0x4e02d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rn", {{ 0x4e020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rn.s", {{ 0x4e028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.ro", {{ 0x4e026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.ro.s", {{ 0x4e02e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.ru", {{ 0x4e021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.ru.s", {{ 0x4e029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rz", {{ 0x4e023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.rz.s", {{ 0x4e02b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w", {{ 0x4e027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma222w.s", {{ 0x4e02f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmma482hw0.rd", {{ 0x02000280, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rd.s", {{ 0x02000a80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rna", {{ 0x02000480, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rna.s", {{ 0x02000c80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rnz", {{ 0x02000580, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rnz.s", {{ 0x02000d80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rn", {{ 0x02000080, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rn.s", {{ 0x02000880, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.ro", {{ 0x02000680, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.ro.s", {{ 0x02000e80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.ru", {{ 0x02000180, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.ru.s", {{ 0x02000980, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rz", {{ 0x02000380, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.rz.s", {{ 0x02000b80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0", {{ 0x02000780, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw0.s", {{ 0x02000f80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerae_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW0_xrounding_silent2_registerAe_registerBp_registerDp_simple */
	{"fmma482hw1.rd", {{ 0x02040280, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rd.s", {{ 0x02040a80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rna", {{ 0x02040480, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rna.s", {{ 0x02040c80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rnz", {{ 0x02040580, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rnz.s", {{ 0x02040d80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rn", {{ 0x02040080, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rn.s", {{ 0x02040880, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.ro", {{ 0x02040680, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.ro.s", {{ 0x02040e80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.ru", {{ 0x02040180, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.ru.s", {{ 0x02040980, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rz", {{ 0x02040380, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.rz.s", {{ 0x02040b80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1", {{ 0x02040780, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmma482hw1.s", {{ 0x02040f80, 0x7f041fc1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerao_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMA482HW1_xrounding_silent2_registerAo_registerBp_registerDp_simple */
	{"fmms212w.rd", {{ 0x4f012000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rd", {{ 0x4f012000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rd.s", {{ 0x4f01a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rd.s", {{ 0x4f01a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rna", {{ 0x4f014000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rna", {{ 0x4f014000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rna.s", {{ 0x4f01c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rna.s", {{ 0x4f01c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rnz", {{ 0x4f015000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rnz", {{ 0x4f015000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rnz.s", {{ 0x4f01d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rnz.s", {{ 0x4f01d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rn", {{ 0x4f010000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rn", {{ 0x4f010000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rn.s", {{ 0x4f018000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rn.s", {{ 0x4f018000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ro", {{ 0x4f016000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ro", {{ 0x4f016000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ro.s", {{ 0x4f01e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ro.s", {{ 0x4f01e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ru", {{ 0x4f011000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ru", {{ 0x4f011000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.ru.s", {{ 0x4f019000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.ru.s", {{ 0x4f019000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rz", {{ 0x4f013000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rz", {{ 0x4f013000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.rz.s", {{ 0x4f01b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.rz.s", {{ 0x4f01b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w", {{ 0x4f017000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq", {{ 0x4f017000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms212w.s", {{ 0x4f01f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmm2swq.s", {{ 0x4f01f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS212W_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmms222w.nt.rd", {{ 0x4f062000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rd.s", {{ 0x4f06a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rna", {{ 0x4f064000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rna.s", {{ 0x4f06c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rnz", {{ 0x4f065000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rnz.s", {{ 0x4f06d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rn", {{ 0x4f060000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rn.s", {{ 0x4f068000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.ro", {{ 0x4f066000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.ro.s", {{ 0x4f06e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.ru", {{ 0x4f061000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.ru.s", {{ 0x4f069000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rz", {{ 0x4f063000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.rz.s", {{ 0x4f06b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt", {{ 0x4f067000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.nt.s", {{ 0x4f06f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rd", {{ 0x4f022040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rd.s", {{ 0x4f02a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rna", {{ 0x4f024040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rna.s", {{ 0x4f02c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rnz", {{ 0x4f025040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rnz.s", {{ 0x4f02d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rn", {{ 0x4f020040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rn.s", {{ 0x4f028040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.ro", {{ 0x4f026040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.ro.s", {{ 0x4f02e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.ru", {{ 0x4f021040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.ru.s", {{ 0x4f029040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rz", {{ 0x4f023040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.rz.s", {{ 0x4f02b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn", {{ 0x4f027040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tn.s", {{ 0x4f02f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rd", {{ 0x4f062040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rd.s", {{ 0x4f06a040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rna", {{ 0x4f064040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rna.s", {{ 0x4f06c040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rnz", {{ 0x4f065040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rnz.s", {{ 0x4f06d040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rn", {{ 0x4f060040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rn.s", {{ 0x4f068040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.ro", {{ 0x4f066040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.ro.s", {{ 0x4f06e040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.ru", {{ 0x4f061040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.ru.s", {{ 0x4f069040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rz", {{ 0x4f063040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.rz.s", {{ 0x4f06b040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt", {{ 0x4f067040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.tt.s", {{ 0x4f06f040, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rd", {{ 0x4f022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rd.s", {{ 0x4f02a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rna", {{ 0x4f024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rna.s", {{ 0x4f02c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rnz", {{ 0x4f025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rnz.s", {{ 0x4f02d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rn", {{ 0x4f020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rn.s", {{ 0x4f028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.ro", {{ 0x4f026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.ro.s", {{ 0x4f02e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.ru", {{ 0x4f021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.ru.s", {{ 0x4f029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rz", {{ 0x4f023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.rz.s", {{ 0x4f02b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w", {{ 0x4f027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmms222w.s", {{ 0x4f02f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMMS222W_multransm_multransl_rounding_silent_registerM_registerP_registerO_simple */
	{"fmuld.rd", {{ 0x58012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rd.s", {{ 0x5801a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rna", {{ 0x58014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rna.s", {{ 0x5801c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rnz", {{ 0x58015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rnz.s", {{ 0x5801d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rn", {{ 0x58010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rn.s", {{ 0x58018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ro", {{ 0x58016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ro.s", {{ 0x5801e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ru", {{ 0x58011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.ru.s", {{ 0x58019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rz", {{ 0x58013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.rz.s", {{ 0x5801b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld", {{ 0x58017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmuld.s", {{ 0x5801f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulho.rd", {{ 0x55072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rd.s", {{ 0x5507a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rna", {{ 0x55074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rna.s", {{ 0x5507c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rnz", {{ 0x55075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rnz.s", {{ 0x5507d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rn", {{ 0x55070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rn.s", {{ 0x55078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.ro", {{ 0x55076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.ro.s", {{ 0x5507e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.ru", {{ 0x55071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.ru.s", {{ 0x55079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rz", {{ 0x55073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.rz.s", {{ 0x5507b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho", {{ 0x55077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulho.s", {{ 0x5507f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulhq.rd", {{ 0x5b012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rd.s", {{ 0x5b01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rna", {{ 0x5b014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rna.s", {{ 0x5b01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rnz", {{ 0x5b015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rnz.s", {{ 0x5b01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rn", {{ 0x5b010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rn.s", {{ 0x5b018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ro", {{ 0x5b016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ro.s", {{ 0x5b01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ru", {{ 0x5b011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.ru.s", {{ 0x5b019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rz", {{ 0x5b013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.rz.s", {{ 0x5b01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq", {{ 0x5b017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhq.s", {{ 0x5b01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhwq.rd", {{ 0x51072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rd.s", {{ 0x5107a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rna", {{ 0x51074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rna.s", {{ 0x5107c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rnz", {{ 0x51075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rnz.s", {{ 0x5107d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rn", {{ 0x51070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rn.s", {{ 0x51078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ro", {{ 0x51076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ro.s", {{ 0x5107e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ru", {{ 0x51071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.ru.s", {{ 0x51079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rz", {{ 0x51073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.rz.s", {{ 0x5107b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq", {{ 0x51077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhwq.s", {{ 0x5107f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHWQ_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulhw.rd", {{ 0x5f022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rd.s", {{ 0x5f02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rna", {{ 0x5f024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rna.s", {{ 0x5f02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rnz", {{ 0x5f025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rnz.s", {{ 0x5f02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rn", {{ 0x5f020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rn.s", {{ 0x5f028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ro", {{ 0x5f026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ro.s", {{ 0x5f02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ru", {{ 0x5f021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.ru.s", {{ 0x5f029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rz", {{ 0x5f023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.rz.s", {{ 0x5f02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw", {{ 0x5f027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulhw.s", {{ 0x5f02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULHW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwcp.c.rd", {{ 0x41022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rd.s", {{ 0x4102a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rna", {{ 0x41024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rna.s", {{ 0x4102c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rnz", {{ 0x41025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rnz.s", {{ 0x4102d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rn", {{ 0x41020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rn.s", {{ 0x41028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.ro", {{ 0x41026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.ro.s", {{ 0x4102e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.ru", {{ 0x41021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.ru.s", {{ 0x41029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rz", {{ 0x41023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.rz.s", {{ 0x4102b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c", {{ 0x41027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.c.s", {{ 0x4102f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rd", {{ 0x40022000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rd.s", {{ 0x4002a000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rna", {{ 0x40024000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rna.s", {{ 0x4002c000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rnz", {{ 0x40025000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rnz.s", {{ 0x4002d000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rn", {{ 0x40020000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rn.s", {{ 0x40028000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.ro", {{ 0x40026000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.ro.s", {{ 0x4002e000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.ru", {{ 0x40021000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.ru.s", {{ 0x40029000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rz", {{ 0x40023000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.rz.s", {{ 0x4002b000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp", {{ 0x40027000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwcp.s", {{ 0x4002f000, 0x7f07f040, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWCP_conjugate_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwc.c.rd", {{ 0x41012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rd", {{ 0x41012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rd.s", {{ 0x4101a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rd.s", {{ 0x4101a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rna", {{ 0x41014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rna", {{ 0x41014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rna.s", {{ 0x4101c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rna.s", {{ 0x4101c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rnz", {{ 0x41015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rnz", {{ 0x41015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rnz.s", {{ 0x4101d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rnz.s", {{ 0x4101d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rn", {{ 0x41010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rn", {{ 0x41010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rn.s", {{ 0x41018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rn.s", {{ 0x41018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ro", {{ 0x41016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ro", {{ 0x41016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ro.s", {{ 0x4101e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ro.s", {{ 0x4101e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ru", {{ 0x41011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ru", {{ 0x41011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.ru.s", {{ 0x41019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.ru.s", {{ 0x41019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rz", {{ 0x41013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rz", {{ 0x41013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.rz.s", {{ 0x4101b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.rz.s", {{ 0x4101b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c", {{ 0x41017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc", {{ 0x41017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.c.s", {{ 0x4101f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulcwc.s", {{ 0x4101f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rd", {{ 0x40012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rd.s", {{ 0x4001a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rna", {{ 0x40014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rna.s", {{ 0x4001c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rnz", {{ 0x40015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rnz.s", {{ 0x4001d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rn", {{ 0x40010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rn.s", {{ 0x40018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ro", {{ 0x40016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ro.s", {{ 0x4001e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ru", {{ 0x40011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.ru.s", {{ 0x40019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rz", {{ 0x40013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.rz.s", {{ 0x4001b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc", {{ 0x40017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwc.s", {{ 0x4001f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWC_conjugate_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwdp.rd", {{ 0x50072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rd.s", {{ 0x5007a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rna", {{ 0x50074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rna.s", {{ 0x5007c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rnz", {{ 0x50075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rnz.s", {{ 0x5007d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rn", {{ 0x50070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rn.s", {{ 0x50078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ro", {{ 0x50076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ro.s", {{ 0x5007e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ru", {{ 0x50071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.ru.s", {{ 0x50079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rz", {{ 0x50073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.rz.s", {{ 0x5007b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp", {{ 0x50077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwdp.s", {{ 0x5007f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWDP_rounding_silent_registerM_registerZ_registerY_simple */
	{"fmulwd.rd", {{ 0x59012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rd.s", {{ 0x5901a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rna", {{ 0x59014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rna.s", {{ 0x5901c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rnz", {{ 0x59015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rnz.s", {{ 0x5901d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rn", {{ 0x59010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rn.s", {{ 0x59018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ro", {{ 0x59016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ro.s", {{ 0x5901e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ru", {{ 0x59011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.ru.s", {{ 0x59019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rz", {{ 0x59013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.rz.s", {{ 0x5901b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd", {{ 0x59017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwd.s", {{ 0x5901f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rd", {{ 0x5a012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rd.s", {{ 0x5a01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rna", {{ 0x5a014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rna.s", {{ 0x5a01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rnz", {{ 0x5a015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rnz.s", {{ 0x5a01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rn", {{ 0x5a010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rn.s", {{ 0x5a018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ro", {{ 0x5a016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ro.s", {{ 0x5a01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ru", {{ 0x5a011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.ru.s", {{ 0x5a019000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rz", {{ 0x5a013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.rz.s", {{ 0x5a01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp", {{ 0x5a017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwp.s", {{ 0x5a01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulwq.rd", {{ 0x5e072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rd.s", {{ 0x5e07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rna", {{ 0x5e074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rna.s", {{ 0x5e07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rnz", {{ 0x5e075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rnz.s", {{ 0x5e07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rn", {{ 0x5e070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rn.s", {{ 0x5e078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ro", {{ 0x5e076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ro.s", {{ 0x5e07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ru", {{ 0x5e071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.ru.s", {{ 0x5e079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rz", {{ 0x5e073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.rz.s", {{ 0x5e07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq", {{ 0x5e077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulwq.s", {{ 0x5e07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fmulw.rd", {{ 0x5e022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rd.s", {{ 0x5e02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rna", {{ 0x5e024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rna.s", {{ 0x5e02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rnz", {{ 0x5e025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rnz.s", {{ 0x5e02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rn", {{ 0x5e020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rn.s", {{ 0x5e028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ro", {{ 0x5e026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ro.s", {{ 0x5e02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ru", {{ 0x5e021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.ru.s", {{ 0x5e029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rz", {{ 0x5e023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.rz.s", {{ 0x5e02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw", {{ 0x5e027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fmulw.s", {{ 0x5e02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FMULW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fnarrowdwp.rd", {{ 0x7c036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rd.s", {{ 0x7c036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rna", {{ 0x7c036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rna.s", {{ 0x7c036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rnz", {{ 0x7c036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rnz.s", {{ 0x7c036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rn", {{ 0x7c036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rn.s", {{ 0x7c036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ro", {{ 0x7c036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ro.s", {{ 0x7c036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ru", {{ 0x7c036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.ru.s", {{ 0x7c036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rz", {{ 0x7c036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.rz.s", {{ 0x7c036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp", {{ 0x7c036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdwp.s", {{ 0x7c036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDWP_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowdw.rd", {{ 0x78036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rd.s", {{ 0x78036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rna", {{ 0x78036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rna.s", {{ 0x78036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rnz", {{ 0x78036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rnz.s", {{ 0x78036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rn", {{ 0x78036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rn.s", {{ 0x78036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ro", {{ 0x78036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ro.s", {{ 0x78036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ru", {{ 0x78036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.ru.s", {{ 0x78036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rz", {{ 0x78036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.rz.s", {{ 0x78036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw", {{ 0x78036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowdw.s", {{ 0x78036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWDW_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwhq.rd", {{ 0x7e036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rd.s", {{ 0x7e036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rna", {{ 0x7e036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rna.s", {{ 0x7e036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rnz", {{ 0x7e036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rnz.s", {{ 0x7e036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rn", {{ 0x7e036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rn.s", {{ 0x7e036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ro", {{ 0x7e036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ro.s", {{ 0x7e036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ru", {{ 0x7e036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.ru.s", {{ 0x7e036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rz", {{ 0x7e036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.rz.s", {{ 0x7e036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq", {{ 0x7e036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhq.s", {{ 0x7e036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHQ_rounding2_silent2_registerW_registerP_simple */
	{"fnarrowwhv.rd", {{ 0x07000240, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rd.s", {{ 0x07000a40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rna", {{ 0x07000440, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rna.s", {{ 0x07000c40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rnz", {{ 0x07000540, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rnz.s", {{ 0x07000d40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rn", {{ 0x07000040, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rn.s", {{ 0x07000840, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ro", {{ 0x07000640, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ro.s", {{ 0x07000e40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ru", {{ 0x07000140, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.ru.s", {{ 0x07000940, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rz", {{ 0x07000340, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.rz.s", {{ 0x07000b40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv", {{ 0x07000740, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwhv.s", {{ 0x07000f40, 0x7f001f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerbp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWHV_xrounding_silent2_registerA_registerBp_simple */
	{"fnarrowwh.rd", {{ 0x7a036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rd.s", {{ 0x7a036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rna", {{ 0x7a036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rna.s", {{ 0x7a036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rnz", {{ 0x7a036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rnz.s", {{ 0x7a036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rn", {{ 0x7a036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rn.s", {{ 0x7a036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ro", {{ 0x7a036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ro.s", {{ 0x7a036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ru", {{ 0x7a036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.ru.s", {{ 0x7a036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rz", {{ 0x7a036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.rz.s", {{ 0x7a036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh", {{ 0x7a036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnarrowwh.s", {{ 0x7a036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNARROWWH_rounding2_silent2_registerW_registerZ_simple */
	{"fnegd", {{ 0x70032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNEGD_registerW_registerZ_simple */
	{"fneghq", {{ 0x76032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNEGHQ_registerW_registerZ_simple */
	{"fnegwp", {{ 0x74032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNEGWP_registerW_registerZ_simple */
	{"fnegw", {{ 0x72032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FNEGW_registerW_registerZ_simple */
	{"frecw.rd", {{ 0x72036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rd", {{ 0x72036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rd.s", {{ 0x72036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rd.s", {{ 0x72036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rna", {{ 0x72036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rna", {{ 0x72036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rna.s", {{ 0x72036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rna.s", {{ 0x72036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rnz", {{ 0x72036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rnz", {{ 0x72036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rnz.s", {{ 0x72036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rnz.s", {{ 0x72036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rn", {{ 0x72036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rn", {{ 0x72036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rn.s", {{ 0x72036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rn.s", {{ 0x72036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ro", {{ 0x72036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ro", {{ 0x72036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ro.s", {{ 0x72036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ro.s", {{ 0x72036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ru", {{ 0x72036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ru", {{ 0x72036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.ru.s", {{ 0x72036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.ru.s", {{ 0x72036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rz", {{ 0x72036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rz", {{ 0x72036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.rz.s", {{ 0x72036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.rz.s", {{ 0x72036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw", {{ 0x72036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw", {{ 0x72036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frecw.s", {{ 0x72036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"finvw.s", {{ 0x72036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRECW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rd", {{ 0x73036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rd", {{ 0x73036200, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rd.s", {{ 0x73036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rd.s", {{ 0x73036a00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rna", {{ 0x73036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rna", {{ 0x73036400, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rna.s", {{ 0x73036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rna.s", {{ 0x73036c00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rnz", {{ 0x73036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rnz", {{ 0x73036500, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rnz.s", {{ 0x73036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rnz.s", {{ 0x73036d00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rn", {{ 0x73036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rn", {{ 0x73036000, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rn.s", {{ 0x73036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rn.s", {{ 0x73036800, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ro", {{ 0x73036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ro", {{ 0x73036600, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ro.s", {{ 0x73036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ro.s", {{ 0x73036e00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ru", {{ 0x73036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ru", {{ 0x73036100, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.ru.s", {{ 0x73036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.ru.s", {{ 0x73036900, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rz", {{ 0x73036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rz", {{ 0x73036300, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.rz.s", {{ 0x73036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.rz.s", {{ 0x73036b00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw", {{ 0x73036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw", {{ 0x73036700, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"frsrw.s", {{ 0x73036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fisrw.s", {{ 0x73036f00, 0x7f03ff00, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FRSRW_rounding2_silent2_registerW_registerZ_simple */
	{"fsbfcdc.rd", {{ 0x5f032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rd", {{ 0x5f032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rd.s", {{ 0x5f03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rd.s", {{ 0x5f03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rna", {{ 0x5f034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rna", {{ 0x5f034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rna.s", {{ 0x5f03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rna.s", {{ 0x5f03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rnz", {{ 0x5f035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rnz", {{ 0x5f035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rnz.s", {{ 0x5f03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rnz.s", {{ 0x5f03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rn", {{ 0x5f030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rn", {{ 0x5f030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rn.s", {{ 0x5f038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rn.s", {{ 0x5f038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ro", {{ 0x5f036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ro", {{ 0x5f036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ro.s", {{ 0x5f03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ro.s", {{ 0x5f03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ru", {{ 0x5f031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ru", {{ 0x5f031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.ru.s", {{ 0x5f039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.ru.s", {{ 0x5f039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rz", {{ 0x5f033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rz", {{ 0x5f033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.rz.s", {{ 0x5f03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.rz.s", {{ 0x5f03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc", {{ 0x5f037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c", {{ 0x5f037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcdc.s", {{ 0x5f03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.c.s", {{ 0x5f03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCDC_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rd", {{ 0x5b072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rd", {{ 0x5b072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rd.s", {{ 0x5b07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rd.s", {{ 0x5b07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rna", {{ 0x5b074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rna", {{ 0x5b074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rna.s", {{ 0x5b07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rna.s", {{ 0x5b07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rnz", {{ 0x5b075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rnz", {{ 0x5b075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rnz.s", {{ 0x5b07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rnz.s", {{ 0x5b07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rn", {{ 0x5b070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rn", {{ 0x5b070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rn.s", {{ 0x5b078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rn.s", {{ 0x5b078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ro", {{ 0x5b076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ro", {{ 0x5b076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ro.s", {{ 0x5b07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ro.s", {{ 0x5b07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ru", {{ 0x5b071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ru", {{ 0x5b071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.ru.s", {{ 0x5b079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.ru.s", {{ 0x5b079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rz", {{ 0x5b073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rz", {{ 0x5b073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.rz.s", {{ 0x5b07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.rz.s", {{ 0x5b07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp", {{ 0x5b077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c", {{ 0x5b077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwcp.s", {{ 0x5b07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.c.s", {{ 0x5b07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWCP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfcwc.rd", {{ 0x57022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rd", {{ 0x57022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rd.s", {{ 0x5702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rd.s", {{ 0x5702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rna", {{ 0x57024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rna", {{ 0x57024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rna.s", {{ 0x5702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rna.s", {{ 0x5702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rnz", {{ 0x57025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rnz", {{ 0x57025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rnz.s", {{ 0x5702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rnz.s", {{ 0x5702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rn", {{ 0x57020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rn", {{ 0x57020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rn.s", {{ 0x57028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rn.s", {{ 0x57028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ro", {{ 0x57026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ro", {{ 0x57026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ro.s", {{ 0x5702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ro.s", {{ 0x5702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ru", {{ 0x57021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ru", {{ 0x57021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.ru.s", {{ 0x57029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.ru.s", {{ 0x57029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rz", {{ 0x57023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rz", {{ 0x57023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.rz.s", {{ 0x5702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.rz.s", {{ 0x5702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc", {{ 0x57027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c", {{ 0x57027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfcwc.s", {{ 0x5702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.c.s", {{ 0x5702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFCWC_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfdp.rd", {{ 0x5e032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rd.s", {{ 0x5e03a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rna", {{ 0x5e034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rna.s", {{ 0x5e03c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rnz", {{ 0x5e035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rnz.s", {{ 0x5e03d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rn", {{ 0x5e030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rn.s", {{ 0x5e038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ro", {{ 0x5e036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ro.s", {{ 0x5e03e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ru", {{ 0x5e031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.ru.s", {{ 0x5e039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rz", {{ 0x5e033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.rz.s", {{ 0x5e03b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp", {{ 0x5e037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfdp.s", {{ 0x5e03f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFDP_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfd.rd", {{ 0x54022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rd.s", {{ 0x5402a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rna", {{ 0x54024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rna.s", {{ 0x5402c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rnz", {{ 0x54025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rnz.s", {{ 0x5402d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rn", {{ 0x54020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rn.s", {{ 0x54028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ro", {{ 0x54026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ro.s", {{ 0x5402e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ru", {{ 0x54021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.ru.s", {{ 0x54029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rz", {{ 0x54023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.rz.s", {{ 0x5402b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd", {{ 0x54027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfd.s", {{ 0x5402f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFD_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfho.rd", {{ 0x57072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rd.s", {{ 0x5707a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rna", {{ 0x57074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rna.s", {{ 0x5707c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rnz", {{ 0x57075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rnz.s", {{ 0x5707d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rn", {{ 0x57070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rn.s", {{ 0x57078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.ro", {{ 0x57076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.ro.s", {{ 0x5707e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.ru", {{ 0x57071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.ru.s", {{ 0x57079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rz", {{ 0x57073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.rz.s", {{ 0x5707b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho", {{ 0x57077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfho.s", {{ 0x5707f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHO_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfhq.rd", {{ 0x56022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rd.s", {{ 0x5602a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rna", {{ 0x56024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rna.s", {{ 0x5602c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rnz", {{ 0x56025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rnz.s", {{ 0x5602d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rn", {{ 0x56020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rn.s", {{ 0x56028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ro", {{ 0x56026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ro.s", {{ 0x5602e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ru", {{ 0x56021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.ru.s", {{ 0x56029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rz", {{ 0x56023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.rz.s", {{ 0x5602b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq", {{ 0x56027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfhq.s", {{ 0x5602f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFHQ_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rd", {{ 0x55022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rd.s", {{ 0x5502a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rna", {{ 0x55024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rna.s", {{ 0x5502c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rnz", {{ 0x55025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rnz.s", {{ 0x5502d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rn", {{ 0x55020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rn.s", {{ 0x55028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ro", {{ 0x55026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ro.s", {{ 0x5502e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ru", {{ 0x55021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.ru.s", {{ 0x55029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rz", {{ 0x55023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.rz.s", {{ 0x5502b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp", {{ 0x55027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwp.s", {{ 0x5502f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWP_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfwq.rd", {{ 0x5a072000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rd.s", {{ 0x5a07a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rna", {{ 0x5a074000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rna.s", {{ 0x5a07c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rnz", {{ 0x5a075000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rnz.s", {{ 0x5a07d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rn", {{ 0x5a070000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rn.s", {{ 0x5a078000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ro", {{ 0x5a076000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ro.s", {{ 0x5a07e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ru", {{ 0x5a071000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.ru.s", {{ 0x5a079000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rz", {{ 0x5a073000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.rz.s", {{ 0x5a07b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq", {{ 0x5a077000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfwq.s", {{ 0x5a07f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFWQ_rounding_silent_registerM_registerP_registerO_simple */
	{"fsbfw.rd", {{ 0x5d022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rd.s", {{ 0x5d02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rna", {{ 0x5d024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rna.s", {{ 0x5d02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rnz", {{ 0x5d025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rnz.s", {{ 0x5d02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rn", {{ 0x5d020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rn.s", {{ 0x5d028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ro", {{ 0x5d026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ro.s", {{ 0x5d02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ru", {{ 0x5d021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.ru.s", {{ 0x5d029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rz", {{ 0x5d023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.rz.s", {{ 0x5d02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw", {{ 0x5d027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fsbfw.s", {{ 0x5d02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-FSBFW_rounding_silent_registerW_registerZ_registerY_simple */
	{"fscalewv.rd.s", {{ 0x07000a00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rd.s.relu", {{ 0x07000a80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rd", {{ 0x07000200, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rd.relu", {{ 0x07000280, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna.s", {{ 0x07000c00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna.s.relu", {{ 0x07000c80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna", {{ 0x07000400, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rna.relu", {{ 0x07000480, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz.s", {{ 0x07000d00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz.s.relu", {{ 0x07000d80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz", {{ 0x07000500, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rnz.relu", {{ 0x07000580, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn.s", {{ 0x07000800, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn.s.relu", {{ 0x07000880, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn", {{ 0x07000000, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rn.relu", {{ 0x07000080, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro.s", {{ 0x07000e00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro.s.relu", {{ 0x07000e80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro", {{ 0x07000600, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ro.relu", {{ 0x07000680, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru.s", {{ 0x07000900, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru.s.relu", {{ 0x07000980, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru", {{ 0x07000100, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.ru.relu", {{ 0x07000180, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz.s", {{ 0x07000b00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz.s.relu", {{ 0x07000b80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz", {{ 0x07000300, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.rz.relu", {{ 0x07000380, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.s", {{ 0x07000f00, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.s.relu", {{ 0x07000f80, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv", {{ 0x07000700, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fscalewv.relu", {{ 0x07000780, 0x7f000fc0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registera_opnd, &kv3_registerb_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSCALEWV_xrounding_silent2_rectify_registerA_registerB_simple */
	{"fsdivd", {{ 0x70035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSDIVD_silent2_registerW_registerP_simple */
	{"fsdivd.s", {{ 0x70035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSDIVD_silent2_registerW_registerP_simple */
	{"fsdivwp", {{ 0x74035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSDIVWP_silent2_registerW_registerP_simple */
	{"fsdivwp.s", {{ 0x74035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSDIVWP_silent2_registerW_registerP_simple */
	{"fsdivw", {{ 0x72035000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSDIVW_silent2_registerW_registerP_simple */
	{"fsdivw.s", {{ 0x72035800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerp_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSDIVW_silent2_registerW_registerP_simple */
	{"fsrecd", {{ 0x70034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECD_silent2_registerW_registerZ_simple */
	{"fsinvd", {{ 0x70034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECD_silent2_registerW_registerZ_simple */
	{"fsrecd.s", {{ 0x70034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECD_silent2_registerW_registerZ_simple */
	{"fsinvd.s", {{ 0x70034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECD_silent2_registerW_registerZ_simple */
	{"fsrecwp", {{ 0x74034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsinvwp", {{ 0x74034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsrecwp.s", {{ 0x74034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsinvwp.s", {{ 0x74034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECWP_silent2_registerW_registerZ_simple */
	{"fsrecw", {{ 0x72034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECW_silent2_registerW_registerZ_simple */
	{"fsinvw", {{ 0x72034000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECW_silent2_registerW_registerZ_simple */
	{"fsrecw.s", {{ 0x72034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECW_silent2_registerW_registerZ_simple */
	{"fsinvw.s", {{ 0x72034800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRECW_silent2_registerW_registerZ_simple */
	{"fsrsrd", {{ 0x78032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRD_registerW_registerZ_simple */
	{"fsisrd", {{ 0x78032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRD_registerW_registerZ_simple */
	{"fsrsrd.s", {{ 0x78032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRD_registerW_registerZ_simple */
	{"fsrsrwp", {{ 0x7c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRWP_registerW_registerZ_simple */
	{"fsisrwp", {{ 0x7c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRWP_registerW_registerZ_simple */
	{"fsrsrwp.s", {{ 0x7c032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRWP_registerW_registerZ_simple */
	{"fsrsrw", {{ 0x7a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRW_registerW_registerZ_simple */
	{"fsisrw", {{ 0x7a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRW_registerW_registerZ_simple */
	{"fsrsrw.s", {{ 0x7a032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FSRSRW_registerW_registerZ_simple */
	{"fwidenlhwp", {{ 0x7c033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENLHWP_silent2_registerW_registerZ_simple */
	{"fwidenlhwp.s", {{ 0x7c033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENLHWP_silent2_registerW_registerZ_simple */
	{"fwidenlhw", {{ 0x7a033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENLHW_silent2_registerW_registerZ_simple */
	{"fwidenlhw.s", {{ 0x7a033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENLHW_silent2_registerW_registerZ_simple */
	{"fwidenlwd", {{ 0x78033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENLWD_silent2_registerW_registerZ_simple */
	{"fwidenlwd.s", {{ 0x78033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENLWD_silent2_registerW_registerZ_simple */
	{"fwidenmhwp", {{ 0x7d033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENMHWP_silent2_registerW_registerZ_simple */
	{"fwidenmhwp.s", {{ 0x7d033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENMHWP_silent2_registerW_registerZ_simple */
	{"fwidenmhw", {{ 0x7b033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENMHW_silent2_registerW_registerZ_simple */
	{"fwidenmhw.s", {{ 0x7b033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENMHW_silent2_registerW_registerZ_simple */
	{"fwidenmwd", {{ 0x79033000, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENMWD_silent2_registerW_registerZ_simple */
	{"fwidenmwd.s", {{ 0x79033800, 0x7f03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-FWIDENMWD_silent2_registerW_registerZ_simple */
	{"get", {{ 0x0fc40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &v2_systems2_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-GET_registerZ_systemS2_simple */
	{"get", {{ 0x0fc40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &v2_systems3_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-GET_registerZ_systemS3_simple */
	{"goto", {{ 0x10000000, 0x78000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_pcrel27_opnd, 0}, "", "%s"}, /* Opcode-v2-GOTO_s027_simple */
	{"icall", {{ 0x0fdc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v2-ICALL_registerZ_simple */
	{"iget", {{ 0x0fcc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v2-IGET_registerZ_simple */
	{"igoto", {{ 0x0fd80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v2-IGOTO_registerZ_simple */
	{"iinvals.l1l2.deqz", {{ 0x3e5e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.dgez", {{ 0x3e5e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.dgtz", {{ 0x3e5e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.dlez", {{ 0x3e5e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.dltz", {{ 0x3e5e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.dnez", {{ 0x3e5e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.even", {{ 0x3e5e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.odd", {{ 0x3e5e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.weqz", {{ 0x3e5e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.wgez", {{ 0x3e5eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.wgtz", {{ 0x3e5ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.wlez", {{ 0x3e5ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.wltz", {{ 0x3e5ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.wnez", {{ 0x3e5e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.deqz", {{ 0x3d5e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.dgez", {{ 0x3d5e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.dgtz", {{ 0x3d5e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.dlez", {{ 0x3d5e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.dltz", {{ 0x3d5e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.dnez", {{ 0x3d5e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.even", {{ 0x3d5e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.odd", {{ 0x3d5e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.weqz", {{ 0x3d5e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.wgez", {{ 0x3d5eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.wgtz", {{ 0x3d5ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.wlez", {{ 0x3d5ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.wltz", {{ 0x3d5ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l2.wnez", {{ 0x3d5e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.deqz", {{ 0x3c5e1000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.dgez", {{ 0x3c5e3000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.dgtz", {{ 0x3c5e5000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.dlez", {{ 0x3c5e4000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.dltz", {{ 0x3c5e2000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.dnez", {{ 0x3c5e0000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.even", {{ 0x3c5e7000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.odd", {{ 0x3c5e6000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.weqz", {{ 0x3c5e9000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.wgez", {{ 0x3c5eb000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.wgtz", {{ 0x3c5ed000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.wlez", {{ 0x3c5ec000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.wltz", {{ 0x3c5ea000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.wnez", {{ 0x3c5e8000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s? [%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_registerZ_simple */
	{"iinvals.l1l2.deqz", {{ 0xbe5e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.dgez", {{ 0xbe5e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.dgtz", {{ 0xbe5e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.dlez", {{ 0xbe5e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.dltz", {{ 0xbe5e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.dnez", {{ 0xbe5e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.even", {{ 0xbe5e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.odd", {{ 0xbe5e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.weqz", {{ 0xbe5e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.wgez", {{ 0xbe5eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.wgtz", {{ 0xbe5ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.wlez", {{ 0xbe5ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.wltz", {{ 0xbe5ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.wnez", {{ 0xbe5e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.deqz", {{ 0xbd5e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.dgez", {{ 0xbd5e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.dgtz", {{ 0xbd5e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.dlez", {{ 0xbd5e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.dltz", {{ 0xbd5e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.dnez", {{ 0xbd5e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.even", {{ 0xbd5e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.odd", {{ 0xbd5e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.weqz", {{ 0xbd5e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.wgez", {{ 0xbd5eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.wgtz", {{ 0xbd5ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.wlez", {{ 0xbd5ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.wltz", {{ 0xbd5ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l2.wnez", {{ 0xbd5e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.deqz", {{ 0xbc5e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dgez", {{ 0xbc5e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dgtz", {{ 0xbc5e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dlez", {{ 0xbc5e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dltz", {{ 0xbc5e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.dnez", {{ 0xbc5e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.even", {{ 0xbc5e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.odd", {{ 0xbc5e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.weqz", {{ 0xbc5e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wgez", {{ 0xbc5eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wgtz", {{ 0xbc5ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wlez", {{ 0xbc5ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wltz", {{ 0xbc5ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.wnez", {{ 0xbc5e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s027_registerZ_double */
	{"iinvals.l1l2.deqz", {{ 0xbe5e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.dgez", {{ 0xbe5e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.dgtz", {{ 0xbe5e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.dlez", {{ 0xbe5e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.dltz", {{ 0xbe5e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.dnez", {{ 0xbe5e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.even", {{ 0xbe5e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.odd", {{ 0xbe5e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.weqz", {{ 0xbe5e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.wgez", {{ 0xbe5eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.wgtz", {{ 0xbe5ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.wlez", {{ 0xbe5ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.wltz", {{ 0xbe5ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l1l2.wnez", {{ 0xbe5e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.deqz", {{ 0xbd5e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.dgez", {{ 0xbd5e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.dgtz", {{ 0xbd5e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.dlez", {{ 0xbd5e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.dltz", {{ 0xbd5e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.dnez", {{ 0xbd5e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.even", {{ 0xbd5e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.odd", {{ 0xbd5e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.weqz", {{ 0xbd5e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.wgez", {{ 0xbd5eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.wgtz", {{ 0xbd5ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.wlez", {{ 0xbd5ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.wltz", {{ 0xbd5ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.l2.wnez", {{ 0xbd5e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.deqz", {{ 0xbc5e1000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dgez", {{ 0xbc5e3000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dgtz", {{ 0xbc5e5000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dlez", {{ 0xbc5e4000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dltz", {{ 0xbc5e2000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.dnez", {{ 0xbc5e0000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.even", {{ 0xbc5e7000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.odd", {{ 0xbc5e6000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.weqz", {{ 0xbc5e9000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wgez", {{ 0xbc5eb000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wgtz", {{ 0xbc5ed000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wlez", {{ 0xbc5ec000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wltz", {{ 0xbc5ea000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals.wnez", {{ 0xbc5e8000, 0xfffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s[%s]"}, /* Opcode-v2-IINVALS_cachelev_lsucond_registerY_s054_registerZ_triple */
	{"iinvals", {{ 0x3c5ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_registerY_registerZ_simple */
	{"iinvals.l1l2", {{ 0x3e5ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_registerY_registerZ_simple */
	{"iinvals.l2", {{ 0x3d5ee000, 0x7ffff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_registerY_registerZ_simple */
	{"iinvals", {{ 0x3c5c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_s010_registerZ_simple */
	{"iinvals.l1l2", {{ 0x3e5c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_s010_registerZ_simple */
	{"iinvals.l2", {{ 0x3d5c0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_s010_registerZ_simple */
	{"iinvals", {{ 0xbc5c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_s037_registerZ_double */
	{"iinvals.l1l2", {{ 0xbe5c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_s037_registerZ_double */
	{"iinvals.l2", {{ 0xbd5c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_s037_registerZ_double */
	{"iinvals", {{ 0xbc5c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_w064_registerZ_triple */
	{"iinvals.l1l2", {{ 0xbe5c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_w064_registerZ_triple */
	{"iinvals.l2", {{ 0xbd5c0000, 0xffff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s[%s]"}, /* Opcode-v2-IINVALS_cachelev_w064_registerZ_triple */
	{"iinval", {{ 0x3f9d0000, 0x7fff0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {0}, "", ""}, /* Opcode-v2-IINVAL_simple */
	{"insf", {{ 0x60030000, 0x7c030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_stopbit2_stopbit4_opnd, &kv3_startbit_opnd, 0}, "", "%s = %s, %s, %s"}, /* Opcode-v2-INSF_registerW_registerZ_u006_u006_simple */
	{"landd", {{ 0x7c014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDD_registerW_registerZ_registerY_simple */
	{"landd", {{ 0xfc014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDD_splat32_registerW_registerZ_w032_double */
	{"landd.@", {{ 0xfc014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDD_splat32_registerW_registerZ_w032_double */
	{"landhq", {{ 0x7c017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDHQ_registerW_registerZ_registerY_simple */
	{"landhq", {{ 0xfc017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDHQ_splat32_registerW_registerZ_w032_double */
	{"landhq.@", {{ 0xfc017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDHQ_splat32_registerW_registerZ_w032_double */
	{"landwp", {{ 0x7c016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDWP_registerW_registerZ_registerY_simple */
	{"landwp", {{ 0xfc016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDWP_splat32_registerW_registerZ_w032_double */
	{"landwp.@", {{ 0xfc016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDWP_splat32_registerW_registerZ_w032_double */
	{"landw", {{ 0x7c015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDW_registerW_registerZ_registerY_simple */
	{"landw", {{ 0xfc015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LANDW_registerW_registerZ_w032_double */
	{"lbs.s", {{ 0x2502e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.s.xs", {{ 0x2502f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.us", {{ 0x2702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.us.xs", {{ 0x2702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.u", {{ 0x2602e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.u.xs", {{ 0x2602f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs", {{ 0x2402e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.xs", {{ 0x2402f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbs.s.deqz", {{ 0x25021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dgez", {{ 0x25023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dgtz", {{ 0x25025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dlez", {{ 0x25024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dltz", {{ 0x25022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.dnez", {{ 0x25020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.even", {{ 0x25027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.odd", {{ 0x25026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.weqz", {{ 0x25029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wgez", {{ 0x2502b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wgtz", {{ 0x2502d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wlez", {{ 0x2502c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wltz", {{ 0x2502a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.wnez", {{ 0x25028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.deqz", {{ 0x27021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dgez", {{ 0x27023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dgtz", {{ 0x27025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dlez", {{ 0x27024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dltz", {{ 0x27022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.dnez", {{ 0x27020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.even", {{ 0x27027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.odd", {{ 0x27026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.weqz", {{ 0x27029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wgez", {{ 0x2702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wgtz", {{ 0x2702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wlez", {{ 0x2702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wltz", {{ 0x2702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.us.wnez", {{ 0x27028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.deqz", {{ 0x26021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dgez", {{ 0x26023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dgtz", {{ 0x26025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dlez", {{ 0x26024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dltz", {{ 0x26022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.dnez", {{ 0x26020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.even", {{ 0x26027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.odd", {{ 0x26026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.weqz", {{ 0x26029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wgez", {{ 0x2602b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wgtz", {{ 0x2602d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wlez", {{ 0x2602c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wltz", {{ 0x2602a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.u.wnez", {{ 0x26028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.deqz", {{ 0x24021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dgez", {{ 0x24023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dgtz", {{ 0x24025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dlez", {{ 0x24024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dltz", {{ 0x24022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.dnez", {{ 0x24020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.even", {{ 0x24027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.odd", {{ 0x24026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.weqz", {{ 0x24029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wgez", {{ 0x2402b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wgtz", {{ 0x2402d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wlez", {{ 0x2402c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wltz", {{ 0x2402a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.wnez", {{ 0x24028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbs.s.deqz", {{ 0xa5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dgez", {{ 0xa5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dgtz", {{ 0xa5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dlez", {{ 0xa5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dltz", {{ 0xa5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.dnez", {{ 0xa5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.even", {{ 0xa5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.odd", {{ 0xa5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.weqz", {{ 0xa5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wgez", {{ 0xa502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wgtz", {{ 0xa502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wlez", {{ 0xa502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wltz", {{ 0xa502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.wnez", {{ 0xa5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.deqz", {{ 0xa7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dgez", {{ 0xa7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dgtz", {{ 0xa7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dlez", {{ 0xa7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dltz", {{ 0xa7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.dnez", {{ 0xa7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.even", {{ 0xa7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.odd", {{ 0xa7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.weqz", {{ 0xa7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wgez", {{ 0xa702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wgtz", {{ 0xa702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wlez", {{ 0xa702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wltz", {{ 0xa702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.us.wnez", {{ 0xa7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.deqz", {{ 0xa6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dgez", {{ 0xa6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dgtz", {{ 0xa6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dlez", {{ 0xa6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dltz", {{ 0xa6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.dnez", {{ 0xa6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.even", {{ 0xa6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.odd", {{ 0xa6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.weqz", {{ 0xa6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wgez", {{ 0xa602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wgtz", {{ 0xa602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wlez", {{ 0xa602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wltz", {{ 0xa602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.u.wnez", {{ 0xa6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.deqz", {{ 0xa4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dgez", {{ 0xa4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dgtz", {{ 0xa4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dlez", {{ 0xa4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dltz", {{ 0xa4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.dnez", {{ 0xa4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.even", {{ 0xa4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.odd", {{ 0xa4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.weqz", {{ 0xa4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wgez", {{ 0xa402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wgtz", {{ 0xa402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wlez", {{ 0xa402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wltz", {{ 0xa402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.wnez", {{ 0xa4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbs.s.deqz", {{ 0xa5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dgez", {{ 0xa5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dgtz", {{ 0xa5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dlez", {{ 0xa5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dltz", {{ 0xa5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.dnez", {{ 0xa5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.even", {{ 0xa5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.odd", {{ 0xa5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.weqz", {{ 0xa5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wgez", {{ 0xa502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wgtz", {{ 0xa502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wlez", {{ 0xa502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wltz", {{ 0xa502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.s.wnez", {{ 0xa5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.deqz", {{ 0xa7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dgez", {{ 0xa7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dgtz", {{ 0xa7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dlez", {{ 0xa7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dltz", {{ 0xa7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.dnez", {{ 0xa7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.even", {{ 0xa7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.odd", {{ 0xa7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.weqz", {{ 0xa7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wgez", {{ 0xa702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wgtz", {{ 0xa702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wlez", {{ 0xa702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wltz", {{ 0xa702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.us.wnez", {{ 0xa7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.deqz", {{ 0xa6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dgez", {{ 0xa6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dgtz", {{ 0xa6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dlez", {{ 0xa6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dltz", {{ 0xa6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.dnez", {{ 0xa6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.even", {{ 0xa6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.odd", {{ 0xa6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.weqz", {{ 0xa6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wgez", {{ 0xa602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wgtz", {{ 0xa602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wlez", {{ 0xa602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wltz", {{ 0xa602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.u.wnez", {{ 0xa6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.deqz", {{ 0xa4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dgez", {{ 0xa4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dgtz", {{ 0xa4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dlez", {{ 0xa4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dltz", {{ 0xa4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.dnez", {{ 0xa4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.even", {{ 0xa4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.odd", {{ 0xa4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.weqz", {{ 0xa4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wgez", {{ 0xa402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wgtz", {{ 0xa402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wlez", {{ 0xa402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wltz", {{ 0xa402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs.wnez", {{ 0xa4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbs", {{ 0x24000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs.s", {{ 0x25000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs.u", {{ 0x26000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs.us", {{ 0x27000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s010_registerZ_simple */
	{"lbs", {{ 0xa4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s037_registerZ_double */
	{"lbs.s", {{ 0xa5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s037_registerZ_double */
	{"lbs.u", {{ 0xa6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s037_registerZ_double */
	{"lbs.us", {{ 0xa7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_s037_registerZ_double */
	{"lbs", {{ 0xa4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_w064_registerZ_triple */
	{"lbs.s", {{ 0xa5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_w064_registerZ_triple */
	{"lbs.u", {{ 0xa6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_w064_registerZ_triple */
	{"lbs.us", {{ 0xa7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBS_variant_registerW_w064_registerZ_triple */
	{"lbz.s", {{ 0x2102e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.s.xs", {{ 0x2102f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.us", {{ 0x2302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.us.xs", {{ 0x2302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.u", {{ 0x2202e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.u.xs", {{ 0x2202f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz", {{ 0x2002e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.xs", {{ 0x2002f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lbz.s.deqz", {{ 0x21021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dgez", {{ 0x21023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dgtz", {{ 0x21025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dlez", {{ 0x21024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dltz", {{ 0x21022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.dnez", {{ 0x21020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.even", {{ 0x21027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.odd", {{ 0x21026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.weqz", {{ 0x21029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wgez", {{ 0x2102b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wgtz", {{ 0x2102d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wlez", {{ 0x2102c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wltz", {{ 0x2102a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.wnez", {{ 0x21028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.deqz", {{ 0x23021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dgez", {{ 0x23023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dgtz", {{ 0x23025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dlez", {{ 0x23024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dltz", {{ 0x23022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.dnez", {{ 0x23020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.even", {{ 0x23027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.odd", {{ 0x23026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.weqz", {{ 0x23029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wgez", {{ 0x2302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wgtz", {{ 0x2302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wlez", {{ 0x2302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wltz", {{ 0x2302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.us.wnez", {{ 0x23028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.deqz", {{ 0x22021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dgez", {{ 0x22023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dgtz", {{ 0x22025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dlez", {{ 0x22024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dltz", {{ 0x22022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.dnez", {{ 0x22020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.even", {{ 0x22027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.odd", {{ 0x22026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.weqz", {{ 0x22029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wgez", {{ 0x2202b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wgtz", {{ 0x2202d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wlez", {{ 0x2202c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wltz", {{ 0x2202a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.u.wnez", {{ 0x22028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.deqz", {{ 0x20021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dgez", {{ 0x20023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dgtz", {{ 0x20025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dlez", {{ 0x20024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dltz", {{ 0x20022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.dnez", {{ 0x20020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.even", {{ 0x20027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.odd", {{ 0x20026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.weqz", {{ 0x20029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wgez", {{ 0x2002b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wgtz", {{ 0x2002d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wlez", {{ 0x2002c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wltz", {{ 0x2002a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.wnez", {{ 0x20028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lbz.s.deqz", {{ 0xa1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dgez", {{ 0xa1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dgtz", {{ 0xa1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dlez", {{ 0xa1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dltz", {{ 0xa1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.dnez", {{ 0xa1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.even", {{ 0xa1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.odd", {{ 0xa1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.weqz", {{ 0xa1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wgez", {{ 0xa102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wgtz", {{ 0xa102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wlez", {{ 0xa102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wltz", {{ 0xa102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.wnez", {{ 0xa1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.deqz", {{ 0xa3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dgez", {{ 0xa3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dgtz", {{ 0xa3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dlez", {{ 0xa3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dltz", {{ 0xa3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.dnez", {{ 0xa3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.even", {{ 0xa3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.odd", {{ 0xa3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.weqz", {{ 0xa3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wgez", {{ 0xa302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wgtz", {{ 0xa302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wlez", {{ 0xa302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wltz", {{ 0xa302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.us.wnez", {{ 0xa3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.deqz", {{ 0xa2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dgez", {{ 0xa2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dgtz", {{ 0xa2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dlez", {{ 0xa2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dltz", {{ 0xa2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.dnez", {{ 0xa2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.even", {{ 0xa2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.odd", {{ 0xa2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.weqz", {{ 0xa2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wgez", {{ 0xa202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wgtz", {{ 0xa202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wlez", {{ 0xa202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wltz", {{ 0xa202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.u.wnez", {{ 0xa2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.deqz", {{ 0xa0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dgez", {{ 0xa0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dgtz", {{ 0xa0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dlez", {{ 0xa0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dltz", {{ 0xa0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.dnez", {{ 0xa0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.even", {{ 0xa0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.odd", {{ 0xa0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.weqz", {{ 0xa0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wgez", {{ 0xa002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wgtz", {{ 0xa002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wlez", {{ 0xa002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wltz", {{ 0xa002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.wnez", {{ 0xa0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lbz.s.deqz", {{ 0xa1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dgez", {{ 0xa1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dgtz", {{ 0xa1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dlez", {{ 0xa1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dltz", {{ 0xa1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.dnez", {{ 0xa1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.even", {{ 0xa1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.odd", {{ 0xa1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.weqz", {{ 0xa1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wgez", {{ 0xa102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wgtz", {{ 0xa102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wlez", {{ 0xa102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wltz", {{ 0xa102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.s.wnez", {{ 0xa1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.deqz", {{ 0xa3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dgez", {{ 0xa3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dgtz", {{ 0xa3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dlez", {{ 0xa3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dltz", {{ 0xa3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.dnez", {{ 0xa3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.even", {{ 0xa3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.odd", {{ 0xa3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.weqz", {{ 0xa3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wgez", {{ 0xa302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wgtz", {{ 0xa302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wlez", {{ 0xa302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wltz", {{ 0xa302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.us.wnez", {{ 0xa3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.deqz", {{ 0xa2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dgez", {{ 0xa2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dgtz", {{ 0xa2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dlez", {{ 0xa2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dltz", {{ 0xa2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.dnez", {{ 0xa2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.even", {{ 0xa2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.odd", {{ 0xa2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.weqz", {{ 0xa2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wgez", {{ 0xa202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wgtz", {{ 0xa202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wlez", {{ 0xa202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wltz", {{ 0xa202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.u.wnez", {{ 0xa2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.deqz", {{ 0xa0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dgez", {{ 0xa0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dgtz", {{ 0xa0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dlez", {{ 0xa0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dltz", {{ 0xa0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.dnez", {{ 0xa0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.even", {{ 0xa0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.odd", {{ 0xa0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.weqz", {{ 0xa0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wgez", {{ 0xa002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wgtz", {{ 0xa002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wlez", {{ 0xa002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wltz", {{ 0xa002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz.wnez", {{ 0xa0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LBZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lbz", {{ 0x20000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz.s", {{ 0x21000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz.u", {{ 0x22000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz.us", {{ 0x23000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s010_registerZ_simple */
	{"lbz", {{ 0xa0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz.s", {{ 0xa1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz.u", {{ 0xa2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz.us", {{ 0xa3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_s037_registerZ_double */
	{"lbz", {{ 0xa0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_w064_registerZ_triple */
	{"lbz.s", {{ 0xa1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_w064_registerZ_triple */
	{"lbz.u", {{ 0xa2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_w064_registerZ_triple */
	{"lbz.us", {{ 0xa3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LBZ_variant_registerW_w064_registerZ_triple */
	{"ld.s", {{ 0x3902e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.s.xs", {{ 0x3902f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.us", {{ 0x3b02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.us.xs", {{ 0x3b02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.u", {{ 0x3a02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.u.xs", {{ 0x3a02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld", {{ 0x3802e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.xs", {{ 0x3802f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_doscale_registerW_registerY_registerZ_simple */
	{"ld.s.deqz", {{ 0x39021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dgez", {{ 0x39023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dgtz", {{ 0x39025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dlez", {{ 0x39024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dltz", {{ 0x39022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.dnez", {{ 0x39020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.even", {{ 0x39027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.odd", {{ 0x39026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.weqz", {{ 0x39029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wgez", {{ 0x3902b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wgtz", {{ 0x3902d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wlez", {{ 0x3902c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wltz", {{ 0x3902a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.wnez", {{ 0x39028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.deqz", {{ 0x3b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dgez", {{ 0x3b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dgtz", {{ 0x3b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dlez", {{ 0x3b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dltz", {{ 0x3b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.dnez", {{ 0x3b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.even", {{ 0x3b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.odd", {{ 0x3b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.weqz", {{ 0x3b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wgez", {{ 0x3b02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wgtz", {{ 0x3b02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wlez", {{ 0x3b02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wltz", {{ 0x3b02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.us.wnez", {{ 0x3b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.deqz", {{ 0x3a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dgez", {{ 0x3a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dgtz", {{ 0x3a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dlez", {{ 0x3a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dltz", {{ 0x3a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.dnez", {{ 0x3a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.even", {{ 0x3a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.odd", {{ 0x3a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.weqz", {{ 0x3a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wgez", {{ 0x3a02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wgtz", {{ 0x3a02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wlez", {{ 0x3a02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wltz", {{ 0x3a02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.u.wnez", {{ 0x3a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.deqz", {{ 0x38021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dgez", {{ 0x38023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dgtz", {{ 0x38025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dlez", {{ 0x38024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dltz", {{ 0x38022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.dnez", {{ 0x38020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.even", {{ 0x38027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.odd", {{ 0x38026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.weqz", {{ 0x38029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wgez", {{ 0x3802b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wgtz", {{ 0x3802d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wlez", {{ 0x3802c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wltz", {{ 0x3802a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.wnez", {{ 0x38028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_registerZ_simple */
	{"ld.s.deqz", {{ 0xb9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dgez", {{ 0xb9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dgtz", {{ 0xb9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dlez", {{ 0xb9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dltz", {{ 0xb9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.dnez", {{ 0xb9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.even", {{ 0xb9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.odd", {{ 0xb9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.weqz", {{ 0xb9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wgez", {{ 0xb902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wgtz", {{ 0xb902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wlez", {{ 0xb902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wltz", {{ 0xb902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.wnez", {{ 0xb9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.deqz", {{ 0xbb021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dgez", {{ 0xbb023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dgtz", {{ 0xbb025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dlez", {{ 0xbb024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dltz", {{ 0xbb022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.dnez", {{ 0xbb020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.even", {{ 0xbb027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.odd", {{ 0xbb026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.weqz", {{ 0xbb029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wgez", {{ 0xbb02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wgtz", {{ 0xbb02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wlez", {{ 0xbb02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wltz", {{ 0xbb02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.us.wnez", {{ 0xbb028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.deqz", {{ 0xba021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dgez", {{ 0xba023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dgtz", {{ 0xba025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dlez", {{ 0xba024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dltz", {{ 0xba022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.dnez", {{ 0xba020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.even", {{ 0xba027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.odd", {{ 0xba026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.weqz", {{ 0xba029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wgez", {{ 0xba02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wgtz", {{ 0xba02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wlez", {{ 0xba02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wltz", {{ 0xba02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.u.wnez", {{ 0xba028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.deqz", {{ 0xb8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dgez", {{ 0xb8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dgtz", {{ 0xb8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dlez", {{ 0xb8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dltz", {{ 0xb8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.dnez", {{ 0xb8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.even", {{ 0xb8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.odd", {{ 0xb8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.weqz", {{ 0xb8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wgez", {{ 0xb802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wgtz", {{ 0xb802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wlez", {{ 0xb802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wltz", {{ 0xb802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.wnez", {{ 0xb8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"ld.s.deqz", {{ 0xb9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dgez", {{ 0xb9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dgtz", {{ 0xb9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dlez", {{ 0xb9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dltz", {{ 0xb9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.dnez", {{ 0xb9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.even", {{ 0xb9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.odd", {{ 0xb9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.weqz", {{ 0xb9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wgez", {{ 0xb902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wgtz", {{ 0xb902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wlez", {{ 0xb902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wltz", {{ 0xb902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.s.wnez", {{ 0xb9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.deqz", {{ 0xbb021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dgez", {{ 0xbb023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dgtz", {{ 0xbb025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dlez", {{ 0xbb024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dltz", {{ 0xbb022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.dnez", {{ 0xbb020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.even", {{ 0xbb027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.odd", {{ 0xbb026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.weqz", {{ 0xbb029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wgez", {{ 0xbb02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wgtz", {{ 0xbb02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wlez", {{ 0xbb02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wltz", {{ 0xbb02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.us.wnez", {{ 0xbb028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.deqz", {{ 0xba021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dgez", {{ 0xba023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dgtz", {{ 0xba025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dlez", {{ 0xba024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dltz", {{ 0xba022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.dnez", {{ 0xba020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.even", {{ 0xba027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.odd", {{ 0xba026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.weqz", {{ 0xba029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wgez", {{ 0xba02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wgtz", {{ 0xba02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wlez", {{ 0xba02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wltz", {{ 0xba02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.u.wnez", {{ 0xba028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.deqz", {{ 0xb8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dgez", {{ 0xb8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dgtz", {{ 0xb8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dlez", {{ 0xb8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dltz", {{ 0xb8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.dnez", {{ 0xb8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.even", {{ 0xb8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.odd", {{ 0xb8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.weqz", {{ 0xb8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wgez", {{ 0xb802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wgtz", {{ 0xb802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wlez", {{ 0xb802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wltz", {{ 0xb802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld.wnez", {{ 0xb8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LD_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"ld", {{ 0x38000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s010_registerZ_simple */
	{"ld.s", {{ 0x39000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s010_registerZ_simple */
	{"ld.u", {{ 0x3a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s010_registerZ_simple */
	{"ld.us", {{ 0x3b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s010_registerZ_simple */
	{"ld", {{ 0xb8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s037_registerZ_double */
	{"ld.s", {{ 0xb9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s037_registerZ_double */
	{"ld.u", {{ 0xba000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s037_registerZ_double */
	{"ld.us", {{ 0xbb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_s037_registerZ_double */
	{"ld", {{ 0xb8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_w064_registerZ_triple */
	{"ld.s", {{ 0xb9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_w064_registerZ_triple */
	{"ld.u", {{ 0xba000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_w064_registerZ_triple */
	{"ld.us", {{ 0xbb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LD_variant_registerW_w064_registerZ_triple */
	{"lhs.s", {{ 0x2d02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.s.xs", {{ 0x2d02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.us", {{ 0x2f02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.us.xs", {{ 0x2f02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.u", {{ 0x2e02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.u.xs", {{ 0x2e02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs", {{ 0x2c02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.xs", {{ 0x2c02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhs.s.deqz", {{ 0x2d021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dgez", {{ 0x2d023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dgtz", {{ 0x2d025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dlez", {{ 0x2d024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dltz", {{ 0x2d022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.dnez", {{ 0x2d020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.even", {{ 0x2d027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.odd", {{ 0x2d026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.weqz", {{ 0x2d029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wgez", {{ 0x2d02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wgtz", {{ 0x2d02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wlez", {{ 0x2d02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wltz", {{ 0x2d02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.wnez", {{ 0x2d028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.deqz", {{ 0x2f021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dgez", {{ 0x2f023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dgtz", {{ 0x2f025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dlez", {{ 0x2f024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dltz", {{ 0x2f022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.dnez", {{ 0x2f020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.even", {{ 0x2f027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.odd", {{ 0x2f026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.weqz", {{ 0x2f029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wgez", {{ 0x2f02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wgtz", {{ 0x2f02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wlez", {{ 0x2f02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wltz", {{ 0x2f02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.us.wnez", {{ 0x2f028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.deqz", {{ 0x2e021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dgez", {{ 0x2e023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dgtz", {{ 0x2e025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dlez", {{ 0x2e024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dltz", {{ 0x2e022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.dnez", {{ 0x2e020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.even", {{ 0x2e027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.odd", {{ 0x2e026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.weqz", {{ 0x2e029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wgez", {{ 0x2e02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wgtz", {{ 0x2e02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wlez", {{ 0x2e02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wltz", {{ 0x2e02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.u.wnez", {{ 0x2e028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.deqz", {{ 0x2c021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dgez", {{ 0x2c023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dgtz", {{ 0x2c025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dlez", {{ 0x2c024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dltz", {{ 0x2c022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.dnez", {{ 0x2c020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.even", {{ 0x2c027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.odd", {{ 0x2c026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.weqz", {{ 0x2c029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wgez", {{ 0x2c02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wgtz", {{ 0x2c02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wlez", {{ 0x2c02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wltz", {{ 0x2c02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.wnez", {{ 0x2c028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhs.s.deqz", {{ 0xad021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dgez", {{ 0xad023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dgtz", {{ 0xad025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dlez", {{ 0xad024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dltz", {{ 0xad022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.dnez", {{ 0xad020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.even", {{ 0xad027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.odd", {{ 0xad026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.weqz", {{ 0xad029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wgez", {{ 0xad02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wgtz", {{ 0xad02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wlez", {{ 0xad02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wltz", {{ 0xad02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.wnez", {{ 0xad028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.deqz", {{ 0xaf021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dgez", {{ 0xaf023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dgtz", {{ 0xaf025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dlez", {{ 0xaf024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dltz", {{ 0xaf022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.dnez", {{ 0xaf020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.even", {{ 0xaf027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.odd", {{ 0xaf026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.weqz", {{ 0xaf029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wgez", {{ 0xaf02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wgtz", {{ 0xaf02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wlez", {{ 0xaf02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wltz", {{ 0xaf02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.us.wnez", {{ 0xaf028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.deqz", {{ 0xae021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dgez", {{ 0xae023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dgtz", {{ 0xae025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dlez", {{ 0xae024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dltz", {{ 0xae022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.dnez", {{ 0xae020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.even", {{ 0xae027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.odd", {{ 0xae026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.weqz", {{ 0xae029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wgez", {{ 0xae02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wgtz", {{ 0xae02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wlez", {{ 0xae02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wltz", {{ 0xae02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.u.wnez", {{ 0xae028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.deqz", {{ 0xac021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dgez", {{ 0xac023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dgtz", {{ 0xac025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dlez", {{ 0xac024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dltz", {{ 0xac022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.dnez", {{ 0xac020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.even", {{ 0xac027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.odd", {{ 0xac026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.weqz", {{ 0xac029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wgez", {{ 0xac02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wgtz", {{ 0xac02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wlez", {{ 0xac02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wltz", {{ 0xac02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.wnez", {{ 0xac028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhs.s.deqz", {{ 0xad021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dgez", {{ 0xad023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dgtz", {{ 0xad025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dlez", {{ 0xad024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dltz", {{ 0xad022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.dnez", {{ 0xad020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.even", {{ 0xad027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.odd", {{ 0xad026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.weqz", {{ 0xad029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wgez", {{ 0xad02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wgtz", {{ 0xad02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wlez", {{ 0xad02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wltz", {{ 0xad02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.s.wnez", {{ 0xad028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.deqz", {{ 0xaf021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dgez", {{ 0xaf023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dgtz", {{ 0xaf025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dlez", {{ 0xaf024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dltz", {{ 0xaf022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.dnez", {{ 0xaf020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.even", {{ 0xaf027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.odd", {{ 0xaf026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.weqz", {{ 0xaf029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wgez", {{ 0xaf02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wgtz", {{ 0xaf02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wlez", {{ 0xaf02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wltz", {{ 0xaf02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.us.wnez", {{ 0xaf028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.deqz", {{ 0xae021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dgez", {{ 0xae023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dgtz", {{ 0xae025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dlez", {{ 0xae024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dltz", {{ 0xae022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.dnez", {{ 0xae020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.even", {{ 0xae027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.odd", {{ 0xae026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.weqz", {{ 0xae029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wgez", {{ 0xae02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wgtz", {{ 0xae02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wlez", {{ 0xae02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wltz", {{ 0xae02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.u.wnez", {{ 0xae028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.deqz", {{ 0xac021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dgez", {{ 0xac023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dgtz", {{ 0xac025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dlez", {{ 0xac024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dltz", {{ 0xac022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.dnez", {{ 0xac020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.even", {{ 0xac027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.odd", {{ 0xac026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.weqz", {{ 0xac029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wgez", {{ 0xac02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wgtz", {{ 0xac02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wlez", {{ 0xac02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wltz", {{ 0xac02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs.wnez", {{ 0xac028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhs", {{ 0x2c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs.s", {{ 0x2d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs.u", {{ 0x2e000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs.us", {{ 0x2f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s010_registerZ_simple */
	{"lhs", {{ 0xac000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s037_registerZ_double */
	{"lhs.s", {{ 0xad000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s037_registerZ_double */
	{"lhs.u", {{ 0xae000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s037_registerZ_double */
	{"lhs.us", {{ 0xaf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_s037_registerZ_double */
	{"lhs", {{ 0xac000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_w064_registerZ_triple */
	{"lhs.s", {{ 0xad000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_w064_registerZ_triple */
	{"lhs.u", {{ 0xae000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_w064_registerZ_triple */
	{"lhs.us", {{ 0xaf000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHS_variant_registerW_w064_registerZ_triple */
	{"lhz.s", {{ 0x2902e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.s.xs", {{ 0x2902f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.us", {{ 0x2b02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.us.xs", {{ 0x2b02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.u", {{ 0x2a02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.u.xs", {{ 0x2a02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz", {{ 0x2802e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.xs", {{ 0x2802f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lhz.s.deqz", {{ 0x29021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dgez", {{ 0x29023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dgtz", {{ 0x29025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dlez", {{ 0x29024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dltz", {{ 0x29022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.dnez", {{ 0x29020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.even", {{ 0x29027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.odd", {{ 0x29026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.weqz", {{ 0x29029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wgez", {{ 0x2902b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wgtz", {{ 0x2902d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wlez", {{ 0x2902c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wltz", {{ 0x2902a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.wnez", {{ 0x29028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.deqz", {{ 0x2b021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dgez", {{ 0x2b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dgtz", {{ 0x2b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dlez", {{ 0x2b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dltz", {{ 0x2b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.dnez", {{ 0x2b020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.even", {{ 0x2b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.odd", {{ 0x2b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.weqz", {{ 0x2b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wgez", {{ 0x2b02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wgtz", {{ 0x2b02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wlez", {{ 0x2b02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wltz", {{ 0x2b02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.us.wnez", {{ 0x2b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.deqz", {{ 0x2a021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dgez", {{ 0x2a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dgtz", {{ 0x2a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dlez", {{ 0x2a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dltz", {{ 0x2a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.dnez", {{ 0x2a020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.even", {{ 0x2a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.odd", {{ 0x2a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.weqz", {{ 0x2a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wgez", {{ 0x2a02b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wgtz", {{ 0x2a02d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wlez", {{ 0x2a02c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wltz", {{ 0x2a02a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.u.wnez", {{ 0x2a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.deqz", {{ 0x28021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dgez", {{ 0x28023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dgtz", {{ 0x28025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dlez", {{ 0x28024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dltz", {{ 0x28022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.dnez", {{ 0x28020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.even", {{ 0x28027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.odd", {{ 0x28026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.weqz", {{ 0x28029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wgez", {{ 0x2802b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wgtz", {{ 0x2802d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wlez", {{ 0x2802c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wltz", {{ 0x2802a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.wnez", {{ 0x28028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lhz.s.deqz", {{ 0xa9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dgez", {{ 0xa9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dgtz", {{ 0xa9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dlez", {{ 0xa9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dltz", {{ 0xa9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.dnez", {{ 0xa9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.even", {{ 0xa9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.odd", {{ 0xa9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.weqz", {{ 0xa9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wgez", {{ 0xa902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wgtz", {{ 0xa902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wlez", {{ 0xa902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wltz", {{ 0xa902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.wnez", {{ 0xa9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.deqz", {{ 0xab021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dgez", {{ 0xab023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dgtz", {{ 0xab025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dlez", {{ 0xab024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dltz", {{ 0xab022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.dnez", {{ 0xab020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.even", {{ 0xab027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.odd", {{ 0xab026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.weqz", {{ 0xab029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wgez", {{ 0xab02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wgtz", {{ 0xab02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wlez", {{ 0xab02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wltz", {{ 0xab02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.us.wnez", {{ 0xab028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.deqz", {{ 0xaa021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dgez", {{ 0xaa023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dgtz", {{ 0xaa025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dlez", {{ 0xaa024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dltz", {{ 0xaa022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.dnez", {{ 0xaa020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.even", {{ 0xaa027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.odd", {{ 0xaa026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.weqz", {{ 0xaa029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wgez", {{ 0xaa02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wgtz", {{ 0xaa02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wlez", {{ 0xaa02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wltz", {{ 0xaa02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.u.wnez", {{ 0xaa028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.deqz", {{ 0xa8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dgez", {{ 0xa8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dgtz", {{ 0xa8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dlez", {{ 0xa8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dltz", {{ 0xa8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.dnez", {{ 0xa8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.even", {{ 0xa8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.odd", {{ 0xa8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.weqz", {{ 0xa8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wgez", {{ 0xa802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wgtz", {{ 0xa802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wlez", {{ 0xa802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wltz", {{ 0xa802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.wnez", {{ 0xa8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lhz.s.deqz", {{ 0xa9021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dgez", {{ 0xa9023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dgtz", {{ 0xa9025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dlez", {{ 0xa9024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dltz", {{ 0xa9022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.dnez", {{ 0xa9020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.even", {{ 0xa9027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.odd", {{ 0xa9026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.weqz", {{ 0xa9029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wgez", {{ 0xa902b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wgtz", {{ 0xa902d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wlez", {{ 0xa902c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wltz", {{ 0xa902a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.s.wnez", {{ 0xa9028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.deqz", {{ 0xab021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dgez", {{ 0xab023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dgtz", {{ 0xab025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dlez", {{ 0xab024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dltz", {{ 0xab022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.dnez", {{ 0xab020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.even", {{ 0xab027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.odd", {{ 0xab026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.weqz", {{ 0xab029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wgez", {{ 0xab02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wgtz", {{ 0xab02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wlez", {{ 0xab02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wltz", {{ 0xab02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.us.wnez", {{ 0xab028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.deqz", {{ 0xaa021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dgez", {{ 0xaa023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dgtz", {{ 0xaa025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dlez", {{ 0xaa024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dltz", {{ 0xaa022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.dnez", {{ 0xaa020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.even", {{ 0xaa027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.odd", {{ 0xaa026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.weqz", {{ 0xaa029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wgez", {{ 0xaa02b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wgtz", {{ 0xaa02d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wlez", {{ 0xaa02c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wltz", {{ 0xaa02a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.u.wnez", {{ 0xaa028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.deqz", {{ 0xa8021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dgez", {{ 0xa8023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dgtz", {{ 0xa8025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dlez", {{ 0xa8024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dltz", {{ 0xa8022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.dnez", {{ 0xa8020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.even", {{ 0xa8027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.odd", {{ 0xa8026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.weqz", {{ 0xa8029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wgez", {{ 0xa802b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wgtz", {{ 0xa802d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wlez", {{ 0xa802c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wltz", {{ 0xa802a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz.wnez", {{ 0xa8028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LHZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lhz", {{ 0x28000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz.s", {{ 0x29000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz.u", {{ 0x2a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz.us", {{ 0x2b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s010_registerZ_simple */
	{"lhz", {{ 0xa8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz.s", {{ 0xa9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz.u", {{ 0xaa000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz.us", {{ 0xab000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_s037_registerZ_double */
	{"lhz", {{ 0xa8000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_w064_registerZ_triple */
	{"lhz.s", {{ 0xa9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_w064_registerZ_triple */
	{"lhz.u", {{ 0xaa000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_w064_registerZ_triple */
	{"lhz.us", {{ 0xab000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LHZ_variant_registerW_w064_registerZ_triple */
	{"lnandd", {{ 0x7d014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDD_registerW_registerZ_registerY_simple */
	{"lnandd", {{ 0xfd014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDD_splat32_registerW_registerZ_w032_double */
	{"lnandd.@", {{ 0xfd014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDD_splat32_registerW_registerZ_w032_double */
	{"lnandhq", {{ 0x7d017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDHQ_registerW_registerZ_registerY_simple */
	{"lnandhq", {{ 0xfd017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDHQ_splat32_registerW_registerZ_w032_double */
	{"lnandhq.@", {{ 0xfd017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDHQ_splat32_registerW_registerZ_w032_double */
	{"lnandwp", {{ 0x7d016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDWP_registerW_registerZ_registerY_simple */
	{"lnandwp", {{ 0xfd016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDWP_splat32_registerW_registerZ_w032_double */
	{"lnandwp.@", {{ 0xfd016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDWP_splat32_registerW_registerZ_w032_double */
	{"lnandw", {{ 0x7d015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDW_registerW_registerZ_registerY_simple */
	{"lnandw", {{ 0xfd015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNANDW_registerW_registerZ_w032_double */
	{"lnord", {{ 0x7f014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORD_registerW_registerZ_registerY_simple */
	{"lnord", {{ 0xff014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORD_splat32_registerW_registerZ_w032_double */
	{"lnord.@", {{ 0xff014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORD_splat32_registerW_registerZ_w032_double */
	{"lnorhq", {{ 0x7f017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORHQ_registerW_registerZ_registerY_simple */
	{"lnorhq", {{ 0xff017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORHQ_splat32_registerW_registerZ_w032_double */
	{"lnorhq.@", {{ 0xff017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORHQ_splat32_registerW_registerZ_w032_double */
	{"lnorwp", {{ 0x7f016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORWP_registerW_registerZ_registerY_simple */
	{"lnorwp", {{ 0xff016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORWP_splat32_registerW_registerZ_w032_double */
	{"lnorwp.@", {{ 0xff016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORWP_splat32_registerW_registerZ_w032_double */
	{"lnorw", {{ 0x7f015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORW_registerW_registerZ_registerY_simple */
	{"lnorw", {{ 0xff015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LNORW_registerW_registerZ_w032_double */
	{"loopdo", {{ 0x0f000000, 0x7f800000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_registerz_opnd, &kv3_pcrel17_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-LOOPDO_registerZ_s017_simple */
	{"lord", {{ 0x7e014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORD_registerW_registerZ_registerY_simple */
	{"lord", {{ 0xfe014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORD_splat32_registerW_registerZ_w032_double */
	{"lord.@", {{ 0xfe014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORD_splat32_registerW_registerZ_w032_double */
	{"lorhq", {{ 0x7e017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORHQ_registerW_registerZ_registerY_simple */
	{"lorhq", {{ 0xfe017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORHQ_splat32_registerW_registerZ_w032_double */
	{"lorhq.@", {{ 0xfe017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORHQ_splat32_registerW_registerZ_w032_double */
	{"lorwp", {{ 0x7e016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORWP_registerW_registerZ_registerY_simple */
	{"lorwp", {{ 0xfe016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORWP_splat32_registerW_registerZ_w032_double */
	{"lorwp.@", {{ 0xfe016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORWP_splat32_registerW_registerZ_w032_double */
	{"lorw", {{ 0x7e015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORW_registerW_registerZ_registerY_simple */
	{"lorw", {{ 0xfe015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-LORW_registerW_registerZ_w032_double */
	{"lo.s", {{ 0x3d06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.s.xs", {{ 0x3d06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.us", {{ 0x3f06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.us.xs", {{ 0x3f06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.u", {{ 0x3e06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.u.xs", {{ 0x3e06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo", {{ 0x3c06e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.xs", {{ 0x3c06f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_doscale_registerN_registerY_registerZ_simple */
	{"lo.s.deqz", {{ 0x3d061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dgez", {{ 0x3d063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dgtz", {{ 0x3d065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dlez", {{ 0x3d064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dltz", {{ 0x3d062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.dnez", {{ 0x3d060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.even", {{ 0x3d067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.odd", {{ 0x3d066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.weqz", {{ 0x3d069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wgez", {{ 0x3d06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wgtz", {{ 0x3d06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wlez", {{ 0x3d06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wltz", {{ 0x3d06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.wnez", {{ 0x3d068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.deqz", {{ 0x3f061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dgez", {{ 0x3f063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dgtz", {{ 0x3f065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dlez", {{ 0x3f064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dltz", {{ 0x3f062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.dnez", {{ 0x3f060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.even", {{ 0x3f067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.odd", {{ 0x3f066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.weqz", {{ 0x3f069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wgez", {{ 0x3f06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wgtz", {{ 0x3f06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wlez", {{ 0x3f06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wltz", {{ 0x3f06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.us.wnez", {{ 0x3f068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.deqz", {{ 0x3e061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dgez", {{ 0x3e063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dgtz", {{ 0x3e065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dlez", {{ 0x3e064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dltz", {{ 0x3e062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.dnez", {{ 0x3e060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.even", {{ 0x3e067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.odd", {{ 0x3e066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.weqz", {{ 0x3e069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wgez", {{ 0x3e06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wgtz", {{ 0x3e06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wlez", {{ 0x3e06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wltz", {{ 0x3e06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.u.wnez", {{ 0x3e068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.deqz", {{ 0x3c061000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dgez", {{ 0x3c063000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dgtz", {{ 0x3c065000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dlez", {{ 0x3c064000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dltz", {{ 0x3c062000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.dnez", {{ 0x3c060000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.even", {{ 0x3c067000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.odd", {{ 0x3c066000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.weqz", {{ 0x3c069000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wgez", {{ 0x3c06b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wgtz", {{ 0x3c06d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wlez", {{ 0x3c06c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wltz", {{ 0x3c06a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.wnez", {{ 0x3c068000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_registerZ_simple */
	{"lo.s.deqz", {{ 0xbd061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dgez", {{ 0xbd063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dgtz", {{ 0xbd065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dlez", {{ 0xbd064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dltz", {{ 0xbd062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.dnez", {{ 0xbd060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.even", {{ 0xbd067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.odd", {{ 0xbd066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.weqz", {{ 0xbd069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wgez", {{ 0xbd06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wgtz", {{ 0xbd06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wlez", {{ 0xbd06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wltz", {{ 0xbd06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.wnez", {{ 0xbd068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.deqz", {{ 0xbf061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dgez", {{ 0xbf063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dgtz", {{ 0xbf065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dlez", {{ 0xbf064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dltz", {{ 0xbf062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.dnez", {{ 0xbf060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.even", {{ 0xbf067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.odd", {{ 0xbf066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.weqz", {{ 0xbf069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wgez", {{ 0xbf06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wgtz", {{ 0xbf06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wlez", {{ 0xbf06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wltz", {{ 0xbf06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.us.wnez", {{ 0xbf068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.deqz", {{ 0xbe061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dgez", {{ 0xbe063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dgtz", {{ 0xbe065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dlez", {{ 0xbe064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dltz", {{ 0xbe062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.dnez", {{ 0xbe060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.even", {{ 0xbe067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.odd", {{ 0xbe066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.weqz", {{ 0xbe069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wgez", {{ 0xbe06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wgtz", {{ 0xbe06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wlez", {{ 0xbe06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wltz", {{ 0xbe06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.u.wnez", {{ 0xbe068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.deqz", {{ 0xbc061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dgez", {{ 0xbc063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dgtz", {{ 0xbc065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dlez", {{ 0xbc064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dltz", {{ 0xbc062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.dnez", {{ 0xbc060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.even", {{ 0xbc067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.odd", {{ 0xbc066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.weqz", {{ 0xbc069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wgez", {{ 0xbc06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wgtz", {{ 0xbc06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wlez", {{ 0xbc06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wltz", {{ 0xbc06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.wnez", {{ 0xbc068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s027_registerZ_double */
	{"lo.s.deqz", {{ 0xbd061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dgez", {{ 0xbd063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dgtz", {{ 0xbd065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dlez", {{ 0xbd064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dltz", {{ 0xbd062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.dnez", {{ 0xbd060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.even", {{ 0xbd067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.odd", {{ 0xbd066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.weqz", {{ 0xbd069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wgez", {{ 0xbd06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wgtz", {{ 0xbd06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wlez", {{ 0xbd06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wltz", {{ 0xbd06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.s.wnez", {{ 0xbd068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.deqz", {{ 0xbf061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dgez", {{ 0xbf063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dgtz", {{ 0xbf065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dlez", {{ 0xbf064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dltz", {{ 0xbf062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.dnez", {{ 0xbf060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.even", {{ 0xbf067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.odd", {{ 0xbf066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.weqz", {{ 0xbf069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wgez", {{ 0xbf06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wgtz", {{ 0xbf06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wlez", {{ 0xbf06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wltz", {{ 0xbf06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.us.wnez", {{ 0xbf068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.deqz", {{ 0xbe061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dgez", {{ 0xbe063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dgtz", {{ 0xbe065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dlez", {{ 0xbe064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dltz", {{ 0xbe062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.dnez", {{ 0xbe060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.even", {{ 0xbe067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.odd", {{ 0xbe066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.weqz", {{ 0xbe069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wgez", {{ 0xbe06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wgtz", {{ 0xbe06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wlez", {{ 0xbe06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wltz", {{ 0xbe06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.u.wnez", {{ 0xbe068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.deqz", {{ 0xbc061000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dgez", {{ 0xbc063000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dgtz", {{ 0xbc065000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dlez", {{ 0xbc064000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dltz", {{ 0xbc062000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.dnez", {{ 0xbc060000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.even", {{ 0xbc067000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.odd", {{ 0xbc066000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.weqz", {{ 0xbc069000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wgez", {{ 0xbc06b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wgtz", {{ 0xbc06d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wlez", {{ 0xbc06c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wltz", {{ 0xbc06a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo.wnez", {{ 0xbc068000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registern_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LO_variant_lsucond_registerY_registerN_s054_registerZ_triple */
	{"lo", {{ 0x3c040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s010_registerZ_simple */
	{"lo.s", {{ 0x3d040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s010_registerZ_simple */
	{"lo.u", {{ 0x3e040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s010_registerZ_simple */
	{"lo.us", {{ 0x3f040000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registern_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s010_registerZ_simple */
	{"lo", {{ 0xbc040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s037_registerZ_double */
	{"lo.s", {{ 0xbd040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s037_registerZ_double */
	{"lo.u", {{ 0xbe040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s037_registerZ_double */
	{"lo.us", {{ 0xbf040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registern_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_s037_registerZ_double */
	{"lo", {{ 0xbc040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_w064_registerZ_triple */
	{"lo.s", {{ 0xbd040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_w064_registerZ_triple */
	{"lo.u", {{ 0xbe040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_w064_registerZ_triple */
	{"lo.us", {{ 0xbf040000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registern_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LO_variant_registerN_w064_registerZ_triple */
	{"lq.s", {{ 0x3d02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.s.xs", {{ 0x3d02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.us", {{ 0x3f02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.us.xs", {{ 0x3f02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.u", {{ 0x3e02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.u.xs", {{ 0x3e02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq", {{ 0x3c02e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.xs", {{ 0x3c02f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_doscale_registerM_registerY_registerZ_simple */
	{"lq.s.deqz", {{ 0x3d021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dgez", {{ 0x3d023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dgtz", {{ 0x3d025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dlez", {{ 0x3d024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dltz", {{ 0x3d022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.dnez", {{ 0x3d020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.even", {{ 0x3d027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.odd", {{ 0x3d026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.weqz", {{ 0x3d029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wgez", {{ 0x3d02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wgtz", {{ 0x3d02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wlez", {{ 0x3d02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wltz", {{ 0x3d02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.wnez", {{ 0x3d028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.deqz", {{ 0x3f021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dgez", {{ 0x3f023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dgtz", {{ 0x3f025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dlez", {{ 0x3f024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dltz", {{ 0x3f022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.dnez", {{ 0x3f020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.even", {{ 0x3f027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.odd", {{ 0x3f026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.weqz", {{ 0x3f029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wgez", {{ 0x3f02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wgtz", {{ 0x3f02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wlez", {{ 0x3f02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wltz", {{ 0x3f02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.us.wnez", {{ 0x3f028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.deqz", {{ 0x3e021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dgez", {{ 0x3e023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dgtz", {{ 0x3e025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dlez", {{ 0x3e024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dltz", {{ 0x3e022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.dnez", {{ 0x3e020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.even", {{ 0x3e027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.odd", {{ 0x3e026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.weqz", {{ 0x3e029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wgez", {{ 0x3e02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wgtz", {{ 0x3e02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wlez", {{ 0x3e02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wltz", {{ 0x3e02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.u.wnez", {{ 0x3e028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.deqz", {{ 0x3c021000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dgez", {{ 0x3c023000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dgtz", {{ 0x3c025000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dlez", {{ 0x3c024000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dltz", {{ 0x3c022000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.dnez", {{ 0x3c020000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.even", {{ 0x3c027000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.odd", {{ 0x3c026000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.weqz", {{ 0x3c029000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wgez", {{ 0x3c02b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wgtz", {{ 0x3c02d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wlez", {{ 0x3c02c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wltz", {{ 0x3c02a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.wnez", {{ 0x3c028000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_registerZ_simple */
	{"lq.s.deqz", {{ 0xbd021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dgez", {{ 0xbd023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dgtz", {{ 0xbd025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dlez", {{ 0xbd024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dltz", {{ 0xbd022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.dnez", {{ 0xbd020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.even", {{ 0xbd027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.odd", {{ 0xbd026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.weqz", {{ 0xbd029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wgez", {{ 0xbd02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wgtz", {{ 0xbd02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wlez", {{ 0xbd02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wltz", {{ 0xbd02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.wnez", {{ 0xbd028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.deqz", {{ 0xbf021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dgez", {{ 0xbf023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dgtz", {{ 0xbf025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dlez", {{ 0xbf024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dltz", {{ 0xbf022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.dnez", {{ 0xbf020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.even", {{ 0xbf027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.odd", {{ 0xbf026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.weqz", {{ 0xbf029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wgez", {{ 0xbf02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wgtz", {{ 0xbf02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wlez", {{ 0xbf02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wltz", {{ 0xbf02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.us.wnez", {{ 0xbf028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.deqz", {{ 0xbe021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dgez", {{ 0xbe023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dgtz", {{ 0xbe025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dlez", {{ 0xbe024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dltz", {{ 0xbe022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.dnez", {{ 0xbe020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.even", {{ 0xbe027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.odd", {{ 0xbe026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.weqz", {{ 0xbe029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wgez", {{ 0xbe02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wgtz", {{ 0xbe02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wlez", {{ 0xbe02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wltz", {{ 0xbe02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.u.wnez", {{ 0xbe028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.deqz", {{ 0xbc021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dgez", {{ 0xbc023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dgtz", {{ 0xbc025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dlez", {{ 0xbc024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dltz", {{ 0xbc022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.dnez", {{ 0xbc020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.even", {{ 0xbc027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.odd", {{ 0xbc026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.weqz", {{ 0xbc029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wgez", {{ 0xbc02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wgtz", {{ 0xbc02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wlez", {{ 0xbc02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wltz", {{ 0xbc02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.wnez", {{ 0xbc028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s027_registerZ_double */
	{"lq.s.deqz", {{ 0xbd021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dgez", {{ 0xbd023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dgtz", {{ 0xbd025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dlez", {{ 0xbd024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dltz", {{ 0xbd022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.dnez", {{ 0xbd020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.even", {{ 0xbd027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.odd", {{ 0xbd026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.weqz", {{ 0xbd029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wgez", {{ 0xbd02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wgtz", {{ 0xbd02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wlez", {{ 0xbd02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wltz", {{ 0xbd02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.s.wnez", {{ 0xbd028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.deqz", {{ 0xbf021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dgez", {{ 0xbf023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dgtz", {{ 0xbf025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dlez", {{ 0xbf024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dltz", {{ 0xbf022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.dnez", {{ 0xbf020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.even", {{ 0xbf027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.odd", {{ 0xbf026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.weqz", {{ 0xbf029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wgez", {{ 0xbf02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wgtz", {{ 0xbf02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wlez", {{ 0xbf02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wltz", {{ 0xbf02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.us.wnez", {{ 0xbf028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.deqz", {{ 0xbe021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dgez", {{ 0xbe023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dgtz", {{ 0xbe025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dlez", {{ 0xbe024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dltz", {{ 0xbe022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.dnez", {{ 0xbe020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.even", {{ 0xbe027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.odd", {{ 0xbe026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.weqz", {{ 0xbe029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wgez", {{ 0xbe02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wgtz", {{ 0xbe02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wlez", {{ 0xbe02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wltz", {{ 0xbe02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.u.wnez", {{ 0xbe028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.deqz", {{ 0xbc021000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dgez", {{ 0xbc023000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dgtz", {{ 0xbc025000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dlez", {{ 0xbc024000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dltz", {{ 0xbc022000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.dnez", {{ 0xbc020000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.even", {{ 0xbc027000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.odd", {{ 0xbc026000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.weqz", {{ 0xbc029000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wgez", {{ 0xbc02b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wgtz", {{ 0xbc02d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wlez", {{ 0xbc02c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wltz", {{ 0xbc02a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq.wnez", {{ 0xbc028000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerm_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LQ_variant_lsucond_registerY_registerM_s054_registerZ_triple */
	{"lq", {{ 0x3c000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s010_registerZ_simple */
	{"lq.s", {{ 0x3d000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s010_registerZ_simple */
	{"lq.u", {{ 0x3e000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s010_registerZ_simple */
	{"lq.us", {{ 0x3f000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerm_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s010_registerZ_simple */
	{"lq", {{ 0xbc000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s037_registerZ_double */
	{"lq.s", {{ 0xbd000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s037_registerZ_double */
	{"lq.u", {{ 0xbe000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s037_registerZ_double */
	{"lq.us", {{ 0xbf000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerm_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_s037_registerZ_double */
	{"lq", {{ 0xbc000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_w064_registerZ_triple */
	{"lq.s", {{ 0xbd000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_w064_registerZ_triple */
	{"lq.u", {{ 0xbe000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_w064_registerZ_triple */
	{"lq.us", {{ 0xbf000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerm_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LQ_variant_registerM_w064_registerZ_triple */
	{"lv.c0.s", {{ 0x2303e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0.s.xs", {{ 0x2303f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0", {{ 0x2203e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0.xs", {{ 0x2203f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1.s", {{ 0x2307e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1.s.xs", {{ 0x2307f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1", {{ 0x2207e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c1.xs", {{ 0x2207f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2.s", {{ 0x230be000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2.s.xs", {{ 0x230bf000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2", {{ 0x220be000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c2.xs", {{ 0x220bf000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3.s", {{ 0x230fe000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3.s.xs", {{ 0x230ff000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3", {{ 0x220fe000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c3.xs", {{ 0x220ff000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_doscale_registerAq_registerY_registerZ_simple */
	{"lv.c0.s.deqz", {{ 0x23031000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dgez", {{ 0x23033000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dgtz", {{ 0x23035000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dlez", {{ 0x23034000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dltz", {{ 0x23032000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.dnez", {{ 0x23030000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.even", {{ 0x23037000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.odd", {{ 0x23036000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.weqz", {{ 0x23039000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wgez", {{ 0x2303b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wgtz", {{ 0x2303d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wlez", {{ 0x2303c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wltz", {{ 0x2303a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.wnez", {{ 0x23038000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.deqz", {{ 0x22031000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dgez", {{ 0x22033000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dgtz", {{ 0x22035000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dlez", {{ 0x22034000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dltz", {{ 0x22032000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.dnez", {{ 0x22030000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.even", {{ 0x22037000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.odd", {{ 0x22036000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.weqz", {{ 0x22039000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wgez", {{ 0x2203b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wgtz", {{ 0x2203d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wlez", {{ 0x2203c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wltz", {{ 0x2203a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.wnez", {{ 0x22038000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.deqz", {{ 0x23071000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dgez", {{ 0x23073000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dgtz", {{ 0x23075000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dlez", {{ 0x23074000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dltz", {{ 0x23072000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.dnez", {{ 0x23070000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.even", {{ 0x23077000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.odd", {{ 0x23076000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.weqz", {{ 0x23079000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wgez", {{ 0x2307b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wgtz", {{ 0x2307d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wlez", {{ 0x2307c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wltz", {{ 0x2307a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.s.wnez", {{ 0x23078000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.deqz", {{ 0x22071000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dgez", {{ 0x22073000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dgtz", {{ 0x22075000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dlez", {{ 0x22074000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dltz", {{ 0x22072000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.dnez", {{ 0x22070000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.even", {{ 0x22077000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.odd", {{ 0x22076000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.weqz", {{ 0x22079000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wgez", {{ 0x2207b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wgtz", {{ 0x2207d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wlez", {{ 0x2207c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wltz", {{ 0x2207a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c1.wnez", {{ 0x22078000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.deqz", {{ 0x230b1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dgez", {{ 0x230b3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dgtz", {{ 0x230b5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dlez", {{ 0x230b4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dltz", {{ 0x230b2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.dnez", {{ 0x230b0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.even", {{ 0x230b7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.odd", {{ 0x230b6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.weqz", {{ 0x230b9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wgez", {{ 0x230bb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wgtz", {{ 0x230bd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wlez", {{ 0x230bc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wltz", {{ 0x230ba000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.s.wnez", {{ 0x230b8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.deqz", {{ 0x220b1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dgez", {{ 0x220b3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dgtz", {{ 0x220b5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dlez", {{ 0x220b4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dltz", {{ 0x220b2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.dnez", {{ 0x220b0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.even", {{ 0x220b7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.odd", {{ 0x220b6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.weqz", {{ 0x220b9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wgez", {{ 0x220bb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wgtz", {{ 0x220bd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wlez", {{ 0x220bc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wltz", {{ 0x220ba000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c2.wnez", {{ 0x220b8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.deqz", {{ 0x230f1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dgez", {{ 0x230f3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dgtz", {{ 0x230f5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dlez", {{ 0x230f4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dltz", {{ 0x230f2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.dnez", {{ 0x230f0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.even", {{ 0x230f7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.odd", {{ 0x230f6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.weqz", {{ 0x230f9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wgez", {{ 0x230fb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wgtz", {{ 0x230fd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wlez", {{ 0x230fc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wltz", {{ 0x230fa000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.s.wnez", {{ 0x230f8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.deqz", {{ 0x220f1000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dgez", {{ 0x220f3000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dgtz", {{ 0x220f5000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dlez", {{ 0x220f4000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dltz", {{ 0x220f2000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.dnez", {{ 0x220f0000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.even", {{ 0x220f7000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.odd", {{ 0x220f6000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.weqz", {{ 0x220f9000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wgez", {{ 0x220fb000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wgtz", {{ 0x220fd000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wlez", {{ 0x220fc000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wltz", {{ 0x220fa000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c3.wnez", {{ 0x220f8000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_registerZ_simple */
	{"lv.c0.s.deqz", {{ 0xa3031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dgez", {{ 0xa3033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dgtz", {{ 0xa3035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dlez", {{ 0xa3034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dltz", {{ 0xa3032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.dnez", {{ 0xa3030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.even", {{ 0xa3037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.odd", {{ 0xa3036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.weqz", {{ 0xa3039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wgez", {{ 0xa303b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wgtz", {{ 0xa303d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wlez", {{ 0xa303c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wltz", {{ 0xa303a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.wnez", {{ 0xa3038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.deqz", {{ 0xa2031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dgez", {{ 0xa2033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dgtz", {{ 0xa2035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dlez", {{ 0xa2034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dltz", {{ 0xa2032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.dnez", {{ 0xa2030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.even", {{ 0xa2037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.odd", {{ 0xa2036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.weqz", {{ 0xa2039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wgez", {{ 0xa203b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wgtz", {{ 0xa203d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wlez", {{ 0xa203c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wltz", {{ 0xa203a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.wnez", {{ 0xa2038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.deqz", {{ 0xa3071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dgez", {{ 0xa3073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dgtz", {{ 0xa3075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dlez", {{ 0xa3074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dltz", {{ 0xa3072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.dnez", {{ 0xa3070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.even", {{ 0xa3077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.odd", {{ 0xa3076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.weqz", {{ 0xa3079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wgez", {{ 0xa307b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wgtz", {{ 0xa307d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wlez", {{ 0xa307c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wltz", {{ 0xa307a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.s.wnez", {{ 0xa3078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.deqz", {{ 0xa2071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dgez", {{ 0xa2073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dgtz", {{ 0xa2075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dlez", {{ 0xa2074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dltz", {{ 0xa2072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.dnez", {{ 0xa2070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.even", {{ 0xa2077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.odd", {{ 0xa2076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.weqz", {{ 0xa2079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wgez", {{ 0xa207b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wgtz", {{ 0xa207d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wlez", {{ 0xa207c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wltz", {{ 0xa207a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c1.wnez", {{ 0xa2078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.deqz", {{ 0xa30b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dgez", {{ 0xa30b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dgtz", {{ 0xa30b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dlez", {{ 0xa30b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dltz", {{ 0xa30b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.dnez", {{ 0xa30b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.even", {{ 0xa30b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.odd", {{ 0xa30b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.weqz", {{ 0xa30b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wgez", {{ 0xa30bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wgtz", {{ 0xa30bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wlez", {{ 0xa30bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wltz", {{ 0xa30ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.s.wnez", {{ 0xa30b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.deqz", {{ 0xa20b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dgez", {{ 0xa20b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dgtz", {{ 0xa20b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dlez", {{ 0xa20b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dltz", {{ 0xa20b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.dnez", {{ 0xa20b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.even", {{ 0xa20b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.odd", {{ 0xa20b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.weqz", {{ 0xa20b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wgez", {{ 0xa20bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wgtz", {{ 0xa20bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wlez", {{ 0xa20bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wltz", {{ 0xa20ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c2.wnez", {{ 0xa20b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.deqz", {{ 0xa30f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dgez", {{ 0xa30f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dgtz", {{ 0xa30f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dlez", {{ 0xa30f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dltz", {{ 0xa30f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.dnez", {{ 0xa30f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.even", {{ 0xa30f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.odd", {{ 0xa30f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.weqz", {{ 0xa30f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wgez", {{ 0xa30fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wgtz", {{ 0xa30fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wlez", {{ 0xa30fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wltz", {{ 0xa30fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.s.wnez", {{ 0xa30f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.deqz", {{ 0xa20f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dgez", {{ 0xa20f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dgtz", {{ 0xa20f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dlez", {{ 0xa20f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dltz", {{ 0xa20f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.dnez", {{ 0xa20f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.even", {{ 0xa20f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.odd", {{ 0xa20f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.weqz", {{ 0xa20f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wgez", {{ 0xa20fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wgtz", {{ 0xa20fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wlez", {{ 0xa20fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wltz", {{ 0xa20fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c3.wnez", {{ 0xa20f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s027_registerZ_double */
	{"lv.c0.s.deqz", {{ 0xa3031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dgez", {{ 0xa3033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dgtz", {{ 0xa3035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dlez", {{ 0xa3034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dltz", {{ 0xa3032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.dnez", {{ 0xa3030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.even", {{ 0xa3037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.odd", {{ 0xa3036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.weqz", {{ 0xa3039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wgez", {{ 0xa303b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wgtz", {{ 0xa303d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wlez", {{ 0xa303c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wltz", {{ 0xa303a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.s.wnez", {{ 0xa3038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.deqz", {{ 0xa2031000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dgez", {{ 0xa2033000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dgtz", {{ 0xa2035000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dlez", {{ 0xa2034000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dltz", {{ 0xa2032000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.dnez", {{ 0xa2030000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.even", {{ 0xa2037000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.odd", {{ 0xa2036000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.weqz", {{ 0xa2039000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wgez", {{ 0xa203b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wgtz", {{ 0xa203d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wlez", {{ 0xa203c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wltz", {{ 0xa203a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0.wnez", {{ 0xa2038000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.deqz", {{ 0xa3071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dgez", {{ 0xa3073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dgtz", {{ 0xa3075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dlez", {{ 0xa3074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dltz", {{ 0xa3072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.dnez", {{ 0xa3070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.even", {{ 0xa3077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.odd", {{ 0xa3076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.weqz", {{ 0xa3079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wgez", {{ 0xa307b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wgtz", {{ 0xa307d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wlez", {{ 0xa307c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wltz", {{ 0xa307a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.s.wnez", {{ 0xa3078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.deqz", {{ 0xa2071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dgez", {{ 0xa2073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dgtz", {{ 0xa2075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dlez", {{ 0xa2074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dltz", {{ 0xa2072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.dnez", {{ 0xa2070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.even", {{ 0xa2077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.odd", {{ 0xa2076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.weqz", {{ 0xa2079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wgez", {{ 0xa207b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wgtz", {{ 0xa207d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wlez", {{ 0xa207c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wltz", {{ 0xa207a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c1.wnez", {{ 0xa2078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.deqz", {{ 0xa30b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dgez", {{ 0xa30b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dgtz", {{ 0xa30b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dlez", {{ 0xa30b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dltz", {{ 0xa30b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.dnez", {{ 0xa30b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.even", {{ 0xa30b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.odd", {{ 0xa30b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.weqz", {{ 0xa30b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wgez", {{ 0xa30bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wgtz", {{ 0xa30bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wlez", {{ 0xa30bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wltz", {{ 0xa30ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.s.wnez", {{ 0xa30b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.deqz", {{ 0xa20b1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dgez", {{ 0xa20b3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dgtz", {{ 0xa20b5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dlez", {{ 0xa20b4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dltz", {{ 0xa20b2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.dnez", {{ 0xa20b0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.even", {{ 0xa20b7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.odd", {{ 0xa20b6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.weqz", {{ 0xa20b9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wgez", {{ 0xa20bb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wgtz", {{ 0xa20bd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wlez", {{ 0xa20bc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wltz", {{ 0xa20ba000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c2.wnez", {{ 0xa20b8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.deqz", {{ 0xa30f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dgez", {{ 0xa30f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dgtz", {{ 0xa30f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dlez", {{ 0xa30f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dltz", {{ 0xa30f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.dnez", {{ 0xa30f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.even", {{ 0xa30f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.odd", {{ 0xa30f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.weqz", {{ 0xa30f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wgez", {{ 0xa30fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wgtz", {{ 0xa30fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wlez", {{ 0xa30fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wltz", {{ 0xa30fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.s.wnez", {{ 0xa30f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.deqz", {{ 0xa20f1000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dgez", {{ 0xa20f3000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dgtz", {{ 0xa20f5000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dlez", {{ 0xa20f4000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dltz", {{ 0xa20f2000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.dnez", {{ 0xa20f0000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.even", {{ 0xa20f7000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.odd", {{ 0xa20f6000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.weqz", {{ 0xa20f9000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wgez", {{ 0xa20fb000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wgtz", {{ 0xa20fd000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wlez", {{ 0xa20fc000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wltz", {{ 0xa20fa000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c3.wnez", {{ 0xa20f8000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registeraq_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_lsucond_registerY_registerAq_s054_registerZ_triple */
	{"lv.c0", {{ 0x22010000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c0.s", {{ 0x23010000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c1", {{ 0x22050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c1.s", {{ 0x23050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c2", {{ 0x22090000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c2.s", {{ 0x23090000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c3", {{ 0x220d0000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c3.s", {{ 0x230d0000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registeraq_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s010_registerZ_simple */
	{"lv.c0", {{ 0xa2010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c0.s", {{ 0xa3010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c1", {{ 0xa2050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c1.s", {{ 0xa3050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c2", {{ 0xa2090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c2.s", {{ 0xa3090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c3", {{ 0xa20d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c3.s", {{ 0xa30d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registeraq_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_s037_registerZ_double */
	{"lv.c0", {{ 0xa2010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c0.s", {{ 0xa3010000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c1", {{ 0xa2050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c1.s", {{ 0xa3050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c2", {{ 0xa2090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c2.s", {{ 0xa3090000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c3", {{ 0xa20d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.c3.s", {{ 0xa30d0000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registeraq_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_column_speculate_registerAq_w064_registerZ_triple */
	{"lv.s", {{ 0x2103e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv.s.xs", {{ 0x2103f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv", {{ 0x2003e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv.xs", {{ 0x2003f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_doscale_registerA_registerY_registerZ_simple */
	{"lv.s.deqz", {{ 0x21031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dgez", {{ 0x21033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dgtz", {{ 0x21035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dlez", {{ 0x21034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dltz", {{ 0x21032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.dnez", {{ 0x21030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.even", {{ 0x21037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.odd", {{ 0x21036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.weqz", {{ 0x21039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wgez", {{ 0x2103b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wgtz", {{ 0x2103d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wlez", {{ 0x2103c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wltz", {{ 0x2103a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.wnez", {{ 0x21038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.deqz", {{ 0x20031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dgez", {{ 0x20033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dgtz", {{ 0x20035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dlez", {{ 0x20034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dltz", {{ 0x20032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.dnez", {{ 0x20030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.even", {{ 0x20037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.odd", {{ 0x20036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.weqz", {{ 0x20039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wgez", {{ 0x2003b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wgtz", {{ 0x2003d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wlez", {{ 0x2003c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wltz", {{ 0x2003a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.wnez", {{ 0x20038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_registerZ_simple */
	{"lv.s.deqz", {{ 0xa1031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dgez", {{ 0xa1033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dgtz", {{ 0xa1035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dlez", {{ 0xa1034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dltz", {{ 0xa1032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.dnez", {{ 0xa1030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.even", {{ 0xa1037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.odd", {{ 0xa1036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.weqz", {{ 0xa1039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wgez", {{ 0xa103b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wgtz", {{ 0xa103d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wlez", {{ 0xa103c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wltz", {{ 0xa103a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.wnez", {{ 0xa1038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.deqz", {{ 0xa0031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dgez", {{ 0xa0033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dgtz", {{ 0xa0035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dlez", {{ 0xa0034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dltz", {{ 0xa0032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.dnez", {{ 0xa0030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.even", {{ 0xa0037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.odd", {{ 0xa0036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.weqz", {{ 0xa0039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wgez", {{ 0xa003b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wgtz", {{ 0xa003d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wlez", {{ 0xa003c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wltz", {{ 0xa003a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.wnez", {{ 0xa0038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s027_registerZ_double */
	{"lv.s.deqz", {{ 0xa1031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dgez", {{ 0xa1033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dgtz", {{ 0xa1035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dlez", {{ 0xa1034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dltz", {{ 0xa1032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.dnez", {{ 0xa1030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.even", {{ 0xa1037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.odd", {{ 0xa1036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.weqz", {{ 0xa1039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wgez", {{ 0xa103b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wgtz", {{ 0xa103d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wlez", {{ 0xa103c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wltz", {{ 0xa103a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.s.wnez", {{ 0xa1038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.deqz", {{ 0xa0031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dgez", {{ 0xa0033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dgtz", {{ 0xa0035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dlez", {{ 0xa0034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dltz", {{ 0xa0032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.dnez", {{ 0xa0030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.even", {{ 0xa0037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.odd", {{ 0xa0036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.weqz", {{ 0xa0039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wgez", {{ 0xa003b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wgtz", {{ 0xa003d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wlez", {{ 0xa003c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wltz", {{ 0xa003a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv.wnez", {{ 0xa0038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registery_opnd, &kv3_registera_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LV_speculate_lsucond_registerY_registerA_s054_registerZ_triple */
	{"lv", {{ 0x20010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_registerA_s010_registerZ_simple */
	{"lv.s", {{ 0x21010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU, {&kv3_registera_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_registerA_s010_registerZ_simple */
	{"lv", {{ 0xa0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registera_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_registerA_s037_registerZ_double */
	{"lv.s", {{ 0xa1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_X, {&kv3_registera_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_registerA_s037_registerZ_double */
	{"lv", {{ 0xa0010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registera_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_registerA_w064_registerZ_triple */
	{"lv.s", {{ 0xa1010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_Y, {&kv3_registera_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LV_speculate_registerA_w064_registerZ_triple */
	{"lws.s", {{ 0x3502e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.s.xs", {{ 0x3502f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.us", {{ 0x3702e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.us.xs", {{ 0x3702f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.u", {{ 0x3602e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.u.xs", {{ 0x3602f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws", {{ 0x3402e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.xs", {{ 0x3402f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_doscale_registerW_registerY_registerZ_simple */
	{"lws.s.deqz", {{ 0x35021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dgez", {{ 0x35023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dgtz", {{ 0x35025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dlez", {{ 0x35024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dltz", {{ 0x35022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.dnez", {{ 0x35020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.even", {{ 0x35027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.odd", {{ 0x35026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.weqz", {{ 0x35029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wgez", {{ 0x3502b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wgtz", {{ 0x3502d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wlez", {{ 0x3502c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wltz", {{ 0x3502a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.wnez", {{ 0x35028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.deqz", {{ 0x37021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dgez", {{ 0x37023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dgtz", {{ 0x37025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dlez", {{ 0x37024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dltz", {{ 0x37022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.dnez", {{ 0x37020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.even", {{ 0x37027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.odd", {{ 0x37026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.weqz", {{ 0x37029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wgez", {{ 0x3702b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wgtz", {{ 0x3702d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wlez", {{ 0x3702c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wltz", {{ 0x3702a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.us.wnez", {{ 0x37028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.deqz", {{ 0x36021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dgez", {{ 0x36023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dgtz", {{ 0x36025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dlez", {{ 0x36024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dltz", {{ 0x36022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.dnez", {{ 0x36020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.even", {{ 0x36027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.odd", {{ 0x36026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.weqz", {{ 0x36029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wgez", {{ 0x3602b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wgtz", {{ 0x3602d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wlez", {{ 0x3602c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wltz", {{ 0x3602a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.u.wnez", {{ 0x36028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.deqz", {{ 0x34021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dgez", {{ 0x34023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dgtz", {{ 0x34025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dlez", {{ 0x34024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dltz", {{ 0x34022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.dnez", {{ 0x34020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.even", {{ 0x34027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.odd", {{ 0x34026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.weqz", {{ 0x34029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wgez", {{ 0x3402b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wgtz", {{ 0x3402d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wlez", {{ 0x3402c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wltz", {{ 0x3402a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.wnez", {{ 0x34028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lws.s.deqz", {{ 0xb5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dgez", {{ 0xb5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dgtz", {{ 0xb5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dlez", {{ 0xb5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dltz", {{ 0xb5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.dnez", {{ 0xb5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.even", {{ 0xb5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.odd", {{ 0xb5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.weqz", {{ 0xb5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wgez", {{ 0xb502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wgtz", {{ 0xb502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wlez", {{ 0xb502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wltz", {{ 0xb502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.wnez", {{ 0xb5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.deqz", {{ 0xb7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dgez", {{ 0xb7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dgtz", {{ 0xb7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dlez", {{ 0xb7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dltz", {{ 0xb7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.dnez", {{ 0xb7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.even", {{ 0xb7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.odd", {{ 0xb7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.weqz", {{ 0xb7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wgez", {{ 0xb702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wgtz", {{ 0xb702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wlez", {{ 0xb702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wltz", {{ 0xb702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.us.wnez", {{ 0xb7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.deqz", {{ 0xb6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dgez", {{ 0xb6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dgtz", {{ 0xb6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dlez", {{ 0xb6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dltz", {{ 0xb6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.dnez", {{ 0xb6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.even", {{ 0xb6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.odd", {{ 0xb6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.weqz", {{ 0xb6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wgez", {{ 0xb602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wgtz", {{ 0xb602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wlez", {{ 0xb602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wltz", {{ 0xb602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.u.wnez", {{ 0xb6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.deqz", {{ 0xb4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dgez", {{ 0xb4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dgtz", {{ 0xb4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dlez", {{ 0xb4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dltz", {{ 0xb4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.dnez", {{ 0xb4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.even", {{ 0xb4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.odd", {{ 0xb4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.weqz", {{ 0xb4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wgez", {{ 0xb402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wgtz", {{ 0xb402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wlez", {{ 0xb402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wltz", {{ 0xb402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.wnez", {{ 0xb4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lws.s.deqz", {{ 0xb5021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dgez", {{ 0xb5023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dgtz", {{ 0xb5025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dlez", {{ 0xb5024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dltz", {{ 0xb5022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.dnez", {{ 0xb5020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.even", {{ 0xb5027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.odd", {{ 0xb5026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.weqz", {{ 0xb5029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wgez", {{ 0xb502b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wgtz", {{ 0xb502d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wlez", {{ 0xb502c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wltz", {{ 0xb502a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.s.wnez", {{ 0xb5028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.deqz", {{ 0xb7021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dgez", {{ 0xb7023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dgtz", {{ 0xb7025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dlez", {{ 0xb7024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dltz", {{ 0xb7022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.dnez", {{ 0xb7020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.even", {{ 0xb7027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.odd", {{ 0xb7026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.weqz", {{ 0xb7029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wgez", {{ 0xb702b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wgtz", {{ 0xb702d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wlez", {{ 0xb702c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wltz", {{ 0xb702a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.us.wnez", {{ 0xb7028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.deqz", {{ 0xb6021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dgez", {{ 0xb6023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dgtz", {{ 0xb6025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dlez", {{ 0xb6024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dltz", {{ 0xb6022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.dnez", {{ 0xb6020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.even", {{ 0xb6027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.odd", {{ 0xb6026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.weqz", {{ 0xb6029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wgez", {{ 0xb602b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wgtz", {{ 0xb602d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wlez", {{ 0xb602c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wltz", {{ 0xb602a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.u.wnez", {{ 0xb6028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.deqz", {{ 0xb4021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dgez", {{ 0xb4023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dgtz", {{ 0xb4025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dlez", {{ 0xb4024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dltz", {{ 0xb4022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.dnez", {{ 0xb4020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.even", {{ 0xb4027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.odd", {{ 0xb4026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.weqz", {{ 0xb4029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wgez", {{ 0xb402b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wgtz", {{ 0xb402d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wlez", {{ 0xb402c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wltz", {{ 0xb402a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws.wnez", {{ 0xb4028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWS_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lws", {{ 0x34000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s010_registerZ_simple */
	{"lws.s", {{ 0x35000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s010_registerZ_simple */
	{"lws.u", {{ 0x36000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s010_registerZ_simple */
	{"lws.us", {{ 0x37000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s010_registerZ_simple */
	{"lws", {{ 0xb4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s037_registerZ_double */
	{"lws.s", {{ 0xb5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s037_registerZ_double */
	{"lws.u", {{ 0xb6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s037_registerZ_double */
	{"lws.us", {{ 0xb7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_s037_registerZ_double */
	{"lws", {{ 0xb4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_w064_registerZ_triple */
	{"lws.s", {{ 0xb5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_w064_registerZ_triple */
	{"lws.u", {{ 0xb6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_w064_registerZ_triple */
	{"lws.us", {{ 0xb7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWS_variant_registerW_w064_registerZ_triple */
	{"lwz.s", {{ 0x3102e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.s.xs", {{ 0x3102f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.us", {{ 0x3302e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.us.xs", {{ 0x3302f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.u", {{ 0x3202e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.u.xs", {{ 0x3202f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz", {{ 0x3002e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.xs", {{ 0x3002f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_registery_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_doscale_registerW_registerY_registerZ_simple */
	{"lwz.s.deqz", {{ 0x31021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dgez", {{ 0x31023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dgtz", {{ 0x31025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dlez", {{ 0x31024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dltz", {{ 0x31022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.dnez", {{ 0x31020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.even", {{ 0x31027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.odd", {{ 0x31026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.weqz", {{ 0x31029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wgez", {{ 0x3102b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wgtz", {{ 0x3102d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wlez", {{ 0x3102c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wltz", {{ 0x3102a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.wnez", {{ 0x31028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.deqz", {{ 0x33021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dgez", {{ 0x33023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dgtz", {{ 0x33025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dlez", {{ 0x33024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dltz", {{ 0x33022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.dnez", {{ 0x33020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.even", {{ 0x33027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.odd", {{ 0x33026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.weqz", {{ 0x33029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wgez", {{ 0x3302b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wgtz", {{ 0x3302d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wlez", {{ 0x3302c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wltz", {{ 0x3302a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.us.wnez", {{ 0x33028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.deqz", {{ 0x32021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dgez", {{ 0x32023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dgtz", {{ 0x32025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dlez", {{ 0x32024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dltz", {{ 0x32022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.dnez", {{ 0x32020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.even", {{ 0x32027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.odd", {{ 0x32026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.weqz", {{ 0x32029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wgez", {{ 0x3202b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wgtz", {{ 0x3202d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wlez", {{ 0x3202c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wltz", {{ 0x3202a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.u.wnez", {{ 0x32028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.deqz", {{ 0x30021000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dgez", {{ 0x30023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dgtz", {{ 0x30025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dlez", {{ 0x30024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dltz", {{ 0x30022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.dnez", {{ 0x30020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.even", {{ 0x30027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.odd", {{ 0x30026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.weqz", {{ 0x30029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wgez", {{ 0x3002b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wgtz", {{ 0x3002d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wlez", {{ 0x3002c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wltz", {{ 0x3002a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.wnez", {{ 0x30028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = [%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_registerZ_simple */
	{"lwz.s.deqz", {{ 0xb1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dgez", {{ 0xb1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dgtz", {{ 0xb1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dlez", {{ 0xb1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dltz", {{ 0xb1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.dnez", {{ 0xb1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.even", {{ 0xb1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.odd", {{ 0xb1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.weqz", {{ 0xb1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wgez", {{ 0xb102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wgtz", {{ 0xb102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wlez", {{ 0xb102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wltz", {{ 0xb102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.wnez", {{ 0xb1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.deqz", {{ 0xb3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dgez", {{ 0xb3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dgtz", {{ 0xb3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dlez", {{ 0xb3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dltz", {{ 0xb3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.dnez", {{ 0xb3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.even", {{ 0xb3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.odd", {{ 0xb3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.weqz", {{ 0xb3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wgez", {{ 0xb302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wgtz", {{ 0xb302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wlez", {{ 0xb302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wltz", {{ 0xb302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.us.wnez", {{ 0xb3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.deqz", {{ 0xb2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dgez", {{ 0xb2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dgtz", {{ 0xb2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dlez", {{ 0xb2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dltz", {{ 0xb2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.dnez", {{ 0xb2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.even", {{ 0xb2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.odd", {{ 0xb2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.weqz", {{ 0xb2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wgez", {{ 0xb202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wgtz", {{ 0xb202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wlez", {{ 0xb202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wltz", {{ 0xb202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.u.wnez", {{ 0xb2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.deqz", {{ 0xb0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dgez", {{ 0xb0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dgtz", {{ 0xb0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dlez", {{ 0xb0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dltz", {{ 0xb0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.dnez", {{ 0xb0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.even", {{ 0xb0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.odd", {{ 0xb0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.weqz", {{ 0xb0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wgez", {{ 0xb002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wgtz", {{ 0xb002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wlez", {{ 0xb002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wltz", {{ 0xb002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.wnez", {{ 0xb0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s027_registerZ_double */
	{"lwz.s.deqz", {{ 0xb1021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dgez", {{ 0xb1023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dgtz", {{ 0xb1025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dlez", {{ 0xb1024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dltz", {{ 0xb1022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.dnez", {{ 0xb1020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.even", {{ 0xb1027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.odd", {{ 0xb1026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.weqz", {{ 0xb1029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wgez", {{ 0xb102b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wgtz", {{ 0xb102d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wlez", {{ 0xb102c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wltz", {{ 0xb102a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.s.wnez", {{ 0xb1028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.deqz", {{ 0xb3021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dgez", {{ 0xb3023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dgtz", {{ 0xb3025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dlez", {{ 0xb3024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dltz", {{ 0xb3022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.dnez", {{ 0xb3020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.even", {{ 0xb3027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.odd", {{ 0xb3026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.weqz", {{ 0xb3029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wgez", {{ 0xb302b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wgtz", {{ 0xb302d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wlez", {{ 0xb302c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wltz", {{ 0xb302a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.us.wnez", {{ 0xb3028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.deqz", {{ 0xb2021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dgez", {{ 0xb2023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dgtz", {{ 0xb2025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dlez", {{ 0xb2024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dltz", {{ 0xb2022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.dnez", {{ 0xb2020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.even", {{ 0xb2027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.odd", {{ 0xb2026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.weqz", {{ 0xb2029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wgez", {{ 0xb202b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wgtz", {{ 0xb202d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wlez", {{ 0xb202c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wltz", {{ 0xb202a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.u.wnez", {{ 0xb2028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.deqz", {{ 0xb0021000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dgez", {{ 0xb0023000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dgtz", {{ 0xb0025000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dlez", {{ 0xb0024000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dltz", {{ 0xb0022000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.dnez", {{ 0xb0020000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.even", {{ 0xb0027000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.odd", {{ 0xb0026000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.weqz", {{ 0xb0029000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wgez", {{ 0xb002b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wgtz", {{ 0xb002d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wlez", {{ 0xb002c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wltz", {{ 0xb002a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz.wnez", {{ 0xb0028000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registery_opnd, &kv3_registerw_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, 0}, "", "%s? %s = %s[%s]"}, /* Opcode-v2-LWZ_variant_lsucond_registerY_registerW_s054_registerZ_triple */
	{"lwz", {{ 0x30000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz.s", {{ 0x31000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz.u", {{ 0x32000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz.us", {{ 0x33000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW, {&kv3_registerw_opnd, &kv3_signed10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s010_registerZ_simple */
	{"lwz", {{ 0xb0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz.s", {{ 0xb1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz.u", {{ 0xb2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz.us", {{ 0xb3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_X, {&kv3_registerw_opnd, &kv3_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_s037_registerZ_double */
	{"lwz", {{ 0xb0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_w064_registerZ_triple */
	{"lwz.s", {{ 0xb1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_w064_registerZ_triple */
	{"lwz.u", {{ 0xb2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_w064_registerZ_triple */
	{"lwz.us", {{ 0xb3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXW_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s[%s]"}, /* Opcode-v2-LWZ_variant_registerW_w064_registerZ_triple */
	{"madddt", {{ 0x58001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDDT_registerM_registerZ_registerY_simple */
	{"madddt", {{ 0x48000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDDT_registerM_registerZ_s010_simple */
	{"madddt", {{ 0xc8000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDDT_registerM_registerZ_s037_double */
	{"madddt", {{ 0xc8000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDDT_registerM_registerZ_w064_triple */
	{"maddd", {{ 0x50000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDD_registerW_registerZ_registerY_simple */
	{"maddd", {{ 0x40000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDD_registerW_registerZ_s010_simple */
	{"maddd", {{ 0xc0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDD_registerW_registerZ_s037_double */
	{"maddd", {{ 0xc0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDD_registerW_registerZ_w064_triple */
	{"maddhq", {{ 0x52000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDHQ_registerW_registerZ_registerY_simple */
	{"maddhq", {{ 0x42000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDHQ_registerW_registerZ_s010_simple */
	{"maddhq", {{ 0xc2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDHQ_registerW_registerZ_s037_double */
	{"maddhq", {{ 0xc2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDHQ_registerW_registerZ_w064_triple */
	{"maddhwq", {{ 0x50004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDHWQ_registerM_registerZ_registerY_simple */
	{"maddsudt", {{ 0x5a001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUDT_registerM_registerZ_registerY_simple */
	{"maddsudt", {{ 0x4a000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUDT_registerM_registerZ_s010_simple */
	{"maddsudt", {{ 0xca000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUDT_registerM_registerZ_s037_double */
	{"maddsudt", {{ 0xca000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUDT_registerM_registerZ_w064_triple */
	{"maddsuhwq", {{ 0x52004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUHWQ_registerM_registerZ_registerY_simple */
	{"maddsuwdp", {{ 0x5a000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUWDP_registerM_registerZ_registerY_simple */
	{"maddsuwd", {{ 0x52003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUWD_registerW_registerZ_registerY_simple */
	{"maddsuwd", {{ 0xd2003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDSUWD_registerW_registerZ_w032_double */
	{"maddudt", {{ 0x59001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUDT_registerM_registerZ_registerY_simple */
	{"maddudt", {{ 0x49000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUDT_registerM_registerZ_s010_simple */
	{"maddudt", {{ 0xc9000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUDT_registerM_registerZ_s037_double */
	{"maddudt", {{ 0xc9000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUDT_registerM_registerZ_w064_triple */
	{"madduhwq", {{ 0x51004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUHWQ_registerM_registerZ_registerY_simple */
	{"madduwdp", {{ 0x59000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUWDP_registerM_registerZ_registerY_simple */
	{"madduwd", {{ 0x51003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUWD_registerW_registerZ_registerY_simple */
	{"madduwd", {{ 0xd1003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUWD_registerW_registerZ_w032_double */
	{"madduzdt", {{ 0x5b001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUZDT_registerM_registerZ_registerY_simple */
	{"madduzdt", {{ 0x4b000000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUZDT_registerM_registerZ_s010_simple */
	{"madduzdt", {{ 0xcb000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUZDT_registerM_registerZ_s037_double */
	{"madduzdt", {{ 0xcb000000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDUZDT_registerM_registerZ_w064_triple */
	{"maddwdp", {{ 0x58000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWDP_registerM_registerZ_registerY_simple */
	{"maddwd", {{ 0x50003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWD_registerW_registerZ_registerY_simple */
	{"maddwd", {{ 0xd0003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWD_registerW_registerZ_w032_double */
	{"maddwp", {{ 0x51000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWP_registerW_registerZ_registerY_simple */
	{"maddwp", {{ 0x41000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWP_registerW_registerZ_s010_simple */
	{"maddwp", {{ 0xc1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWP_registerW_registerZ_s037_double */
	{"maddwp", {{ 0xc1000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDWP_registerW_registerZ_w064_triple */
	{"maddw", {{ 0x53003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDW_registerW_registerZ_registerY_simple */
	{"maddw", {{ 0xd3003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MADDW_registerW_registerZ_w032_double */
	{"make", {{ 0x60000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_signed16_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-MAKE_registerW_s016_simple */
	{"make", {{ 0xe0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_extend6_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-MAKE_registerW_s043_double */
	{"make", {{ 0xe0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-MAKE_registerW_w064_triple */
	{"maxd", {{ 0x73010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXD_registerW_registerZ_registerY_simple */
	{"maxd", {{ 0x63000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXD_registerW_registerZ_s010_simple */
	{"maxd", {{ 0xe3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXD_registerW_registerZ_s037_double */
	{"maxd", {{ 0xe3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXD_registerW_registerZ_w064_triple */
	{"maxhq", {{ 0x73013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXHQ_registerW_registerZ_registerY_simple */
	{"maxhq", {{ 0xf3013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXHQ_splat32_registerW_registerZ_w032_double */
	{"maxhq.@", {{ 0xf3013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXHQ_splat32_registerW_registerZ_w032_double */
	{"maxud", {{ 0x77010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUD_registerW_registerZ_registerY_simple */
	{"maxud", {{ 0x67000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUD_registerW_registerZ_s010_simple */
	{"maxud", {{ 0xe7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUD_registerW_registerZ_s037_double */
	{"maxud", {{ 0xe7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUD_registerW_registerZ_w064_triple */
	{"maxuhq", {{ 0x77013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUHQ_registerW_registerZ_registerY_simple */
	{"maxuhq", {{ 0xf7013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUHQ_splat32_registerW_registerZ_w032_double */
	{"maxuhq.@", {{ 0xf7013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUHQ_splat32_registerW_registerZ_w032_double */
	{"maxuwp", {{ 0x77012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUWP_registerW_registerZ_registerY_simple */
	{"maxuwp", {{ 0xf7012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUWP_splat32_registerW_registerZ_w032_double */
	{"maxuwp.@", {{ 0xf7012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUWP_splat32_registerW_registerZ_w032_double */
	{"maxuw", {{ 0x77011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUW_registerW_registerZ_registerY_simple */
	{"maxuw", {{ 0x77000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUW_registerW_registerZ_s010_simple */
	{"maxuw", {{ 0xf7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXUW_registerW_registerZ_s037_double */
	{"maxwp", {{ 0x73012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXWP_registerW_registerZ_registerY_simple */
	{"maxwp", {{ 0xf3012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXWP_splat32_registerW_registerZ_w032_double */
	{"maxwp.@", {{ 0xf3012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXWP_splat32_registerW_registerZ_w032_double */
	{"maxw", {{ 0x73011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXW_registerW_registerZ_registerY_simple */
	{"maxw", {{ 0x73000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXW_registerW_registerZ_s010_simple */
	{"maxw", {{ 0xf3000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MAXW_registerW_registerZ_s037_double */
	{"mind", {{ 0x72010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MIND_registerW_registerZ_registerY_simple */
	{"mind", {{ 0x62000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MIND_registerW_registerZ_s010_simple */
	{"mind", {{ 0xe2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MIND_registerW_registerZ_s037_double */
	{"mind", {{ 0xe2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MIND_registerW_registerZ_w064_triple */
	{"minhq", {{ 0x72013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINHQ_registerW_registerZ_registerY_simple */
	{"minhq", {{ 0xf2013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINHQ_splat32_registerW_registerZ_w032_double */
	{"minhq.@", {{ 0xf2013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINHQ_splat32_registerW_registerZ_w032_double */
	{"minud", {{ 0x76010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUD_registerW_registerZ_registerY_simple */
	{"minud", {{ 0x66000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUD_registerW_registerZ_s010_simple */
	{"minud", {{ 0xe6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUD_registerW_registerZ_s037_double */
	{"minud", {{ 0xe6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUD_registerW_registerZ_w064_triple */
	{"minuhq", {{ 0x76013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUHQ_registerW_registerZ_registerY_simple */
	{"minuhq", {{ 0xf6013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUHQ_splat32_registerW_registerZ_w032_double */
	{"minuhq.@", {{ 0xf6013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUHQ_splat32_registerW_registerZ_w032_double */
	{"minuwp", {{ 0x76012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUWP_registerW_registerZ_registerY_simple */
	{"minuwp", {{ 0xf6012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUWP_splat32_registerW_registerZ_w032_double */
	{"minuwp.@", {{ 0xf6012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUWP_splat32_registerW_registerZ_w032_double */
	{"minuw", {{ 0x76011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUW_registerW_registerZ_registerY_simple */
	{"minuw", {{ 0x76000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUW_registerW_registerZ_s010_simple */
	{"minuw", {{ 0xf6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINUW_registerW_registerZ_s037_double */
	{"minwp", {{ 0x72012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINWP_registerW_registerZ_registerY_simple */
	{"minwp", {{ 0xf2012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINWP_splat32_registerW_registerZ_w032_double */
	{"minwp.@", {{ 0xf2012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINWP_splat32_registerW_registerZ_w032_double */
	{"minw", {{ 0x72011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINW_registerW_registerZ_registerY_simple */
	{"minw", {{ 0x72000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINW_registerW_registerZ_s010_simple */
	{"minw", {{ 0xf2000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MINW_registerW_registerZ_s037_double */
	{"mm212w", {{ 0x53001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MM212W_registerM_registerZ_registerY_simple */
	{"mm2wq", {{ 0x53001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MM212W_registerM_registerZ_registerY_simple */
	{"mma212w", {{ 0x5b000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA212W_registerM_registerZ_registerY_simple */
	{"mm2awq", {{ 0x5b000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA212W_registerM_registerZ_registerY_simple */
	{"mma4164bw", {{ 0x02000000, 0x7f0410c1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA4164BW_registerAp_registerBp_registerDp_simple */
	{"mma4164subw", {{ 0x02040000, 0x7f0410c1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA4164SUBW_registerAp_registerBp_registerDp_simple */
	{"mma4164ubw", {{ 0x02001000, 0x7f0410c1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA4164UBW_registerAp_registerBp_registerDp_simple */
	{"mma4164usbw", {{ 0x02041000, 0x7f0410c1, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registerap_opnd, &kv3_registerbp_opnd, &kv3_registerdp_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA4164USBW_registerAp_registerBp_registerDp_simple */
	{"mma444hd", {{ 0x02000040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444HD_registerAq_registerB_registerD_simple */
	{"mm4ahd", {{ 0x02000040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444HD_registerAq_registerB_registerD_simple */
	{"mma444suhd", {{ 0x02080040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444SUHD_registerAq_registerB_registerD_simple */
	{"mm4asuhd", {{ 0x02080040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444SUHD_registerAq_registerB_registerD_simple */
	{"mma444uhd", {{ 0x02040040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444UHD_registerAq_registerB_registerD_simple */
	{"mm4auhd", {{ 0x02040040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444UHD_registerAq_registerB_registerD_simple */
	{"mma444ushd", {{ 0x020c0040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444USHD_registerAq_registerB_registerD_simple */
	{"mm4aushd", {{ 0x020c0040, 0x7f0c00c0, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerb_opnd, &kv3_registerd_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMA444USHD_registerAq_registerB_registerD_simple */
	{"mms212w", {{ 0x5f000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMS212W_registerM_registerZ_registerY_simple */
	{"mm2swq", {{ 0x5f000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MMS212W_registerM_registerZ_registerY_simple */
	{"movetq", {{ 0x7f02f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_CRWH, {&kv3_registerah_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MOVETQ_registerAh_registerZ_registerY_simple */
	{"movetq", {{ 0x7f02e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_CRWL, {&kv3_registeral_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MOVETQ_registerAl_registerZ_registerY_simple */
	{"msbfdt", {{ 0x5c001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFDT_registerM_registerZ_registerY_simple */
	{"msbfd", {{ 0x54000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFD_registerW_registerZ_registerY_simple */
	{"msbfhq", {{ 0x56000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFHQ_registerW_registerZ_registerY_simple */
	{"msbfhwq", {{ 0x54004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFHWQ_registerM_registerZ_registerY_simple */
	{"msbfsudt", {{ 0x5e001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFSUDT_registerM_registerZ_registerY_simple */
	{"msbfsuhwq", {{ 0x56004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFSUHWQ_registerM_registerZ_registerY_simple */
	{"msbfsuwdp", {{ 0x5e000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFSUWDP_registerM_registerZ_registerY_simple */
	{"msbfsuwd", {{ 0x56003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFSUWD_registerW_registerZ_registerY_simple */
	{"msbfsuwd", {{ 0xd6003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFSUWD_registerW_registerZ_w032_double */
	{"msbfudt", {{ 0x5d001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFUDT_registerM_registerZ_registerY_simple */
	{"msbfuhwq", {{ 0x55004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFUHWQ_registerM_registerZ_registerY_simple */
	{"msbfuwdp", {{ 0x5d000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFUWDP_registerM_registerZ_registerY_simple */
	{"msbfuwd", {{ 0x55003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFUWD_registerW_registerZ_registerY_simple */
	{"msbfuwd", {{ 0xd5003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFUWD_registerW_registerZ_w032_double */
	{"msbfuzdt", {{ 0x5f001000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFUZDT_registerM_registerZ_registerY_simple */
	{"msbfwdp", {{ 0x5c000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFWDP_registerM_registerZ_registerY_simple */
	{"msbfwd", {{ 0x54003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFWD_registerW_registerZ_registerY_simple */
	{"msbfwd", {{ 0xd4003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFWD_registerW_registerZ_w032_double */
	{"msbfwp", {{ 0x55000000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFWP_registerW_registerZ_registerY_simple */
	{"msbfw", {{ 0x57003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFW_registerW_registerZ_registerY_simple */
	{"msbfw", {{ 0xd7003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_AUXR_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MSBFW_registerW_registerZ_w032_double */
	{"mt44d", {{ 0x07001740, 0x7f0c3f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-MT44D_registerAq_registerBq_simple */
	{"mt4x4d", {{ 0x07001740, 0x7f0c3f40, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TCA, ((int)0 << 8) | (int)Reservation_kv3_TCA, {&kv3_registeraq_opnd, &kv3_registerbq_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-MT44D_registerAq_registerBq_simple */
	{"mulcwc", {{ 0x55002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULCWC_registerW_registerZ_registerY_simple */
	{"mulcwdc", {{ 0x57002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULCWDC_registerM_registerZ_registerY_simple */
	{"muldt", {{ 0x58041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULDT_registerM_registerZ_registerY_simple */
	{"muldt", {{ 0x48040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULDT_registerM_registerZ_s010_simple */
	{"muldt", {{ 0xc8040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULDT_registerM_registerZ_s037_double */
	{"muldt", {{ 0xc8040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULDT_registerM_registerZ_w064_triple */
	{"muld", {{ 0x54001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULD_registerW_registerZ_registerY_simple */
	{"muld", {{ 0x44000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULD_registerW_registerZ_s010_simple */
	{"muld", {{ 0xc4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULD_registerW_registerZ_s037_double */
	{"muld", {{ 0xc4000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULD_registerW_registerZ_w064_triple */
	{"mulhq", {{ 0x56001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULHQ_registerW_registerZ_registerY_simple */
	{"mulhq", {{ 0x46000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULHQ_registerW_registerZ_s010_simple */
	{"mulhq", {{ 0xc6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULHQ_registerW_registerZ_s037_double */
	{"mulhq", {{ 0xc6000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULHQ_registerW_registerZ_w064_triple */
	{"mulhwq", {{ 0x58004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULHWQ_registerM_registerZ_registerY_simple */
	{"mulsudt", {{ 0x5a041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUDT_registerM_registerZ_registerY_simple */
	{"mulsudt", {{ 0x4a040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUDT_registerM_registerZ_s010_simple */
	{"mulsudt", {{ 0xca040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUDT_registerM_registerZ_s037_double */
	{"mulsudt", {{ 0xca040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUDT_registerM_registerZ_w064_triple */
	{"mulsuhwq", {{ 0x5a004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUHWQ_registerM_registerZ_registerY_simple */
	{"mulsuwdp", {{ 0x52001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUWDP_registerM_registerZ_registerY_simple */
	{"mulsuwd", {{ 0x5a003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUWD_registerW_registerZ_registerY_simple */
	{"mulsuwd", {{ 0xda003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULSUWD_registerW_registerZ_w032_double */
	{"muludt", {{ 0x59041000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUDT_registerM_registerZ_registerY_simple */
	{"muludt", {{ 0x49040000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUDT_registerM_registerZ_s010_simple */
	{"muludt", {{ 0xc9040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUDT_registerM_registerZ_s037_double */
	{"muludt", {{ 0xc9040000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUDT_registerM_registerZ_w064_triple */
	{"muluhwq", {{ 0x59004000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUHWQ_registerM_registerZ_registerY_simple */
	{"muluwdp", {{ 0x51001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUWDP_registerM_registerZ_registerY_simple */
	{"muluwd", {{ 0x59003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUWD_registerW_registerZ_registerY_simple */
	{"muluwd", {{ 0xd9003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULUWD_registerW_registerZ_w032_double */
	{"mulwc", {{ 0x57001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWC_registerW_registerZ_registerY_simple */
	{"mulwc", {{ 0x47000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWC_registerW_registerZ_s010_simple */
	{"mulwc", {{ 0xc7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWC_registerW_registerZ_s037_double */
	{"mulwc", {{ 0xc7000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWC_registerW_registerZ_w064_triple */
	{"mulwdc", {{ 0x56002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWDC_registerM_registerZ_registerY_simple */
	{"mulwdp", {{ 0x50001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWDP_registerM_registerZ_registerY_simple */
	{"mulwd", {{ 0x58003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWD_registerW_registerZ_registerY_simple */
	{"mulwd", {{ 0xd8003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWD_registerW_registerZ_w032_double */
	{"mulwp", {{ 0x55001000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWP_registerW_registerZ_registerY_simple */
	{"mulwp", {{ 0x45000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWP_registerW_registerZ_s010_simple */
	{"mulwp", {{ 0xc5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWP_registerW_registerZ_s037_double */
	{"mulwp", {{ 0xc5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_MAU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_MAU_Y, ((int)0 << 8) | (int)Reservation_kv3_MAU_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWP_registerW_registerZ_w064_triple */
	{"mulwq", {{ 0x54002000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerm_opnd, &kv3_registerp_opnd, &kv3_registero_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULWQ_registerM_registerP_registerO_simple */
	{"mulw", {{ 0x5b003000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_MAU, ((int)0 << 8) | (int)Reservation_kv3_MAU, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULW_registerW_registerZ_registerY_simple */
	{"mulw", {{ 0xdb003000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_MAU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_MAU_X, ((int)0 << 8) | (int)Reservation_kv3_MAU_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-MULW_registerW_registerZ_w032_double */
	{"nandd", {{ 0x79010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDD_registerW_registerZ_registerY_simple */
	{"nandd", {{ 0x69000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDD_registerW_registerZ_s010_simple */
	{"nandd", {{ 0xe9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDD_registerW_registerZ_s037_double */
	{"nandd.@", {{ 0xf9010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDD_registerW_registerZ_w032_double */
	{"nandd", {{ 0xe9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDD_registerW_registerZ_w064_triple */
	{"nandw", {{ 0x79011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDW_registerW_registerZ_registerY_simple */
	{"nandw", {{ 0x79000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDW_registerW_registerZ_s010_simple */
	{"nandw", {{ 0xf9000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NANDW_registerW_registerZ_s037_double */
	{"nop", {{ 0x7f03f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_NOP, ((int)0 << 8) | (int)Reservation_kv3_ALU_NOP, {0}, "", ""}, /* Opcode-v2-NOP_simple */
	{"nord", {{ 0x7b010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORD_registerW_registerZ_registerY_simple */
	{"nord", {{ 0x6b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORD_registerW_registerZ_s010_simple */
	{"nord", {{ 0xeb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORD_registerW_registerZ_s037_double */
	{"nord.@", {{ 0xfb010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORD_registerW_registerZ_w032_double */
	{"nord", {{ 0xeb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORD_registerW_registerZ_w064_triple */
	{"norw", {{ 0x7b011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORW_registerW_registerZ_registerY_simple */
	{"norw", {{ 0x7b000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORW_registerW_registerZ_s010_simple */
	{"norw", {{ 0xfb000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NORW_registerW_registerZ_s037_double */
	{"nxord", {{ 0x7d010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORD_registerW_registerZ_registerY_simple */
	{"nxord", {{ 0x6d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORD_registerW_registerZ_s010_simple */
	{"nxord", {{ 0xed000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORD_registerW_registerZ_s037_double */
	{"nxord.@", {{ 0xfd010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORD_registerW_registerZ_w032_double */
	{"nxord", {{ 0xed000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORD_registerW_registerZ_w064_triple */
	{"nxorw", {{ 0x7d011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORW_registerW_registerZ_registerY_simple */
	{"nxorw", {{ 0x7d000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORW_registerW_registerZ_s010_simple */
	{"nxorw", {{ 0xfd000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-NXORW_registerW_registerZ_s037_double */
	{"ord", {{ 0x7a010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORD_registerW_registerZ_registerY_simple */
	{"ord", {{ 0x6a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORD_registerW_registerZ_s010_simple */
	{"ord", {{ 0xea000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORD_registerW_registerZ_s037_double */
	{"ord.@", {{ 0xfa010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORD_registerW_registerZ_w032_double */
	{"ord", {{ 0xea000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORD_registerW_registerZ_w064_triple */
	{"ornd", {{ 0x7f010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORND_registerW_registerZ_registerY_simple */
	{"ornd", {{ 0x6f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORND_registerW_registerZ_s010_simple */
	{"ornd", {{ 0xef000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORND_registerW_registerZ_s037_double */
	{"ornd.@", {{ 0xff010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORND_registerW_registerZ_w032_double */
	{"ornd", {{ 0xef000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORND_registerW_registerZ_w064_triple */
	{"ornw", {{ 0x7f011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORNW_registerW_registerZ_registerY_simple */
	{"ornw", {{ 0x7f000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORNW_registerW_registerZ_s010_simple */
	{"ornw", {{ 0xff000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORNW_registerW_registerZ_s037_double */
	{"orw", {{ 0x7a011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORW_registerW_registerZ_registerY_simple */
	{"orw", {{ 0x7a000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORW_registerW_registerZ_s010_simple */
	{"orw", {{ 0xfa000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ORW_registerW_registerZ_s037_double */
	{"pcrel", {{ 0x70000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_signed16_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-PCREL_registerW_s016_simple */
	{"pcrel", {{ 0xf0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_extend6_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-PCREL_registerW_s043_double */
	{"pcrel", {{ 0xf0000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_FULL_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_Y, {&kv3_registerw_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-PCREL_registerW_w064_triple */
	{"recvv.b", {{ 0x7303f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_CRWL_CRWH, {&kv3_registera_opnd, 0}, "", "%s"}, /* Opcode-v2-RECVV_channel_registerA_simple */
	{"recvv.f", {{ 0x7203f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_CRWL_CRWH, {&kv3_registera_opnd, 0}, "", "%s"}, /* Opcode-v2-RECVV_channel_registerA_simple */
	{"ret", {{ 0x0fd00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {0}, "", ""}, /* Opcode-v2-RET_simple */
	{"rfe", {{ 0x0fd40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-RFE_simple */
	{"rolwps", {{ 0x7e028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ROLWPS_registerW_registerZ_registerY_simple */
	{"rolwps", {{ 0x7e024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ROLWPS_registerW_registerZ_u006_simple */
	{"rolw", {{ 0x7e027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ROLW_registerW_registerZ_registerY_simple */
	{"rolw", {{ 0x7e023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-ROLW_registerW_registerZ_u006_simple */
	{"rorwps", {{ 0x7f028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-RORWPS_registerW_registerZ_registerY_simple */
	{"rorwps", {{ 0x7f024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-RORWPS_registerW_registerZ_u006_simple */
	{"rorw", {{ 0x7f027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-RORW_registerW_registerZ_registerY_simple */
	{"rorw", {{ 0x7f023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-RORW_registerW_registerZ_u006_simple */
	{"rswap", {{ 0x0fc80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_registerz_opnd, &v2_systemalone_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-RSWAP_registerZ_systemAlone_simple */
	{"rswap", {{ 0x0fc80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &v2_systems3_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-RSWAP_registerZ_systemS3_simple */
	{"rswap", {{ 0x0fc80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, &v2_systems4_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-RSWAP_registerZ_systemS4_simple */
	{"satd", {{ 0x7e026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SATD_registerW_registerZ_registerY_simple */
	{"satd", {{ 0x7e022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SATD_registerW_registerZ_u006_simple */
	{"satud", {{ 0x7f026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SATUD_registerW_registerZ_registerY_simple */
	{"satud", {{ 0x7f022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SATUD_registerW_registerZ_u006_simple */
	{"sbfcd.i", {{ 0x7f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD.I_registerW_registerZ_registerY_simple */
	{"sbfcid", {{ 0x7f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD.I_registerW_registerZ_registerY_simple */
	{"sbfd.ci", {{ 0x7f01d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD.I_registerW_registerZ_registerY_simple */
	{"sbfcd.i", {{ 0xff01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD.I_registerW_registerZ_w032_double */
	{"sbfcid", {{ 0xff01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD.I_registerW_registerZ_w032_double */
	{"sbfd.ci", {{ 0xff01d000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD.I_registerW_registerZ_w032_double */
	{"sbfcd", {{ 0x7f01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD_registerW_registerZ_registerY_simple */
	{"sbfd.c", {{ 0x7f01c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_FULL, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD_registerW_registerZ_registerY_simple */
	{"sbfcd", {{ 0xff01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD_registerW_registerZ_w032_double */
	{"sbfd.c", {{ 0xff01c000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_FULL_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_FULL_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCD_registerW_registerZ_w032_double */
	{"sbfchcp", {{ 0x7d013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCHCP_registerW_registerZ_registerY_simple */
	{"sbfchcp", {{ 0xfd013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCHCP_splat32_registerW_registerZ_w032_double */
	{"sbfchcp.@", {{ 0xfd013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCHCP_splat32_registerW_registerZ_w032_double */
	{"sbfcwc", {{ 0x7d012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCWC_registerW_registerZ_registerY_simple */
	{"sbfcwc", {{ 0xfd012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCWC_splat32_registerW_registerZ_w032_double */
	{"sbfcwc.@", {{ 0xfd012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFCWC_splat32_registerW_registerZ_w032_double */
	{"sbfd", {{ 0x75010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFD_registerW_registerZ_registerY_simple */
	{"sbfd", {{ 0x65000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFD_registerW_registerZ_s010_simple */
	{"sbfd", {{ 0xe5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFD_registerW_registerZ_s037_double */
	{"sbfd", {{ 0xe5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFD_registerW_registerZ_w064_triple */
	{"sbfhq", {{ 0x75013000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFHQ_registerW_registerZ_registerY_simple */
	{"sbfhq", {{ 0xf5013000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFHQ_splat32_registerW_registerZ_w032_double */
	{"sbfhq.@", {{ 0xf5013800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFHQ_splat32_registerW_registerZ_w032_double */
	{"sbfsd", {{ 0x7f01a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSD_registerW_registerZ_registerY_simple */
	{"sbfsd", {{ 0x6f010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSD_registerW_registerZ_s010_simple */
	{"sbfsd", {{ 0xef010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSD_registerW_registerZ_s037_double */
	{"sbfsd", {{ 0xef010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSD_registerW_registerZ_w064_triple */
	{"sbfshq", {{ 0x7f01f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSHQ_registerW_registerZ_registerY_simple */
	{"sbfshq", {{ 0xff01f000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSHQ_splat32_registerW_registerZ_w032_double */
	{"sbfshq.@", {{ 0xff01f800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSHQ_splat32_registerW_registerZ_w032_double */
	{"sbfswp", {{ 0x7f01e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSWP_registerW_registerZ_registerY_simple */
	{"sbfswp", {{ 0xff01e000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSWP_splat32_registerW_registerZ_w032_double */
	{"sbfswp.@", {{ 0xff01e800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSWP_splat32_registerW_registerZ_w032_double */
	{"sbfsw", {{ 0x7f01b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSW_registerW_registerZ_registerY_simple */
	{"sbfsw", {{ 0xff01b000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFSW_registerW_registerZ_w032_double */
	{"sbfuwd", {{ 0x7b014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFUWD_registerW_registerZ_registerY_simple */
	{"sbfuwd", {{ 0xfb014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFUWD_splat32_registerW_registerZ_w032_double */
	{"sbfuwd.@", {{ 0xfb014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFUWD_splat32_registerW_registerZ_w032_double */
	{"sbfwd", {{ 0x79014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFWD_registerW_registerZ_registerY_simple */
	{"sbfwd", {{ 0xf9014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFWD_splat32_registerW_registerZ_w032_double */
	{"sbfwd.@", {{ 0xf9014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFWD_splat32_registerW_registerZ_w032_double */
	{"sbfwp", {{ 0x75012000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFWP_registerW_registerZ_registerY_simple */
	{"sbfwp", {{ 0xf5012000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFWP_splat32_registerW_registerZ_w032_double */
	{"sbfwp.@", {{ 0xf5012800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFWP_splat32_registerW_registerZ_w032_double */
	{"sbfw", {{ 0x75011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFW_registerW_registerZ_registerY_simple */
	{"sbfw", {{ 0x75000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFW_registerW_registerZ_s010_simple */
	{"sbfw", {{ 0xf5000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFW_registerW_registerZ_s037_double */
	{"sbfx16d", {{ 0x77014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16D_registerW_registerZ_registerY_simple */
	{"sbfx16d", {{ 0xf7014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16D_splat32_registerW_registerZ_w032_double */
	{"sbfx16d.@", {{ 0xf7014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16D_splat32_registerW_registerZ_w032_double */
	{"sbfx16hq", {{ 0x77017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16HQ_registerW_registerZ_registerY_simple */
	{"sbfx16hq", {{ 0xf7017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx16hq.@", {{ 0xf7017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx16uwd", {{ 0x7f018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16UWD_registerW_registerZ_registerY_simple */
	{"sbfx16uwd", {{ 0xff018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16UWD_registerW_registerZ_w032_double */
	{"sbfx16wd", {{ 0x77018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16WD_registerW_registerZ_registerY_simple */
	{"sbfx16wd", {{ 0xf7018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16WD_registerW_registerZ_w032_double */
	{"sbfx16wp", {{ 0x77016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16WP_registerW_registerZ_registerY_simple */
	{"sbfx16wp", {{ 0xf7016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16WP_splat32_registerW_registerZ_w032_double */
	{"sbfx16wp.@", {{ 0xf7016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16WP_splat32_registerW_registerZ_w032_double */
	{"sbfx16w", {{ 0x77015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16W_registerW_registerZ_registerY_simple */
	{"sbfx16w", {{ 0xf7015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX16W_registerW_registerZ_w032_double */
	{"sbfx2d", {{ 0x71014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2D_registerW_registerZ_registerY_simple */
	{"sbfx2d", {{ 0xf1014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2D_splat32_registerW_registerZ_w032_double */
	{"sbfx2d.@", {{ 0xf1014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2D_splat32_registerW_registerZ_w032_double */
	{"sbfx2hq", {{ 0x71017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2HQ_registerW_registerZ_registerY_simple */
	{"sbfx2hq", {{ 0xf1017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx2hq.@", {{ 0xf1017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx2uwd", {{ 0x79018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2UWD_registerW_registerZ_registerY_simple */
	{"sbfx2uwd", {{ 0xf9018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2UWD_registerW_registerZ_w032_double */
	{"sbfx2wd", {{ 0x71018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2WD_registerW_registerZ_registerY_simple */
	{"sbfx2wd", {{ 0xf1018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2WD_registerW_registerZ_w032_double */
	{"sbfx2wp", {{ 0x71016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2WP_registerW_registerZ_registerY_simple */
	{"sbfx2wp", {{ 0xf1016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2WP_splat32_registerW_registerZ_w032_double */
	{"sbfx2wp.@", {{ 0xf1016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2WP_splat32_registerW_registerZ_w032_double */
	{"sbfx2w", {{ 0x71015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2W_registerW_registerZ_registerY_simple */
	{"sbfx2w", {{ 0xf1015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX2W_registerW_registerZ_w032_double */
	{"sbfx4d", {{ 0x73014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4D_registerW_registerZ_registerY_simple */
	{"sbfx4d", {{ 0xf3014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4D_splat32_registerW_registerZ_w032_double */
	{"sbfx4d.@", {{ 0xf3014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4D_splat32_registerW_registerZ_w032_double */
	{"sbfx4hq", {{ 0x73017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4HQ_registerW_registerZ_registerY_simple */
	{"sbfx4hq", {{ 0xf3017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx4hq.@", {{ 0xf3017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx4uwd", {{ 0x7b018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4UWD_registerW_registerZ_registerY_simple */
	{"sbfx4uwd", {{ 0xfb018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4UWD_registerW_registerZ_w032_double */
	{"sbfx4wd", {{ 0x73018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4WD_registerW_registerZ_registerY_simple */
	{"sbfx4wd", {{ 0xf3018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4WD_registerW_registerZ_w032_double */
	{"sbfx4wp", {{ 0x73016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4WP_registerW_registerZ_registerY_simple */
	{"sbfx4wp", {{ 0xf3016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4WP_splat32_registerW_registerZ_w032_double */
	{"sbfx4wp.@", {{ 0xf3016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4WP_splat32_registerW_registerZ_w032_double */
	{"sbfx4w", {{ 0x73015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4W_registerW_registerZ_registerY_simple */
	{"sbfx4w", {{ 0xf3015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX4W_registerW_registerZ_w032_double */
	{"sbfx8d", {{ 0x75014000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8D_registerW_registerZ_registerY_simple */
	{"sbfx8d", {{ 0xf5014000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8D_splat32_registerW_registerZ_w032_double */
	{"sbfx8d.@", {{ 0xf5014800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8D_splat32_registerW_registerZ_w032_double */
	{"sbfx8hq", {{ 0x75017000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8HQ_registerW_registerZ_registerY_simple */
	{"sbfx8hq", {{ 0xf5017000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx8hq.@", {{ 0xf5017800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8HQ_splat32_registerW_registerZ_w032_double */
	{"sbfx8uwd", {{ 0x7d018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8UWD_registerW_registerZ_registerY_simple */
	{"sbfx8uwd", {{ 0xfd018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8UWD_registerW_registerZ_w032_double */
	{"sbfx8wd", {{ 0x75018000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8WD_registerW_registerZ_registerY_simple */
	{"sbfx8wd", {{ 0xf5018000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8WD_registerW_registerZ_w032_double */
	{"sbfx8wp", {{ 0x75016000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8WP_registerW_registerZ_registerY_simple */
	{"sbfx8wp", {{ 0xf5016000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8WP_splat32_registerW_registerZ_w032_double */
	{"sbfx8wp.@", {{ 0xf5016800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8WP_splat32_registerW_registerZ_w032_double */
	{"sbfx8w", {{ 0x75015000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8W_registerW_registerZ_registerY_simple */
	{"sbfx8w", {{ 0xf5015000, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBFX8W_registerW_registerZ_w032_double */
	{"sbmm8", {{ 0x7e020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMM8_registerW_registerZ_registerY_simple */
	{"sbmm8", {{ 0x6e020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMM8_registerW_registerZ_s010_simple */
	{"sbmm8", {{ 0xee020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMM8_registerW_registerZ_s037_double */
	{"sbmm8.@", {{ 0xfe020800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMM8_registerW_registerZ_w032_double */
	{"sbmm8", {{ 0xee020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMM8_registerW_registerZ_w064_triple */
	{"sbmmt8", {{ 0x7f020000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMMT8_registerW_registerZ_registerY_simple */
	{"sbmmt8", {{ 0x6f020000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMMT8_registerW_registerZ_s010_simple */
	{"sbmmt8", {{ 0xef020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMMT8_registerW_registerZ_s037_double */
	{"sbmmt8.@", {{ 0xff020800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LITE_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMMT8_registerW_registerZ_w032_double */
	{"sbmmt8", {{ 0xef020000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LITE_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SBMMT8_registerW_registerZ_w064_triple */
	{"sb", {{ 0x2403e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SB_doscale_registerY_registerZ_registerT_simple */
	{"sb.xs", {{ 0x2403f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SB_doscale_registerY_registerZ_registerT_simple */
	{"sb.deqz", {{ 0x24031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dgez", {{ 0x24033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dgtz", {{ 0x24035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dlez", {{ 0x24034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dltz", {{ 0x24032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.dnez", {{ 0x24030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.even", {{ 0x24037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.odd", {{ 0x24036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.weqz", {{ 0x24039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wgez", {{ 0x2403b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wgtz", {{ 0x2403d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wlez", {{ 0x2403c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wltz", {{ 0x2403a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.wnez", {{ 0x24038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_registerZ_registerT_simple */
	{"sb.deqz", {{ 0xa4031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dgez", {{ 0xa4033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dgtz", {{ 0xa4035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dlez", {{ 0xa4034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dltz", {{ 0xa4032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.dnez", {{ 0xa4030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.even", {{ 0xa4037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.odd", {{ 0xa4036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.weqz", {{ 0xa4039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wgez", {{ 0xa403b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wgtz", {{ 0xa403d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wlez", {{ 0xa403c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wltz", {{ 0xa403a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.wnez", {{ 0xa4038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s027_registerZ_registerT_double */
	{"sb.deqz", {{ 0xa4031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dgez", {{ 0xa4033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dgtz", {{ 0xa4035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dlez", {{ 0xa4034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dltz", {{ 0xa4032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.dnez", {{ 0xa4030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.even", {{ 0xa4037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.odd", {{ 0xa4036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.weqz", {{ 0xa4039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wgez", {{ 0xa403b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wgtz", {{ 0xa403d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wlez", {{ 0xa403c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wltz", {{ 0xa403a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb.wnez", {{ 0xa4038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SB_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sb", {{ 0x24010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SB_s010_registerZ_registerT_simple */
	{"sb", {{ 0xa4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SB_s037_registerZ_registerT_double */
	{"sb", {{ 0xa4010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SB_w064_registerZ_registerT_triple */
	{"scall", {{ 0x0fe40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v2-SCALL_registerZ_simple */
	{"scall", {{ 0x0fe00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&kv3_sysnumber_opnd, 0}, "", "%s"}, /* Opcode-v2-SCALL_u012_simple */
	{"sd", {{ 0x2703e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SD_doscale_registerY_registerZ_registerT_simple */
	{"sd.xs", {{ 0x2703f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SD_doscale_registerY_registerZ_registerT_simple */
	{"sd.deqz", {{ 0x27031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dgez", {{ 0x27033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dgtz", {{ 0x27035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dlez", {{ 0x27034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dltz", {{ 0x27032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.dnez", {{ 0x27030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.even", {{ 0x27037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.odd", {{ 0x27036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.weqz", {{ 0x27039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wgez", {{ 0x2703b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wgtz", {{ 0x2703d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wlez", {{ 0x2703c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wltz", {{ 0x2703a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.wnez", {{ 0x27038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_registerZ_registerT_simple */
	{"sd.deqz", {{ 0xa7031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dgez", {{ 0xa7033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dgtz", {{ 0xa7035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dlez", {{ 0xa7034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dltz", {{ 0xa7032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.dnez", {{ 0xa7030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.even", {{ 0xa7037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.odd", {{ 0xa7036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.weqz", {{ 0xa7039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wgez", {{ 0xa703b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wgtz", {{ 0xa703d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wlez", {{ 0xa703c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wltz", {{ 0xa703a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.wnez", {{ 0xa7038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s027_registerZ_registerT_double */
	{"sd.deqz", {{ 0xa7031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dgez", {{ 0xa7033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dgtz", {{ 0xa7035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dlez", {{ 0xa7034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dltz", {{ 0xa7032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.dnez", {{ 0xa7030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.even", {{ 0xa7037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.odd", {{ 0xa7036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.weqz", {{ 0xa7039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wgez", {{ 0xa703b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wgtz", {{ 0xa703d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wlez", {{ 0xa703c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wltz", {{ 0xa703a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd.wnez", {{ 0xa7038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SD_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sd", {{ 0x27010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SD_s010_registerZ_registerT_simple */
	{"sd", {{ 0xa7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SD_s037_registerZ_registerT_double */
	{"sd", {{ 0xa7010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SD_w064_registerZ_registerT_triple */
	{"sendv.b", {{ 0x7103f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_CRRP, {&kv3_registera_opnd, 0}, "", "%s"}, /* Opcode-v2-SENDV_channel_registerA_simple */
	{"sendv.f", {{ 0x7003f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_CRRP, {&kv3_registera_opnd, 0}, "", "%s"}, /* Opcode-v2-SENDV_channel_registerA_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&v2_systemalone_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SET_systemAlone_registerZ_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_systemra_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SET_systemRA_registerZ_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&v2_systemt3_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SET_systemT3_registerZ_simple */
	{"set", {{ 0x0fc00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&v2_systemt4_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SET_systemT4_registerZ_simple */
	{"sh", {{ 0x2503e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SH_doscale_registerY_registerZ_registerT_simple */
	{"sh.xs", {{ 0x2503f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SH_doscale_registerY_registerZ_registerT_simple */
	{"sh.deqz", {{ 0x25031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dgez", {{ 0x25033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dgtz", {{ 0x25035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dlez", {{ 0x25034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dltz", {{ 0x25032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.dnez", {{ 0x25030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.even", {{ 0x25037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.odd", {{ 0x25036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.weqz", {{ 0x25039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wgez", {{ 0x2503b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wgtz", {{ 0x2503d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wlez", {{ 0x2503c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wltz", {{ 0x2503a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.wnez", {{ 0x25038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_registerZ_registerT_simple */
	{"sh.deqz", {{ 0xa5031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dgez", {{ 0xa5033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dgtz", {{ 0xa5035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dlez", {{ 0xa5034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dltz", {{ 0xa5032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.dnez", {{ 0xa5030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.even", {{ 0xa5037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.odd", {{ 0xa5036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.weqz", {{ 0xa5039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wgez", {{ 0xa503b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wgtz", {{ 0xa503d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wlez", {{ 0xa503c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wltz", {{ 0xa503a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.wnez", {{ 0xa5038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s027_registerZ_registerT_double */
	{"sh.deqz", {{ 0xa5031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dgez", {{ 0xa5033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dgtz", {{ 0xa5035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dlez", {{ 0xa5034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dltz", {{ 0xa5032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.dnez", {{ 0xa5030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.even", {{ 0xa5037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.odd", {{ 0xa5036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.weqz", {{ 0xa5039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wgez", {{ 0xa503b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wgtz", {{ 0xa503d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wlez", {{ 0xa503c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wltz", {{ 0xa503a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh.wnez", {{ 0xa5038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SH_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sh", {{ 0x25010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SH_s010_registerZ_registerT_simple */
	{"sh", {{ 0xa5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SH_s037_registerZ_registerT_double */
	{"sh", {{ 0xa5010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SH_w064_registerZ_registerT_triple */
	{"sleep", {{ 0x0fa40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-SLEEP_simple */
	{"slld", {{ 0x79026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLD_registerW_registerZ_registerY_simple */
	{"slld", {{ 0x79022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLD_registerW_registerZ_u006_simple */
	{"sllhqs", {{ 0x79029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLHQS_registerW_registerZ_registerY_simple */
	{"sllhqs", {{ 0x79025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLHQS_registerW_registerZ_u006_simple */
	{"sllwps", {{ 0x79028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLWPS_registerW_registerZ_registerY_simple */
	{"sllwps", {{ 0x79024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLWPS_registerW_registerZ_u006_simple */
	{"sllw", {{ 0x79027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLW_registerW_registerZ_registerY_simple */
	{"sllw", {{ 0x79023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLLW_registerW_registerZ_u006_simple */
	{"slsd", {{ 0x7c026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSD_registerW_registerZ_registerY_simple */
	{"slsd", {{ 0x7c022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSD_registerW_registerZ_u006_simple */
	{"slshqs", {{ 0x7c029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSHQS_registerW_registerZ_registerY_simple */
	{"slshqs", {{ 0x7c025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSHQS_registerW_registerZ_u006_simple */
	{"slswps", {{ 0x7c028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSWPS_registerW_registerZ_registerY_simple */
	{"slswps", {{ 0x7c024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSWPS_registerW_registerZ_u006_simple */
	{"slsw", {{ 0x7c027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSW_registerW_registerZ_registerY_simple */
	{"slsw", {{ 0x7c023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SLSW_registerW_registerZ_u006_simple */
	{"so", {{ 0x2807e000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SO_doscale_registerY_registerZ_registerV_simple */
	{"so.xs", {{ 0x2807f000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SO_doscale_registerY_registerZ_registerV_simple */
	{"so.deqz", {{ 0x28071000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dgez", {{ 0x28073000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dgtz", {{ 0x28075000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dlez", {{ 0x28074000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dltz", {{ 0x28072000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.dnez", {{ 0x28070000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.even", {{ 0x28077000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.odd", {{ 0x28076000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.weqz", {{ 0x28079000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wgez", {{ 0x2807b000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wgtz", {{ 0x2807d000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wlez", {{ 0x2807c000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wltz", {{ 0x2807a000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.wnez", {{ 0x28078000, 0x7f0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_registerZ_registerV_simple */
	{"so.deqz", {{ 0xa8071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dgez", {{ 0xa8073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dgtz", {{ 0xa8075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dlez", {{ 0xa8074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dltz", {{ 0xa8072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.dnez", {{ 0xa8070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.even", {{ 0xa8077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.odd", {{ 0xa8076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.weqz", {{ 0xa8079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wgez", {{ 0xa807b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wgtz", {{ 0xa807d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wlez", {{ 0xa807c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wltz", {{ 0xa807a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.wnez", {{ 0xa8078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s027_registerZ_registerV_double */
	{"so.deqz", {{ 0xa8071000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dgez", {{ 0xa8073000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dgtz", {{ 0xa8075000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dlez", {{ 0xa8074000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dltz", {{ 0xa8072000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.dnez", {{ 0xa8070000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.even", {{ 0xa8077000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.odd", {{ 0xa8076000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.weqz", {{ 0xa8079000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wgez", {{ 0xa807b000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wgtz", {{ 0xa807d000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wlez", {{ 0xa807c000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wltz", {{ 0xa807a000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so.wnez", {{ 0xa8078000, 0xff0ff000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SO_lsucond_registerY_s054_registerZ_registerV_triple */
	{"so", {{ 0x28050000, 0x7f0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SO_s010_registerZ_registerV_simple */
	{"so", {{ 0xa8050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SO_s037_registerZ_registerV_double */
	{"so", {{ 0xa8050000, 0xff0f0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registerv_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SO_w064_registerZ_registerV_triple */
	{"sq", {{ 0x2803e000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SQ_doscale_registerY_registerZ_registerU_simple */
	{"sq.xs", {{ 0x2803f000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SQ_doscale_registerY_registerZ_registerU_simple */
	{"sq.deqz", {{ 0x28031000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dgez", {{ 0x28033000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dgtz", {{ 0x28035000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dlez", {{ 0x28034000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dltz", {{ 0x28032000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.dnez", {{ 0x28030000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.even", {{ 0x28037000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.odd", {{ 0x28036000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.weqz", {{ 0x28039000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wgez", {{ 0x2803b000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wgtz", {{ 0x2803d000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wlez", {{ 0x2803c000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wltz", {{ 0x2803a000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.wnez", {{ 0x28038000, 0x7f07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_registerZ_registerU_simple */
	{"sq.deqz", {{ 0xa8031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dgez", {{ 0xa8033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dgtz", {{ 0xa8035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dlez", {{ 0xa8034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dltz", {{ 0xa8032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.dnez", {{ 0xa8030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.even", {{ 0xa8037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.odd", {{ 0xa8036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.weqz", {{ 0xa8039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wgez", {{ 0xa803b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wgtz", {{ 0xa803d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wlez", {{ 0xa803c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wltz", {{ 0xa803a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.wnez", {{ 0xa8038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s027_registerZ_registerU_double */
	{"sq.deqz", {{ 0xa8031000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dgez", {{ 0xa8033000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dgtz", {{ 0xa8035000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dlez", {{ 0xa8034000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dltz", {{ 0xa8032000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.dnez", {{ 0xa8030000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.even", {{ 0xa8037000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.odd", {{ 0xa8036000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.weqz", {{ 0xa8039000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wgez", {{ 0xa803b000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wgtz", {{ 0xa803d000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wlez", {{ 0xa803c000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wltz", {{ 0xa803a000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq.wnez", {{ 0xa8038000, 0xff07f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SQ_lsucond_registerY_s054_registerZ_registerU_triple */
	{"sq", {{ 0x28010000, 0x7f070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SQ_s010_registerZ_registerU_simple */
	{"sq", {{ 0xa8010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SQ_s037_registerZ_registerU_double */
	{"sq", {{ 0xa8010000, 0xff070000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registeru_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SQ_w064_registerZ_registerU_triple */
	{"srad", {{ 0x7a026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAD_registerW_registerZ_registerY_simple */
	{"srad", {{ 0x7a022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAD_registerW_registerZ_u006_simple */
	{"srahqs", {{ 0x7a029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAHQS_registerW_registerZ_registerY_simple */
	{"srahqs", {{ 0x7a025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAHQS_registerW_registerZ_u006_simple */
	{"srawps", {{ 0x7a028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAWPS_registerW_registerZ_registerY_simple */
	{"srawps", {{ 0x7a024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAWPS_registerW_registerZ_u006_simple */
	{"sraw", {{ 0x7a027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAW_registerW_registerZ_registerY_simple */
	{"sraw", {{ 0x7a023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRAW_registerW_registerZ_u006_simple */
	{"srld", {{ 0x7b026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLD_registerW_registerZ_registerY_simple */
	{"srld", {{ 0x7b022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLD_registerW_registerZ_u006_simple */
	{"srlhqs", {{ 0x7b029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLHQS_registerW_registerZ_registerY_simple */
	{"srlhqs", {{ 0x7b025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLHQS_registerW_registerZ_u006_simple */
	{"srlwps", {{ 0x7b028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLWPS_registerW_registerZ_registerY_simple */
	{"srlwps", {{ 0x7b024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLWPS_registerW_registerZ_u006_simple */
	{"srlw", {{ 0x7b027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLW_registerW_registerZ_registerY_simple */
	{"srlw", {{ 0x7b023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRLW_registerW_registerZ_u006_simple */
	{"srsd", {{ 0x78026000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSD_registerW_registerZ_registerY_simple */
	{"srsd", {{ 0x78022000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSD_registerW_registerZ_u006_simple */
	{"srshqs", {{ 0x78029000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSHQS_registerW_registerZ_registerY_simple */
	{"srshqs", {{ 0x78025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSHQS_registerW_registerZ_u006_simple */
	{"srswps", {{ 0x78028000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSWPS_registerW_registerZ_registerY_simple */
	{"srswps", {{ 0x78024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSWPS_registerW_registerZ_u006_simple */
	{"srsw", {{ 0x78027000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSW_registerW_registerZ_registerY_simple */
	{"srsw", {{ 0x78023000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_unsigned6_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-SRSW_registerW_registerZ_u006_simple */
	{"stop", {{ 0x0fa80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-STOP_simple */
	{"stsud", {{ 0x70010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-STSUD_registerW_registerZ_registerY_simple */
	{"stsuw", {{ 0x70011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-STSUW_registerW_registerZ_registerY_simple */
	{"sv", {{ 0x2903e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SV_doscale_registerY_registerZ_registerE_simple */
	{"sv.xs", {{ 0x2903f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SV_doscale_registerY_registerZ_registerE_simple */
	{"sv.deqz", {{ 0x29031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dgez", {{ 0x29033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dgtz", {{ 0x29035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dlez", {{ 0x29034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dltz", {{ 0x29032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.dnez", {{ 0x29030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.even", {{ 0x29037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.odd", {{ 0x29036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.weqz", {{ 0x29039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wgez", {{ 0x2903b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wgtz", {{ 0x2903d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wlez", {{ 0x2903c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wltz", {{ 0x2903a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.wnez", {{ 0x29038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_registerZ_registerE_simple */
	{"sv.deqz", {{ 0xa9031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dgez", {{ 0xa9033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dgtz", {{ 0xa9035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dlez", {{ 0xa9034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dltz", {{ 0xa9032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.dnez", {{ 0xa9030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.even", {{ 0xa9037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.odd", {{ 0xa9036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.weqz", {{ 0xa9039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wgez", {{ 0xa903b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wgtz", {{ 0xa903d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wlez", {{ 0xa903c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wltz", {{ 0xa903a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.wnez", {{ 0xa9038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s027_registerZ_registerE_double */
	{"sv.deqz", {{ 0xa9031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dgez", {{ 0xa9033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dgtz", {{ 0xa9035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dlez", {{ 0xa9034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dltz", {{ 0xa9032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.dnez", {{ 0xa9030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.even", {{ 0xa9037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.odd", {{ 0xa9036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.weqz", {{ 0xa9039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wgez", {{ 0xa903b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wgtz", {{ 0xa903d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wlez", {{ 0xa903c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wltz", {{ 0xa903a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv.wnez", {{ 0xa9038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SV_lsucond_registerY_s054_registerZ_registerE_triple */
	{"sv", {{ 0x29010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SV_s010_registerZ_registerE_simple */
	{"sv", {{ 0xa9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SV_s037_registerZ_registerE_double */
	{"sv", {{ 0xa9010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_CRRP_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registere_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SV_w064_registerZ_registerE_triple */
	{"sw", {{ 0x2603e000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SW_doscale_registerY_registerZ_registerT_simple */
	{"sw.xs", {{ 0x2603f000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SW_doscale_registerY_registerZ_registerT_simple */
	{"sw.deqz", {{ 0x26031000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dgez", {{ 0x26033000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dgtz", {{ 0x26035000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dlez", {{ 0x26034000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dltz", {{ 0x26032000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.dnez", {{ 0x26030000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.even", {{ 0x26037000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.odd", {{ 0x26036000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.weqz", {{ 0x26039000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wgez", {{ 0x2603b000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wgtz", {{ 0x2603d000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wlez", {{ 0x2603c000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wltz", {{ 0x2603a000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.wnez", {{ 0x26038000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_registery_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? [%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_registerZ_registerT_simple */
	{"sw.deqz", {{ 0xa6031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dgez", {{ 0xa6033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dgtz", {{ 0xa6035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dlez", {{ 0xa6034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dltz", {{ 0xa6032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.dnez", {{ 0xa6030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.even", {{ 0xa6037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.odd", {{ 0xa6036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.weqz", {{ 0xa6039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wgez", {{ 0xa603b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wgtz", {{ 0xa603d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wlez", {{ 0xa603c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wltz", {{ 0xa603a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.wnez", {{ 0xa6038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_registery_opnd, &kv3_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s027_registerZ_registerT_double */
	{"sw.deqz", {{ 0xa6031000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dgez", {{ 0xa6033000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dgtz", {{ 0xa6035000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dlez", {{ 0xa6034000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dltz", {{ 0xa6032000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.dnez", {{ 0xa6030000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.even", {{ 0xa6037000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.odd", {{ 0xa6036000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.weqz", {{ 0xa6039000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wgez", {{ 0xa603b000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wgtz", {{ 0xa603d000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wlez", {{ 0xa603c000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wltz", {{ 0xa603a000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw.wnez", {{ 0xa6038000, 0xff03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_registery_opnd, &kv3_extend27_offset27_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s? %s[%s] = %s"}, /* Opcode-v2-SW_lsucond_registerY_s054_registerZ_registerT_triple */
	{"sw", {{ 0x26010000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LSU, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR, {&kv3_signed10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SW_s010_registerZ_registerT_simple */
	{"sw", {{ 0xa6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_LSU_X, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_X, {&kv3_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SW_s037_registerZ_registerT_double */
	{"sw", {{ 0xa6010000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_LSU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_LSU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_LSU_Y, ((int)0 << 8) | (int)Reservation_kv3_LSU_AUXR_Y, {&kv3_extend27_upper27_lower10_opnd, &kv3_registerz_opnd, &kv3_registert_opnd, 0}, "", "%s[%s] = %s"}, /* Opcode-v2-SW_w064_registerZ_registerT_triple */
	{"sxlbhq", {{ 0x76025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SXLBHQ_registerW_registerZ_simple */
	{"sxlhwp", {{ 0x76024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SXLHWP_registerW_registerZ_simple */
	{"sxmbhq", {{ 0x77025000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SXMBHQ_registerW_registerZ_simple */
	{"sxmhwp", {{ 0x77024000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_LITE, ((int)0 << 8) | (int)Reservation_kv3_ALU_LITE, {&kv3_registerw_opnd, &kv3_registerz_opnd, 0}, "", "%s = %s"}, /* Opcode-v2-SXMHWP_registerW_registerZ_simple */
	{"syncgroup", {{ 0x0fb40000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v2-SYNCGROUP_registerZ_simple */
	{"tlbdinval", {{ 0x0f8c0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-TLBDINVAL_simple */
	{"tlbiinval", {{ 0x0f900000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-TLBIINVAL_simple */
	{"tlbprobe", {{ 0x0f840000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-TLBPROBE_simple */
	{"tlbread", {{ 0x0f800000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-TLBREAD_simple */
	{"tlbwrite", {{ 0x0f880000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {0}, "", ""}, /* Opcode-v2-TLBWRITE_simple */
	{"waitit", {{ 0x0fb00000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU_TINY_TINY_MAU_XNOP, {&kv3_registerz_opnd, 0}, "", "%s"}, /* Opcode-v2-WAITIT_registerZ_simple */
	{"wfxl", {{ 0x0fb80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&v2_systemalone_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-WFXL_systemAlone_registerZ_simple */
	{"wfxl", {{ 0x0fb80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&v2_systemt2_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-WFXL_systemT2_registerZ_simple */
	{"wfxl", {{ 0x0fb80000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&v2_systemt4_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-WFXL_systemT4_registerZ_simple */
	{"wfxm", {{ 0x0fbc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_ALL, ((int)0 << 8) | (int)Reservation_kv3_ALL, {&v2_systemalone_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-WFXM_systemAlone_registerZ_simple */
	{"wfxm", {{ 0x0fbc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&v2_systemt2_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-WFXM_systemT2_registerZ_simple */
	{"wfxm", {{ 0x0fbc0000, 0x7ffc0000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_BCU, ((int)0 << 8) | (int)Reservation_kv3_BCU, {&v2_systemt4_opnd, &kv3_registerz_opnd, 0}, "", "%s, %s"}, /* Opcode-v2-WFXM_systemT4_registerZ_simple */
	{"xord", {{ 0x7c010000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORD_registerW_registerZ_registerY_simple */
	{"xord", {{ 0x6c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORD_registerW_registerZ_s010_simple */
	{"xord", {{ 0xec000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORD_registerW_registerZ_s037_double */
	{"xord.@", {{ 0xfc010800, 0xff03f800, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower5_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORD_registerW_registerZ_w032_double */
	{"xord", {{ 0xec000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x80000000, 0xe0000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX1|kvxOPCODE_FLAG_ALU }, }, 3, 96,
((int)0 << 8) | (int)Bundling_kv3_TINY_Y, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_Y, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_extend27_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORD_registerW_registerZ_w064_triple */
	{"xorw", {{ 0x7c011000, 0x7f03f000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_registery_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORW_registerW_registerZ_registerY_simple */
	{"xorw", {{ 0x7c000000, 0x7f030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, }, 1, 32,
((int)0 << 8) | (int)Bundling_kv3_TINY, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_signed10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORW_registerW_registerZ_s010_simple */
	{"xorw", {{ 0xfc000000, 0xff030000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32 }, { 0x00000000, 0x60000000, kvxOPCODE_FLAG_MODE64|kvxOPCODE_FLAG_MODE32|kvxOPCODE_FLAG_IMMX0|kvxOPCODE_FLAG_ALU }, }, 2, 64,
((int)0 << 8) | (int)Bundling_kv3_TINY_X, ((int)0 << 8) | (int)Reservation_kv3_ALU_TINY_X, {&kv3_registerw_opnd, &kv3_registerz_opnd, &kv3_upper27_lower10_opnd, 0}, "", "%s = %s, %s"}, /* Opcode-v2-XORW_registerW_registerZ_s037_double */
/* Number of instructions : 6310 */
	{"", { }, 0, 0, 0, 0, { }, "", ""}};

const Kvx_Core_Info kvx_kv3_v2_core_info = { kv3_v2_optab, { "kv3-2", "\0" }, kv3_v2_resources, true, { ELF_KVX_CORE_KV3_2, -1 }  };

const Kvx_Core_Info *kvx_core_info_table[] = 
{
  & kvx_kv3_v1_core_info,
  & kvx_kv3_v2_core_info
};

const kvx_Register *kvx_registers_table[] = 
{
  kvx_kv3_v1_registers,
  kvx_kv3_v2_registers
};

const int *kvx_regfiles_table[] = 
{
  kvx_kv3_v1_regfiles,
  kvx_kv3_v2_regfiles
};

